#pragma once

#include <MmcTypeDefinition.h>
#include <StdString.h>
#include <time.h>

class CMmcDataConversion
{
public:
    CMmcDataConversion();
    virtual ~CMmcDataConversion();

    bool HexByteStr2Byte(CStdString p_hexValueStr, BYTE* p_pubHexValue, bool p_oShowMsg = true);

    bool IsDigit(TCHAR data);

    bool DecByteStr2Byte(CStdString p_decValueStr, BYTE* p_pubDecValue, bool p_oShowMsg = true);
    bool DecCharStr2Char(CStdString p_decValueStr, char* decValue, bool p_oShowMsg = true);
    bool DecDWordStr2DWord(CStdString p_decValueStr, DWORD* decValue, bool p_oShowMsg = true);
    bool DecFloatStr2Float(CStdString p_decValueStr, float* p_pfDecValue, bool p_oShowMsg = true);
    bool DecFloatStr2Fractional(CStdString p_decValueStr, WORD* pFractionNum, WORD* pFractionDen, bool p_oShowMsg = true);
    bool DecFloatStr2ShortFractional(CStdString p_decValueStr, short* pFractionNum, WORD* pFractionDen, bool p_oShowMsg = true);
    bool DecFloat2Short(float p_fValue, WORD & p_rusValue, bool p_oShowMsg = true);

    bool DecIntStr2Int(CStdString p_decValueStr, int* decValue, bool p_oShowMsg = true);
    bool DecShortStr2Short(CStdString p_decValueStr, short* decValue, bool p_oShowMsg = true);
    bool DecWordStr2Word(CStdString p_decValueStr, WORD* decValue, bool p_oShowMsg = true);
    bool DecInt64Str2Int64(CStdString p_decValueStr, __int64* decValue, bool p_oShowMsg = true);

    bool HexCharStr2Char(CStdString p_hexValueStr, char* hexValue, bool p_oShowMsg = true);
    bool HexDWordStr2DWord(CStdString p_hexValueStr, DWORD* hexValue, bool p_oShowMsg = true);
    bool HexFloatStr2Float(CStdString p_hexValueStr, float* p_pfHexValue, bool p_oShowMsg = true);
    bool HexShortStr2Short(CStdString p_hexValueStr, short* hexValue, bool p_oShowMsg = true);
    bool HexWordStr2Word(CStdString p_hexValueStr, WORD* p_pusHexValue, bool p_oShowMsg = true);
    bool HexIntStr2Int(CStdString p_hexValueStr, int* hexValue, bool p_oShowMsg = true);
    bool HexInt64Str2Int64(CStdString p_hexValueStr, __int64* hexValue, bool p_oShowMsg = true);

    bool ByteStr2Byte(CStdString p_valueStr, BYTE* p_pubValue, bool p_oShowMsg = true);
    bool Byte2HexByteStr(BYTE hexValue, CStdString* p_hexValueStr);

    bool CharStr2Char(CStdString p_valueStr, char* pValue, bool p_oShowMsg = true);
    bool DWordStr2DWord(CStdString p_valueStr, DWORD* pValue, bool p_oShowMsg = true);
    bool FloatStr2Float(CStdString p_valueStr, float* p_pfValue, bool p_oShowMsg = true);
    bool IntStr2Int(CStdString p_valueStr, int* pValue, bool p_oShowMsg = true);
    bool ShortStr2Short(CStdString p_valueStr, short* pValue, bool p_oShowMsg = true);
    bool WordStr2Word(CStdString p_valueStr, WORD* p_pusValue, bool p_oShowMsg = true);
    bool Int32Str2Int32(CStdString p_value, int* p_plValue, bool p_oShowMsg = true);
    bool Int64Str2Int64(CStdString p_valueStr, __int64* pValue, bool p_oShowMsg = true);

    bool TimeStr2Time(CStdString timeStr, time_t* time, bool p_oShowMsg = true);

    bool Char2DecCharStr(BYTE decValue, CStdString* p_pDecValueStr);
    bool Char2HexCharStr(BYTE hexValue, CStdString* p_pHexValueStr);
    bool DWord2DecDWordStr(DWORD decValue, CStdString* p_pDecValueStr);
    bool DWord2HexDWordStr(DWORD hexValue, CStdString* p_pHexValueStr);
    bool Float2DecFloatStr(float decValue, CStdString* p_pDecValueStr);
    bool Float2DecFloatStr(float decValue, DWORD ulDecimalPlaces, CStdString* p_pDecValueStr);
    bool Float2HexFloatStr(float p_fHexValue, CStdString* p_pp_pHexValueStr);
    bool Double2DecDoubleStr(double decValue, CStdString* p_pDecValueStr);
    bool Double2DecDoubleStr(double decValue, DWORD ulDecimalPlaces, CStdString* p_pDecValueStr);
    bool Double2HexDoubleStr(double hexValue, CStdString* p_pHexValueStr);
    bool Fractional2DecFloatStr(WORD fractionNum, WORD fractionDen, CStdString* pDecValueStr);
    bool Int2DecIntStr(int decValue, CStdString* p_pDecValueStr);
    bool Short2DecShortStr(short decValue, CStdString* p_pDecValueStr);
    bool Short2HexShortStr(short hexValue, CStdString* p_pHexValueStr);

    bool Time2TimeStr(time_t time, CStdString* timeStr);

    bool UChar2DecUCharStr(BYTE decValue, CStdString* p_pDecValueStr);
    bool UChar2HexUCharStr(BYTE hexValue, CStdString* p_pHexValueStr);
    bool Word2DecWordStr(WORD decValue, CStdString* p_pDecValueStr);
    bool Word2HexWordStr(WORD hexValue, CStdString* p_pHexValueStr);

    bool Int64_2DecInt64Str(__int64 decValue, CStdString* p_pDecValueStr);
    bool Int64_2HexInt64Str(__int64 hexValue, CStdString* p_pHexValueStr);

    bool UInt64_2HexUInt64Str(__uint64 hexValue, CStdString* p_pHexValueStr);
    bool UInt64_2DecUInt64Str(__uint64 decValue, CStdString* p_pDecValueStr);
    bool UInt64Str2UInt64(CStdString p_valueStr, __uint64* pValue, bool p_oShowMsg = true);
    bool HexUInt64Str2UInt64(CStdString p_hexValueStr, __uint64* hexValue, bool p_oShowMsg = true);
    bool DecUInt64Str2UInt64(CStdString p_decValueStr, __uint64* decValue, bool p_oShowMsg = true);

    bool Int2HexIntStr(int hexValue, CStdString* p_pHexValueStr);

private:
    enum FormatType{UnknownFormat, DecFormat, HexFormat};

    bool Char2HexDigit(TCHAR data, BYTE* p_pubDigit);
    bool Char2DecDigit(TCHAR data, BYTE* p_pubDigit);
    FormatType GetFormatType(CStdString p_valueStr);
};
