#pragma once

#include <string>

enum EAccessType
{
    AT_READ_WRITE,
    AT_READ_WRITE_READ,
    AT_READ_WRITE_WRITE,
    AT_WRITE_ONLY,
    AT_READ_ONLY,
    AT_READ_ONLY_CONST
};

enum EConnectionMode
{
    CM_RS232    = 0,
    CM_CAN_OPEN
};

enum EObjectDataType
{
    ODT_UNKNOWN         = 0,
    ODT_BOOLEAN         = 1,
    ODT_INT8            = 2,
    ODT_INT16           = 3,
    ODT_INT32           = 4,

    ODT_UINT8           = 5,
    ODT_UINT16          = 6,
    ODT_UINT32          = 7,

    ODT_INT64           = 21,
    ODT_UINT64          = 27,

    ODT_FLOAT           = 8,
    ODT_STRING          = 9,
    ODT_DOMAIN          = 15,
    ODT_MEMORY_BLOCK    = 10,

    ODT_ARRAY           = 100
};

enum EOperationMode
{
    OM_UNKNOWN_MODE                 = 0,
    OM_PROFILE_POSITION_MODE        = 1,
    OM_PROFILE_VELOCITY_MODE        = 3,
    OM_PROFILE_TORQUE_MODE          = 4,
    OM_HOMING_MODE                  = 6,
    OM_INTERPOLATED_POSITION_MODE   = 7,
    OM_CYCLIC_SYNC_POSITION_MODE    = 8,
    OM_CYCLIC_SYNC_VELOCITY_MODE    = 9,
    OM_CYCLIC_SYNC_TORQUE_MODE      = 10,
    OM_CYCLIC_SYNC_TORQUE_COMMUTATION_ANGLE_MODE = 11,
    OM_POSITION_MODE                = -1,
    OM_VELOCITY_MODE                = -2,
    OM_CURRENT_MODE                 = -3,
    OM_DIAGNOSTIC_MODE              = -4,
    OM_MASTER_ENCODER_MODE          = -5,
    OM_STEP_DIRECTION_MODE          = -6,
    OM_POSITION_TUNING_MODE         = -11,
    OM_CURRENT_TUNING_MODE          = -12,
    OM_CURRENT_PULSE_MODE           = -21,
    OM_MECHANICAL_MAIN_LOOP_TUNING_MODE = -124,
    OM_MECHANICAL_AUX_LOOP_TUNING_MODE = -125,
    OM_BACK_EMF_TUNING_MODE         = -126,
    OM_ELECTRICAL_TUNING_MODE       = -127,
    OM_MECHANICAL_TUNING_MODE       = -128,
};

enum EOfflineMode
{
    OFM_IDLE                     = 0,
    OFM_COMMUNICATION_BLOCKED    = 1,
    OFM_COMMUNICATION_FAILED     = 2
};

//--- Data Recorder Definitions --------------------------------------
const std::string RDA_FILE_VERSION = "rdaFileVersion_1.00";

const unsigned short MAX_DATA_RECORDER_BUFFER_SIZE           = 1024;
const unsigned short MAX_NB_OF_DATA_RECORDER_VARIABLES       = 4;

struct SRecorderConfiguration
{
    unsigned short usMovementTrigger:     1;
    unsigned short usErrorTrigger:        1;
    unsigned short usDigInputTrigger:     1;
    unsigned short usEndOfProfileTrigger: 1;
    unsigned short usReserved:            11;
    unsigned short usAddressMode:         1;
};

enum ERecorderStatus
{
    RS_STOPPED          = 0x0000,
    RS_RUNNING          = 0x0001,
    RS_TRIGGERED1       = 0x0002,
    RS_TRIGGERED2       = 0x0003,
};

enum ERecorderTriggerType
{
    RTT_MOVEMENT_START_TRIGGER      = 1, //bit 1
    RTT_ERROR_TRIGGER               = 2, //bit 2
    RTT_DIGITAL_INPUT_TRIGGER       = 4, //bit 3
    RTT_MOVEMENT_END_TRIGGER        = 8  //bit 4
};

enum EDataType
{
    DT_BYTE,
    DT_WORD,
    DT_INT,
    DT_LWORD,
    DT_LINT,
    DT_FLOAT,
    DT_MEMORY_BLOCK,
    DT_BITS8,
    DT_BITS16,
    DT_BITS32,
    DT_BOOL,
    DT_UNKNOWN
};

enum EObjectValueStringFormat
{
    OVF_UNKNOWN         = 0,
    OVF_DEC             = 1,
    OVF_HEX             = 2,
    OVF_VOID            = 3,
};

#ifdef WINVER

    static EOperationMode ConvertToOperationMode(int p_value)
    {
        EOperationMode operationMode(OM_UNKNOWN_MODE);

        switch(p_value)
        {
        case OM_UNKNOWN_MODE:
        case OM_PROFILE_POSITION_MODE:
        case OM_PROFILE_VELOCITY_MODE:
        case OM_PROFILE_TORQUE_MODE:

        case OM_HOMING_MODE:
        case OM_INTERPOLATED_POSITION_MODE:
        case OM_CYCLIC_SYNC_POSITION_MODE:
        case OM_CYCLIC_SYNC_VELOCITY_MODE:
        case OM_CYCLIC_SYNC_TORQUE_MODE:
        case OM_POSITION_MODE:
        case OM_VELOCITY_MODE:
        case OM_CURRENT_MODE:
        case OM_DIAGNOSTIC_MODE:
        case OM_MASTER_ENCODER_MODE:
        case OM_STEP_DIRECTION_MODE:
        case OM_POSITION_TUNING_MODE:
        case OM_CURRENT_TUNING_MODE:
        case OM_CURRENT_PULSE_MODE:
            {
                operationMode = (EOperationMode)p_value;
            }
            break;
        default:
            break;
        }
        return operationMode;
    }
#endif
