#if !defined(XML_PARSER_H)
#define XML_PARSER_H

#if _MSC_VER > 1000
#pragma once
#endif //_MSC_VER > 1000

#include <vector>
#include <StdString.h>
#include <Xml/rapidxml.hpp>

class XML_PARSER
{
public:
    XML_PARSER();
    virtual ~XML_PARSER();

    //Document Functions
    bool Load_XML_Document(CStdString p_FileName);
    bool Save_XML_Document(CStdString p_FileName);
    bool Load_XML_From_Buffer(CStdStringA* p_pBuffer);
    bool Load_XML_From_Buffer(char* p_pBuffer, size_t p_BufferSize);
    bool Reset_XML_Document();
    bool Get_XML_Document(CStdString* p_pDocument);

    //Header
    bool Set_Header(CStdString p_Header, CStdString p_Name, CStdString p_Value);

    //Elements
    bool Add_FirstChildNode(CStdString p_Name);
    bool Add_LastChildNode(CStdString p_Name);
    CStdString Get_TextValue();
    CStdString Get_CurrentTag();
    CStdString Get_CurrentTagName();
    bool Set_TextValue(CStdString p_TextValue);

    //Element Navigation
    bool Go_to_Parent();                                
    int  Get_Child_Count();                             
    bool Go_to_Child();                                 
    bool Go_Forward();                                  
    bool Go_to_Child(CStdString p_NodeName);              
    bool Go_Forward(CStdString p_NodeName);               
    void Go_to_Root();                                  
    bool Go_to_Node(CStdString p_XPathNode);              
    bool Is_Root();                                     
    bool Is_Tag(CStdString p_Tag);                       
    bool Is_Current_Node();

    //Attributes
    bool Set_Attribute(CStdString p_AttribName, CStdString p_AttribValue);
    bool Is_Having_Attribute(CStdString p_Name);
    CStdString Get_Attribute_Value(int p_lIndex);
    CStdString Get_Attribute_Value();
    int Get_Attribute_Count();

    //Helper Functions
    bool Clone_Node(XML_PARSER* p_pSourceDocument, CStdString p_XPathNode, int p_lNodeIndex, rapidxml::xml_node<char>** p_pClonedNode);
    bool Copy_Node(XML_PARSER* p_pSourceDocument, CStdString p_XPathSourceNode, int p_lSourceNodeIndex, CStdString p_XPathTarget);
    bool Remove_Node(CStdString p_XPathNode);
    bool Remove_Node();
    bool Remove_Child(CStdString p_NodeName);
    bool Remove_Childs(CStdString p_NodeName);

    rapidxml::xml_node<char>* Clone_Node(const rapidxml::xml_node<char>* p_pSource);
    rapidxml::xml_node<char>* Clone_CurrentNode();

    bool FormatDocument();
    CStdString GetXml();

    //Tabbing
    void IncTabLevel();
    void DecTabLevel();
    bool AddNewLineTab();

    //Parsing Callback
    virtual void Parse_XML_Document();                  //===> YOU MUST IMPLEMENT IT FOR PARSING PROCESS <===

private:
    void CheckError();
    bool Append_Child(CStdString p_XPathNode, rapidxml::xml_node<char>* p_pChildNode);
    rapidxml::xml_node<char>* SelectSingleNode(CStdString p_XPathNode, int p_lNodeIndex);
    void Parse_Objects(rapidxml::xml_node<char>* p_pObject);

private:
    rapidxml::xml_document<char> m_Doc;
    rapidxml::xml_node<char>* m_pCurrentNode;
    rapidxml::xml_attribute<char>* m_pCurrentAttribute;
    CStdString m_LastErrorDescription;    
    int m_lTabLevel;
};

#endif