#pragma once

#include <StdString.h>
#include <MmcTypeDefinition.h>
#include <fstream>

#include <ObjectDictionary/ObjectEntryIterator.h>
#include <ObjectDictionary/ObjectEntryBaseIterator.h>
#include <ObjectDictionary/ObjectEntry.h>
#include <ObjectDictionary/ObjectFilter.h>
#include <Classes/XML_PARSER.h>

class CObjectDictionary: public XML_PARSER
{
public:
    //Object
    BOOL AddObject(CObjectEntryBase* p_pNewObjectEntry);
    BOOL DeleteObject(CObjectEntryBase* p_pObjectEntry);
    BOOL DeleteAllObjects();
    CObjectEntry* SearchObject(WORD p_usIndex, WORD p_usSubIndex);
    CObjectEntryBase* SearchObject(WORD p_usIndex);
    CObjectEntry* SearchObjectReverse(WORD p_usIndex, WORD p_usSubIndex);
    CObjectEntryBase* SearchObjectReverse(WORD p_usIndex);
    BOOL ObjectExists(CObjectEntryBase* p_pObjectEntry);

    //Iteration
    CObjectEntryIterator* CreateObjectEntryIterator();
    CObjectEntryBaseIterator* CreateObjectEntryBaseIterator();

    //Information
    int GetNumberOfObjects();
    int GetNumberOfSubObjects(WORD p_usIndex);
    CStdString GetObjectName(WORD p_usIndex, WORD p_usSubIndex);
    BOOL GetHighLimit(WORD p_usIndex, WORD p_usSubIndex, CStdString *p_pHighLimit);
    BOOL GetLowLimit(WORD p_usIndex, WORD p_usSubIndex, CStdString *p_pLowLimit);

    //Object Filter
    BOOL SetInternalObjectFilter(BOOL p_oActive);
    BOOL SetObjectFilter(CObjectFilter p_ObjectFilter);
    BOOL ResetObjectFilter();
    BOOL IsObjectFilterEmpty();

    BOOL StoreObjectDictionarySchema(CStdString* p_pXmlData);
    BOOL LoadObjectDictionarySchema(CStdStringA* p_pXmlData);

    CObjectDictionary();
    virtual ~CObjectDictionary();

    //Common
    CObjectDictionary& operator=(CObjectDictionary& p_rObjectDictionary);
    void Serialize(CArchive& p_rArchive);
    BOOL Reset();
    BOOL ResetValues();

    void SortObjects();

private:
    BOOL DeleteObjectList();
    void Parse_XML_Document();

    static bool IndexCompare(CObjectEntryBase* p_pLeftObjectEntryBase, CObjectEntryBase* p_pRightObjectEntryBase);

private:
    tObjectEntryBaseList m_ObjectList;
    CObjectFilter m_ObjectFilter;
};
