#pragma once

#include <Classes/TypeDefinitions.h>
#include <MmcTypeDefinition.h>
#include <ObjectDictionary/ObjectEntryBase.h>
#include <Ui/MmcDragDrop.h>

class XML_PARSER;

class CObjectEntry : public CObjectEntryBase, public CMmcDragDrop
{
public:
    //Initialisation
    void Init() { m_strClassType = "CObjectEntry"; }

    CObjectEntry();
    virtual ~CObjectEntry();

    //Get Information
    BYTE GetSubIndex();
    BOOL GetSubIndex(CStdString& p_rSubIndex);
    EObjectDataType GetDataType();
    WORD GetDataTypeNumber();
    BOOL GetDataTypeNumber(CStdString& p_rDataType);
    DWORD GetSize();
    WORD GetWordSize();
    BOOL GetDataType(CStdString& p_rDataType, BOOL p_oLongString);
    __int64 GetDataTypeMaxValue();
    __int64 GetDataTypeMinValue();
    BOOL GetLowLimit(CStdString& p_rLowLimit);
    BOOL GetHighLimit(CStdString& p_rHighLimit);
    EAccessType GetAccessType();
    BOOL GetAccessType(CStdString& p_rAccessType, bool p_oLowerCase = false);
    BOOL GetAccessTypeAttribute(CStdString& p_rAccessType);
    BOOL GetDefaultValue(CStdString& p_rDefaultValue);
    BOOL GetPDOMapping(CStdString& p_rPdoMapping);
    BOOL GetPDOMappingToken(CStdString& p_rPdoMapping);
    DWORD GetObjFlags();
    BOOL GetObjFlags(CStdString& p_rObjFlags);
    CStdString GetDataTypeLongStr();
    CStdString GetParameterNameStr();

    //Get Value
    BOOL GetValue(BYTE* p_pulValue, DWORD p_ulValueSize);
    BOOL GetValue(CStdString& p_rValue, EObjectValueStringFormat p_eFormat = OVF_UNKNOWN);
    BOOL IsValueAvailable();
    DWORD GetValueSize();
    EObjectValueStringFormat GetObjectValueStringFormat();

    //Set Information
    BOOL SetSubIndex(BYTE p_ubSubIndex);
    BOOL SetSubIndex(CStdString p_subIndex);
    BOOL SetDataType(WORD p_ulDataType);
    BOOL SetDataTypeLongStr(CStdString p_dataType);
    BOOL SetDataType(CStdString p_dataType);
    BOOL SetLowLimit(CStdString p_lowLimit);
    BOOL SetHighLimit(CStdString p_highLimit);
    BOOL SetAccessType(CStdString p_accessType);
    BOOL SetDefaultValue(CStdString p_defaultValue);
    BOOL SetPDOMapping(BYTE p_ubPdoMapping);
    BOOL SetPDOMapping(CStdString p_strPDOMapping);
    BOOL SetPDOMappingToken(CStdString p_strPDOMapping);
    BOOL SetObjFlags(DWORD p_ulObjFlags);
    BOOL SetObjFlags(CStdString p_objFlags);

    //Unit
    BOOL SetUnit(float p_fMultiplier, CStdString p_Unit);
    BOOL GetUnit(float &p_rfMultiplier, CStdString &p_rUnit);

    //Set Value
    BOOL SetValue(BYTE* p_pValue, DWORD p_ulValueSize);
    BOOL SetValue(CStdString p_strValue);
    BOOL SetValueStringFormat(CStdString p_strFormattedValue);

    //Information
    CObjectEntry* SearchObject(WORD p_usIndex, WORD p_usSubIndex);
    int GetNumberOfSubObjects();
    virtual BOOL IsVisible();

    //Common
    virtual BOOL Reset();
    virtual BOOL operator==(CObjectEntry& other);
    virtual CObjectEntry& operator=(CObjectEntry& other);
    virtual CObjectEntryBase* Clone();
    CObjectEntry* CloneObjectEntry();
    BOOL IsEqual(void* p_pValue, DWORD p_ulValueSize);
    BOOL IsEqual(CObjectEntry& other);
    virtual BOOL ResetValue();

    //PDO Mapping
    BOOL IsPdoMappable();
    BOOL IsRxPdoMappable();
    BOOL IsTxPdoMappable();

    //Access Information
    BOOL IsWritable();
    BOOL IsReadable();
    BOOL IsWriteOnDownloadAllowed();
    BOOL IsReadOnUploadAllowed();

    //Xml
    virtual void Serialize(CArchive &ar);
    BOOL StoreObjectDictionarySchema(XML_PARSER* p_pXmlParser);
    void Parse_XML_Document(XML_PARSER* p_pXmlParser);

private:
    //Common
    BOOL AddClonedObjectEntry(tObjectEntryList* p_pObjectEntryList, CObjectEntry* p_pObjectEntry);
    BOOL InitValue(DWORD p_ulSize);
    CStdString GetDataTypeStr();
    BOOL StartsWith(CStdString stringValue, CStdString stringStart);

private:
    BYTE m_uSubIndex;
    EObjectDataType m_DataType;
    CStdString m_strLowLimit;
    CStdString m_strHighLimit;
    EAccessType m_AccessType;
    CStdString m_strDefault;
    BOOL m_oPDOMapping;
    CStdString m_strObjFlags;
    CStdString m_Unit;
    float m_fMultiplier;

    BYTE* m_pValue;
    DWORD m_dValueSize;
    EObjectValueStringFormat m_ValueStringFormat;
};
