#pragma once

#include <map>
#include <MmcTypeDefinition.h>

class CXmlReader;
class CXmlWriter;

class CObjectFilter
{
public:
    //Persistence
    BOOL ReadFrom(CXmlReader* p_pXmlReader, int p_lCount);
    BOOL WriteTo(CXmlWriter* p_pXmlWriter);

    BOOL SetInternalObjectFilter(BOOL p_oActive);
    void SetServiceModeActive(BOOL p_oActive);
    void Serialize(CArchive& p_rArchive);
    BOOL IsVisibleObject(CStdString p_ObjectName, WORD p_usIndex, BYTE p_ubSubIndex);
    BOOL IsVisibleStructuredObject(CStdString p_ObjectName, WORD p_usIndex);
    BOOL IsVisibleObjectExisting(WORD p_usIndex, BYTE p_ubSubIndex);
    BOOL IsVisibleStructuredObjectExisting(WORD p_usIndex);

    BOOL IsEmpty();
    void SetName(CStdString p_Name);
    void SetDescription(CStdString p_Description);
    CStdString GetName();
    CStdString GetDescription();
    CObjectFilter();
    CObjectFilter(CObjectFilter& p_rObjectFilter);
    virtual ~CObjectFilter();

    BOOL Reset();
    BOOL LoadObjectFilterSchema(CStdString* p_pXmlData);
    BOOL StoreObjectFilterSchema(CStdString* p_pXmlData);

    CObjectFilter& operator=(CObjectFilter &p_rObjectFilter);
    BOOL AddVisibleStructuredObject(WORD p_usIndex);
    BOOL AddVisibleObject(WORD p_usIndex, BYTE p_ubSubIndex);
    BOOL DeleteVisibleObject(WORD p_usIndex, BYTE p_ubSubIndex);
    BOOL DeleteVisibleStructuredObject(WORD p_usIndex);
    BOOL DeleteVisibleStructuredObjects(WORD p_usIndex);
    BOOL DeleteAllVisibleObjects();

private:
    BOOL Parse_XML_Document(CXmlReader* p_pXmlReader);
    BOOL IsObjectNameVisible(CStdString p_ObjectName);
    BOOL AddVisibleStructuredObject(CStdString p_Index);
    BOOL AddVisibleObject(CStdString p_Index, CStdString p_SubIndex);

    //Persistence
    BOOL ReadStructuredObjectFilterSchemaFrom(CXmlReader* p_pXmlReader);
    BOOL ReadVisibleObjectFilterSchemaFrom(CXmlReader* p_pXmlReader);
    BOOL WriteVisibleObjectFilterSchemaTo(CXmlWriter* p_pXmlWriter);
    BOOL WriteStructuredObjectFilterSchemaTo(CXmlWriter* p_pXmlWriter);

private:
    BOOL m_oFilterIsEmpty;
    BOOL m_oServiceModeActive;
    BOOL m_oInternalObjectsVisible;
    CStdString m_Name;
    CStdString m_Description;
    std::map<WORD, std::vector<BYTE> > m_VisibleObject;
    std::vector<WORD> m_VisibleStructuredObject;
};
