#include <stdafx.h>
#include <math.h>
#include <Math/MmcMath.h>
#include <PathPlanner/TrapezePositionTrajectory.h>
#include <PathPlanner/PositionPathPlanner.h>

CPositionPathPlanner::CPositionPathPlanner()
    : m_ulIncrementsPerTurn(0)
    , m_dProfileVelocity(0)
    , m_dProfileAcceleration(0)
    , m_dProfileDeceleration(0)    
    , m_dInitialVelocity(0)    
{    
}

CPositionPathPlanner::~CPositionPathPlanner()
{
}

void CPositionPathPlanner::SetMainSensorResolution(DWORD p_ulIncrementsPerTurn)
{
    m_ulIncrementsPerTurn = p_ulIncrementsPerTurn;
}

DWORD CPositionPathPlanner::GetMainSensorResolution()
{
    return m_ulIncrementsPerTurn;
}
    
void CPositionPathPlanner::SetProfileVelocity(double p_ffProfileVelocity)
{
    ConvertRevolutionProMinuteToIncrementProMilisecond(p_ffProfileVelocity, m_dProfileVelocity);
}
    
void CPositionPathPlanner::SetInitialVelocity(double p_ffInitialVelocity)
{
    m_dInitialVelocity = p_ffInitialVelocity;
}

void CPositionPathPlanner::SetProfileAcceleration(double p_ffProfileAcceleration)
{
    ConvertRevolutionProMinuteProSecondToIncrementProMilisecondQuadrat(p_ffProfileAcceleration, m_dProfileAcceleration);
}
    
void CPositionPathPlanner::SetProfileDeceleration(double p_ffProfileDeceleration)
{
    ConvertRevolutionProMinuteProSecondToIncrementProMilisecondQuadrat(p_ffProfileDeceleration, m_dProfileDeceleration);
}

bool CPositionPathPlanner::InitTrajectory(CTrapezePositionTrajectory* p_pTrajectory)
{
    bool oResult = false;

    if(p_pTrajectory)
    {
        p_pTrajectory->SetVelocity(PTP_ACCELERATION, m_dInitialVelocity);        
        p_pTrajectory->SetAcceleration(PTP_ACCELERATION, abs(m_dProfileAcceleration));
        p_pTrajectory->SetVelocity(PTP_CONSTANT, abs(m_dProfileVelocity));
        p_pTrajectory->SetAcceleration(PTP_DECELERATION, abs(m_dProfileDeceleration));

        oResult = true;
    }

    return oResult;
}

BOOL CPositionPathPlanner::GetRunTotalTime(double p_dStartPosition, double p_dStopPosition, long& p_rlTotalTime)
{
    BOOL oResult = TRUE;
    CTrapezePositionTrajectory* pTrajectory = new CTrapezePositionTrajectory();

    if(pTrajectory) 
    {   
        if(InitTrajectory(pTrajectory))
        {
            pTrajectory->SetPosition(PTP_ACCELERATION, p_dStartPosition);
            pTrajectory->SetPosition(PTP_END, p_dStopPosition);

            p_rlTotalTime = pTrajectory->GetTotalTime();
        }

        delete pTrajectory;
    }

    return oResult;
}

void CPositionPathPlanner::ConvertRevolutionProMinuteToIncrementProMilisecond(double p_dRevolutionProMinute, double& p_rdIncrementProMilisecond)
{
    const double MILISECONDS_PRO_MINUTE = 60000;

    p_rdIncrementProMilisecond = p_dRevolutionProMinute*m_ulIncrementsPerTurn/MILISECONDS_PRO_MINUTE;    
}

void CPositionPathPlanner::ConvertRevolutionProMinuteProSecondToIncrementProMilisecondQuadrat(double p_dRevolutionProMinuteproSecond, double& p_rdIncrementProMilisecondQuadrat)
{
    const double MILISECONDS_PRO_MINUTE = 60000;
    const double SECOND_TO_MILISECOND = 1000;

    p_rdIncrementProMilisecondQuadrat = p_dRevolutionProMinuteproSecond*m_ulIncrementsPerTurn/(MILISECONDS_PRO_MINUTE*SECOND_TO_MILISECOND);    
}