
#if !defined(CommunicationModelDefinitions_04082003)
#define CommunicationModelDefinitions_04082003

#include <MmcTypeDefinition.h>

//Tracing definitions
//#define _Trace_CommunicationModelLock

//Addressing
const BYTE          NODE_ID_GATEWAY             = 0x00;
const BYTE          NODE_ID_DEFAULT             = 0x01;
const BYTE          NODE_ID_UNDEFINED           = 0xFF;
const CStdString    NODE_ID_UNDEFINED_TEXT      = _T("Invalid");

const DWORD         VENDOR_ID_MAXON             = 0x000000FB;
const BYTE          STATION_ID_UNDEFINED        = 0x00;

//Transfer rate
const DWORD         TRANSFER_RATE_DEFAULT_CAN_OPEN  = 1000000;
const DWORD         TRANSFER_RATE_AUTO              = (DWORD)-1;
const CStdString    TRANSFER_RATE_AUTO_TEXT         = _T("Auto Bitrate");
const DWORD         TIMEOUT_DEFAULT_CAN_OPEN        = 500;
const DWORD         TIMEOUT_SCANNING_CAN_OPEN       = 100;

//Axis
const WORD          DEFAULT_AXIS_ID                 = 1;

//Device addressing
enum EAddressingMode
{
    AM_UNKNOWN_ADDRESSING,      //AddressingMode is statically defined
    AM_DIRECT_ADDRESSING,       //Addressing via NodeId 0 (Serial)
    AM_NODE_ID_ADDRESSING,      //Addressing via configured NodeId (Serial, CanOpen)
    AM_STATION_ID_ADDRESSING,   //Addressing via configured StationId (EtherCat)
    AM_POSITION_ADDRESSING      //Addressing via position in segment (EtherCat)
};

enum ELayer
{
    NO_LAYER                    = 0,
    INTERFACE_LAYER             = 1,
    PROTOCOL_STACK_LAYER        = 2,
    DEVICE_COMMAND_SET_LAYER    = 3,
    VIRTUAL_COMMAND_SET_LAYER   = 4
};

enum EParameterType
{
    PT_PORT                     = 0,
    PT_INTERFACE                = 1,
    PT_PROTOCOL_STACK           = 2,
    PT_DEVICE                   = 3,
    PT_VIRTUAL_DEVICE           = 4
};

enum ECommandStatus
{
    CMD_IDLE                    = 0,
    CMD_SUCCESSFUL              = 1,
    CMD_FAILED                  = 2,
};

//EtherCAT command types
enum ECommandType
{
    CT_NOP          = 0,
    CT_APRD         = 1, //Auto Increment Physical Read
    CT_APWR         = 2, //Auto Increment Physical Write
    CT_APRW         = 3, //Auto Increment Physical Read/Write
    CT_FPRD         = 4, //Fix Address Physical Read
    CT_FPWR         = 5, //Fix Address Physical Write
    CT_FPRW         = 6, //Fix Address Physical Read/Write
    CT_BRD          = 7, //Broadcast Read
    CT_BWR          = 8, //Broadcast Write
    CT_BRW          = 9, //Broadcast read/write
    CT_LRD          = 10, //Logical Read
    CT_LWR          = 11, //Logical Write
    CT_LRW          = 12, //Logical Read/Write
    CT_ARMW         = 13, //Auto Increment Read Multiple Write
    CT_EXT          = 255,
};

#ifdef WINVER
typedef void (__stdcall *T_JournalCallback)(HANDLE, TCHAR*, int);
#endif

//Definitionen Layer Strings
const TCHAR NO_LAYER_STRING[]                   = _T("No Layer");
const TCHAR VIRTUAL_COMMAND_SET_LAYER_STRING[]  = _T("Application Layer");
const TCHAR COMMAND_SET_LAYER_STRING[]          = _T("Device Layer");
const TCHAR PROTOCOL_STACK_LAYER_STRING[]       = _T("Protocol Layer");
const TCHAR INTERFACE_LAYER_STRING[]            = _T("Interface Layer");

//Definition VirtualDevices
const TCHAR VIRTUAL_DEVICE_DRIVE[]              = _T("DRIVE");
const TCHAR VIRTUAL_DEVICE_DRIVE2[]             = _T("DRIVE 2");
const TCHAR VIRTUAL_DEVICE_DRIVE3[]             = _T("DRIVE 3");
const TCHAR VIRTUAL_DEVICE_PLC[]                = _T("PLC");
const TCHAR VIRTUAL_DEVICE_PLC2[]               = _T("PLC 2");
const TCHAR VIRTUAL_DEVICE_SERVO[]              = _T("SERVO");
const TCHAR VIRTUAL_DEVICE_CONTROLLER[]         = _T("CONTROLLER");
const TCHAR VIRTUAL_DEVICE_AXIS[]               = _T("AXIS");

//Definition Devices
const TCHAR DEVICE_EPOS[]                       = _T("EPOS");
const TCHAR DEVICE_ESAM[]                       = _T("EPOS P");
const TCHAR DEVICE_EPOS2[]                      = _T("EPOS2");
const TCHAR DEVICE_EPOS3[]                      = _T("EPOS3");
const TCHAR DEVICE_EPOS4[]                      = _T("EPOS4");
const TCHAR DEVICE_ESAM2[]                      = _T("EPOS2 P");
const TCHAR DEVICE_ESCON[]                      = _T("ESCON");
const TCHAR DEVICE_MAXPOS[]                     = _T("MAXPOS");
const TCHAR DEVICE_HPSC[]                       = _T("HPSC");
const TCHAR DEVICE_UAVESC[]                     = _T("UAV-ESC");
const TCHAR DEVICE_IDX[]                        = _T("IDX");

//Definition Modules
const TCHAR DEVICE_EPOS4_ETHERCAT_MODULE[]            = _T("EtherCAT");

//Defintion ProtocolStacks
const TCHAR PROTOCOL_CAN_OPEN[]                 = _T("CANopen");
const TCHAR PROTOCOL_MAXON_SERIAL_V1[]          = _T("MAXON_RS232");
const TCHAR PROTOCOL_MAXON_SERIAL_V2[]          = _T("MAXON SERIAL V2");
const TCHAR PROTOCOL_INFOTEAM_SERIAL[]          = _T("INFOTEAM SERIAL");
const TCHAR PROTOCOL_ETHERCAT_COE[]             = _T("EtherCAT-CoE");
const TCHAR PROTOCOL_SPI[]                      = _T("SPI");

//Old Definitions ProtocolStacks (only used for compatibility)
const TCHAR PROTOCOL_EPOS2_USB[]                = _T("EPOS2_USB");

//Definition Interfaces
const TCHAR INTERFACE_NI[]                      = _T("NI");
const TCHAR INTERFACE_KVASER[]                  = _T("Kvaser");
const TCHAR INTERFACE_IXXAT[]                   = _T("IXXAT");
const TCHAR INTERFACE_VECTOR[]                  = _T("Vector");
const TCHAR INTERFACE_RS232[]                   = _T("RS232");
const TCHAR INTERFACE_USB[]                     = _T("USB");
const TCHAR INTERFACE_ETH[]                     = _T("ETH");
const TCHAR INTERFACE_SOCKET[]                  = _T("Socket");
const TCHAR INTERFACE_TWINCAT[]                 = _T("TwinCAT");
const TCHAR INTERFACE_IPC[]                     = _T("IPC");
const TCHAR INTERFACE_CAN[]                     = _T("CAN");

//Definition Drivers
const TCHAR DRIVER_NAMED_PIPE[]                 = _T("NAMED PIPE");
const TCHAR DRIVER_VCI2[]                       = _T("VCI2");
const TCHAR DRIVER_VCI3[]                       = _T("VCI3");
const TCHAR DRIVER_VCI4[]                       = _T("VCI4");
const TCHAR DRIVER_NI_CAN[]                     = _T("NI CAN");
const TCHAR DRIVER_NI_XNET[]                    = _T("NI XNET");
const TCHAR DRIVER_RS232[]                      = _T("RS232");
const TCHAR DRIVER_SOCKET_LIB[]                 = _T("SocketLib");
const TCHAR DRIVER_TWIN_CAT_ADS[]               = _T("TwinCAT ADS");
const TCHAR DRIVER_FTD2XX[]                     = _T("FTD2xx");
const TCHAR DRIVER_HID[]                        = _T("HID");
const TCHAR DRIVER_MCP[]                        = _T("MCP");
const TCHAR DRIVER_XLIB[]                       = _T("XL Driver");
const TCHAR DRIVER_KVASER_CAN[]                 = _T("Kvaser CAN");
const TCHAR DRIVER_SOCKETCAN[]                  = _T("SocketCAN");

//Defintion Driver Parameters
const TCHAR USB_VENDOR_PRODUCT_ID_EPOS2[]       = _T("VID_0403&PID_A8B0");
const TCHAR USB_VENDOR_PRODUCT_ID_EPOS4[]       = _T("VID_24E7&PID_3B01");
const TCHAR USB_VENDOR_PRODUCT_ID_ESCON[]       = _T("VID_24E7&PID_1C01");
const TCHAR USB_VENDOR_PRODUCT_ID_MAXPOS[]      = _T("VID_24E7&PID_2D01");
const TCHAR USB_VENDOR_PRODUCT_ID_FTDI[]        = _T("VID_0403&PID_6001");
const TCHAR USB_VENDOR_PRODUCT_ID_MCP2210[]     = _T("VID_04D8&PID_00DE");
const TCHAR USB_VENDOR_PRODUCT_ID_MAXPOS_TEST[] = _T("VID_24E7&PID_0000");
const TCHAR USB_VENDOR_PRODUCT_ID_HPSC[]        = _T("VID_24E7&PID_4E01");
const TCHAR USB_VENDOR_PRODUCT_ID_UAVESC[]      = _T("VID_24E7&PID_5E01");

//Definition CommandGroups
const TCHAR COMMAND_GROUP_STANDARD[]            = _T("Standard");

//Definition CommandSets
const TCHAR COMMAND_SET_CANOPEN_COMPLETE_ACCESS[]                   = _T("CompleteAccess");
const TCHAR COMMAND_SET_CANOPEN_GENERAL_GATEWAY[]                   = _T("CAN Commands");
const TCHAR COMMAND_SET_CANOPEN_LAYER_SETTING_SERVICES[]            = _T("LayerSettingServices");
const TCHAR COMMAND_SET_CANOPEN_NETWORK_MANAGEMENT[]                = _T("NetworkManagement");
const TCHAR COMMAND_SET_CANOPEN_OBJECT_DICTIONARY[]                 = _T("Object Dictionary");
const TCHAR COMMAND_SET_CONFIGURATION[]                             = _T("Configuration");
const TCHAR COMMAND_SET_CONFIGURATION_CONTROLLER[]                  = _T("Controller");
const TCHAR COMMAND_SET_CONFIGURATION_MOTOR[]                       = _T("Motor");
const TCHAR COMMAND_SET_CONFIGURATION_REGULATION[]                  = _T("Regulation");
const TCHAR COMMAND_SET_CONFIGURATION_SAFETY[]                      = _T("Limits");
const TCHAR COMMAND_SET_CONFIGURATION_SENSOR[]                      = _T("Sensor");
const TCHAR COMMAND_SET_CONFIGURATION_UNITS[]                       = _T("Units");
const TCHAR COMMAND_SET_DATA_RECORDER_DATA_RECORDING[]              = _T("Data Recording");
const TCHAR COMMAND_SET_DEVICE_CONTROL[]                            = _T("Device Control");
const TCHAR COMMAND_SET_DEVICE_INFOS[]                              = _T("Version");
const TCHAR COMMAND_SET_ERROR_HANDLING[]                            = _T("Error Handling");
const TCHAR COMMAND_SET_ERROR_HANDLING_ERROR_HISTORY[]              = _T("Error History");
const TCHAR COMMAND_SET_ERROR_HANDLING_RESET[]                      = _T("Error Reset");
const TCHAR COMMAND_SET_ERROR_HANDLING_SAFETY[]                     = _T("Error Safety");
const TCHAR COMMAND_SET_ETHERCAT_APPLICATION[]                      = _T("EtherCAT");
const TCHAR COMMAND_SET_ETHERCAT_EEPROM[]                           = _T("EtherCAT");
const TCHAR COMMAND_SET_ETHERCAT_FOE[]                              = _T("File Transfer");
const TCHAR COMMAND_SET_ETHERCAT_REGISTER[]                         = _T("EtherCAT");
const TCHAR COMMAND_SET_FIRMWARE_DOWNLOAD[]                         = _T("Firmware");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS[]                            = _T("Inputs and Outputs");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_ANALOG_INPUTS[]              = _T("Analog Inputs");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_ANALOG_OUTPUTS[]             = _T("Analog Outputs");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_DIGITAL_INPUTS[]             = _T("Digital Inputs");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_DIGITAL_OUTPUTS[]            = _T("Digital Outputs");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_POSITION_COMPARE[]           = _T("Position Compare");
const TCHAR COMMAND_SET_INPUTS_OUTPUTS_POSITION_MARKER[]            = _T("Position Marker");
const TCHAR COMMAND_SET_MOTION_INFO[]                               = _T("Motion Info");
const TCHAR COMMAND_SET_OPERATION_MODE_CURRENT_MODE[]               = _T("Current Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_CURRENT_PULSE_MODE[]         = _T("Current Pulse Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_CYCLIC_SYNC_TORQUE_MODE[]    = _T("Cyclic Sync Torque Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_HOMING_MODE[]                = _T("Homing Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_INITIALISATION[]             = _T("Operation Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_INTERPOLATED_POSITION_MODE[] = _T("Interpolated Position Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_MASTER_ENCODER_MODE[]        = _T("Master Encoder Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_POSITION_MODE[]              = _T("Position Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_POSITION_MODE_ANALOG[]       = _T("Position Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_PROFILE_POSITION_MODE[]      = _T("Profile Position Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_PROFILE_VELOCITY_MODE[]      = _T("Profile Velocity Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_STEP_DIRECTION_MODE[]        = _T("Step Direction Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_TUNING_MODE[]                = _T("Tuning Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_VELOCITY_MODE[]              = _T("Velocity Mode");
const TCHAR COMMAND_SET_OPERATION_MODE_VELOCITY_MODE_ANALOG[]       = _T("Velocity Mode");
const TCHAR COMMAND_SET_PARAMETER_PERSISTENCE[]                     = _T("Parameter");
const TCHAR COMMAND_SET_PARAMETER_VALUES[]                          = _T("Parameter");
const TCHAR COMMAND_SET_SCADA_INITIALISATION[]                      = _T("Initialisation");
const TCHAR COMMAND_SET_SCADA_PROCESS_IMAGE[]                       = _T("Process Image Access");
const TCHAR COMMAND_SET_SCADA_PROCESS_INPUT_OUTPUT[]                = _T("Process Input Output Access");
const TCHAR COMMAND_SET_SCADA_PROGRAM_STATUS[]                      = _T("Program Status");
const TCHAR COMMAND_SET_SCADA_PROGRAM_VARIABLE[]                    = _T("Program Variable Access");
const TCHAR COMMAND_SET_SCANNING_CONFIGURATION[]                    = _T("Scanning");
const TCHAR COMMAND_SET_SCANNING_IDENTIFICATION[]                   = _T("Scanning");
const TCHAR COMMAND_SET_STATE_MACHINE[]                             = _T("State Machine");
const TCHAR COMMAND_SET_STATE_MACHINE_MAXON[]                       = _T("State Machine maxon");
const TCHAR COMMAND_SET_STATE_MACHINE_CIA402[]                      = _T("State Machine");

#endif    //CommunicationModelDefinitions_04082003
