
#pragma once

#include <Classes/XXMLFile.h>
#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <Classes/TypeDefinitions.h>
#include <MmcTypeDefinition.h>

class CParameterInfo;
class CLayerParameterStack;
class CLayerParameterSet;
class CJournalTime;
class CJournalManagerBase;
class CLayerManagerBase;
class CGateway;
class CErrorInfo;

class CCommandRoot
{
public:
    CCommandRoot();
    virtual ~CCommandRoot();

    DWORD GetParameterArraySize(int p_lParameterIndex);
    DWORD GetReturnParameterArraySize(int p_lParameterIndex);

    CStdString GetLayerStr();
    
    BOOL IsSuccessful();
    BOOL IsIdle();
    void ResetStatus();
    void SetStatus(BOOL p_oResult, CErrorInfo* p_pErrorInfo);
    void SetStatus(BOOL p_oResult, DWORD p_ulErrorCode);

    BOOL IsEditableParameterExisting();
    BOOL IsVisibleParameterExisting();
    BOOL IsVisibleReturnParameterExisting();

    BOOL SetTimeout(DWORD p_ulTimeout);
    BOOL ResetTimeout();
    DWORD GetTimeout();
    BOOL IsTimeoutValid();

    BOOL IsSubLayerLocked();
    BOOL IsSupportedByGateway();

    BOOL LoadFromXMLString(CStdString* p_pDataString);
    BOOL LoadFromXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pCommandElement);
    CXXMLFile::CElement* StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, BOOL p_oCheckVisibility, BOOL p_oCheckGatewaySupport);
    CXXMLFile::CElement* StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, BOOL p_oCheckVisibility = FALSE);
    BOOL StoreToXMLString(CStdString* p_pDataString, BOOL p_oCheckVisibility = FALSE);

    int GetNbOfParameter();
    int GetNbOfReturnParameter();

    BOOL FindParameterIndex(CStdString p_Name, int& p_rIndex);
    BOOL FindReturnParameterIndex(CStdString p_Name, int& p_rIndex);

    BOOL GetParameterInfo(int p_lParameterIndex, CStdString* p_pName, CStdString* p_pType, CStdString* p_pData, EObjectValueStringFormat p_format);
    BOOL GetReturnParameterInfo(int p_lParameterIndex, CStdString* p_pName, CStdString* p_pType, CStdString* p_pData, EObjectValueStringFormat p_format);

    BOOL SetParameterData(int p_lParameterIndex, void* p_pData, size_t p_ulLength);
    BOOL SetReturnParameterData(int p_lParameterIndex, CStdString p_data);
    BOOL SetReturnParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength);
    BOOL SetParameterDataString(int p_lParameterIndex, CStdString p_value, BOOL p_oShowMsg);
    BOOL SetReturnParameterDataString(int p_lParameterIndex, CStdString p_value, BOOL p_oShowMsg);

    BOOL GetReturnParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength);
    BOOL GetParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength);
    BOOL GetParameterDataString(int p_lParameterIndex, CStdString* p_pValue, EObjectValueStringFormat p_format);
    BOOL GetReturnParameterDataString(int p_lParameterIndex, CStdString* p_pValue, EObjectValueStringFormat p_format);

    DWORD GetParameterLength(int p_lParameterIndex);
    DWORD GetReturnParameterLength(int p_lParameterIndex);
    CStdString GetParameterName(int p_lParameterIndex);
    CStdString GetReturnParameterName(int p_lParameterIndex);
    EObjectDataType GetParameterType(int p_lParameterIndex);
    EObjectDataType GetReturnParameterType(int p_lParameterIndex);
    BOOL IsParameterEditable(int p_lParameterIndex);
    BOOL IsParameterVisible(int p_lParameterIndex);
    BOOL IsReturnParameterVisible(int p_lParameterIndex);

    BOOL GetErrorInfo(CErrorInfo* pErrorInfo);
    BOOL GetErrorCode(DWORD* pdErrorCode);

    void InitCommand(CStdString p_commandName, ELayer p_eLayer, DWORD p_ulCommandId);
    void AddParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType);
    void AddParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, BOOL p_oVisible, BOOL p_oEditable);
    void AddParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize);
    void AddParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize, BOOL p_oVisible, BOOL p_oEditable);
    void AddReturnParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType);
    void AddReturnParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, int p_lVisible);
    void AddReturnParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize);
    void AddReturnParameter(int p_lParameterIndex, CStdString p_parameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize, BOOL p_oVisible);

    CStdString GetCommandName();
    DWORD GetCommandId();
    ELayer GetLayer();
#ifdef WINVER
    void Serialize(CArchive& ar);
#endif
    CCommandRoot& operator=(CCommandRoot& other);
    CCommandRoot* CloneCommand();

    void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    BOOL InitGateway(CGateway* p_pGateway);

    void ResetJournalManager();
    void ResetGateway();

    void StartCommandTime();
    void StopCommandTime();
    virtual BOOL Execute(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle);

    BOOL DoTransactionEvent(HANDLE p_hTransactionHandle);
    BOOL EnableTransactionEvent(BOOL p_oEnabled);

    CStdString GetJournalRunTimeString();
    CStdString GetCommandTimeString();

    //Layer Parameter Stack
    BOOL PushLayerParameterSet(CLayerParameterSet& p_rParameterSet);
    BOOL SetLayerParameterStack(CLayerParameterStack& p_rParameterStack);
    BOOL GetLayerParameterStack(CLayerParameterStack& p_rParameterStack);
    BOOL GetLayerParameter(ELayer p_eLayer, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    BOOL GetLayerParameter(ELayer p_eLayer, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ResetLayerParameterStack();

    //Logging
    CStdString GetLogMessage(CStdString p_Identifier, BOOL p_oReturnParameter = FALSE);
    BOOL ActivateLogMessage();    
    BOOL DeactivateLogMessage();
    BOOL IsLogMessageActive();
    
protected:
    void ResetCommand();
    void ResetErrorInfo();

private:
    CStdString GetCommandIdStr();
    CStdString GetCommandStatusStr(ECommandStatus p_eCmdStatus);
    CStdString GetErrorCodeStr();

    BOOL InitCommandName(CStdString p_commandName);
    BOOL InitLayer(CStdString p_layer);
    BOOL InitCommandId(CStdString p_commandId);
    BOOL InitCommandStatus(CStdString p_cmdStatus, CStdString p_errorCode);

    void SetJournalRunTime(CStdString* p_pJournalRunTime);
    void SetCommandTime(CStdString* p_pCommandTime);

    CXXMLFile::CElement* FindCommandElement(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement);
    CXXMLFile::CElement* DeleteCommandElement(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, CXXMLFile::CElement* p_pCommandElement);

    BOOL ProcessWithoutHandles();
    BOOL UpdateStatus();

protected:
    CJournalManagerBase* m_pJournalManager;
    CParameterInfo* m_pParameterInfo;
    CLayerParameterStack* m_pLayerParameterStack;
    CGateway* m_pGateway;
    CJournalTime* m_pJournalTime;

    CStdString m_CommandName;
    ELayer m_eLayer;
    DWORD m_dCommandId;
    DWORD m_dTimeout;

    ECommandStatus m_CommandStatus;
    CErrorInfo* m_pErrorInfo;
    BOOL m_oSubLayerRemainsLocked;

private:
    BOOL m_oTransactionEventEnabled;
    BOOL m_oLogMessageActive;
};
