#pragma once

#include <Classes/TypeDefinitions.h>
#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <MmcTypeDefinition.h>

class CParameterBase
{
public:
    CParameterBase();
    virtual ~CParameterBase();
#ifdef WINVER
       virtual void Serialize(CArchive& p_rArchive);
#endif

    virtual CParameterBase& operator=(CParameterBase& p_rOther);

    void Init(int p_lIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize, BOOL p_oEditable, BOOL p_oVisible);
    BOOL InitIndex(CStdString p_index);
    BOOL InitName(CStdString p_name);
    BOOL InitType(CStdString p_type);
    BOOL InitEditable(CStdString p_editable);
    BOOL InitVisible(CStdString p_visible);

    BOOL IsVisible();
    BOOL IsEditable();

    int GetIndex();
    CStdString GetIndexStr();

    EObjectDataType GetType();
    CStdString GetTypeStr();

    BOOL SetValue(CStdString p_valueStr, BOOL p_oShowMsg = FALSE);
    CStdString GetValueStr(EObjectValueStringFormat p_eFormat);

    CStdString GetName();
    CStdString GetEditableStr();
    CStdString GetVisibleStr();
    DWORD GetArraySize();

    BOOL SetData(void* p_pData, DWORD p_ulLength);
    BOOL GetData(void* p_pData, DWORD p_ulLength);
    DWORD GetLength();
    void ResetData();

protected:
    virtual void Reset();

private:
    void DeleteDataBuffer();
    void InitDataBuffer();

    DWORD GetParameterTypeLength(EObjectDataType p_eType);

    CStdString GetVoidValueStr();
    CStdString GetBooleanValueStr(void* p_pData);
    BOOL SetData(void* p_pData, DWORD p_ulLength, DWORD& p_rulOffset);
    BOOL SetVoidValueStr(CStdString p_valueStr, BOOL p_oShowMsg = FALSE);
    BOOL SetBooleanValueStr(CStdString p_valueStr);
    CStdString ExtractSubValueStr(CStdString* p_pValueStr);

protected:
    CStdString m_strName;
    int m_iIndex;
    BOOL m_oEditable;
    BOOL m_oVisible;
    EObjectDataType m_eType;
    DWORD m_dArraySize;

    void* m_pDataBuffer;
    DWORD m_dDataBufferLength;
};
