#pragma once

#include <list>
#include <Classes/TypeDefinitions.h>
#include <Classes/XXMLFile.h>

class CParameter;
class CReturnParameter;

class CParameterInfo
{
public:
    void Reset();
    void AddParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType);
    void AddParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, BOOL p_oVisible, BOOL p_oEditable);
    void AddParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize);
    void AddParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize, BOOL p_oVisible, BOOL p_oEditable);

    void AddReturnParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType);
    void AddReturnParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, int p_ulVisible);
    void AddReturnParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize);
    void AddReturnParameter(int p_lParamIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize, BOOL p_oVisible);

    BOOL IsEditableParameterExisting();
    BOOL IsVisibleParameterExisting();
    BOOL IsVisibleReturnParameterExisting();

    //Parameter
    int GetNbOfParameter();
    BOOL FindParameterIndex(CStdString p_Name, int& p_rIndex);
    BOOL IsParameterExisting(int p_lParamIndex);
    CStdString GetParameterName(int p_lParamIndex);
    EObjectDataType GetParameterType(int p_lParamIndex);
    DWORD GetParameterLength(int p_lParamIndex);
    CStdString GetParameterTypeStr(int p_lParamIndex);
    CStdString GetParameterDataStr(int p_lParamIndex, EObjectValueStringFormat p_eFormat);
    BOOL IsParameterEditable(int p_lParamIndex);
    BOOL IsParameterVisible(int p_lParamIndex);
    DWORD GetParameterArraySize(int p_lParamIndex);

    void ResetParameterData();
    BOOL SetParameterData(int p_lParamIndex, void* p_pData, size_t p_lLength);
    BOOL GetParameterData(int p_lParamIndex, void* p_pData, DWORD p_ulLength);
    BOOL SetParameterDataString(int p_lParamIndex, CStdString p_strValue, BOOL p_oShowMsg);
    BOOL GetParameterDataString(int p_lParamIndex, CStdString* p_pStrValue, EObjectValueStringFormat p_eFormat);

    //ReturnParameter
    int GetNbOfReturnParameter();
    BOOL FindReturnParameterIndex(CStdString p_Name, int& p_rIndex);
    BOOL IsReturnParameterExisting(int p_lParamIndex);
    CStdString GetReturnParameterName(int p_lParamIndex);
    EObjectDataType GetReturnParameterType(int p_lParamIndex);
    DWORD GetReturnParameterLength(int p_lParamIndex);
    CStdString GetReturnParameterTypeStr(int p_lParamIndex);
    CStdString GetReturnParameterDataStr(int p_lParamIndex, EObjectValueStringFormat p_eFormat);
    BOOL IsReturnParameterVisible(int p_lParamIndex);
    DWORD GetReturnParameterArraySize(int p_lParamIndex);

    void ResetReturnParameterData();
    BOOL SetReturnParameterData(int p_lParamIndex, void* p_pDdata, size_t p_lLength);
    BOOL GetReturnParameterData(int p_lParamIndex, void* p_pData, DWORD p_ulLength);
    BOOL SetReturnParameterDataString(int p_lParamIndex, CStdString p_strValue, BOOL p_oShowMsg);
    BOOL GetReturnParameterDataString(int p_lParamIndex, CStdString* p_pStrValue, EObjectValueStringFormat p_eFormat);

    CParameterInfo();
    virtual ~CParameterInfo();
#ifdef WINVER
    void Serialize(CArchive& ar);
#endif
    CParameterInfo& operator=(CParameterInfo& p_rOther);

    BOOL LoadFromXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParameterInfoElement);
    BOOL StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, BOOL p_oCheckVisibility = FALSE);

private:
    CParameter* FindParameter(int p_lParamIndex);
    CReturnParameter* FindReturnParameter(int p_lParamIndex);
    void DeleteParameterList();
    void DeleteReturnParameterList();

private:
    std::list<CParameter*> m_ParameterList;
    std::list<CReturnParameter*> m_ReturnParameterList;
};
