#if !defined(DcsEpos4Def_H__20150528__INCLUDED_)
#define DcsEpos4Def_H__20150528__INCLUDED_

#include <MmcTypeDefinition.h>

//Default protocol stack settings
const DWORD     DEFAULT_TRANSFER_RATE_RS232             = 115200;
const DWORD     DEFAULT_TRANSFER_RATE_USB               = 1000000;
const DWORD     DEFAULT_TRANSFER_RATE_CAN_OPEN          = 1000000;
const DWORD     DEFAULT_TRANSFER_RATE_ETHERCAT          = 100000000;
const DWORD     DEFAULT_TIMEOUT                         = 500;
const DWORD     UNDEFINED_TIMEOUT                       = (DWORD)-1;

//Object dictionary access commands
const DWORD     EPOS4_READ_OBJECT                       = 0x00070010;
const DWORD     EPOS4_INITIATE_SEGMENT_READ             = 0x00070011;
const DWORD     EPOS4_SEGMENT_READ                      = 0x00070012;
const DWORD     EPOS4_WRITE_OBJECT                      = 0x00070013;
const DWORD     EPOS4_INITIATE_SEGMENT_WRITE            = 0x00070014;
const DWORD     EPOS4_SEGMENT_WRITE                     = 0x00070015;
const DWORD     EPOS4_ABORT_TRANSFER                    = 0x00070016;

//Network management commands
const DWORD     EPOS4_SEND_NMT_SERVICE                  = 0x00070020;

//General gateway
const DWORD     EPOS4_SEND_CAN_FRAME                    = 0x00070030;
const DWORD     EPOS4_REQUEST_CAN_FRAME                 = 0x00070031;
const DWORD     EPOS4_READ_CAN_FRAME                    = 0x00070032;

//Layer setting services
const DWORD     EPOS4_SEND_LSS_FRAME                    = 0x00070040;
const DWORD     EPOS4_READ_LSS_FRAME                    = 0x00070041;

//ESC memory commands
const DWORD     EPOS4_READ_ESC_MEMORY                   = 0x00070050;
const DWORD     EPOS4_WRITE_ESC_MEMORY                  = 0x00070051;

//ESM
const DWORD     EPOS4_INIT_ESM                          = 0x00070060;
const DWORD     EPOS4_CHANGE_ESM_STATE                  = 0x00070061;
const DWORD     EPOS4_GET_ESM_STATE                     = 0x00070062;

//FoE
const DWORD     EPOS4_READ_FILE                         = 0x00070070;
const DWORD     EPOS4_WRITE_FILE                        = 0x00070071;
const DWORD     EPOS4_INITIATE_SEGMENT_WRITE_FILE       = 0x00070072;
const DWORD     EPOS4_INITIATE_SEGMENT_READ_FILE        = 0x00070073;
const DWORD     EPOS4_SEGMENT_WRITE_FILE                = 0x00070074;
const DWORD     EPOS4_SEGMENT_READ_FILE                 = 0x00070075;

//Mechanical Tuning Mode
const WORD      START_MECHANICAL_IDENTIFICATION         = 0x001F;
const WORD      STOP_MECHANICAL_IDENTIFICATION          = 0x000F;

//Device command set
#pragma pack(push, 1)
union UEpos4ControlByte
{
    struct SControlByte
    {
        BYTE    bToggle         :1;
        BYTE    bLastSegment    :1;
        BYTE    bDummy          :6;
    }structure;
    BYTE bValue;
};

//Communication state
enum EDeviceComState
{
    DCS_DISCONNECTED,
    DCS_PAUSED,
    DCS_OFFLINE,
    DCS_DISABLED,
    DCS_ENABLED,
    DCS_STOPPED,
    DCS_FAULT,
    DCS_UNKNOWN
};

enum EEpos4ModeOfOperation
{
    MOO_EPOS4_UNKNOWN = 0,
    MOO_EPOS4_PPM = 0x001,
    MOO_EPOS4_PVM = 0x003,
    MOO_EPOS4_HOMING = 0x006,
    MOO_EPOS4_CSP = 0x008,
    MOO_EPOS4_CSV = 0x009,
    MOO_EPOS4_CST = 0x000A,
    MOO_EPOS4_CST_CA = 0x000B
};

// Motor type
const BYTE MOTOR_TYPE_DC_MOTOR                                  = 0x01;
const BYTE MOTOR_TYPE_EC_SINUS                                  = 0x0A;
const BYTE MOTOR_TYPE_EC_BLOCK                                  = 0x0B;

// Sensor
const BYTE SENSOR_NONE                                          = 0x00;
const BYTE SENSOR_1                                             = 0x01;
const BYTE SENSOR_2                                             = 0x02;
const BYTE SENSOR_3                                             = 0x03;

// Sensor type
const BYTE SENSOR_TYPE_NONE                                     = 0x00;
const BYTE SENSOR_TYPE_UNKNOWN                                  = 0x00;
const BYTE SENSOR_TYPE_INC_ENCODER_3_CHANNEL                    = 0x01;
const BYTE SENSOR_TYPE_INC_ENCODER_2_CHANNEL                    = 0x02;
const BYTE SENSOR_TYPE_HALL_SENSORS                             = 0x03;
const BYTE SENSOR_TYPE_SSI_ABS_ENCODER_BINARY                   = 0x04;
const BYTE SENSOR_TYPE_SSI_ABS_ENCODER_GREY                     = 0x05;
const BYTE SENSOR_TYPE_INC_ENCODER2_3_CHANNEL                   = 0x06;
const BYTE SENSOR_TYPE_INC_ENCODER2_2_CHANNEL                   = 0x07;
const BYTE SENSOR_TYPE_ANALOG_INC_ENCODER_3_CHANNEL             = 0x08;
const BYTE SENSOR_TYPE_ANALOG_INC_ENCODER_2_CHANNEL             = 0x09;

// Sensor 1 Type
const BYTE SENSOR1_TYPE_NONE                                    = 0x00;
const BYTE SENSOR1_TYPE_DIGITAL_INC_ENCODER_1                   = 0x01;

// Sensor 2 Type
const BYTE SENSOR2_TYPE_NONE                                    = 0x00;
const BYTE SENSOR2_TYPE_DIGITAL_INC_ENCODER_2                   = 0x01;
const BYTE SENSOR2_TYPE_ANALOG_INC_ENCODER                      = 0x02;
const BYTE SENSOR2_TYPE_SSI_ENCODER                             = 0x03;
const BYTE SENSOR2_TYPE_ENDAT_ENCODER                           = 0x04;
const BYTE SENSOR2_TYPE_BISS                                    = 0x05;

// Sensor 3 Type
const BYTE SENSOR3_TYPE_NONE                                    = 0x00;
const BYTE SENSOR3_TYPE_DIGITAL_HALL_SENSOR                     = 0x10;

//In- and outputs
//Digital input configuration
const BYTE DIC_NEGATIVE_LIMIT_SWITCH                            = 0;
const BYTE DIC_POSITIVE_LIMIT_SWITCH                            = 1;
const BYTE DIC_HOME_SWITCH                                      = 2;
const BYTE DIC_POSITION_MARKER                                  = 3;
const BYTE DIC_DRIVE_ENABLE                                     = 4;
const BYTE DIC_QUICK_STOP                                       = 5;
const BYTE DIC_TOUCH_PROBE_1                                    = 6;
const BYTE DIC_NOT_USED                                         = 7;
const BYTE DIC_GENERAL_PURPOSE_H                                = 8;
const BYTE DIC_GENERAL_PURPOSE_G                                = 9;
const BYTE DIC_GENERAL_PURPOSE_F                                = 10;
const BYTE DIC_GENERAL_PURPOSE_E                                = 11;
const BYTE DIC_GENERAL_PURPOSE_D                                = 12;
const BYTE DIC_GENERAL_PURPOSE_C                                = 13;
const BYTE DIC_GENERAL_PURPOSE_B                                = 14;
const BYTE DIC_GENERAL_PURPOSE_A                                = 15;
const BYTE DIC_NO_FUNCTIONALITY                                 = 255;

//Digital input configuration EPOS4
const BYTE EPOS4_DIC_NEGATIVE_LIMIT_SWITCH_CHECK                = 24;
const BYTE EPOS4_DIC_POSITIVE_LIMIT_SWITCH_CHECK                = 25;
const BYTE EPOS4_DIC_NEGATIVE_LIMIT_SWITCH                      = 0;
const BYTE EPOS4_DIC_POSITIVE_LIMIT_SWITCH                      = 1;
const BYTE EPOS4_DIC_HOME_SWITCH                                = 2;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_H                          = 23;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_G                          = 22;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_F                          = 21;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_E                          = 20;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_D                          = 19;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_C                          = 18;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_B                          = 17;
const BYTE EPOS4_DIC_GENERAL_PURPOSE_A                          = 16;
const BYTE EPOS4_DIC_TOUCH_PROBE_1                              = 26;
const BYTE EPOS4_DIC_DRIVE_ENABLE                               = 27;
const BYTE EPOS4_DIC_QUICK_STOP                                 = 28;
const BYTE EPOS4_DIC_NO_FUNCTIONALITY                           = 255;

//Digital output configuration
const BYTE DOC_READY_FAULT                                      = 0;
const BYTE DOC_POSITION_COMPARE                                 = 1;
const BYTE DOC_HOLDING_BRAKE                                    = 2;
const BYTE DOC_NOT_USED1                                        = 3;
const BYTE DOC_NOT_USED2                                        = 4;
const BYTE DOC_NOT_USED3                                        = 5;
const BYTE DOC_NOT_USED4                                        = 6;
const BYTE DOC_SET_BRAKE                                        = 7;
const BYTE DOC_GENERAL_PURPOSE_H                                = 8;
const BYTE DOC_GENERAL_PURPOSE_G                                = 9;
const BYTE DOC_GENERAL_PURPOSE_F                                = 10;
const BYTE DOC_GENERAL_PURPOSE_E                                = 11;
const BYTE DOC_GENERAL_PURPOSE_D                                = 12;
const BYTE DOC_GENERAL_PURPOSE_C                                = 13;
const BYTE DOC_GENERAL_PURPOSE_B                                = 14;
const BYTE DOC_GENERAL_PURPOSE_A                                = 15;

//Digital output configuration EPOS4
const BYTE EPOS4_DOC_READY_FAULT                                = 25;
const BYTE EPOS4_DOC_HOLDING_BRAKE                              = 24;
const BYTE EPOS4_DOC_GENERAL_PURPOSE_C                          = 18;
const BYTE EPOS4_DOC_GENERAL_PURPOSE_B                          = 17;
const BYTE EPOS4_DOC_GENERAL_PURPOSE_A                          = 16;
const BYTE EPOS4_DOC_NO_FUNCTIONALITY                           = 255;

//Analog input configuration
const BYTE AIC_ANALOG_CURRENT_SETPOINT                          = 0;
const BYTE AIC_ANALOG_VELOCITY_SETPOINT                         = 1;
const BYTE AIC_ANALOG_POSITION_SETPOINT                         = 2;
const BYTE AIC_ANALOG_GENERAL_PURPOSE_A                         = 16;

// Regulation Tuning
// EController
const WORD PI_CURRENT_CONTROLLER                                = 1;
const WORD PI_VELOCITY_CONTROLLER                               = 10;
const WORD PI_VELOCITY_CONTROLLER_WITH_OBSERVER                 = 11;
const WORD PID_POSITION_CONTROLLER                              = 20;
const WORD DUAL_LOOP_POSITION_CONTROLLER                        = 21;

// EGain (PI_CURRENT_CONTROLLER)
const WORD PICC_P_GAIN                                          = 1;
const WORD PICC_I_GAIN                                          = 2;

// EGain (PI_VELOCITY_CONTROLLER)
const WORD PIVC_P_GAIN                                          = 1;
const WORD PIVC_I_GAIN                                          = 2;
const WORD PIVC_FEED_FORWARD_VELOCITY_GAIN                      = 10;
const WORD PIVC_FEED_FORWARD_ACCELERATION_GAIN                  = 11;

// EGain (PI_VELOCITY_CONTROLLER_WITH_OBSERVER)
const WORD PIVCWO_P_GAIN                                        = 1;
const WORD PIVCWO_I_GAIN                                        = 2;
const WORD PIVCWO_FEED_FORWARD_VELOCITY_GAIN                    = 10;
const WORD PIVCWO_FEED_FORWARD_ACCELERATION_GAIN                = 11;
const WORD PIVCWO_OBSERVER_THETA_GAIN                           = 20;
const WORD PIVCWO_OBSERVER_OMEGA_GAIN                           = 21;
const WORD PIVCWO_OBSERVER_TAU_GAIN                             = 22;

// EGain (PID_POSITION_CONTROLLER)
const WORD PIDPC_P_GAIN                                         = 1;
const WORD PIDPC_I_GAIN                                         = 2;
const WORD PIDPC_D_GAIN                                         = 3;
const WORD PIDPC_FEED_FORWARD_VELOCITY_GAIN                     = 10;
const WORD PIDPC_FEED_FORWARD_ACCELERATION_GAIN                 = 11;

// EGain (DUAL_LOOP_POSITION_CONTROLLER)
const WORD DLPC_AUXILIARY_LOOP_P_GAIN                           = 1;
const WORD DLPC_AUXILIARY_LOOP_I_GAIN                           = 2;
const WORD DLPC_AUXILIARY_LOOP_FEED_FORWARD_VELOCITY_GAIN       = 10;
const WORD DLPC_AUXILIARY_LOOP_FEED_FORWARD_ACCELERATION_GAIN   = 11;
const WORD DLPC_AUXILIARY_LOOP_OBSERVER_THETA_GAIN              = 20;
const WORD DLPC_AUXILIARY_LOOP_OBSERVER_OMEGA_GAIN              = 21;
const WORD DLPC_AUXILIARY_LOOP_OBSERVER_TAU_GAIN                = 22;
const WORD DLPC_MAIN_LOOP_P_GAIN_LOW                            = 101;
const WORD DLPC_MAIN_LOOP_P_GAIN_HIGH                           = 102;
const WORD DLPC_MAIN_LOOP_GAIN_SCHEDULING_WEIGHT                = 110;
const WORD DLPC_MAIN_LOOP_FILTER_COEFFICIENT_A                  = 120;
const WORD DLPC_MAIN_LOOP_FILTER_COEFFICIENT_B                  = 121;
const WORD DLPC_MAIN_LOOP_FILTER_COEFFICIENT_C                  = 122;
const WORD DLPC_MAIN_LOOP_FILTER_COEFFICIENT_D                  = 123;
const WORD DLPC_MAIN_LOOP_FILTER_COEFFICIENT_E                  = 124;

//Units
//VelocityDimension
const BYTE VD_RPM                                               = 0xA4;

//Units EPOS4
const BYTE EPOS4_UNIT_REVOLUTION                                = 0xB4;
const BYTE EPOS4_UNIT_INCREMENT                                 = 0xB5;
const BYTE EPOS4_UNIT_SECOND                                    = 0x03;
const BYTE EPOS4_UNIT_MINUT                                     = 0x47;

// Registry constants
const TCHAR REGISTRY_MAXON[]        = _T("Software\\maxon motor ag");
const TCHAR REGISTRY_USER_LEVEL[]   = _T("UserLevel");

#pragma pack(pop)

const WORD      INDEX_PROGRAM_DATA                              = 0x1F50;
const BYTE      SUBINDEX_PROGRAM_DATA_PRG_NB_1                  = 0x01;

const WORD      INDEX_PROGRAM_CONTROL                           = 0x1F51;
const BYTE      SUBINDEX_PROGRAM_CONTROL_PRG_NB_1               = 0x01;

const WORD      INDEX_PROGRAM_SOFTWARE_IDENTIFICATION               = 0x1F56;
const BYTE      SUBINDEX_PROGRAM_SOFTWARE_IDENTIFICATION_PRG_NB_1   = 0x01;

const WORD      INDEX_FLASH_STATUS_IDENTIFICATION               = 0x1F57;
const BYTE      SUBINDEX_FLASH_STATUS_IDENTIFICATION_PRG_NB_1   = 0x01;

const WORD      INDEX_NODE_ID                                   = 0x2000;
const BYTE      SUBINDEX_NODE_ID                                = 0x00;

const WORD      INDEX_CAN_BITRATE                               = 0x2001;
const BYTE      SUBINDEX_CAN_BITRATE                            = 0x00;

const WORD      INDEX_BAUDRATE                                  = 0x2002;
const BYTE      SUBINDEX_BAUDRATE                               = 0x00;

const WORD      INDEX_RS232_FRAME_TIMEOUT                       = 0x2005;
const BYTE      SUBINDEX_RS232_FRAME_TIMEOUT                    = 0x00;

const WORD      INDEX_USB_FRAME_TIMEOUT                         = 0x2006;
const BYTE      SUBINDEX_USB_FRAME_TIMEOUT                      = 0x00;

const WORD      INDEX_ACTIVE_FIELD_BUS                          = 0x2010;
const BYTE      SUBINDEX_ACTIVE_FIELD_BUS                       = 0x00;

const WORD      INDEX_ACTIVE_CONTROLLER_TYPE                    = 0x2011;
const BYTE      SUBINDEX_ACTIVE_CONTROLLER_TYPE                 = 0x00;

const WORD      INDEX_SERIAL_NUMBER                             = 0x2100;
const BYTE      SUBINDEX_SERIAL_NUMBER                          = 0x01;

const WORD      INDEX_EXTENSION_1_VERSION                       = 0x2101;
const BYTE      SUBINDEX_EXTENSION_1_SW_VERSION                 = 0x01;
const BYTE      SUBINDEX_EXTENSION_1_HW_VERSION                 = 0x02;
const BYTE      SUBINDEX_EXTENSION_1_APP_NUMBER                 = 0x03;
const BYTE      SUBINDEX_EXTENSION_1_APP_VERSION                = 0x04;
const BYTE      SUBINDEX_EXTENSION_1_SERIAL_NUMBER              = 0x05;
const BYTE      SUBINDEX_EXTENSION_1_TYPE                       = 0x06;

const WORD      INDEX_POWER_SUPPLY                              = 0x2200;
const BYTE      SUBINDEX_POWER_SUPPLY_VOLTAGE                   = 0x01;
const BYTE      SUBINDEX_LOGIC_SUPPLY_STATUS                    = 0x02;

const WORD      INDEX_INTERNAL_DEVICE_CONTROL                   = 0x2EC0;
const BYTE      SUBINDEX_INTERNAL_BOOTLOADER_REVISION_NUMBER    = 0x03;
const BYTE      SUBINDEX_INTERNAL_BOOTLOADER_PRODUCT_CODE       = 0x04;
const BYTE      SUBINDEX_INTERNAL_ESM_STATE                     = 0x06;

const WORD      INDEX_INTERNAL_CTO_REFERENCE_PARAMETER          = 0x2EC9;
const BYTE      SUBINDEX_INTERNAL_CTO_SYSTEM_WITH_GEAR          = 0x0A;

//*********************************************************************************
//Axis Object Definitions (0x3000 - 0x32FF)
//*********************************************************************************

//EPOS4 Op Codes
const WORD      EPOS4_INDEX_SENSOR_DATA                             = 0x3000;
const BYTE      EPOS4_SUBINDEX_SENSORS_CONFIGURATION                = 0x01;
const BYTE      EPOS4_SUBINDEX_CONTROL_STRUCTURE                    = 0x02;
const BYTE      EPOS4_SUBINDEX_COMMUTATIONS_SENSORS                 = 0x03;
const BYTE      EPOS4_SUBINDEX_AXIS_CONFIG_MISCELLANEOUS            = 0x04;
const BYTE      EPOS4_SUBINDEX_MAIN_SENSOR_RESOLUTION               = 0x05;

const WORD      EPOS4_INDEX_MOTOR_DATA                              = 0x3001;
const BYTE      EPOS4_SUBINDEX_MAX_CON_CURRENT                      = 0x01;
const BYTE      EPOS4_SUBINDEX_MAX_PEAK_CURRENT                     = 0x02;
const BYTE      EPOS4_SUBINDEX_POLE_PAIR_NUMBER                     = 0x03;
const BYTE      EPOS4_SUBINDEX_THERMAL_TIME_CONSTANT                = 0x04;
const BYTE      EPOS4_SUBINDEX_TORQUE_CONSTANT                      = 0x05;

const WORD      EPOS4_INDEX_ELECTRICAL_SYSTEM_PARAMETERS            = 0x3002;
const BYTE      EPOS4_SUBINDEX_ELECTRICAL_RESISTANCE                = 0x01;
const BYTE      EPOS4_SUBINDEX_ELECTRICAL_INDUCTANCE                = 0x02;

const WORD      EPOS4_INDEX_GEAR_CONFIGURATION                      = 0x3003;
const BYTE      EPOS4_SUBINDEX_GEAR_REDUCTION_NUMERATOR             = 0x01;
const BYTE      EPOS4_SUBINDEX_GEAR_REDUCTION_DENOMINATOR           = 0x02;
const BYTE      EPOS4_SUBINDEX_MAX_GEAR_INPUT_SPEED                 = 0x03;

const WORD      EPOS4_INDEX_DIGITAL_INC_ENCODER_1                   = 0x3010;
const BYTE      EPOS4_SUBINDEX_ENCODER_1_NUMBER_OF_PULSES           = 0x01;
const BYTE      EPOS4_SUBINDEX_ENCODER_1_TYPE                       = 0x02;

const WORD      EPOS4_INDEX_ANALOG_INC_ENCODER                      = 0x3011;
const BYTE      EPOS4_SUBINDEX_ANALOG_ENCODER_TYPE                  = 0x01;
const BYTE      EPOS4_SUBINDEX_ANALOG_ENCODER_RESOLUTION            = 0x02;

const WORD      EPOS4_INDEX_SSI_ABSOLUTE_ENCODER                    = 0x3012;
const BYTE      EPOS4_SUBINDEX_SSI_DATARATE                         = 0x01;
const BYTE      EPOS4_SUBINDEX_SSI_NUMBER_OF_DATABITS               = 0x02;
const BYTE      EPOS4_SUBINDEX_SSI_ENCODING_TYPE                    = 0x03;
const BYTE      EPOS4_SUBINDEX_SSI_TIMEOUT_TIME                     = 0x05;
const BYTE      EPOS4_SUBINDEX_SSI_POWERUP_TIME                     = 0x08;

const WORD      EPOS4_INDEX_DIGITAL_HALL_SENSOR                     = 0x301A;
const BYTE      EPOS4_SUBINDEX_HALL_SENSOR_TYPE                     = 0x01;

const WORD      EPOS4_INDEX_DIGITAL_INC_ENCODER_2                   = 0x3020;
const BYTE      EPOS4_SUBINDEX_ENCODER_2_NUMBER_OF_PULSES           = 0x01;
const BYTE      EPOS4_SUBINDEX_ENCODER_2_TYPE                       = 0x02;

const WORD      EPOS4_INDEX_CURRENT_GAIN                            = 0x30A0;
const BYTE      EPOS4_SUBINDEX_CURRENT_P_GAIN                       = 0x01;
const BYTE      EPOS4_SUBINDEX_CURRENT_I_GAIN                       = 0x02;

const WORD      EPOS4_INDEX_POSITION_GAIN                           = 0x30A1;
const BYTE      EPOS4_SUBINDEX_POSITION_P_GAIN                      = 0x01;
const BYTE      EPOS4_SUBINDEX_POSITION_I_GAIN                      = 0x02;
const BYTE      EPOS4_SUBINDEX_POSITION_D_GAIN                      = 0x03;
const BYTE      EPOS4_SUBINDEX_POSITION_VELOCITY_FEED_FORWARD       = 0x04;
const BYTE      EPOS4_SUBINDEX_POSITION_ACCELERATION_FEED_FORWARD   = 0x05;

const WORD      EPOS4_INDEX_VELOCITY_GAIN                           = 0x30A2;
const BYTE      EPOS4_SUBINDEX_VELOCITY_P_GAIN                      = 0x01;
const BYTE      EPOS4_SUBINDEX_VELOCITY_I_GAIN                      = 0x02;
const BYTE      EPOS4_SUBINDEX_VELOCITY_VELOCITY_FEED_FORWARD       = 0x03;
const BYTE      EPOS4_SUBINDEX_VELOCITY_ACCELERATION_FEED_FORWARD   = 0x04;

const WORD      EPOS4_INDEX_VELOCITY_OBSERVER_GAIN                  = 0x30A3;
const BYTE      EPOS4_SUBINDEX_VELOCITY_OBSERVER_POS_CORR_GAIN      = 0x01;
const BYTE      EPOS4_SUBINDEX_VELOCITY_OBSERVER_VEL_CORR_GAIN      = 0x02;
const BYTE      EPOS4_SUBINDEX_VELOCITY_OBSERVER_LOAD_CORR_GAIN     = 0x03;
const BYTE      EPOS4_SUBINDEX_VELOCITY_OBSERVER_FRICTION           = 0x04;
const BYTE      EPOS4_SUBINDEX_VELOCITY_OBSERVER_INERTIA            = 0x05;

const WORD      EPOS4_INDEX_VELOCITY_FILTER_BANDWIDTH               = 0x30A4;
const BYTE      EPOS4_SUBINDEX_VELOCITY_FILTER_BANDWIDTH            = 0x01;

const WORD      EPOS4_INDEX_DUAL_LOOP_CONTROL_PARAMETER                             = 0x30AE;
const BYTE      EPOS4_SUBINDEX_DUAL_LOOP_CONTROL_PARAMETER                          = 0x00;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_P_GAIN_LOW_BANDWIDTH           = 0x01;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_P_GAIN_HIGH_BANDWIDTH          = 0x02;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_GAIN_SCHEDULING_WEIGHT         = 0x03;

const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_A         = 0x10;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_B         = 0x11;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_C         = 0x12;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_D         = 0x13;
const BYTE      EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_E         = 0x14;

const BYTE      EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_P_GAIN                         = 0x20;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_I_GAIN                         = 0x21;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_FF_VELOCITY_GAIN               = 0x22;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_FF_ACCELERATION_GAIN           = 0x23;

const BYTE      EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_POSITION_CORRECTION_GAIN         = 0x30;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_VELOCITY_CORRECTION_GAIN         = 0x31;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_LOAD_CORRECTION_GAIN             = 0x32;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_FRICTION                         = 0x33;
const BYTE      EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_INERTIA                          = 0x34;

const BYTE      EPOS4_SUBINDEX_DUAL_LOOP_CONFIGURATION_MISCELLANEOUS                = 0x40;

const WORD      EPOS4_INDEX_HOME_POSITION                           = 0x30B0;
const BYTE      EPOS4_SUBINDEX_HOME_POSITION                        = 0x00;

const WORD      EPOS4_INDEX_HOME_OFFSET                             = 0x30B1;
const BYTE      EPOS4_SUBINDEX_HOME_OFFSET                          = 0x00;

const WORD      EPOS4_INDEX_HOME_CURRENT_THRESHOLD                  = 0x30B2;
const BYTE      EPOS4_SUBINDEX_HOME_CURRENT_THRESHOLD               = 0x00;

const WORD      EPOS4_INDEX_CURRENT_DEMAND_VALUE                    = 0x30D0;
const BYTE      EPOS4_SUBINDEX_CURRENT_DEMAND_VALUE                 = 0x00;

const WORD      EPOS4_INDEX_CURRENT_ACTUAL_VALUE                    = 0x30D1;
const BYTE      EPOS4_SUBINDEX_CURRENT_ACTUAL_AVERAGE_VALUE         = 0x01;
const BYTE      EPOS4_SUBINDEX_CURRENT_ACTUAL_VALUE                 = 0x02;

const WORD      EPOS4_INDEX_VELOCITY_ACTUAL_AVERAGE_VALUE           = 0x30D3;
const BYTE      EPOS4_SUBINDEX_VELOCITY_ACTUAL_AVERAGE_VALUE        = 0x01;

const WORD      EPOS4_INDEX_STANDSTILL_WINDOW_CONFIGURATION         = 0x30E0;
const BYTE      EPOS4_SUBINDEX_STANDSTILL_WINDOW_TIME               = 0x02;
const BYTE      EPOS4_SUBINDEX_STANDSTILL_WINDOW_TIMEOUT            = 0x03;

const WORD      INDEX_DIGITAL_INPUT_PROPERTIES                      = 0x3141;
const BYTE      SUBINDEX_DIGITAL_INPUT_LOGIC_STATE                  = 0x01;
const BYTE      SUBINDEX_DIGITAL_INPUT_POLARITY_CONFIGURATION       = 0x02;

const WORD      INDEX_DIGITAL_INPUT_CONFIGURATION                   = 0x3142;
const BYTE      SUBINDEX_DIGITAL_INPUT_CONFIGURATION                = 0x00;
const BYTE      SUBINDEX_DIGITAL_INPUT_1_CONFIGURATION              = 0x01;
const BYTE      SUBINDEX_DIGITAL_INPUT_2_CONFIGURATION              = 0x02;
const BYTE      SUBINDEX_DIGITAL_INPUT_3_CONFIGURATION              = 0x03;
const BYTE      SUBINDEX_DIGITAL_INPUT_4_CONFIGURATION              = 0x04;
const BYTE      SUBINDEX_DIGITAL_INPUT_5_CONFIGURATION              = 0x05;
const BYTE      SUBINDEX_DIGITAL_INPUT_6_CONFIGURATION              = 0x06;
const BYTE      SUBINDEX_DIGITAL_INPUT_7_CONFIGURATION              = 0x07;
const BYTE      SUBINDEX_DIGITAL_INPUT_8_CONFIGURATION              = 0x08;

const WORD      INDEX_DIGITAL_OUTPUT_POLARITY_CONFIGURATION         = 0x3150;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_LOGIC_STATE                 = 0x01;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_POLARITY_CONFIGURATION      = 0x02;

const WORD      INDEX_DIGITAL_OUTPUT_CONFIGURATION                  = 0x3151;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_CONFIGURATION               = 0x00;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_1_CONFIGURATION             = 0x01;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_2_CONFIGURATION             = 0x02;
const BYTE      SUBINDEX_DIGITAL_OUTPUT_3_CONFIGURATION             = 0x03;

const WORD      EPOS4_INDEX_HOLDING_BRAKE                           = 0x3158;
const BYTE      EPOS4_SUBINDEX_HOLDING_BRAKE_RISE_TIME              = 0x01;
const BYTE      EPOS4_SUBINDEX_HOLDING_BRAKE_FALL_TIME              = 0x02;

const WORD      INDEX_ANALOG_INPUTS                                 = 0x3160;
const BYTE      SUBINDEX_ANALOG_INPUT_1_VOLTAGE                     = 0x01;
const BYTE      SUBINDEX_ANALOG_INPUT_2_VOLTAGE                     = 0x02;

const WORD      INDEX_ANALOG_INPUT_CONFIGURATION                    = 0x3161;
const BYTE      SUBINDEX_ANALOG_INPUT_CONFIGURATION                 = 0x00;
const BYTE      SUBINDEX_ANALOG_INPUT_1_CONFIGURATION               = 0x01;
const BYTE      SUBINDEX_ANALOG_INPUT_2_CONFIGURATION               = 0x02;

const WORD      INDEX_ANALOG_OUTPUTS                                = 0x3180;
const BYTE      SUBINDEX_ANALOG_OUTPUTS                             = 0x00;
const BYTE      SUBINDEX_ANALOG_OUTPUT_1_VOLTAGE                    = 0x01;
const BYTE      SUBINDEX_ANALOG_OUTPUT_2_VOLTAGE                    = 0x02;

const WORD      INDEX_ANALOG_OUTPUT_CONFIGURATION                   = 0x3181;
const BYTE      SUBINDEX_ANALOG_OUTPUT_CONFIGURATION                = 0x00;
const BYTE      SUBINDEX_ANALOG_OUTPUT_1_CONFIGURATION              = 0x01;
const BYTE      SUBINDEX_ANALOG_OUTPUT_2_CONFIGURATION              = 0x02;

const WORD      INDEX_ANALOG_OUTPUT_GENERAL_PURPOSE                 = 0x3182;
const BYTE      SUBINDEX_ANALOG_OUTPUT_GENERAL_PURPOSE              = 0x00;
const BYTE      SUBINDEX_ANALOG_OUTPUT_GENERAL_PURPOSE_A            = 0x01;
const BYTE      SUBINDEX_ANALOG_OUTPUT_GENERAL_PURPOSE_B            = 0x02;

const WORD      INDEX_INTERNAL_RECORDER_VARIABLE_INDEX              = 0x31E0;
const BYTE      SUBINDEX_INTERNAL_RECORDER_VARIABLE_1_INDEX         = 0x01;
const BYTE      SUBINDEX_INTERNAL_RECORDER_VARIABLE_2_INDEX         = 0x02;
const BYTE      SUBINDEX_INTERNAL_RECORDER_VARIABLE_3_INDEX         = 0x03;
const BYTE      SUBINDEX_INTERNAL_RECORDER_VARIABLE_4_INDEX         = 0x04;

const WORD      INDEX_INTERNAL_RECORDER_CONFIGURATION               = 0x31E1;
const BYTE      SUBINDEX_INTERNAL_RECORDER_SAMPLING_PERIOD          = 0x01;
const BYTE      SUBINDEX_INTERNAL_RECORDER_MAX_NB_OF_SAMPLES        = 0x02;

const WORD      INDEX_INTERNAL_RECORDER_TRIGGER                     = 0x31E2;
const BYTE      SUBINDEX_INTERNAL_RECORDER_TRIGGER_VARIABLE_INDEX   = 0x01;
const BYTE      SUBINDEX_INTERNAL_RECORDER_TRIGGER_MODE             = 0x02;
const BYTE      SUBINDEX_INTERNAL_RECORDER_TRIGGER_HIGH_VALUE       = 0x03;
const BYTE      SUBINDEX_INTERNAL_RECORDER_TRIGGER_LOW_VALUE        = 0x04;
const BYTE      SUBINDEX_INTERNAL_RECORDER_TRIGGER_MASK             = 0x05;
const BYTE      SUBINDEX_INTERNAL_RECORDER_PRECEDING_SAMPLES        = 0x06;

const WORD      INDEX_INTERNAL_RECORDER_CONTROL                     = 0x31E4;
const BYTE      SUBINDEX_INTERNAL_RECORDER_CONTROLWORD              = 0x01;
const BYTE      SUBINDEX_INTERNAL_RECORDER_STATUSWORD               = 0x02;

const WORD      INDEX_INTERNAL_RECORDER_DATA                        = 0x31E5;
const BYTE      SUBINDEX_INTERNAL_RECORDER_NB_OF_RECORDED_SAMPLES   = 0x01;
const BYTE      SUBINDEX_INTERNAL_RECORDER_BUFFER_TIMESTAMP         = 0x02;
const BYTE      SUBINDEX_INTERNAL_RECORDER_SNAPSHOT                 = 0x03;

const WORD      INDEX_INTERNAL_RECORDER_BUFFER_VARIABLE             = 0x31E6;
const WORD      SUBINDEX_INTERNAL_RECORDER_BUFFER_VARIABLE_COMPLETE = 0x01;

const WORD      EPOS4_INDEX_ELECTRICAL_ROTOR_ANGLE_VALUE            = 0x3241;
const BYTE      EPOS4_SUBINDEX_ELECTRICAL_ROTOR_ANGLE_VALUE         = 0x06;

const WORD      INDEX_CURRENT_OSCILLATOR_PARAMETER                  = 0x3248;
const BYTE      SUBINDEX_CURRENT_AMPLITUDE_VALUE                    = 0x01;

const WORD      INDEX_CURRENT_OSCILLATOR_VALUES                     = 0x3249;
const BYTE      SUBINDEX_RESISTANCE_VALUE                           = 0x01;
const BYTE      SUBINDEX_INDUCTIVITY_VALUE                          = 0x02;
const BYTE      SUBINDEX_FREQUENCY_VALUE                            = 0x03;

const WORD      INDEX_CONFIGURATION_MECHANICAL_REGULATION_TUNING    = 0x324B;
const BYTE      SUBINDEX_CUT_OFF_FREQUENCY_OF_OSCILLATOR            = 0x01;
const BYTE      SUBINDEX_TARGET_AMPLITUDE_OF_OSCILLATOR             = 0x02;
const BYTE      SUBINDEX_RESISTANCE                                 = 0x04;
const BYTE      SUBINDEX_INDUCTANCE                                 = 0x05;
const BYTE      SUBINDEX_AUX_TARGET_VELOCITY                        = 0x20;
const BYTE      SUBINDEX_AUX_VELOCITY_ACCELERATION                  = 0x21;
const BYTE      SUBINDEX_AUX_VELOCITY_DECELERATION                  = 0x22;

const WORD      INDEX_PARAMETERS_MECHANICAL_REGULATION_TUNING       = 0x324C;
const BYTE      SUBINDEX_CYCLE_DURATION                             = 0x01;
const BYTE      SUBINDEX_SET_VALUE_OF_CURRENT                       = 0x02;
const BYTE      SUBINDEX_ACTUAL_POSITION_PEAK_VALUE                 = 0x03;
const BYTE      SUBINDEX_ACTUAL_VALUE_OF_DYNAMIC_FRICTION           = 0x04;
const BYTE      SUBINDEX_NUMBER_OF_OSCILLATION_CYCLES               = 0x05;
const BYTE      SUBINDEX_AUX_SUBINDEX_VELOCITY_DEMAND_VALUE         = 0x20;
const BYTE      SUBINDEX_AUX_VELOCITY_ACTUAL_AVERAGE_VALUE          = 0x21;

const WORD      INDEX_PRBS_MECHANICAL_REGULATION_TUNING             = 0x324D;
const BYTE      SUBINDEX_PRBS_MECHANICAL_REGULATION_TUNING          = 0x0;
const BYTE      SUBINDEX_PRBS_STEP_AMPLITUDE                        = 0x1;
const BYTE      SUBINDEX_PRBS_CYCLE_TIME                            = 0x2;

const WORD      INDEX_PRBS_VELOCITY_TUNING_PARAMETER                = 0x324E;
const BYTE      SUBINDEX_PRBS_VELOCITY_TUNING_PARAMETER             = 0x00;
const BYTE      SUBINDEX_PRBS_VELOCITY_TUNING_EXCITATION_AMPLITUDE  = 0x01;

const WORD      INDEX_PRBS_TUNING_EXCITATION_OFFSETS                = 0x3250;
const BYTE      SUBINDEX_PRBS_TUNING_EXCITATION_OFFSETS             = 0x0;

const WORD      INDEX_PRBS_TUNING_POSITION_VALUES                   = 0x324F;
const BYTE      SUBINDEX_PRBS_TUNING_POSITION_VALUES                = 0x0;


#endif // !defined(DcsEpos4Def_H__20150528__INCLUDED_)
