#pragma once

//Application layer control/status
enum EApplicationLayerStatus
{
    ALS_NOT_DEFINED         = 0x0000,
    ALS_INIT                = 0x0001,
    ALS_PRE_OPERATIONAL     = 0x0002,
    ALS_BOOTSTRAP           = 0x0003,
    ALS_SAFE_OPERATIONAL    = 0x0004,
    ALS_OPERATIONAL         = 0x0008,
    ALS_MASK                = 0x000F,
    ALS_ERROR               = 0x0010
};

//EtherCAT state machine
const int MAX_LENGTH_DATA   = 100;
const int LENGTH_FMMU       = 16;

enum EInitCommandTransition
{
    ICT_INIT_PREOP          = 0x0001,   //Init -> Pre-Operational

    ICT_PREOP_SAFEOP        = 0x0002,   //Pre-Operational -> Safe-Operational
    ICT_PREOP_INIT          = 0x0004,   //Pre-Operational -> Init

    ICT_SAFEOP_PREOP        = 0x0008,   //Safe-Operational -> Pre-Operational
    ICT_SAFEOP_OP           = 0x0010,   //Safe-Operational -> Operational
    ICT_SAFEOP_INIT         = 0x0020,   //Safe-Operational -> Init

    ICT_OP_SAFEOP           = 0x0040,   //Operational -> Safe-Operational
    ICT_OP_PREOP            = 0x0080,   //Operational -> Pre-Operational
    ICT_OP_INIT             = 0x0100,   //Operational -> Init

    ICT_INIT_BOOTSTRAP      = 0x0200,   //Init -> Bootstrap
    ICT_BOOTSTRAP_INIT      = 0x0400,   //Bootstrap -> Init
};

typedef struct SCmdValidate
{
    char            pbData[MAX_LENGTH_DATA];
    char            pbDataMask[MAX_LENGTH_DATA];
} SCmdValidate, *PSCmdValidate;

typedef struct SInitCmd
{
    WORD            usTransition;
    BYTE            ubCommand;
    WORD            usAddress;
    WORD            usOffset;
    WORD            usTimeout;
    WORD            usRetries;
    WORD            usLength;
    char            pbData[MAX_LENGTH_DATA];
    BOOL            oValidate;
    SCmdValidate    cmdValidate;
} SInitCmd, *PSInitCmd;

//SII coding

enum EEsiEepromControl
{
    EEC_Read                = 0x0100
};

enum ESiiParameterAddress
{
    SII_VENDOR_ID           = 0x0008,
    SII_PRODUCT_CODE        = 0x000A,   //Equivalent maxon application number
    SII_HARDWARE_VERSION    = 0x000B,
    SII_REVISION_NUMBER     = 0x000C,   //Equivalent maxon application version
    SII_SOFTWARE_VERSION    = 0x000D,
    SII_SERIAL_NUMBER       = 0x000E,
    SII_SIZE                = 0x003E,   //Size of EEPROM in KBit-1
    SII_CATEGORIES          = 0x0040    //Start of Category Section
};

//SII categories
enum ESiiCategory
{
    SII_CAT_NOP             = 0,        // No Info
    SII_CAT_STRINGS         = 10,       // String repository for other Categories
    SII_CAT_DATATYPES       = 20,       // Data Types for future use
    SII_CAT_GENERAL         = 30,       // General information structure
    SII_CAT_FMMU            = 40,       // FMMUs to be used
    SII_CAT_SYNC_M          = 41,       // Sync Manager Configuration
    SII_CAT_TXPDO           = 50,       // TxPDO description
    SII_CAT_RXPDO           = 51,       // RxPDO description
    SII_CAT_DC              = 60,       // Distributed Clock for future use
    SII_CAT_END             = 0xFFFF,   // End of Category List
};

//Struct Defintions
#pragma pack(push, 1)
struct SSiiControlStatus
{
    BYTE    WriteAccess             : 1;
    BYTE    Reserved1               : 5;
    BYTE    ReadSize                : 1;
    BYTE    AddressAlgorithm        : 1;
    BYTE    ReadOperation           : 1;
    BYTE    WriteOperation          : 1;
    BYTE    ReloadOperation         : 1;
    BYTE    CheckSErrDLu            : 1;
    BYTE    DeviceInfoError         : 1;
    BYTE    CommandError            : 1;
    BYTE    WriteError              : 1;
    BYTE    Busy                    : 1;
};

struct SSiiControlStatusAddress
{
    SSiiControlStatus    sControlStatus;
    DWORD    ulAddress;
};

struct SSiiCatagoryHeader
{
    WORD    usType              : 15;
    WORD    usVendorSpecific    : 1;
    WORD    usDataSize;
};

struct SSiiCoeDetails
{
    BYTE    bEnableSdo                  : 1;
    BYTE    bEnableSdoInfo              : 1;
    BYTE    bEnablePdoAssign            : 1;
    BYTE    bEnablePdoConfiguration     : 1;
    BYTE    bEnableUploadAtStartup      : 1;
    BYTE    bEnableSdoCompleteAccess    : 1;
};

struct SSiiFoeDetails
{
    BYTE    bEnableFoE                  : 1;
};

struct SSiiEoeDetails
{
    BYTE    bEnableEoE                  : 1;
};

struct SSiiFlags
{
    BYTE    bEnableSafeOp               : 1;
    BYTE    bEnableNotLrw               : 1;
};

struct SSiiCatagoryGeneral
{
    BYTE                uGroupIdx;
    BYTE                uImgIdx;
    BYTE                uOrderIdx;
    BYTE                uNameIdx;
    BYTE                uReserved;
    SSiiCoeDetails      sCoeDetails;
    SSiiFoeDetails      sFoeDtails;
    SSiiEoeDetails      sEoeDtails;
    BYTE                uSoeChannels;
    BYTE                uDS402Channels;
    BYTE                uSysmanClass;
    SSiiFlags           sFlags;
    short               sCurrentOnEBus;
    BYTE                abPAD_Byte1[2];
    WORD                usPhysicalPort;
    BYTE                abPAD_Byte2[14];
};

#pragma pack(pop)

//ESM AL Status Code Definitions
const WORD      AL_STATUS_CODE_NO_ERROR                             = 0x0000;
const WORD      AL_STATUS_CODE_UNSPECIFIED                          = 0x0001;
const WORD      AL_STATUS_CODE_INVALID_REQUESTED_STATE              = 0x0011;
const WORD      AL_STATUS_CODE_UNKNOWN_REQUESTED_STATE              = 0x0012;
const WORD      AL_STATUS_CODE_BOOTSTRAP_NOT_SUPPORTED              = 0x0013;
const WORD      AL_STATUS_CODE_NO_VALID_FIRMWARE                    = 0x0014;
const WORD      AL_STATUS_CODE_INVALID_MAILBOX_CONF                 = 0x0015;
const WORD      AL_STATUS_CODE_INVALID_MAILBOX_CONF2                = 0x0016;
const WORD      AL_STATUS_CODE_INVALID_SYNC_MANAGER_CONF            = 0x0017;
const WORD      AL_STATUS_CODE_NO_VALID_INPUTS_AVAILABLE            = 0x0018;
const WORD      AL_STATUS_CODE_NO_VALID_OUTPUTS_AVAILABLE           = 0x0019;
const WORD      AL_STATUS_CODE_SYNCHRONISATION_ERROR                = 0x001A;
const WORD      AL_STATUS_CODE_SYNC_MANAGER_WATCHDOG                = 0x001B;
const WORD      AL_STATUS_CODE_INVALID_SYNC_MANAGER_TYPE            = 0x001C;
const WORD      AL_STATUS_CODE_INVALID_OUTPUT_CONF                  = 0x001D;
const WORD      AL_STATUS_CODE_INVALID_INPUT_CONF                   = 0x001E;
const WORD      AL_STATUS_CODE_INVALID_WATCHDOG_CONF                = 0x001F;
const WORD      AL_STATUS_CODE_SLAVE_NEEDS_COLDSTART                = 0x0020;
const WORD      AL_STATUS_CODE_SLAVE_NEEDS_INIT                     = 0x0021;
const WORD      AL_STATUS_CODE_SLAVE_NEEDS_PREOP                    = 0x0022;
const WORD      AL_STATUS_CODE_SLAVE_NEEDS_SAFEOP                   = 0x0023;
const WORD      AL_STATUS_CODE_INVALID_OUTPUT_FMMU_CONF             = 0x002D;
const WORD      AL_STATUS_CODE_INVALID_INPUT_FMMU_CONF              = 0x002E;
const WORD      AL_STATUS_CODE_INVALID_DC_SYNC_CONF                 = 0x0030;
const WORD      AL_STATUS_CODE_INVALID_DC_LATCH_CONF                = 0x0031;
const WORD      AL_STATUS_CODE_INVALID_PLL_ERROR                    = 0x0032;
const WORD      AL_STATUS_CODE_INVALID_DC_IO_ERROR                  = 0x0033;
const WORD      AL_STATUS_CODE_INVALID_DC_TIMEOUT_ERROR             = 0x0034;
const WORD      AL_STATUS_CODE_MBX_EOE                              = 0x0042;
const WORD      AL_STATUS_CODE_MBX_COE                              = 0x0043;
const WORD      AL_STATUS_CODE_MBX_FOE                              = 0x0044;
const WORD      AL_STATUS_CODE_MBX_SOE                              = 0x0045;
const WORD      AL_STATUS_CODE_MBX_VOE                              = 0x004F;

//String Table Index
const BYTE      STRING_TABLE_NAME_INDEX                 = 0;

//ESC address space definitions

const WORD      ADDRESS_TYPE                            = 0x0000;
const WORD      ADDRESS_REVISION                        = 0x0001;
const WORD      ADDRESS_BUILD                           = 0x0002;   //2 Bytes
const WORD      ADDRESS_FMMUS_SUPPORTED                 = 0x0004;
const WORD      ADDRESS_SYNC_MANAGER_SUPPORTED          = 0x0005;
const WORD      ADDRESS_RAM_SIZE                        = 0x0006;
const WORD      ADDRESS_PORT_DESCRIPTOR                 = 0x0007;
const WORD      ADDRESS_ESC_FEATURES_SUPPORTED          = 0x0008;

//Station address
const WORD      ADDRESS_CONFIGURED_STATION_ADDRESS      = 0x0010;   //2 Bytes
const WORD      ADDRESS_CONFIGURED_STATION_ALIAS        = 0x0012;   //2 Bytes

//Write protection
const WORD      ADDRESS_WRITE_REGISTER_ENABLE           = 0x0020;
const WORD      ADDRESS_WRITE_REGISTER_PROTECTION       = 0x0021;
const WORD      ADDRESS_ESC_WRITE_ENABLE                = 0x0030;
const WORD      ADDRESS_ESC_WRITE_PROTECTION            = 0x0031;

//Data link layer
const WORD      ADDRESS_ESC_RESET_ECAT                  = 0x0040;
const WORD      ADDRESS_ESC_RESES_PDI                   = 0x0041;
const WORD      ADDRESS_ESC_DL_CONTROL                  = 0x0100;   //4 Bytes
const WORD      ADDRESS_PHYSICAL_READ_WRITE_OFFSET      = 0x0108;   //2 Bytes
const WORD      ADDRESS_ESC_DL_STATUS                   = 0x0110;   //2 Bytes

//Application layer
const WORD      ADDRESS_AL_CONTROL                      = 0x0120;   //2 Bytes
const WORD      ADDRESS_AL_STATUS                       = 0x0130;   //2 Bytes
const WORD      ADDRESS_AL_STATUS_CODE                  = 0x0134;   //2 Bytes

//ESI EEPROM interface
const WORD      ADDRESS_ESI_EEPROM_CONFIGURATION        = 0x0500;
const WORD      ADDRSSS_ESI_EEPROM_CONTROL              = 0x0502;   //2 Bytes
const WORD      ADDRSSS_ESI_EEPROM_ADDRESS              = 0x0504;   //4 Bytes
const WORD      ADDRSSS_ESI_EEPROM_DATA                 = 0x0508;   //4 Bytes

//FMMU
const WORD      ADDRESS_FMMU                            = 0x0600;

//SyncManager
const WORD      ADDRESS_SYNC_MANAGER                    = 0x0800;

//Distributed clock
const WORD      ADDRESS_DISTRIBUTED_CLOCK               = 0x0980;
const WORD      ADDRESS_DC_START_TIME                   = 0x0990;
const WORD      ADDRESS_DC_CYCLE_TIME                   = 0x09a0;
const WORD      ADDRESS_DC_LATCH_CFG                    = 0x09a8;

//Definition of mandatory communication objects (index/subindex)
const WORD      INDEX_SYNC_MANAGER_CHANNEL2                             = 0x1C12;
const WORD      INDEX_SYNC_MANAGER_CHANNEL3                             = 0x1C13;
const BYTE      SUBINDEX_SYNC_MANAGER_CHANNEL_NB_OF_ASSINGED_PDOS       = 0x00;

