#pragma once

#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>
#include "Command_VCS_Configuration_Definitions.h"

class CCommand_VCS_Configuration : public CCommand_VCS
{
public:
    CCommand_VCS_Configuration();
    CCommand_VCS_Configuration(DWORD p_ulCommandId);
    virtual ~CCommand_VCS_Configuration();

    BOOL InitCommand(DWORD p_ulCommandId);

    CCommand_VCS_Configuration& operator=(CCommand_VCS_Configuration& p_rOther);
    CCommandRoot*CloneCommand();

private:
    BOOL InitCommand_ControllerGains(DWORD p_ulCommandId);
    BOOL InitCommand_PositionRegulator(DWORD p_ulCommandId);
    BOOL InitCommand_VelocityRegulator(DWORD p_ulCommandId);
    BOOL InitCommand_CurrentRegulator(DWORD p_ulCommandId);
    BOOL InitCommand_Safety(DWORD p_ulCommandId);
    BOOL InitCommand_Units(DWORD p_ulCommandId);
    BOOL InitCommand_Motor(DWORD p_ulCommandId);
    BOOL InitCommand_Sensors(DWORD p_ulCommandId);

    // Controller gains
    void SetDefaultParameter_SetControllerGain();
    void SetDefaultParameter_GetControllerGain();

    //Position regulator
    void SetDefaultParameter_SetPositionRegulatorGain();
    void SetDefaultParameter_GetPositionRegulatorGain();
    void SetDefaultParameter_SetPositionRegulatorFeedForward();
    void SetDefaultParameter_GetPositionRegulatorFeedForward();
    //Velocity regulator
    void SetDefaultParameter_SetVelocityRegulatorGain();
    void SetDefaultParameter_GetVelocityRegulatorGain();
    void SetDefaultParameter_SetVelocityRegulatorFeedForward();
    void SetDefaultParameter_GetVelocityRegulatorFeedForward();
    //Current regulator
    void SetDefaultParameter_SetCurrentRegulatorGain();
    void SetDefaultParameter_GetCurrentRegulatorGain();
    //Safety
    void SetDefaultParameter_SetMaxFollowingError();
    void SetDefaultParameter_GetMaxFollowingError();
    void SetDefaultParameter_SetMaxProfileVelocity();
    void SetDefaultParameter_GetMaxProfileVelocity();
    void SetDefaultParameter_SetMaxAcceleration();
    void SetDefaultParameter_GetMaxAcceleration();
    //Units
    void SetDefaultParameter_SetVelocityUnits();
    void SetDefaultParameter_GetVelocityUnits();
    //Motor
    void SetDefaultParameter_SetMotorParameter();
    void SetDefaultParameter_GetMotorParameter();
    void SetDefaultParameter_SetMotorType();
    void SetDefaultParameter_GetMotorType();
    void SetDefaultParameter_SetDcMotorParameter();
    void SetDefaultParameter_GetDcMotorParameter();
    void SetDefaultParameter_SetEcMotorParameter();
    void SetDefaultParameter_GetEcMotorParameter();
    //Sensors
    void SetDefaultParameter_SetEncoderParameter();
    void SetDefaultParameter_GetEncoderParameter();
    void SetDefaultParameter_SetSensorType();
    void SetDefaultParameter_GetSensorType();
    void SetDefaultParameter_SetIncEncoderParameter();
    void SetDefaultParameter_GetIncEncoderParameter();
    void SetDefaultParameter_SetHallSensorParameter();
    void SetDefaultParameter_GetHallSensorParameter();
    void SetDefaultParameter_SetSsiAbsEncoderParameter();
    void SetDefaultParameter_SetSsiAbsEncoderParameterEx();
    void SetDefaultParameter_GetSsiAbsEncoderParameter();
    void SetDefaultParameter_GetSsiAbsEncoderParameterEx();
    void SetDefaultParameter_GetMainSensorResolution();
    void SetDefaultParameter_GetAuxiliarySensorResolution();
};

