#pragma once

#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>
#include "Command_VCS_OperationMode_Definitions.h"

class CCommand_VCS_OperationMode : public CCommand_VCS
{
public:
    CCommand_VCS_OperationMode();
    CCommand_VCS_OperationMode(DWORD p_ulCommandId);
    virtual ~CCommand_VCS_OperationMode();

    BOOL InitCommand(DWORD p_ulCommandId);

    CCommand_VCS_OperationMode& operator=(CCommand_VCS_OperationMode& p_rOther);
    CCommandRoot*CloneCommand();

private:
    BOOL InitCommand_Initialisation(DWORD p_ulCommandId);

    BOOL InitCommand_CurrentMode(DWORD p_ulCommandId);
    BOOL InitCommand_CurrentPulseMode(DWORD p_ulCommandId);

    BOOL InitCommand_VelocityMode(DWORD p_ulCommandId);
    BOOL InitCommand_VelocityModeAnalog(DWORD p_ulCommandId);

    BOOL InitCommand_PositionMode(DWORD p_ulCommandId);
    BOOL InitCommand_PositionModeAnalog(DWORD p_ulCommandId);

    BOOL InitCommand_CyclicSyncTorqueMode(DWORD p_ulCommandId);

    BOOL InitCommand_HomingMode(DWORD p_ulCommandId);
    BOOL InitCommand_ProfilePositionMode(DWORD p_ulCommandId);
    BOOL InitCommand_ProfileVelocityMode(DWORD p_ulCommandId);
    BOOL InitCommand_InterpolatedPositionMode(DWORD p_ulCommandId);

    BOOL InitCommand_MasterEncoderMode(DWORD p_ulCommandId);
    BOOL InitCommand_StepDirectionMode(DWORD p_ulCommandId);

    BOOL InitCommand_TuningMode(DWORD p_ulCommandId);

    //Initialisation
    void SetDefaultParameter_GetOperationMode();
    void SetDefaultParameter_SetOperationMode();
    //Current mode
    void SetDefaultParameter_GetCurrentMust();
    void SetDefaultParameter_SetCurrentMust();
    //Current mode analog
    void SetDefaultParameter_ActivateAnalogCurrentSetpoint();
    void SetDefaultParameter_DeactivateAnalogCurrentSetpoint();
    void SetDefaultParameter_EnableAnalogCurrentSetpoint();
    void SetDefaultParameter_DisableAnalogCurrentSetpoint();
    //Velocity mode
    void SetDefaultParameter_GetVelocityMust();
    void SetDefaultParameter_SetVelocityMust();
    //Velocity mode analog
    void SetDefaultParameter_ActivateAnalogVelocitySetpoint();
    void SetDefaultParameter_DeactivateAnalogVelocitySetpoint();
    void SetDefaultParameter_EnableAnalogVelocitySetpoint();
    void SetDefaultParameter_DisableAnalogVelocitySetpoint();
    //Position mode
    void SetDefaultParameter_GetPositionMust();
    void SetDefaultParameter_SetPositionMust();
    //Position mode analog
    void SetDefaultParameter_ActivateAnalogPositionSetpoint();
    void SetDefaultParameter_DeactivateAnalogPositionSetpoint();
    void SetDefaultParameter_EnableAnalogPositionSetpoint();
    void SetDefaultParameter_DisableAnalogPositionSetpoint();
    //Cyclic sync torque mode
    void SetDefaultParameter_InitCyclicSyncTorqueMode();
    void SetDefaultParameter_SetTargetTorque();
    void SetDefaultParameter_GetTargetTorque();
    void SetDefaultParameter_GetTorqueIs();
    void SetDefaultParameter_ProgramHardwareInfos();
    //Homing mode
    void SetDefaultParameter_FindHome();
    void SetDefaultParameter_GetHomingParameter();
    void SetDefaultParameter_SetHomingParameter();
    void SetDefaultParameter_StopHoming();
    void SetDefaultParameter_DefinePosition();
    void SetDefaultParameter_GetHomingState();
    //Profile position mode
    void SetDefaultParameter_GetPositionProfile();
    void SetDefaultParameter_GetTargetPosition();
    void SetDefaultParameter_HaltPositionMovement();
    void SetDefaultParameter_MoveToPosition();
    void SetDefaultParameter_SetPositionProfile();
    void SetDefaultParameter_EnablePositionWindow();
    void SetDefaultParameter_DisablePositionWindow();
    //Profile velocity mode
    void SetDefaultParameter_GetTargetVelocity();
    void SetDefaultParameter_GetVelocityProfile();
    void SetDefaultParameter_HaltVelocityMovement();
    void SetDefaultParameter_MoveWithVelocity();
    void SetDefaultParameter_SetVelocityProfile();
    void SetDefaultParameter_EnableVelocityWindow();
    void SetDefaultParameter_DisableVelocityWindow();
    //Interpolated position mode
    void SetDefaultParameter_SetIpmBufferParameter();
    void SetDefaultParameter_GetIpmBufferParameter();
    void SetDefaultParameter_ClearIpmBuffer();
    void SetDefaultParameter_GetFreeIpmBufferSize();
    void SetDefaultParameter_AddPvtValueToIpmBuffer();
    void SetDefaultParameter_StartIpmTrajectory();
    void SetDefaultParameter_StopIpmTrajectory();
    void SetDefaultParameter_GetIpmStatus();
    //Master encoder mode
    void SetDefaultParameter_SetStepDirectionParameter();
    void SetDefaultParameter_GetStepDirectionParameter();
    //Step direction mode
    void SetDefaultParameter_SetMasterEncoderParameter();
    void SetDefaultParameter_GetMasterEncoderParameter();
};
