#pragma once

#include <MmcTypeDefinition.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorInfo.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/CommandRoot.h>

class CLayerManagerBase;
class CParameterSet;
class CMmcCriticalSectionEx;

class CGateway
{
public:
    BOOL InitGateway();

    CGateway();
    virtual ~CGateway();
    virtual CGateway* Clone();
    CGateway& operator=(CGateway& p_rOther);

    //Funktionalit�t
    virtual BOOL ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle);
    virtual BOOL AbortCommands(BOOL p_oActive);

    //ErrorHandling
    virtual BOOL InitErrorHandling();
    virtual BOOL InitParameterSet();
    virtual BOOL InitCriticalSection();
    virtual BOOL DeleteErrorHandling();
    virtual BOOL DeleteParameterSet();
    virtual BOOL DeleteCriticalSection();

    //ParameterSet
    BOOL ResetParameterSet();
    BOOL AddParameter(CStdString p_Name, BYTE* p_pDefaultValue, DWORD p_ulSize);
    BOOL AddParameter(CStdString p_Name, CStdString p_DefaultValue);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo = NULL);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL AreParameterEqual(CGateway* p_pGateway);
    BOOL SetParameterSet(CParameterSet& p_rParameterSet);
    BOOL GetParameterSet(CParameterSet& p_rParameterSet);

    //Layer Parameter Stack
    virtual BOOL InitLayerParameterStack(CCommandRoot* p_pCommand);

    //Critical Section
    BOOL Lock(CCommandRoot* p_pCommand);
    BOOL IsLocked(CCommandRoot* p_pCommand);
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL IsLocked();
    BOOL Unlock();
    BOOL AbortLock(BOOL p_oActive);

    //Singleton
    static CMmcCriticalSection* GetSyncInstance();

protected:
    BOOL CheckLayers(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager);
    BOOL GetTraceString(CStdString& p_rTraceString);

protected:
    CErrorHandling* m_pErrorHandling;
    CParameterSet* m_pParameterSet;

    ELayer m_eInputLayer;
    ELayer m_eOutputLayer;

    BOOL m_oAbortCommands;

    //CriticalSection
    CMmcCriticalSectionEx* m_pCriticalSection;

    //Singleton
    static CMmcCriticalSection m_CriticalSectionSingleton;
};
