#pragma once

/***********************************************************************************/
//0x1xxxxxxx    Allgemeine Fehler

//0x2xxxxxxx    interface Layer
//0x21xxxxxx    interface RS232
//0x22xxxxxx    interface CAN
//0x23xxxxxx    interface USB
//0x24xxxxxx    interface HID
//0x25xxxxxx    interface Socket
//0x26xxxxxx    interface TwinCAT

//0x3xxxxxxx    ProtocolStack Layer
//0x31xxxxxx    ProtocolStack EposRS232
//0x32xxxxxx    ProtocolStack CANopen
//0x33xxxxxx    ProtocolStack EsamRS232
//0x34xxxxxx    ProtocolStack Epos2USB
//0x35xxxxxx    ProtocolStack EtherCAT

//0x4xxxxxxx    DeviceCommandSet Layer
//0x41xxxxxx    Device Epos (+ Fehler definiert in Epos Firmware 0x0xxxxxxx)

//0x5xxxxxxx    VirtualCommandSet Layer
//0x51xxxxxx    VirtualDevice
/************************************************************************************/

const DWORD NO_ERROR_DETECTED                                           = 0x00000000;

//Allgemeine Fehler (0x1xxxxxxx)
const DWORD ERROR_INTERNAL                                              = 0x10000001;
const DWORD ERROR_NULL_POINTER                                          = 0x10000002;
const DWORD ERROR_HANDLE_NOT_VALID                                      = 0x10000003;
const DWORD ERROR_BAD_VIRTUAL_DEVICE_NAME                               = 0x10000004;
const DWORD ERROR_BAD_DEVICE_NAME                                       = 0x10000005;
const DWORD ERROR_BAD_PROTOCOL_STACK_NAME                               = 0x10000006;
const DWORD ERROR_BAD_INTERFACE_NAME                                    = 0x10000007;
const DWORD ERROR_BAD_PORT_NAME                                         = 0x10000008;
const DWORD ERROR_LIBRARY_NOT_LOADED                                    = 0x10000009;
const DWORD ERROR_EXECUTING_COMMAND                                     = 0x1000000A;
const DWORD ERROR_TIMEOUT_ELAPSED                                       = 0x1000000B;
const DWORD ERROR_BAD_PARAMETER                                         = 0x1000000C;
const DWORD ERROR_COMMAND_ABORTED_BY_USER                               = 0x1000000D;
const DWORD ERROR_BUFFER_TOO_SMALL                                      = 0x1000000E;
const DWORD ERROR_NO_COMMUNICATION_FOUND                                = 0x1000000F;
const DWORD ERROR_FUNCTION_NOT_SUPPORTED                                = 0x10000010;
const DWORD ERROR_PARAMETER_ALREADY_USED                                = 0x10000011;
const DWORD ERROR_BAD_VIRTUAL_DEVICE_HANDLE                             = 0x10000012;
const DWORD ERROR_BAD_DEVICE_HANDLE                                     = 0x10000013;
const DWORD ERROR_BAD_PROTOCOL_STACK_HANDLE                             = 0x10000014;
const DWORD ERROR_BAD_INTERFACE_HANDLE                                  = 0x10000015;
const DWORD ERROR_BAD_PORT_HANDLE                                       = 0x10000016;
const DWORD ERROR_BAD_ADDRESS_PARAMETER                                 = 0x10000017;
const DWORD ERROR_BAD_VARIABLE_INFO_FILE                                = 0x10000018;
const DWORD ERROR_VARIABLE_NAME_NOT_FOUND                               = 0x10000019;
const DWORD ERROR_BAD_DEVICE_STATE                                      = 0x10000020;
const DWORD ERROR_BAD_FILE_CONTENT                                      = 0x10000021;
const DWORD ERROR_PATH_DOES_NOT_EXISTS                                  = 0x10000022;
const DWORD ERROR_EXPEDITED_TRANSFER_EXPECTED                           = 0x10000023;
const DWORD ERROR_CROSS_THREAD_ERROR                                    = 0x10000024;
const DWORD ERROR_NO_LOGIC_SUPPLY                                       = 0x10000025;
const DWORD ERROR_GATEWAY_NOT_SUPPORTED                                 = 0x10000026;
const DWORD ERROR_UPDATING_SERIAL_NUMBER                                = 0x10000027;
const DWORD ERROR_COMMUNICATION_INTERFACE_NOT_SUPPORTED                 = 0x10000028;
const DWORD ERROR_FIRMWARE_VERSION_NOT_SUPPORTED                        = 0x10000029;
const DWORD ERROR_FIRMWARE_FILE_WRONG_FOR_CONNECTED_HARDWARE            = 0x1000002A;
const DWORD ERROR_FIRMWARE_FILE_WRONG_OR_BAD                            = 0x1000002B;
const DWORD ERROR_PARAMETER_ACCESS_DENIED                               = 0x1000002C;
const DWORD ERROR_DATA_RECORDER_NOT_CONFIGURED                          = 0x1000002D;
const DWORD ERROR_FILE_FORMAT_NOT_SUPPORTED                             = 0x1000002E;
const DWORD ERROR_FAILED_SAVING_DATA                                    = 0x1000002F;

//InterfaceLayer (0x2xxxxxxx)
const DWORD ERROR_I_OPENING_INTERFACE                                   = 0x20000001;
const DWORD ERROR_I_CLOSING_INTERFACE                                   = 0x20000002;
const DWORD ERROR_I_INTERFACE_NOT_OPEN                                  = 0x20000003;
const DWORD ERROR_I_OPENING_PORT                                        = 0x20000004;
const DWORD ERROR_I_CLOSING_PORT                                        = 0x20000005;
const DWORD ERROR_I_PORT_NOT_OPEN                                       = 0x20000006;
const DWORD ERROR_I_RESET_PORT                                          = 0x20000007;
const DWORD ERROR_I_SET_PORT_SETTINGS                                   = 0x20000008;
const DWORD ERROR_I_SET_PORT_MODE                                       = 0x20000009;
const DWORD ERROR_I_GET_PORT_SETTINGS                                   = 0x2000000A;
const DWORD ERROR_I_ACCESS_DENIED                                       = 0x2000000B;

//Interface RS232 (0x21xxxxxx)
const DWORD ERROR_RS232_WRITE_DATA                                      = 0x21000001;
const DWORD ERROR_RS232_READ_DATA                                       = 0x21000002;
//Interface CAN (0x22xxxxxx)
const DWORD ERROR_CAN_RECEIVE_CAN_FRAME                                 = 0x22000001;
const DWORD ERROR_CAN_TRANSMIT_CAN_FRAME                                = 0x22000002;
//Interface USB (0x23xxxxxx)
const DWORD ERROR_USB_WRITE_DATA                                        = 0x23000001;
const DWORD ERROR_USB_READ_DATA                                         = 0x23000002;
const DWORD ERROR_USB_RESCAN                                            = 0x23000003;
const DWORD ERROR_USB_RELOAD                                            = 0x23000004;
//Interface HID (0x24xxxxxx)
const DWORD ERROR_HID_WRITE_DATA                                        = 0x24000001;
const DWORD ERROR_HID_READ_DATA                                         = 0x24000002;
//Interface Socket (0x25xxxxxx)
const DWORD ERROR_SOCKET_WRITE_DATA                                     = 0x25000001;
const DWORD ERROR_SOCKET_READ_DATA                                      = 0x25000002;
const DWORD ERROR_SOCKET_DATA_FRAME_EMPTY                               = 0x25000003;
const DWORD ERROR_SOCKET_DATA_FRAME_TOO_LONG                            = 0x25000004;
//Interface Socket (0x26xxxxxx)
const DWORD ERROR_TWINCAT_WRITE_DATA                                    = 0x26000001;
const DWORD ERROR_TWINCAT_READ_DATA                                     = 0x26000002;
//Interface IPC (0x27xxxxxx)
const DWORD ERROR_IPC_WRITE_DATA                                        = 0x27000001;
const DWORD ERROR_IPC_READ_DATA                                         = 0x27000002;

//ProtocolStackLayer (0x3xxxxxxx)
//ProtocolStack MaxonSerialV1 (0x31xxxxxx)
const DWORD ERROR_MAXON_SERIAL_V1_NEG_ACK_RECEIVED                      = 0x31000001;
const DWORD ERROR_MAXON_SERIAL_V1_BAD_CRC_RECEIVED                      = 0x31000002;
const DWORD ERROR_MAXON_SERIAL_V1_BAD_DATA_SIZE_RECEIVED                = 0x31000003;
//ProtocolStack CANopen (0x32xxxxxx)
const DWORD ERROR_CAN_OPEN_SDO_RECEIVE_FRAME_NOT_RECEIVED               = 0x32000001;
const DWORD ERROR_CAN_OPEN_REQUESTED_CAN_FRAME_NOT_RECEIVED             = 0x32000002;
const DWORD ERROR_CAN_OPEN_CAN_FRAME_NOT_RECEIVED                       = 0x32000003;
//ProtocolStack EsamRS232 (0x33xxxxxx)
const DWORD ERROR_INFOTEAM_SERIAL_BAD_ACK_RECEIVED                      = 0x33000001;
const DWORD ERROR_INFOTEAM_SERIAL_REP_ACK_RECEIVED                      = 0x33000002;
const DWORD ERROR_INFOTEAM_SERIAL_BAD_CRC_RECEIVED                      = 0x33000003;
const DWORD ERROR_INFOTEAM_SERIAL_BAD_DATA_SIZE_RECEIVED                = 0x33000004;
const DWORD ERROR_INFOTEAM_SERIAL_CHUNK_SIZE_TOO_HIGH                   = 0x33000005;
//ProtocolStack Epos2USB (0x34xxxxxx)
const DWORD ERROR_MAXON_SERIAL_V2_STUFFING                              = 0x34000001;
const DWORD ERROR_MAXON_SERIAL_V2_DESTUFFING                            = 0x34000002;
const DWORD ERROR_MAXON_SERIAL_V2_BAD_CRC_RECEIVED                      = 0x34000003;
const DWORD ERROR_MAXON_SERIAL_V2_BAD_DATA_SIZE_RECEIVED                = 0x34000004;
const DWORD ERROR_MAXON_SERIAL_V2_BAD_DATA_SIZE_WRITTEN                 = 0x34000005;
const DWORD ERROR_MAXON_SERIAL_V2_SEND_FRAME                            = 0x34000006;
const DWORD ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME                         = 0x34000007;
//ProtocolStack EtherCAT (0x35xxxxxx)
const DWORD ERROR_ETHERCAT_SEND_FRAME                                   = 0x35000001;
const DWORD ERROR_ETHERCAT_RECEIVE_FRAME                                = 0x35000002;
const DWORD ERROR_ETHERCAT_FRAME_NOT_RECEIVED                           = 0x35000003;

//DeviceCommandSet Layer (0x4xxxxxxx)
//DeviceEpos (0x41xxxxxx + Fehler definiert in EPOS Firmware 0x0xxxxxxx)
const DWORD ERROR_DEVICE_CAN_TOGGLE                                     = 0x05030000;
const DWORD ERROR_DEVICE_CAN_SDO_TIME_OUT                               = 0x05040000;
const DWORD ERROR_DEVICE_CAN_CLIENT_SERVER                              = 0x05040001;
const DWORD ERROR_DEVICE_CAN_INVALID_BLOCK_SIZE                         = 0x05040002;
const DWORD ERROR_DEVICE_CAN_INVALID_SEQUENCE                           = 0x05040003;
const DWORD ERROR_DEVICE_CAN_CRC_ERROR                                  = 0x05040004;
const DWORD ERROR_DEVICE_CAN_OUT_OF_MEMORY                              = 0x05040005;
const DWORD ERROR_DEVICE_CAN_ACCESS                                     = 0x06010000;
const DWORD ERROR_DEVICE_CAN_WRITE_ONLY                                 = 0x06010001;
const DWORD ERROR_DEVICE_CAN_READ_ONLY                                  = 0x06010002;
const DWORD ERROR_DEVICE_CAN_OBJECT_NOT_EXIST                           = 0x06020000;
const DWORD ERROR_DEVICE_CAN_PDO_MAPPING                                = 0x06040041;
const DWORD ERROR_DEVICE_CAN_PDO_LENGTH                                 = 0x06040042;
const DWORD ERROR_DEVICE_CAN_GENERAL_PARAMETER                          = 0x06040043;
const DWORD ERROR_DEVICE_CAN_GENERAL_INTERN_INCOMPATIBILITY             = 0x06040047;
const DWORD ERROR_DEVICE_CAN_HARDWARE                                   = 0x06060000;
const DWORD ERROR_DEVICE_CAN_SERVICE_PARAMETER                          = 0x06070010;
const DWORD ERROR_DEVICE_CAN_SERVICE_PARAMETER_TO_HIGH                  = 0x06070012;
const DWORD ERROR_DEVICE_CAN_SERVICE_PARAMETER_TO_LOW                   = 0x06070013;
const DWORD ERROR_DEVICE_CAN_SUB_OBJECT                                 = 0x06090011;
const DWORD ERROR_DEVICE_CAN_VALUE_RANGE                                = 0x06090030;
const DWORD ERROR_DEVICE_CAN_VALUE_TO_HIGH                              = 0x06090031;
const DWORD ERROR_DEVICE_CAN_VALUE_TO_LOW                               = 0x06090032;
const DWORD ERROR_DEVICE_CAN_MAX_LESS_MIN                               = 0x06090036;
const DWORD ERROR_DEVICE_CAN_GENERAL                                    = 0x08000000;
const DWORD ERROR_DEVICE_CAN_TRANSFER_OR_STORE                          = 0x08000020;
const DWORD ERROR_DEVICE_CAN_LOCAL_CONTROL                              = 0x08000021;
const DWORD ERROR_DEVICE_CAN_WRONG_DEVICE                               = 0x08000022;

const DWORD ERROR_DEVICE_EPOS_UNKNOWN_NETWORK_ID                        = 0x0A000001;
const DWORD ERROR_DEVICE_EPOS_UNKNOWN_NODE_ID                           = 0x0A000002;

const DWORD ERROR_DEVICE_EPOS_CAN_ID                                    = 0x0F00FFB9;
const DWORD ERROR_DEVICE_EPOS_SERVICE_MODE                              = 0x0F00FFBC;
const DWORD ERROR_DEVICE_EPOS_PASSWORD                                  = 0x0F00FFBE;
const DWORD ERROR_DEVICE_EPOS_ILLEGAL_COMMAND                           = 0x0F00FFBF;
const DWORD ERROR_DEVICE_EPOS_WRONG_NMT_STATE                           = 0x0F00FFC0;

const DWORD ERROR_DEVICE_ESAM2_SEGMENTED_TRANSFER_REQUIRED_INIT_DONE    = 0x0F00FFC2;
const DWORD ERROR_DEVICE_ESAM2_DATAHISTORY_BUFFER_OVERRUN               = 0x0F010072;

const DWORD ERROR_DEVICE_EPOS_COMMUNICATION_ABORTED                     = 0x0FFFFFF1;
const DWORD ERROR_DEVICE_EPOS_COMMUNICATION_OVERFLOW                    = 0x0FFFFFF2;
const DWORD ERROR_DEVICE_EPOS_SEGMENTED_COMMUNICATION_ERROR             = 0x0FFFFFF9;
const DWORD ERROR_DEVICE_EPOS_WRONG_AXIS_NUMBER                         = 0x0FFFFFFA;
const DWORD ERROR_DEVICE_EPOS_WRONG_CAN_DEVICE                          = 0x0FFFFFFB;
const DWORD ERROR_DEVICE_EPOS_WRONG_CAN_PORT                            = 0x0FFFFFFC;
const DWORD ERROR_DEVICE_EPOS_WRONG_CALLING_PARAMETER                   = 0x0FFFFFFD;
const DWORD ERROR_DEVICE_EPOS_GENERAL_COMMUNICATION_ERROR               = 0x0FFFFFFE;
const DWORD ERROR_DEVICE_EPOS_TIMEOUT                                   = 0x0FFFFFFF;

//OldDeviceEpos (0x41xxxxxx + Fehler definiert in Epos Firmware 0x0xxxxxxx)
const DWORD ERROR_OLD_DEVICE_EPOS_DEVICE_OLD_MASK                       = 0x0F000000;
const DWORD ERROR_OLD_DEVICE_EPOS_TOGGLE_ERR                            = 0x0F00FFFF;
const DWORD ERROR_OLD_DEVICE_EPOS_SDO_TIME_OUT                          = 0x0F00FFFE;
const DWORD ERROR_OLD_DEVICE_EPOS_CCS_ERR                               = 0x0F00FFFD;
const DWORD ERROR_OLD_DEVICE_EPOS_INVALID_BLOCK_SIZE                    = 0x0F00FFFC;
const DWORD ERROR_OLD_DEVICE_EPOS_INVALID_SEQUENCE_NO                   = 0x0F00FFFB;
const DWORD ERROR_OLD_DEVICE_EPOS_CRC_ERR                               = 0x0F00FFFA;
const DWORD ERROR_OLD_DEVICE_EPOS_OUT_OF_MEMORY                         = 0x0F00FFF9;
const DWORD ERROR_OLD_DEVICE_EPOS_ACCESS_ERR                            = 0x0F00FFF8;
const DWORD ERROR_OLD_DEVICE_EPOS_WRITE_ONLY                            = 0x0F00FFF7;
const DWORD ERROR_OLD_DEVICE_EPOS_READ_ONLY                             = 0x0F00FFF6;
const DWORD ERROR_OLD_DEVICE_EPOS_OBJ_NOT_EXIST                         = 0x0F00FFF5;
const DWORD ERROR_OLD_DEVICE_EPOS_NO_PDO_OBJ                            = 0x0F00FFF4;
const DWORD ERROR_OLD_DEVICE_EPOS_PDO_LENGTH_ERR                        = 0x0F00FFF3;
const DWORD ERROR_OLD_DEVICE_EPOS_GENERAL_PARAM_ERR                     = 0x0F00FFF2;
const DWORD ERROR_OLD_DEVICE_EPOS_GENERAL_INTERN_INCOMP                 = 0x0F00FFF1;
const DWORD ERROR_OLD_DEVICE_EPOS_HW_ERR                                = 0x0F00FFF0;
const DWORD ERROR_OLD_DEVICE_EPOS_SERVICE_PARAM_ERR                     = 0x0F00FFEF;
const DWORD ERROR_OLD_DEVICE_EPOS_SERVICE_PARAM_TOO_HIGH                = 0x0F00FFEE;
const DWORD ERROR_OLD_DEVICE_EPOS_SERVICE_PARAM_TOO_LOW                 = 0x0F00FFED;
const DWORD ERROR_OLD_DEVICE_EPOS_SUB_INDX_ERR                          = 0x0F00FFEC;
const DWORD ERROR_OLD_DEVICE_EPOS_VALUE_RANGE                           = 0x0F00FFEB;
const DWORD ERROR_OLD_DEVICE_EPOS_VAL_TOO_HIGH                          = 0x0F00FFEA;
const DWORD ERROR_OLD_DEVICE_EPOS_VAL_TOO_LOW                           = 0x0F00FFE9;
const DWORD ERROR_OLD_DEVICE_EPOS_MAX_LESS_MIN                          = 0x0F00FFE8;
const DWORD ERROR_OLD_DEVICE_EPOS_GENERAL_ERR                           = 0x0F00FFE7;
const DWORD ERROR_OLD_DEVICE_EPOS_TRANSF_OR_STORE_ERR                   = 0x0F00FFE6;
const DWORD ERROR_OLD_DEVICE_EPOS_LOCAL_CONTROL_ERR                     = 0x0F00FFE5;
const DWORD ERROR_OLD_DEVICE_EPOS_WRONG_DEVICE_STATE                    = 0x0F00FFE4;
const DWORD ERROR_OLD_DEVICE_EPOS_WRONG_NMT_STATE                       = 0x0F00FFC0;
const DWORD ERROR_OLD_DEVICE_EPOS_ILLEGAL_COMMAND                       = 0x0F00FFBF;
const DWORD ERROR_OLD_DEVICE_EPOS_ERROR_PASSWORD                        = 0x0F00FFBE;
const DWORD ERROR_OLD_DEVICE_EPOS_ERROR_LENGTH                          = 0x0F00FFBD;
const DWORD ERROR_OLD_DEVICE_EPOS_ERROR_SERVICE                         = 0x0F00FFBC;

const DWORD ERROR_OLD_DEVICE_EPOS_NO_MORE_SEGMENTS                      = 0x0F00FFBB;
const DWORD ERROR_OLD_DEVICE_EPOS_SDO_ABORT                             = 0x0F00FFBA;
const DWORD ERROR_OLD_DEVICE_EPOS_ERROR_CAN_ID                          = 0x0F00FFB9;
const DWORD ERROR_OLD_DEVICE_EPOS_ERROR_ADRESS                          = 0x0F00FFB8;
const DWORD ERROR_OLD_DEVICE_EPOS_SUB_INDEX_DOES_NOT_EXIST              = 0x0F00FFB7;

//VirtualCommandSetLayer (0x5xxxxxxx)
//VirtualDevice (0x51xxxxxx)
const DWORD ERROR_VIRTUAL_DEVICE_BAD_OBJECT_SIZE_EXPECTED               = 0x51000001;
const DWORD ERROR_VIRTUAL_DEVICE_HOMING_ERROR                           = 0x51000002;
const DWORD ERROR_VIRTUAL_DEVICE_BAD_SII_CATEGORY                       = 0x51000003;
const DWORD ERROR_VIRTUAL_DEVICE_BAD_SII_STRING_TABLE                   = 0x51000004;
const DWORD ERROR_VIRTUAL_DEVICE_STORE_FAILED                           = 0x51000005;
const DWORD ERROR_VIRTUAL_DEVICE_RESTORE_FAILED                         = 0x51000006;
const DWORD ERROR_VIRTUAL_DEVICE_SENSOR_CONFIG_NOT_SUPPORTED            = 0x51000007;
const DWORD ERROR_VIRTUAL_DEVICE_SENSOR_CONFIG_UNKNOWN                  = 0x51000008;
const DWORD ERROR_VIRTUAL_DEVICE_CONFIG_NOT_SUPPORTED                   = 0x51000009;
const DWORD ERROR_VIRTUAL_DEVICE_MASKING_DIGITAL_INPUT_NOT_SUPPORTED    = 0x5100000A;
const DWORD ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED          = 0x5100000B;
