#pragma once

#include <MmcTypeDefinition.h>
#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

class CCommandRoot;
class CJournalManagerBase;
class CErrorInfo;

class CLayerManagerBase
{
public:
    ELayer GetLayer();

    CLayerManagerBase();
    CLayerManagerBase(int p_lInstanceValue);
    CLayerManagerBase(const CLayerManagerBase& p_rObject);
    virtual ~CLayerManagerBase();
    bool IsKindOf(CStdString strClassType);

    //JournalManager
    virtual BOOL InitJournalManager(HANDLE p_hHandle, CJournalManagerBase* p_pJournalManager);
    virtual BOOL ResetJournalManager(HANDLE p_hHandle);
    virtual void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    virtual void ResetJournalManager();

    //Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, HANDLE p_hHandle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(HANDLE p_hHandle, ELayer p_eLayer, CStdString* p_pCommandInfo);

    virtual BOOL AbortCommands(HANDLE p_hHandle, BOOL p_oActive);

    //ErrorHandling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

    //Parameter
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);

    //LayerManager
    virtual BOOL GetLayerManager(HANDLE p_hHandle, ELayer p_eLayer, CLayerManagerBase** p_ppLayerManager, HANDLE* p_pLayerHandle);

protected:
    BOOL CheckLayer(CCommandRoot* p_pCommand);
    BOOL CheckLayer(ELayer p_eLayer);

    CStdString GetLogMessage(CStdString p_className, CStdString p_commandName);

protected:
    CErrorHandling* m_pErrorHandling;
    CMmcCriticalSection m_Sync;

    //Instance Value for Singletons
    int m_lInstanceValue;
    ELayer m_eLayer;
    CStdString m_strClassType;
};
