#pragma once

#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>
#include <list>

class CCommandRoot;
class CGatewayDCStoPS;
class CJournalManagerBase;
class CProtocolStackManagerBase;

class CDeviceBase
{
public:
    CDeviceBase();
    CDeviceBase(const CDeviceBase& rObject);
    virtual ~CDeviceBase();
    virtual CDeviceBase* Clone();
    virtual BOOL IsEqual(CDeviceBase* p_pDevice);

//Help functions
    virtual BOOL DCS_GetProtocolStackNameSelection(CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DCS_GetInterfaceNameSelection(CStdString p_strProtocolStackName, CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DCS_GetPortNameSelection(CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DCS_GetBaudrateSelection(CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, CStdDWordArray* p_paulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL DCS_GetDeviceName(CStdString* p_pDeviceName);

//Enumeration
    virtual BOOL DCS_ResetPortEnumeration(CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);

//Interne Funktionnen
    virtual BOOL AddManager(CProtocolStackManagerBase* p_pManager);
    virtual BOOL RemoveManager(CProtocolStackManagerBase* p_pManager);
    virtual BOOL FindCorrectManager(CStdString p_strProtocolStackName, CProtocolStackManagerBase** p_ppManager);
    virtual BOOL FindCorrectManager(HANDLE p_hPS_BaseLayerHandle, CProtocolStackManagerBase** p_ppManager);
    virtual BOOL FindCorrectLibrary(CStdString p_strProtocolStackName, CProtocolStackManagerBase** p_ppManager);
    virtual BOOL FindCorrectLibrary(HANDLE p_hPS_BaseLayerHandle, CProtocolStackManagerBase** p_ppManager);

    virtual BOOL InitDevice(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitDevice(CStdString p_strProtocolStack, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitDevice(CStdString p_strProtocolStack, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitDevice(CStdString p_strProtocolStack, CStdString p_strInterfaceName, CStdString p_strDriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitGatewayDevice(CStdString p_protocolStackName, CStdString p_gatewayDeviceName, CStdString p_gatewayProtocolStackName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitGateway(CStdString strProtocolStackName);
    virtual BOOL InitBaudrateSelection(CStdString p_strProtocolStack, CStdString p_strInterfaceName, CStdDWordArray& p_aulBaudrateSel);
    virtual BOOL InitDefaultProtocolStackSettings(CStdString p_strProtocolStack, CStdString p_strInterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

//JournalManager
    virtual void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    virtual void ResetJournalManager();

//Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, CProtocolStackManagerBase* p_pProtocolStackManager, HANDLE p_hPS_Handle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(CStdString* p_pCommandInfo);
    virtual BOOL AbortCommands(BOOL p_oActive);

//ErrorHandling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

//Instance Value for Singletons
    virtual BOOL InitInstanceValue(int p_lInstanceValue);

//Parameter
    BOOL AreParameterEqual(CDeviceBase* p_pDevice);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue);

//Critical Section
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL Unlock();
    BOOL IsLocked();

protected:
    BOOL GetFirstManager(CProtocolStackManagerBase** p_ppManager);
    void DeletePrototcolStackManagerList();
    void DeleteGateway();
    BOOL InitDefaultProtocolStackSettings(CProtocolStackManagerBase* p_pProtocolStackManager, CStdString p_ProtocolStackName);
    virtual BOOL InitDefaultProtocolStackSettings(CProtocolStackManagerBase* p_pProtocolStackManager, CStdString p_ProtocolStackName, CStdString p_InterfaceName);

private:
    virtual BOOL VerifyGateway(CStdString p_strProtocolStackName);
    BOOL AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray);

protected:
    CStdString m_strDeviceName;
    CJournalManagerBase* m_pJournalManager;

    std::list<CProtocolStackManagerBase*> m_ProtocolStackManagerList;
    CGatewayDCStoPS* m_pGateway;

    int m_lInstanceValue;
    CErrorHandling* m_pErrorHandling;
};
