#pragma once

#include <CommunicationModel/CommonLayer/LayerManagerBase.h>

class CJournalManagerBase;

class CInterfaceManagerBase  : public CLayerManagerBase
{
public:
//Internal Structure Funktionen
    virtual BOOL I_InitAllInterfaces(CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL I_InitInterface(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_InitInterface(CStdString p_InterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_InitBaudrateSelection(CStdString p_InterfaceName, CStdDWordArray& p_rulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_InitDefaultInterfaceSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_UpdateInterface(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_AddInterfacePort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_DeleteInterfacePort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);

//Initialisation
    virtual HANDLE I_OpenInterface(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_CloseInterface(HANDLE p_hI_Handle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_CloseAllInterfaces(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_AreAllInterfacesClosed();

//Parameter
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);

//Hilfsfunktionen
    virtual BOOL I_GetKeyHandle(CStdString p_InterfaceName, CStdString p_PortName, HANDLE* p_pIKeyHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_ResetInterface(HANDLE p_hI_Handle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetDefaultInterfaceSettings(CStdString p_InterfaceName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetDefaultInterfaceSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_IsInterfaceNameSupported(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo = NULL);

//Selection Update
    virtual BOOL I_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_ResetPortEnumeration(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_UpdatePortNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);

//Selection Funktionen
    virtual BOOL I_GetInterfaceNameSelection(CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetPortNameSelection(CStdString p_InterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetBaudrateSelection(CStdString p_InterfaceName, CStdString p_PortName, CStdDWordArray* pdBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetInterfaceModeSelection(CStdString p_InterfaceName, CStdStringArray* p_pInterfaceModeSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetPortModeSelection(CStdString p_InterfaceName, CStdString p_PortName, CStdStringArray* p_pInterfaceModeSel, CErrorInfo* p_pErrorInfo = NULL);

//Setting Funktionen
    virtual BOOL I_GetInterfaceSettings(HANDLE p_hI_Handle, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetInterfaceSettings(HANDLE p_hI_Handle, DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL I_GetInterfaceMode(HANDLE p_hI_Handle, WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetInterfaceMode(HANDLE p_hI_Handle, WORD p_usModeIndex, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL I_GetPortMode(HANDLE p_hI_Handle, WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetPortMode(HANDLE p_hI_Handle, WORD p_usModeIndex, CErrorInfo* p_pErrorInfo = NULL);

//Name Funktionen
    virtual BOOL I_GetInterfaceName(HANDLE p_hI_Handle, CStdString* p_pInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetPortName(HANDLE p_hI_Handle, CStdString* p_pPortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetDriverInfos(HANDLE p_hI_Handle, CStdString& p_rDriverName, CStdString& p_rDriverParameters, CErrorInfo* p_pErrorInfo = NULL);

//Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, HANDLE p_hHandle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(HANDLE p_hHandle, ELayer p_eLayer, CStdString* p_pCommandInfo);

//Tracing
    virtual BOOL EnableTracing(HANDLE p_hI_Handle, CStdString p_TracingFileName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DisableTracing(HANDLE p_hI_Handle, CErrorInfo* p_pErrorInfo = NULL);

//LayerManager
    virtual BOOL GetLayerManager(HANDLE p_hHandle, ELayer p_eLayer, CLayerManagerBase** p_ppLayerManager, HANDLE* p_pLayerHandle);
    virtual BOOL GetLayerManager(CStdString p_strDeviceName, CStdString p_strProtocolStackName, ELayer p_eLayer, CLayerManagerBase** p_ppLayerManager);

    CInterfaceManagerBase();
    CInterfaceManagerBase(int p_lInstanceValue);
    CInterfaceManagerBase(const CInterfaceManagerBase& p_rObject);
    virtual ~CInterfaceManagerBase();
    virtual CInterfaceManagerBase* Clone();
};
