#pragma once

#include <Drv/SocketCAN/MmcSocketCANHndl.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>

class CCommand_I;
class CCANDeviceInfoHandling;

class CGatewayCANToSocketCAN : public CGatewayIToDrv
{
public:
    CGatewayCANToSocketCAN();
    virtual ~CGatewayCANToSocketCAN();

    virtual CGateway* Clone();
    CGatewayCANToSocketCAN& operator=(CGatewayCANToSocketCAN& p_rOther);

    //Hardware infos
    BOOL GetInterfaceHardwareNames(CStdStringArray& p_rNames);

    //Internal structure functions
    virtual BOOL InitPortList();
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);

    //Initialisation
    virtual BOOL OpenInterface(CStdString p_interfaceName, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL CloseInterface(CErrorInfo* p_pErrorInfo = 0);

    virtual BOOL OpenPort(CStdString p_portName, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL ClosePort(CErrorInfo* p_pErrorInfo = 0);

    //Help functions
    virtual BOOL ResetPort(CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL IsPortNameSupported(CStdString p_portName);

    //Setting functions
    virtual BOOL SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = 0);

    virtual BOOL SetPortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo = 0);

    //Functionality
    virtual BOOL ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle);

private:
    BOOL Process_TransmitCanFrame(CCommand_I* p_pCommand);
    BOOL Process_ReceiveCanFrame(CCommand_I* p_pCommand);
    BOOL Process_ReceiveFilteredCanFrame(CCommand_I* p_pCommand);

    BOOL TransmitCanFrame(DWORD p_ulCobId, BOOL p_oRtr, BYTE p_ubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo = 0);
    BOOL ReceiveCanFrame(DWORD p_ulTimeout, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, CErrorInfo* p_pErrorInfo = 0);
    BOOL ReceiveFilteredCanFrame(DWORD p_ulCobIdFilter, BOOL p_oRtrFilter, DWORD p_ulTimeout, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, CErrorInfo* p_pErrorInfo = 0);

    BOOL InitErrorHandling();
    BOOL InitBaudrateSelection();
    BOOL InitDefaultPortSettings();
    BOOL CheckStatusAndReset();
    BOOL ConvertFromDriverToInterfaceErrorCode(EDrvErrorCode p_eDriverErrorCdoe, DWORD& p_rulErrorCode);

    //Device infos
    BOOL DeleteDeviceInfoList(tSocketCANDeviceInfoList& p_rDeviceInfoList);

private:
    CMmcSocketCANHndl m_CANHndl;
    CCANDeviceInfoHandling* m_pInfoHandling;
};
