#pragma once

#include <Drv/Rs232/MmcRs232Hndl.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>

class CCommand_I;
class CRs232DeviceInfoHandling;

class CGatewayRS232ToWin32 : public CGatewayIToDrv
{
public:
    CGatewayRS232ToWin32();
    virtual ~CGatewayRS232ToWin32();

    virtual CGateway* Clone();
    CGatewayRS232ToWin32& operator=(CGatewayRS232ToWin32& p_rOther);

//Interne Struktur Funktionen
    virtual BOOL InitPortList();
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);

//Initialisation
    virtual BOOL OpenPort(CStdString p_portName, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL ClosePort(CErrorInfo* p_pErrorInfo = 0);

//Hilfsfunktionen
    virtual BOOL ResetPort(CErrorInfo* p_pErrorInfo = 0);

//Setting Funktionen
    virtual BOOL SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = 0);

    virtual BOOL SetPortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo = 0);

//Selection
    virtual BOOL GetPortModeSelection(CStdStringArray* p_pPortModeSel, CErrorInfo* p_pErrorInfo = 0);

//Funktionalität
    virtual BOOL ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle);

private:
    BOOL InitBaudrateSelection();
    BOOL InitDefaultPortSettings();
    BOOL InitErrorHandling();
    BOOL Process_ReadData(CCommand_I* p_pCommand);
    BOOL Process_WriteData(CCommand_I* p_pCommand);
    BOOL PurgeBuffer();
    BOOL ReadData(void* p_pDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo = 0);
    BOOL SetTimeout(DWORD p_ulTimeout);
    BOOL WriteData(void* pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo = 0);

    //Device Infos
    BOOL DeleteDeviceInfoList(tRs232DeviceInfoList& p_rDeviceInfoList);
    BOOL UpdatePortList(CRs232DeviceInfoHandling* p_pDeviceInfoHandling);

private:
    CMmcRs232Hndl m_Rs232Hndl;
    CRs232DeviceInfoHandling* m_pInfoHandling;
};
