#pragma once

#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>
#include <Drv/BaseClasses/UsbDeviceInfo.h>
#include <Storage/MmcUserSettings.h>
#include <Thread/MmcCriticalSection.h>

#include "../../BaseClasses/DeviceInfoHandlingBase.h"

class CUsbDeviceInfo;
class CMmcSemaphore;

class CUsbDeviceInfoHandling : public CDeviceInfoHandlingBase
{
public:
    CUsbDeviceInfoHandling(void);
    CUsbDeviceInfoHandling(int p_lInstanceValue);
    ~CUsbDeviceInfoHandling(void);
    CUsbDeviceInfoHandling& operator=(CUsbDeviceInfoHandling& other);
    BOOL CopyDeviceInfos(CUsbDeviceInfoHandling* p_pOther);

    //Flags
    BOOL InitRegistryKeyType(BOOL p_oGlobal);

    //Structs
    struct SVendorProductIDs
    {
        CStdString driverName;
        WORD vendorId;
        WORD productId;
    };

    //DeviceInfos
    BOOL ResetDeviceInfos();
    BOOL InitDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_deviceClassType);
    BOOL UpdateDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_deviceClassType, tPortList& p_rOpenPortList);
    BOOL UpdateDeviceInfo(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_deviceClassType, CStdString p_PortName);
    BOOL UpdatePortOpened(CStdString p_PortName, BOOL p_oOpened);
    BOOL GetPortNames(CStdStringArray& p_rPortNames);
    BOOL GetPortNames(CUsbDeviceInfo& p_rDeviceType, CStdStringArray& p_rPortNames);
    CUsbDeviceInfo* GetDeviceInfo(CStdString p_PortName);

    //Logging
    BOOL WriteToLog(CStdString p_Action, tUsbDeviceInfoList& p_rUsbDeviceInfoList);
    BOOL WriteToLog(CStdString p_Action, CStdString portName);
    BOOL WriteToLog(CStdString p_Action);

    //Persistence
    BOOL DeleteFromRegistry();
    BOOL WriteToRegistry();
    BOOL ReadFromRegistry();

    //Vendor and Product IDs
    BOOL ResetVendorProductIds();
    BOOL AddVendorProductIds(CStdString p_DriverName, WORD p_VendorId, WORD p_ProductId);
    BOOL AddVendorProductIds(CStdString p_DriverName, CStdString p_DriverParameters);
    BOOL GetVendorProductIds(CStdString p_DriverName, std::list<SVendorProductIDs>& p_rIdsList);
    BOOL GetAllVendorProductIds(std::list<SVendorProductIDs>& p_rIdsList);
    BOOL AreVendorProductIdsSupportedByDriver(CStdString p_DriverName, WORD p_VendorId, WORD p_ProductId);

    //Ignore Hardware Serial Number
    BOOL UpdateRegistryIgnoreHwSerialNumber();

private:
    //Initialisation
    BOOL CreateRegistrySemaphore();
    BOOL DeleteRegistrySemaphore();

    //Flags
    BOOL UpdateRegistryKeyType(CStdString p_applicationName);

    //Update Devices
    BOOL UpdateRemovedDevices(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_deviceClassType);
    BOOL UpdateAddedDevices(tUsbDeviceInfoList& p_rDeviceInfoList, tPortList& p_rOpenPortList);
    BOOL UpdateNewAddedDevices(tUsbDeviceInfoList& p_rDeviceInfoList, tPortList& p_rOpenPortList);
    BOOL RemoveOrphanDevices(tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL SearchOrphanDevices(tUsbDeviceInfoList p_deviceInfoList, CUsbDeviceInfo* p_pUsbDeviceInfo, tUsbDeviceInfoList& p_OrphanDevices);
    BOOL IsDeviceInfoInList(tUsbDeviceInfoList& p_deviceInfoList, CUsbDeviceInfo* pUsbDeviceInfo);

    //Port
    BOOL IsPortNameUsed(CStdString p_PortName, tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL SetReopenFlag(CStdString p_PortName, tPortList& p_rOpenPortList);
    BOOL GetNextFreePortName(CStdString& p_rPortName);

    //List Handling
    BOOL IsPortExistingInList(CUsbDeviceInfo& p_rDeviceInfo, tUsbDeviceInfoList& p_rDeviceInfoList, CUsbDeviceInfo*& p_rpExistingDeviceInfo, CUsbDeviceInfo::ECompareMode p_eCompareMode);
    BOOL IsClassTypeEqual(CStdString deviceClassType, CUsbDeviceInfo& p_rDeviceInfo);
    BOOL ReplaceInList(tUsbDeviceInfoList&p_rUsbDrvDeviceInfoList, CUsbDeviceInfo* p_pDeviceInfo, CUsbDeviceInfo* p_pNewDeviceInfo);
    BOOL AddToDeviceInfoList(CUsbDeviceInfo& p_rDeviceInfo);
    BOOL DeleteFromDeviceInfoList(CUsbDeviceInfo* p_pDeviceInfo);
    BOOL DeleteDeviceInfoList(tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL CopyDeviceInfoList(tUsbDeviceInfoList& p_rSourceDeviceInfoList, tUsbDeviceInfoList& p_rTargetDeviceInfoList);
    BOOL SortPortNames(CStdStringArray& p_rPortNames);
    BOOL SortDeviceInfoListByPortName(tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL FindDeviceInfoByPortName(CStdString p_PortName, tUsbDeviceInfoList& p_rDeviceInfoList, CUsbDeviceInfo*& p_rpDeviceInfo);
    BOOL FindDeviceInfoBySerialNumber(CUsbDeviceInfo& p_rDeviceInfo, tUsbDeviceInfoList& p_rDeviceInfoList, CUsbDeviceInfo*& p_rpFoundDeviceInfo);
    BOOL FindNonPluggedInDeviceInfoInList(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_deviceClassType, CUsbDeviceInfo*& p_rpNonPluggedInDeviceInfo);
    BOOL ResetDeviceInfoList();

    //Infos
    BOOL GetModuleVersion(CStdString& p_rVersion);
    BOOL GetApplicationInfo(CStdString& p_rApplicationName, CStdString& p_rVersion);
    CStdString ExtractApplicationName(CStdString p_FileName);

    //Registry
    BOOL GetRegistryKey(CStdString& p_rRegistryKey);
    BOOL ReadDeviceInfoFromRegistry(CMmcUserSettings* p_pUserSettings, CUsbDeviceInfo*& p_rpDeviceInfo);
    BOOL WriteDeviceInfoToRegistry(CMmcUserSettings* p_pUserSettings, int p_lIndex, CUsbDeviceInfo* p_pDeviceInfo);
    BOOL WriteOpenedToRegistry(CMmcUserSettings* p_pUserSettings, BOOL p_oOpened);
    BOOL UpdateRegistry(CStdString p_PortName, BOOL p_oOpened);

    //Helpers
    CStdString RemoveFileExtension(CStdString p_rFileName);
    CStdString ConvertKeyNameToClassType(CStdString p_KeyName);
    BOOL GetClassTypeName(CUsbDeviceInfo* p_pDeviceInfo, int p_lIndex, CStdString& p_rClassType);

    //Vendor and Product IDs
    BOOL IsVendorProductIdExisting(CStdString p_DriverName, WORD p_VendorId, WORD p_ProductId);
    BOOL ExtractVendorProductIDs(CStdString& p_rDriverParameters, WORD& p_rVendorId, WORD& p_rProductId);
    BOOL ExtractVendorIdValue(CStdString p_VendorProductIDs, WORD& p_rVendorId);
    BOOL ExtractProductIdValue(CStdString p_VendorProductIDs, WORD& p_rProductId);
#ifdef WINVER
    //Ignore Hardware Serial Number
    BOOL WriteToRegistryIgnoreHwSerialNumber(HKEY& p_rRootKey, SVendorProductIDs& p_rVendorProductIds);
#endif

private:
    //Singleton
    CMmcSemaphore* m_pRegistrySemaphore;

    //DeviceInfo
    tUsbDeviceInfoList m_DeviceInfoList;

    //Vendor and Product IDs
    static std::list<SVendorProductIDs> m_VendorProductIdList;

    //Flags
    BOOL m_oGlobalRegistryKey;
};
