// GatewayUSBToFtd2xxDrv.h: Schnittstelle für die Klasse CGatewayUSBToFtd2xxDrv.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GatewayUSBToFtd2xxDrv_H__89B9836A_BF97_42DF_8370_3E0AF46F87FE__INCLUDED_)
#define AFX_GatewayUSBToFtd2xxDrv_H__89B9836A_BF97_42DF_8370_3E0AF46F87FE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef _TODO
#include <ftdi.h>
#endif

#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>
#include <CommunicationModel/Interface/Classes/Gateway/USB/Classes/UsbDeviceInfoHandling.h>
#include <Drv/Ftd2xx/MmcFtd2xxHndl.h>

class CCommand_I;
class CRxBufferThread;
class CInterfaceBase;
class CFtd2xxDeviceInfo;
class CUsbDeviceInfoHandling;

class CGatewayUSBToFtd2xxDrv : public CGatewayIToDrv
{
public:
    CGatewayUSBToFtd2xxDrv();
    virtual ~CGatewayUSBToFtd2xxDrv();

    virtual CGateway* Clone();
    //Interne Struktur Funktionen
    virtual BOOL InitDriverInfos(CStdString p_DriverName, CStdString p_DriverParameters);
    virtual BOOL InitPortList();
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);
    virtual BOOL UpdateDeviceInfos(tPortList& p_rOpenPortList);
    virtual BOOL UpdatePortList(tPortList& p_rOpenPortList);

//Initialisation
    virtual BOOL OpenPort(CStdString strPortName, CErrorInfo* pErrorInfo = 0);
    virtual BOOL ReloadDriver();
    virtual BOOL ClosePort(CErrorInfo* pErrorInfo = 0);

//Hilfsfunktionen
    virtual BOOL ResetPort(CErrorInfo* pErrorInfo = 0);

//Setting Funktionen
    virtual BOOL SetPortSettings(DWORD dBaudrate, DWORD dTimeout, BOOL oChangeOnly, CErrorInfo* pErrorInfo = 0);
    virtual BOOL GetPortSettings(DWORD* pdBaudrate, DWORD* pdTimeout, CErrorInfo* pErrorInfo = 0);

    CGatewayUSBToFtd2xxDrv& operator=(CGatewayUSBToFtd2xxDrv& other);

    virtual BOOL SetPortMode(WORD wPortMode, CErrorInfo* pErrorInfo = 0);
    virtual BOOL GetPortMode(WORD* pwPortMode, CErrorInfo* pErrorInfo = 0);

//Funktionalität
    virtual BOOL ProcessCommand(CCommandRoot*pCommand, CLayerManagerBase* pManager, HANDLE h_Handle, HANDLE hTransactionHandle);

    BOOL WaitForRxData(DWORD dTimeout);
    BOOL UpdateRxBuffer();

private:
    //baudrate selection
    static DWORD     BAUDRATE_1000000;

    //Initialisation
    BOOL InitBaudrateSelection();
    BOOL InitDefaultPortSettings();
    BOOL InitErrorHandling();

    //Settings
    BOOL ConfigurePortSettings(DWORD dBaudrate, DWORD dTimeout, BOOL oChangeOnly, CErrorInfo* pErrorInfo, BOOL oDoLock);
    BOOL ConfigurePortMode(WORD wPortMode, CErrorInfo* pErrorInfo, BOOL oDoLock);

    //Commands
    BOOL Process_ReadData(CCommand_I* pCommand);
    BOOL Process_WriteData(CCommand_I* pCommand);
    BOOL Process_EE_Read(CCommand_I* pCommand);
    BOOL Process_EE_Program(CCommand_I* pCommand);
    BOOL Process_Rescan(CCommand_I* pCommand);
    BOOL Process_Reload(CCommand_I* pCommand);

    BOOL PurgeBuffer();
    BOOL SetTimeout(DWORD p_ulTimeout);
    BOOL SetBaudrate(DWORD p_ulBaudrate);
    BOOL ReadData(void* p_pDataBuffer, DWORD p_dNbOfBytesToRead, DWORD* p_pdNbOfBytesRead, CErrorInfo* p_pErrorInfo = 0);
    BOOL WriteData(void* p_pDataBuffer, DWORD p_dNbOfBytesToWrite, DWORD* p_pdNbOfBytesWritten, CErrorInfo* p_pErrorInfo = 0);

    BOOL Rescan(CErrorInfo* pErrorInfo = 0);
    BOOL Reload(WORD p_usVendorId, WORD p_usProductId, CErrorInfo* pErrorInfo = 0);

    //DeviceInfos
    BOOL UpdatePortList(CUsbDeviceInfoHandling* pDeviceInfoHandling);
    BOOL GetLocationId(CStdString p_PortName, DWORD& p_rulLocationId);
    BOOL GetSerialNumber(CStdString p_PortName, CStdString& p_serialNumber);
    BOOL UpdateLocationId(CUsbDeviceInfoHandling* pDeviceInfoHandling, CStdString p_PortName, DWORD& p_rulLocationId);
    BOOL GetDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString& p_rDeviceClassType);
    BOOL DeleteDeviceInfoList(tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL UpdatePortOpened(CStdString p_PortName, BOOL p_oOpened);

    BOOL ResetRxBuffer();
    BOOL GetRxData(void* pBuffer, DWORD dNbOfBytesToRead, DWORD* pdNbOfBytesRead);

    //DriverStatus
    CStdString GetFtStatusDescription(unsigned long status);

    //Vendor and Product IDs
    BOOL InitVendorProductIds(CStdString p_DriverName, CStdString p_DriverParameters);

    //Ftd2xx
    BOOL Ftd2xxWrite(void* p_pDataBuffer, unsigned int p_dNbOfBytesToWrite, unsigned int* p_pdNbOfBytesWritten);
    BOOL Ftd2xxRead(void* p_pDataBuffer, unsigned int p_dNbOfBytesToRead, unsigned int* p_pdNbOfBytesRead);
    BOOL Ftd2xxRescan();
    BOOL Ftd2xxReload(WORD p_usVendorId, WORD p_usProductId);
    BOOL Ftd2xxEeRead(unsigned short& p_rusVendorId, unsigned short& p_rusProductId, TCHAR*& p_rpManufacturer, unsigned int& p_rulManufacturerLength, TCHAR*& p_rpDescription, unsigned int& p_rulDescriptionLength, TCHAR*& p_rpSerialNumber, unsigned int& p_rulSerialNumberLength, bool& p_roUseVcpDriver);
    BOOL Ftd2xxEeProgram(unsigned short p_usVendorId, unsigned short p_usProductId, TCHAR* p_pManufacturer, unsigned int p_ulManufacturerLength, TCHAR* p_pDescription, unsigned int p_ulDescriptionLength, TCHAR* p_pSerialNumber, unsigned int p_ulSerialNumberLength, bool p_oUseVcpDriver);

private:
    BOOL m_oFirstInit;
    BOOL m_oUpdatePortOpened;

    //RxBuffer
    BYTE m_RxBuffer[10000];
    DWORD m_dRxBufferCount;
    int m_iGetIndex;
    int m_iPutIndex;

    CMmcFtd2xxHndl    m_Ftd2xxHndl;
    CMmcCriticalSection m_Sync;

    //UsbDeviceInfoHandling
    CUsbDeviceInfoHandling* m_pInfoHandling;

};

#endif // !defined(AFX_GatewayUSBToFtd2xxDrv_H__89B9836A_BF97_42DF_8370_3E0AF46F87FE__INCLUDED_)
