// GatewayHIDToHidDrv.h: Interface for the CGatewayUSBToHidDrv class.
//
//////////////////////////////////////////////////////////////////////

#pragma once

#include <MmcTypeDefinition.h>
#include <Time/MmcHiResTimer.h>

#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>
#include <CommunicationModel/Interface/Classes/Gateway/USB/Classes/UsbDeviceInfoHandling.h>

class CCommand_I;
class CInterfaceBase;
class CHidDeviceInfo;
class CUsbDeviceInfoHandling;
class CMmcHidHndl;
class CMmcHiResTimer;

class CGatewayUSBToHidDrv : public CGatewayIToDrv
{
public:
    CGatewayUSBToHidDrv();
    virtual ~CGatewayUSBToHidDrv();

    virtual CGateway* Clone();
    CGatewayUSBToHidDrv& operator=(CGatewayUSBToHidDrv& p_rOther);

    //Internal structure functions
    virtual BOOL InitDriverInfos(CStdString p_DriverName, CStdString p_DriverParameters);
    virtual BOOL InitPortList();
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);
    virtual BOOL UpdateDeviceInfos(tPortList& p_rOpenPortList);
    virtual BOOL UpdatePortList(tPortList& p_rOpenPortList);

    //Initialization
    virtual BOOL OpenPort(CStdString p_PortName, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL ReloadDriver();
    virtual BOOL ClosePort(CErrorInfo* p_pErrorInfo = 0);

    //Help functions
    virtual BOOL ResetPort(CErrorInfo* p_pErrorInfo = 0);

    //Setting functions
    virtual BOOL SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = 0);

    //Functionality
    virtual BOOL ProcessCommand(CCommandRoot*p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle);

    //DeviceInfoHandling
    virtual BOOL InitUsbDeviceInfoHandling(CUsbDeviceInfoHandling* p_pHidDeviceInfoHandling);

private:
    //baudrate selection
    static DWORD BAUDRATE_1000000;

    //Initialization
    BOOL InitBaudrateSelection();
    BOOL InitDefaultPortSettings();
    BOOL InitErrorHandling();

    //Settings
    BOOL ConfigurePortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo, BOOL p_oDoLock);

    //Commands
    BOOL Process_ReadData(CCommand_I* p_pCommand);
    BOOL Process_WriteData(CCommand_I* p_pCommand);

    BOOL PurgeBuffer();
    BOOL SetTimeout(DWORD p_ulTimeout);
    BOOL SetBaudrate(DWORD p_ulBaudrate);
    BOOL ReadData(void* p_pDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, int p_lMaxRetry, CErrorInfo* p_pErrorInfo = 0);
    BOOL ReadReport(CErrorInfo* p_pErrorInfo = 0);
    BOOL WriteData(void* p_pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, int p_lMaxRetry, CErrorInfo* p_pErrorInfo = 0);
    BOOL WriteReport(BYTE* p_pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo = 0);
    BOOL WaitForOverlappedResult(DWORD & NumberOfBytesTransferred);

    //DeviceInfos
    BOOL UpdatePortList(CUsbDeviceInfoHandling* p_pDeviceInfoHandling);
    BOOL GetDevicePath(CStdString p_PortName, CStdString& p_rDevicePath);
    BOOL GetDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString& p_rDeviceClassType);
    BOOL DeleteDeviceInfoList(tUsbDeviceInfoList& p_rDeviceInfoList);
    BOOL UpdatePortOpened(CStdString p_PortName, BOOL p_oOpened);

    //InputBuffer
    BOOL InitInputBuffer(DWORD p_ulReadInputBufferSize);
    BOOL UpdateInputBuffer(DWORD p_ulNbOfBytesToRead, int p_lMaxRetry, CErrorInfo* p_pErrorInfo);
    BOOL DeleteInputBuffer();
    BOOL GetDataFromInputBuffer(BYTE* p_pInputBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo);

    //Vendor and Product IDs
    BOOL InitVendorProductIds(CStdString p_DriverName, CStdString p_DriverParameters);

    //Hid
    BOOL HidWriteFile(void* p_pBuffer, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten);
    BOOL HidReadFile(void* p_pBuffer, DWORD p_ulNumberOfBytesToRead, DWORD* p_pulNumberOfBytesRead);
    BOOL HidWaitForOverlapped(unsigned long p_ulTimeout);
    BOOL HidGetOverlappedResult(DWORD* p_pulNumberOfBytesTransferred, BOOL p_oWait);

    //InhibitTime
    BOOL StartWriteInhibitTime();
    BOOL WaitForWriteInhibitTimeElapsed();
    DWORD GetWriteInhibitTimeInMicroSeconds();

private:
    BOOL m_oFirstInit;
    BYTE* m_pInputBuffer;
    DWORD m_ulInputBufferSize;
    DWORD m_ulInputBufferSizeActual;
    DWORD m_ulGetInputBufferIndex;
    BOOL m_oUpdatePortOpened;
    CUsbDeviceInfoHandling* m_pInfoHandling;
    CMmcHidHndl* m_pHidHndl;
    CMmcHiResTimer m_WriteInhibitTimer;
};
