#pragma once

#include <list>
#include <map>

#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>
#include <MmcTypeDefinition.h>

class CErrorHandling;
class CHandleRegistration_I;
class CInterfaceBase;
class CPortBase;

typedef std::list<CHandleRegistration_I*> tHandleRegistrationI_List;

class CHandleRegistrationMap_I
{
private:
    //Embedded Singleton Class
    class CSingleton
    {
    public:
        CSingleton();
        virtual ~CSingleton();

        CHandleRegistrationMap_I* GetInstance(void* p_pParent);
        BOOL ReleaseInstance(void* p_pParent);
        int GetParentCount();

    private:
        CHandleRegistrationMap_I* m_pRegistrationMap;
        std::list<void*> m_ParentRegistrationList;
    };

public:
    CHandleRegistrationMap_I();
    virtual ~CHandleRegistrationMap_I();
    static CHandleRegistrationMap_I* GetInstance(int p_lInstanceValue, void* p_pParent);
    static BOOL ReleaseInstance(int p_lInstanceValue, void* p_pParent);

    //Registration
    HANDLE RegisterHandle(CInterfaceBase* p_pInterface, CPortBase* p_pPort);
    BOOL UpdateMapRegistration(HANDLE p_hInterfaceHandle, CInterfaceBase* p_pInterface, CPortBase* p_pPort);
    BOOL DeleteMapRegistration(HANDLE p_hInterfaceHandle);
    BOOL IsRegistrationMapEmpty();

    BOOL GetRegistrationValues(HANDLE p_hInterfaceHandle, CInterfaceBase** p_ppInterface, CPortBase** p_ppPort);
    BOOL GetRegistration(HANDLE p_hInterfaceHandle, CHandleRegistration_I** p_ppRegistration);
    BOOL GetRegistrationList(tHandleRegistrationI_List** p_ppList);
    BOOL GetInterfaceHandleSelection(BOOL p_oStartOfSelection, HANDLE* p_hInterfaceHandleSel, BOOL* p_poEndOfSelection);
    BOOL GetRegisteredPorts(CInterfaceBase* p_pInterface, tPortList& p_rPortList);
    BOOL GetPendingPortsToReopen(CStdString p_InterfaceName, CStdStringArray& p_rPortsToReopen);
    BOOL ResetReopenFlag(CStdString p_InterfaceName, CStdString p_portName);

    HANDLE IsRegistrationExisting(CInterfaceBase* p_pInterface, CStdString p_PortName);
    HANDLE IsRegistrationExisting(CInterfaceBase* p_pInterface);
    int GetRegistrationCount(CInterfaceBase* p_pInterface);

    //Parameter
    BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo = NULL);
    BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hI_Handle, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);

private:
    HANDLE AddNewRegistration(CInterfaceBase* p_pInterface, CPortBase* p_pPort);
    CHandleRegistration_I* GetNextFreeRegistration();

    BOOL InitErrorHandling();
    BOOL DeleteErrorHandling();
    void DeleteHandleRegistrationList();

private:
    static std::map<void*, void*>* m_pSingletonMap;
    CErrorHandling* m_pErrorHandling;

    std::size_t m_strSelectionListIndex;
    std::list<CHandleRegistration_I*> m_HandleRegistrationList;
};
