
#pragma once

#include <StdString.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>
#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/DeviceInfoHandlingBase.h>

class CJournalManagerBase;
class CCommandRoot;
class CGatewayIToDrv;
class CErrorInfo;
class CPortBase;

class CInterfaceBase
{
public:
    CInterfaceBase();
    CInterfaceBase(const CInterfaceBase& rObject);

    virtual void Init() = 0;
    bool IsKindOf(CStdString p_strClassType);
    CStdString GetClassType();

    virtual ~CInterfaceBase();
    virtual CInterfaceBase* Clone();
    virtual BOOL IsEqual(CInterfaceBase* p_pInterface);

    virtual BOOL InitInterfaceName(CStdString p_Name);
    virtual BOOL InitInterface(CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL InitInterface(CStdString p_DriverName, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL InitBaudrateSelection(CStdDWordArray& p_ulBaudrateSel);
    virtual BOOL InitDefaultInterfaceSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout);

    //Initialisation
    virtual BOOL I_OpenInterface(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_OpenInterfacePort(CPortBase* p_pPort, CStdString p_strPortName, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL I_CloseInterface(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_CloseInterfacePort(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo = NULL);

    //Hilfsfunktionen
    virtual BOOL I_GetDefaultInterfaceSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetDefaultInterfaceSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_ResetInterface(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL I_FindInterfacePortIndex(CStdString p_strPortName, short* p_psPortIndex);
    virtual BOOL I_FindPort(CStdString p_strInterfacePortName, CStdString p_strPortName, CPortBase** ppPort);
    virtual BOOL I_AddPort(CStdString p_PortName);
    virtual BOOL I_DeletePort(CStdString p_PortName);

    //Selection Update
    virtual BOOL I_ResetPortEnumeration(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_UpdatePortNameSelection(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo = NULL);

    //Selection Funktionen
    virtual BOOL I_GetPortNameSelection(CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetInterfaceModeSelection(CStdStringArray* p_pInterfaceModeSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetBaudrateSelection(CStdString p_strPortName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);

    //Setting Funktionen
    virtual BOOL I_GetInterfaceSettings(CPortBase* p_pPort, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetInterfaceSettings(CPortBase* p_pPort, DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL I_GetInterfaceMode(WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_SetInterfaceMode(WORD p_usModeIndex, CErrorInfo* p_pErrorInfo = NULL);

    //Name Funktionen
    virtual BOOL I_GetInterfaceName(CStdString* p_pInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL I_GetPortName(CPortBase* p_pPort, CStdString* p_pPortName, CErrorInfo* p_pErrorInfo = NULL);

    //Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, CPortBase* p_pPort, HANDLE p_hTransactionHandle);
    virtual BOOL AbortCommands(BOOL p_oActive);

    //Tracing
    virtual BOOL EnableTracing(CPortBase* p_pPort, CStdString p_FileName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DisableTracing(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo = NULL);

    //Error handling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

    //Instance Value for Singletons
    virtual BOOL InitInstanceValue(int p_lInstanceValue);
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);

    //Hardware Infos
    virtual BOOL GetInterfaceHardwareNames(CStdStringArray& p_rNames);
    virtual BOOL GetInterfaceHardwareNames(CStdString p_DriverName, CStdStringArray& p_rNames);

    //Parameter
    BOOL AreParameterEqual(CInterfaceBase* p_pDevice);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue);

    //Critical Section
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL Unlock();
    BOOL IsLocked();

    //Journal manager
    virtual void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    virtual void ResetJournalManager();

protected:
    BOOL AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray);
    void DeleteGateway();

    virtual void DeletePortList();

private:
    BOOL SortPortNames(CStdStringArray& p_rPortNames);

protected:
    int m_lInstanceValue;
    CErrorHandling* m_pErrorHandling;
    CGatewayIToDrv* m_pGateway;
    CJournalManagerBase* m_pJournalManager;
    CStdString m_strInterfaceName;
    std::list<CPortBase*> m_PortList;
    WORD m_usModeIndex;
    CStdString m_strClassType;
};
