// PortBase.h: Schnittstelle für die Klasse CPortBase.
//
//////////////////////////////////////////////////////////////////////

#pragma once

#include <list>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorInfo.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

class CJournalManagerBase;
class CGatewayIToDrv;
class CCommandRoot;
class CPortBase;
class CDeviceInfoHandlingBase;
class CParameterSet;

typedef std::list<CPortBase*> tPortList;

class CPortBase
{
public:
    CPortBase();
    CPortBase(const CPortBase& rObject);
    virtual ~CPortBase();
    virtual CPortBase* Clone();
    virtual BOOL IsEqual(CStdString p_PortName);

    //Internal structure functionality
    virtual BOOL InitPort(CStdString p_DriverName, CGatewayIToDrv* p_pGateway, CErrorInfo* p_pErrorInfo = 0);
    virtual BOOL InitGateway(CGatewayIToDrv* p_pGateway);
    virtual BOOL InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling);
    virtual BOOL InitInterfacePortName(CStdString p_Name);
    virtual BOOL InitBaudrateSelection(CStdDWordArray& p_rulBaudrateSel);
    virtual BOOL InitDefaultPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout);
    virtual BOOL InitDefaultPortMode(WORD p_usPortMode);

    //Initialisation
    virtual BOOL OpenPort(CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL ClosePort(CErrorInfo* p_pErrorInfo = NULL);

    //Reopen
    virtual BOOL SetReopenFlag();
    virtual BOOL ResetReopenFlag();
    virtual BOOL IsReopenPending();
    virtual BOOL ReloadDriver();

    //Hilfsfunktionen
    virtual BOOL GetDefaultPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL ResetPort(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL IsPortNameSupported(CStdString p_PortName);
    virtual BOOL SetDefaultPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);

    //Selection Update
    virtual BOOL ResetPortEnumeration(CErrorInfo* pErrorInfo = NULL);
    virtual BOOL UpdateDeviceInfos(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL UpdatePortNameSelection(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo = NULL);

    //Selection Funktionen
    virtual BOOL GetPortNameSelection(CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetBaudrateSelection(CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetPortModeSelection(CStdStringArray* p_pPortModeSel, CErrorInfo* p_pErrorInfo = NULL);

    //Setting Funktionen
    virtual BOOL GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL SetPortMode(WORD wPortMode, CErrorInfo* p_pErrorInfo = NULL);

    //Name Funktionen
    virtual BOOL SetPortName(CStdString p_PortName);
    virtual BOOL GetPortName(CStdString* p_pPortName);
    virtual BOOL GetInterfacePortName(CStdString* p_pInterfacePortName);
    virtual BOOL GetDriverInfos(CStdString& p_rDriverName, CStdString& p_rDriverParameters);

    //Funktionalität
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(CStdString* p_pCommandInfo);

    //JournalManager
    virtual BOOL InitJournalManager(HANDLE p_hHandle, CJournalManagerBase* p_pJournalManager);
    virtual BOOL ResetJournalManager(HANDLE p_hHandle);
    virtual void InitJournalManager(CJournalManagerBase *p_pJournalManager);
    virtual void ResetJournalManager();
    CJournalManagerBase* GetJournalManager();

    //Tracing
    virtual BOOL EnableTracing(CStdString p_FileName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DisableTracing(CErrorInfo* p_pErrorInfo = NULL);

    //Error handling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

    //Parameter
    BOOL AreParameterEqual(CPortBase* p_pDevice);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue);
    BOOL SetParameterSet(CParameterSet& p_rParameterSet);
    BOOL GetParameterSet(CParameterSet& p_rParameterSet);

    //Critical Section
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL Unlock();
    BOOL IsLocked();

protected:
    void DeleteGateway();

private:
    void LogHotPlugInReopenMessage(BOOL p_oSetFlag);

protected:
    CErrorHandling* m_pErrorHandling;
    CGatewayIToDrv* m_pGateway;
    CJournalManagerBase* m_pJournalManager;
    CStdString m_strInterfacePortName;
    CStdString m_strPortName;
    BOOL m_oReopenPending;
};
