#pragma once

#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include "../BaseClasses/GatewayPStoDCS.h"

class CCommand_DCS_Epos4;

class CGatewayCANopenToEpos4 : public CGatewayPStoDCS
{
public:
    CGatewayCANopenToEpos4();
    virtual ~CGatewayCANopenToEpos4();

    virtual CGateway* Clone();
    CGatewayCANopenToEpos4& operator=(CGatewayCANopenToEpos4& p_rOther);

    BOOL ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle);

    // Device Timeout
    BOOL GetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD* p_pulTimeout);
    BOOL SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD p_ulTimeout);

private:
    void InitCommands();
    void DeleteCommands();

    // PS layer
    BOOL Process_InitiateSDODownload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_DownloadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_InitiateSDOUpload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_UploadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_NetworkIndication(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_AbortSDOTransfer(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_SendNMTService(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    BOOL Process_CleanupCommunication(CCommand_PS* p_pCommand, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);

    // DCS layer
    BOOL DCS_ReadObject(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usIndex, BYTE p_ubSubIndex, BYTE* p_pObjectBuffer, DWORD p_ulObjectBufferLength, DWORD* p_pulObjectLengthRead, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_InitiateSegmentedRead(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usIndex, BYTE p_ubSubIndex, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_SegmentedRead(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, BOOL p_oToggle, BOOL* p_poLastDataSegment, void** p_ppSegmentBuffer, DWORD* p_pulSegmentBufferLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_WriteObject(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usIndex, BYTE p_ubSubIndex, BYTE* p_pObjectBuffer, DWORD p_ulObjectLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_InitiateSegmentedWrite(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usIndex, BYTE p_ubSubIndex, DWORD p_ulObjectLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_SegmentedWrite(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, BOOL p_oLastSegment, BOOL p_oToggle, void* p_pSegmentBuffer, DWORD p_ulSegmentLength, BOOL* p_poRetToggle, DWORD* pulSegmentLengthWritten, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_AbortTransfer(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usIndex, WORD p_usSubIndex, DWORD p_ulAbortCode, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_SendNMTService(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usCommandSpecifier, CErrorInfo* p_pErrorInfo = NULL);

    // layer Parameter Stack
    BOOL InitLayerParameterStack(CCommandRoot* p_pCommand);

    // Helper functions
    WORD GetNodeId(DWORD p_ulSdoCobIdClientServer);
    DWORD CheckDataBufferLength(BOOL p_oSizeIndicated, BYTE p_ubNonValidNbOfBytes, DWORD p_ulDataBufferLength, DWORD p_ulMaxDataBufferLength);

    // Buffer functions
    BOOL ResetBufferedData();
    BOOL IsBufferedDataAvailable(DWORD p_ulSize, DWORD* p_pulSizeAvailable, BOOL* p_poNoMoreSegments);
    BOOL AddBufferedData(void* p_pDataBuffer, DWORD p_ulDataBufferSize, BOOL p_oNoMoreSegments);
    BOOL GetBufferedData(void* p_pDataBuffer, DWORD p_ulDataBufferSize, DWORD* p_pulDataRead, BOOL* p_poNoMoreSegments);
    BOOL ReverseBufferedData(DWORD p_ulReverseSize);

    // Addressing
    BOOL GetAddressParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo);
    BOOL UpdateNodeId(CCommand_DCS_Epos4* p_pCommand, BYTE p_ubNodeId);
    BOOL IsDeviceResponding(BOOL p_oResult, CErrorInfo* p_pErrorInfo);

    // CANopen Status
    BOOL IsGatewayInCanPassiveState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);
    BOOL ResetGatewayCommunication(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);
    BOOL ClearGatewayErrors(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);

    // Gateway Timeout
    BOOL SetGatewayTimeout(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, DWORD p_ulTimeout);
    BOOL RestoreGatewayTimeout(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle);

private:
    CCommand_DCS_Epos4* m_pCommand_WriteObject;
    CCommand_DCS_Epos4* m_pCommand_InitiateSegmentedWrite;
    CCommand_DCS_Epos4* m_pCommand_SegmentedWrite;
    CCommand_DCS_Epos4* m_pCommand_ReadObject;
    CCommand_DCS_Epos4* m_pCommand_InitiateSegmentedRead;
    CCommand_DCS_Epos4* m_pCommand_SegmentedRead;
    CCommand_DCS_Epos4* m_pCommand_AbortTransfer;
    CCommand_DCS_Epos4* m_pCommand_SendNMTService;

    BOOL m_oNoMoreSegments;
    BOOL m_oToggle;
    DWORD m_ulBufferedDataRead;
    DWORD m_ulBufferedDataSize;
    DWORD m_ulWriteSegmentSize;
    void* m_pBufferedData;

    DWORD m_ulDeviceTimeoutBackup;
};
