#pragma once

#include <list>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorInfo.h>
#include <MmcTypeDefinition.h>

#include "../InterfacePlugInManager/InterfacePlugInManager.h"

class CCommandRoot;
class CDeviceCommandSetManagerBase;
class CGatewayPStoDCS;
class CGatewayPStoI;
class CInterfaceManagerBase;
class CJournalManagerBase;
class CLayerManagerBase;
class CMmcSemaphore;
class CMmcSingleLock;

class CProtocolStackBase
{
public:
    CProtocolStackBase();
    CProtocolStackBase(const CProtocolStackBase& p_rObject);
    virtual ~CProtocolStackBase();
    virtual CProtocolStackBase* Clone();
    virtual BOOL IsEqual(CProtocolStackBase* p_pProtocolStack);

//Initialisation
    virtual BOOL InitProtocolStack(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitProtocolStack(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitProtocolStackEx(CStdString p_InterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitProtocolStack(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CErrorInfo* p_pErrorInfo = NULL);

//Selection Update
    virtual BOOL PS_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_ResetPortEnumeration(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_UpdatePortNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);

//Hilfsfunktionen
    virtual BOOL PS_GetInterfaceNameSelection(CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetPortNameSelection(CStdString p_InterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetBaudrateSelection(CStdString p_InterfaceName, CStdString p_PortName, CStdDWordArray* pdBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetProtocolStackModeSelection(CStdStringArray* p_pProtocolStackModeSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_SetDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetDeviceNameSelection(CStdStringArray* p_pDeviceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetProtocolStackNameSelection(CStdString p_strDeviceName, CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_GetProtocolStackName(CStdString* p_pProtocolStackName);
    virtual BOOL PS_GetProtocolStackMode(WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL PS_SetProtocolStackMode(WORD p_usModeIndex, CErrorInfo* p_pErrorInfo = NULL);

//Interne Funktionen
    virtual BOOL AddManager(CInterfaceManagerBase* p_pManager);
    virtual BOOL RemoveManager(CInterfaceManagerBase* p_pManager);
    virtual BOOL FindCorrectManager(CStdString p_InterfaceName, CInterfaceManagerBase** p_ppManager);
    virtual BOOL FindCorrectLibrary(CStdString p_InterfaceName, CInterfaceManagerBase** p_ppManager);
    virtual BOOL AddPort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DeletePort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitGatewayToInterface(CStdString p_InterfaceName);

    virtual BOOL AddManager(CDeviceCommandSetManagerBase* p_pManager);
    virtual BOOL FindCorrectManager(CStdString p_strDeviceName, CDeviceCommandSetManagerBase** p_ppManager);
    virtual BOOL FindCorrectLibrary(CStdString p_strDeviceName, CDeviceCommandSetManagerBase** p_ppManager);
    virtual BOOL InitGatewayToDevice(CStdString p_strDeviceName);

    virtual BOOL InitBaudrateSelection(CStdString p_InterfaceName, CStdDWordArray& p_rulBaudrateSel);
    virtual BOOL InitDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

    BOOL UpdateProtocolStackSettings(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle);
    BOOL UpdateProtocolStackSettings(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManagerManager, HANDLE p_hDCS_Handle);
    BOOL SetTimeout(DWORD p_ulTimeout);
    DWORD GetTimeout();
    DWORD GetBaudrate();
    DWORD GetEvaluatedTimeout(DWORD p_ulCommandTimeout);
    BOOL SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD p_ulTimeout);

//Funktionalit�t
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pLayerManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(CStdString* p_pCommandInfo);
    virtual BOOL AbortCommands(BOOL p_oActive);

//JournalManager
    virtual void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    virtual void ResetJournalManager();

//ErrorHandling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

//Instance Value for Singletons
    virtual BOOL InitInstanceValue(int p_lInstanceValue);

//Parameter
    BOOL AreParameterEqual(CProtocolStackBase* p_pDevice);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue);

//Critical Section
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL Unlock();
    BOOL IsLocked();

    BOOL CreateReopenSemaphore(CStdString p_interfaceName, CStdString p_portName);
    BOOL DeleteReopenSemaphore();

//Logging
    BOOL WriteToLogFile(CStdString p_Action);

private:
    BOOL AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* pSourceArray);
    virtual BOOL VerifyGatewayToInterface(CStdString p_InterfaceName);
    virtual BOOL VerifyGatewayToDevice(CStdString strDeviceName);

    void DoLockReopen(CMmcSingleLock*& p_rLock);
    void DoUnlockReopen(CMmcSingleLock*& p_rLock);

protected:
    BOOL CreatePlugInManager();
    BOOL DeletePlugInManager();

    BOOL GetFirstManager(CInterfaceManagerBase** p_ppManager);
    BOOL GetFirstManager(CDeviceCommandSetManagerBase** p_ppManager);

    void DeleteInterfaceManagerList();
    void DeleteDeviceCommandSetManagerList();
    void DeleteGatewayToInterface();
    void DeleteGatewayToDevice();

protected:
    CStdString m_strProtocolStackName;
    DWORD m_dTimeout;
    DWORD m_ulBaudrate;
    int m_lInstanceValue;

    CErrorHandling* m_pErrorHandling;
    CInterfacePlugInManager* m_pPlugInManager;
    CJournalManagerBase* m_pJournalManager;
    CMmcSemaphore* m_pReopenSemaphore;

    //ProtocolStack -> Interface
    CGatewayPStoI* m_pGatewayToInterface;
    std::list<CInterfaceManagerBase*> m_InterfaceManagerList;

    //ProtocolStack -> Device -> ProtocolStack (Router)
    CGatewayPStoDCS* m_pGatewayToDevice;
    std::list<CDeviceCommandSetManagerBase*> m_DeviceCommandSetManagerList;
};
