#pragma once

#include <MmcTypeDefinition.h>

class CProtocolStackBase;
class CInterfacePlugInThread;
class CMmcSemaphore;

class CInterfacePlugInManager
{
public:
    CInterfacePlugInManager(void);
    ~CInterfacePlugInManager(void);

    struct SPluginManagerInterface
    {
        CStdString m_InterfaceName;
        CStdString m_DriverName;
        CStdString m_DriverParameters;
    };

    BOOL Init(CProtocolStackBase* p_pProtocolStack);
    BOOL InitInstanceValue(int p_lInstanceValue);

    //NotificationDelay
    BOOL SetNotificationDelay(DWORD p_ulNotificationDelay);
    BOOL GetNotificationDelay(DWORD& p_rulNotificationDelay);

    //Register PlugIn Interfaces
    BOOL RegisterInterface(CStdString p_InterfaceName);
    BOOL RegisterInterface(CStdString p_InterfaceName, CStdString p_DriverName);
    BOOL UnregisterInterface(CStdString p_InterfaceName);

    //Update
    BOOL UpdateDevice();
    CMmcSemaphore* GetUpdateSemaphore();

    //Logging
    BOOL WriteToLogFile(CStdString p_Action);

    //Usb support
    BOOL IsUsbInterfaceRegistered();
    BOOL IsDriverSupported(WORD p_usVendorId, WORD p_usProductId);
    BOOL IsDriverSupported(CStdString p_DriverName);
    BOOL IsDriverSupported(CStdString p_DriverName, CStdString p_DriverParameters);

private:
    BOOL CheckPlugInThread();
    BOOL FindRegisteredInterface(CStdString p_InterfaceName, int& p_riIndex);
    BOOL FindRegisteredDriver(CStdString p_DriverName, int& p_riIndex);
    BOOL ExtractDriverInfos(CStdString p_DriverName, CStdString& p_rDriverName, CStdString& p_rDriverParameters);
    BOOL ExtractVendorProductIDs(CStdString& p_rDriverParameters, WORD& p_rVendorId, WORD& p_rProductId);
    BOOL ExtractVendorIdValue(CStdString p_VendorProductIDs, WORD& p_rVendorId);
    BOOL ExtractProductIdValue(CStdString p_VendorProductIDs, WORD& p_rProductId);

    //Create
    BOOL CreatePlugInThread();
    BOOL DeletePlugInThread();

    //Start and Stop
    BOOL StartPlugInMonitoring();
    BOOL StopPlugInMonitoring();

    //Update
    BOOL UpdateDevice(CStdString p_InterfaceName);

    //Multithreading
    BOOL CreateUpdateSemaphore();
    BOOL DeleteUpdateSemaphore();

    //Usb Enumeration
    void ResetUsbEnumerationTree();

private:
    CProtocolStackBase* m_pProtocolStack;
    std::list<SPluginManagerInterface> m_RegisteredInterfaces;

    CInterfacePlugInThread* m_pPlugInThread;
    DWORD m_ulNotificationDelay;
    int m_lInstanceValue;

    CMmcSemaphore* m_pUpdateSemaphore;
};
