#pragma once

#include <list>
#include "InterfacePlugInMonitorDef.h"
#include <Drv/Ftd2xx/ftd2xx.h>

class CInterfacePlugInManager;

class CInterfacePlugInMonitorBase : public CInterfacePlugInMonitorDef
{
public:
    CInterfacePlugInMonitorBase();
    virtual ~CInterfacePlugInMonitorBase();
    BOOL Init(CInterfacePlugInManager* p_pPlugInManager);

    BOOL IsReady();
    BOOL UpdateHotPlugIn();
    BOOL CheckNotification();

    BOOL DeviceAdded(libusb_device *p_pDevice);
    BOOL DeviceRemoved(libusb_device *p_pDevice);

    BOOL IsDriverSupported(WORD p_usVendorId, WORD p_usProductId);

private:
    BOOL RegisterForNotifications();
    BOOL UnregisterNotifications();

    BOOL RegisterForHidNotifications();
    BOOL UnregisterHidNotifications();

    BOOL RegisterForFtdiNotifications();
    BOOL UnregisterFtdiNotifications();

    BOOL DevicePlugInStateChanged(libusb_device *p_pDevice, bool p_oPluggedIn);

    void ConvertMilisecondsToTimeval(unsigned int milliseconds, timeval& p_rTimeval);

    static int UsbHotplugNotification(libusb_context *p_pContext, libusb_device *p_pDevice, libusb_hotplug_event p_Event, void *p_pUserData);

    class CPlugInDevice
    {
    public:
        uint8_t m_ucBusNumber;
        uint8_t m_ucPortNumber;
        bool m_oPluggedIn;
        DWORD m_ulTimestamp;
        bool m_oUpdated;
    };

    BOOL SearchPlugInDevice(uint8_t p_BusNumber, uint8_t p_PortNumber, CPlugInDevice*& p_pPlugInDevice);
    void DeletePlugInDevices();
    BOOL GetLastNotificationTimestamp(std::list<CPlugInDevice*> p_plugInDeviceList, DWORD& p_rTimestamp);
    BOOL MarkAllDevicesAsUpdated(std::list<CPlugInDevice*> p_plugInDeviceList);

private:
    CInterfacePlugInManager* m_pPlugInManager;
    libusb_hotplug_callback_handle m_HidHotplugHandle;
    libusb_hotplug_callback_handle m_FdtiHotplugHandle;
    BOOL m_oNotificationsRegistered;
    std::list<CPlugInDevice*> m_PlugInDeviceList;
};
