#pragma once

#include <CommunicationModel/CommonLayer/LayerManagerBase.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <MmcTypeDefinition.h>

class CDeviceCommandSetManagerBase;
class CErrorInfo;
class CInterfaceManagerBase;
class CJournalManagerBase;
class CProtocolStackManagerBase;

class CVirtualCommandSet_ManagerBase : public CLayerManagerBase
{
public:
//Internal structure functions
    virtual BOOL VCS_InitVirtualDevice(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_InitVirtualDevice(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString strDriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_InitVirtualGatewayDevice(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_gatewayDeviceName, CStdString p_gatewayProtocolStackName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_InitBaudrateSelection(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdDWordArray& p_rulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_InitDefaultProtocolStackSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);

//Initialization
    virtual HANDLE VCS_OpenVirtualDevice(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_CloseVirtualDevice(HANDLE p_hVCS_Handle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_CloseAllVirtualDevices(CErrorInfo* p_pErrorInfo = NULL);

    virtual HANDLE VCS_OpenVirtualDevice(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, HANDLE p_hPS_BaseLayerHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_CloseVirtualDevice(HANDLE p_hVCS_Handle, HANDLE p_hPS_BaseLayerHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_CloseAllVirtualDevices(HANDLE p_hPS_BaseLayerHandle, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL VCS_AreAllVirtualDevicesClosed();

    //Initialization gateway
    virtual HANDLE VCS_OpenVirtualDevice(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, HANDLE p_hVirtualCommandSet, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_RemoveAllVirtualDevices(HANDLE p_hVirtualCommandSet, CErrorInfo* p_pErrorInfo = NULL);

//Parameter
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo = NULL);

//Help functions
    virtual BOOL VCS_GetKeyHandle(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, HANDLE* p_pVCSKeyHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetKeyHandle(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, HANDLE* p_pVCSKeyHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetKeyHandle(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CLayerParameterStack& p_rAddressParameter, HANDLE* p_pVCSKeyHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetDefaultProtocolStackSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetGatewayKeyHandle(HANDLE p_hVCS_Handle, HANDLE* p_phVCS_GatewayHandle, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetBaseLayerKeyHandle(HANDLE p_hVCS_Handle,HANDLE* p_phPS_BaseLayerKeyHandle,CErrorInfo* p_pErrorInfo = NULL);

//Selection functions
    virtual BOOL VCS_GetVirtualDeviceNameSelection(CStdStringArray* p_pVirtualDeviceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetDeviceNameSelection(CStdString p_strVirtualDeviceName, CStdStringArray* p_pDeviceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetProtocolStackNameSelection(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetInterfaceNameSelection(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetPortNameSelection(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetBaudrateSelection(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);

//Enumeration
    virtual BOOL VCS_ResetPortEnumeration(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);

//Setting functions
    virtual BOOL VCS_GetProtocolStackSettings(HANDLE p_hVCS_Handle, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_SetProtocolStackSettings(HANDLE p_hVCS_Handle, DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo = NULL);

//Name functions
    virtual BOOL VCS_GetVirtualDeviceName(HANDLE p_hVCS_Handle, CStdString* p_pVirtualDeviceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetDeviceName(HANDLE p_hVCS_Handle, CStdString* p_pDeviceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetProtocolStackName(HANDLE p_hVCS_Handle, CStdString* p_pProtocolStackName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetInterfaceName(HANDLE p_hVCS_Handle, CStdString* p_pInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetPortName(HANDLE p_hVCS_Handle, CStdString* p_pPortName, CErrorInfo* p_pErrorInfo = NULL);

//Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, HANDLE p_hHandle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(HANDLE p_hHandle,ELayer p_eLayer,CStdString* p_pCommandInfo);

//Tracing
    virtual BOOL EnableTracing(HANDLE p_hVCS_Handle, CStdString p_TracingFileName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL DisableTracing(HANDLE p_hVCS_Handle, CErrorInfo* p_pErrorInfo = NULL);

//LayerManager
    virtual BOOL GetLayerManager(HANDLE p_hHandle, ELayer p_eLayer, CLayerManagerBase** p_ppLayerManager, HANDLE* p_pLayerHandle);
    virtual BOOL SetInterfaceManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CInterfaceManagerBase* p_pInterfaceManager);
    virtual BOOL GetInterfaceManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CInterfaceManagerBase** p_ppInterfaceManager);
    virtual BOOL RemoveInterfaceManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CInterfaceManagerBase* p_pInterfaceManager);
    virtual BOOL SetProtocolStackManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CProtocolStackManagerBase* p_pProtocolStackManager);
    virtual BOOL GetProtocolStackManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CProtocolStackManagerBase** p_ppProtocolStackManager);
    virtual BOOL RemoveProtocolStackManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CProtocolStackManagerBase* p_pProtocolStackManager);
    virtual BOOL SetDeviceCommandSetManager(CStdString p_strVirtualDeviceName, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager);
    virtual BOOL GetDeviceCommandSetManager(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CDeviceCommandSetManagerBase** p_ppDeviceCommandSetManager);
    virtual BOOL RemoveDeviceCommandSetManager(CStdString p_strVirtualDeviceName, CDeviceCommandSetManagerBase* p_ppDeviceCommandSetManager);

    CVirtualCommandSet_ManagerBase();
    CVirtualCommandSet_ManagerBase(int p_lInstanceValue);
    CVirtualCommandSet_ManagerBase(const CVirtualCommandSet_ManagerBase& p_rObject);
    virtual ~CVirtualCommandSet_ManagerBase();
    virtual CVirtualCommandSet_ManagerBase* Clone();
};
