#pragma once

#include <vector>
#include <memory>

#include "../BaseClasses/GatewayVCStoDCS.h"
#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEtherCatDefinitions.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>
#include <Thread/MmcCriticalSection.h>

class CDeviceCommandSetManagerBase;
class CCommand_DCS_Epos4;
class CCia402StateMachine;

class CGatewayVCStoEpos4 : public CGatewayVCStoDCS
{
public:
    CGatewayVCStoEpos4();
    virtual ~CGatewayVCStoEpos4();
    CGatewayVCStoEpos4& operator=(CGatewayVCStoEpos4& p_rOther);

protected:
    BOOL ReadInt8Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, __int8* p_pbValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadInt16Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, short* p_psValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadInt32Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, int* p_plValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadUInt8Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_puValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadUInt16Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, WORD* p_pusValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadUInt32Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, DWORD* p_pulValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadUInt64Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, __uint64* p_pullValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadStringObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, CStdString* p_pValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadDataBufferObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, void* p_pData, DWORD p_ulNumberOfBytesToRead, DWORD* p_pulNumberOfBytesRead, CErrorInfo* p_pErrorInfo = NULL);

    BOOL WriteInt8Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, __int8 p_bValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteInt16Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, short p_sValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteInt32Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, int p_lValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteUInt8Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE p_ubValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteUInt16Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, WORD p_usValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteUInt32Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, DWORD p_ulValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteUInt64Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, __uint64 p_ullValue, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteDataBufferObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, void* p_pData, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten, CErrorInfo* p_pErrorInfo = NULL);

    BOOL UpdateBitsInUInt16Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, WORD p_ulValue, WORD p_ulMask, CErrorInfo* p_pErrorInfo);
    BOOL UpdateBitsInUInt32Object(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, DWORD p_ulValue, DWORD p_ulMask, CErrorInfo* p_pErrorInfo);

    BOOL SetDisableState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo = NULL);
    BOOL CheckDeviceStates(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, std::vector<EDeviceState> p_usStates, CErrorInfo* p_pErrorInfo);
    BOOL IsStateInStates(EDeviceState p_usState, std::vector<EDeviceState> p_usStates);
    BOOL CheckDeviceState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usState, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadDeviceState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD* p_pusDeviceState, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadStatusWord(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD& p_rusStatusWord, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadControlWord(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD& p_rusControlWord, CErrorInfo* p_pErrorInfo = NULL);

    // Functionality network management
    BOOL DCS_SendNMTService(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeIdentifier, WORD p_usCommandSpecifier, CErrorInfo* p_pErrorInfo = NULL);

    // Functionality general gateway
    BOOL DCS_SendCANFrame(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usCobID, WORD p_usLength, void* p_pData, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_RequestCANFrame(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usCobID, WORD p_usLength, void* p_pData, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_ReadCANFrame(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usCobID, WORD p_usLength, void* p_pData, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo = NULL);

    // Layer setting services
    BOOL DCS_SendLSSFrame(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_ReadLSSFrame(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD usTimeout, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo = NULL);

    // ESC register
    BOOL DCS_WriteEscRegister(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usAdo, WORD p_usLength, void* p_pData, WORD* p_pusLengthWritten, CErrorInfo* p_pErrorInfo = 0);
    BOOL DCS_ReadEscRegister(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usAdo, WORD p_usLength, void* p_pData, WORD* p_pusLengthRead, CErrorInfo* p_pErrorInfo = 0);

    // SII parameter
    BOOL DCS_ReadSiiParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, ESiiParameterAddress p_eParameterAddress, void* p_pData, DWORD p_ulNbOfBytesToRead, CErrorInfo* p_pErrorInfo = 0);
    BOOL DCS_ReadSiiCategorySize(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, ESiiCategory p_eCategory, DWORD* p_pulCategorySize, CErrorInfo* p_pErrorInfo = 0);
    BOOL DCS_ReadSiiCategoryData(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, ESiiCategory p_eCategory, void* p_pData, DWORD p_ulNbOfBytesToRead, CErrorInfo* p_pErrorInfo = 0);

    // ESM
    BOOL DCS_InitEsm(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD p_ulVendorId, DWORD p_ulProductCode, DWORD p_ulRevisionNb, __uint64 p_ullSyncManager0, __uint64 p_ullSyncManager1, __uint64 p_ullSyncManager2, __uint64 p_ullSyncManager3, void* p_pFmmu0, void* p_pFmmu1, __uint64 p_ullBootStrap, CErrorInfo* p_pErrorInfo = 0);
    BOOL DCS_ChangeEsmState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usState, CErrorInfo* p_pErrorInfo = 0);
    BOOL DCS_GetEsmState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD& p_rusState, CErrorInfo* p_pErrorInfo = 0);

    //Addressing
    BOOL GetAddressParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rubNodeId, CErrorInfo* p_pErrorInfo);
    BOOL GetAddressParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rubNodeId, CErrorInfo* p_pErrorInfo);

    // Layer parameter stack
    BOOL InitLayerParameterStack(CCommandRoot* p_pCommand);

    // Device timeout
    BOOL GetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD& p_rulTimeout);
    BOOL SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulTimeout);
    BOOL InitGatewayTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);

    // Device transfer rate
    BOOL GetDeviceTransferRate(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD& p_rulTransferRate, CErrorInfo* p_pErrorInfo);
    BOOL SetDeviceTransferRate(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulTransferRate, CErrorInfo* p_pErrorInfo);

    // Parameter
    BOOL RestoreParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);
    BOOL RestoreFactoryParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);
    BOOL StoreParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);
    BOOL ResetDevice(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL p_oBroadcast, BOOL p_oWaitForReboot, CErrorInfo* p_pErrorInfo);
    BOOL WaitForReboot(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);

    // Sensors
    BOOL GetMainSensorType(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD& p_rusSensorType, CErrorInfo* p_pErrorInfo);
    BOOL GetAuxiliarySensorType(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD& p_rusSensorType, CErrorInfo* p_pErrorInfo);
    BOOL CalculateMainSensorResolution(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD& p_rulMainSensorResolution, CErrorInfo* p_pErrorInfo);
    BOOL CalculateAuxiliarySensorResolution(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD& p_rulMainSensorResolution, CErrorInfo* p_pErrorInfo);

    // Bit Mapping
    void MapBitFromDwordToWord(DWORD p_ulSource, BYTE p_ubSourceBit, WORD& p_rusDestination, BYTE p_ubDestinationBit);
    void MapBitFromWordToDword(WORD p_usSource, BYTE p_ubSourceBit, DWORD& p_rulDestination, BYTE p_ubDestinationBit);
    bool IsBitSetInDword(DWORD p_ulSource, BYTE p_ubBitNumber);
    bool IsBitSetInWord(WORD p_usSource, BYTE p_ubBitNumber);

    // Transfer rate conversion
    DWORD ConvertBitrateIndexToValue_CiaStandard(WORD p_usBaudrateIndex);
    WORD ConvertBitrateValueToIndex_CiaStandard(DWORD p_ulBaudrate);

    // SII categories
    BOOL ReadSiiSize(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD& p_rulSize, CErrorInfo* p_pErrorInfo);
    BOOL ReadSiiCategoryGeneral(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, SSiiCatagoryGeneral& p_sCategoryGeneral, CErrorInfo* p_pErrorInfo = 0);
    BOOL ReadSiiCategoryString(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_uStringIndex, CStdString& p_rStringValue, CErrorInfo* p_pErrorInfo = 0);
    BOOL WriteEscRegister(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usAdo, WORD p_usLength, void* p_pData, CErrorInfo* p_pErrorInfo = 0);
    BOOL ReadEscRegister(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usAdo, WORD p_usLength, void* p_pData, CErrorInfo* p_pErrorInfo = 0);

    WORD CalculateDeviceTimeout(DWORD p_ulTimeout);
    BOOL HandleBackwardsCompatiblity(BOOL p_oResult, CErrorInfo& p_rErrorInfo);

    //CANopen status
    BOOL IsGatewayInCanPassiveState(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);
    BOOL ResetGatewayCommunication(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);
    BOOL ClearGatewayErrors(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle);

private:
    BOOL ReadObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pubDataBuffer, DWORD p_ulMaxNumberOfBytesToRead, DWORD* p_pulNumberOfBytesRead, CErrorInfo* p_pErrorInfo = NULL);
    BOOL ReadSegmentedObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pubDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo = NULL);
    BOOL WriteObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pubDataBuffer, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten, CErrorInfo* p_pErrorInfo = NULL);
    BOOL RestoreDevice(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL p_oFactory, CErrorInfo* p_pErrorInfo);

    // State Machine
    BOOL GetDeviceTransitionsTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, std::vector<EDeviceState> p_targetStates, DWORD& p_rulTimeout, CErrorInfo* p_pErrorInfo);
    BOOL GetDeviceTransitionTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, EDeviceState p_targetState, DWORD& p_rulTimeout, CErrorInfo* p_pErrorInfo);
    BOOL GetDeviceStateTimeoutValue(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTransition, DWORD& p_rulTimeoutDeviceState, CErrorInfo* p_pErrorInfo);
    BOOL GetDecelerationTime(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTransition, DWORD& p_rulDecelerationTime, CErrorInfo* p_pErrorInfo);
    BOOL GetStandstillTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTransition, WORD& p_rusStandstillWindowTimeout, CErrorInfo* p_pErrorInfo);
    BOOL IsHoldingBrakeConfigured(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo);
    BOOL GetBrakeRiseTime(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTransition, WORD& p_rusHoldingBrakeRiseTime, CErrorInfo* p_pErrorInfo);
    BOOL GetBrakeFallTime(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTransition, WORD& p_rusHoldingBrakeFallTime, CErrorInfo* p_pErrorInfo);

    // Sensors
    BOOL CalculateSensorResolution(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usSensorType, DWORD& p_rulSensorResolution, CErrorInfo* p_pErrorInfo);
    BOOL GetMainSensor(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE& p_rubMainSensor, CErrorInfo* p_pErrorInfo);
    BOOL GetAuxiliarySensor(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE& p_rubAuxiliarySensor, CErrorInfo* p_pErrorInfo);
    BOOL GetSensor1Type(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubSensor, WORD& p_rusSensor1Type, CErrorInfo* p_pErrorInfo);
    BOOL GetSensor2Type(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubSensor, WORD& p_rusSensor2Type, CErrorInfo* p_pErrorInfo);
    BOOL GetSensor3Type(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubSensor, WORD& p_rusSensor3Type, CErrorInfo* p_pErrorInfo);
    BOOL GetSensorType(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubSensor, WORD& p_rusSensorType, CErrorInfo* p_pErrorInfo);

private:
    // Objection dictionary instruction set
    BOOL DCS_ReadObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pObjectBuffer, DWORD p_ulObjectLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_WriteObject(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pObjectBuffer, DWORD p_ulObjectLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_InitiateSegmentedRead(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, BYTE* p_pubSegmentBuffer, DWORD p_ulSegmentBufferLength, DWORD* p_pulSegmentLengthRead, DWORD* p_pulCompleteLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_InitiateSegmentedWrite(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, DWORD p_ulObjectLength, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_SegmentedRead(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BOOL p_oToggle, BOOL* p_poLastSegment, BYTE* p_pSegmentBuffer, DWORD p_ulSegmentBufferLength, DWORD* p_pulSegmentLengthRead, CErrorInfo* p_pErrorInfo = NULL);
    BOOL DCS_SegmentedWrite(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BOOL p_oToggle, BOOL p_oLastSegment, BYTE* p_pubSegmentBuffer, DWORD p_ulSegmentLength, DWORD* p_pulSegmentLengthWritten, CErrorInfo* p_pErrorInfo = NULL);

    BOOL DCS_AbortTransfer(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usIndex, WORD p_usSubIndex, DWORD p_ulAbortCode, CErrorInfo* p_pErrorInfo = NULL);

    void InitCommands_ObjectDictionary();
    void InitCommands_NetworkManagement();
    void InitCommands_GeneralGateway();
    void InitCommands_LayerSettingServices();
    void InitCommands_Register();
    void InitCommands_EtherCatStateMachine();

    void DeleteCommands_ObjectDictionary();
    void DeleteCommands_NetworkManagement();
    void DeleteCommands_GeneralGateway();
    void DeleteCommands_LayerSettingServices();
    void DeleteCommands_Register();
    void DeleteCommands_EtherCatStateMachine();

    // Transfer rate conversion
    BOOL CheckBitrateIndexCiaStandard(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usBaudrateIndex);
    BOOL CheckBitrateValueCiaStandard(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulBaudrate);
    BOOL CheckBitrateCiaStandard(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle);
    DWORD ConvertBaudrateIndexToValue(WORD p_usBaudrateIndex);
    WORD ConvertBaudrateValueToIndex(DWORD p_ulBaudrate);
    DWORD ConvertBitrateIndexToValue_Maxon(WORD p_usBaudrateIndex);
    WORD ConvertBitrateValueToIndex_Maxon(DWORD p_ulBaudrate);

    BOOL IsCompleteSizeRead(DWORD p_ulCompleteSize, DWORD p_ulTotalNbOfBytesRead);

    BOOL CreateStateMachine();
    BOOL DeleteStateMachine();
    BOOL IsGatewayErrorCode(CErrorInfo& p_rErrorInfo);
    BOOL IsEtherCatInterface(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle);
    BOOL IsCANopenInterface(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle);

    BOOL RemoveStationId(CCommandRoot* p_pCommand, CLayerParameterStack& p_rBackupLayerParameterStack);
    BOOL RestoreLayerParameterStack(CCommandRoot* p_pCommand, CLayerParameterStack& p_rBackupLayerParameterStack);

private:
    CCia402StateMachine* m_pCia402StateMachine;
    CCommand_DCS_Epos4* m_pCommand_AbortTransfer;
    CCommand_DCS_Epos4* m_pCommand_InitiateSegmentedRead;
    CCommand_DCS_Epos4* m_pCommand_InitiateSegmentedWrite;
    CCommand_DCS_Epos4* m_pCommand_ReadCANFrame;
    CCommand_DCS_Epos4* m_pCommand_ReadLSSFrame;
    CCommand_DCS_Epos4* m_pCommand_ReadObject;
    CCommand_DCS_Epos4* m_pCommand_RequestCANFrame;
    CCommand_DCS_Epos4* m_pCommand_SegmentedRead;
    CCommand_DCS_Epos4* m_pCommand_SegmentedWrite;
    CCommand_DCS_Epos4* m_pCommand_SendCANFrame;
    CCommand_DCS_Epos4* m_pCommand_SendLSSFrame;
    CCommand_DCS_Epos4* m_pCommand_SendNMTService;
    CCommand_DCS_Epos4* m_pCommand_WriteObject;
    CCommand_DCS_Epos4* m_pCommand_InitEsm;
    CCommand_DCS_Epos4* m_pCommand_ChangeEsmState;
    CCommand_DCS_Epos4* m_pCommand_GetEsmState;
    CCommand_DCS_Epos4* m_pCommand_WriteEscRegister;
    CCommand_DCS_Epos4* m_pCommand_ReadEscRegister;

    static std::auto_ptr<CMmcCriticalSection> m_SyncObject;
};
