#pragma once

#include <MmcTypeDefinition.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

class CJournalManagerBase;
class CDeviceCommandSetManagerBase;
class CProtocolStackManagerBase;
class CGatewayVCStoDCS;
class CCommandRoot;

class CVirtualDeviceBase
{
public:
    CVirtualDeviceBase();
    CVirtualDeviceBase(const CVirtualDeviceBase& p_rOjbect);
    virtual ~CVirtualDeviceBase();
    virtual CVirtualDeviceBase* Clone();
    virtual BOOL IsEqual(CVirtualDeviceBase* p_pVirtualDevice);

    //Initialisation
    virtual BOOL InitVirtualDevice(CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitVirtualDevice(CStdString p_strDeviceName, CErrorInfo* p_pErrorInfo);
    virtual BOOL InitVirtualDevice(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitVirtualDevice(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strDriverName, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL InitVirtualGatewayDevice(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_gatewayDeviceName, CStdString p_gatewayProtocolStackName, CErrorInfo* p_pErrorInfo = NULL);

    //Help functions
    virtual BOOL VCS_GetDeviceNameSelection(CStdStringArray* pDeviceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetProtocolStackNameSelection(CStdString p_strDeviceName, CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetInterfaceNameSelection(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetPortNameSelection(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo = NULL);
    virtual BOOL VCS_GetBaudrateSelection(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo = NULL);

    virtual BOOL VCS_GetVirtualDeviceName(CStdString* p_pVirtualDeviceName);

    //Enumeration
    virtual BOOL VCS_ResetPortEnumeration(CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo = NULL);

    //Interne Funktionnen
    virtual BOOL AddManager(CDeviceCommandSetManagerBase* p_pManager);
    virtual BOOL RemoveManager(CDeviceCommandSetManagerBase* p_pManager);
    virtual BOOL FindCorrectManager(CStdString p_strDeviceName, CDeviceCommandSetManagerBase** p_ppManager);
    virtual BOOL FindCorrectLibrary(CStdString p_strDeviceName, CDeviceCommandSetManagerBase** p_ppManager);
    virtual BOOL InitGateway(CStdString p_strVirtualDeviceName);
    virtual BOOL InitBaudrateSelection(CStdString p_strDeviceName, CStdString p_strProtocolStack, CStdString p_strInterfaceName, CStdDWordArray& p_ulBaudrateSel);
    virtual BOOL InitDefaultProtocolStackSettings(CStdString p_strDeviceName, CStdString p_strProtocolStack, CStdString p_strInterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

    //Functionality
    virtual BOOL ExecuteCommand(CCommandRoot* p_pCommand, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle);
    virtual BOOL GetCommands(CStdString* p_pCommandInfo);
    virtual BOOL AbortCommands(BOOL p_oActive);

    //JournalManager
    virtual void InitJournalManager(CJournalManagerBase* p_pJournalManager);
    virtual void ResetJournalManager();

    //ErrorHandling
    virtual BOOL InitErrorHandling();
    virtual BOOL DeleteErrorHandling();

    //Instance Value for Singletons
    virtual BOOL InitInstanceValue(int p_lInstanceValue);

    //Parameter
    BOOL AreParameterEqual(CVirtualDeviceBase* p_pVirtualDevice);
    BOOL SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL SetParameter(CStdString p_Name, CStdString p_Value);
    BOOL GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize);
    BOOL GetParameter(CStdString p_Name, CStdString& p_rValue);

    //Critical Section
    BOOL Lock(DWORD p_ulTimeout = -1);
    BOOL Unlock();
    BOOL IsLocked();

    BOOL SetGatewayDeviceTimeout(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, DWORD p_ulGatewayTimeout);

protected:
    BOOL GetFirstManager(CDeviceCommandSetManagerBase** p_ppManager);
    void DeleteDCS_ManagerList();
    void DeleteGateway();

private:
    virtual BOOL VerifyGateway(CStdString p_virtualDeviceName);
    BOOL AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray);

protected:
    std::list<CDeviceCommandSetManagerBase*> m_DeviceCommandSetManagerList;
    CStdString m_strVirtualDeviceName;
    CGatewayVCStoDCS* m_pGateway;
    CJournalManagerBase* m_pJournalManager;

    int m_lInstanceValue;
    CErrorHandling* m_pErrorHandling;
};
