#pragma once

#include <CommunicationModel/VirtualCommandSet/Dialogs/SearchCommunicationDefinitions.h>

class CCommandRoot;
class CCommunicationSetting;
class CSearchCommunicationThread;
class CVirtualCommandSet_ManagerBase;

#ifdef WINVER
class CSearchCommunicationDlg : public CDialog
{
public:
    CSearchCommunicationDlg(CWnd* pParent = NULL);

    void Init(CVirtualCommandSet_ManagerBase* p_pManager, HANDLE p_hHandle, CCommandRoot* p_pCommand, CStdString p_searchingInfo, EDialogMode p_eDialogMode);
    void InitRootCommunicationSetting(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName);
    void InitStartCommunicationSetting(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

    BOOL SearchSucceeded();
    BOOL GetHandle(HANDLE* p_phHandle);
    BOOL GetCommunicationSetting(CStdString* p_pVirtualDeviceName, CStdString* p_pDeviceName, CStdString* p_pProtocolStackName, CStdString* p_pInterfaceName, CStdString* p_pPortName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout);
    void SetCommunicationSetting(CCommunicationSetting* p_pSetting, HANDLE p_hHandle);

    CProgressCtrl m_SearchProgress;

public:
    virtual BOOL DestroyWindow();
protected:
    virtual void DoDataExchange(CDataExchange* pDX);

protected:
    LRESULT OnNextCommunicationSetting(WPARAM wParam, LPARAM lParam);
    LRESULT OnEndCommunicationSearch(WPARAM wParam, LPARAM lParam);
    afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
    virtual BOOL OnInitDialog();
    virtual void OnOK();

    DECLARE_MESSAGE_MAP()

private:
    void InitSearchingInfo(CStdString p_searchingInfo);
    void InitStartHandle(HANDLE p_hHandle);

    void CreateSearchCommunicationThread();
    void DeleteSearchCommunicationThread();

    void DeleteCommunicationSetting();
    void DeleteRootCommunicationSetting();
    void DeleteStartCommunicationSetting();

    void InitCommunicationSettingList();
    void AddCommunicationSettings();
    void AddCommunicationSettings(CStdString strVirtualDeviceName);
    void AddCommunicationSettings(CStdString p_virtualDeviceName, CStdString p_deviceName);
    void AddCommunicationSettings(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName);
    void AddCommunicationSettings(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName);
    void AddCommunicationSettings(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName);
    BOOL IsCommunicationSettingFiltered(CStdString p_virtualDeviceName, CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName, DWORD p_ulBaudrate, DWORD p_ulTimeout);
    void DeleteCommunicationSettingList();

    CStdStringArray::iterator FindStartIndex(CStdStringArray* p_pNameArray, CStdString p_name);
    CStdDWordArray::iterator FindStartIndex(CStdDWordArray* p_pValueArray, DWORD p_ulValue);
    CStdDWordArray::iterator FindMaxValueIndex(CStdDWordArray* p_pValueArray);

    void SortNameArray(CStdStringArray* p_pNameArray, CStdString p_name);
    void SortValueArray(CStdDWordArray* p_pValueArray, DWORD p_ulValue);

    void UpdateDialogSize();
    void UpdateSearchSettingStrings(int p_lSettingIndex);
    void UpdateSearchSettingLineStrings(int p_lLineIndex, CStdString p_title, CStdString p_value);
    void ShowMessage();
    CStdString GetMessageString();
    BOOL IsConfirmDlgEnabled();
    BOOL IsProgressDlgEnabled();

private:
    CVirtualCommandSet_ManagerBase* m_pVirtualCommandSetManager;

    CSearchCommunicationThread* m_pSearchCommunicationThread;

    CCommunicationSetting* m_pRootCommunicationSetting;
    BOOL m_oIsDeviceNameVisible;
    BOOL m_oIsProtocolStackNameVisible;
    BOOL m_oIsInterfaceNameVisible;
    BOOL m_oIsPortNameVisible;

    CStdString m_searchingInfo;
    std::list<CCommunicationSetting*> m_CommunicationSettingList;
    CCommunicationSetting* m_pStartCommunicationSetting;

    BOOL m_oCommunicationFound;
    HANDLE m_hHandle;
    CCommunicationSetting* m_pCommunicationSetting;

    EDialogMode m_eDialogMode;
};
#endif //WINVER
