#pragma once

#ifdef WINVER
#include <CommunicationModel/VirtualCommandSet/Dialogs/SearchCommunicationDefinitions.h>

class CVirtualCommandSet_ManagerBase;
class CCommunicationSetting;
class CSearchCommunicationWholeCanIdRangeThread;

class CSearchCommunicationWholeCanIdRangeDlg : public CDialog
{
public:
    CSearchCommunicationWholeCanIdRangeDlg(CWnd* pParent = NULL);

    void Init(CVirtualCommandSet_ManagerBase* p_pManager, HANDLE p_hHandle, CCommandRoot* p_pCommand, CStdString p_strSearchingInfo, EDialogMode p_eDialogMode);
    void InitRootCommunicationSetting(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName);
    void InitStartCommunicationSetting(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

    BOOL SearchSucceeded();
    BOOL GetHandle(HANDLE* p_phHandle);
    BOOL GetNodeId(WORD* p_pusNodeId);
    BOOL GetCommunicationSetting(CStdString* p_StrVirtualDeviceName, CStdString* p_pStrDeviceName, CStdString* p_pStrProtocolStackName, CStdString* p_pStrInterfaceName, CStdString* p_pStrPortName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout);
    void SetCommunicationSetting(CCommunicationSetting* p_pSetting, HANDLE p_hHandle);

    CProgressCtrl m_SearchProgress;

public:
    virtual BOOL DestroyWindow();

protected:
    virtual void DoDataExchange(CDataExchange* pDX);

protected:
    LRESULT OnNextCommunicationSetting(WPARAM p_ulParam, LPARAM p_lParam);
    LRESULT OnEndCommunicationSearch(WPARAM p_ulParam, LPARAM p_lParam);
    LRESULT OnUpdateProgress(WPARAM p_ulParam, LPARAM p_lParam);
    afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
    virtual BOOL OnInitDialog();
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    virtual void OnOK();

    DECLARE_MESSAGE_MAP()

private:
    void InitSearchingInfo(CStdString p_SearchingInfoString);
    void InitStartHandle(HANDLE p_hHandle);

    void CreateSearchCommunicationWholeCanIdRangeThread();
    void DeleteSearchCommunicationWholeCanIdRangeThread();

    void DeleteCommunicationSetting();
    void DeleteRootCommunicationSetting();
    void DeleteStartCommunicationSetting();

    void InitCommunicationSettingList();
    void InitProgressRange();
    void AddCommunicationSettings();
    void AddCommunicationSettings(CStdString p_strVirtualDeviceName);
    void AddCommunicationSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName);
    void AddCommunicationSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName);
    void AddCommunicationSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName);
    void AddCommunicationSettings(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName);
    BOOL IsCommunicationSettingFiltered(CStdString p_strVirtualDeviceName, CStdString p_strDeviceName, CStdString p_strProtocolStackName, CStdString p_strInterfaceName, CStdString p_strPortName, DWORD p_ulBaudrate, DWORD p_ulTimeout);

    void DeleteCommunicationSettingList();

    CStdStringArray::iterator FindStartIndex(CStdStringArray* p_pNameArray, CStdString p_strName);
    CStdDWordArray::iterator FindStartIndex(CStdDWordArray* p_pValueArray, DWORD p_ulValue);
    CStdDWordArray::iterator FindMaxValueIndex(CStdDWordArray* p_pValueArray);

    void SortNameArray(CStdStringArray* p_pNameArray, CStdString p_strName);
    void SortValueArray(CStdDWordArray* p_pValueArray, DWORD p_ulValue);

    void UpdateDialogSize();
    void UpdateSearchSettingStrings(int p_lSettingIndex);
    void UpdateSearchSettingLineStrings(int p_lLineIndex, CStdString p_strTitle, CStdString p_strValue);
    void ShowMessage();
    CStdString GetMessageString();

    BOOL IsConfirmDlgEnabled();
    BOOL IsProgressDlgEnabled();

    void UpdateWindowVisibility();

private:
    CVirtualCommandSet_ManagerBase* m_pVirtualCommandSetManager;

    CSearchCommunicationWholeCanIdRangeThread* m_pSearchCommunicationWholeCanIdRangeThread;

    CCommunicationSetting* m_pRootCommunicationSetting;
    BOOL m_oIsDeviceNameVisible;
    BOOL m_oIsProtocolStackNameVisible;
    BOOL m_oIsInterfaceNameVisible;
    BOOL m_oIsPortNameVisible;

    CStdString m_strSearchingInfoString;
    std::list<CCommunicationSetting*> m_CommunicationSettingList;
    CCommunicationSetting* m_pStartCommunicationSetting;

    BOOL m_oCommunicationFound;
    HANDLE m_hHandle;
    CCommunicationSetting* m_pCommunicationSetting;
    WORD m_usNodeId;

    EDialogMode m_eDialogMode;
};
#endif //WINVER
