#pragma once

#include <CommunicationModel/VirtualCommandSet/Dialogs/SearchCommunicationDefinitions.h>

class CSearchSubCommunicationThread;
class CVirtualCommandSet_ManagerBase;

#ifdef WINVER
class CSearchSubCommunicationDlg : public CDialog
{
public:
    CSearchSubCommunicationDlg(CWnd* pParent = NULL);

    void Init(CVirtualCommandSet_ManagerBase* p_pManager, HANDLE p_hDevice, CStdString p_searchingInfo, EDialogMode p_eDialogMode);

    BOOL GetObject(HANDLE p_hDevice, BYTE p_ubNodeId, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, void* p_pData, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, DWORD* p_pErrorCode = 0);
    BOOL SetObject(HANDLE p_hDevice, BYTE p_ubNodeId, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, void* p_pData, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, DWORD* p_pErrorCode = 0);

    BOOL GetParameter(HANDLE* p_phHandle, BYTE* p_pubNodeId);

    BOOL GetGatewaySettings(DWORD& p_rulBaudrate, DWORD& p_rulTimeout);
    BOOL ChangeGatewaySettings(DWORD p_ulBaudrate, DWORD p_ulTimeout);
    BOOL ResetGatewaySettings();

public:
    CProgressCtrl m_SearchProgress;

public:
    virtual BOOL DestroyWindow();

protected:
    virtual void DoDataExchange(CDataExchange* pDX);

protected:
    virtual BOOL OnInitDialog();
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    virtual void OnOK();

    afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);

    LRESULT OnEndCommunicationSearch(WPARAM wParam, LPARAM lParam);
    LRESULT OnNextCommunicationSetting(WPARAM wParam, LPARAM lParam);
    LRESULT OnNextNodeSetting(WPARAM wParam, LPARAM lParam);

    DECLARE_MESSAGE_MAP()

private:
    void CreateSearchCommunicationThread();
    void DeleteSearchCommunicationThread();

    BOOL Store(HANDLE p_hDevice, BYTE p_ubNodeId, DWORD* p_pErrorCode = 0);
    BOOL ResetDevice(HANDLE p_hDevice, BYTE p_ubNodeId, DWORD* p_pErrorCode = 0);

    void InitDeviceName();
    void InitDeviceNodeId();
    void InitSearchingInfo(CStdString p_searchingInfo);
    void InitBaudrateList();
    void InitNodeIdList();
    void InitProgressRange();

    void UpdateDialogSize();
    void UpdateSearchSettingCommunicationStrings(int p_lBaudrateIndex);
    void UpdateSearchSettingNodeStrings();
    void UpdateSearchSettingLineStrings(int p_lLineIndex, CStdString p_title, CStdString p_value);
    BOOL IsConfirmDlgEnabled();
    BOOL IsProgressDlgEnabled();

    void SortValueArray(std::list<DWORD>* p_paulValues, DWORD p_ulValue);
    std::list<DWORD>::iterator FindStartIndex(std::list<DWORD>* p_paulValues, DWORD p_ulValue);

    void ShowMessage();
    CStdString GetMessageString();

    BOOL GetBaudrate(CStdString& p_rBaudrate);
    CStdString GetBaudrate(int p_lBaudrateIndex);
    BOOL GetSubDeviceName(CStdString& p_rSubDeviceName);

    void OpenSubDevice();

    void UpdateWindowVisibility();

private:
    BOOL m_oDeviceFound;
    BYTE m_ubNodeId;
    BYTE m_ubNodeIdSearch;
    CSearchSubCommunicationThread* m_pSearchCommunicationThread;
    CStdString m_deviceName;
    CStdString m_searchingInfo;
    CStdString m_subDeviceName;
    CVirtualCommandSet_ManagerBase* m_pVirtualCommandSetManager;
    DWORD m_ulBaudrateBackup;
    DWORD m_ulTimeoutBackup;
    EDialogMode m_eDialogMode;
    HANDLE m_hDevice;
    HANDLE m_hSubDevice;
    std::list<BYTE> m_aNodeId;
    std::list<DWORD> m_aBaudrate;
};
#endif //WINVER
