#pragma once

#include <Classes/TypeDefinitions.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <RecorderCmdManager/Classes/RecorderCmdDefinitions.h>

class CRecorderCmdChannel;
class CRecorderCmdManager;
class CVirtualDeviceCommunicationModel;

class CRecorderCmdConfiguration
{
public:
    void SetAquisitionMode(EAquisitionMode p_eAquisitionMode);
    EAquisitionMode GetAquisitionMode();

    CRecorderCmdConfiguration();
    virtual ~CRecorderCmdConfiguration();

    BOOL Init(CRecorderCmdManager* p_pRecorderManager);
    void InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel);
    void Init(HANDLE p_hKeyHandle, CLayerParameterStack& p_rAddressParameter);

    void GetTotalTimeSelection(CStdStringArray* p_pTotalTimeSelection);
    void GetSamplingPeriodSelection(CStdStringArray* p_pSamplingPeriodSelection);
    void GetVariableSizeSelection(CStdStringArray* p_pVariableSizeSelection);

    void Serialize(CArchive& ar);
    CRecorderCmdConfiguration& operator=(CRecorderCmdConfiguration &other);

    //RecorderChannel Methods
    void AddNewRecorderChannel();
    CRecorderCmdChannel* GetRecorderChannel(int p_lChannelIndex);
    int GetNbOfRecorderChannels();
    int GetNbOfConfiguredChannels();

    WORD GetSelObjectSize(int p_lChannelIndex);
    BOOL IsSelObjectSizeAvailable(int p_lChannelIndex);

    //Data Sampling Mode
    void SetTotalTimeMaster(BOOL p_oTotalTimeMaster);
    BOOL IsTotalTimeMaster();

    //String Methods
    CStdString GetTotalTimeStr();
    CStdString GetSamplingPeriodStr();
    CStdString GetNbOfSamplesStr();
    CStdString GetPrecedingTimeStr();
    CStdString GetMaxPrecedingTimeStr();
    CStdString GetMaxNbOfPrecedingSamplesStr();
    CStdString GetNbOfPrecedingSamplesStr();

    //Setting Methods
    void SetMicroSecondsSamplingPeriod(DWORD p_ulMicroSecondsSamplingPeriod);
    void SetMicroSecondsSamplingPeriodStr(CStdString p_strMicroSecondsSamplingPeriod);
    void SetMicroSecondsTotalTime(DWORD p_ulMicroSecondsTotalTime);
    void SetMicroSecondsTotalTimeStr(CStdString p_strMicroSecondsTotalTime);

    BOOL SetSelObject(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex, CStdString p_type);
    BOOL SetSelObject(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex, WORD p_usType);
    BOOL GetSelObject(int p_lChannelIndex, WORD& p_rusIndex, BYTE& p_rubSubIndex, WORD& p_rusType);

    void SetPrecedingSamples(WORD p_usNbOfPrecedingSamples);
    void SetPrecedingTimeFactor(float p_fPrecedingTimeFactor);

    //DataRecorder Mode
    void SetObjectMode();

    //Value Methods
    WORD GetPrecedingSamples();
    WORD GetMaxPrecedingSamples();
    void ActivateErrorTrigger(BOOL p_oActive);
    void ActivateMovementTrigger(BOOL p_oActive);
    void ActivateEndOfProfileTrigger(BOOL p_oActive);
    void ActivateDigInputTrigger(BOOL p_oActive);
    BOOL IsErrorTriggerActive();
    BOOL IsMovementTriggerActive();
    BOOL IsEndOfProfileTriggerActive();
    BOOL IsDigInputTriggerActive();

    //Device Writing/Reading
    BOOL UpdateData(BOOL p_oSaveAndValidate);

    //Address Parameter
    BOOL UpdateAddressParameter(CLayerParameterStack& p_rAddressParameter);

private:
    BOOL ReadVariable(BYTE p_ubChannelNumber, WORD* p_pusIndex, BYTE* p_pubSubIndex, WORD* p_pusType);
    BOOL WriteVariable(BYTE p_ubChannelNumber, WORD p_usIndex, BYTE p_ubSubIndex, WORD p_usType);

    BOOL ReadChannelFromDevice(int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel);
    BOOL RecalculateMaxNbOfSamples();
    BOOL UpdateAllChannelActivation(BOOL p_oSaveAndValidate);
    BOOL UpdateAllChannels(BOOL p_oSaveAndValidate);
    BOOL UpdateChannel(BOOL p_oSaveAndValidate, int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel);
    BOOL UpdateDataSampling(BOOL p_oSaveAndValidate);
    BOOL UpdateRecorderConfiguration(BOOL p_oSaveAndValidate);
    BOOL StartRecorder(BOOL p_oSaveAndValidate);
    BOOL WriteChannelToDevice(int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel);
    CStdString GetTimeStr(DWORD p_ulMicroSeconds);
    DWORD GetTimeValue(CStdString p_strTimeValue);
    void DeleteRecorderChannelList();
    void InitRecorderChannelList();
    void InitSelections();

    //Object Access
    BOOL GetObject(WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE* p_pData, DWORD p_ulNbOfBytesToRead, DWORD* p_pulErrorCode);
    BOOL SetObject(WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE* p_pData, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulErrorCode);

private:
    const WORD MICRO_SECONDS_SAMPLING_CYCLE;

    //Variables of Device
    SRecorderConfiguration m_RecorderConfiguration;
    WORD m_usSamplingPeriod;
    WORD m_usNbOfPrecedingSamples;
    WORD m_usMaxNbOfSamples;
    std::vector<CRecorderCmdChannel*> m_RecorderChannelList;

    //Additional Variables
    BOOL m_oTotalTimeMaster;
    DWORD m_ulMicroSeconds_TotalTime;
    DWORD m_ulMicroSeconds_SamplingPeriod;
    DWORD m_ulErrorCode;
    WORD m_usNbOfSamples;
    float m_fPrecedingTimeFactor;

    EAquisitionMode m_AquisitionMode;

    //TotalTime Selection
    CStdDWordArray m_TotalTimeSelection;
    //SamplingPeriod Selection
    CStdDWordArray m_SamplingPeriodSelection;
    //VariableSize Selection
    CStdStringArray m_VariableSizeSelection;

    CRecorderCmdManager* m_pRecorderManager;
    CVirtualDeviceCommunicationModel* m_pCommunicationModel;
    HANDLE m_hKeyHandle;
    CLayerParameterStack m_AddressParameterStack;
};
