#pragma once

#include <MmcTypeDefinition.h>

class CLayerParameterStack;
class CRecorderCmdConfiguration;
class CRecorderConfigurationMap;
class CVirtualDeviceCommunicationModel;

class CRecorderCmdManager
{
public:
    CRecorderCmdManager(void);
    virtual ~CRecorderCmdManager(void);

    BOOL InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel);

    CRecorderConfigurationMap* GetRecorderConfigurationMap();
    CVirtualDeviceCommunicationModel* GetCommunicationModel();

//DataRecorder Setup
    BOOL SetRecorderParameter(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usSamplingPeriod, WORD p_usNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetRecorderParameter(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusSamplingPeriod, WORD* p_pusNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL EnableTrigger(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubTriggerType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL DisableAllTriggers(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ActivateChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE p_ubObjectSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ActivateChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, WORD p_usObjectType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ActiveChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD* p_pusObjectIndex, BYTE* p_pubObjectSubIndex, WORD* p_pusObjectType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL DeactivateAllChannels(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetVariable(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usIndex, WORD* p_pusVarIndex, WORD* p_pusVarSubIndex, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetMaxNbOfSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusMaxNbOfSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetNbOfRecordedSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL SetNbOfPrecedingSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetNbOfPrecedingSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetNbOfSamplingVariables(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfSamplingVariables, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetSamplingPeriod(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusSamplingPeriod, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);

//DataRecorder Status
    BOOL StartRecorder(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL StopRecorder(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ForceTrigger(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL IsRecorderRunning(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BOOL* p_oRunning, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL IsRecorderTriggered(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BOOL* p_oTriggered, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetRecorderStatus(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_ulRecorderStatus, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL SetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usRecorderConfiguration, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusRecorderConfiguration, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);

//DataRecorder Data
    BOOL ReadChannelVectorSize(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulVectorSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ReadChannelDataVector(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, BYTE* p_pubDataVector, DWORD p_ulVectorSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL GetVectorStartOffset(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusVectorStartOffset, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);

//Advanced Functions
    BOOL ReadDataBuffer(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE* p_pubDataBuffer, DWORD p_ulBufferSizeToRead, DWORD* p_pulBufferSizeRead, WORD* p_pusVectorStartOffset, WORD* p_pusMaxNbOfSamples, WORD* p_pusNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);
    BOOL ExtractChannelDataVector(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, BYTE* p_pubDataBuffer, DWORD p_ulBufferSize, BYTE* p_pubDataVector, DWORD p_ulVectorSize, WORD p_usVectorStartOffset, WORD p_usMaxNbOfSamples, WORD p_usNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle = NULL, BOOL p_oDoLock = TRUE);

private:
    CRecorderCmdConfiguration* GetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter);
    BOOL UpdateAddressParameter(CLayerParameterStack& p_rAddressParameter);
    WORD ConvertSizeToType(BYTE p_ubObjectSize);

private:
    CRecorderConfigurationMap* m_pRecorderConfigurationMap;
    CVirtualDeviceCommunicationModel* m_pCommunicationModel;
};
