#include "stdafx.h"

#include <Classes/MmcDataConversion.h>
#include <CommunicationModel/Common/Journal/BaseClasses/JournalManagerBase.h>
#include <CommunicationModel/Common/Journal/Time/JournalTime.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/ParameterInfo/ParameterInfo.h>
#include <CommunicationModel/CommonLayer/Classes/Gateway/Gateway.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/CommandRoot.h>

using namespace std;

CCommandRoot::CCommandRoot()
{
    m_oTransactionEventEnabled = TRUE;
    m_pErrorInfo = 0;

    m_pParameterInfo = new CParameterInfo();
    m_pLayerParameterStack = new CLayerParameterStack();
    m_pJournalTime = new CJournalTime();

    ResetCommand();
    ActivateLogMessage();
}

CCommandRoot::~CCommandRoot()
{
    if(m_pParameterInfo)
    {
        delete m_pParameterInfo;
        m_pParameterInfo = 0;
    }

    if(m_pLayerParameterStack)
    {
        delete m_pLayerParameterStack;
        m_pLayerParameterStack = 0;
    }

    if(m_pJournalTime)
    {
        delete m_pJournalTime;
        m_pJournalTime = 0;
    }

    ResetErrorInfo();
}

void CCommandRoot::ResetErrorInfo()
{
    if(m_pErrorInfo)
    {
        delete m_pErrorInfo;
        m_pErrorInfo = 0;
    }
}

BOOL CCommandRoot::SetTimeout(DWORD p_ulTimeout)
{
    BOOL oResult = TRUE;

    m_dTimeout = p_ulTimeout;

    return oResult;
}

BOOL CCommandRoot::ResetTimeout()
{
    BOOL oResult = TRUE;

    m_dTimeout = UINT_MAX;

    return oResult;
}

DWORD CCommandRoot::GetTimeout()
{
    return m_dTimeout;
}

BOOL CCommandRoot::IsTimeoutValid()
{
    return (m_dTimeout != UINT_MAX);
}

void CCommandRoot::AddParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddParameter(p_lParameterIndex, p_ParameterName, p_eParameterType);
    }
}

void CCommandRoot::AddParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, BOOL p_oVisible, BOOL p_oEditable)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_oVisible, p_oEditable);
    }
}

void CCommandRoot::AddParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_ulArraySize);
    }
}

void CCommandRoot::AddParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize, BOOL p_oVisible, BOOL p_oEditable)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_oVisible, p_oEditable);
    }
}

void CCommandRoot::AddReturnParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddReturnParameter(p_lParameterIndex, p_ParameterName, p_eParameterType);
    }
}

void CCommandRoot::AddReturnParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, BOOL p_oVisible)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddReturnParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_oVisible);
    }
}

void CCommandRoot::AddReturnParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddReturnParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_ulArraySize);
    }
}

void CCommandRoot::AddReturnParameter(int p_lParameterIndex, CStdString p_ParameterName, EObjectDataType p_eParameterType, DWORD p_ulArraySize, BOOL p_oVisible)
{
    if(m_pParameterInfo)
    {
        m_pParameterInfo->AddReturnParameter(p_lParameterIndex, p_ParameterName, p_eParameterType, p_oVisible);
    }
}

CCommandRoot* CCommandRoot::CloneCommand()
{
    CCommandRoot* pNewCommand;

    pNewCommand = new CCommandRoot();
    *pNewCommand = *this;

    return pNewCommand;
}

BOOL CCommandRoot::DoTransactionEvent(HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(p_hTransactionHandle != 0 && m_oTransactionEventEnabled)
    {
        if(m_pJournalManager)
        {
            oResult = m_pJournalManager->TransactionEvent(m_eLayer, p_hTransactionHandle, this);
        }
    }

    return oResult;
}

BOOL CCommandRoot::EnableTransactionEvent(BOOL p_oEnabled)
{
    m_oTransactionEventEnabled = p_oEnabled;

    return TRUE;
}

BOOL CCommandRoot::Execute(CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        //Start time
        StartCommandTime();

        //Init
        ResetStatus();

        //Process command
        oResult = m_pGateway->ProcessCommand(this, p_pManager, p_h_Handle, p_hTransactionHandle);

        //Stop time
        StopCommandTime();

        //Status
        UpdateStatus();

        //Transaction event
        DoTransactionEvent(p_hTransactionHandle);

        //Update SubLayer Lock State
        m_oSubLayerRemainsLocked = m_pGateway->IsLocked();
    }

    return oResult;
}

BOOL CCommandRoot::UpdateStatus()
{
    BOOL oResult(TRUE);

    if (m_CommandStatus == CMD_IDLE)
    {
        SetStatus(FALSE, ERROR_FUNCTION_NOT_SUPPORTED);
    }

    return oResult;
}

CStdString CCommandRoot::GetCommandName()
{
    return m_CommandName;
}

ELayer CCommandRoot::GetLayer()
{
    return m_eLayer;
}

CStdString CCommandRoot::GetJournalRunTimeString()
{
    CStdString runTimeString(_T(""));

    // Actual run-time
    if(m_pJournalTime)
    {
        runTimeString = m_pJournalTime->GetJournalRunTimeString();
    }

    return runTimeString;
}

CStdString CCommandRoot::GetCommandTimeString()
{
    CStdString timeString(_T(""));

    if(m_pJournalTime)
    {
        timeString = m_pJournalTime->GetCommandTimeString();
    }

    return timeString;
}

void CCommandRoot::SetCommandTime(CStdString* p_pCommandTime)
{
    if(m_pJournalTime)
    {
        m_pJournalTime->SetCommandTime(p_pCommandTime);
    }
}

void CCommandRoot::SetJournalRunTime(CStdString* p_pJournalRunTime)
{
    if(m_pJournalTime)
    {
        m_pJournalTime->SetJournalRunTime(p_pJournalRunTime);
    }
}

void CCommandRoot::InitCommand(CStdString p_CommandName, ELayer p_eLayer, DWORD p_ulCommandId)
{
    m_CommandName = p_CommandName;
    m_eLayer = p_eLayer;
    m_dCommandId = p_ulCommandId;
}

BOOL CCommandRoot::InitCommandName(CStdString p_CommandName)
{
    m_CommandName = p_CommandName;

    return TRUE;
}

BOOL CCommandRoot::InitLayer(CStdString p_Layer)
{
    BOOL oResult(FALSE);

    m_eLayer = NO_LAYER;

    if(p_Layer.CompareNoCase(VIRTUAL_COMMAND_SET_LAYER_STRING) == 0)
    {
        m_eLayer = VIRTUAL_COMMAND_SET_LAYER;
        oResult = TRUE;
    }
    else if(p_Layer.CompareNoCase(COMMAND_SET_LAYER_STRING) == 0)
    {
        m_eLayer = DEVICE_COMMAND_SET_LAYER;
        oResult = TRUE;
    }
    else if(p_Layer.CompareNoCase(PROTOCOL_STACK_LAYER_STRING) == 0)
    {
        m_eLayer = PROTOCOL_STACK_LAYER;
        oResult = TRUE;
    }
    else if(p_Layer.CompareNoCase(INTERFACE_LAYER_STRING) == 0)
    {
        m_eLayer = INTERFACE_LAYER;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::InitCommandId(CStdString p_CommandId)
{
    CMmcDataConversion dataConversion;

    return dataConversion.DecDWordStr2DWord(p_CommandId, &m_dCommandId, FALSE);
}

void CCommandRoot::StartCommandTime()
{
    if(m_pJournalManager)
    {
        m_pJournalManager->StartCommandTime(m_pJournalTime);
    }
}

void CCommandRoot::StopCommandTime()
{
    if(m_pJournalManager)
    {
        m_pJournalManager->StopCommandTime(m_pJournalTime);
    }
}

BOOL CCommandRoot::LoadFromXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pCommandElement)
{
    BOOL oResult(FALSE);
    CStdString sCmdErrorCode;
    CStdString sCmdId;
    CStdString sCmdLayer;
    CStdString sCmdName;
    CStdString sCmdStatus;
    CStdString sCommandTime;
    CStdString sJournalRunTime;
    CXXMLFile::CElement* pParameterInfoElement;
    CXXMLFile::CElementPart* pElementPart;
    list<CXXMLFile::CElementPart*> childList;

    if(p_pFile && p_pCommandElement)
    {
        if(p_pCommandElement->m_Text.CompareNoCase(_T("Command")) == 0)
        {
            oResult = TRUE;

            //Command elements
            tMmcStringToStringMap::iterator it;

            if(oResult && (it = p_pCommandElement->AttributeToValue.find("Name")) == p_pCommandElement->AttributeToValue.end())
            {
                oResult = FALSE;
            }

            sCmdName = (*it).second;
            if(oResult && !InitCommandName(sCmdName))
            {
                oResult = FALSE;
            }

            if(oResult && (it = p_pCommandElement->AttributeToValue.find("Layer")) == p_pCommandElement->AttributeToValue.end())
            {
                oResult = FALSE;
            }

            sCmdLayer = (*it).second;
            if(oResult && !InitLayer(sCmdLayer))
            {
                oResult = FALSE;
            }

            if(oResult && (it = p_pCommandElement->AttributeToValue.find("CommandId")) == p_pCommandElement->AttributeToValue.end())
            {
                oResult = FALSE;
            }

            sCmdId = (*it).second;
            if(oResult && !InitCommandId(sCmdId))
            {
                oResult = FALSE;
            }

            if(oResult)
            {
                //Status
                if((it = p_pCommandElement->AttributeToValue.find("Status")) != p_pCommandElement->AttributeToValue.end())
                {
                    sCmdStatus = (*it).second;
                }

                if((it = p_pCommandElement->AttributeToValue.find("ErrorCode")) != p_pCommandElement->AttributeToValue.end())
                {
                    sCmdErrorCode = (*it).second;
                }

                InitCommandStatus(sCmdStatus, sCmdErrorCode);

                //Time
                if((it = p_pCommandElement->AttributeToValue.find("JournalRunTime")) != p_pCommandElement->AttributeToValue.end())
                {
                    sJournalRunTime = (*it).second;
                }

                if((it = p_pCommandElement->AttributeToValue.find("CommandDuration")) != p_pCommandElement->AttributeToValue.end())
                {
                    sCommandTime = (*it).second;
                }

                SetJournalRunTime(&sJournalRunTime);
                SetCommandTime(&sCommandTime);

                //ParameterInfo element
                for(list<CXXMLFile::CElementPart*>::iterator it = p_pCommandElement->begin(); it != p_pCommandElement->end(); it++)
                {
                    pElementPart = (*it);
                    if(p_pFile->IsElement(pElementPart))
                    {
                        pParameterInfoElement = (CXXMLFile::CElement*)pElementPart;
                        if(pParameterInfoElement->m_Text.CompareNoCase(_T("ParameterInfo")) == 0)
                        {
                            if(m_pParameterInfo && !m_pParameterInfo->LoadFromXMLFile(p_pFile, pParameterInfoElement))
                            {
                                oResult = FALSE;
                                break;
                            }
                        }

                        break;
                    }
                }
            }
        }
    }

    return oResult;
}

BOOL CCommandRoot::LoadFromXMLString(CStdString* p_pDataString)
{
    BOOL oResult(FALSE);
    CXXMLFile file;
    CXXMLFile::CElement* pRootElement;
    CXXMLFile::CElement* pCommandElement;
    CXXMLFile::CElementPart* pElementPart;

    // Read XML file
    if(file.ReadFromString(p_pDataString))
    {
        // Root
        pElementPart = file.Root();
        if(file.IsElement(pElementPart))
        {
            pRootElement = (CXXMLFile::CElement*)pElementPart;
            pCommandElement = FindCommandElement(&file, pRootElement);
            if(pCommandElement)
            {
                oResult = LoadFromXMLFile(&file, pCommandElement);
            }
        }
    }

    return oResult;
}

CXXMLFile::CElement* CCommandRoot::FindCommandElement(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement)
{
    if(p_pFile && p_pParentElement)
    {
        for(list<CXXMLFile::CElementPart*>::iterator it = p_pParentElement->begin(); it != p_pParentElement->end(); it++)
        {
            CXXMLFile::CElementPart* pElementPart = (*it);
            if(p_pFile->IsElement(pElementPart))
            {
                CXXMLFile::CElement* pElement = (CXXMLFile::CElement*)pElementPart;
                if(pElement->m_Text.CompareNoCase(_T("Command")) == 0)
                {
                    return pElement;
                }
                else
                {
                    pElement = FindCommandElement(p_pFile, pElement);
                    if(pElement)
                    {
                        return pElement;
                    }
                }
            }
        }
    }

    return 0;
}

CXXMLFile::CElement* CCommandRoot::DeleteCommandElement(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, CXXMLFile::CElement* p_pCommandElement)
{
    CXXMLFile::CElement* pElement;

    if(p_pFile && p_pParentElement && p_pCommandElement)
    {
        for(list<CXXMLFile::CElementPart*>::iterator it = p_pParentElement->begin(); it != p_pParentElement->end(); it++)
        {
            CXXMLFile::CElementPart* pElementPart = (*it);
            if(p_pFile->IsElement(pElementPart))
            {
                pElement = (CXXMLFile::CElement*)pElementPart;
                if(pElement == p_pCommandElement)
                {
                    if(p_pFile->DeleteElement(p_pParentElement, pElement))
                    {
                        return p_pParentElement;
                    }
                }
                else
                {
                    pElement = DeleteCommandElement(p_pFile, pElement, p_pCommandElement);
                    if(pElement)
                    {
                        return pElement;
                    }
                }
            }
        }
    }

    return 0;
}

CCommandRoot& CCommandRoot::operator=(CCommandRoot& p_rOther)
{
    if(this != &p_rOther)
    {
        ResetCommand();

        m_CommandName = p_rOther.m_CommandName;
        m_dCommandId = p_rOther.m_dCommandId;
        m_eLayer = p_rOther.m_eLayer;
        m_dTimeout = p_rOther.m_dTimeout;

        m_CommandStatus = p_rOther.m_CommandStatus;
        if(p_rOther.m_pErrorInfo)
        {
            m_pErrorInfo = p_rOther.m_pErrorInfo->Clone();
        }

        m_pJournalManager = p_rOther.m_pJournalManager;
        m_pGateway = p_rOther.m_pGateway;
        m_oTransactionEventEnabled = p_rOther.m_oTransactionEventEnabled;

        if(m_pJournalTime && p_rOther.m_pJournalTime)
        {
            *m_pJournalTime = *p_rOther.m_pJournalTime;
        }

        if(m_pParameterInfo && p_rOther.m_pParameterInfo)
        {
            *m_pParameterInfo = *p_rOther.m_pParameterInfo;
        }

        if(m_pLayerParameterStack && p_rOther.m_pLayerParameterStack)
        {
            *m_pLayerParameterStack = *p_rOther.m_pLayerParameterStack;
        }
    }

    return *this;
}

void CCommandRoot::ResetCommand()
{
    m_oTransactionEventEnabled = TRUE;
    m_pJournalManager = 0;
    m_pGateway = 0;
    m_CommandName.clear();
    m_eLayer = NO_LAYER;
    m_dCommandId = 0;
    m_CommandStatus = CMD_IDLE;
    m_dTimeout = UINT_MAX;
    m_oSubLayerRemainsLocked = FALSE;
    ResetErrorInfo();

    if(m_pParameterInfo)
    {
        m_pParameterInfo->Reset();
    }

    if(m_pJournalTime)
    {
        m_pJournalTime->Reset();
    }
}

#ifdef WINVER
void CCommandRoot::Serialize(CArchive &p_rArchive)
{
    const DWORD COMMAND_STRUCTURE_VERSION = 1;

    int lLayer, lCommandStatus;
    BOOL oErrorInfoExisting;
    BOOL oParameterInfoExisting;
    BOOL oJournalTimeExisting;
    DWORD ulCommandStructureVersion;

    if(p_rArchive.IsStoring())
    {
        p_rArchive << COMMAND_STRUCTURE_VERSION;

        p_rArchive << m_CommandName;
        p_rArchive << m_eLayer;
        p_rArchive << m_dCommandId;
        p_rArchive << m_dTimeout;
        p_rArchive << m_CommandStatus;
        p_rArchive << m_oSubLayerRemainsLocked;

        //ErrorInfo
        oErrorInfoExisting = BOOL(m_pErrorInfo);
        p_rArchive << oErrorInfoExisting;
        if(oErrorInfoExisting) m_pErrorInfo->Serialize(p_rArchive);

        //ParameterInfo
        oParameterInfoExisting = BOOL(m_pParameterInfo);
        p_rArchive << oParameterInfoExisting;
        if(oParameterInfoExisting) m_pParameterInfo->Serialize(p_rArchive);

        //JournalTime
        p_rArchive << m_oTransactionEventEnabled;
        oJournalTimeExisting = BOOL(m_pJournalTime);
        p_rArchive << oJournalTimeExisting;
        if(oJournalTimeExisting) m_pJournalTime->Serialize(p_rArchive);
    }
    else
    {
        ResetCommand();

        p_rArchive >> ulCommandStructureVersion;
        if(ulCommandStructureVersion == COMMAND_STRUCTURE_VERSION)
        {
            p_rArchive >> m_CommandName;
            p_rArchive >> lLayer; m_eLayer = (ELayer)lLayer;
            p_rArchive >> m_dCommandId;
            p_rArchive >> m_dTimeout;
            p_rArchive >> lCommandStatus; m_CommandStatus = (ECommandStatus)lCommandStatus;
            p_rArchive >> m_oSubLayerRemainsLocked;

            //ErrorInfo
            p_rArchive >> oErrorInfoExisting;
            if(oErrorInfoExisting)
            {
                m_pErrorInfo = new CErrorInfo();
                m_pErrorInfo->Serialize(p_rArchive);
            }

            //ParameterInfo
            p_rArchive >> oParameterInfoExisting;
            if(oParameterInfoExisting)
            {
                if(m_pParameterInfo == 0) m_pParameterInfo = new CParameterInfo();
                m_pParameterInfo->Serialize(p_rArchive);
            }

            //JournalTime
            p_rArchive >> m_oTransactionEventEnabled;
            p_rArchive >> oJournalTimeExisting;
            if(oJournalTimeExisting)
            {
                if(m_pJournalTime == 0) m_pJournalTime = new CJournalTime();
                m_pJournalTime->Serialize(p_rArchive);
            }
        }
    }
}
#endif

BOOL CCommandRoot::SetParameterData(int p_lParameterIndex, void* p_pData, size_t p_ulLength)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->SetParameterData(p_lParameterIndex, p_pData, p_ulLength);
    }

    return oResult;
}

BOOL CCommandRoot::SetReturnParameterData(int p_lParameterIndex, CStdString p_Data)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->SetReturnParameterData(p_lParameterIndex, (void*)p_Data.c_str(), p_Data.size());
    }

    return oResult;
}

BOOL CCommandRoot::SetReturnParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->SetReturnParameterData(p_lParameterIndex, p_pData, p_ulLength);
    }

    return oResult;
}

BOOL CCommandRoot::GetReturnParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->GetReturnParameterData(p_lParameterIndex, p_pData, p_ulLength);
    }

    return oResult;
}

BOOL CCommandRoot::GetParameterData(int p_lParameterIndex, void* p_pData, DWORD p_ulLength)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->GetParameterData(p_lParameterIndex, p_pData, p_ulLength);
    }

    return oResult;
}

CXXMLFile::CElement* CCommandRoot::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, BOOL p_oCheckVisibility, BOOL p_oCheckGatewaySupport)
{
    BOOL oSupported(TRUE);
    CXXMLFile::CElement* pResult(0);

    if(p_oCheckGatewaySupport)
    {
        oSupported = IsSupportedByGateway();
    }

    if(oSupported)
    {
        pResult = StoreToXMLFile(p_pFile, p_pParentElement, p_oCheckVisibility);
    }

    return pResult;
}

CXXMLFile::CElement* CCommandRoot::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElement* p_pParentElement, BOOL p_oCheckVisibility)
{
    CXXMLFile::CElement* pElement = 0;
    CStdString runTime;
    CStdString commandTime;

    if(p_pFile && p_pParentElement)
    {
        // Time
        runTime = GetJournalRunTimeString();
        commandTime = GetCommandTimeString();

        // Command elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "Command");

        pElement->SetAt("Name", m_CommandName);
        pElement->SetAt("Layer", GetLayerStr());
        pElement->SetAt("CommandId", GetCommandIdStr());

        pElement->SetAt("Status", GetCommandStatusStr(m_CommandStatus));
        pElement->SetAt("ErrorCode", GetErrorCodeStr());

        pElement->SetAt("JournalRunTime", runTime);
        pElement->SetAt("CommandDuration", commandTime);

        // Parameter elements
        if(m_pParameterInfo && !m_pParameterInfo->StoreToXMLFile(p_pFile, pElement, p_oCheckVisibility))
        {
            pElement = 0;
        }
    }

    return pElement;
}

BOOL CCommandRoot::StoreToXMLString(CStdString* p_pDataString, BOOL p_oCheckVisibility)
{
    BOOL oResult(FALSE);
    CXXMLFile file;
    CXXMLFile::CElementPart* pElementPart;
    CXXMLFile::CElement* pParentElement;

    if(p_pDataString)
    {
        // Root
        file.RemoveAll();
        pElementPart = file.Root();
        if(file.IsElement(pElementPart))
        {
            pParentElement = (CXXMLFile::CElement*)pElementPart;

            // Store
            if(StoreToXMLFile(&file, pParentElement, p_oCheckVisibility))
            {
                // Write XML string
                p_pDataString->Empty();
                file.WriteToString(p_pDataString);

                oResult = TRUE;
            }
        }
    }

    return oResult;
}

void CCommandRoot::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;
}

BOOL CCommandRoot::InitGateway(CGateway* p_pGateway)
{
    m_pGateway = p_pGateway;

    return TRUE;
}

void CCommandRoot::ResetGateway()
{
    m_pGateway = 0;
}

void CCommandRoot::ResetJournalManager()
{
    m_pJournalManager = 0;
}

void CCommandRoot::SetStatus(BOOL p_oResult, DWORD p_ulErrorCode)
{
    CErrorInfo errorInfo;

    errorInfo.Init(p_ulErrorCode);
    SetStatus(p_oResult, &errorInfo);
}

void CCommandRoot::SetStatus(BOOL p_oResult, CErrorInfo* p_pErrorInfo)
{
    if(p_oResult)
    {
        m_CommandStatus = CMD_SUCCESSFUL;
    }
    else
    {
        m_CommandStatus = CMD_FAILED;
    }

    ResetErrorInfo();

    if(p_pErrorInfo)
    {
        m_pErrorInfo = p_pErrorInfo->Clone();

        if(!m_pErrorInfo->IsCommandNameSet())
        {
            m_pErrorInfo->SetCommandName(m_CommandName);
        }
    }
}

void CCommandRoot::ResetStatus()
{
    m_CommandStatus = CMD_IDLE;

    ResetErrorInfo();
}

CStdString CCommandRoot::GetLayerStr()
{
    CStdString layer(_T(""));

    switch(m_eLayer)
    {
        case VIRTUAL_COMMAND_SET_LAYER: layer = VIRTUAL_COMMAND_SET_LAYER_STRING; break;
        case DEVICE_COMMAND_SET_LAYER: layer = COMMAND_SET_LAYER_STRING; break;
        case PROTOCOL_STACK_LAYER: layer = PROTOCOL_STACK_LAYER_STRING; break;
        case INTERFACE_LAYER: layer = INTERFACE_LAYER_STRING; break;
        case NO_LAYER: layer = NO_LAYER_STRING; break;
    }

    return layer;
}

CStdString CCommandRoot::GetCommandIdStr()
{
    CMmcDataConversion dataConversion;
    CStdString commandId(_T(""));

    if(!dataConversion.DWord2DecDWordStr(m_dCommandId, &commandId))
    {
        commandId = _T("");
    }

    return commandId;
}

CStdString CCommandRoot::GetErrorCodeStr()
{
    DWORD ulErrorCode = 0;
    CMmcDataConversion dataConversion;
    CStdString errorCode = _T("");

    if(m_pErrorInfo)
    {
        ulErrorCode = m_pErrorInfo->GetErrorCode();
    }

    if(!dataConversion.DWord2DecDWordStr(ulErrorCode, &errorCode))
    {
        errorCode = _T("");
    }

    return errorCode;
}

CStdString CCommandRoot::GetCommandStatusStr(ECommandStatus p_eCommandStatus)
{
    switch(p_eCommandStatus)
    {
        case CMD_IDLE:          return "Idle";
        case CMD_SUCCESSFUL:    return "Successful";
        case CMD_FAILED:        return "Failed";
        default: return "";
    }
}

BOOL CCommandRoot::InitCommandStatus(CStdString p_CmdStatus, CStdString p_CmdErrorCode)
{
    BOOL oResult(FALSE);
    CMmcDataConversion dataConversion;
    DWORD ulErrorCode;

    if(dataConversion.DecDWordStr2DWord(p_CmdErrorCode, &ulErrorCode, FALSE))
    {
        // Error info
        ResetErrorInfo();
        m_pErrorInfo = new CErrorInfo();
        m_pErrorInfo->Init(ulErrorCode);

        // Command status
        if(p_CmdStatus.Compare(_T("Idle")) == 0)
        {
            m_CommandStatus = CMD_IDLE;
            oResult = TRUE;
        }
        else if(p_CmdStatus.Compare(_T("Successful")) == 0)
        {
            m_CommandStatus = CMD_SUCCESSFUL;
            oResult = TRUE;
        }
        else if(p_CmdStatus.Compare(_T("Failed")) == 0)
        {
            m_CommandStatus = CMD_FAILED;
            oResult = TRUE;
        }
        else
        {
            m_CommandStatus = CMD_IDLE;
        }
    }

    return oResult;
}

DWORD CCommandRoot::GetCommandId()
{
    return m_dCommandId;
}

BOOL CCommandRoot::GetErrorInfo(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pErrorInfo)
    {
        if(m_pErrorInfo)
        {
            *p_pErrorInfo = *m_pErrorInfo;
        }
        else
        {
            p_pErrorInfo->Reset();
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::GetErrorCode(DWORD* p_pulErrorCode)
{
    BOOL oResult(FALSE);

    if(p_pulErrorCode && m_pErrorInfo)
    {
        *p_pulErrorCode = m_pErrorInfo->GetErrorCode();
        oResult = TRUE;
    }

    return oResult;
}

DWORD CCommandRoot::GetParameterLength(int p_lParameterIndex)
{
    DWORD ulReturn(0);

    if(m_pParameterInfo)
    {
        ulReturn = m_pParameterInfo->GetParameterLength(p_lParameterIndex);
    }

    return ulReturn;
}

DWORD CCommandRoot::GetReturnParameterLength(int p_lParameterIndex)
{
    DWORD ulReturn(0);

    if(m_pParameterInfo)
    {
        ulReturn = m_pParameterInfo->GetReturnParameterLength(p_lParameterIndex);
    }

    return ulReturn;
}

int CCommandRoot::GetNbOfParameter()
{
    int lReturn(0);

    if(m_pParameterInfo)
    {
        lReturn = m_pParameterInfo->GetNbOfParameter();
    }

    return lReturn;
}

int CCommandRoot::GetNbOfReturnParameter()
{
    int lReturn(0);

    if(m_pParameterInfo)
    {
        lReturn = m_pParameterInfo->GetNbOfReturnParameter();
    }

    return lReturn;
}

BOOL CCommandRoot::FindParameterIndex(CStdString p_Name, int& p_rIndex)
{
    BOOL oResult = FALSE;

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->FindParameterIndex(p_Name, p_rIndex);
    }

    return oResult;
}

BOOL CCommandRoot::FindReturnParameterIndex(CStdString p_Name, int& p_rIndex)
{
    BOOL oResult = FALSE;

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->FindReturnParameterIndex(p_Name, p_rIndex);
    }

    return oResult;
}

CStdString CCommandRoot::GetParameterName(int p_lParameterIndex)
{
    CStdString name(_T(""));

    if(m_pParameterInfo)
    {
        name = m_pParameterInfo->GetParameterName(p_lParameterIndex);
    }

    return name;
}

CStdString CCommandRoot::GetReturnParameterName(int p_lParameterIndex)
{
    CStdString name(_T(""));

    if(m_pParameterInfo)
    {
        name = m_pParameterInfo->GetReturnParameterName(p_lParameterIndex);
    }

    return name;
}

BOOL CCommandRoot::IsSuccessful()
{
    return (m_CommandStatus == CMD_SUCCESSFUL);
}

BOOL CCommandRoot::IsIdle()
{
    return (m_CommandStatus == CMD_IDLE);
}

BOOL CCommandRoot::GetParameterInfo(int p_lParameterIndex, CStdString* p_pName, CStdString* p_pType, CStdString* p_pData, EObjectValueStringFormat p_Format)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo->IsParameterExisting(p_lParameterIndex))
    {
        if(p_pName)
        {
            *p_pName = m_pParameterInfo->GetParameterName(p_lParameterIndex);
        }

        if(p_pType)
        {
            *p_pType = m_pParameterInfo->GetParameterTypeStr(p_lParameterIndex);
        }

        if(p_pData)
        {
            *p_pData = m_pParameterInfo->GetParameterDataStr(p_lParameterIndex, p_Format);
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::GetReturnParameterInfo(int p_lParameterIndex, CStdString* p_pName, CStdString* p_pType, CStdString* p_pData, EObjectValueStringFormat p_Format)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo->IsReturnParameterExisting(p_lParameterIndex))
    {
        if(p_pName)
        {
            *p_pName = m_pParameterInfo->GetReturnParameterName(p_lParameterIndex);
        }

        if(p_pType)
        {
            *p_pType = m_pParameterInfo->GetReturnParameterTypeStr(p_lParameterIndex);
        }

        if(p_pData)
        {
            *p_pData = m_pParameterInfo->GetReturnParameterDataStr(p_lParameterIndex, p_Format);
        }

        oResult = TRUE;
    }

    return oResult;
}

EObjectDataType CCommandRoot::GetParameterType(int p_lParameterIndex)
{
    EObjectDataType objectDataType(ODT_UNKNOWN);

    if(m_pParameterInfo)
    {
        objectDataType = m_pParameterInfo->GetParameterType(p_lParameterIndex);
    }

    return objectDataType;
}

EObjectDataType CCommandRoot::GetReturnParameterType(int p_lParameterIndex)
{
    EObjectDataType objectDataType(ODT_UNKNOWN);

    if(m_pParameterInfo)
    {
        objectDataType = m_pParameterInfo->GetReturnParameterType(p_lParameterIndex);
    }

    return objectDataType;
}

BOOL CCommandRoot::IsParameterEditable(int p_lParameterIndex)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsParameterEditable(p_lParameterIndex);
    }

    return oResult;
}

BOOL CCommandRoot::IsParameterVisible(int p_lParameterIndex)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsParameterVisible(p_lParameterIndex);
    }

    return oResult;
}

BOOL CCommandRoot::IsReturnParameterVisible(int p_lParameterIndex)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsReturnParameterVisible(p_lParameterIndex);
    }

    return oResult;
}

BOOL CCommandRoot::SetParameterDataString(int p_lParameterIndex, CStdString p_Value, BOOL p_oShowMsg)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->SetParameterDataString(p_lParameterIndex, p_Value, p_oShowMsg);
    }

    return oResult;
}

BOOL CCommandRoot::SetReturnParameterDataString(int p_lParameterIndex, CStdString p_Value, BOOL p_oShowMsg)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->SetReturnParameterDataString(p_lParameterIndex, p_Value, p_oShowMsg);
    }

    return oResult;
}

BOOL CCommandRoot::GetParameterDataString(int p_lParameterIndex, CStdString* p_pValue, EObjectValueStringFormat p_Format)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->GetParameterDataString(p_lParameterIndex, p_pValue, p_Format);
    }

    return oResult;
}

BOOL CCommandRoot::GetReturnParameterDataString(int p_lParameterIndex, CStdString* p_pValue, EObjectValueStringFormat p_Format)
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->GetReturnParameterDataString(p_lParameterIndex, p_pValue, p_Format);
    }

    return oResult;
}

BOOL CCommandRoot::IsEditableParameterExisting()
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsEditableParameterExisting();
    }

    return oResult;
}

BOOL CCommandRoot::IsVisibleParameterExisting()
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsVisibleParameterExisting();
    }

    return oResult;
}

BOOL CCommandRoot::IsVisibleReturnParameterExisting()
{
    BOOL oResult(FALSE);

    if(m_pParameterInfo)
    {
        oResult = m_pParameterInfo->IsVisibleReturnParameterExisting();
    }

    return oResult;
}

DWORD CCommandRoot::GetParameterArraySize(int p_lParameterIndex)
{
    DWORD ulResult(0);

    if(m_pParameterInfo)
    {
        ulResult = m_pParameterInfo->GetParameterArraySize(p_lParameterIndex);
    }

    return ulResult;
}

DWORD CCommandRoot::GetReturnParameterArraySize(int p_lParameterIndex)
{
    DWORD ulResult(0);

    if(m_pParameterInfo)
    {
        ulResult = m_pParameterInfo->GetReturnParameterArraySize(p_lParameterIndex);
    }

    return ulResult;
}

BOOL CCommandRoot::PushLayerParameterSet(CLayerParameterSet& p_rParameterSet)
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        oResult = m_pLayerParameterStack->PushLayer(p_rParameterSet);
    }

    return oResult;
}

BOOL CCommandRoot::SetLayerParameterStack(CLayerParameterStack& p_rParameterStack)
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        *m_pLayerParameterStack = p_rParameterStack;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::GetLayerParameterStack(CLayerParameterStack& p_rParameterStack)
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        p_rParameterStack = *m_pLayerParameterStack;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::GetLayerParameter(ELayer p_eLayer, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        //Init
        oResult = TRUE;

        if(!m_pLayerParameterStack->GetLayerParameter(p_eLayer, p_Name, p_pValue, p_ulSize))
        {
            if(p_pErrorInfo)
            {
                p_pErrorInfo->Init(ERROR_BAD_PARAMETER, 0);
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CCommandRoot::GetLayerParameter(ELayer p_eLayer, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        oResult = TRUE;

        if(!m_pLayerParameterStack->GetLayerParameter(p_eLayer, p_Name, p_rValue))
        {
            if(p_pErrorInfo)
            {
                p_pErrorInfo->Init(ERROR_BAD_PARAMETER, 0);
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CCommandRoot::ResetLayerParameterStack()
{
    BOOL oResult(FALSE);
    CLayerParameterStack addressParameter;

    if(m_pLayerParameterStack)
    {
        oResult = m_pLayerParameterStack->Reset();
    }

    return oResult;
}

BOOL CCommandRoot::IsSubLayerLocked()
{
    return m_oSubLayerRemainsLocked;
}

BOOL CCommandRoot::IsSupportedByGateway()
{
    BOOL oResult(FALSE);

    if(m_pGateway != 0)
    {
        ResetStatus();
        ProcessWithoutHandles();

        oResult = !IsIdle();
        ResetStatus();
    }

    return oResult;
}

BOOL CCommandRoot::ProcessWithoutHandles()
{
    BOOL oResult(FALSE);

    if(m_pGateway != 0)
    {
        m_pGateway->ProcessCommand(this, 0, 0, 0);
    }

    return oResult;
}

BOOL CCommandRoot::ActivateLogMessage()
{
    BOOL oResult = FALSE;

    if(!IsLogMessageActive())
    {
        m_oLogMessageActive = TRUE;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::DeactivateLogMessage()
{
    BOOL oResult = FALSE;

    if(IsLogMessageActive())
    {
        m_oLogMessageActive = FALSE;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommandRoot::IsLogMessageActive()
{
    return m_oLogMessageActive;
}

CStdString CCommandRoot::GetLogMessage(CStdString p_Identifier, BOOL p_oReturnParameter/* = FALSE*/)
{
    CStdString outputMsg(_T(""));
    CStdString msg(_T(""));
    CStdString paramMsg;
    CStdString returnParamMsg;    
    CStdString status(_T(""));
    CStdString separator(_T(";"));

    if(m_CommandStatus == CMD_IDLE)
    {
        status = _T("Idle");
    }
    else if(m_CommandStatus == CMD_SUCCESSFUL)
    {
        status = _T("OK");
    }
    else if(m_CommandStatus == CMD_FAILED)
    {
        status = _T("Err");
    }

    msg = _T("Cmd");
    msg += separator;
    msg += p_oReturnParameter ? _T("OUT") : _T("IN");
    msg += separator;
    msg += status;
    msg += separator;
    msg += p_Identifier;
    msg += separator;
    msg += m_CommandName;

    outputMsg = msg;

    if(m_pParameterInfo)
    {    
        if(GetNbOfParameter()>0)
        {
            outputMsg += separator;

            for(int i = 0; i < GetNbOfParameter(); i++)
            {
                CStdString name = m_pParameterInfo->GetParameterName(i);
                CStdString data = m_pParameterInfo->GetParameterDataStr(i, OVF_HEX);

                paramMsg = name + separator + data;

                outputMsg += paramMsg;
                
                if(i < GetNbOfParameter() - 1)
                {
                    outputMsg += separator;
                }
            }
        }
         
        if(p_oReturnParameter)
        {
            if(GetNbOfReturnParameter()>0)
            {
                outputMsg += separator;

                for(int i = 0; i < GetNbOfReturnParameter(); i++)
                {
                    CStdString name = m_pParameterInfo->GetReturnParameterName(i);
                    CStdString data = m_pParameterInfo->GetReturnParameterDataStr(i, OVF_HEX);
                                        
                    returnParamMsg = name + separator + data;

                    outputMsg += returnParamMsg;

                    if(i < GetNbOfReturnParameter() - 1)
                    {
                        outputMsg += separator;
                    }
                }        
            }
        }
    }

    return outputMsg;
}
