
#include "stdafx.h"

#include <malloc.h>
#include <memory.h>
#include <Classes/MmcDataConversion.h>
#include <Classes/TypeDefinitions.h>
#include <MmcTypeDefinition.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/ParameterInfo/Parameter/ParameterBase.h>

CParameterBase::CParameterBase()
{
    m_pDataBuffer = NULL;
    m_dDataBufferLength = 0;
    Reset();
}

CParameterBase::~CParameterBase()
{
    DeleteDataBuffer();
}

CParameterBase& CParameterBase::operator=(CParameterBase& p_rOther)
{
    if(this != &p_rOther)
    {
        Reset();
        m_strName = p_rOther.m_strName;
        m_iIndex = p_rOther.m_iIndex;
        m_oEditable = p_rOther.m_oEditable;
        m_oVisible = p_rOther.m_oVisible;
        m_eType = p_rOther.m_eType;
        m_dArraySize = p_rOther.m_dArraySize;

        DeleteDataBuffer();
        if(p_rOther.m_dDataBufferLength > 0)
        {
            m_dDataBufferLength = p_rOther.m_dDataBufferLength;
            m_pDataBuffer = malloc(m_dDataBufferLength);
            memcpy(m_pDataBuffer, p_rOther.m_pDataBuffer, m_dDataBufferLength);
        }
    }

    return *this;
}

#ifdef WINVER
void CParameterBase::Serialize(CArchive& p_rArchive)
{
    BYTE* p;
    int type;

    if(p_rArchive.IsStoring())
    {
        p_rArchive << m_strName;
        p_rArchive << m_iIndex;
        p_rArchive << m_oEditable;
        p_rArchive << m_oVisible;
        p_rArchive << m_eType;
        p_rArchive << m_dArraySize;

        p_rArchive << m_dDataBufferLength;
        if(m_dDataBufferLength > 0)
        {
            p = (BYTE*)m_pDataBuffer;
            for(DWORD i = 0; i < m_dDataBufferLength; i++)
            {
                p_rArchive << *p;
                p++;
            }
        }
    }
    else
    {
        Reset();
        p_rArchive >> m_strName;
        p_rArchive >> m_iIndex;
        p_rArchive >> m_oEditable;
        p_rArchive >> m_oVisible;
        p_rArchive >> type; m_eType = (EObjectDataType)type;
        p_rArchive >> m_dArraySize;

        DeleteDataBuffer();
        p_rArchive >> m_dDataBufferLength;

        if(m_dDataBufferLength > 0)
        {
            m_pDataBuffer = malloc(m_dDataBufferLength);
            p = (BYTE*)m_pDataBuffer;
            for(DWORD i = 0; i < m_dDataBufferLength; i++)
            {
                p_rArchive >> *p;
                p++;
            }
        }
    }
}
#endif

void CParameterBase::Reset()
{
    m_strName = _T("");
    m_iIndex = 0;
    m_oEditable = TRUE;
    m_oVisible = TRUE;
    m_eType = ODT_UNKNOWN;
    m_dArraySize = 1;
    DeleteDataBuffer();
}

void CParameterBase::DeleteDataBuffer()
{
    if(m_pDataBuffer)
    {
        free(m_pDataBuffer);
        m_pDataBuffer = NULL;
        m_dDataBufferLength = 0;
    }
}

void CParameterBase::InitDataBuffer()
{
    DWORD ulLength;

    DeleteDataBuffer();

    ulLength = GetParameterTypeLength(m_eType);
    m_dDataBufferLength = ulLength * m_dArraySize;

    if(m_dDataBufferLength > 0)
    {
        m_pDataBuffer = malloc(m_dDataBufferLength);
    }

    ResetData();
}

void CParameterBase::Init(int p_lIndex, CStdString p_name, EObjectDataType p_eType, DWORD p_ulArraySize, BOOL p_oVisible, BOOL p_oEditable)
{
    m_iIndex = p_lIndex;
    m_strName = p_name;
    m_eType = p_eType;
    m_dArraySize = p_ulArraySize;
    m_oEditable = p_oEditable;
    m_oVisible = p_oVisible;
    InitDataBuffer();
}

BOOL CParameterBase::IsVisible()
{
    return m_oVisible;
}

BOOL CParameterBase::IsEditable()
{
    return m_oEditable;
}

BOOL CParameterBase::InitIndex(CStdString p_indexStr)
{
    BOOL oResult(FALSE);
    CMmcDataConversion dataConversion;

    oResult = dataConversion.DecIntStr2Int(p_indexStr, &m_iIndex, FALSE);

    return oResult;
}

BOOL CParameterBase::InitName(CStdString p_name)
{
    m_strName = p_name;

    return TRUE;
}

BOOL CParameterBase::InitType(CStdString p_typeStr)
{
    BOOL oResult(TRUE);
    CMmcDataConversion dataConversion;
    CStdString strParameterType, strArraySize;
    int lIndex1, lIndex2;

    //Reset data
    DeleteDataBuffer();

    //Array size
    lIndex1 = p_typeStr.Find('[');
    lIndex2 = p_typeStr.Find(']');
    if((lIndex1 == -1) || (lIndex2 == -1))
    {
        m_dArraySize = 1;
        strParameterType = p_typeStr;
    }
    else
    {
        strParameterType = p_typeStr.Left(lIndex1);
        strArraySize = p_typeStr.Mid(lIndex1 + 1, lIndex2 - lIndex1 - 1);
        if(!dataConversion.DecDWordStr2DWord(strArraySize, &m_dArraySize, FALSE))
        {
            oResult = FALSE;
        }
    }

    if(oResult)
    {
        //Convert TypeStr
        if(strParameterType.CompareNoCase(_T("UInt8")) == 0)
        {
            m_eType = ODT_UINT8;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Int8")) == 0)
        {
            m_eType = ODT_INT8;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("UInt16")) == 0)
        {
            m_eType = ODT_UINT16;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Int16")) == 0)
        {
            m_eType = ODT_INT16;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("UInt32")) == 0)
        {
            m_eType = ODT_UINT32;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Int32")) == 0)
        {
            m_eType = ODT_INT32;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("UInt64")) == 0)
        {
            m_eType = ODT_UINT64;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Int64")) == 0)
        {
            m_eType = ODT_INT64;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Float")) == 0)
        {
            m_eType = ODT_FLOAT;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Void")) == 0)
        {
            m_eType = ODT_MEMORY_BLOCK;
            InitDataBuffer();
        }
        else if(strParameterType.CompareNoCase(_T("Bool")) == 0)
        {
            m_eType = ODT_BOOLEAN;
            InitDataBuffer();
        }
        else
        {
            m_eType = ODT_UNKNOWN;
            oResult = FALSE;
        }
    }

    return oResult;
}

int CParameterBase::GetIndex()
{
    return m_iIndex;
}

CStdString CParameterBase::GetIndexStr()
{
    CMmcDataConversion dataConversion;
    CStdString indexStr(_T(""));

    dataConversion.Int2DecIntStr(m_iIndex, &indexStr);

    return indexStr;
}

CStdString CParameterBase::GetTypeStr()
{
    CStdString typeStr(_T(""));
    BOOL oAddArraySize(FALSE);

    switch(m_eType)
    {
        case ODT_UNKNOWN:       typeStr = _T("Unknown"); oAddArraySize = FALSE; break;
        case ODT_UINT8:         typeStr = _T("UInt8"); oAddArraySize = TRUE; break;
        case ODT_INT8:          typeStr = _T("Int8"); oAddArraySize = TRUE; break;
        case ODT_UINT16:        typeStr = _T("UInt16"); oAddArraySize = TRUE; break;
        case ODT_INT16:         typeStr = _T("Int16"); oAddArraySize = TRUE; break;
        case ODT_UINT32:        typeStr = _T("UInt32"); oAddArraySize = TRUE; break;
        case ODT_INT32:         typeStr = _T("Int32"); oAddArraySize = TRUE; break;
        case ODT_UINT64:        typeStr = _T("UInt64"); oAddArraySize = TRUE; break;
        case ODT_INT64:         typeStr = _T("Int64"); oAddArraySize = TRUE; break;
        case ODT_FLOAT:         typeStr = _T("Float"); oAddArraySize = TRUE; break;
        case ODT_MEMORY_BLOCK:  typeStr = _T("Void"); oAddArraySize = FALSE; break;
        case ODT_BOOLEAN:       typeStr = _T("Bool"); oAddArraySize = TRUE; break;
        default:                typeStr = _T("");
    }

    if(oAddArraySize && (m_dArraySize > 1))
    {
        TCHAR *pszArraySize = new TCHAR[32];
        _stprintf(pszArraySize, _T("[%i]"), (int)m_dArraySize);
        typeStr = typeStr + pszArraySize;
        delete []pszArraySize;
    }

    return typeStr;
}

EObjectDataType CParameterBase::GetType()
{
    return m_eType;
}

DWORD CParameterBase::GetArraySize()
{
    return m_dArraySize;
}

BOOL CParameterBase::SetValue(CStdString p_valueStr, BOOL p_oShowMsg)
{
    BOOL oResult(FALSE);
    CMmcDataConversion dataConversion;
    BYTE ubValue;
    char bValue;
    WORD usValue;
    short sValue;
    DWORD ulValue;
    int lValue;
    __uint64 ullValue;
    __int64 llValue;
    float fValue;
    DWORD ulOffset(0);
    CStdString subValueStr;

    if(m_eType == ODT_MEMORY_BLOCK)
    {
        oResult = SetVoidValueStr(p_valueStr, p_oShowMsg) > 0;
    }
    else
    {
        for(DWORD i = 0; i < m_dArraySize; i++)
        {
            //Extract sub string
            subValueStr = ExtractSubValueStr(&p_valueStr);
            if(subValueStr.size() > 0)
            {
                //Detect data format
                if((subValueStr.size() >= (int)strlen("0x")) && (subValueStr.Left((int)strlen("0x")).CompareNoCase(_T("0x")) == 0))
                {
                    //Hex format
                    switch(m_eType)
                    {
                        case ODT_UNKNOWN:   break;
                        case ODT_UINT8:     oResult = dataConversion.HexByteStr2Byte(subValueStr, &ubValue, p_oShowMsg == TRUE); SetData(&ubValue, sizeof(ubValue), ulOffset); break;
                        case ODT_INT8:      oResult = dataConversion.HexCharStr2Char(subValueStr, &bValue, p_oShowMsg == TRUE); SetData(&bValue, sizeof(bValue), ulOffset); break;
                        case ODT_UINT16:    oResult = dataConversion.HexWordStr2Word(subValueStr, &usValue, p_oShowMsg == TRUE); SetData(&usValue, sizeof(usValue), ulOffset); break;
                        case ODT_INT16:     oResult = dataConversion.HexShortStr2Short(subValueStr, &sValue, p_oShowMsg == TRUE); SetData(&sValue, sizeof(sValue), ulOffset); break;
                        case ODT_UINT32:    oResult = dataConversion.HexDWordStr2DWord(subValueStr, &ulValue, p_oShowMsg == TRUE); SetData(&ulValue, sizeof(ulValue), ulOffset); break;
                        case ODT_INT32:     oResult = dataConversion.HexIntStr2Int(subValueStr, &lValue, p_oShowMsg == TRUE); SetData(&lValue, sizeof(lValue), ulOffset); break;
                        case ODT_UINT64:    oResult = dataConversion.HexUInt64Str2UInt64(subValueStr, &ullValue, p_oShowMsg == TRUE); SetData(&ullValue, sizeof(ullValue), ulOffset); break;
                        case ODT_INT64:     oResult = dataConversion.HexInt64Str2Int64(subValueStr, &llValue, p_oShowMsg == TRUE); SetData(&llValue, sizeof(llValue), ulOffset); break;
                        case ODT_FLOAT:     oResult = dataConversion.HexFloatStr2Float(subValueStr, &fValue, p_oShowMsg == TRUE); SetData(&fValue, sizeof(fValue), ulOffset); break;
                        case ODT_BOOLEAN:   oResult = (SetBooleanValueStr(subValueStr) == TRUE); break;
                        default:            break;
                    }
                }
                else
                {
                    //Dec format
                    switch(m_eType)
                    {
                        case ODT_UNKNOWN:   break;
                        case ODT_UINT8:     oResult = dataConversion.DecByteStr2Byte(subValueStr, &ubValue, p_oShowMsg == TRUE); SetData(&ubValue, sizeof(ubValue), ulOffset); break;
                        case ODT_INT8:      oResult = dataConversion.DecCharStr2Char(subValueStr, &bValue, p_oShowMsg == TRUE); SetData(&bValue, sizeof(bValue), ulOffset); break;
                        case ODT_UINT16:    oResult = dataConversion.DecWordStr2Word(subValueStr, &usValue, p_oShowMsg == TRUE); SetData(&usValue, sizeof(usValue), ulOffset); break;
                        case ODT_INT16:     oResult = dataConversion.DecShortStr2Short(subValueStr, &sValue, p_oShowMsg == TRUE); SetData(&sValue, sizeof(sValue), ulOffset); break;
                        case ODT_UINT32:    oResult = dataConversion.DecDWordStr2DWord(subValueStr, &ulValue, p_oShowMsg == TRUE); SetData(&ulValue, sizeof(ulValue), ulOffset); break;
                        case ODT_INT32:     oResult = dataConversion.DecIntStr2Int(subValueStr, &lValue, p_oShowMsg == TRUE); SetData(&lValue, sizeof(lValue), ulOffset); break;
                        case ODT_UINT64:    oResult = dataConversion.DecUInt64Str2UInt64(subValueStr, &ullValue, p_oShowMsg == TRUE); SetData(&ullValue, sizeof(ullValue), ulOffset); break;
                        case ODT_INT64:     oResult = dataConversion.DecInt64Str2Int64(subValueStr, &llValue, p_oShowMsg == TRUE); SetData(&llValue, sizeof(llValue), ulOffset); break;
                        case ODT_FLOAT:     oResult = dataConversion.DecFloatStr2Float(subValueStr, &fValue, p_oShowMsg == TRUE); SetData(&fValue, sizeof(fValue), ulOffset); break;
                        case ODT_BOOLEAN:   oResult = (SetBooleanValueStr(subValueStr) > 0); break;
                        default:            break;
                    }
                }
            }
            else
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

CStdString CParameterBase::ExtractSubValueStr(CStdString* p_pValueStr)
{
    CStdString subStr = _T("");
    int lIndex;

    if(p_pValueStr && p_pValueStr->size() > 0)
    {
        p_pValueStr->TrimRight();
        *p_pValueStr += ' ';
        lIndex = p_pValueStr->Find(' ', 0);
        if(lIndex != -1)
        {
            subStr = p_pValueStr->Left(lIndex);
            *p_pValueStr = p_pValueStr->Right((int)p_pValueStr->size() - lIndex);
            p_pValueStr->TrimLeft();
        }
    }

    return subStr;
}

CStdString CParameterBase::GetValueStr(EObjectValueStringFormat p_format)
{
    BYTE* pData;
    CStdString valueStr = _T("");
    CStdString subValueStr;
    CMmcDataConversion dataConversion;
    DWORD ulParameterTypeLength;

    if(m_pDataBuffer)
    {
        if(m_eType == ODT_MEMORY_BLOCK || p_format == OVF_VOID)
        {
            valueStr = GetVoidValueStr();
        }
        else
        {
            ulParameterTypeLength = GetParameterTypeLength(m_eType);
            pData = (BYTE*)m_pDataBuffer;
            for(DWORD i = 0; i < m_dArraySize; i++)
            {
                if(p_format == OVF_DEC)
                {
                    switch(m_eType)
                    {
                        case ODT_UNKNOWN:   break;
                        case ODT_UINT8:     dataConversion.UChar2DecUCharStr(*(BYTE*)pData, &subValueStr); break;
                        case ODT_INT8:      dataConversion.Char2DecCharStr(*(char*)pData, &subValueStr); break;
                        case ODT_UINT16:    dataConversion.Word2DecWordStr(*(WORD*)pData, &subValueStr); break;
                        case ODT_INT16:     dataConversion.Short2DecShortStr(*(short*)pData, &subValueStr); break;
                        case ODT_UINT32:    dataConversion.DWord2DecDWordStr(*(DWORD*)pData, &subValueStr); break;
                        case ODT_INT32:     dataConversion.Int2DecIntStr(*(int*)pData, &subValueStr); break;
                        case ODT_UINT64:    dataConversion.UInt64_2DecUInt64Str(*(__uint64*)pData, &subValueStr); break;
                        case ODT_INT64:     dataConversion.Int64_2DecInt64Str(*(__int64*)pData, &subValueStr); break;
                        case ODT_FLOAT:     dataConversion.Float2DecFloatStr(*(float*)pData, &subValueStr); break;
                        case ODT_BOOLEAN:   subValueStr = GetBooleanValueStr(pData);break;
                        default:            break;
                    }
                }
                else if(p_format == OVF_HEX)
                {
                    switch(m_eType)
                    {
                        case ODT_UNKNOWN:   break;
                        case ODT_UINT8:     dataConversion.UChar2HexUCharStr(*(BYTE*)pData, &subValueStr); break;
                        case ODT_INT8:      dataConversion.Char2HexCharStr(*(char*)pData, &subValueStr); break;
                        case ODT_UINT16:    dataConversion.Word2HexWordStr(*(WORD*)pData, &subValueStr); break;
                        case ODT_INT16:     dataConversion.Short2HexShortStr(*(short*)pData, &subValueStr); break;
                        case ODT_UINT32:    dataConversion.DWord2HexDWordStr(*(DWORD*)pData, &subValueStr); break;
                        case ODT_INT32:     dataConversion.Int2HexIntStr(*(int*)pData, &subValueStr); break;
                        case ODT_UINT64:    dataConversion.UInt64_2HexUInt64Str(*(__uint64*)pData, &subValueStr); break;
                        case ODT_INT64:     dataConversion.Int64_2HexInt64Str(*(__int64*)pData, &subValueStr); break;
                        case ODT_FLOAT:     dataConversion.Float2HexFloatStr(*(float*)pData, &subValueStr); break;
                        case ODT_BOOLEAN:   subValueStr = GetBooleanValueStr(pData);break;
                        default:            break;
                    }
                }
                valueStr = valueStr + subValueStr + _T(" ");
                pData = pData + ulParameterTypeLength;
            }
            valueStr.TrimRight();
        }
    }

    return valueStr;
}

CStdString CParameterBase::GetName()
{
    return m_strName;
}

BOOL CParameterBase::SetData(void* p_pData, DWORD p_ulLength)
{
    BOOL oResult(FALSE);
    DWORD ulDataBufferLength = 0;

    if((m_eType == ODT_MEMORY_BLOCK) || (m_eType == ODT_STRING))
    {
        DeleteDataBuffer();

        if(p_pData && (p_ulLength > 0))
        {
            if(m_eType == ODT_STRING)
            {
                m_dDataBufferLength = p_ulLength * sizeof(TCHAR);
                
                m_pDataBuffer = malloc(m_dDataBufferLength);

                memcpy(m_pDataBuffer, p_pData, p_ulLength * sizeof(TCHAR));
            }
            else
            {
                m_dDataBufferLength = p_ulLength;
                
                m_pDataBuffer = malloc(m_dDataBufferLength);

                memcpy(m_pDataBuffer, p_pData, p_ulLength);
            }
        }
        oResult = TRUE;
    }
    else
    {
        if(p_pData && (p_ulLength > 0))
        {
            ulDataBufferLength = m_dDataBufferLength;
            
            if(p_ulLength < ulDataBufferLength)
            {
                ulDataBufferLength = p_ulLength;
            }
            
            if(m_pDataBuffer && (ulDataBufferLength > 0))
            {
                memcpy(m_pDataBuffer, p_pData, ulDataBufferLength);
            }

            oResult = (m_dDataBufferLength == p_ulLength);
        }
    }

    return oResult;
}

BOOL CParameterBase::SetData(void* p_pData, DWORD p_ulLength, DWORD& p_rulOffset)
{
    BOOL oRetVal(FALSE);
    BYTE* pByte;
    DWORD ulMinLength(0);
    DWORD ulOffset(0);

    ulOffset = p_rulOffset;

    if(p_pData && (p_ulLength > 0))
    {
        if(m_eType != ODT_MEMORY_BLOCK)
        {
            if(ulOffset < m_dDataBufferLength)
            {
                ulMinLength = m_dDataBufferLength - ulOffset;
                if(p_ulLength < ulMinLength) ulMinLength = p_ulLength;

                if(m_pDataBuffer && (ulMinLength > 0))
                {
                    pByte = (BYTE*)m_pDataBuffer + ulOffset;
                    memcpy(pByte, p_pData, ulMinLength);
                }
                p_rulOffset += p_ulLength;
                oRetVal = TRUE;
            }
        }
    }
    return oRetVal;
}

BOOL CParameterBase::GetData(void* p_pData, DWORD p_ulLength)
{
    BOOL oResult(FALSE);
    DWORD ulDataBufferLength(0);
    DWORD ulOverhead = 0;
    DWORD ulRedundantDataLength = 0;

    if(p_pData && (p_ulLength > 0))
    {
        ulDataBufferLength = m_dDataBufferLength;

        if(p_ulLength < ulDataBufferLength)
        {
            ulDataBufferLength = p_ulLength;
        }
        else if (p_ulLength > ulDataBufferLength)
        {
            ulRedundantDataLength = p_ulLength - ulDataBufferLength;

            memset(((BYTE*)p_pData + ulDataBufferLength), 0, ulRedundantDataLength);
        }

        if(m_pDataBuffer && (ulDataBufferLength > 0))
        {
            memcpy(p_pData, m_pDataBuffer, ulDataBufferLength);
        }

        oResult = (p_ulLength <= m_dDataBufferLength);
    }

    return oResult;
}

DWORD CParameterBase::GetLength()
{
    return m_dDataBufferLength;
}

void CParameterBase::ResetData()
{
    if(m_pDataBuffer) memset(m_pDataBuffer, 0, m_dDataBufferLength);
}

DWORD CParameterBase::GetParameterTypeLength(EObjectDataType p_type)
{
    DWORD ulTypeLength(0);

    switch( p_type)
    {
        case ODT_UNKNOWN:       ulTypeLength = 0; break;
        case ODT_UINT8:
        case ODT_INT8:          ulTypeLength = 1; break;
        case ODT_UINT16:
        case ODT_INT16:         ulTypeLength = 2; break;
        case ODT_BOOLEAN:
        case ODT_UINT32:
        case ODT_INT32:         ulTypeLength = 4; break;
        case ODT_UINT64:
        case ODT_INT64:         ulTypeLength = 8; break;
        case ODT_FLOAT:         ulTypeLength = 4; break;
        case ODT_MEMORY_BLOCK:  ulTypeLength = 0; break;
        default:                ulTypeLength = 0;
    }

    return ulTypeLength;
}

CStdString CParameterBase::GetVoidValueStr()
{
    BYTE* p;
    CStdString str = _T("");

    if(m_pDataBuffer && (m_dDataBufferLength > 0))
    {
        p = (BYTE*)m_pDataBuffer;
        for(DWORD i = 0;i < m_dDataBufferLength; i++)
        {
            if(p)
            {
                TCHAR* pszAddStr = new TCHAR[32];
                _stprintf(pszAddStr, _T("0x%.2X"), *p++);
                str += pszAddStr;
                delete []pszAddStr;

                if(i < m_dDataBufferLength-1)
                {
                    str += _T(" ");
                }
            }
        }
    }

    return str;
}

CStdString CParameterBase::GetBooleanValueStr(void* p_pData)
{
    CStdString result(_T(""));

    if(p_pData)
    {
        if(*((BOOL*)p_pData))
        {
            result = _T("True");
        }
        else
        {
            result = _T("False");
        }
    }

    return result;
}

BOOL CParameterBase::SetVoidValueStr(CStdString p_valueStr, BOOL p_oShowMsg)
{
    BOOL oResult(TRUE);
    BYTE ubValue;
    BYTE* p(0);
    BYTE* pDataBuffer(0);
    CMmcDataConversion dataConversion;
    CStdByteArray byteArray;
    CStdString byteStr;
    int lIndex;

    if(p_valueStr.size() > 0)
    {
        //Convert string
        p_valueStr.TrimRight();
        p_valueStr += ' ';
        while(!p_valueStr.IsEmpty())
        {
            lIndex = p_valueStr.Find(' ', 0);
            if(lIndex != -1)
            {
                byteStr = p_valueStr.Left(lIndex);
                p_valueStr = p_valueStr.Right(p_valueStr.GetLength() - lIndex);
                p_valueStr.TrimLeft();
                if(dataConversion.HexByteStr2Byte(byteStr, &ubValue, p_oShowMsg > 0 ? true: false))
                {
                    byteArray.push_back(ubValue);
                }
            }
            else
            {
                break;
            }
        }
        //Write data
        pDataBuffer = (BYTE*)malloc(byteArray.size());
        if(pDataBuffer == NULL)
        {
            oResult = FALSE;
        }
        else
        {
            p = pDataBuffer;
            for(std::size_t i = 0; i < byteArray.size(); i++)
            {
                *p++ = byteArray.at(i);
            }

            SetData(pDataBuffer, (DWORD)byteArray.size());
            free(pDataBuffer);
        }
    }
    else
    {
        //Reset data buffer
        SetData(NULL, 0);
    }

    return oResult;
}

BOOL CParameterBase::SetBooleanValueStr(CStdString p_valueStr)
{
    BOOL oValue = FALSE;

    if(p_valueStr.CompareNoCase(_T("True")) == 0)
    {
        oValue = TRUE;
    }
    else
    {
        oValue = FALSE;
    }

    SetData(&oValue, sizeof(oValue));

    return TRUE;
}

CStdString CParameterBase::GetEditableStr()
{
    CStdString result(_T(""));

    if(m_oEditable)
    {
        result = _T("True");
    }
    else
    {
        result = _T("False");
    }

    return result;
}

CStdString CParameterBase::GetVisibleStr()
{
    CStdString result;

    if(m_oVisible)
    {
        result = _T("True");
    }
    else
    {
        result = _T("False");
    }

    return result;
}

BOOL CParameterBase::InitEditable(CStdString p_editableStr)
{
    BOOL oResult(FALSE);

    if(p_editableStr.CompareNoCase(_T("True")) == 0)
    {
        m_oEditable = TRUE;
        oResult = TRUE;
    }
    else if(p_editableStr.CompareNoCase(_T("False")) == 0)
    {
        m_oEditable = FALSE;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CParameterBase::InitVisible(CStdString p_visibleStr)
{
    BOOL oResult(FALSE);

    if(p_visibleStr.CompareNoCase(_T("True")) == 0)
    {
        m_oVisible = TRUE;
        oResult = TRUE;
    }
    else if(p_visibleStr.CompareNoCase(_T("False")) == 0)
    {
        m_oVisible = FALSE;
        oResult = TRUE;
    }

    return oResult;
}
