#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos4Def.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Epos4.h>

CCommand_DCS_Epos4::CCommand_DCS_Epos4()
{
}

CCommand_DCS_Epos4::CCommand_DCS_Epos4(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_DCS_Epos4::~CCommand_DCS_Epos4()
{
}

CCommandRoot* CCommand_DCS_Epos4::CloneCommand()
{
    CCommand_DCS_Epos4* pNewCommand;

    pNewCommand = new CCommand_DCS_Epos4();
    *pNewCommand = *this;

    return pNewCommand;
}

CCommand_DCS_Epos4& CCommand_DCS_Epos4::operator=(CCommand_DCS_Epos4& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&p_rOther);
    }

    return *this;
}

BOOL CCommand_DCS_Epos4::InitCommand(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    ResetCommand();

    if(InitCommand_ObjectDictionary(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_NetworkManagement(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_GeneralGateway(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_LayerSettingServices(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_Register(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_EtherCatStateMachine(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if (InitCommand_FoE(p_ulCommandId))
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_ObjectDictionary(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EPOS4_READ_OBJECT:
            {
                CCommand_DCS::InitCommand("ReadObject", EPOS4_READ_OBJECT);
                AddParameter(0, "index", ODT_UINT16);
                AddParameter(1, "subIndex", ODT_UINT8);
                AddParameter(2, "dataSize", ODT_UINT32, FALSE, FALSE);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadObject();
                oResult = TRUE;
                break;
            };
        case EPOS4_INITIATE_SEGMENT_READ:
            {
                CCommand_DCS::InitCommand("InitiateSegmentedRead", EPOS4_INITIATE_SEGMENT_READ);
                AddParameter(0, "index", ODT_UINT16);
                AddParameter(1, "subIndex", ODT_UINT8);
                AddParameter(2, "dataSize", ODT_UINT32, FALSE, FALSE);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "completeSize", ODT_UINT32);
                AddReturnParameter(2, "dataRead", ODT_UINT32);
                AddReturnParameter(3, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_InitiateSegmentedRead();
                oResult = TRUE;
                break;
            };
        case EPOS4_SEGMENT_READ:
            {
                CCommand_DCS::InitCommand("SegmentedRead", EPOS4_SEGMENT_READ);
                AddParameter(0, "controlByte", ODT_UINT8);
                AddParameter(1, "dataSize", ODT_UINT32);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "controlByte", ODT_UINT8);
                AddReturnParameter(2, "dataRead", ODT_UINT32);
                AddReturnParameter(3, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_SegmentedRead();
                oResult = TRUE;
                break;
            };
        case EPOS4_WRITE_OBJECT:
            {
                CCommand_DCS::InitCommand("WriteObject", EPOS4_WRITE_OBJECT);
                AddParameter(0, "index", ODT_UINT16);
                AddParameter(1, "subIndex", ODT_UINT8);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                AddParameter(3, "dataSize", ODT_UINT32, FALSE, FALSE);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "dataSize", ODT_UINT32);
                SetDefaultParameter_WriteObject();
                oResult = TRUE;
                break;
            };
        case EPOS4_INITIATE_SEGMENT_WRITE:
            {
                CCommand_DCS::InitCommand("InitiateSegmentedWrite", EPOS4_INITIATE_SEGMENT_WRITE);
                AddParameter(0, "index", ODT_UINT16);
                AddParameter(1, "subIndex", ODT_UINT8);
                AddParameter(2, "dataSize", ODT_UINT32);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_InitiateSegmentedWrite();
                oResult = TRUE;
                break;
            };
        case EPOS4_SEGMENT_WRITE:
            {
                CCommand_DCS::InitCommand("SegmentedWrite", EPOS4_SEGMENT_WRITE);
                AddParameter(0, "controlByte", ODT_UINT8);
                AddParameter(1, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "controlByte", ODT_UINT8);
                AddReturnParameter(2, "dataWritten", ODT_UINT32);
                SetDefaultParameter_SegmentedWrite();
                oResult = TRUE;
                break;
            };
        case EPOS4_ABORT_TRANSFER:
            {
                CCommand_DCS::InitCommand("AbortTransfer", EPOS4_ABORT_TRANSFER);
                AddParameter(0, "index", ODT_UINT16);
                AddParameter(1, "subIndex", ODT_UINT8);
                AddParameter(2, "abortCode", ODT_UINT32);
                SetDefaultParameter_AbortTransfer();
                oResult = TRUE;
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_NetworkManagement(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EPOS4_SEND_NMT_SERVICE:
            {
                CCommand_DCS::InitCommand("SendNMTService", EPOS4_SEND_NMT_SERVICE);
                AddParameter(0, "nodeIdentifier", ODT_UINT16);
                AddParameter(1, "commandSpecifier", ODT_UINT16);
                SetDefaultParameter_SendNMTService();
                oResult = TRUE;
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_GeneralGateway(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EPOS4_REQUEST_CAN_FRAME:
            {
                CCommand_DCS::InitCommand("RequestCANFrame", EPOS4_REQUEST_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT16);
                AddParameter(1, "length", ODT_UINT16);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_RequestCANFrame();
                oResult = TRUE;
                break;
            };
        case EPOS4_SEND_CAN_FRAME:
            {
                CCommand_DCS::InitCommand("SendCANFrame", EPOS4_SEND_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT16);
                AddParameter(1, "length", ODT_UINT16);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_SendCANFrame();
                oResult = TRUE;
                break;
            };
        case EPOS4_READ_CAN_FRAME:
            {
                CCommand_DCS::InitCommand("ReadCANFrame", EPOS4_READ_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT16);
                AddParameter(1, "length", ODT_UINT16);
                AddParameter(2, "timeout", ODT_UINT32);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadCANFrame();
                oResult = TRUE;
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_LayerSettingServices(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EPOS4_SEND_LSS_FRAME:
            {
                CCommand_DCS::InitCommand("SendLSSFrame", EPOS4_SEND_LSS_FRAME);
                AddParameter(0, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_SendLSSFrame();
                oResult = TRUE;
                break;
            };
        case EPOS4_READ_LSS_FRAME:
            {
                CCommand_DCS::InitCommand("ReadLSSFrame", EPOS4_READ_LSS_FRAME);
                AddParameter(0, "timeout", ODT_UINT16);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadLSSFrame();
                oResult = TRUE;
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_Register(DWORD p_ulCommandId)
{
    BOOL oResult = FALSE;

    switch(p_ulCommandId)
    {
        case EPOS4_READ_ESC_MEMORY:
            {
                CCommand_DCS::InitCommand("ReadRegister", EPOS4_READ_ESC_MEMORY);
                AddParameter(0, "address", ODT_UINT16);
                AddParameter(1, "length", ODT_UINT16);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "lengthRead", ODT_UINT16);
                AddReturnParameter(2, "data", ODT_MEMORY_BLOCK);
                oResult = SetDefaultParameter_ReadEscRegister();
                break;
            };
        case EPOS4_WRITE_ESC_MEMORY:
            {
                CCommand_DCS::InitCommand("WriteRegister", EPOS4_WRITE_ESC_MEMORY);
                AddParameter(0, "address", ODT_UINT16);
                AddParameter(1, "length", ODT_UINT16);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "lengthWritten", ODT_UINT16);
                oResult = SetDefaultParameter_WriteEscRegister();
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_EtherCatStateMachine(DWORD p_ulCommandId)
{
    BOOL oResult = FALSE;

    switch(p_ulCommandId)
    {
        case EPOS4_INIT_ESM:
            {
                CCommand_DCS::InitCommand("Init ESM", EPOS4_INIT_ESM);
                AddParameter(0, "vendorId", ODT_UINT32);
                AddParameter(1, "productCode", ODT_UINT32);
                AddParameter(2, "revisionNo", ODT_UINT32);
                AddParameter(3, "sm MBoxOut", ODT_UINT64);
                AddParameter(4, "sm MBoxIn", ODT_UINT64);
                AddParameter(5, "sm Outputs", ODT_UINT64);
                AddParameter(6, "sm Inputs", ODT_UINT64);
                AddParameter(7, "fmmu 0", ODT_MEMORY_BLOCK);
                AddParameter(8, "fmmu 1", ODT_MEMORY_BLOCK);
                AddParameter(9, "eeprom BootStrap", ODT_UINT64);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                oResult = SetDefaultParameter_InitEsm();
                break;
            };
        case EPOS4_CHANGE_ESM_STATE:
            {
                CCommand_DCS::InitCommand("Change ESM State", EPOS4_CHANGE_ESM_STATE);
                AddParameter(0, "state", ODT_UINT16);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                oResult = SetDefaultParameter_ChangeEsmState();
                break;
            };
        case EPOS4_GET_ESM_STATE:
            {
                CCommand_DCS::InitCommand("Get ESM State", EPOS4_GET_ESM_STATE);
                AddReturnParameter(0, "state", ODT_UINT16);
                AddReturnParameter(1, "errorCode", ODT_UINT32);

                oResult = SetDefaultParameter_GetEsmState();
                break;
            };
    }

    return oResult;
}

BOOL CCommand_DCS_Epos4::InitCommand_FoE(DWORD p_ulCommandId)
{
    BOOL oResult = FALSE;

    switch (p_ulCommandId)
    {
        case EPOS4_READ_FILE:
            {
                CCommand_DCS::InitCommand("ReadFile", EPOS4_READ_FILE);
                AddParameter(0, "password", ODT_UINT32);
                AddParameter(1, "filename", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(2, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadFile();
                break;
            };
        case EPOS4_WRITE_FILE:
            {
                CCommand_DCS::InitCommand("WriteFile", EPOS4_WRITE_FILE);
                AddParameter(0, "password", ODT_UINT32);
                AddParameter(1, "filename", ODT_MEMORY_BLOCK);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_WriteFile();
                break;
            };
        case EPOS4_INITIATE_SEGMENT_READ_FILE:
            {
                CCommand_DCS::InitCommand("InitiateSegmentedReadFile", EPOS4_INITIATE_SEGMENT_READ_FILE);
                AddParameter(0, "password", ODT_UINT32);
                AddParameter(1, "filename", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_InitiateSegmentedReadFile();
                break;
            };
        case EPOS4_INITIATE_SEGMENT_WRITE_FILE:
            {
                CCommand_DCS::InitCommand("InitiateSegmentedWriteFile", EPOS4_INITIATE_SEGMENT_WRITE_FILE);
                AddParameter(0, "password", ODT_UINT32);
                AddParameter(1, "filename", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                SetDefaultParameter_InitiateSegmentedWriteFile();
                break;
            };
        case EPOS4_SEGMENT_READ_FILE:
            {
                CCommand_DCS::InitCommand("SegmentReadFile", EPOS4_SEGMENT_READ_FILE);
                AddParameter(0, "segmentNumber", ODT_UINT32);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "moreSegments", ODT_BOOLEAN);
                AddReturnParameter(2, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_SegmentReadFile();
                break;
            };

        case EPOS4_SEGMENT_WRITE_FILE:
            {
                CCommand_DCS::InitCommand("SegmentWriteFile", EPOS4_SEGMENT_WRITE_FILE);
                AddParameter(0, "segmentNumber", ODT_UINT32);
                AddParameter(1, "moreSegments", ODT_BOOLEAN);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "errorCode", ODT_UINT32);
                AddReturnParameter(1, "dataWritten", ODT_UINT32);
                AddReturnParameter(2, "isBusy", ODT_BOOLEAN);
                AddReturnParameter(3, "progressInPermille", ODT_UINT32);
                SetDefaultParameter_SegmentWriteFile();
                break;
            };
    }

    return oResult;
}

void CCommand_DCS_Epos4::SetDefaultParameter_InitiateSegmentedRead()
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulErrorCode = 0;
    DWORD ulCompleteSize = 0;
    DWORD ulDateRead = 0;
    DWORD ulDataSize = 0;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));
    SetParameterData(1, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(2, &ulDataSize, sizeof(ulDataSize));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &ulCompleteSize, sizeof(ulCompleteSize));
    SetReturnParameterData(2, &ulDateRead, sizeof(ulDateRead));
    SetReturnParameterData(3, pData, 0);
}

void CCommand_DCS_Epos4::SetDefaultParameter_InitiateSegmentedWrite()
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulDataSize = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));
    SetParameterData(1, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(2, &ulDataSize, sizeof(ulDataSize));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
}

void CCommand_DCS_Epos4::SetDefaultParameter_ReadObject()
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulDataSize = 0;
    void* pData = NULL;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));
    SetParameterData(1, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(2, &ulDataSize, sizeof(ulDataSize));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, pData, 0);
}

void CCommand_DCS_Epos4::SetDefaultParameter_SegmentedRead()
{
    BYTE ubControlByte = 0;
    DWORD ulDataSize = 0;
    DWORD ulErrorCode = 0;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &ubControlByte, sizeof(ubControlByte));
    SetParameterData(1, &ulDataSize, sizeof(ulDataSize));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &ubControlByte, sizeof(ubControlByte));
    SetReturnParameterData(2, &ulDataSize, sizeof(ulDataSize));
    SetReturnParameterData(3, pData, 0);
}

void CCommand_DCS_Epos4::SetDefaultParameter_SegmentedWrite()
{
    BYTE ubControlByte = 0;
    DWORD ulDateWritten = 0;
    DWORD ulErrorCode = 0;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &ubControlByte, sizeof(ubControlByte));
    SetParameterData(1, pData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &ubControlByte, sizeof(ubControlByte));
    SetReturnParameterData(2, &ulDateWritten, sizeof(ulDateWritten));
}

void CCommand_DCS_Epos4::SetDefaultParameter_WriteObject()
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulDataSize = 0;
    DWORD ulErrorCode = 0;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));
    SetParameterData(1, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(2, pData, 0);
    SetParameterData(3, &ulDataSize, sizeof(ulDataSize));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &ulDataSize, sizeof(ulDataSize));
}

void CCommand_DCS_Epos4::SetDefaultParameter_AbortTransfer()
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));
    SetParameterData(1, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(2, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_DCS_Epos4::SetDefaultParameter_SendNMTService()
{
    WORD usNodeIdentifier = 0;
    WORD usCmdSpecifier = 0;

    //Parameter
    SetParameterData(0, &usNodeIdentifier, sizeof(usNodeIdentifier));
    SetParameterData(1, &usCmdSpecifier, sizeof(usCmdSpecifier));
}

void CCommand_DCS_Epos4::SetDefaultParameter_RequestCANFrame()
{
    WORD usCobId = 0;
    WORD usLength= 0;
    void* pData = NULL;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usCobId, sizeof(usCobId));
    SetParameterData(1, &usLength, sizeof(usLength));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, pData, usLength);
}

void CCommand_DCS_Epos4::SetDefaultParameter_SendCANFrame()
{
    WORD usCobId = 0;
    WORD usLength = 0;
    void* pData = NULL;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usCobId, sizeof(usCobId));
    SetParameterData(1, &usLength, sizeof(usLength));
    SetParameterData(2, pData, usLength);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
}

void CCommand_DCS_Epos4::SetDefaultParameter_ReadCANFrame()
{
    WORD usCobId = 0;
    WORD usLength = 0;
    DWORD ulTimeout = 0;
    void* pData = NULL;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usCobId, sizeof(usCobId));
    SetParameterData(1, &usLength, sizeof(usLength));
    SetParameterData(2, &ulTimeout, sizeof(ulTimeout));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, pData, usLength);
}

void CCommand_DCS_Epos4::SetDefaultParameter_SendLSSFrame()
{
    DWORD ulErrorCode = 0;
    void* pData = NULL;

    //Parameter
    SetParameterData(0, pData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
}

void CCommand_DCS_Epos4::SetDefaultParameter_ReadLSSFrame()
{
    DWORD ulErrorCode = 0;
    void* pData = NULL;
    WORD usTimeout = 10;

    //Parameter
    SetParameterData(0, &usTimeout, sizeof(usTimeout));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, pData, 0);
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_ReadEscRegister()
{
    BOOL oResult = TRUE;
    WORD usAdo = 0;
    WORD usLength = 0;
    WORD usLengthRead = 0;
    DWORD ulTimeout = 0;
    void* pData = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usAdo, sizeof(usAdo));
    SetParameterData(1, &usLength, sizeof(usLength));
    SetParameterData(2, &ulTimeout, sizeof(ulTimeout));

    //ReturnParameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &usLengthRead, sizeof(usLengthRead));
    SetReturnParameterData(2, pData, usLength);

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_WriteEscRegister()
{
    BOOL oResult = TRUE;
    WORD usAdo = 0;
    WORD usLength = 0;
    WORD usLengthWritten = 0;
    void* pData = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &usAdo, sizeof(usAdo));
    SetParameterData(1, &usLength, sizeof(usLength));
    SetParameterData(2, pData, usLength);

    //ReturnParameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &usLengthWritten, sizeof(usLengthWritten));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_InitEsm()
{
    BOOL oResult = TRUE;
    DWORD ulErrorCode = 0;
    DWORD ulProductCode = 0;
    DWORD ulRevisionNo = 0;
    DWORD ulVendorId = 0;
    int ulLength = 0;
    __uint64 ullSyncManager0 = 0;
    __uint64 ullSyncManager1 = 0;
    __uint64 ullSyncManager2 = 0;
    __uint64 ullSyncManager3 = 0;
    void* pFmmu0 = 0;
    void* pFmmu1 = 0;
    __uint64 ullBootstrap = 0;

    //Parameter
    SetParameterData(0, &ulVendorId, sizeof(ulVendorId));
    SetParameterData(1, &ulProductCode, sizeof(ulProductCode));
    SetParameterData(2, &ulRevisionNo, sizeof(ulRevisionNo));
    SetParameterData(3, &ullSyncManager0, sizeof(ullSyncManager0));
    SetParameterData(4, &ullSyncManager1, sizeof(ullSyncManager1));
    SetParameterData(5, &ullSyncManager2, sizeof(ullSyncManager2));
    SetParameterData(6, &ullSyncManager3, sizeof(ullSyncManager3));
    SetParameterData(7, pFmmu0, ulLength);
    SetParameterData(8, pFmmu1, ulLength);
    SetParameterData(9, &ullBootstrap, sizeof(ullBootstrap));

    //ReturnParameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_ChangeEsmState()
{
    BOOL oResult = TRUE;
    DWORD ulErrorCode = 0;
    WORD usState = 0;

    //Parameter
    SetParameterData(0, &usState, sizeof(usState));

    //ReturnParameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_GetEsmState()
{
    BOOL oResult = TRUE;
    DWORD ulErrorCode = 0;
    WORD usState = 0;

    //ReturnParameter
    SetReturnParameterData(0, &usState, sizeof(usState));
    SetReturnParameterData(1, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_ReadFile()
{
    BOOL oResult = TRUE;
    DWORD ulPassword = 0;
    void* pFilename = 0;
    DWORD ulErrorCode = 0;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &ulPassword, sizeof(ulPassword));
    SetParameterData(1, pFilename, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, pData, 0);

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_WriteFile()
{
    BOOL oResult = TRUE;
    DWORD ulPassword = 0;
    void* pFilename = 0;
    void* pData = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &ulPassword, sizeof(ulPassword));
    SetParameterData(1, pFilename, 0);
    SetParameterData(2, pData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_InitiateSegmentedReadFile()
{
    BOOL oResult = TRUE;
    DWORD ulPassword = 0;
    void* pFilename = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &ulPassword, sizeof(ulPassword));
    SetParameterData(1, pFilename, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_InitiateSegmentedWriteFile()
{
    BOOL oResult = TRUE;
    DWORD ulPassword = 0;
    void* pFilename = 0;
    DWORD ulErrorCode = 0;

    //Parameter
    SetParameterData(0, &ulPassword, sizeof(ulPassword));
    SetParameterData(1, pFilename, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_SegmentReadFile()
{
    BOOL oResult = TRUE;
    DWORD ulSegmentNumber = 0;
    DWORD ulErrorCode = 0;
    BOOL oMoreSegments = FALSE;
    void* pData = 0;

    //Parameter
    SetParameterData(0, &ulSegmentNumber, sizeof(ulSegmentNumber));

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &oMoreSegments, sizeof(oMoreSegments));
    SetReturnParameterData(2, pData, 0);

    return oResult;
}

BOOL CCommand_DCS_Epos4::SetDefaultParameter_SegmentWriteFile()
{
    BOOL oResult = TRUE;
    DWORD ulSegmentNumber = 0;
    BOOL oMoreSegments = FALSE;
    void* pData = 0;
    DWORD ulErrorCode = 0;
    DWORD ulDataWritten = 0;
    BOOL oIsBusy = FALSE;
    DWORD ulProgressInPermille = 0;

    //Parameter
    SetParameterData(0, &ulSegmentNumber, sizeof(ulSegmentNumber));
    SetParameterData(1, &oMoreSegments, sizeof(oMoreSegments));
    SetParameterData(2, pData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
    SetReturnParameterData(1, &ulDataWritten, sizeof(ulDataWritten));
    SetReturnParameterData(2, &oIsBusy, sizeof(oIsBusy));
    SetReturnParameterData(3, &ulProgressInPermille, sizeof(ulProgressInPermille));

    return oResult;
}