#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/Command_PS_CANopen.h>

CCommand_PS_CANopen::CCommand_PS_CANopen()
{
}

CCommand_PS_CANopen::CCommand_PS_CANopen(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_PS_CANopen::~CCommand_PS_CANopen()
{
}

CCommandRoot*CCommand_PS_CANopen::CloneCommand()
{
    CCommand_PS_CANopen* pNewCommand;

    pNewCommand = new CCommand_PS_CANopen();
    *pNewCommand = *this;

    return pNewCommand;
}

CCommand_PS_CANopen& CCommand_PS_CANopen::operator=(CCommand_PS_CANopen& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&p_rOther);
    }

    return *this;
}

BOOL CCommand_PS_CANopen::InitCommand(DWORD p_ulCommandId)
{
    const DWORD DATA_SIZE = 4;
    const DWORD SEGMENT_DATA_SIZE = 7;

    BOOL oResult(FALSE);

    ResetCommand();

    switch(p_ulCommandId)
    {
        case CANOPEN_INITIATE_SDO_DOWNLOAD:
            {
                CCommand_PS::InitCommand("Initiate SDO Download", CANOPEN_INITIATE_SDO_DOWNLOAD);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "cobId Server->Client", ODT_UINT32, FALSE, TRUE);
                AddParameter(2, "expeditedTransfer", ODT_BOOLEAN);
                AddParameter(3, "sizeIndicated", ODT_BOOLEAN);
                AddParameter(4, "nonValidNbOfBytes", ODT_UINT8);
                AddParameter(5, "index", ODT_UINT16);
                AddParameter(6, "subIndex", ODT_UINT8);
                AddParameter(7, "data", ODT_MEMORY_BLOCK, DATA_SIZE);
                AddReturnParameter(0, "cobId Server->Client", ODT_UINT32);
                AddReturnParameter(1, "index", ODT_UINT16, FALSE);
                AddReturnParameter(2, "subIndex", ODT_UINT8, FALSE);
                AddReturnParameter(3, "abortCode", ODT_UINT32);
                SetDefaultParameter_InitiateSDODownload();
                oResult = TRUE;
                break;
            };
        case CANOPEN_DOWNLOAD_SDO_SEGMENT:
            {
                CCommand_PS::InitCommand("Download SDO Segment", CANOPEN_DOWNLOAD_SDO_SEGMENT);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "cobId Server->Client", ODT_UINT32, FALSE, TRUE);
                AddParameter(2, "toggle", ODT_BOOLEAN);
                AddParameter(3, "nonValidNbOfBytes", ODT_UINT8);
                AddParameter(4, "noMoreSegments", ODT_BOOLEAN);
                AddParameter(5, "segData", ODT_MEMORY_BLOCK, SEGMENT_DATA_SIZE);
                AddReturnParameter(0, "cobId Server->Client", ODT_UINT32);
                AddReturnParameter(1, "toggle", ODT_BOOLEAN);
                AddReturnParameter(2, "abortCode", ODT_UINT32);
                SetDefaultParameter_DownloadSDOSegment();
                oResult = TRUE;
                break;
            };
        case CANOPEN_INITIATE_SDO_UPLOAD:
            {
                CCommand_PS::InitCommand("Initiate SDO Upload", CANOPEN_INITIATE_SDO_UPLOAD);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "cobId Server->Client", ODT_UINT32, FALSE, TRUE);
                AddParameter(2, "index", ODT_UINT16);
                AddParameter(3, "subIndex", ODT_UINT8);
                AddReturnParameter(0, "cobId Server->Client", ODT_UINT32);
                AddReturnParameter(1, "expeditedTransfer", ODT_BOOLEAN);
                AddReturnParameter(2, "sizeIndicated", ODT_BOOLEAN);
                AddReturnParameter(3, "nonValidNbOfBytes", ODT_UINT8);
                AddReturnParameter(4, "index", ODT_UINT16, FALSE);
                AddReturnParameter(5, "subIndex", ODT_UINT8, FALSE);
                AddReturnParameter(6, "data", ODT_MEMORY_BLOCK, DATA_SIZE);
                AddReturnParameter(7, "abortCode", ODT_UINT32);
                SetDefaultParameter_InitiateSDOUpload();
                oResult = TRUE;
                break;
            };
        case CANOPEN_UPLOAD_SDO_SEGMENT:
            {
                CCommand_PS::InitCommand("Upload SDO Segment", CANOPEN_UPLOAD_SDO_SEGMENT);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "cobId Server->Client", ODT_UINT32, FALSE, TRUE);
                AddParameter(2, "toggle", ODT_BOOLEAN);
                AddReturnParameter(0, "cob-Id Server->Client", ODT_UINT32);
                AddReturnParameter(1, "toggle", ODT_BOOLEAN);
                AddReturnParameter(2, "nonValidNbOfBytes", ODT_UINT8);
                AddReturnParameter(3, "noMoreSegments", ODT_BOOLEAN);
                AddReturnParameter(4, "segData", ODT_MEMORY_BLOCK, SEGMENT_DATA_SIZE);
                AddReturnParameter(5, "abortCode", ODT_UINT32);
                SetDefaultParameter_UploadSDOSegment();
                oResult = TRUE;
                break;
            };
        case CANOPEN_ABORT_SDO_TRANSFER:
            {
                CCommand_PS::InitCommand("Abort SDO Transfer", CANOPEN_ABORT_SDO_TRANSFER);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "index", ODT_UINT16);
                AddParameter(2, "subIndex", ODT_UINT8);
                AddParameter(3, "abortCode", ODT_UINT32);
                SetDefaultParameter_AbortSDOTransfer();
                oResult = TRUE;
                break;
            };
        case CANOPEN_NETWORK_INDICATION:
            {
                CCommand_PS::InitCommand("Network Indication", CANOPEN_NETWORK_INDICATION);
                AddParameter(0, "cobId Client->Server", ODT_UINT32);
                AddParameter(1, "cobId Server->Client", ODT_UINT32, FALSE, TRUE);
                AddParameter(2, "targetNetworkId", ODT_UINT16);
                AddParameter(3, "targetNodeId", ODT_UINT8);
                AddReturnParameter(0, "cobId Server->Client", ODT_UINT32);
                AddReturnParameter(1, "targetNetworkId", ODT_UINT16);
                AddReturnParameter(2, "targetNodeId", ODT_UINT8);
                AddReturnParameter(3, "abortCode", ODT_UINT32);
                SetDefaultParameter_NetworkIndication();
                oResult = TRUE;
                break;
            };
        case CANOPEN_SEND_NMT_SERVICE:
            {
                CCommand_PS::InitCommand("Send NMT Service", CANOPEN_SEND_NMT_SERVICE);
                AddParameter(0, "commandSpecifier", ODT_UINT8);
                AddParameter(1, "nodeId", ODT_UINT8);
                SetDefaultParameter_SendNMTService();
                oResult = TRUE;
                break;
            };
        case CANOPEN_SEND_CAN_FRAME:
            {
                CCommand_PS::InitCommand("Send CAN Frame", CANOPEN_SEND_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT32);
                AddParameter(1, "length", ODT_UINT8);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_SendCANFrame();
                oResult = TRUE;
                break;
            };
        case CANOPEN_READ_CAN_FRAME:
            {
                CCommand_PS::InitCommand("Read CAN Frame", CANOPEN_READ_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT32);
                AddParameter(1, "length", ODT_UINT8);
                AddParameter(2, "timeout", ODT_UINT32);
                AddReturnParameter(0, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadCANFrame();
                oResult = TRUE;
                break;
            };
        case CANOPEN_REQUEST_CAN_FRAME:
            {
                CCommand_PS::InitCommand("Request CAN Frame", CANOPEN_REQUEST_CAN_FRAME);
                AddParameter(0, "cobId", ODT_UINT32);
                AddParameter(1, "length", ODT_UINT8);
                AddReturnParameter(0, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_RequestCANFrame();
                oResult = TRUE;
                break;
            };
        case CANOPEN_SEND_LSS_FRAME:
            {
                CCommand_PS::InitCommand("Send LSS Frame", CANOPEN_SEND_LSS_FRAME);
                AddParameter(0, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_SendLSSFrame();
                oResult = TRUE;
                break;
            };
        case CANOPEN_READ_LSS_FRAME:
            {
                CCommand_PS::InitCommand("Read LSS Frame", CANOPEN_READ_LSS_FRAME);
                AddParameter(0, "timeout", ODT_UINT16);
                AddReturnParameter(0, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_ReadLSSFrame();
                oResult = TRUE;
                break;
            };
        case CANOPEN_CLEANUP_COMMUNICATION:
            {
                CCommand_PS::InitCommand("Cleanup Communication", CANOPEN_CLEANUP_COMMUNICATION);
                AddParameter(0, "cleanupCanPassive", ODT_BOOLEAN);
                SetDefaultParameter_CleanupCommunication();
                oResult = TRUE;
                break;
            };
    }

    return oResult;
}

void CCommand_PS_CANopen::SetDefaultParameter_InitiateSDODownload()
{
    DWORD ulCobId = 0;
    BOOL oExpeditedTransfer = TRUE;
    BOOL oSizeIndicated = FALSE;
    BYTE ubNonValidNbOfBytes = 0;
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    void* pData = NULL;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ulCobId, sizeof(ulCobId));
    SetParameterData(2, &oExpeditedTransfer, sizeof(oExpeditedTransfer));
    SetParameterData(3, &oSizeIndicated, sizeof(oSizeIndicated));
    SetParameterData(4, &ubNonValidNbOfBytes, sizeof(ubNonValidNbOfBytes));
    SetParameterData(5, &usIndex, sizeof(usIndex));
    SetParameterData(6, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(7, pData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulCobId, sizeof(ulCobId));
    SetReturnParameterData(1, &usIndex, sizeof(usIndex));
    SetReturnParameterData(2, &ubSubIndex, sizeof(ubSubIndex));
    SetReturnParameterData(3, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_DownloadSDOSegment()
{
    DWORD ulCobId = 0;
    BOOL oToggle = TRUE;
    BYTE ubNonValidNbOfBytes = 0;
    BOOL oNoMoreSegments = FALSE;
    void* pSegData = NULL;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ulCobId, sizeof(ulCobId));
    SetParameterData(2, &oToggle, sizeof(oToggle));
    SetParameterData(3, &ubNonValidNbOfBytes, sizeof(ubNonValidNbOfBytes));
    SetParameterData(4, &oNoMoreSegments, sizeof(oNoMoreSegments));
    SetParameterData(5, pSegData, 0);

    //Return parameter
    SetReturnParameterData(0, &ulCobId, sizeof(ulCobId));
    SetReturnParameterData(1, &oToggle, sizeof(oToggle));
    SetReturnParameterData(2, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_InitiateSDOUpload()
{
    DWORD ulCobId = 0;
    BOOL oExpeditedTransfer = TRUE;
    BOOL oSizeIndicated = FALSE;
    BYTE ubNonValidNbOfBytes = 0;
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    void* pData = NULL;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ulCobId, sizeof(ulCobId));
    SetParameterData(2, &usIndex, sizeof(usIndex));
    SetParameterData(3, &ubSubIndex, sizeof(ubSubIndex));

    //Return parameter
    SetReturnParameterData(0, &ulCobId, sizeof(ulCobId));
    SetReturnParameterData(1, &oExpeditedTransfer, sizeof(oExpeditedTransfer));
    SetReturnParameterData(2, &oSizeIndicated, sizeof(oSizeIndicated));
    SetReturnParameterData(3, &ubNonValidNbOfBytes, sizeof(ubNonValidNbOfBytes));
    SetReturnParameterData(4, &usIndex, sizeof(usIndex));
    SetReturnParameterData(5, &ubSubIndex, sizeof(ubSubIndex));
    SetReturnParameterData(6, pData, 0);
    SetReturnParameterData(7, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_UploadSDOSegment()
{
    DWORD ulCobId = 0;
    BOOL oToggle = TRUE;
    BYTE ubNonValidNbOfBytes = 0;
    BOOL oNoMoreSegments = FALSE;
    void* pSegData = NULL;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ulCobId, sizeof(ulCobId));
    SetParameterData(2, &oToggle, sizeof(oToggle));

    //Return parameter
    SetReturnParameterData(0, &ulCobId, sizeof(ulCobId));
    SetReturnParameterData(1, &oToggle, sizeof(oToggle));
    SetReturnParameterData(2, &ubNonValidNbOfBytes, sizeof(ubNonValidNbOfBytes));
    SetReturnParameterData(3, &oNoMoreSegments, sizeof(oNoMoreSegments));
    SetReturnParameterData(4, pSegData, 0);
    SetReturnParameterData(5, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_AbortSDOTransfer()
{
    DWORD ulCobId = 0;
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &usIndex, sizeof(usIndex));
    SetParameterData(2, &ubSubIndex, sizeof(ubSubIndex));
    SetParameterData(3, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_NetworkIndication()
{
    DWORD ulCobId = 0;
    WORD usTargetNetworkId = 0;
    BYTE ubTargetNodeId = 0;
    DWORD ulAbortCode = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ulCobId, sizeof(ulCobId));
    SetParameterData(2, &usTargetNetworkId, sizeof(usTargetNetworkId));
    SetParameterData(3, &ubTargetNodeId, sizeof(ubTargetNodeId));

    //Return parameter
    SetReturnParameterData(0, &ulCobId, sizeof(ulCobId));
    SetReturnParameterData(1, &usTargetNetworkId, sizeof(usTargetNetworkId));
    SetReturnParameterData(2, &ubTargetNodeId, sizeof(ubTargetNodeId));
    SetReturnParameterData(3, &ulAbortCode, sizeof(ulAbortCode));
}

void CCommand_PS_CANopen::SetDefaultParameter_SendNMTService()
{
    BYTE ubCommandSpecifier = 0;
    BYTE ubNodeId = 0;

    //Parameter
    SetParameterData(0, &ubCommandSpecifier, sizeof(ubCommandSpecifier));
    SetParameterData(1, &ubNodeId, sizeof(ubNodeId));
}

void CCommand_PS_CANopen::SetDefaultParameter_SendCANFrame()
{
    DWORD ulCobId = 0;
    BYTE ubLength = 0;
    void* pData = NULL;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ubLength, sizeof(ubLength));
    SetParameterData(2, pData, ubLength);
}

void CCommand_PS_CANopen::SetDefaultParameter_ReadCANFrame()
{
    DWORD ulCobId = 0;
    BYTE ubLength = 0;
    void* pData = NULL;
    DWORD ulTimeout = 0;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ubLength, sizeof(ubLength));
    SetParameterData(2, &ulTimeout, sizeof(ulTimeout));

    //Return parameter
    SetReturnParameterData(0, pData, ubLength);
}

void CCommand_PS_CANopen::SetDefaultParameter_RequestCANFrame()
{
    DWORD ulCobId = 0;
    BYTE ubLength = 0;
    void* pData = NULL;

    //Parameter
    SetParameterData(0, &ulCobId, sizeof(ulCobId));
    SetParameterData(1, &ubLength, sizeof(ubLength));

    //Return parameter
    SetReturnParameterData(0, pData, ubLength);
}

void CCommand_PS_CANopen::SetDefaultParameter_SendLSSFrame()
{
    void* pData = NULL;

    //Parameter
    SetParameterData(0, pData, 0);
}

void CCommand_PS_CANopen::SetDefaultParameter_ReadLSSFrame()
{
    WORD usTimeout = 100;

    //Parameter
    SetParameterData(0, &usTimeout, sizeof(usTimeout));
}

void CCommand_PS_CANopen::SetDefaultParameter_CleanupCommunication()
{
    BOOL oCleanupAfterCanPassive = FALSE;

    //Parameter
    SetParameterData(0, &oCleanupAfterCanPassive, sizeof(oCleanupAfterCanPassive));
}
