#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_DataRecorder.h>

CCommand_VCS_DataRecorder::CCommand_VCS_DataRecorder()
{
}

CCommand_VCS_DataRecorder::CCommand_VCS_DataRecorder(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_VCS_DataRecorder::~CCommand_VCS_DataRecorder()
{
}

CCommandRoot*CCommand_VCS_DataRecorder::CloneCommand()
{
    CCommand_VCS_DataRecorder* pNewCommand;

    pNewCommand = new CCommand_VCS_DataRecorder();
    *pNewCommand = *this;

    return pNewCommand;
}

CCommand_VCS_DataRecorder& CCommand_VCS_DataRecorder::operator=(CCommand_VCS_DataRecorder& other)
{
    if(this != &other)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&other);
    }

    return *this;
}

BOOL CCommand_VCS_DataRecorder::InitCommand(DWORD p_ulCommandId)
{
    ResetCommand();

    if(InitCommand_DataRecording(p_ulCommandId)) return TRUE;

    return FALSE;
}

BOOL CCommand_VCS_DataRecorder::InitCommand_DataRecording(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case REC_SET_RECORDER_PARAMETER:
            {
                CCommand_VCS::InitCommand("SetRecorderParameter", REC_SET_RECORDER_PARAMETER);

                AddParameter(0, "samplingPeriod", ODT_UINT16);
                AddParameter(1, "nbOfPrecedingSamples", ODT_UINT16);

                SetDefaultParameter_SetRecorderParameter();
                oResult = TRUE;
                break;
            };
        case REC_GET_RECORDER_PARAMETER:
            {
                CCommand_VCS::InitCommand("GetRecorderParameter", REC_GET_RECORDER_PARAMETER);

                AddReturnParameter(0, "samplingPeriod", ODT_UINT16);
                AddReturnParameter(1, "nbOfPrecedingSamples", ODT_UINT16);

                SetDefaultParameter_GetRecorderParameter();
                oResult = TRUE;
                break;
            };
        case REC_ENABLE_TRIGGER:
            {
                CCommand_VCS::InitCommand("EnableTrigger", REC_ENABLE_TRIGGER);

                AddParameter(0, "triggerType", ODT_UINT8);

                SetDefaultParameter_EnableTrigger();
                oResult = TRUE;
                break;
            };
        case REC_DISABLE_ALL_TRIGGERS:
            {
                CCommand_VCS::InitCommand("DisableAllTrigger", REC_DISABLE_ALL_TRIGGERS);

                SetDefaultParameter_DisableAllTriggers();
                oResult = TRUE;
                break;
            };
        case REC_ACTIVATE_CHANNEL:
            {
                CCommand_VCS::InitCommand("ActivateChannel", REC_ACTIVATE_CHANNEL);

                AddParameter(0, "channelNumber", ODT_UINT8);
                AddParameter(1, "objectIndex", ODT_UINT16);
                AddParameter(2, "objectSubIndex", ODT_UINT8);
                AddParameter(3, "objectSize", ODT_UINT8);

                SetDefaultParameter_ActivateChannel();
                oResult = TRUE;
                break;
            };
        case REC_DEACTIVATE_ALL_CHANNELS:
            {
                CCommand_VCS::InitCommand("DeactivateAllChannels", REC_DEACTIVATE_ALL_CHANNELS);

                SetDefaultParameter_DeactivateAllChannels();
                oResult = TRUE;
                break;
            };
        case REC_ACTIVE_CHANNEL:
            {
                CCommand_VCS::InitCommand("ActiveChannel", REC_ACTIVATE_CHANNEL);

                AddParameter(0, "channelNumber", ODT_UINT8);
                AddReturnParameter(0, "objectIndex", ODT_UINT16);
                AddReturnParameter(1, "objectSubIndex", ODT_UINT8);
                AddReturnParameter(2, "objectSize", ODT_UINT8);

                SetDefaultParameter_ActiveChannel();
                oResult = TRUE;
                break;
            };
        case REC_GET_VARIABLE:
            {
                CCommand_VCS::InitCommand("GetVariable", REC_GET_VARIABLE);

                AddParameter(0, "index", ODT_UINT16);

                AddReturnParameter(0, "variableIndex", ODT_UINT16);
                AddReturnParameter(1, "variableSubIndex", ODT_UINT16);
                AddReturnParameter(2, "variableSize", ODT_UINT16);

                SetDefaultParameter_GetVariable();
                oResult = TRUE;
                break;
            };
        case REC_GET_MAX_NUMBER_OF_SAMPLES:
            {
                CCommand_VCS::InitCommand("GetMaxNumberOfSamples", REC_GET_MAX_NUMBER_OF_SAMPLES);

                AddReturnParameter(0, "maxNbOfSamples", ODT_UINT16);

                SetDefaultParameter_GetMaxNbOfSamples();
                oResult = TRUE;
                break;
            };
        case REC_GET_NUMBER_OF_RECORDED_SAMPLES:
            {
                CCommand_VCS::InitCommand("GetNbOfRecordedSamples", REC_GET_NUMBER_OF_RECORDED_SAMPLES);

                AddReturnParameter(0, "nbOfRecordedSamples", ODT_UINT16);

                SetDefaultParameter_GetNbOfRecordedSamples();
                oResult = TRUE;
                break;
            };
        case REC_SET_NUMBER_OF_PRECEDING_SAMPLES:
            {
                CCommand_VCS::InitCommand("SetNbOfPrecedingSamples", REC_SET_NUMBER_OF_PRECEDING_SAMPLES);

                AddParameter(0, "precedingSamples", ODT_UINT16);

                SetDefaultParameter_SetNbOfPrecedingSamples();
                oResult = TRUE;
                break;
            };
        case REC_GET_NUMBER_OF_PRECEDING_SAMPLES:
            {
                CCommand_VCS::InitCommand("GetNbOfPrecedingSamples", REC_GET_NUMBER_OF_PRECEDING_SAMPLES);

                AddReturnParameter(0, "precedingSamples", ODT_UINT16);

                SetDefaultParameter_GetNbOfPrecedingSamples();
                oResult = TRUE;
                break;
            };
        case REC_GET_NUMBER_OF_SAMPLING_VARIABLES:
            {
                CCommand_VCS::InitCommand("GetNumberOfSamplingVariables", REC_GET_NUMBER_OF_SAMPLING_VARIABLES);

                AddReturnParameter(0, "samplingVariables", ODT_UINT16);

                SetDefaultParameter_GetNbOfSamplingVariables();
                oResult = TRUE;
                break;
            };
        case REC_GET_SAMPLING_PERIOD:
            {
                CCommand_VCS::InitCommand("GetSamplingPeriod", REC_GET_SAMPLING_PERIOD);

                AddReturnParameter(0, "period", ODT_UINT16);

                SetDefaultParameter_GetSamplingPeriod();
                oResult = TRUE;
                break;
            };
        case REC_START_RECORDER:
            {
                CCommand_VCS::InitCommand("StartRecorder", REC_START_RECORDER);
                SetDefaultParameter_StartRecorder();
                oResult = TRUE;
                break;
            };
        case REC_STOP_RECORDER:
            {
                CCommand_VCS::InitCommand("StopRecorder", REC_STOP_RECORDER);
                SetDefaultParameter_StopRecorder();
                oResult = TRUE;
                break;
            };
        case REC_FORCE_TRIGGER:
            {
                CCommand_VCS::InitCommand("ForceTrigger", REC_FORCE_TRIGGER);
                SetDefaultParameter_ForceTrigger();
                oResult = TRUE;
                break;
            };
        case REC_IS_RECORDER_RUNNING:
            {
                CCommand_VCS::InitCommand("IsRecorderRunning", REC_IS_RECORDER_RUNNING);

                AddReturnParameter(0, "running", ODT_BOOLEAN);

                SetDefaultParameter_IsRecorderRunning();
                oResult = TRUE;
                break;
            };
        case REC_IS_RECORDER_TRIGGERED:
            {
                CCommand_VCS::InitCommand("IsRecorderTriggered", REC_IS_RECORDER_TRIGGERED);

                AddReturnParameter(0, "triggered", ODT_BOOLEAN);

                SetDefaultParameter_IsRecorderTriggered();
                oResult = TRUE;
                break;
            };
        case REC_GET_RECORDER_STATUS:
            {
                CCommand_VCS::InitCommand("GetRecorderStatus", REC_GET_RECORDER_STATUS);

                AddReturnParameter(0, "status", ODT_UINT16);

                SetDefaultParameter_GetRecorderStatus();
                oResult = TRUE;
                break;
            };
        case REC_SET_RECORDER_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("SetRecorderConfiguration", REC_SET_RECORDER_CONFIGURATION);

                AddParameter(0, "configuration", ODT_UINT16);

                SetDefaultParameter_SetRecorderConfiguration();
                oResult = TRUE;
                break;
            };
        case REC_GET_RECORDER_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("GetRecorderConfiguration", REC_GET_RECORDER_CONFIGURATION);

                AddReturnParameter(0, "configuration", ODT_UINT16);

                SetDefaultParameter_GetRecorderConfiguration();
                oResult = TRUE;
                break;
            };
        case REC_READ_CHANNEL_VECTOR_SIZE:
            {
                CCommand_VCS::InitCommand("ReadChannelVectorSize", REC_READ_CHANNEL_VECTOR_SIZE);

                AddReturnParameter(0, "vectorSize", ODT_UINT32);

                SetDefaultParameter_ReadChannelVectorSize();
                oResult = TRUE;
                break;
            };
        case REC_READ_CHANNEL_DATA_VECTOR:
            {
                CCommand_VCS::InitCommand("ReadChannelDataVector", REC_READ_CHANNEL_DATA_VECTOR);

                AddParameter(0, "channelNumber", ODT_UINT8);
                AddParameter(1, "vectorSize", ODT_UINT32);
                AddReturnParameter(0, "dataVector", ODT_MEMORY_BLOCK);

                SetDefaultParameter_ReadChannelDataVector();
                oResult = TRUE;
                break;
            };
        case REC_GET_VECTOR_START_OFFSET:
            {
                CCommand_VCS::InitCommand("GetVectorStartOffset", REC_GET_VECTOR_START_OFFSET);

                AddReturnParameter(0, "vectorStartOffset", ODT_UINT16);

                SetDefaultParameter_GetVectorStartOffset();
                oResult = TRUE;
                break;
            };
        case REC_READ_DATA_BUFFER:
            {
                CCommand_VCS::InitCommand("ReadDataBuffer", REC_READ_DATA_BUFFER);

                AddParameter(0, "bufferSizeToRead", ODT_UINT32);
                AddReturnParameter(0, "dataBuffer", ODT_MEMORY_BLOCK);
                AddReturnParameter(1, "bufferSizeRead", ODT_UINT32);
                AddReturnParameter(2, "vectorStartOffset", ODT_UINT16);
                AddReturnParameter(3, "maxNbOfSamples", ODT_UINT16);
                AddReturnParameter(4, "nbOfRecordedSamples", ODT_UINT16);

                SetDefaultParameter_ReadDataBuffer();
                oResult = TRUE;
                break;
            };
    }

    return FALSE;
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_SetRecorderParameter()
{
    WORD usSamplingPeriod = 10;
    WORD usNbOfPrecedingSamples = 0;

    //Parameter
    SetParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
    SetParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetRecorderParameter()
{
    WORD usSamplingPeriod = 10;
    WORD usNbOfPrecedingSamples = 0;

    //ReturnParameter
    SetReturnParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
    SetReturnParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_EnableTrigger()
{
    BYTE ubTriggerType = 0;

    //Parameter
    SetParameterData(0, &ubTriggerType, sizeof(ubTriggerType));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_DisableAllTriggers()
{
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ActivateChannel()
{
    BYTE ubChannelNumber = 0;
    WORD usObjectIndex = 0;
    BYTE ObjectSubIndex = 0;
    BYTE ObjectSize = 0;

    //Parameter
    SetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));
    SetParameterData(1, &usObjectIndex, sizeof(usObjectIndex));
    SetParameterData(2, &ObjectSubIndex, sizeof(ObjectSubIndex));
    SetParameterData(3, &ObjectSize, sizeof(ObjectSize));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_DeactivateAllChannels()
{
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ActiveChannel()
{
    BYTE ubChannelNumber = 0;
    WORD usObjectIndex = 0;
    BYTE ObjectSubIndex = 0;
    BYTE ObjectSize = 0;

    // Parameter
    SetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));

    // Return parameter
    SetParameterData(0, &usObjectIndex, sizeof(usObjectIndex));
    SetParameterData(1, &ObjectSubIndex, sizeof(ObjectSubIndex));
    SetParameterData(2, &ObjectSize, sizeof(ObjectSize));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetVariable()
{
    WORD usIndex = 1;
    WORD usVariableIndex = 0;
    WORD usVariableSubIndex = 0;
    WORD usVariableSize = 0;

    // Parameter
    SetParameterData(0, &usIndex, sizeof(usIndex));

    // Return parameter
    SetReturnParameterData(0, &usVariableIndex, sizeof(usVariableIndex));
    SetReturnParameterData(1, &usVariableSubIndex, sizeof(usVariableSubIndex));
    SetReturnParameterData(2, &usVariableSize, sizeof(usVariableSize));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetMaxNbOfSamples()
{
    WORD usMaxNbOfSamples = 0;

    // Return parameter
    SetReturnParameterData(0, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetNbOfRecordedSamples()
{
    WORD usNbOfRecordedSamples = 0;

    // Return parameter
    SetReturnParameterData(0, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_SetNbOfPrecedingSamples()
{
    WORD usNbOfPrecedingSamples = 0;

    // Parameter
    SetParameterData(0, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetNbOfPrecedingSamples()
{
    WORD usNbOfPrecedingSamples = 0;

    // Return parameter
    SetReturnParameterData(0, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetNbOfSamplingVariables()
{
    WORD usNbOfSamplingVariables = 0;

    // Return parameter
    SetReturnParameterData(0, &usNbOfSamplingVariables, sizeof(usNbOfSamplingVariables));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetSamplingPeriod()
{
    WORD usSamplingPeriod = 0;

    // Return parameter
    SetReturnParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_StartRecorder()
{
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_StopRecorder()
{
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ForceTrigger()
{
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_IsRecorderRunning()
{
    BOOL oRunning = FALSE;

    //ReturnParameter
    SetReturnParameterData(0, &oRunning, sizeof(oRunning));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_IsRecorderTriggered()
{
    BOOL oTriggered = FALSE;

    //ReturnParameter
    SetReturnParameterData(0, &oTriggered, sizeof(oTriggered));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetRecorderStatus()
{
    WORD usRecorderStatus = 0;

    // Return parameter
    SetReturnParameterData(0, &usRecorderStatus, sizeof(usRecorderStatus));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_SetRecorderConfiguration()
{
    WORD usRecorderConfiguration = 0;

    // Parameter
    SetParameterData(0, &usRecorderConfiguration, sizeof(usRecorderConfiguration));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetRecorderConfiguration()
{
    WORD usRecorderConfiguration = 0;

    // Return parameter
    SetReturnParameterData(0, &usRecorderConfiguration, sizeof(usRecorderConfiguration));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ReadChannelVectorSize()
{
    DWORD ulVectorSize = 512;

    //ReturnParameter
    SetReturnParameterData(0, &ulVectorSize, sizeof(ulVectorSize));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ReadChannelDataVector()
{
    BYTE ubChannelNumber = 0;
    DWORD ulVectorSize = 0;
    BYTE* pDataVector = NULL;

    //Parameter
    SetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));
    SetParameterData(1, &ulVectorSize, sizeof(ulVectorSize));

    //ReturnParameter
    SetReturnParameterData(0, pDataVector, ulVectorSize);
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_GetVectorStartOffset()
{
    WORD usVectorStartOffset = 0;

    // Return parameter
    SetReturnParameterData(0, &usVectorStartOffset, sizeof(usVectorStartOffset));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ReadDataBuffer()
{
    DWORD ulBufferSizeToRead = 0;
    DWORD ulBufferSizeRead = 0;
    BYTE* pDataBuffer = NULL;
    WORD usVectorStartOffset = 0;
    WORD usMaxNbOfSamples = 0;
    WORD usNbOfRecordedSamples = 0;

    //Parameter
    SetParameterData(0, &ulBufferSizeToRead, sizeof(ulBufferSizeToRead));

    //ReturnParameter
    SetReturnParameterData(0, pDataBuffer, ulBufferSizeToRead);
    SetReturnParameterData(1, &ulBufferSizeRead, ulBufferSizeRead);
    SetReturnParameterData(2, &usVectorStartOffset, sizeof(usVectorStartOffset));
    SetReturnParameterData(3, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
    SetReturnParameterData(4, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));
}

void CCommand_VCS_DataRecorder::SetDefaultParameter_ExtractChannelDataVector()
{
    BYTE ubChannelNumber = 0;
    DWORD ulVectorSize = 0;
    BYTE* pDataVector = NULL;
    DWORD ulBufferSize = 0;
    BYTE* pDataBuffer = NULL;
    WORD usVectorStartOffset = 0;
    WORD usMaxNbOfSamples = 0;
    WORD usNbOfRecordedSamples = 0;

    //Parameter
    SetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));
    SetParameterData(1, &ulBufferSize, sizeof(ulBufferSize));
    SetParameterData(2, &ulVectorSize, sizeof(ulVectorSize));
    SetParameterData(3, &usVectorStartOffset, sizeof(usVectorStartOffset));
    SetParameterData(4, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
    SetParameterData(5, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));

    //ReturnParameter
    SetReturnParameterData(0, pDataBuffer, ulBufferSize);
    SetReturnParameterData(1, pDataVector, ulVectorSize);
}
