
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_ErrorHandling.h>

CCommand_VCS_ErrorHandling::CCommand_VCS_ErrorHandling()
{
}

CCommand_VCS_ErrorHandling::CCommand_VCS_ErrorHandling(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_VCS_ErrorHandling::~CCommand_VCS_ErrorHandling()
{
}

BOOL CCommand_VCS_ErrorHandling::InitCommand(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    ResetCommand();

    if(!oResult && InitCommand_Reset(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_ErrorHistory(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_Safety(p_ulCommandId))
    {
        oResult = TRUE;
    }

    return oResult;
}

CCommand_VCS_ErrorHandling& CCommand_VCS_ErrorHandling::operator=(CCommand_VCS_ErrorHandling& p_Other)
{
    if(this != &p_Other)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&p_Other);
    }

    return *this;
}

CCommandRoot* CCommand_VCS_ErrorHandling::CloneCommand()
{
    CCommand_VCS_ErrorHandling* pNewCommand;

    pNewCommand = new CCommand_VCS_ErrorHandling();
    *pNewCommand = *this;

    return pNewCommand;
}

BOOL CCommand_VCS_ErrorHandling::InitCommand_Reset(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EH_RESET_DEVICE:
            {
                CCommand_VCS::InitCommand("ResetDevice", EH_RESET_DEVICE);
                AddParameter(0, "waitForReboot", ODT_BOOLEAN);
                AddParameter(1, "rebootTimeout", ODT_UINT32);
                SetDefaultParameter_ResetDevice();
                oResult = TRUE; break;
            };
        case EH_WAIT_FOR_REBOOT:
            {
                CCommand_VCS::InitCommand("WaitForReboot", EH_WAIT_FOR_REBOOT);
                AddParameter(0, "rebootTimeout", ODT_UINT32);
                SetDefaultParameter_WaitForReboot();
                oResult = TRUE; break;
            };
    }
    return oResult;
}

BOOL CCommand_VCS_ErrorHandling::InitCommand_ErrorHistory(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EH_CLEAR_DEVICE_ERRORS:
            {
                CCommand_VCS::InitCommand("ClearDeviceErrors", EH_CLEAR_DEVICE_ERRORS);
                SetDefaultParameter_ClearDeviceErrors();
                oResult = TRUE; break;
            };
        case EH_GET_NB_OF_DEVICE_ERRORS:
            {
                CCommand_VCS::InitCommand("GetNbOfDeviceErrors", EH_GET_NB_OF_DEVICE_ERRORS);
                AddReturnParameter(0, "nbOfDeviceError", ODT_UINT8);
                SetDefaultParameter_GetNbOfDeviceErrors();
                oResult = TRUE; break;
            };
        case EH_GET_DEVICE_ERROR_CODE:
            {
                CCommand_VCS::InitCommand("GetDeviceErrorCode", EH_GET_DEVICE_ERROR_CODE);
                AddParameter(0, "errorNumber", ODT_UINT8);
                AddReturnParameter(0, "deviceErrorCode", ODT_UINT32);
                SetDefaultParameter_GetDeviceErrorCode();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_ErrorHandling::InitCommand_Safety(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case EH_GOTO_SAFE_STATE:
            {
                CCommand_VCS::InitCommand("GotoSafeState", EH_GOTO_SAFE_STATE);
                SetDefaultParameter_GotoSafeState();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_ResetDevice()
{
    BOOL oWaitForReboot(FALSE);
    DWORD ulRebootTimeout(0);

    //Parameter
    SetParameterData(0, &oWaitForReboot, sizeof(oWaitForReboot));
    SetParameterData(1, &ulRebootTimeout, sizeof(ulRebootTimeout));
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_WaitForReboot()
{
    DWORD ulRebootTimeout(0);

    //Parameter
    SetParameterData(0, &ulRebootTimeout, sizeof(ulRebootTimeout));
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_ClearDeviceErrors()
{
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_GetNbOfDeviceErrors()
{
    BYTE ubNbDeviceErrors = 0;

    //ReturnParameter
    SetReturnParameterData(0, &ubNbDeviceErrors, sizeof(ubNbDeviceErrors));
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_GetDeviceErrorCode()
{
    BYTE ubErrorIndex = 0;
    DWORD ulDeviceErrorCode = 0;

    //Parameter
    SetParameterData(0, &ubErrorIndex, sizeof(ubErrorIndex));

    //ReturnParameter
    SetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
}

void CCommand_VCS_ErrorHandling::SetDefaultParameter_GotoSafeState()
{
}
