
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>

CCommand_VCS_InputsOutputs::CCommand_VCS_InputsOutputs()
{
}

CCommand_VCS_InputsOutputs::CCommand_VCS_InputsOutputs(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_VCS_InputsOutputs::~CCommand_VCS_InputsOutputs()
{
}

CCommandRoot* CCommand_VCS_InputsOutputs::CloneCommand()
{
    CCommand_VCS_InputsOutputs* pNewCommand;

    pNewCommand = new CCommand_VCS_InputsOutputs();
    *pNewCommand = *this;

    return pNewCommand;
}

CCommand_VCS_InputsOutputs& CCommand_VCS_InputsOutputs::operator=(CCommand_VCS_InputsOutputs& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&p_rOther);
    }

    return *this;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    ResetCommand();

    if(InitCommand_DigitalInputs(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_DigitalOutputs(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_AnalogInputs(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_AnalogOutputs(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_PositionMarker(p_ulCommandId))
    {
        oResult = TRUE;
    }
    else if(InitCommand_PositionCompare(p_ulCommandId))
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_DigitalInputs(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_DIGITAL_INPUT_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("DigitalInputConfiguration", IO_DIGITAL_INPUT_CONFIGURATION);
                AddParameter(0, "inputNb", ODT_UINT16);
                AddParameter(1, "configuration", ODT_UINT16);
                AddParameter(2, "mask", ODT_BOOLEAN);
                AddParameter(3, "polarity", ODT_BOOLEAN);
                AddParameter(4, "executionMask", ODT_BOOLEAN);
                SetDefaultParameter_DigitalInputConfiguration();
                oResult = TRUE; break;
            };
        case IO_GET_ALL_DIGITAL_INPUTS:
            {
                CCommand_VCS::InitCommand("GetAllDigitalInputs", IO_GET_ALL_DIGITAL_INPUTS);
                AddReturnParameter(0, "inputs", ODT_UINT16);
                SetDefaultParameter_GetAllDigitalInputs();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_DigitalOutputs(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_DIGITAL_OUTPUT_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("DigitalOutputConfiguration", IO_DIGITAL_OUTPUT_CONFIGURATION);
                AddParameter(0, "outputNb", ODT_UINT16);
                AddParameter(1, "configuration", ODT_UINT16);
                AddParameter(2, "state", ODT_BOOLEAN);
                AddParameter(3, "mask", ODT_BOOLEAN);
                AddParameter(4, "polarity", ODT_BOOLEAN);
                SetDefaultParameter_DigitalOutputConfiguration();
                oResult = TRUE; break;
            };
        case IO_GET_ALL_DIGITAL_OUTPUTS:
            {
                CCommand_VCS::InitCommand("GetAllDigitalOutputs", IO_GET_ALL_DIGITAL_OUTPUTS);
                AddReturnParameter(0, "outputs", ODT_UINT16);
                SetDefaultParameter_GetAllDigitalOutputs();
                oResult = TRUE; break;
            };
        case IO_SET_ALL_DIGITAL_OUTPUTS:
            {
                CCommand_VCS::InitCommand("SetAllDigitalOutputs", IO_SET_ALL_DIGITAL_OUTPUTS);
                AddParameter(0, "outputs", ODT_UINT16);
                SetDefaultParameter_SetAllDigitalOutputs();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_AnalogInputs(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_ANALOG_INPUT_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("AnalogInputConfiguration", IO_ANALOG_INPUT_CONFIGURATION);
                AddParameter(0, "inputNb", ODT_UINT16);
                AddParameter(1, "configuration", ODT_UINT16);
                AddParameter(2, "executionMask", ODT_BOOLEAN);
                SetDefaultParameter_AnalogInputConfiguration();
                oResult = TRUE; break;
            };
        case IO_GET_ANALOG_INPUT:
            {
                CCommand_VCS::InitCommand("GetAnalogInput", IO_GET_ANALOG_INPUT);
                AddParameter(0, "number", ODT_UINT16);
                AddReturnParameter(0, "analog", ODT_UINT16);
                SetDefaultParameter_GetAnalogInput();
                oResult = TRUE; break;
            };
        case IO_GET_ANALOG_INPUT_VOLTAGE:
            {
                CCommand_VCS::InitCommand("GetAnalogInputVoltage", IO_GET_ANALOG_INPUT_VOLTAGE);
                AddParameter(0, "number", ODT_UINT16);
                AddReturnParameter(0, "voltageValue", ODT_INT32);
                SetDefaultParameter_GetAnalogInputVoltage();
                oResult = TRUE; break;
            };
        case IO_GET_ANALOG_INPUT_STATE:
            {
                CCommand_VCS::InitCommand("GetAnalogInputState", IO_GET_ANALOG_INPUT_STATE);
                AddParameter(0, "configuration", ODT_UINT16);
                AddReturnParameter(0, "stateValue", ODT_INT32);
                SetDefaultParameter_GetAnalogInputState();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_AnalogOutputs(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_ANALOG_OUTPUT_CONFIGURATION:
            {
                CCommand_VCS::InitCommand("AnalogOutputConfiguration", IO_ANALOG_OUTPUT_CONFIGURATION);
                AddParameter(0, "outputNb", ODT_UINT16);
                AddParameter(1, "configuration", ODT_UINT16);
                SetDefaultParameter_AnalogOutputConfiguration();
                oResult = TRUE; break;
            };
        case IO_SET_ANALOG_OUTPUT:
            {
                CCommand_VCS::InitCommand("SetAnalogOutput", IO_SET_ANALOG_OUTPUT);
                AddParameter(0, "number", ODT_UINT16);
                AddParameter(1, "outputValue", ODT_UINT16);
                SetDefaultParameter_SetAnalogOutput();
                oResult = TRUE; break;
            };
        case IO_SET_ANALOG_OUTPUT_VOLTAGE:
            {
                CCommand_VCS::InitCommand("SetAnalogOutputVoltage", IO_SET_ANALOG_OUTPUT_VOLTAGE);
                AddParameter(0, "number", ODT_UINT16);
                AddParameter(1, "voltageValue", ODT_INT32);
                SetDefaultParameter_SetAnalogOutputVoltage();
                oResult = TRUE; break;
            };
        case IO_SET_ANALOG_OUTPUT_STATE:
            {
                CCommand_VCS::InitCommand("SetAnalogOutputState", IO_SET_ANALOG_OUTPUT_STATE);
                AddParameter(0, "configuration", ODT_UINT16);
                AddParameter(1, "stateValue", ODT_INT32);
                SetDefaultParameter_SetAnalogOutput();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_PositionCompare(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_SET_POSITION_COMPARE_PARAMETER:
            {
                CCommand_VCS::InitCommand("SetPositionCompareParameter", IO_SET_POSITION_COMPARE_PARAMETER);
                AddParameter(0, "operationalMode", ODT_UINT8);
                AddParameter(1, "intervalMode", ODT_UINT8);
                AddParameter(2, "directionDependency", ODT_UINT8);
                AddParameter(3, "intervalWidth", ODT_UINT16);
                AddParameter(4, "intervalRepetitions", ODT_UINT16);
                AddParameter(5, "pulseWidth", ODT_UINT16);
                SetDefaultParameter_SetPositionCompareParameter();
                oResult = TRUE; break;
            };
        case IO_GET_POSITION_COMPARE_PARAMETER:
            {
                CCommand_VCS::InitCommand("GetPositionCompareParameter", IO_GET_POSITION_COMPARE_PARAMETER);
                AddReturnParameter(0, "operationalMode", ODT_UINT8);
                AddReturnParameter(1, "intervalMode", ODT_UINT8);
                AddReturnParameter(2, "directionDependency", ODT_UINT8);
                AddReturnParameter(3, "intervalWidth", ODT_UINT16);
                AddReturnParameter(4, "intervalRepetitions", ODT_UINT16);
                AddReturnParameter(5, "pulseWidth", ODT_UINT16);
                SetDefaultParameter_GetPositionCompareParameter();
                oResult = TRUE; break;
            };
        case IO_ACTIVATE_POSITION_COMPARE:
            {
                CCommand_VCS::InitCommand("ActivatePositionCompare", IO_ACTIVATE_POSITION_COMPARE);
                AddParameter(0, "number", ODT_UINT16);
                AddParameter(1, "polarity", ODT_BOOLEAN);
                SetDefaultParameter_ActivatePositionCompare();
                oResult = TRUE; break;
            };
        case IO_DEACTIVATE_POSITION_COMPARE:
            {
                CCommand_VCS::InitCommand("DeactivatePositionCompare", IO_DEACTIVATE_POSITION_COMPARE);
                AddParameter(0, "number", ODT_UINT16);
                SetDefaultParameter_DeactivatePositionCompare();
                oResult = TRUE; break;
            };
        case IO_ENABLE_POSITION_COMPARE:
            {
                CCommand_VCS::InitCommand("EnablePositionCompare", IO_ENABLE_POSITION_COMPARE);
                SetDefaultParameter_EnablePositionCompare();
                oResult = TRUE; break;
            };
        case IO_DISABLE_POSITION_COMPARE:
            {
                CCommand_VCS::InitCommand("DisablePositionCompare", IO_DISABLE_POSITION_COMPARE);
                SetDefaultParameter_DisablePositionCompare();
                oResult = TRUE; break;
            };
        case IO_SET_POSITION_COMPARE_REFERENCE_POSITION:
            {
                CCommand_VCS::InitCommand("SetPositionCompareReferencePosition", IO_SET_POSITION_COMPARE_REFERENCE_POSITION);
                AddParameter(0, "referencePosition", ODT_INT32);
                SetDefaultParameter_SetPositionCompareReferencePosition();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_InputsOutputs::InitCommand_PositionMarker(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case IO_SET_POSITION_MARKER_PARAMETER:
            {
                CCommand_VCS::InitCommand("SetPositionMarkerParameter", IO_SET_POSITION_MARKER_PARAMETER);
                AddParameter(0, "number", ODT_UINT16);
                AddParameter(1, "positionMarkerEdgeType", ODT_UINT8);
                AddParameter(2, "positionMarkerMode", ODT_UINT8);
                SetDefaultParameter_SetPositionMarkerParameter();
                oResult = TRUE; break;
            };
        case IO_GET_POSITION_MARKER_PARAMETER:
            {
                CCommand_VCS::InitCommand("GetPositionMarkerParameter", IO_GET_POSITION_MARKER_PARAMETER);
                AddReturnParameter(0, "positionMarkerEdgeType", ODT_UINT8);
                AddReturnParameter(1, "positionMarkerMode", ODT_UINT8);
                SetDefaultParameter_GetPositionMarkerParameter();
                oResult = TRUE; break;
            };
        case IO_ACTIVATE_POSITION_MARKER:
            {
                CCommand_VCS::InitCommand("ActivatePositionMarker", IO_ACTIVATE_POSITION_MARKER);
                AddParameter(0, "number", ODT_UINT16);
                AddParameter(1, "polarity", ODT_BOOLEAN);
                SetDefaultParameter_ActivatePositionMarker();
                oResult = TRUE; break;
            };
        case IO_DEACTIVATE_POSITION_MARKER:
            {
                CCommand_VCS::InitCommand("DeactivatePositionMarker", IO_DEACTIVATE_POSITION_MARKER);
                AddParameter(0, "number", ODT_UINT16);
                SetDefaultParameter_DeactivatePositionMarker();
                oResult = TRUE; break;
            };
        case IO_READ_POSITION_MARKER_COUNTER:
            {
                CCommand_VCS::InitCommand("ReadPositionMarkerCounter", IO_READ_POSITION_MARKER_COUNTER);
                AddReturnParameter(0, "counter", ODT_UINT16);
                SetDefaultParameter_ReadPositionMarkerCounter();
                oResult = TRUE; break;
            };
        case IO_READ_POSITION_MARKER_CAPTURED_POSITION:
            {
                CCommand_VCS::InitCommand("ReadPositionMarkerCapturedPosition", IO_READ_POSITION_MARKER_CAPTURED_POSITION);
                AddParameter(0, "counterIndex", ODT_UINT16);
                AddReturnParameter(0, "capturedPosition", ODT_INT32);
                SetDefaultParameter_ReadPositionMarkerCapturedPosition();
                oResult = TRUE; break;
            };
        case IO_RESET_POSITION_MARKER_COUNTER:
            {
                CCommand_VCS::InitCommand("ResetPositionMarkerCounter", IO_RESET_POSITION_MARKER_COUNTER);
                SetDefaultParameter_ResetPositionMarkerCounter();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_DigitalInputConfiguration()
{
    WORD usInputNb = 0;
    WORD usConfiguration = 0x0000;
    BOOL oMask = FALSE;
    BOOL oPolarity = FALSE;
    BOOL oExecutionMask = FALSE;

    // Parameter
    SetParameterData(0, &usInputNb, sizeof(usInputNb));
    SetParameterData(1, &usConfiguration, sizeof(usConfiguration));
    SetParameterData(2, &oMask, sizeof(oMask));
    SetParameterData(3, &oPolarity, sizeof(oPolarity));
    SetParameterData(4, &oExecutionMask, sizeof(oExecutionMask));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_DigitalOutputConfiguration()
{
    WORD usOutputNb = 0;
    WORD usConfiguration = 0x0000;
    BOOL oState = FALSE;
    BOOL oMask = FALSE;
    BOOL oPolarity = FALSE;

    // Parameter
    SetParameterData(0, &usOutputNb, sizeof(usOutputNb));
    SetParameterData(1, &usConfiguration, sizeof(usConfiguration));
    SetParameterData(2, &oState, sizeof(oState));
    SetParameterData(3, &oMask, sizeof(oMask));
    SetParameterData(4, &oPolarity, sizeof(oPolarity));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_AnalogInputConfiguration()
{
    WORD usInputNb = 0;
    WORD usConfiguration = 0x0000;
    BOOL oExecutionMask = FALSE;

    // Parameter
    SetParameterData(0, &usInputNb, sizeof(usInputNb));
    SetParameterData(1, &usConfiguration, sizeof(usConfiguration));
    SetParameterData(2, &oExecutionMask, sizeof(oExecutionMask));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_AnalogOutputConfiguration()
{
    WORD usOutputNb = 0;
    WORD usConfiguration = 0x0000;

    // Parameter
    SetParameterData(0, &usOutputNb, sizeof(usOutputNb));
    SetParameterData(1, &usConfiguration, sizeof(usConfiguration));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetAllDigitalInputs()
{
    WORD usInputs = 0x0000;

    // Parameter

    // Return parameter
    SetReturnParameterData(0, &usInputs, sizeof(usInputs));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetAllDigitalOutputs()
{
    WORD usOutputs = 0x0000;

    // Parameter

    // Return parameter
    SetReturnParameterData(0, &usOutputs, sizeof(usOutputs));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetAnalogInput()
{
    WORD usNumber = 0;
    WORD usAnalog = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));

    // Return parameter
    SetReturnParameterData(0, &usAnalog, sizeof(usAnalog));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetAnalogInputVoltage()
{
    WORD usNumber = 0;
    long lVoltageValue = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));

    // Return parameter
    SetReturnParameterData(0, &lVoltageValue, sizeof(lVoltageValue));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetAnalogInputState()
{
    WORD usNumber = 0;
    long lStateValue = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));

    // Return parameter
    SetReturnParameterData(0, &lStateValue, sizeof(lStateValue));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetAllDigitalOutputs()
{
    WORD usOutputs = 0x0000;

    // Parameter
    SetParameterData(0, &usOutputs, sizeof(usOutputs));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetAnalogOutput()
{
    WORD usNumber = 0;
    WORD usOutput = 0x0000;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
    SetParameterData(1, &usOutput, sizeof(usOutput));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetAnalogOutputVoltage()
{
    WORD usNumber = 0;
    long lVoltageValue = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
    SetParameterData(1, &lVoltageValue, sizeof(lVoltageValue));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetAnalogOutputState()
{
    WORD usNumber = 0;
    long lStateValue = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
    SetParameterData(1, &lStateValue, sizeof(lStateValue));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetPositionCompareParameter()
{
    BYTE ubOperationalMode = 0;
    BYTE ubIntervalMode = 0;
    BYTE ubDirectionDependency = 0;
    WORD usIntervalWidth = 0;
    WORD usIntervalRepetitions = 0;
    WORD usPulseWidth = 0;

    // Parameter
    SetParameterData(0, &ubOperationalMode, sizeof(ubOperationalMode));
    SetParameterData(1, &ubIntervalMode, sizeof(ubIntervalMode));
    SetParameterData(2, &ubDirectionDependency, sizeof(ubDirectionDependency));
    SetParameterData(3, &usIntervalWidth, sizeof(usIntervalWidth));
    SetParameterData(4, &usIntervalRepetitions, sizeof(usIntervalRepetitions));
    SetParameterData(5, &usPulseWidth, sizeof(usPulseWidth));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetPositionCompareParameter()
{
    BYTE ubOperationalMode = 0;
    BYTE ubIntervalMode = 0;
    BYTE ubDirectionDependency = 0;
    WORD usIntervalWidth = 0;
    WORD usIntervalRepetitions = 0;
    WORD usPulseWidth = 0;

    // Return parameter
    SetReturnParameterData(0, &ubOperationalMode, sizeof(ubOperationalMode));
    SetReturnParameterData(1, &ubIntervalMode, sizeof(ubIntervalMode));
    SetReturnParameterData(2, &ubDirectionDependency, sizeof(ubDirectionDependency));
    SetReturnParameterData(3, &usIntervalWidth, sizeof(usIntervalWidth));
    SetReturnParameterData(4, &usIntervalRepetitions, sizeof(usIntervalRepetitions));
    SetReturnParameterData(5, &usPulseWidth, sizeof(usPulseWidth));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_ActivatePositionCompare()
{
    WORD usNumber = 0;
    BOOL oPolarity = FALSE;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
    SetParameterData(1, &oPolarity, sizeof(oPolarity));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_DeactivatePositionCompare()
{
    WORD usNumber = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_EnablePositionCompare()
{
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_DisablePositionCompare()
{
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetPositionCompareReferencePosition()
{
    int lReferencePosition = 0;

    // Parameter
    SetParameterData(0, &lReferencePosition, sizeof(lReferencePosition));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_SetPositionMarkerParameter()
{
    BYTE ubPositionMarkerEdgeType = 0;
    BYTE ubPositionMarkerMode = 1;

    // Parameter
    SetParameterData(0, &ubPositionMarkerEdgeType, sizeof(ubPositionMarkerEdgeType));
    SetParameterData(1, &ubPositionMarkerMode, sizeof(ubPositionMarkerMode));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_GetPositionMarkerParameter()
{
    BYTE ubPositionMarkerEdgeType = 0;
    BYTE ubPositionMarkerMode = 1;

    // Return parameter
    SetReturnParameterData(0, &ubPositionMarkerEdgeType, sizeof(ubPositionMarkerEdgeType));
    SetReturnParameterData(1, &ubPositionMarkerMode, sizeof(ubPositionMarkerMode));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_ActivatePositionMarker()
{
    WORD usNumber = 0;
    BOOL oPolarity = FALSE;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
    SetParameterData(1, &oPolarity, sizeof(oPolarity));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_DeactivatePositionMarker()
{
    WORD usNumber = 0;

    // Parameter
    SetParameterData(0, &usNumber, sizeof(usNumber));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_ReadPositionMarkerCounter()
{
    WORD usCount = 0;

    // Return parameter
    SetReturnParameterData(0, &usCount, sizeof(usCount));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_ReadPositionMarkerCapturedPosition()
{
    WORD usCounterIndex = 0;
    int lCapturedPosition = 0;

    // Parameter
    SetParameterData(0, &usCounterIndex, sizeof(usCounterIndex));

    // Return parameter
    SetReturnParameterData(0, &lCapturedPosition, sizeof(lCapturedPosition));
}

void CCommand_VCS_InputsOutputs::SetDefaultParameter_ResetPositionMarkerCounter()
{
}
