
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

CCommand_VCS_Scada::CCommand_VCS_Scada()
{
}

CCommand_VCS_Scada::CCommand_VCS_Scada(DWORD p_ulCommandId)
{
    InitCommand(p_ulCommandId);
}

CCommand_VCS_Scada::~CCommand_VCS_Scada()
{
}

BOOL CCommand_VCS_Scada::InitCommand(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    ResetCommand();

    if(!oResult && InitCommand_Initialisation(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_ProgramStatus(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_ProgramVariableAccess(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_ProcessInputOutputAccess(p_ulCommandId))
    {
        oResult = TRUE;
    }
    if(!oResult && InitCommand_ProcessImageAccess(p_ulCommandId))
    {
        oResult = TRUE;
    }

    return oResult;
}

CCommand_VCS_Scada& CCommand_VCS_Scada::operator=(CCommand_VCS_Scada& p_Other)
{
    if(this != &p_Other)
    {
        *((CCommandRoot*)this) = *((CCommandRoot*)&p_Other);
    }

    return *this;
}

CCommandRoot* CCommand_VCS_Scada::CloneCommand()
{
    CCommand_VCS_Scada* pNewCommand;

    pNewCommand = new CCommand_VCS_Scada();
    *pNewCommand = *this;

    return pNewCommand;
}

BOOL CCommand_VCS_Scada::InitCommand_Initialisation(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case SCADA_SET_BOOTUP_BEHAVIOUR:
            {
                CCommand_VCS::InitCommand("SetBootupBehavior", SCADA_SET_BOOTUP_BEHAVIOUR);
                AddParameter(0, "programControl", ODT_UINT16);
                SetDefaultParameter_SetBootupBehavior();
                oResult = TRUE; break;
            };
        case SCADA_GET_BOOTUP_BEHAVIOUR:
            {
                CCommand_VCS::InitCommand("GetBootupBehavior", SCADA_GET_BOOTUP_BEHAVIOUR);
                AddReturnParameter(0, "programControl", ODT_UINT16);
                SetDefaultParameter_GetBootupBehavior();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_Scada::InitCommand_ProgramStatus(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case SCADA_COLDSTART_PLC:
            {
                CCommand_VCS::InitCommand("ColdstartPlc", SCADA_COLDSTART_PLC);
                SetDefaultParameter_ColdstartPlc();
                oResult = TRUE; break;
            };
        case SCADA_WARMSTART_PLC:
            {
                CCommand_VCS::InitCommand("WarmstartPlc", SCADA_WARMSTART_PLC);
                SetDefaultParameter_WarmstartPlc();
                oResult = TRUE; break;
            };
        case SCADA_HOTSTART_PLC:
            {
                CCommand_VCS::InitCommand("HotstartPlc", SCADA_HOTSTART_PLC);
                SetDefaultParameter_HotstartPlc();
                oResult = TRUE; break;
            };
        case SCADA_STOP_PLC:
            {
                CCommand_VCS::InitCommand("StopPlc", SCADA_STOP_PLC);
                SetDefaultParameter_StopPlc();
                oResult = TRUE; break;
            };
        case SCADA_ERASE_PLC:
            {
                CCommand_VCS::InitCommand("ErasePlc", SCADA_ERASE_PLC);
                SetDefaultParameter_ErasePlc();
                oResult = TRUE; break;
            };
        case SCADA_GET_PLC_STATUS:
            {
                CCommand_VCS::InitCommand("GetPlcStatus", SCADA_GET_PLC_STATUS);
                AddReturnParameter(0, "isProgramRunning", ODT_BOOLEAN);
                AddReturnParameter(1, "isProgramAvailable", ODT_BOOLEAN);
                SetDefaultParameter_GetPlcStatus();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_Scada::InitCommand_ProgramVariableAccess(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case SCADA_INIT_VARIABLE_TABLE:
            {
                CCommand_VCS::InitCommand("InitVariableTable", SCADA_INIT_VARIABLE_TABLE);
                AddParameter(0, "length", ODT_UINT16);
                AddParameter(1, "projectPathName", ODT_MEMORY_BLOCK);
                SetDefaultParameter_InitVariableTable();
                oResult = TRUE; break;
            };
        case SCADA_SET_VARIABLE:
            {
                CCommand_VCS::InitCommand("SetVariable", SCADA_SET_VARIABLE);
                AddParameter(0, "nameLength", ODT_UINT32);
                AddParameter(1, "variableName", ODT_MEMORY_BLOCK);
                AddParameter(2, "nbOfBytesToWrite", ODT_UINT32);
                AddParameter(3, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "nbOfBytesWritten", ODT_UINT32);
                SetDefaultParameter_SetVariable();
                oResult = TRUE; break;
            };
        case SCADA_GET_VARIABLE:
            {
                CCommand_VCS::InitCommand("GetVariable", SCADA_GET_VARIABLE);
                AddParameter(0, "nameLength", ODT_UINT32);
                AddParameter(1, "variableName", ODT_MEMORY_BLOCK);
                AddParameter(2, "nbOfBytesToRead", ODT_UINT32);
                AddReturnParameter(0, "nbOfBytesRead", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_GetVariable();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_Scada::InitCommand_ProcessInputOutputAccess(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case SCADA_SET_PROCESS_INPUT:
            {
                CCommand_VCS::InitCommand("SetProcessInput", SCADA_SET_PROCESS_INPUT);
                AddParameter(0, "processInputType", ODT_UINT16);
                AddParameter(1, "elementNumber", ODT_UINT8);
                AddParameter(2, "nbOfBytesToWrite", ODT_UINT32);
                AddParameter(3, "data", ODT_MEMORY_BLOCK);
                AddReturnParameter(0, "nbOfBytesWritten", ODT_UINT32);
                SetDefaultParameter_SetProcessInput();
                oResult = TRUE; break;
            };
        case SCADA_GET_PROCESS_OUTPUT:
            {
                CCommand_VCS::InitCommand("GetProcessOutput", SCADA_GET_PROCESS_OUTPUT);
                AddParameter(0, "processOutputType", ODT_UINT16);
                AddParameter(1, "elementNumber", ODT_UINT8);
                AddParameter(2, "nbOfBytesToRead", ODT_UINT32);
                AddReturnParameter(0, "nbOfBytesRead", ODT_UINT32);
                AddReturnParameter(1, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_GetProcessOutput();
                oResult = TRUE; break;
            };
        case SCADA_SET_PROCESS_INPUT_BIT:
            {
                CCommand_VCS::InitCommand("SetProcessInputBit", SCADA_SET_PROCESS_INPUT_BIT);
                AddParameter(0, "processInputType", ODT_UINT16);
                AddParameter(1, "elementNumber", ODT_UINT8);
                AddParameter(2, "bitNumber", ODT_UINT8);
                AddParameter(3, "bitState", ODT_UINT8);
                SetDefaultParameter_SetProcessInputBit();
                oResult = TRUE; break;
            };
        case SCADA_GET_PROCESS_OUTPUT_BIT:
            {
                CCommand_VCS::InitCommand("GetProcessOutputBit", SCADA_GET_PROCESS_OUTPUT_BIT);
                AddParameter(0, "processOutputType", ODT_UINT16);
                AddParameter(1, "elementNumber", ODT_UINT8);
                AddParameter(2, "bitNumber", ODT_UINT8);
                AddReturnParameter(0, "bitState", ODT_UINT8);
                SetDefaultParameter_GetProcessOutputBit();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

BOOL CCommand_VCS_Scada::InitCommand_ProcessImageAccess(DWORD p_ulCommandId)
{
    BOOL oResult(FALSE);

    switch(p_ulCommandId)
    {
        case SCADA_SET_PROCESS_INPUT_IMAGE:
            {
                CCommand_VCS::InitCommand("SetProcessInputImage", SCADA_SET_PROCESS_INPUT_IMAGE);
                AddParameter(0, "selectionStart", ODT_UINT32);
                AddParameter(1, "selectionLength", ODT_UINT32);
                AddParameter(2, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_SetProcessInputImage();
                oResult = TRUE; break;
            };
        case SCADA_GET_PROCESS_OUTPUT_IMAGE:
            {
                CCommand_VCS::InitCommand("GetProcessOutputImage", SCADA_GET_PROCESS_OUTPUT_IMAGE);
                AddParameter(0, "selectionStart", ODT_UINT32);
                AddParameter(1, "selectionLength", ODT_UINT32);
                AddReturnParameter(0, "data", ODT_MEMORY_BLOCK);
                SetDefaultParameter_GetProcessOutputImage();
                oResult = TRUE; break;
            };
    }

    return oResult;
}

void CCommand_VCS_Scada::SetDefaultParameter_SetBootupBehavior()
{
    WORD usProgramControl(0);

    //Parameter
    SetParameterData(0, &usProgramControl, sizeof(usProgramControl));
}

void CCommand_VCS_Scada::SetDefaultParameter_GetBootupBehavior()
{
    WORD usProgramControl(0);

    //ReturnParameter
    SetReturnParameterData(0, &usProgramControl, sizeof(usProgramControl));
}

void CCommand_VCS_Scada::SetDefaultParameter_ColdstartPlc()
{
}

void CCommand_VCS_Scada::SetDefaultParameter_WarmstartPlc()
{
}

void CCommand_VCS_Scada::SetDefaultParameter_HotstartPlc()
{
}

void CCommand_VCS_Scada::SetDefaultParameter_StopPlc()
{
}

void CCommand_VCS_Scada::SetDefaultParameter_ErasePlc()
{
}

void CCommand_VCS_Scada::SetDefaultParameter_GetPlcStatus()
{
    BOOL oIsProgramAvailable(FALSE);
    BOOL oIsProgramRunnig(FALSE);

    //ReturnParameter
    SetReturnParameterData(0, &oIsProgramRunnig, sizeof(oIsProgramRunnig));
    SetReturnParameterData(1, &oIsProgramAvailable, sizeof(oIsProgramAvailable));
}

void CCommand_VCS_Scada::SetDefaultParameter_InitVariableTable()
{
    DWORD ulDataLength(0);
    void* pData(NULL);

    //Parameter
    SetParameterData(0, pData, ulDataLength);
}

void CCommand_VCS_Scada::SetDefaultParameter_SetVariable()
{
    CStdString VariableName(_T(""));
    DWORD ulNbOfBytesToWrite(0);
    DWORD ulNbOfBytesWrite(0);
    void* pData(NULL);

    //Parameter
    SetParameterData(0, &VariableName, VariableName.size());
    SetParameterData(1, &ulNbOfBytesToWrite, sizeof(ulNbOfBytesToWrite));
    SetParameterData(2, pData, ulNbOfBytesToWrite);

    //ReturnParameter
    SetReturnParameterData(0, &ulNbOfBytesWrite, sizeof(ulNbOfBytesWrite));
}

void CCommand_VCS_Scada::SetDefaultParameter_GetVariable()
{
    CStdString VariableName(_T(""));
    DWORD ulNbOfBytesRead(0);
    DWORD ulNbOfBytesToRead(0);
    void* pData(NULL);

    //Parameter
    SetParameterData(0, &VariableName, VariableName.size());
    SetParameterData(1, &ulNbOfBytesToRead, sizeof(ulNbOfBytesToRead));

    //ReturnParameter
    SetReturnParameterData(0, &ulNbOfBytesRead, sizeof(ulNbOfBytesRead));
    SetReturnParameterData(1, pData, ulNbOfBytesToRead);
}

void CCommand_VCS_Scada::SetDefaultParameter_SetProcessInput()
{
    BYTE ubElementNumber(0);
    DWORD ulNbOfBytesToWrite(0);
    DWORD ulNbOfBytesWrite(0);
    void* pData(NULL);
    WORD usProcessInputType(0);

    //Parameter
    SetParameterData(0, &usProcessInputType, sizeof(usProcessInputType));
    SetParameterData(1, &ubElementNumber, sizeof(ubElementNumber));
    SetParameterData(2, &ulNbOfBytesToWrite, sizeof(ulNbOfBytesToWrite));
    SetParameterData(3, pData, ulNbOfBytesToWrite);

    //ReturnParameter
    SetReturnParameterData(0, &ulNbOfBytesWrite, sizeof(ulNbOfBytesWrite));
}

void CCommand_VCS_Scada::SetDefaultParameter_GetProcessOutput()
{
    BYTE ubElementNumber(0);
    DWORD ulNbOfBytesRead(0);
    DWORD ulNbOfBytesToRead(0);
    void* pData(NULL);
    WORD usProcessOutputType(0);

    //Parameter
    SetParameterData(0, &usProcessOutputType, sizeof(usProcessOutputType));
    SetParameterData(1, &ubElementNumber, sizeof(ubElementNumber));
    SetParameterData(2, &ulNbOfBytesToRead, sizeof(ulNbOfBytesToRead));

    //ReturnParameter
    SetReturnParameterData(0, &ulNbOfBytesRead, sizeof(ulNbOfBytesRead));
    SetReturnParameterData(1, pData, ulNbOfBytesToRead);
}

void CCommand_VCS_Scada::SetDefaultParameter_SetProcessInputBit()
{
    WORD usProcessInputType(0);
    BYTE ubElementNumber(0);
    BYTE ubBitNumber(0);
    BYTE ubBitState(0);

    //Parameter
    SetParameterData(0, &usProcessInputType, sizeof(usProcessInputType));
    SetParameterData(1, &ubElementNumber, sizeof(ubElementNumber));
    SetParameterData(2, &ubBitNumber, sizeof(ubBitNumber));
    SetParameterData(3, &ubBitState, sizeof(ubBitState));
}

void CCommand_VCS_Scada::SetDefaultParameter_GetProcessOutputBit()
{
    WORD usProcessOutputType(0);
    BYTE ubElementNumber(0);
    BYTE ubBitNumber(0);
    BYTE ubBitState(0);

    //Parameter
    SetParameterData(0, &usProcessOutputType, sizeof(usProcessOutputType));
    SetParameterData(1, &ubElementNumber, sizeof(ubElementNumber));
    SetParameterData(2, &ubBitNumber, sizeof(ubBitNumber));

    //ReturnParameter
    SetReturnParameterData(0, &ubBitState, sizeof(ubBitState));
}

void CCommand_VCS_Scada::SetDefaultParameter_SetProcessInputImage()
{
    DWORD ulSelectionStart(0);
    DWORD ulSelectionLength(0);
    void* pData(NULL);

    //Parameter
    SetParameterData(0, &ulSelectionStart, sizeof(ulSelectionStart));
    SetParameterData(1, &ulSelectionLength, sizeof(ulSelectionLength));
    SetParameterData(2, pData, ulSelectionLength);
}

void CCommand_VCS_Scada::SetDefaultParameter_GetProcessOutputImage()
{
    DWORD ulSelectionStart(0);
    DWORD ulSelectionLength(0);
    void* pData(NULL);

    //Parameter
    SetParameterData(0, &ulSelectionStart, sizeof(ulSelectionStart));
    SetParameterData(1, &ulSelectionLength, sizeof(ulSelectionLength));

    //ReturnParameter
    SetReturnParameterData(0, pData, ulSelectionLength);
}