#include "stdafx.h"
#include <Ui/MmcMessageDialog.h>
#include <MmcTypeDefinition.h>

#include <CommunicationModel/CommonLayer/ErrorHandling/General/ErrorHandlingGeneral.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/Interface/ErrorHandlingInterface.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ProtocolStack/ErrorHandlingProtocolStack.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/DeviceCommandSet/ErrorHandlingDeviceCommandSet.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/VirtualCommandSet/ErrorHandlingVirtualCommandSet.h>

#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CErrorHandling::CErrorHandling()
{
    m_pErrorProducer = 0;
}

CErrorHandling::~CErrorHandling()
{
    DeleteErrorProducer();
}

BOOL CErrorHandling::InitErrorProducer(CErrorProducer *p_pErrorProducer)
{
    BOOL oResult(FALSE);
    DeleteErrorProducer();

    if(p_pErrorProducer)
    {
        m_pErrorProducer = p_pErrorProducer->Clone();

        oResult = TRUE;
    }

    return oResult;
}

BOOL CErrorHandling::GetError(DWORD p_ulErrorCode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pErrorInfo)
    {
        p_pErrorInfo->Init(p_ulErrorCode, m_pErrorProducer);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CErrorHandling::GetErrorProducerInfos(ELayer& p_rLayer, CStdString& p_rClassName, CStdString& p_rCommandName)
{
    BOOL oResult = FALSE;

    if(m_pErrorProducer)
    {
        p_rLayer = m_pErrorProducer->GetLayer();
        p_rClassName = m_pErrorProducer->GetClassName();
        p_rCommandName = m_pErrorProducer->GetCommandName();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CErrorHandling::GetErrorDescription(DWORD p_ulErrorCode, CStdString* p_pDescription)
{
    BOOL oResult(FALSE);

    if(p_pDescription)
    {
        if(CErrorHandlingGeneral::GetErrorDescription(p_ulErrorCode, p_pDescription)
            || CErrorHandlingInterface::GetErrorDescription(p_ulErrorCode, p_pDescription)
            || CErrorHandlingProtocolStack::GetErrorDescription(p_ulErrorCode, p_pDescription)
            || CErrorHandlingDeviceCommandSet::GetErrorDescription(p_ulErrorCode, p_pDescription)
            || CErrorHandlingVirtualCommandSet::GetErrorDescription(p_ulErrorCode, p_pDescription))
        {
            oResult = TRUE;
        }
        else
        {
            *p_pDescription = _T("Unknown Error");
        }
    }

    return oResult;
}

BOOL CErrorHandling::IsDeviceErrorCode(CErrorInfo& p_rErrorInfo)
{
    DWORD ulErrorCode = p_rErrorInfo.GetErrorCode();

    return IsDeviceErrorCode(&ulErrorCode);
}

BOOL CErrorHandling::IsDeviceErrorCode(DWORD* p_pulErrorCode)
{
    BOOL oResult(FALSE);
    CStdString description(_T(""));

    if(p_pulErrorCode)
    {
        if(CErrorHandlingDeviceCommandSet::GetErrorDescription(*p_pulErrorCode, &description))
        {
            oResult = TRUE;
        }
        else if(ERROR_MAXON_SERIAL_V2_BAD_DATA_SIZE_RECEIVED == *p_pulErrorCode)
        {
            //Wrong defined size
            oResult = TRUE;
        }
    }

    return oResult;
}

void CErrorHandling::DeleteErrorProducer()
{
    if(m_pErrorProducer)
    {
        delete m_pErrorProducer;
        m_pErrorProducer = 0;
    }
}



