
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Esam.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEsamDef.h>

#include <CommunicationModel/DeviceCommandSet/Classes/CommandSets/Esam/CommandSetDeviceControl_DCS_Esam.h>

CCommandSetDeviceControl_DCS_Esam::CCommandSetDeviceControl_DCS_Esam()
{
    m_strCommandSetName = COMMAND_SET_DEVICE_CONTROL;

    m_pCommand_Login = NULL;
    m_pCommand_Logout = NULL;
    m_pCommand_SetState = NULL;

    InitCommands();
}

CCommandSetDeviceControl_DCS_Esam::~CCommandSetDeviceControl_DCS_Esam()
{
    DeleteCommands();
}

void CCommandSetDeviceControl_DCS_Esam::InitCommands()
{
    DeleteCommands();

    //Init Command Login
    m_pCommand_Login = new CCommand_DCS_Esam();
    m_pCommand_Login->InitCommand(ESAM_LOGIN);
    //Init Command Logout
    m_pCommand_Logout = new CCommand_DCS_Esam();
    m_pCommand_Logout->InitCommand(ESAM_LOGOUT);
    //Init Command SetState
    m_pCommand_SetState = new CCommand_DCS_Esam();
    m_pCommand_SetState->InitCommand(ESAM_SET_STATE);
}

void CCommandSetDeviceControl_DCS_Esam::DeleteCommands()
{
    if(m_pCommand_Login)
    {
        delete m_pCommand_Login;
        m_pCommand_Login = NULL;
    }
    if(m_pCommand_Logout)
    {
        delete m_pCommand_Logout;
        m_pCommand_Logout = NULL;
    }
    if(m_pCommand_SetState)
    {
        delete m_pCommand_SetState;
        m_pCommand_SetState = NULL;
    }
}

void CCommandSetDeviceControl_DCS_Esam::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_Login) m_pCommand_Login->InitJournalManager(p_pJournalManager);
    if(m_pCommand_Logout) m_pCommand_Logout->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetState) m_pCommand_SetState->InitJournalManager(p_pJournalManager);
}

void CCommandSetDeviceControl_DCS_Esam::ResetJournalManager()
{
    if(m_pCommand_Login) m_pCommand_Login->ResetJournalManager();
    if(m_pCommand_Logout) m_pCommand_Logout->ResetJournalManager();
    if(m_pCommand_SetState) m_pCommand_SetState->ResetJournalManager();
}

BOOL CCommandSetDeviceControl_DCS_Esam::InitGateway(CGateway* p_pGateway)
{
    if(m_pCommand_Login && !m_pCommand_Login->InitGateway(p_pGateway)) return FALSE;
    if(m_pCommand_Logout && !m_pCommand_Logout->InitGateway(p_pGateway)) return FALSE;
    if(m_pCommand_SetState && !m_pCommand_SetState->InitGateway(p_pGateway)) return FALSE;
    return TRUE;
}
CXXMLFile::CElementPart* CCommandSetDeviceControl_DCS_Esam::StoreToXMLFile(CXXMLFile* pFile, CXXMLFile::CElementPart* pParentElement)
{
    CXXMLFile::CElement* pElement;
    BOOL oCheckVisibility = FALSE;

    if(pFile && pParentElement)
    {
        //CommandSet Elements
        pElement = (CXXMLFile::CElement*)pFile->AddElement(pParentElement);
        pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command Elements
        if(m_pCommand_Login && !m_pCommand_Login->StoreToXMLFile(pFile, pElement, oCheckVisibility)) return FALSE;
        if(m_pCommand_Logout && !m_pCommand_Logout->StoreToXMLFile(pFile, pElement, oCheckVisibility)) return FALSE;
        if(m_pCommand_SetState && !m_pCommand_SetState->StoreToXMLFile(pFile, pElement, oCheckVisibility)) return FALSE;

        return pElement;
    }

    return NULL;
}

BOOL CCommandSetDeviceControl_DCS_Esam::DCS_SetState(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uRunCommand, CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;
    BYTE uReturnCode;

    if(m_pCommand_SetState)
    {
        //Set Parameter Data
        m_pCommand_SetState->ResetStatus();
        m_pCommand_SetState->SetParameterData(0, &uRunCommand, sizeof(uRunCommand));

        //Execute command
        oResult = m_pCommand_SetState->Execute(pManager, hHandle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SetState->GetReturnParameterData(0, &uReturnCode, sizeof(uReturnCode));

        //Get ErrorCode
        m_pCommand_SetState->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDeviceControl_DCS_Esam::DCS_Login(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, CErrorInfo* pErrorInfo)
{
    const DWORD TIMEOUT = 1000;
    const WORD MAX_RECEIVE_SIZE = 257;

    BOOL oResult = FALSE;
    WORD wMaxReceiveSize = MAX_RECEIVE_SIZE;
    BYTE uReturnCode;

    if(m_pCommand_Login)
    {
        //Set Parameter Data
        m_pCommand_Login->ResetStatus();
        m_pCommand_Login->SetTimeout(TIMEOUT);
        m_pCommand_Login->SetParameterData(0, &wMaxReceiveSize, sizeof(wMaxReceiveSize));

        //Execute command
        oResult = m_pCommand_Login->Execute(pManager, hHandle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_Login->GetReturnParameterData(0, &uReturnCode, sizeof(uReturnCode));

        //Get ErrorCode
        m_pCommand_Login->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDeviceControl_DCS_Esam::DCS_Logout(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;
    BYTE uReturnCode;

    if(m_pCommand_Logout)
    {
        //Set Parameter Data
        m_pCommand_Logout->ResetStatus();

        //Execute command
        oResult = m_pCommand_Logout->Execute(pManager, hHandle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_Logout->GetReturnParameterData(0, &uReturnCode, sizeof(uReturnCode));

        //Get ErrorCode
        m_pCommand_Logout->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

