
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Esam2.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEsam2Def.h>

#include <CommunicationModel/DeviceCommandSet/Classes/CommandSets/Esam2/CommandSetLayerSettingServices_DCS_Esam2.h>

CCommandSetLayerSettingServices_DCS_Esam2::CCommandSetLayerSettingServices_DCS_Esam2()
{
    m_strCommandSetName = COMMAND_SET_CANOPEN_LAYER_SETTING_SERVICES;

    m_pCommand_ReadLSSFrame = NULL;
    m_pCommand_SendLSSFrame = NULL;

    InitCommands();
}

CCommandSetLayerSettingServices_DCS_Esam2::~CCommandSetLayerSettingServices_DCS_Esam2()
{
    DeleteCommands();
}

BOOL CCommandSetLayerSettingServices_DCS_Esam2::DCS_SendLSSFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, WORD wNetworkId, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SendLSSFrame)
    {
        //Set parameter data
        m_pCommand_SendLSSFrame->ResetStatus();
        m_pCommand_SendLSSFrame->SetParameterData(0, &wNetworkId, sizeof(wNetworkId));
        m_pCommand_SendLSSFrame->SetParameterData(1, pDataBuffer, dDataBufferLength);

        //Execute command
        oResult = m_pCommand_SendLSSFrame->Execute(pManager, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SendLSSFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_SendLSSFrame->GetErrorInfo(pErrorInfo);

    }

    return oResult;
}

BOOL CCommandSetLayerSettingServices_DCS_Esam2::DCS_ReadLSSFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, WORD wNetworkId, WORD wTimeout, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadLSSFrame)
    {
        //Set parameter data
        m_pCommand_ReadLSSFrame->ResetStatus();
        m_pCommand_ReadLSSFrame->SetParameterData(0, &wNetworkId, sizeof(wNetworkId));
        m_pCommand_ReadLSSFrame->SetParameterData(1, &wTimeout, sizeof(wTimeout));

        //Execute command
        oResult = m_pCommand_ReadLSSFrame->Execute(pManager, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_ReadLSSFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_ReadLSSFrame->GetReturnParameterData(1, pDataBuffer, dDataBufferLength);

        //Get error code
        m_pCommand_ReadLSSFrame->GetErrorInfo(pErrorInfo);

    }

    return oResult;
}

void CCommandSetLayerSettingServices_DCS_Esam2::InitCommands()
{
    DeleteCommands();

    //Init Command SendLSSFrame
    m_pCommand_SendLSSFrame = new CCommand_DCS_Esam2();
    m_pCommand_SendLSSFrame->InitCommand(ESAM2_SEND_LSS_FRAME);

    //Init Command ReadLSSFrame
    m_pCommand_ReadLSSFrame = new CCommand_DCS_Esam2();
    m_pCommand_ReadLSSFrame->InitCommand(ESAM2_SEND_LSS_FRAME);
}

void CCommandSetLayerSettingServices_DCS_Esam2::DeleteCommands()
{
    if(m_pCommand_SendLSSFrame)
    {
        delete m_pCommand_SendLSSFrame;
        m_pCommand_SendLSSFrame = NULL;
    }

    if(m_pCommand_ReadLSSFrame)
    {
        delete m_pCommand_ReadLSSFrame;
        m_pCommand_ReadLSSFrame = NULL;
    }
}

void CCommandSetLayerSettingServices_DCS_Esam2::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_SendLSSFrame) m_pCommand_SendLSSFrame->InitJournalManager(p_pJournalManager);
    if(m_pCommand_ReadLSSFrame) m_pCommand_ReadLSSFrame->InitJournalManager(p_pJournalManager);
}

void CCommandSetLayerSettingServices_DCS_Esam2::ResetJournalManager()
{
    if(m_pCommand_SendLSSFrame) m_pCommand_SendLSSFrame->ResetJournalManager();
    if(m_pCommand_ReadLSSFrame) m_pCommand_ReadLSSFrame->ResetJournalManager();
}

BOOL CCommandSetLayerSettingServices_DCS_Esam2::InitGateway(CGateway* p_pGateway)
{
    if(m_pCommand_SendLSSFrame && !m_pCommand_SendLSSFrame->InitGateway(p_pGateway)) return FALSE;
    if(m_pCommand_ReadLSSFrame && !m_pCommand_ReadLSSFrame->InitGateway(p_pGateway)) return FALSE;
    return TRUE;
}

CXXMLFile::CElementPart* CCommandSetLayerSettingServices_DCS_Esam2::StoreToXMLFile(CXXMLFile* pFile, CXXMLFile::CElementPart* pParentElement)
{
    CXXMLFile::CElement* pElement;
    BOOL oCheckVisibility = FALSE;

    if(pFile && pParentElement)
    {
        //CommandSet Elements
        pElement = (CXXMLFile::CElement*)pFile->AddElement(pParentElement);
        pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command Elements
        if(m_pCommand_SendLSSFrame && !m_pCommand_SendLSSFrame->StoreToXMLFile(pFile, pElement, oCheckVisibility)) return FALSE;
        if(m_pCommand_ReadLSSFrame && !m_pCommand_ReadLSSFrame->StoreToXMLFile(pFile, pElement, oCheckVisibility)) return FALSE;

        return pElement;
    }

    return NULL;
}

