﻿#include "stdafx.h"

#include <algorithm>

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/BaseClasses/Command_DCS.h>
#include <CommunicationModel/DeviceCommandSet/Classes/Gateway/BaseClasses/GatewayDCStoPS.h>
#include <CommunicationModel/ProtocolStack/ProtocolStackManager.h>

#include <CommunicationModel/DeviceCommandSet/Classes/Devices/BaseClasses/DeviceBase.h>

using namespace std;

CDeviceBase::CDeviceBase()
{
    m_lInstanceValue = 0;
    m_strDeviceName = "";
    m_pJournalManager = 0;
    m_pGateway = 0;

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CDeviceBase::CDeviceBase(const CDeviceBase& p_rObject)
{
    //Instance value
    m_lInstanceValue = p_rObject.m_lInstanceValue;

    //Copy device name
    m_strDeviceName = p_rObject.m_strDeviceName;

    //Journal manager
    m_pJournalManager = p_rObject.m_pJournalManager;

    //Gateway
    if(p_rObject.m_pGateway)
    {
        m_pGateway = (CGatewayDCStoPS*)p_rObject.m_pGateway->Clone();
    }
    else
    {
        m_pGateway = 0;
    }

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CDeviceBase* CDeviceBase::Clone()
{
    return 0;
}

BOOL CDeviceBase::IsEqual(CDeviceBase* p_pDevice)
{
    BOOL oResult(FALSE);

    if(p_pDevice)
    {
        oResult = TRUE;

        if(oResult && (m_strDeviceName != p_pDevice->m_strDeviceName))
        {
            oResult = FALSE;
        }

        if(oResult && (m_lInstanceValue != p_pDevice->m_lInstanceValue))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

CDeviceBase::~CDeviceBase()
{
    DeleteErrorHandling();
    DeleteGateway();
    DeletePrototcolStackManagerList();
}

BOOL CDeviceBase::InitInstanceValue(int p_lInstanceValue)
{
    BOOL oResult(TRUE);

    m_lInstanceValue = p_lInstanceValue;

    return oResult;
}

BOOL CDeviceBase::InitErrorHandling()
{
    DeleteErrorHandling();
    m_pErrorHandling = new CErrorHandling();
    return TRUE;
}

BOOL CDeviceBase::DeleteErrorHandling()
{
    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = 0;
    }

    return TRUE;
}

BOOL CDeviceBase::InitDevice(CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CDeviceBase::InitDevice(CStdString p_protocolStack, CErrorInfo* p_pErrorInfo)
{
    CProtocolStackManagerBase* pManager(0);
    BOOL oNewCreated(FALSE);
    BOOL oResult(FALSE);

    if(p_protocolStack.IsEmpty())
    {
        //Don't create a protocol stack
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_protocolStack, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CProtocolStackManagerBase*) new CProtocolStackManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create device class structure
        if((VerifyGateway(p_protocolStack)) && (pManager->PS_InitProtocolStack(p_protocolStack, p_pErrorInfo)))
        {
            //Default settings
            CDeviceBase::InitDefaultProtocolStackSettings(pManager, p_protocolStack);

            if(oNewCreated) m_ProtocolStackManagerList.push_back(pManager);
            oResult = TRUE;
        }
        else
        {
            if(oNewCreated) delete pManager;
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitDevice(CStdString p_protocolStack, CStdString p_interfaceName, CErrorInfo* p_pErrorInfo)
{
    CProtocolStackManagerBase* pManager = 0;
    BOOL oNewCreated = FALSE;
    BOOL oResult(FALSE);

    if(p_protocolStack.IsEmpty())
    {
        //Don't create a protocol stack
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_protocolStack, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CProtocolStackManagerBase*) new CProtocolStackManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create device class structure
        if((VerifyGateway(p_protocolStack)) && (pManager->PS_InitProtocolStack(p_protocolStack, p_interfaceName, p_pErrorInfo)))
        {
            //Default settings
            InitDefaultProtocolStackSettings(pManager, p_protocolStack, p_interfaceName);

            if(oNewCreated) m_ProtocolStackManagerList.push_back(pManager);
            oResult = TRUE;
        }
        else
        {
            if(oNewCreated) delete pManager;
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitDevice(CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_driverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oNewCreated = FALSE;
    BOOL oResult(FALSE);
    CProtocolStackManagerBase* pManager = 0;

    if(p_protocolStackName.IsEmpty())
    {
        //Don't create a protocol stack
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_protocolStackName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CProtocolStackManagerBase*) new CProtocolStackManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create device class structure
        if((VerifyGateway(p_protocolStackName)) && (pManager->PS_InitProtocolStackEx(p_protocolStackName, p_interfaceName, p_driverName, p_pErrorInfo)))
        {
            //Default settings
            InitDefaultProtocolStackSettings(pManager, p_protocolStackName, p_interfaceName);

            if(oNewCreated)
            {
                m_ProtocolStackManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitGatewayDevice(CStdString p_protocolStackName, CStdString p_gatewayDeviceName, CStdString p_gatewayProtocolStackName, CErrorInfo* p_pErrorInfo)
{
    BOOL oNewCreated = FALSE;
    BOOL oResult(FALSE);
    CProtocolStackManagerBase* pManager = 0;

    if(p_protocolStackName.IsEmpty())
    {
        //Don't create a protocol stack
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_protocolStackName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CProtocolStackManagerBase*) new CProtocolStackManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create device class structure
        if((VerifyGateway(p_protocolStackName)) && (pManager->PS_InitProtocolStack(p_protocolStackName, p_gatewayDeviceName, p_gatewayProtocolStackName, p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_ProtocolStackManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitGateway(CStdString p_protocolStackName)
{
    return FALSE;
}

BOOL CDeviceBase::InitBaudrateSelection(CStdString p_protocolStack, CStdString p_interfaceName, CStdDWordArray& p_ulBaudrateSel)
{
    BOOL oResult(TRUE);

    for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
    {
        CProtocolStackManagerBase* pProtocolStackManager = (*it);
        if(pProtocolStackManager)
        {
            if(!pProtocolStackManager->PS_InitBaudrateSelection(p_protocolStack, p_interfaceName, p_ulBaudrateSel))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitDefaultProtocolStackSettings(CStdString p_protocolStack, CStdString p_interfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
    BOOL oResult(TRUE);

    for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
    {
        CProtocolStackManagerBase* pProtocolStackManager = (*it);
        if(pProtocolStackManager)
        {
            if(!pProtocolStackManager->PS_InitDefaultProtocolStackSettings(p_protocolStack, p_interfaceName, p_ulBaudrate, p_ulTimeout))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CDeviceBase::VerifyGateway(CStdString p_protocolStackName)
{
    return FALSE;
}

BOOL CDeviceBase::DCS_GetDeviceName(CStdString* p_pDeviceName)
{
    BOOL oResult(FALSE);

    if(p_pDeviceName)
    {
        *p_pDeviceName = m_strDeviceName;
        oResult = TRUE;
    }

    return oResult;
}

void CDeviceBase::DeletePrototcolStackManagerList()
{
    for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
    {
        CProtocolStackManagerBase* pProtocolStackManager = (*it);
        delete pProtocolStackManager;
        pProtocolStackManager = 0;
    }

    m_ProtocolStackManagerList.clear();
}

void CDeviceBase::DeleteGateway()
{
    if(m_pGateway)
    {
        delete m_pGateway;
        m_pGateway = 0;
    }
}

BOOL CDeviceBase::AddManager(CProtocolStackManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_ProtocolStackManagerList.push_back(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CDeviceBase::RemoveManager(CProtocolStackManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_ProtocolStackManagerList.remove(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CDeviceBase::GetFirstManager(CProtocolStackManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin();
    if(it != m_ProtocolStackManagerList.end())
    {
        CProtocolStackManagerBase* pListManager = (*it);
        if(pListManager)
        {
            if(p_ppManager)
            {
                *p_ppManager = pListManager;
            }

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CDeviceBase::FindCorrectManager(CStdString p_protocolStackName, CProtocolStackManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    if(p_ppManager)
    {
        for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
        {
            CProtocolStackManagerBase* pListManager = (*it);
            if(pListManager)
            {
                if(pListManager->PS_IsProtocolStackNameSupported(p_protocolStackName))
                {
                    *p_ppManager = pListManager;

                    oResult = TRUE;
                    break;
                }
            }
        }

        if(!oResult)
        {
            oResult = FindCorrectLibrary(p_protocolStackName, p_ppManager);
        }
    }

    return oResult;
}

BOOL CDeviceBase::FindCorrectManager(HANDLE p_hPS_BaseLayerHandle, CProtocolStackManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);
    CStdString protocolStackName = _T("");

    if(p_ppManager)
    {
        for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
        {
            CProtocolStackManagerBase* pListManager = (*it);
            if(pListManager)
            {
                if(pListManager->PS_GetProtocolStackName(p_hPS_BaseLayerHandle, &protocolStackName))
                {
                    if(pListManager->PS_IsProtocolStackNameSupported(protocolStackName))
                    {
                        *p_ppManager = pListManager;
                        oResult = TRUE;
                        break;
                    }
                }
            }
        }

        if(!oResult)
        {
            oResult = FindCorrectLibrary(p_hPS_BaseLayerHandle, p_ppManager);
        }
    }

    return oResult;
}

BOOL CDeviceBase::FindCorrectLibrary(CStdString p_protocolStackName, CProtocolStackManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    if(p_ppManager)
    {
        *p_ppManager = 0;
    }

    return oResult;
}

BOOL CDeviceBase::FindCorrectLibrary(HANDLE p_hPS_BaseLayerHandle, CProtocolStackManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    if(p_ppManager)
    {
        *p_ppManager = 0;
    }

    return oResult;
}

BOOL CDeviceBase::DCS_GetProtocolStackNameSelection(CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo)
{
    CStdStringArray selArray;

    if(p_pProtocolStackNameSel)
    {
        p_pProtocolStackNameSel->clear();

        for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
        {
            CProtocolStackManagerBase* pManager = (*it);
            if(pManager)
            {
                if(pManager->PS_GetProtocolStackNameSelection(&selArray))
                {
                    AppendSelectionArray(p_pProtocolStackNameSel, &selArray);
                }
            }
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    return FALSE;
}

BOOL CDeviceBase::AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray)
{
    BOOL oFound;
    BOOL oResult(FALSE);
    CStdString strDestName;
    CStdString strSourceName;

    if(p_pDestArray && p_pSourceArray)
    {
        for(size_t i = 0; i < p_pSourceArray->size(); i++)
        {
            oFound = FALSE;
            strSourceName = p_pSourceArray->at(i);
            for(size_t j = 0; j < p_pDestArray->size(); j++)
            {
                strDestName = p_pDestArray->at(j);

                if(strSourceName.CompareNoCase(strDestName) == 0)
                {
                    oFound = TRUE;
                    break;
                }
            }

            if(!oFound)
            {
                p_pDestArray->push_back(strSourceName);
            }
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CDeviceBase::DCS_GetInterfaceNameSelection(CStdString p_protocolStackName, CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CProtocolStackManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_protocolStackName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PROTOCOL_STACK_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get interface name selection
    if(oResult && !pManager->PS_GetInterfaceNameSelection(p_protocolStackName, p_pInterfaceNameSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return TRUE;
}

BOOL CDeviceBase::DCS_GetPortNameSelection(CStdString p_protocolStackName, CStdString p_interfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CProtocolStackManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_protocolStackName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PROTOCOL_STACK_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get port name selection
    if(oResult && !pManager->PS_GetPortNameSelection(p_protocolStackName, p_interfaceName, p_pPortSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CDeviceBase::DCS_GetBaudrateSelection(CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CProtocolStackManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_protocolStackName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PROTOCOL_STACK_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get baudrate selection
    if(oResult && !pManager->PS_GetBaudrateSelection(p_protocolStackName, p_interfaceName, p_portName, p_pulBaudrateSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CDeviceBase::DCS_ResetPortEnumeration(CStdString p_protocolStackName, CStdString p_interfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CProtocolStackManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_protocolStackName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PROTOCOL_STACK_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Enumeration
    if(oResult && !pManager->PS_ResetPortEnumeration(p_protocolStackName, p_interfaceName, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

void CDeviceBase::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;

    for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
    {
        CProtocolStackManagerBase* pManager = (*it);
        pManager->InitJournalManager(p_pJournalManager);
    }
}

void CDeviceBase::ResetJournalManager()
{
    m_pJournalManager = 0;

    for(list<CProtocolStackManagerBase*>::iterator it = m_ProtocolStackManagerList.begin(); it != m_ProtocolStackManagerList.end(); it++)
    {
        CProtocolStackManagerBase* pManager = (*it);
        pManager->ResetJournalManager();
    }
}

BOOL CDeviceBase::ExecuteCommand(CCommandRoot* p_pCommand, CProtocolStackManagerBase* p_pProtocolStackManager, HANDLE p_hPS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(p_pCommand)
    {
        //Init references
        p_pCommand->InitGateway(m_pGateway);
        p_pCommand->InitJournalManager(m_pJournalManager);

        //Execute
        oResult = p_pCommand->Execute(p_pProtocolStackManager, p_hPS_Handle, p_hTransactionHandle);

        //Reset references
        p_pCommand->ResetGateway();
        p_pCommand->ResetJournalManager();
    }

    return oResult;
}

BOOL CDeviceBase::AbortCommands(BOOL p_oActive)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->AbortCommands(p_oActive);
    }

    return oResult;
}

BOOL CDeviceBase::GetCommands(CStdString* p_pCommandInfo)
{
    return FALSE;
}

BOOL CDeviceBase::AreParameterEqual(CDeviceBase* p_pDevice)
{
    BOOL oResult(FALSE);

    if(p_pDevice && m_pGateway)
    {
        oResult = m_pGateway->AreParameterEqual(p_pDevice->m_pGateway);
    }

    return oResult;
}

BOOL CDeviceBase::SetParameter(CStdString p_name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CDeviceBase::SetParameter(CStdString p_Name, CStdString p_Value)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_Name, p_Value);
    }

    return oResult;
}

BOOL CDeviceBase::GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CDeviceBase::GetParameter(CStdString p_Name, CStdString& p_rValue)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_rValue);
    }

    return oResult;
}

BOOL CDeviceBase::Lock(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Lock(p_ulTimeout);
    }

    return oResult;
}

BOOL CDeviceBase::Unlock()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Unlock();
    }

    return oResult;
}

BOOL CDeviceBase::IsLocked()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->IsLocked();
    }

    return oResult;
}

BOOL CDeviceBase::InitDefaultProtocolStackSettings(CProtocolStackManagerBase* p_pProtocolStackManager, CStdString p_ProtocolStackName)
{
    BOOL oResult(FALSE);
    CStdString interfaceName(_T(""));
    CStdStringArray aInterfaceNameSel;

    if(p_pProtocolStackManager)
    {
        if(p_pProtocolStackManager->PS_GetInterfaceNameSelection(p_ProtocolStackName, &aInterfaceNameSel))
        {
            //Init
            oResult = TRUE;
            for(CStdStringArray::iterator it = aInterfaceNameSel.begin(); it != aInterfaceNameSel.end(); it++)
            {
                interfaceName = (*it);
                if(!InitDefaultProtocolStackSettings(p_pProtocolStackManager, p_ProtocolStackName, interfaceName))
                {
                    oResult = FALSE;
                }
            }
        }
    }

    return oResult;
}

BOOL CDeviceBase::InitDefaultProtocolStackSettings(CProtocolStackManagerBase* p_pProtocolStackManager, CStdString p_ProtocolStackName, CStdString p_InterfaceName)
{
    //No initialization
    return FALSE;
}
