#include "stdafx.h"
#include <algorithm>

#include <CommunicationModel/DeviceCommandSet/Classes/CommandGroups/Epos4/CommandGroupStandard_DCS_Epos4.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos4Def.h>
#include <CommunicationModel/DeviceCommandSet/Classes/Devices/Factory/DeviceEpos4Factory.h>
#include <CommunicationModel/ProtocolStack/ProtocolStackManager.h>

#include <CommunicationModel/DeviceCommandSet/Classes/Devices/Device_Epos4.h>

using namespace std;

CDevice_Epos4::CDevice_Epos4()
{
    InitErrorHandling();

    FillGroupList();

    m_strDeviceName = DEVICE_EPOS4;
    m_pJournalManager = 0;
}

CDevice_Epos4::CDevice_Epos4(const CDevice_Epos4& p_rObject):CDeviceBase(p_rObject)
{
    InitErrorHandling();

    FillGroupList();
    m_strDeviceName = p_rObject.m_strDeviceName;

    InitJournalManager(p_rObject.m_pJournalManager);
    InitGroupList(m_pGateway);
}

CDeviceBase* CDevice_Epos4::Clone()
{
    CDevice_Epos4* pDevice;

    pDevice = new CDevice_Epos4(*this);

    return pDevice;
}

CDevice_Epos4::~CDevice_Epos4()
{
    DeleteGroupList();
}

BOOL CDevice_Epos4::InitDevice(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    list<CStdString> aIdentifiers;

    oResult = CDeviceEpos4Factory::GetAllIdentifiers(aIdentifiers);

    if(oResult)
    {
        for(list<CStdString>::iterator it = aIdentifiers.begin(); it != aIdentifiers.end() && oResult; it++)
        {
             if(!CDeviceBase::InitDevice((*it), p_pErrorInfo))
             {
                oResult = FALSE;
             }
        }
    }

    return oResult;
}

BOOL CDevice_Epos4::InitDefaultProtocolStackSettings(CProtocolStackManagerBase* p_pProtocolStackManager, CStdString p_protocolStackName, CStdString p_interfaceName)
{
    BOOL oResult(FALSE);

    if(p_pProtocolStackManager)
    {
        if(p_protocolStackName == PROTOCOL_CAN_OPEN)
        {
            oResult = p_pProtocolStackManager->PS_InitDefaultProtocolStackSettings(p_protocolStackName, p_interfaceName, DEFAULT_TRANSFER_RATE_CAN_OPEN, DEFAULT_TIMEOUT);
        }
        else if(p_interfaceName == INTERFACE_RS232)
        {
            oResult = p_pProtocolStackManager->PS_InitDefaultProtocolStackSettings(p_protocolStackName, p_interfaceName, DEFAULT_TRANSFER_RATE_RS232, DEFAULT_TIMEOUT);
        }
        else if(p_interfaceName == INTERFACE_USB)
        {
            oResult = p_pProtocolStackManager->PS_InitDefaultProtocolStackSettings(p_protocolStackName, p_interfaceName, DEFAULT_TRANSFER_RATE_USB, DEFAULT_TIMEOUT);
        }
        else if(p_protocolStackName == PROTOCOL_ETHERCAT_COE)
        {
            oResult = p_pProtocolStackManager->PS_InitDefaultProtocolStackSettings(p_protocolStackName, p_interfaceName, DEFAULT_TRANSFER_RATE_ETHERCAT, DEFAULT_TIMEOUT);
        }
    }

    return oResult;
}

void CDevice_Epos4::FillGroupList()
{
    m_pCommandGroupStandard = new CCommandGroupStandard_DCS_Epos4();
}

BOOL CDevice_Epos4::InitGateway(CStdString p_protocolStackName)
{
    DeleteGateway();

    BOOL oResult = CDeviceEpos4Factory::CreateInstance(p_protocolStackName, m_pGateway);

    if(oResult)
    {
        if(m_pGateway->InitGateway())
        {
            InitParameterSet();
            InitGroupList(m_pGateway);
        }
        else
        {
            DeleteGateway();

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CDevice_Epos4::InitParameterSet()
{
    const BYTE DEFAULT_NODE_ID(1);
    const BYTE DEFAULT_GATEWAY_NODE_ID(0);
    const WORD DEFAULT_STATION_ID(0);
    const WORD DEFAULT_POSITION_ADDR(0);

    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->ResetParameterSet();

        if (oResult)
        {
            oResult = m_pGateway->AddParameter(_T("NodeId"), (BYTE*)&DEFAULT_NODE_ID, sizeof(DEFAULT_NODE_ID));
        }

        if (oResult)
        {
            oResult = m_pGateway->AddParameter(_T("GatewayNodeId"), (BYTE*)&DEFAULT_GATEWAY_NODE_ID, sizeof(DEFAULT_GATEWAY_NODE_ID));
        }

        if(oResult)
        {
            oResult = m_pGateway->AddParameter(_T("StationId"), (BYTE*)&DEFAULT_STATION_ID, sizeof(DEFAULT_STATION_ID));
        }

        if(oResult)
        {
            oResult = m_pGateway->AddParameter(_T("PositionAddr"), (BYTE*)&DEFAULT_POSITION_ADDR, sizeof(DEFAULT_POSITION_ADDR));
        }
    }

    return oResult;
}

BOOL CDevice_Epos4::InitGroupList(CGatewayDCStoPS* p_pGateway)
{
    BOOL oResult(FALSE);

    if(m_pCommandGroupStandard)
    {
        if(m_pCommandGroupStandard->InitGateway(p_pGateway))
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CDevice_Epos4::VerifyGateway(CStdString p_protocolStackName)
{
    list<CStdString> aIdentifiers;

    BOOL oResult = CDeviceEpos4Factory::GetAllIdentifiers(aIdentifiers);

    if(oResult)
    {
        oResult = FALSE;

        for(list<CStdString>::iterator it = aIdentifiers.begin(); it != aIdentifiers.end(); it++)
        {
            if(p_protocolStackName.CompareNoCase((*it)) == 0)
            {
                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

void CDevice_Epos4::DeleteGroupList()
{
    if(m_pCommandGroupStandard)
    {
        delete m_pCommandGroupStandard;
        m_pCommandGroupStandard = 0;
    }
}

void CDevice_Epos4::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    CDeviceBase::InitJournalManager(p_pJournalManager);

    if(m_pCommandGroupStandard)
    {
        m_pCommandGroupStandard->InitJournalManager(p_pJournalManager);
    }
}

void CDevice_Epos4::ResetJournalManager()
{
    CDeviceBase::ResetJournalManager();

    if(m_pCommandGroupStandard)
    {
        m_pCommandGroupStandard->ResetJournalManager();
    }
}

BOOL CDevice_Epos4::GetCommands(CStdString* p_pCommandInfo)
{
    CXXMLFile xmlFile;
    CXXMLFile::CElementPart* pElementPart;
    CXXMLFile::CElement* pElement;
    BOOL oResult(FALSE);

    if(p_pCommandInfo)
    {
        //Init
        oResult = TRUE;

        //Root
        pElementPart = xmlFile.Root();
        if(xmlFile.IsElement(pElementPart))
        {
            //CommandInfo elements
            pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElementPart);
            xmlFile.SetText(pElement, "CommandInfo");

            //CommandGroup elements
            if(m_pCommandGroupStandard)
            {
                if(!m_pCommandGroupStandard->StoreToXMLFile(&xmlFile, pElement))
                {
                    oResult = FALSE;
                }
            }

            //Write to string
            xmlFile.WriteToString(p_pCommandInfo);
        }
    }

    return oResult;
}

BOOL CDevice_Epos4::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("Device_Epos4"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(DEVICE_COMMAND_SET_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}
