#include "stdafx.h"
#include <stdio.h>
#include <algorithm>

#include <CommunicationModel/DeviceCommandSet/Classes/Devices/BaseClasses/DeviceBase.h>
#include <CommunicationModel/ProtocolStack/BaseClasses/ProtocolStackManagerBase.h>
#include <Thread/MmcCriticalSection.h>
#include <Thread/MmcCriticalSectionEx.h>

#include <CommunicationModel/DeviceCommandSet/Classes/Handle/HandleRegistration_DCS.h>

CHandleRegistration_DCS::CHandleRegistration_DCS()
{
    m_Handle_DCS = 0;
    m_pDevice = 0;
    m_pManager = 0;
    m_Handle_PS = 0;
    m_pCriticalSection = 0;

    CreateCriticalSection();
}

CHandleRegistration_DCS::~CHandleRegistration_DCS()
{
    if(m_pDevice)
    {
        delete m_pDevice;
    }

    DeleteCriticalSection();
}

BOOL CHandleRegistration_DCS::CreateCriticalSection()
{
    BOOL oResult = FALSE;

    if(!m_pCriticalSection)
    {
        m_pCriticalSection = new CMmcCriticalSectionEx();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CHandleRegistration_DCS::DeleteCriticalSection()
{
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        delete m_pCriticalSection;
        m_pCriticalSection = 0;
        oResult = TRUE;
    }

    return oResult;
}

HANDLE CHandleRegistration_DCS::GetKeyHandle()
{
    return m_Handle_DCS;
}

HANDLE CHandleRegistration_DCS::GetProtocolStackHandle()
{
    return m_Handle_PS;
}

BOOL CHandleRegistration_DCS::GetRegistrationValues(CDeviceBase** p_ppDevice, CProtocolStackManagerBase** p_ppManager, HANDLE* p_phProtocolStackHandle)
{
    if(p_ppDevice)
    {
        *p_ppDevice = m_pDevice;
    }

    if(p_ppManager)
    {
        *p_ppManager = m_pManager;
    }

    if(p_phProtocolStackHandle)
    {
        *p_phProtocolStackHandle = m_Handle_PS;
    }

    return TRUE;
}

BOOL CHandleRegistration_DCS::IsEmpty()
{
    return (!m_pDevice || !m_pManager || !m_Handle_PS);
}

void CHandleRegistration_DCS::SetKeyHandle(HANDLE p_hDeviceCommandSetHandle)
{
    m_Handle_DCS = p_hDeviceCommandSetHandle;
}

void CHandleRegistration_DCS::Init(CDeviceBase* p_pDevice, CProtocolStackManagerBase* p_pManager, HANDLE p_hProtocolStackHandle)
{
    if(p_pDevice)
    {
        m_pDevice = p_pDevice->Clone();
    }

    m_pManager = p_pManager;
    m_Handle_PS = p_hProtocolStackHandle;
}

void CHandleRegistration_DCS::DeleteRegistration()
{
    m_Handle_DCS = 0;

    if(m_pDevice)
    {
        delete m_pDevice;
    }

    m_pDevice = 0;
    m_pManager = 0;
    m_Handle_PS = 0;
}

BOOL CHandleRegistration_DCS::IsEqual(CDeviceBase* p_pDevice, CProtocolStackManagerBase* p_pManager, HANDLE p_hProtocolStackHandle)
{
    CStdString protocolStackName = _T("");
    CStdString protocolStackName2 = _T("");
    BOOL oResult = FALSE;

    if(m_pDevice && p_pDevice && m_pManager && p_pManager)
    {
        //Init
        oResult = TRUE;

        //Device
        if(oResult && !m_pDevice->IsEqual(p_pDevice))
        {
            oResult = FALSE;
        }

        //ProtocolStack
        m_pManager->PS_GetProtocolStackName(m_Handle_PS, &protocolStackName);
        p_pManager->PS_GetProtocolStackName(p_hProtocolStackHandle, &protocolStackName2);

        if(oResult && protocolStackName.CompareNoCase(protocolStackName2) != 0)
        {
            oResult = FALSE;
        }

        if(oResult && (m_Handle_PS != p_hProtocolStackHandle))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CHandleRegistration_DCS::DoLock(DWORD p_ulTimeout)
{
    CStdString traceString = _T("");
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        //TraceString
        GetTraceString(traceString);

        //Lock
        oResult = m_pCriticalSection->Lock(p_ulTimeout, traceString);
    }

    return oResult;
}

BOOL CHandleRegistration_DCS::DoUnlock()
{
    CStdString traceString = _T("");
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        //TraceString
        GetTraceString(traceString);

        //Unlock
        oResult = m_pCriticalSection->Unlock(traceString);
    }

    return oResult;
}

BOOL CHandleRegistration_DCS::IsLocked()
{
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        oResult = m_pCriticalSection->IsLocked();
    }

    return oResult;
}

bool CHandleRegistration_DCS::AbortLock(bool p_oActive)
{
    bool oResult = false;

    if(m_pCriticalSection)
    {
        oResult = m_pCriticalSection->AbortLock(p_oActive);
    }

    return oResult;
}

BOOL CHandleRegistration_DCS::GetTraceString(CStdString& p_rTraceString)
{
    BOOL oResult = TRUE;

    #ifdef _Trace_CommunicationModelLock
        p_rTraceString.Format(_T("HandleRegistration DCS 0x%.8x"), (unsigned int)m_Handle_DCS);
    #endif

    return oResult;
}
