#include "stdafx.h"

#include <Drv/SocketCAN/SocketCANDeviceInfo.h>
#include <Thread/MmcCriticalSection.h>

#include <CommunicationModel/Interface/Classes/Gateway/CAN/Classes/CANDeviceInfoHandling.h>

CMmcCriticalSection CCANDeviceInfoHandling::m_CriticalSection;

CCANDeviceInfoHandling::CCANDeviceInfoHandling(void):
    m_lInstanceValue(0)
{
}

CCANDeviceInfoHandling::CCANDeviceInfoHandling(int p_lInstanceValue):
    CDeviceInfoHandlingBase(p_lInstanceValue)
{
}

CCANDeviceInfoHandling::~CCANDeviceInfoHandling(void)
{
    DeleteDeviceInfoList(m_DeviceInfoList);
}

CMmcCriticalSection* CCANDeviceInfoHandling::GetSyncInstance()
{
    return &m_CriticalSection;
}

BOOL CCANDeviceInfoHandling::GetInterfaceNames(CStdStringArray& p_rNames)
{
    BOOL oResult = TRUE;

    //Reset
    p_rNames.clear();
    //Add names
    for(tSocketCANDeviceInfoList::iterator it = m_DeviceInfoList.begin(); it != m_DeviceInfoList.end(); it++)
    {
        CSocketCANDeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo != 0)
        {
            p_rNames.push_back(pDeviceInfo->GetInterfaceName());
        }
    }

    return oResult;
}

BOOL CCANDeviceInfoHandling::GetPortNames(CStdString p_InterfaceName, CStdStringArray& p_rNames)
{
    BOOL oResult(FALSE);
    CSocketCANDeviceInfo* pDeviceInfo(0);

    pDeviceInfo = FindDeviceInfo(p_InterfaceName);
    if(pDeviceInfo)
    {
        p_rNames = pDeviceInfo->GetPortNames();
        oResult = TRUE;
    }

    return oResult;
}

CSocketCANDeviceInfo* CCANDeviceInfoHandling::FindDeviceInfo(CStdString p_rInterfaceName)
{
    CSocketCANDeviceInfo* pResultDeviceInfo(0);

    for(tSocketCANDeviceInfoList::iterator it = m_DeviceInfoList.begin(); it != m_DeviceInfoList.end(); it++)
    {
        CSocketCANDeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo != 0)
        {
            //Check interface name
            if(p_rInterfaceName == pDeviceInfo->GetInterfaceName())
            {
                pResultDeviceInfo = pDeviceInfo;
                break;
            }
        }
    }

    return pResultDeviceInfo;
}

BOOL CCANDeviceInfoHandling::InitDeviceInfos(tSocketCANDeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult = TRUE;

    //Reset
    DeleteDeviceInfoList(m_DeviceInfoList);

    //Copy device infos
    for(tSocketCANDeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CSocketCANDeviceInfo* pDeviceInfo = ((CSocketCANDeviceInfo*)(*it))->Clone();
        if(pDeviceInfo)
        {
            m_DeviceInfoList.push_back(pDeviceInfo);
        }
    }

    return oResult;
}

BOOL CCANDeviceInfoHandling::DeleteDeviceInfoList(tSocketCANDeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult = TRUE;

    for(tSocketCANDeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CSocketCANDeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo != 0) delete pDeviceInfo;
    }

    p_rDeviceInfoList.clear();
    return oResult;
}
