#include "stdafx.h"
#include <Thread/MmcSingleLock.h>
#include <Time/MmcHiResTimer.h>
#include <Log/MmcLog/MmcLog.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/Command_I_CAN.h>
#include <CommunicationModel/Interface/BaseClasses/InterfaceManagerBase.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/Classes/CANDeviceInfoHandling.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/GatewayCANToSocketCAN.h>

CGatewayCANToSocketCAN::CGatewayCANToSocketCAN():
    m_pInfoHandling(0)
{
    InitErrorHandling();
    InitBaudrateSelection();
    InitDefaultPortSettings();
}

CGatewayCANToSocketCAN::~CGatewayCANToSocketCAN()
{
}

BOOL CGatewayCANToSocketCAN::InitBaudrateSelection()
{
    m_BaudrateSelection.clear();
    m_BaudrateSelection.push_back(1000000);
    m_BaudrateSelection.push_back(800000);
    m_BaudrateSelection.push_back(500000);
    m_BaudrateSelection.push_back(250000);
    m_BaudrateSelection.push_back(125000);
    m_BaudrateSelection.push_back(50000);
    m_BaudrateSelection.push_back(20000);

    return TRUE;
}

BOOL CGatewayCANToSocketCAN::InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CCANDeviceInfoHandling*>(p_pInfoHandling);

    return oResult;
}

BOOL CGatewayCANToSocketCAN::InitDefaultPortSettings()
{
    m_dDefaultBaudrate = TRANSFER_RATE_DEFAULT_CAN_OPEN;
    m_dDefaultTimeout = TIMEOUT_DEFAULT_CAN_OPEN;

    GetDefaultPortSettings(&m_dBaudrate, &m_dTimeout);

    m_CANHndl.SetPortSettings(m_dBaudrate, m_dTimeout);

    return TRUE;
}

BOOL CGatewayCANToSocketCAN::IsPortNameSupported(CStdString p_portName)
{
    BOOL oResult(FALSE);
    CStdString strName;

    //Prepare port name
    if(p_portName.Find(_T("CAN")) == 0) p_portName = p_portName.Right((int)(_tcslen(p_portName) - _tcslen(_T("CAN"))));
    p_portName = "CAN" + p_portName;

    for(CStdStringArray::iterator it = m_strPortArray.begin(); it != m_strPortArray.end(); it++)
    {
        strName = (*it);
        if(strName.CompareNoCase(p_portName) == 0)
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CCommand_I* pCommand_I;
    
    if(CGateway::ProcessCommand(p_pCommand, p_pManager, p_h_Handle, p_hTransactionHandle))
    {
        if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayCANToSocketCAN")));
        }

        if(CheckLayers(p_pCommand, p_pManager))
        {
            pCommand_I = (CCommand_I*)p_pCommand;
            
            switch(p_pCommand->GetCommandId())
            {
                case CAN_TRANSMIT_CAN_FRAME:
                    {
                        oResult = Process_TransmitCanFrame(pCommand_I);
                        break;
                    }
                case CAN_RECEIVE_CAN_FRAME:
                    {
                        oResult = Process_ReceiveCanFrame(pCommand_I);
                        break;
                    }
                case CAN_RECEIVE_FILTERED_CAN_FRAME:
                    {
                        oResult = Process_ReceiveFilteredCanFrame(pCommand_I);
                        break;
                    }
            }
        }

        if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayCANToSocketCAN"), TRUE));
        }
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::Process_TransmitCanFrame(CCommand_I* p_pCommand)
{
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_RTR = 1;
    const int PARAMETER_INDEX_DLC = 2;
    const int PARAMETER_INDEX_DATA = 3;

    BOOL oResult = FALSE;
    DWORD ulCobId;
    BYTE ubRtr;
    BYTE ubDlc;
    void* pDataBuffer = NULL;
    DWORD ulDataBufferLength;
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Prepare data buffer
        ulDataBufferLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(ulDataBufferLength > 0) pDataBuffer = malloc(ulDataBufferLength);

        //Get I parameter data
        p_pCommand->GetParameterData(PARAMETER_INDEX_COB_ID, &ulCobId, sizeof(ulCobId));
        p_pCommand->GetParameterData(PARAMETER_INDEX_RTR, &ubRtr, sizeof(ubRtr));
        p_pCommand->GetParameterData(PARAMETER_INDEX_DLC, &ubDlc, sizeof(ubDlc));
        p_pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);

        //Execute command
        oResult = TransmitCanFrame(ulCobId, (BOOL)ubRtr, ubDlc, pDataBuffer, ulDataBufferLength, &errorInfo);

        //Set PS return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::Process_ReceiveCanFrame(CCommand_I* p_pCommand)
{
    const int RETURN_PARAMETER_INDEX_COB_ID = 0;
    const int RETURN_PARAMETER_INDEX_RTR = 1;
    const int RETURN_PARAMETER_INDEX_DLC = 2;
    const int RETURN_PARAMETER_INDEX_DATA = 3;

    BOOL oResult = FALSE;
    DWORD ulTimeout;
    DWORD ulCobId;
    BOOL oRtr;
    BYTE ubDlc;
    void* pDataBuffer = NULL;
    DWORD ulDataBufferLength;
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Get I parameter data
        if(p_pCommand->IsTimeoutValid()) ulTimeout = p_pCommand->GetTimeout(); else ulTimeout = m_dTimeout;

        //Execute command
        oResult = ReceiveCanFrame(ulTimeout, &ulCobId, &oRtr, &ubDlc, &pDataBuffer, &ulDataBufferLength, &errorInfo);

        //Set I return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID, &ulCobId, sizeof(ulCobId));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_RTR, &oRtr, sizeof(oRtr));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DLC, &ubDlc, sizeof(ubDlc));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::Process_ReceiveFilteredCanFrame(CCommand_I* p_pCommand)
{
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_RTR = 1;

    const int RETURN_PARAMETER_INDEX_COB_ID = 0;
    const int RETURN_PARAMETER_INDEX_RTR = 1;
    const int RETURN_PARAMETER_INDEX_DLC = 2;
    const int RETURN_PARAMETER_INDEX_DATA = 3;

    BOOL oResult = FALSE;
    DWORD ulCobIdFilter;
    BYTE ubRtrFilter;
    DWORD ulTimeout;
    DWORD ulCobId;
    BOOL oRtr;
    BYTE ubDlc;
    void* pDataBuffer = NULL;
    DWORD ulDataBufferLength;
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Get I parameter data
        p_pCommand->GetParameterData(PARAMETER_INDEX_COB_ID, &ulCobIdFilter, sizeof(ulCobIdFilter));
        p_pCommand->GetParameterData(PARAMETER_INDEX_RTR, &ubRtrFilter, sizeof(ubRtrFilter));
        if(p_pCommand->IsTimeoutValid()) ulTimeout = p_pCommand->GetTimeout(); else ulTimeout = m_dTimeout;

        //Execute command
        oResult = ReceiveFilteredCanFrame(ulCobIdFilter, (BOOL)ubRtrFilter, ulTimeout, &ulCobId, &oRtr, &ubDlc, &pDataBuffer, &ulDataBufferLength, &errorInfo);

        //Set I return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID, &ulCobIdFilter, sizeof(ulCobIdFilter));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_RTR, &ubRtrFilter, sizeof(ubRtrFilter));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DLC, &ubDlc, sizeof(ubDlc));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::TransmitCanFrame(DWORD p_ulCobId, BOOL p_oRtr, BYTE p_ubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.TransmitCanFrame(p_ulCobId, p_oRtr, p_ubDlc, p_pDataBuffer, p_ulDataBufferLength))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_CAN_TRANSMIT_CAN_FRAME, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::ReceiveCanFrame(DWORD p_ulTimeout, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.ReceiveCanFrame(p_ulTimeout, p_pulCobId, p_poRtr, p_pubDlc, p_ppDataBuffer, p_pulDataBufferLength))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_CAN_RECEIVE_CAN_FRAME, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::ReceiveFilteredCanFrame(DWORD p_ulCobIdFilter, BOOL p_oRtrFilter, DWORD p_ulTimeout, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.ReceiveFilteredCanFrame(p_ulCobIdFilter, p_oRtrFilter, p_ulTimeout, p_pulCobId, p_poRtr, p_pubDlc, p_ppDataBuffer, p_pulDataBufferLength))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_CAN_RECEIVE_CAN_FRAME, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

CGateway* CGatewayCANToSocketCAN::Clone()
{
    CGatewayCANToSocketCAN* pClonedGateway;

    pClonedGateway = new CGatewayCANToSocketCAN();
    *pClonedGateway = *this;

    return pClonedGateway;
}

CGatewayCANToSocketCAN& CGatewayCANToSocketCAN::operator=(CGatewayCANToSocketCAN& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayIToDrv*)this) = *((CGatewayIToDrv*)&p_rOther);

        m_pInfoHandling = p_rOther.m_pInfoHandling;
        m_CANHndl = p_rOther.m_CANHndl;
    }

    return *this;
}

BOOL CGatewayCANToSocketCAN::ConvertFromDriverToInterfaceErrorCode(EDrvErrorCode p_eDriverErrorCode, DWORD& p_rulErrorCode)
{
	BOOL oResult = FALSE;

	switch(p_eDriverErrorCode)
	{
		case DEC_ACCESS_DENIED:
			p_rulErrorCode = ERROR_I_ACCESS_DENIED;
			oResult = TRUE;
			break;
		default:
			break;
	}

	return oResult;
}

BOOL CGatewayCANToSocketCAN::OpenInterface(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CSocketCANDeviceInfo* pDeviceType(0);
    DWORD ulErrorCode(0);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(m_pInfoHandling)
    {
        pDeviceType = m_pInfoHandling->FindDeviceInfo(p_InterfaceName);

        if(pDeviceType)
        {
            if(m_CANHndl.OpenInterface(*pDeviceType))
            {
                oResult = TRUE;
            }
            else
            {
            	if(!ConvertFromDriverToInterfaceErrorCode(m_CANHndl.GetLastError(), ulErrorCode))
            	{
            		ulErrorCode = ERROR_I_OPENING_INTERFACE;
            	}
            }
        }
        else
        {
            ulErrorCode = ERROR_BAD_INTERFACE_NAME;
        }
    }
    else
    {
        ulErrorCode = ERROR_NULL_POINTER;
    }

    //Error handling
    if(!oResult && m_pErrorHandling)
    {
        m_pErrorHandling->GetError(ulErrorCode, p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::CloseInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.CloseInterface())
    {
        if(m_pErrorHandling)
        {
        	m_pErrorHandling->GetError(ERROR_I_CLOSING_INTERFACE, p_pErrorInfo);
        }

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::OpenPort(CStdString p_PortName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CSocketCANDeviceInfo* pDeviceType(0);
    CStdString physicalPortName;

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(m_pInfoHandling)
	{
    	oResult = FALSE;

		pDeviceType = m_pInfoHandling->FindDeviceInfo(m_CANHndl.GetInterfaceName());

		if(pDeviceType)
		{
			physicalPortName = pDeviceType->GetPhysicalPortName(p_PortName);

			oResult = m_CANHndl.OpenPort(physicalPortName);
		}

		if(!oResult)
		{
			if(m_pErrorHandling)
			{
				m_pErrorHandling->GetError(ERROR_I_OPENING_PORT, p_pErrorInfo);
			}
		}
	}

    return oResult;
}

BOOL CGatewayCANToSocketCAN::ClosePort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.ClosePort())
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_PORT_NOT_OPEN, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::ResetPort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.ResetPort())
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_RESET_PORT, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::CheckStatusAndReset()
{
    BOOL oResult(FALSE);

    if(m_CANHndl.CheckStatusAndReset())
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.GetPortSettings(p_pulBaudrate, p_pulTimeout))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_GET_PORT_SETTINGS, p_pErrorInfo);

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    DWORD ulErrorCode = 0;

    //Reset error info
    if(p_pErrorInfo)
    {
        p_pErrorInfo->Reset();
    }

    if(!m_CANHndl.SetPortSettings(p_ulBaudrate, p_ulTimeout))
    {
    	if(!ConvertFromDriverToInterfaceErrorCode(m_CANHndl.GetLastError(), ulErrorCode))
		{
			ulErrorCode = ERROR_I_SET_PORT_SETTINGS;
		}

        if(m_pErrorHandling)
        {
        	m_pErrorHandling->GetError(ulErrorCode, p_pErrorInfo);
        }

        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo)
{
    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);

    return FALSE;
}

BOOL CGatewayCANToSocketCAN::SetPortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo)
{
    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);

    return FALSE;
}

BOOL CGatewayCANToSocketCAN::InitPortList()
{
    BOOL oResult(FALSE);
    CSocketCANDeviceInfo deviceInfo;
    CStdStringArray portNames;

    if(m_pInfoHandling)
    {
        if(m_pInfoHandling->GetPortNames(m_strInterfacePortName, portNames))
        {
            oResult = TRUE;

            for(CStdStringArray::iterator it = portNames.begin(); it != portNames.end(); it++)
            {
                AddPortList(*it);
            }
        }
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("GatewayCANToSocketCAN"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(INTERFACE_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::GetInterfaceHardwareNames(CStdStringArray& p_rNames)
{
    CMmcSingleLock lock(CCANDeviceInfoHandling::GetSyncInstance(), TRUE);

    BOOL oResult(FALSE);
    tSocketCANDeviceInfoList deviceInfoList;

    if(m_pInfoHandling)
    {
        //Get all physically available ports
        if(m_CANHndl.GetDeviceInfos(deviceInfoList))
        {
            if(m_pInfoHandling->InitDeviceInfos(deviceInfoList))
            {
                oResult = m_pInfoHandling->GetInterfaceNames(p_rNames);
            }

            DeleteDeviceInfoList(deviceInfoList);
        }
        else
        {
            //No ports available
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayCANToSocketCAN::DeleteDeviceInfoList(tSocketCANDeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult(FALSE);

    for(tSocketCANDeviceInfoList::iterator it=p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CSocketCANDeviceInfo* pDeviceInfo = (*it);
        delete pDeviceInfo;
    }

    p_rDeviceInfoList.clear();

    return oResult;
}
