#include "stdafx.h"
#include <CommunicationModel/Interface/Classes/Gateway/RS232/Classes/Rs232DeviceInfoHandling.h>

#include <Drv/Rs232/Rs232DeviceInfo.h>
#include <Thread/MmcCriticalSection.h>

CMmcCriticalSection CRs232DeviceInfoHandling::m_CriticalSection;

CRs232DeviceInfoHandling::CRs232DeviceInfoHandling(void):
    m_lInstanceValue(0)
{

}

CRs232DeviceInfoHandling::CRs232DeviceInfoHandling(int p_lInstanceValue):
    CDeviceInfoHandlingBase(p_lInstanceValue)
{

}

CRs232DeviceInfoHandling::~CRs232DeviceInfoHandling(void)
{
    DeleteDeviceInfoList(m_DeviceInfoList);
}

CMmcCriticalSection* CRs232DeviceInfoHandling::GetSyncInstance()
{
    return &m_CriticalSection;
}

BOOL CRs232DeviceInfoHandling::GetPortNames(CStdStringArray& p_rNames)
{
    BOOL oResult = TRUE;
    //Reset
    p_rNames.clear();

    //Add Names
    for(tRs232DeviceInfoList::iterator it = m_DeviceInfoList.begin(); it != m_DeviceInfoList.end(); it++)
    {
        CRs232DeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo != 0)
        {
            p_rNames.push_back(pDeviceInfo->GetPortName());
        }
    }

    return oResult;
}

CRs232DeviceInfo* CRs232DeviceInfoHandling::FindDeviceInfo(CStdString p_rPortName)
{
    CRs232DeviceInfo* pResultDeviceInfo(0);

    //Add Names
    for(tRs232DeviceInfoList::iterator it = m_DeviceInfoList.begin(); it != m_DeviceInfoList.end(); it++)
    {
        CRs232DeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo != 0)
        {
            //Check Interface Name
            if(p_rPortName == pDeviceInfo->GetPortName())
            {
                pResultDeviceInfo = pDeviceInfo;
                break;
            }
        }
    }

    return pResultDeviceInfo;
}

BOOL CRs232DeviceInfoHandling::InitDeviceInfos(tRs232DeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult = TRUE;

    DeleteDeviceInfoList(m_DeviceInfoList);
    for(tRs232DeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CRs232DeviceInfo* pDeviceInfo = ((CRs232DeviceInfo*)(*it))->Clone();
        if(pDeviceInfo)
        {
            m_DeviceInfoList.push_back(pDeviceInfo);
        }
    }

    return oResult;
}

BOOL CRs232DeviceInfoHandling::DeleteDeviceInfoList(tRs232DeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult = TRUE;

    for(tRs232DeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CRs232DeviceInfo* pDeviceInfo = (*it);
        if(pDeviceInfo!=0) delete pDeviceInfo;
    }

    p_rDeviceInfoList.clear();
    return oResult;
}

