
#include "stdafx.h"
#include <CommunicationModel/Interface/Classes/Gateway/USB/GatewayUSBToFtd2xxDrv.h>
#include <CommunicationModel/Interface/Classes/Gateway/USB/Classes/UsbDeviceInfoHandling.h>

#include <malloc.h>
#include <memory.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>
#include <Thread/MmcSingleLock.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/BaseClasses/Command_I.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/Command_I_USB.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/Command_I_FTD2XX.h>
#include <CommunicationModel/Interface/BaseClasses/InterfaceManagerBase.h>
#include <Drv/Ftd2xx/Ftd2xxDeviceInfo.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

DWORD CGatewayUSBToFtd2xxDrv::BAUDRATE_1000000 = 1000000;

CGatewayUSBToFtd2xxDrv::CGatewayUSBToFtd2xxDrv()
{
    m_oFirstInit = FALSE;

    m_pInfoHandling = 0;
    m_PortName = _T("");
    m_oUpdatePortOpened = TRUE;

    ResetRxBuffer();

    InitErrorHandling();
    InitBaudrateSelection();
    InitDefaultPortSettings();
}

CGatewayUSBToFtd2xxDrv::~CGatewayUSBToFtd2xxDrv()
{
}

BOOL CGatewayUSBToFtd2xxDrv::InitDriverInfos(CStdString p_DriverName, CStdString p_DriverParameters)
{
    BOOL oResult(FALSE);

    if(CGatewayIToDrv::InitDriverInfos(p_DriverName, p_DriverParameters))
    {
        oResult = InitVendorProductIds(p_DriverName, p_DriverParameters);
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::InitBaudrateSelection()
{
    m_BaudrateSelection.clear();
    m_BaudrateSelection.push_back(BAUDRATE_1000000);

    return TRUE;
}

BOOL CGatewayUSBToFtd2xxDrv::InitDefaultPortSettings()
{
    m_dDefaultBaudrate = BAUDRATE_1000000;
    m_dDefaultTimeout = 100;

    GetDefaultPortSettings(&m_dBaudrate, &m_dTimeout);

    return TRUE;
}

BOOL CGatewayUSBToFtd2xxDrv::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle)
{
    CCommand_I* pCommand_I;
    BOOL oResult = FALSE;

    if(CGateway::ProcessCommand(p_pCommand, p_pManager, p_h_Handle, p_hTransactionHandle))
    {
        if(CheckLayers(p_pCommand, p_pManager))
        {
            pCommand_I = (CCommand_I*)p_pCommand;

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv")));
            }

            switch(p_pCommand->GetCommandId())
            {
                case USB_WRITE_DATA:
                    {
                        oResult = Process_WriteData(pCommand_I);
                        break;
                    }
                case USB_READ_DATA:
                    {
                        oResult = Process_ReadData(pCommand_I);
                        break;
                    }
                case USB_RESCAN:
                    {
                        oResult = Process_Rescan(pCommand_I);
                        break;
                    }
                case USB_RELOAD:
                    {
                        oResult = Process_Reload(pCommand_I);
                        break;
                    }
                case FTD2XX_EE_READ:
                    {
                        oResult = Process_EE_Read(pCommand_I);
                        break;
                    }
                case FTD2XX_EE_PROGRAM:
                    {
                        oResult = Process_EE_Program(pCommand_I);
                        break;
                    }
            }

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv"), TRUE));
            }
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_WriteData(CCommand_I* p_pCommand)
{
    //*Constants I*
    const int PARAMETER_INDEX_DATA(0);
    const int RETURN_PARAMETER_INDEX_NB_OF_BYTES_WRITTEN(0);

    //*Variables I*
    //Parameter
    void* pDataBuffer(0);
    DWORD ulDataBufferLength;
    //ReturnParameter
    DWORD ulNbOfBytesWritten(0);

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Prepare data buffer
        ulDataBufferLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(ulDataBufferLength > 0) pDataBuffer = malloc(ulDataBufferLength);

        //Get I Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);

        if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv::WriteData->Before")));
        }

        //Execute command
        oResult = WriteData(pDataBuffer, ulDataBufferLength, &ulNbOfBytesWritten, &errorInfo);

        //Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NB_OF_BYTES_WRITTEN, &ulNbOfBytesWritten, sizeof(ulNbOfBytesWritten));

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);

        if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv::WriteData->After")));
        }

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_ReadData(CCommand_I* p_pCommand)
{
    //*Constants I*
    const int PARAMETER_INDEX_NB_OF_BYTES_TO_READ(0);
    const int RETURN_PARAMETER_INDEX_DATA(0);

    //*Variables I*
    //Parameter
    DWORD ulNbOfBytesToRead;

    //ReturnParameter
    void* pDataBuffer(0);
    DWORD ulDataBufferLength;
    DWORD ulNbOfBytesRead(0);

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        if(CMmcLog::IsActive(LMT_DEBUG, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv::ReadData->Before")));
        }

        //Get I Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_NB_OF_BYTES_TO_READ, &ulNbOfBytesToRead, sizeof(ulNbOfBytesToRead));

        //Prepare data buffer
        ulDataBufferLength = ulNbOfBytesToRead;
        if(ulDataBufferLength > 0) pDataBuffer = malloc(ulDataBufferLength);

        //Execute command
        oResult = ReadData(pDataBuffer, ulNbOfBytesToRead, &ulNbOfBytesRead, &errorInfo);

        //Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pDataBuffer, ulNbOfBytesRead);

        if(CMmcLog::IsActive(LMT_DEBUG, CCommunicationMessageLevel(CML_I)))
        {
            CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), p_pCommand->GetLogMessage(_T("CGatewayUSBToFtd2xxDrv::ReadData->After")));
        }

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_EE_Read(CCommand_I* p_pCommand)
{
    BOOL oResult(FALSE);

    //*Constants I*
    const int RETURN_PARAMETER_INDEX_VENDOR_ID(0);
    const int RETURN_PARAMETER_INDEX_PRODUCT_ID(1);
    const int RETURN_PARAMETER_INDEX_MANUFACTURER(2);
    const int RETURN_PARAMETER_INDEX_DESCRIPTION(3);
    const int RETURN_PARAMETER_INDEX_SERIAL_NUMBER(4);
    const int RETURN_PARAMETER_INDEX_USE_VCP_DRIVER(5);

    //*Variables I*
    //ReturnParameter
    WORD usVendorId(0);
    WORD usProductId(0);
    TCHAR* pManufacturer(0);
    unsigned int ulManufacturerLength(0);
    TCHAR* pDescription(0);
    unsigned int ulDescriptionLength(0);
    TCHAR* pSerialNumber(0);
    unsigned int ulSerialNumberLength(0);
    bool oUseVcpDriver = false;

    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Execute command
        oResult = Ftd2xxEeRead(usVendorId, usProductId, pManufacturer, ulManufacturerLength, pDescription, ulDescriptionLength, pSerialNumber, ulSerialNumberLength, oUseVcpDriver);

        //Set I ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_VENDOR_ID, &usVendorId, sizeof(usVendorId));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_PRODUCT_ID, &usProductId, sizeof(usProductId));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_MANUFACTURER, pManufacturer, ulManufacturerLength);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DESCRIPTION, pDescription, ulDescriptionLength);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SERIAL_NUMBER, pSerialNumber, ulSerialNumberLength);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_USE_VCP_DRIVER, &oUseVcpDriver, sizeof(oUseVcpDriver));

        //Free Buffers
        if(pManufacturer) free(pManufacturer);
        if(pDescription) free(pDescription);
        if(pSerialNumber) free(pSerialNumber);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_EE_Program(CCommand_I* p_pCommand)
{
    BOOL oResult(FALSE);

    //*Constants I*
    const int PARAMETER_INDEX_VENDOR_ID(0);
    const int PARAMETER_INDEX_PRODUCT_ID(1);
    const int PARAMETER_INDEX_MANUFACTURER(2);
    const int PARAMETER_INDEX_DESCRIPTION(3);
    const int PARAMETER_INDEX_SERIAL_NUMBER(4);
    const int PARAMETER_INDEX_USE_VCP_DRIVER(5);

    //*Variables I*
    //Parameter
    WORD usVendorId(0);
    WORD usProductId(0);
    TCHAR* pManufacturer(0);
    DWORD ulManufacturerLength(0);
    TCHAR* pDescription(0);
    DWORD ulDescriptionLength(0);
    TCHAR* pSerialNumber(0);
    DWORD ulSerialNumberLength(0);
    BOOL oUseVcpDriver(FALSE);

    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Prepare Buffers
        ulManufacturerLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_MANUFACTURER);
        if(ulManufacturerLength > 0) pManufacturer = (TCHAR*)malloc(ulManufacturerLength*sizeof(TCHAR)+1);
        if(pManufacturer) memset(pManufacturer, 0, ulManufacturerLength*sizeof(TCHAR)+1);

        ulDescriptionLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_DESCRIPTION);
        if(ulDescriptionLength > 0) pDescription = (TCHAR*)malloc(ulDescriptionLength*sizeof(TCHAR)+1);
        if(pDescription) memset(pDescription, 0, ulDescriptionLength*sizeof(TCHAR)+1);

        ulSerialNumberLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_SERIAL_NUMBER);
        if(ulSerialNumberLength > 0) pSerialNumber = (TCHAR*)malloc(ulSerialNumberLength*sizeof(TCHAR)+1);
        if(pSerialNumber) memset(pSerialNumber, 0, ulSerialNumberLength*sizeof(TCHAR)+1);

        //Get I Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_VENDOR_ID, &usVendorId, sizeof(usVendorId));
        p_pCommand->GetParameterData(PARAMETER_INDEX_PRODUCT_ID, &usProductId, sizeof(usProductId));
        p_pCommand->GetParameterData(PARAMETER_INDEX_MANUFACTURER, pManufacturer, ulManufacturerLength);
        p_pCommand->GetParameterData(PARAMETER_INDEX_DESCRIPTION, pDescription, ulDescriptionLength);
        p_pCommand->GetParameterData(PARAMETER_INDEX_SERIAL_NUMBER, pSerialNumber, ulSerialNumberLength);
        p_pCommand->GetParameterData(PARAMETER_INDEX_USE_VCP_DRIVER, &oUseVcpDriver, sizeof(oUseVcpDriver));

        //Execute command
        oResult = Ftd2xxEeProgram(usVendorId, usProductId, pManufacturer, ulManufacturerLength, pDescription, ulDescriptionLength, pSerialNumber, ulSerialNumberLength, oUseVcpDriver==TRUE) ? TRUE : FALSE;

        //Set I ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);

        //Free Buffers
        if(pManufacturer) free(pManufacturer);
        if(pDescription) free(pDescription);
        if(pSerialNumber) free(pSerialNumber);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_Rescan(CCommand_I* p_pCommand)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Execute command
        oResult = Rescan(&errorInfo);

        //Set I ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Process_Reload(CCommand_I* p_pCommand)
{
    //*Constants I*
    const int PARAMETER_INDEX_VENDOR_ID(0);
    const int PARAMETER_INDEX_PRODUCT_ID(1);

    //*Variables I*
    //Parameter
    WORD usVendorId(0);
    WORD usProductId(0);

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;

    //Lock critical section
    if(p_pCommand && Lock(p_pCommand->GetTimeout()))
    {
        //Get I Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_VENDOR_ID, &usVendorId, sizeof(usVendorId));
        p_pCommand->GetParameterData(PARAMETER_INDEX_PRODUCT_ID, &usProductId, sizeof(usProductId));

        //Execute command
        oResult = Reload(usVendorId, usProductId, &errorInfo);

        //Set I ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

CGateway* CGatewayUSBToFtd2xxDrv::Clone()
{
    CGatewayUSBToFtd2xxDrv* pClonedGateway;

    pClonedGateway = new CGatewayUSBToFtd2xxDrv();
    *pClonedGateway = *this;

    return pClonedGateway;
}

CGatewayUSBToFtd2xxDrv& CGatewayUSBToFtd2xxDrv::operator=(CGatewayUSBToFtd2xxDrv& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayIToDrv*)this) = *((CGatewayIToDrv*)&p_rOther);
        m_pInfoHandling = p_rOther.m_pInfoHandling;
    }

    return *this;
}

BOOL CGatewayUSBToFtd2xxDrv::OpenPort(CStdString p_portName, CErrorInfo* p_pErrorInfo)
{
    const BOOL CHANGE_ONLY(FALSE);
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);
    CUsbDeviceInfo* pDeviceInfo(0);

    if(m_pInfoHandling)
    {
        pDeviceInfo = m_pInfoHandling->GetDeviceInfo(p_portName);
        if(pDeviceInfo && pDeviceInfo->IsKindOf(_T("CFtd2xxDeviceInfo")))
        {
            if(m_Ftd2xxHndl.OpenPort(*(CFtd2xxDeviceInfo*)pDeviceInfo))
            {
                //Configure default settings for port
                if(ConfigurePortSettings(m_dBaudrate, m_dTimeout, CHANGE_ONLY, p_pErrorInfo, FALSE))
                {
                    m_PortName = p_portName;
                    if(m_oUpdatePortOpened) UpdatePortOpened(m_PortName, TRUE);
                    oResult = TRUE;
                }
                else
                {
                    ClosePort();
                    oResult = FALSE;
                }
            }
            else
            {
                ClosePort();
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_OPENING_PORT, p_pErrorInfo);
                oResult = FALSE;
            }
        }
        else
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_OPENING_PORT, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::ReloadDriver()
{
    //Reload FTDI Driver
    m_Ftd2xxHndl.Reload();
    MmcSleep(500);

    return TRUE;
}

BOOL CGatewayUSBToFtd2xxDrv::SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo)
{
    return ConfigurePortSettings(p_ulBaudrate, p_ulTimeout, p_oChangeOnly, p_pErrorInfo, TRUE);
}

BOOL CGatewayUSBToFtd2xxDrv::ConfigurePortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo, BOOL p_oDoLock)
{
    BOOL oResult(TRUE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), FALSE);

    //Lock
    if(p_oDoLock) lock.Lock();

    //Do only if changed
    if(!p_oChangeOnly || (p_ulTimeout != m_dTimeout) || (p_ulBaudrate != m_dBaudrate))
    {
        //Do only if changed
        if(oResult)
        {
            if(!p_oChangeOnly || (p_ulTimeout != m_dTimeout))
            {
                //Set Timeout
                if(!SetTimeout(p_ulTimeout))
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_SET_PORT_SETTINGS, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
        }

        if(oResult)
        {
            if(!p_oChangeOnly || !m_oFirstInit)
            {
                //Configure PortMode
                if(!ConfigurePortMode(m_wPortMode, p_pErrorInfo, FALSE))
                {
                    oResult = FALSE;
                }
            }
        }

        //Do only if changed
        if(oResult)
        {
            if(!p_oChangeOnly || (p_ulBaudrate != m_dBaudrate))
            {
                if(SetBaudrate(p_ulBaudrate))
                {
                    //Baudrate is set
                    m_dBaudrate = p_ulBaudrate;
                }
                else
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_SET_PORT_SETTINGS, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
        }

        if(oResult)
        {
            if(!p_oChangeOnly || !m_oFirstInit)
            {
                //Purge the buffer
                if(!PurgeBuffer())
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_SET_PORT_SETTINGS, p_pErrorInfo);
                    oResult = FALSE;
                }
                else
                {
                    m_oFirstInit = TRUE;
                }
            }
        }
    }

    //Tracing
    Trace_Settings(p_ulBaudrate, p_ulTimeout);

    //Unlock
    if(p_oDoLock) lock.Unlock();

    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_SET_PORT_SETTINGS, p_pErrorInfo);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::SetPortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo)
{
    return ConfigurePortMode(p_usPortMode, p_pErrorInfo, TRUE);
}

BOOL CGatewayUSBToFtd2xxDrv::ConfigurePortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);

    CMmcSingleLock lock(CGateway::GetSyncInstance(), FALSE);

    //Lock
    if(p_oDoLock)
    {
        lock.Lock();
    }

    if(m_wPortMode == 0)
    {
        oResult = m_Ftd2xxHndl.Configure() ? TRUE : FALSE;
    }

    //Unlock
    if(p_oDoLock)
    {
         lock.Unlock();
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::SetTimeout(DWORD p_ulTimeout)
{
    const DWORD WRITE_TIMEOUT(200);

    DWORD ulReadTimeout(0);
    DWORD ulWriteTimeout(0);
    BOOL oResult(FALSE);

    //Prepare Parameter
    oResult = TRUE;
    ulReadTimeout = p_ulTimeout;
    ulWriteTimeout = WRITE_TIMEOUT;

    //SetTimeouts
    if(!m_Ftd2xxHndl.SetTimeouts(ulReadTimeout, ulWriteTimeout))
        oResult = FALSE;
    else
        m_dTimeout = p_ulTimeout;

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::SetBaudrate(DWORD p_ulBaudrate)
{
    return m_Ftd2xxHndl.SetBaudrate(p_ulBaudrate) ? TRUE : FALSE;
}

BOOL CGatewayUSBToFtd2xxDrv::PurgeBuffer()
{
    unsigned int dAmountInRxQueue(1);
    unsigned int dAmountInTxQueue(1);
    unsigned int dEventStatus(0);
    BYTE uData(0);
    unsigned int dDataRead(1);
    BOOL oResult(FALSE);

    oResult = TRUE;

    while(oResult && (dAmountInRxQueue > 0))
    {
        if(m_Ftd2xxHndl.GetStatus(&dAmountInRxQueue, &dAmountInTxQueue, &dEventStatus))
        {
            if(dAmountInRxQueue > 0)
            {
                //Read all Characters in Rx Buffer
                oResult = TRUE;
                while(dAmountInRxQueue > 0)
                {
                    if(m_Ftd2xxHndl.ReadByte(&uData, &dDataRead))
                    {
                        dAmountInRxQueue -= dDataRead;
                    }
                    else
                    {
                        oResult = FALSE;
                        break;
                    }
                }
            }
            else
            {
                //No Characters in Rx Buffer
                oResult = TRUE;
            }
        }
        else
        {
            //Failed reading Status of Rx Buffer
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::WriteData(void* p_pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_RETRY = 2;
    const DWORD MAX_SIZE = 62;

    DWORD ulNumberOfBytesToWrite(0);
    unsigned int ulNumberOfBytesWritten(0);
    DWORD ulTotalNumberOfBytesToWrite(0);
    DWORD ulTotalNumberOfByteWritten(0);
    void* pDataBuffer(0);
    DWORD ulRetryCount(0);
    BOOL oResult(FALSE);
    CStdString msg;

    msg.Format(_T("WriteData Buffer Before: NbOfBytesToWrite %i\n"), p_ulNbOfBytesToWrite);

    CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), msg);

    //ResetErrorInfo
    if(p_pErrorInfo) p_pErrorInfo->Reset();

    //Execute command
    if(p_pDataBuffer && (p_ulNbOfBytesToWrite > 0))
    {
        //Init
        ulTotalNumberOfBytesToWrite = p_ulNbOfBytesToWrite;
        oResult = TRUE;

        while(oResult && (ulTotalNumberOfBytesToWrite > 0))
        {
            //Limit Size
            if(ulTotalNumberOfBytesToWrite < MAX_SIZE) ulNumberOfBytesToWrite = ulTotalNumberOfBytesToWrite; else ulNumberOfBytesToWrite = MAX_SIZE;
            pDataBuffer = (void*)((BYTE*)p_pDataBuffer + ulTotalNumberOfByteWritten);

            //Write Data
            if(Ftd2xxWrite(pDataBuffer, ulNumberOfBytesToWrite, &ulNumberOfBytesWritten) && (ulNumberOfBytesWritten > 0))
            {
                ulTotalNumberOfBytesToWrite -= ulNumberOfBytesWritten;
                ulTotalNumberOfByteWritten += ulNumberOfBytesWritten;
                if(p_pulNbOfBytesWritten) *p_pulNbOfBytesWritten = ulTotalNumberOfByteWritten;
                ulRetryCount = 0;
                oResult = TRUE;
            }
            else
            {
                if(ulRetryCount < MAX_RETRY)
                {
                    //Retry
                    ulRetryCount++;
                    ulTotalNumberOfBytesToWrite -= ulNumberOfBytesWritten;
                    ulTotalNumberOfByteWritten += ulNumberOfBytesWritten;
                    if(p_pulNbOfBytesWritten) *p_pulNbOfBytesWritten = ulTotalNumberOfByteWritten;
                    oResult = TRUE;
                }
                else
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_USB_WRITE_DATA, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
        oResult = FALSE;
    }

    msg.Format(_T("WriteData Buffer End: NbOfBytesToWrite %i NbOfBytesWritten %i\n"), p_ulNbOfBytesToWrite, *p_pulNbOfBytesWritten);
    CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), msg);

    Trace_WriteData(oResult, p_pDataBuffer, p_ulNbOfBytesToWrite, p_pulNbOfBytesWritten);

    return oResult;
}

CStdString CGatewayUSBToFtd2xxDrv::GetFtStatusDescription(unsigned long p_ulStatus)
{
    return m_Ftd2xxHndl.GetFtStatusDescription(p_ulStatus).c_str();
}

BOOL CGatewayUSBToFtd2xxDrv::ReadData(void* p_pDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_RETRY = 2;
    const DWORD MAX_SIZE = 62;
    BOOL oResult(FALSE);
    DWORD ulNumberOfBytesToRead(0);
    unsigned int ulNumberOfBytesRead(0);
    DWORD ulTotalNumberOfBytesToRead(0);
    DWORD ulTotalNumberOfByteRead(0);
    void* pDataBuffer(0);
    DWORD ulRetryCount(0);
    CStdString msg;

    //ResetErrorInfo
    if(p_pErrorInfo) p_pErrorInfo->Reset();

    //Execute command
    if(p_pDataBuffer && (p_ulNbOfBytesToRead > 0) && p_pulNbOfBytesRead)
    {
        //Init
        ulTotalNumberOfBytesToRead = p_ulNbOfBytesToRead;
        oResult = TRUE;

        msg.Format(_T("ReadData BlockBuffer Before: NbOfBytesToRead %i\n"), p_ulNbOfBytesToRead);
        CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), msg);

        while(oResult && (ulTotalNumberOfBytesToRead > 0))
        {
            //Limit Size
            if(ulTotalNumberOfBytesToRead < MAX_SIZE) ulNumberOfBytesToRead = ulTotalNumberOfBytesToRead; else ulNumberOfBytesToRead = MAX_SIZE;
            pDataBuffer = (void*)((BYTE*)p_pDataBuffer + ulTotalNumberOfByteRead);
            ulNumberOfBytesRead = 0;

            //Write Data
            if(Ftd2xxRead(pDataBuffer, ulNumberOfBytesToRead, &ulNumberOfBytesRead) && (ulNumberOfBytesRead > 0))
            {
                ulTotalNumberOfBytesToRead -= ulNumberOfBytesRead;
                ulTotalNumberOfByteRead += ulNumberOfBytesRead;
                if(p_pulNbOfBytesRead) *p_pulNbOfBytesRead = ulTotalNumberOfByteRead;
                ulRetryCount = 0;
                oResult = TRUE;
            }
            else
            {
                if(ulRetryCount < MAX_RETRY)
                {
                    //Retry
                    ulRetryCount++;
                    ulTotalNumberOfBytesToRead -= ulNumberOfBytesRead;
                    ulTotalNumberOfByteRead += ulNumberOfBytesRead;
                    if(p_pulNbOfBytesRead) *p_pulNbOfBytesRead = ulTotalNumberOfByteRead;
                    oResult = TRUE;
                }
                else
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_USB_READ_DATA, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
        }

        msg.Format(_T("ReadData BlockBuffer End: NbOfBytesToRead %i NbOfBytesRead %i\n"), p_ulNbOfBytesToRead, ulTotalNumberOfByteRead);
        CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_I), msg);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
        oResult = FALSE;
    }

    Trace_ReadData(oResult, p_pDataBuffer, p_ulNbOfBytesToRead, p_pulNbOfBytesRead);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Rescan(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //ResetErrorInfo
    if(p_pErrorInfo) p_pErrorInfo->Reset();

    //Rescan
    if(Ftd2xxRescan())
    {
        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_USB_RESCAN, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Reload(WORD p_usVendorId, WORD p_usProductId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //ResetErrorInfo
    if(p_pErrorInfo) p_pErrorInfo->Reset();

    //Reload
    if(Ftd2xxReload(p_usVendorId, p_usProductId))
    {
        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_USB_RELOAD, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::WaitForRxData(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::ResetRxBuffer()
{
    CMmcSingleLock lock(&m_Sync, TRUE);
    BOOL oResult(TRUE);

    memset(m_RxBuffer, 0, sizeof(m_RxBuffer));
    m_dRxBufferCount = 0;
    m_iGetIndex = 0;
    m_iPutIndex = 0;

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdateRxBuffer()
{
    CMmcSingleLock lock(&m_Sync, TRUE);

    BOOL oResult(FALSE);
    unsigned int dAmountRxQueue(0);
    unsigned int dRead(0);

    if(m_iPutIndex < 9000)
    {
        if(m_Ftd2xxHndl.GetQueueStatus(&dAmountRxQueue))
        {
            if(dAmountRxQueue > 0)
            {
                if(m_Ftd2xxHndl.Read(&m_RxBuffer[m_iPutIndex], dAmountRxQueue, &dRead) && (dRead == dAmountRxQueue))
                {
                    m_iPutIndex += dRead;
                    m_dRxBufferCount += dRead;
                }
            }

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::GetRxData(void* p_pBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(&m_Sync, TRUE);

    if(p_pBuffer)
    {
        if(m_iPutIndex < 9000)
        {
            if(m_dRxBufferCount >= p_ulNbOfBytesToRead)
            {
                memcpy(p_pBuffer, &m_RxBuffer[m_iGetIndex], p_ulNbOfBytesToRead);
                m_iGetIndex += p_ulNbOfBytesToRead;
                m_dRxBufferCount -= p_ulNbOfBytesToRead;
                if(p_pulNbOfBytesRead) *p_pulNbOfBytesRead = p_ulNbOfBytesToRead;
                oResult = TRUE;
            }
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    if(p_pulBaudrate) *p_pulBaudrate = m_dBaudrate;
    if(p_pulTimeout) *p_pulTimeout = m_dTimeout;
    return TRUE;
}

BOOL CGatewayUSBToFtd2xxDrv::GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pusPortMode)
    {
        *p_pusPortMode = m_wPortMode;
        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::ClosePort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    if(m_Ftd2xxHndl.ClosePort())
    {
        Trace_Close();

        if(m_oUpdatePortOpened) UpdatePortOpened(m_PortName, FALSE);

        m_PortName = _T("");
        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_CLOSING_PORT, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::ResetPort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = TRUE;

    if(!PurgeBuffer())
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_I_RESET_PORT, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::InitPortList()
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);
    tUsbDeviceInfoList deviceInfoList;
    CStdString deviceClassType(_T(""));

    if(m_pInfoHandling)
    {
        //Get All physically available Ports
        if(GetDeviceInfos(deviceInfoList, deviceClassType))
        {
            if(m_pInfoHandling->InitDeviceInfos(deviceInfoList, deviceClassType))
            {
                oResult = UpdatePortList(m_pInfoHandling);
            }

            DeleteDeviceInfoList(deviceInfoList);
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdateDeviceInfos(tPortList& p_rOpenPortList)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);
    tUsbDeviceInfoList deviceInfoList;
    CStdString deviceClassType(_T(""));

    if(m_pInfoHandling)
    {
        //Get All physically available Ports
        if(GetDeviceInfos(deviceInfoList, deviceClassType))
        {
            if(m_pInfoHandling->UpdateDeviceInfos(deviceInfoList, deviceClassType, p_rOpenPortList))
            {
                oResult = TRUE;
            }

            DeleteDeviceInfoList(deviceInfoList);
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdatePortList(tPortList& p_rOpenPortList)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    if(m_pInfoHandling)
    {
        //Update PortList
        oResult = UpdatePortList(m_pInfoHandling);
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdatePortList(CUsbDeviceInfoHandling* p_pDeviceInfoHandling)
{
    CStdStringArray portNames;
    CStdString portName(_T(""));
    CFtd2xxDeviceInfo deviceType;
    BOOL oResult(FALSE);

    //Reset PortList
    DeletePortList();

    if(p_pDeviceInfoHandling)
    {
        //Add Ports
        if(p_pDeviceInfoHandling->GetPortNames(deviceType, portNames))
        {
            oResult = TRUE;
            for(std::size_t i=0;i<portNames.size();i++)
            {
                portName = portNames.at(i);
                AddPortList(portName);
            }
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("GatewayUSBToFtd2xxDrv"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(INTERFACE_LAYER, className);
        m_pErrorHandling->InitErrorProducer(&errorProducer);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdatePortOpened(CStdString p_PortName, BOOL p_oOpened)
{
    BOOL oResult(FALSE);

    if(m_pInfoHandling)
    {
        oResult = m_pInfoHandling->UpdatePortOpened(p_PortName, p_oOpened);
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::GetSerialNumber(CStdString p_PortName, CStdString& p_serialNumber)
{
    BOOL oResult(FALSE);
    CUsbDeviceInfo* pDeviceInfo(0);

    if(m_pInfoHandling)
    {
        pDeviceInfo = m_pInfoHandling->GetDeviceInfo(p_PortName);
        if(pDeviceInfo && pDeviceInfo->IsKindOf(_T("CFtd2xxDeviceInfo")))
        {
            CFtd2xxDeviceInfo* pFtd2xxDeviceInfo = (CFtd2xxDeviceInfo*)pDeviceInfo;
            p_serialNumber = pFtd2xxDeviceInfo->GetSerialNumber();
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::GetLocationId(CStdString p_PortName, DWORD& p_rulLocationId)
{
    BOOL oResult(FALSE);
    CUsbDeviceInfo* pDeviceInfo(0);

    if(m_pInfoHandling)
    {
        pDeviceInfo = m_pInfoHandling->GetDeviceInfo(p_PortName);
        if(pDeviceInfo && pDeviceInfo->IsKindOf(_T("CFtd2xxDeviceInfo")))
        {
            CFtd2xxDeviceInfo* pFtd2xxDeviceInfo = (CFtd2xxDeviceInfo*)pDeviceInfo;
            p_rulLocationId = pFtd2xxDeviceInfo->GetLocationId();

            if(p_rulLocationId == 0)
            {
                //LocationId is not valid, update LocationId
                oResult = UpdateLocationId(m_pInfoHandling, p_PortName, p_rulLocationId);
            }
            else
            {
                //LocationId is valid
                oResult = TRUE;
            }
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::UpdateLocationId(CUsbDeviceInfoHandling* p_pDeviceInfoHandling, CStdString p_PortName, DWORD& p_rulLocationId)
{
    BOOL oResult(FALSE);
    tUsbDeviceInfoList deviceInfoList;
    CStdString deviceClassType(_T(""));
    CUsbDeviceInfo* pDeviceInfo(0);

    if(p_pDeviceInfoHandling)
    {
        //Get All physically available Ports
        if(GetDeviceInfos(deviceInfoList, deviceClassType))
        {
            //Update LocationId
            if(p_pDeviceInfoHandling->UpdateDeviceInfo(deviceInfoList, deviceClassType, p_PortName))
            {
                //Get Updated LocationId
                pDeviceInfo = m_pInfoHandling->GetDeviceInfo(p_PortName);
                if(pDeviceInfo && pDeviceInfo->IsKindOf(_T("CFtd2xxDeviceInfo")))
                {
                    CFtd2xxDeviceInfo* pFtd2xxDeviceInfo = (CFtd2xxDeviceInfo*)pDeviceInfo;
                    p_rulLocationId = pFtd2xxDeviceInfo->GetLocationId();
                    oResult = TRUE;
                }
            }

            DeleteDeviceInfoList(deviceInfoList);
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::GetDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString& p_rDeviceClassType)
{
    BOOL oResult(FALSE);
    CUsbDeviceInfoHandling::SVendorProductIDs vendorProductId;
    std::list<CUsbDeviceInfoHandling::SVendorProductIDs> vendorProductIdList;
    BOOL oIsFtd2xxDevice(FALSE);

    if(m_pInfoHandling)
    {
        //Reset
        DeleteDeviceInfoList(p_rDeviceInfoList);

        if(m_pInfoHandling->GetAllVendorProductIds(vendorProductIdList))
        {
            //Init
            oResult = TRUE;
            p_rDeviceClassType = m_Ftd2xxHndl.GetDeviceClassType();

            //Enum Devices
            for(std::list<CUsbDeviceInfoHandling::SVendorProductIDs>::iterator it = vendorProductIdList.begin(); it != vendorProductIdList.end(); it++)
            {
                vendorProductId = (*it);
                oIsFtd2xxDevice = m_pInfoHandling->AreVendorProductIdsSupportedByDriver(m_DriverName, vendorProductId.vendorId, vendorProductId.productId);

                if(oResult && !m_Ftd2xxHndl.GetDeviceInfos(p_rDeviceInfoList, vendorProductId.vendorId, vendorProductId.productId, oIsFtd2xxDevice)) oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::DeleteDeviceInfoList(tUsbDeviceInfoList& p_rDeviceInfoList)
{
    BOOL oResult(FALSE);

    for(tUsbDeviceInfoList::iterator it=p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CUsbDeviceInfo* pDeviceInfo = (*it);
        delete pDeviceInfo;
    }

    p_rDeviceInfoList.clear();

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CUsbDeviceInfoHandling*>(p_pInfoHandling);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::InitVendorProductIds(CStdString p_DriverName, CStdString p_DriverParameters)
{
    BOOL oResult(FALSE);

    if(m_pInfoHandling)
    {
        if(p_DriverParameters.size() == 0)
        {
            //Default IDs
            oResult = TRUE;
            if(!m_pInfoHandling->AddVendorProductIds(p_DriverName, USB_VENDOR_PRODUCT_ID_FTDI)) oResult = FALSE;
            if(!m_pInfoHandling->AddVendorProductIds(p_DriverName, USB_VENDOR_PRODUCT_ID_EPOS2)) oResult = FALSE;
        }
        else
        {
            //Driver Parameter IDs (Format: "VID_XXXX&PIDYYYY VID_ZZZZ&PID_WWWW ...")
            oResult = m_pInfoHandling->AddVendorProductIds(p_DriverName, p_DriverParameters);
        }

        //Usb Flags IgnoreHwSerialNumber
        m_pInfoHandling->UpdateRegistryIgnoreHwSerialNumber();

    }

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxWrite(void* p_pDataBuffer, unsigned int p_ulNbOfBytesToWrite, unsigned int* p_pulNbOfBytesWritten)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.Write(p_pDataBuffer, p_ulNbOfBytesToWrite, p_pulNbOfBytesWritten);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxRead(void* p_pDataBuffer, unsigned int p_ulNbOfBytesToRead, unsigned int* p_pulNbOfBytesRead)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.Read(p_pDataBuffer, p_ulNbOfBytesToRead, p_pulNbOfBytesRead);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxRescan()
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.Rescan();

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxReload(WORD p_usVendorId, WORD p_usProductId)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.Reload(p_usVendorId, p_usProductId);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxEeRead(unsigned short& p_rusVendorId, unsigned short& p_rusProductId, TCHAR*& p_rpManufacturer, unsigned int& p_rulManufacturerLength, TCHAR*& p_rpDescription, unsigned int& p_rulDescriptionLength, TCHAR*& p_rpSerialNumber, unsigned int& p_rulSerialNumberLength, bool& p_roUseVcpDriver)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.EE_Read(p_rusVendorId, p_rusProductId, p_rpManufacturer, p_rulManufacturerLength, p_rpDescription, p_rulDescriptionLength, p_rpSerialNumber, p_rulSerialNumberLength, p_roUseVcpDriver);

    return oResult;
}

BOOL CGatewayUSBToFtd2xxDrv::Ftd2xxEeProgram(unsigned short p_usVendorId, unsigned short p_usProductId, TCHAR* p_pManufacturer, unsigned int p_ulManufacturerLength, TCHAR* p_pDescription, unsigned int p_ulDescriptionLength, TCHAR* p_pSerialNumber, unsigned int p_ulSerialNumberLength, bool p_oUseVcpDriver)
{
    BOOL oResult(FALSE);
    CMmcSingleLock lock(CGateway::GetSyncInstance(), TRUE);

    oResult = m_Ftd2xxHndl.EE_Program(p_usVendorId, p_usProductId, p_pManufacturer, p_ulManufacturerLength, p_pDescription, p_ulDescriptionLength, p_pSerialNumber, p_ulSerialNumberLength, p_oUseVcpDriver);

    return oResult;
}
