#include "stdafx.h"
#include <stdio.h>

#include <CommunicationModel/Interface/Classes/Handle/HandleRegistration_I.h>
#include <CommunicationModel/Interface/Classes/Interfaces/BaseClasses/InterfaceBase.h>
#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>
#include <Thread/MmcCriticalSection.h>
#include <Thread/MmcCriticalSectionEx.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CHandleRegistration_I::CHandleRegistration_I()
{
    m_hInterfaceHandle = 0;
    m_pInterface = NULL;
    m_pPort = NULL;
    m_pCriticalSection = NULL;

    CreateCriticalSection();
}

CHandleRegistration_I::~CHandleRegistration_I()
{
    DeleteCriticalSection();
}

BOOL CHandleRegistration_I::CreateCriticalSection()
{
    BOOL oResult = FALSE;

    if(!m_pCriticalSection)
    {
        m_pCriticalSection = new CMmcCriticalSectionEx();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CHandleRegistration_I::DeleteCriticalSection()
{
    BOOL oResult = FALSE;
    if(m_pCriticalSection)
    {
        delete m_pCriticalSection;
        m_pCriticalSection = NULL;
        oResult = TRUE;
    }

    return oResult;
}

HANDLE CHandleRegistration_I::GetKeyHandle()
{
    return m_hInterfaceHandle;
}

BOOL CHandleRegistration_I::GetRegistrationValues(CInterfaceBase** p_ppInterface, CPortBase** p_ppPort)
{
    if(p_ppInterface) *p_ppInterface = m_pInterface;
    if(p_ppPort) *p_ppPort = m_pPort;

    return TRUE;
}

BOOL CHandleRegistration_I::IsEqual(CInterfaceBase* p_pInterface, CStdString p_PortName)
{
    CStdString portName = _T("");
    BOOL oResult = FALSE;

    if((m_pInterface && p_pInterface) && m_pPort)
    {
        //Init
        oResult = TRUE;

        //Interface
        if(oResult && !m_pInterface->IsEqual(p_pInterface)) oResult = FALSE;

        //Port
        if(oResult && !m_pPort->IsEqual(p_PortName)) oResult = FALSE;
    }

    return oResult;
}

BOOL CHandleRegistration_I::IsEqual(CInterfaceBase* p_pInterface)
{
    BOOL oResult = FALSE;

    if(m_pInterface && p_pInterface)
    {
        //Init
        oResult = TRUE;

        //Interface
        if(oResult && !m_pInterface->IsEqual(p_pInterface)) oResult = FALSE;
    }

    return oResult;
}

BOOL CHandleRegistration_I::IsEmpty()
{
    return ((m_pInterface == NULL) || (m_pPort == NULL));
}

void CHandleRegistration_I::SetKeyHandle(HANDLE p_hInterfaceHandle)
{
    m_hInterfaceHandle = p_hInterfaceHandle;
}

void CHandleRegistration_I::Init(CInterfaceBase* p_pInterface, CPortBase* p_pPort)
{
    if(p_pInterface && p_pPort)
    {
        m_pInterface = p_pInterface->Clone();
        m_pPort = p_pPort->Clone();
    }
}

void CHandleRegistration_I::UpdateRegistration(CInterfaceBase* p_pInterface, CPortBase* p_pPort)
{
    HANDLE hHandle = m_hInterfaceHandle;
    CJournalManagerBase* pJournalManager(0);

    if(m_pPort)
    {
        pJournalManager = m_pPort->GetJournalManager();
    }

    DeleteRegistration();
    Init(p_pInterface, p_pPort);

    if(pJournalManager)
    {
        m_pPort->InitJournalManager(pJournalManager);
    }

    m_hInterfaceHandle = hHandle;
}

void CHandleRegistration_I::DeleteRegistration()
{
    if(m_pInterface)
        delete m_pInterface;
    m_pInterface = NULL;

    if(m_pPort)
        delete m_pPort;
    m_pPort = NULL;

    m_hInterfaceHandle = 0;
}

BOOL CHandleRegistration_I::DoLock(DWORD p_ulTimeout)
{
    CStdString traceString = _T("");
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        //TraceString
        GetTraceString(traceString);

        //Lock
        oResult = m_pCriticalSection->Lock(p_ulTimeout, traceString);
    }

    return oResult;
}

BOOL CHandleRegistration_I::DoUnlock()
{
    CStdString traceString = _T("");
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        //TraceString
        GetTraceString(traceString);

        //Unlock
        oResult = m_pCriticalSection->Unlock(traceString);
    }

    return oResult;
}

BOOL CHandleRegistration_I::IsLocked()
{
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        oResult = m_pCriticalSection->IsLocked();
    }

    return oResult;
}

BOOL CHandleRegistration_I::AbortLock(BOOL p_oActive)
{
    BOOL oResult = FALSE;

    if(m_pCriticalSection)
    {
        oResult = m_pCriticalSection->AbortLock(p_oActive == TRUE) ? TRUE : FALSE;
    }

    return oResult;
}

BOOL CHandleRegistration_I::GetTraceString(CStdString& p_rTraceString)
{
    BOOL oResult = TRUE;

    #ifdef _Trace_CommunicationModelLock
        p_rTraceString.Format(_T("HandleRegistration I 0x%.8x"), (unsigned int)m_hInterfaceHandle);
    #endif

    return oResult;
}
