#include "stdafx.h"
#include <algorithm>

#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/DeviceInfoHandlingBase.h>
#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>

#include <CommunicationModel/Interface/Classes/Interfaces/BaseClasses/InterfaceBase.h>

using namespace std;

CInterfaceBase::CInterfaceBase()
{
    m_strClassType = _T("CInterfaceBase");

    m_lInstanceValue = 0;
    m_strInterfaceName = "";
    m_pJournalManager = 0;
    m_pGateway = 0;
    m_usModeIndex = 0;

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CInterfaceBase::CInterfaceBase(const CInterfaceBase& rObject)
{
    m_strClassType = _T("CInterfaceBase");

    //Instance Value
    m_lInstanceValue = rObject.m_lInstanceValue;

    //InterfaceName
    m_strInterfaceName = rObject.m_strInterfaceName;

    //JournalManager
    m_pJournalManager = rObject.m_pJournalManager;

    //Gateway
    if(rObject.m_pGateway) m_pGateway = (CGatewayIToDrv*)rObject.m_pGateway->Clone(); else m_pGateway = 0;

    m_pErrorHandling = 0;

    InitErrorHandling();
}

CInterfaceBase::~CInterfaceBase()
{
    DeleteGateway();
    DeletePortList();
    DeleteErrorHandling();
}

bool CInterfaceBase::IsKindOf(CStdString p_classType)
{
    return (m_strClassType.CompareNoCase(p_classType) == 0);
}

CStdString CInterfaceBase::GetClassType()
{
    return m_strClassType;
}

BOOL CInterfaceBase::GetInterfaceHardwareNames(CStdStringArray& p_rNames)
{
    //By Default Hardware name corresponds to interface name
    p_rNames.push_back(m_strInterfaceName);
    return TRUE;
}

BOOL CInterfaceBase::GetInterfaceHardwareNames(CStdString p_DriverName, CStdStringArray& p_rNames)
{
    //By Default Hardware name corresponds to interface name
    p_rNames.push_back(m_strInterfaceName);
    return TRUE;
}

BOOL CInterfaceBase::InitInstanceValue(int p_lInstanceValue)
{
    BOOL oResult = TRUE;

    m_lInstanceValue = p_lInstanceValue;

    return oResult;
}

BOOL CInterfaceBase::AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray)
{
    BOOL oResult(FALSE);
    BOOL oFound;
    CStdString destName;
    CStdString sourceName;

    if(p_pDestArray && p_pSourceArray)
    {
        for(size_t i = 0; i < p_pSourceArray->size(); i++)
        {
            oFound = FALSE;
            sourceName = p_pSourceArray->at(i);
            for(size_t j = 0; j < p_pDestArray->size(); j++)
            {
                destName = p_pDestArray->at(j);

                if(sourceName.CompareNoCase(destName) == 0)
                {
                    oFound = TRUE;
                    break;
                }
            }

            if(!oFound)
            {
                p_pDestArray->push_back(sourceName);
            }
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfaceBase::InitErrorHandling()
{
    DeleteErrorHandling();
    m_pErrorHandling = new CErrorHandling();
    return TRUE;
}

BOOL CInterfaceBase::DeleteErrorHandling()
{
    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = 0;
    }

    return TRUE;
}

CInterfaceBase* CInterfaceBase::Clone()
{
    return 0;
}

BOOL CInterfaceBase::IsEqual(CInterfaceBase* p_pInterface)
{
    BOOL oResult = FALSE;

    if(p_pInterface)
    {
        oResult = TRUE;
        if(oResult && (m_strInterfaceName != p_pInterface->m_strInterfaceName)) oResult = FALSE;
        if(oResult && (m_lInstanceValue != p_pInterface->m_lInstanceValue)) oResult = FALSE;
    }

    return oResult;
}

BOOL CInterfaceBase::InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling)
{
    return TRUE;
}

BOOL CInterfaceBase::I_ResetPortEnumeration(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(!pPort->ResetPortEnumeration(p_pErrorInfo))
        {
            oResult = FALSE;
            break;
        }
    }

    return oResult;
}

BOOL CInterfaceBase::I_UpdatePortNameSelection(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = TRUE;

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);

        if(!pPort->UpdateDeviceInfos(p_rOpenPortList, p_pErrorInfo))
        {
            oResult = FALSE;
        }
    }

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);

        if(!pPort->UpdatePortNameSelection(p_rOpenPortList, p_pErrorInfo))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CInterfaceBase::InitInterfaceName(CStdString p_Name)
{
    BOOL oResult(TRUE);

    m_strInterfaceName = p_Name;
    if(m_pGateway) m_pGateway->InitInterfaceName(p_Name);

    return oResult;
}

BOOL CInterfaceBase::InitInterface(CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CInterfaceBase::InitInterface(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CInterfaceBase::InitBaudrateSelection(CStdDWordArray& p_ulBaudrateSel)
{
    BOOL oResult = TRUE;

    //Init baudrate Selection of all ports
    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(pPort)
        {
            if(!pPort->InitBaudrateSelection(p_ulBaudrateSel))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CInterfaceBase::InitDefaultInterfaceSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
    BOOL oResult = TRUE;

    //Init Default Settings of interface
    if(m_pGateway)
    {
        if(!m_pGateway->InitDefaultPortSettings(p_ulBaudrate, p_ulTimeout)) oResult = FALSE;
    }

    //Init baudrate Selection of all ports
    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(pPort)
        {
            if(!pPort->InitDefaultPortSettings(p_ulBaudrate, p_ulTimeout))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

void CInterfaceBase::DeleteGateway()
{
    if(m_pGateway)
    {
        delete m_pGateway;
        m_pGateway = 0;
    }
}

void CInterfaceBase::DeletePortList()
{
    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        delete pPort;
    }
    m_PortList.clear();
}

BOOL CInterfaceBase::I_GetInterfaceName(CStdString* p_pInterfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pInterfaceName)
    {
        *p_pInterfaceName = m_strInterfaceName;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfaceBase::I_GetPortName(CPortBase* p_pPort, CStdString* p_pPortName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->GetPortName(p_pPortName);
    }

    return oResult;
}

BOOL CInterfaceBase::I_GetPortNameSelection(CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CStdStringArray portSel;

    if(p_pPortSel)
    {
        p_pPortSel->clear();
        for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
        {
            CPortBase* pPort = (*it);
            if(pPort->GetPortNameSelection(&portSel, p_pErrorInfo))
            {
                AppendSelectionArray(p_pPortSel, &portSel);
            }
        }

        oResult = SortPortNames(*p_pPortSel);
    }

    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    return oResult;
}

BOOL CInterfaceBase::I_GetInterfaceModeSelection(CStdStringArray* p_pInterfaceModeSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->GetPortModeSelection(p_pInterfaceModeSel, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_GetBaudrateSelection(CStdString p_portName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CPortBase* pPort = 0;

    //Find port
    if(!I_FindPort(m_strInterfaceName, p_portName, &pPort) || (pPort == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PORT_NAME, p_pErrorInfo);
    }
    else
    {
        oResult = TRUE;
    }

    //GetBaudrateSelection
    if(oResult && pPort)
    {
        oResult = pPort->GetBaudrateSelection(p_pulBaudrateSel, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_FindInterfacePortIndex(CStdString p_portName, short* p_psPortIndex)
{
    BOOL oResult(FALSE);
    CStdString name;
    short sIndex = 0;

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(pPort->GetInterfacePortName(&name))
        {
            if(p_portName.CompareNoCase(name) == 0)
            {
                if(p_psPortIndex)
                {
                    *p_psPortIndex = sIndex;
                    oResult = TRUE;
                    break;
                }
            }
        }
        sIndex++;
    }

    if(!oResult && p_psPortIndex) *p_psPortIndex = -1;

    return oResult;
}

BOOL CInterfaceBase::I_AddPort(CStdString p_PortName)
{
    return TRUE;
}

BOOL CInterfaceBase::I_DeletePort(CStdString p_PortName)
{
    return TRUE;
}

BOOL CInterfaceBase::I_FindPort(CStdString p_interfacePortName, CStdString p_portName, CPortBase** p_ppPort)
{
    BOOL oResult(FALSE);
    CStdString listInterfacePortName;

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(pPort->GetInterfacePortName(&listInterfacePortName))
        {
            if(listInterfacePortName.CompareNoCase(p_interfacePortName) == 0)
            {
                if(pPort->IsPortNameSupported(p_portName))
                {
                    if(p_ppPort) *p_ppPort = pPort;
                    oResult = TRUE;
                    break;
                }
            }
        }
    }

    return oResult;
}

BOOL CInterfaceBase::I_OpenInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->OpenInterface(m_strInterfaceName, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_OpenInterfacePort(CPortBase* p_pPort, CStdString p_portName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        if(p_pPort->InitGateway(m_pGateway))
        {
            oResult = p_pPort->OpenPort(p_portName, p_pErrorInfo);
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_CloseInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        if(m_pGateway->CloseInterface(p_pErrorInfo))
        {
            DeleteGateway();
            oResult = TRUE;
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_CloseInterfacePort(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->ClosePort(p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_SetInterfaceSettings(CPortBase* p_pPort, DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->SetPortSettings(p_ulBaudrate, p_ulTimeout, p_oChangeOnly, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_GetInterfaceSettings(CPortBase* p_pPort, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->GetPortSettings(p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_ResetInterface(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->ResetPort(p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::I_SetInterfaceMode(WORD p_usModeIndex, CErrorInfo* p_pErrorInfo)
{
    m_usModeIndex = p_usModeIndex;

    return TRUE;
}

BOOL CInterfaceBase::I_GetInterfaceMode(WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo)
{
    *p_pusModeIndex = m_usModeIndex;

    return TRUE;
}

void CInterfaceBase::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        pPort->InitJournalManager(p_pJournalManager);
    }
}

void CInterfaceBase::ResetJournalManager()
{
    m_pJournalManager = 0;

    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        pPort->ResetJournalManager();
    }
}

BOOL CInterfaceBase::I_GetDefaultInterfaceSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Get Default portSettings
    if(m_pGateway)
    {
        //Defined by interface
        oResult = m_pGateway->GetDefaultPortSettings(p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
    }
    else if(m_PortList.size() > 0)
    {
        //Defined by first port
        CPortBase* pPort = m_PortList.front();
        if(pPort)
        {
            oResult = pPort->GetDefaultPortSettings(p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
        }
    }

    return oResult;
}

BOOL CInterfaceBase::I_SetDefaultInterfaceSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //Init Default Settings of interface
    if(m_pGateway)
    {
        if(!m_pGateway->SetDefaultPortSettings(p_ulBaudrate, p_ulTimeout, p_pErrorInfo)) oResult = FALSE;
    }

    //Init Default Settings of all ports
    for(list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
    {
        CPortBase* pPort = (*it);
        if(pPort != 0)
        {
            if(!pPort->SetDefaultPortSettings(p_ulBaudrate, p_ulTimeout, p_pErrorInfo))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CInterfaceBase::ExecuteCommand(CCommandRoot* p_pCommand, CPortBase* p_pPort, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->ExecuteCommand(p_pCommand, p_hTransactionHandle);
    }
    return oResult;
}

BOOL CInterfaceBase::AbortCommands(BOOL p_oActive)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->AbortCommands(p_oActive);
    }
    return oResult;
}

BOOL CInterfaceBase::EnableTracing(CPortBase* p_pPort, CStdString p_FileName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->EnableTracing(p_FileName, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::DisableTracing(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(p_pPort)
    {
        oResult = p_pPort->DisableTracing(p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterfaceBase::AreParameterEqual(CInterfaceBase* p_pInterface)
{
    BOOL oResult = FALSE;

    if(p_pInterface && m_pGateway)
    {
        oResult = m_pGateway->AreParameterEqual(p_pInterface->m_pGateway);
    }

    return oResult;
}

BOOL CInterfaceBase::SetParameter(CStdString p_name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CInterfaceBase::SetParameter(CStdString p_name, CStdString p_Value)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_name, p_Value);
    }

    return oResult;
}

BOOL CInterfaceBase::GetParameter(CStdString p_name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CInterfaceBase::GetParameter(CStdString p_name, CStdString& p_rValue)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_name, p_rValue);
    }

    return oResult;
}

BOOL CInterfaceBase::Lock(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Lock(p_ulTimeout);
    }

    return oResult;
}

BOOL CInterfaceBase::Unlock()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Unlock();
    }

    return oResult;
}

BOOL CInterfaceBase::IsLocked()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->IsLocked();
    }

    return oResult;
}

BOOL CInterfaceBase::SortPortNames(CStdStringArray& p_rPortNames)
{
    BOOL oExchanged(FALSE);
    CStdString portName(_T(""));
    CStdString nextPortName(_T(""));
    size_t n(0);

    n = p_rPortNames.size() - 1;

    do
    {
        oExchanged = FALSE;
        for(int i = 0; i < (int)n; i++)
        {
            // Port name
            portName = p_rPortNames.at(i);
            // Next port name
            nextPortName = p_rPortNames.at(i + 1);
            // Compare port name
            if(portName > nextPortName)
            {
                // Interchange values
                p_rPortNames[i] = nextPortName;
                p_rPortNames[i + 1] = portName;
                oExchanged = TRUE;
            }
        }
        n = n - 1;
    }
    while(oExchanged);

    return TRUE;
}
