#include "stdafx.h"

#include <CommunicationModel/Interface/Classes/Ports/Port_CAN.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/GatewayCANToSocketCAN.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/Classes/CANDeviceInfoHandling.h>
#include <CommunicationModel/Interface/Classes/Interfaces/Interface_CAN.h>

CInterface_CAN::CInterface_CAN():
    m_pInfoHandling(0)
{
    Init();

    InitErrorHandling();

    m_strInterfaceName = INTERFACE_CAN;
    m_pJournalManager = NULL;
}

CInterface_CAN::CInterface_CAN(const CInterface_CAN& p_rObject):CInterfaceBase(p_rObject)
{
    InitErrorHandling();
    InitJournalManager(p_rObject.m_pJournalManager);

    //Device info handling
    m_pInfoHandling = p_rObject.m_pInfoHandling;
}

CInterface_CAN::~CInterface_CAN()
{
}

CInterfaceBase* CInterface_CAN::Clone()
{
    CInterfaceBase* pInterfaceBase = new CInterface_CAN(*this);
    return pInterfaceBase;
}

void CInterface_CAN::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    CInterfaceBase::InitJournalManager(p_pJournalManager);
}

void CInterface_CAN::ResetJournalManager()
{
    CInterfaceBase::ResetJournalManager();
}

BOOL CInterface_CAN::InitInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    if(!InitInterface(DRIVER_SOCKETCAN, p_pErrorInfo))
    {
       oResult = FALSE;
    }

    return oResult;
}

BOOL CInterface_CAN::InitInterface(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Gateway
    if(InitGateway(p_DriverName))
    {
        //Port
        oResult = InitPort(p_DriverName, p_pErrorInfo);
    }

    return oResult;
}

BOOL CInterface_CAN::InitPort(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CPortBase* pPort(0);

    //Creation
    pPort = new CPort_CAN();

    //Initialisation
    if(pPort)
    {
        pPort->InitInfoHandling(m_pInfoHandling);
        pPort->InitInterfacePortName(m_strInterfaceName);

        if(pPort->InitPort(p_DriverName, m_pGateway, p_pErrorInfo))
        {
            m_PortList.push_back(pPort);
            oResult = TRUE;
        }
        else
        {
            delete pPort;
        }
    }

    return oResult;
}

BOOL CInterface_CAN::InitGateway(CStdString p_DriverName)
{
    BOOL oRetVal(FALSE);
    CGatewayCANToSocketCAN* pGateway(0);

    //Reset
    DeleteGateway();
    //Creation
    if(DRIVER_SOCKETCAN == p_DriverName)
    {
        pGateway = new CGatewayCANToSocketCAN();
    }
    //Initialisation
    if(pGateway)
    {
        oRetVal = TRUE;
        if(!pGateway->InitInfoHandling(m_pInfoHandling))
        {
            oRetVal = FALSE;
        }
        if(!pGateway->InitInterfaceName(m_strInterfaceName))
        {
            oRetVal = FALSE;
        }

        if(oRetVal)
        {
            m_pGateway = pGateway;
            InitParameterSet();
        }
        else
        {
            delete pGateway;
        }
    }

    return oRetVal;
}

BOOL CInterface_CAN::InitParameterSet()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = TRUE;
        if(oResult) oResult &= m_pGateway->ResetParameterSet();
    }

    return oResult;
}

BOOL CInterface_CAN::I_OpenInterface(CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_OpenInterface(p_pErrorInfo);
}

BOOL CInterface_CAN::I_OpenInterfacePort(CPortBase* p_pPort, CStdString p_PortName, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_OpenInterfacePort(p_pPort, p_PortName, p_pErrorInfo);
}

BOOL CInterface_CAN::I_CloseInterface(CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_CloseInterface(p_pErrorInfo);
}

BOOL CInterface_CAN::I_CloseInterfacePort(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_CloseInterfacePort(p_pPort, p_pErrorInfo);
}

BOOL CInterface_CAN::I_SetInterfaceSettings(CPortBase* p_pPort, DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceSettings(p_pPort, p_ulBaudrate, p_ulTimeout, p_oChangeOnly, p_pErrorInfo);
}

BOOL CInterface_CAN::I_GetInterfaceSettings(CPortBase* p_pPort, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceSettings(p_pPort, p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
}

BOOL CInterface_CAN::I_ResetInterface(CPortBase* p_pPort, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_ResetInterface(p_pPort, p_pErrorInfo);
}

BOOL CInterface_CAN::I_SetInterfaceMode(WORD p_usModeIndex, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceMode(p_usModeIndex, p_pErrorInfo);
}

BOOL CInterface_CAN::I_GetInterfaceMode(WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceMode(p_pusModeIndex, p_pErrorInfo);
}

BOOL CInterface_CAN::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("Interface_CAN"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(INTERFACE_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CInterface_CAN::GetInterfaceHardwareNames(CStdStringArray& p_rNames)
{
    BOOL oResult(TRUE);

    if(!GetInterfaceHardwareNames(DRIVER_SOCKETCAN, p_rNames)) oResult = FALSE;

    return oResult;
}

BOOL CInterface_CAN::GetInterfaceHardwareNames(CStdString p_DriverName, CStdStringArray& p_rNames)
{
    BOOL oResult(FALSE);
    CGatewayIToDrv* pGateway(0);

    //Create
    if(DRIVER_SOCKETCAN == p_DriverName)
    {
        pGateway = new CGatewayCANToSocketCAN();
    }
    //Get infos
    if(pGateway)
    {
        oResult = TRUE;
        if(oResult && !pGateway->InitInfoHandling(m_pInfoHandling)) oResult = FALSE;
        if(oResult && !pGateway->InitInterfaceName(m_strInterfaceName)) oResult = FALSE;
        if(oResult && !pGateway->GetInterfaceHardwareNames(p_rNames)) oResult = FALSE;
        delete pGateway;
    }

    return oResult;
}

BOOL CInterface_CAN::InitInfoHandling(CDeviceInfoHandlingBase* p_pDeviceInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CCANDeviceInfoHandling*>(p_pDeviceInfoHandling);

    return oResult;
}
