// Interface_RS232.cpp: Implementierung der Klasse CInterface_RS232.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <CommunicationModel/Interface/Classes/Interfaces/Interface_RS232.h>

#include <CommunicationModel/Interface/Classes/Ports/Port_RS232.h>
#include <CommunicationModel/Interface/Classes/Gateway/RS232/GatewayRS232.h>
#include <CommunicationModel/Interface/Classes/Gateway/RS232/Classes/Rs232DeviceInfoHandling.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////
CInterface_RS232::CInterface_RS232():
    m_pInfoHandling(0)
{
    Init();

    InitErrorHandling();

    m_strInterfaceName = INTERFACE_RS232;
    m_pJournalManager = NULL;
}

CInterface_RS232::CInterface_RS232(const CInterface_RS232& rObject):CInterfaceBase(rObject)
{
    InitErrorHandling();

    //JournalManager
    InitJournalManager(rObject.m_pJournalManager);

    //DeviceInfoHandling
    m_pInfoHandling = rObject.m_pInfoHandling;
}

CInterface_RS232::~CInterface_RS232()
{
}

CInterfaceBase* CInterface_RS232::Clone()
{
    CInterfaceBase* pInterfaceBase = new CInterface_RS232(*this);
    return pInterfaceBase;
}

BOOL CInterface_RS232::I_OpenInterface(CErrorInfo* pErrorInfo)
{
    //No Interface to open
    return TRUE;
}

BOOL CInterface_RS232::I_OpenInterfacePort(CPortBase* pPort, CStdString strPortName, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_OpenInterfacePort(pPort, strPortName, pErrorInfo);
}

BOOL CInterface_RS232::I_CloseInterface(CErrorInfo* pErrorInfo)
{
    //No Interface to close
    return TRUE;
}

BOOL CInterface_RS232::I_CloseInterfacePort(CPortBase* pPort, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_CloseInterfacePort(pPort, pErrorInfo);
}

BOOL CInterface_RS232::I_SetInterfaceSettings(CPortBase* pPort, DWORD dBaudrate, DWORD dTimeout, BOOL oChangeOnly, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceSettings(pPort, dBaudrate, dTimeout, oChangeOnly, pErrorInfo);
}

BOOL CInterface_RS232::I_GetInterfaceSettings(CPortBase* pPort, DWORD* pdBaudrate, DWORD* pdTimeout, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceSettings(pPort, pdBaudrate, pdTimeout, pErrorInfo);
}

BOOL CInterface_RS232::I_ResetInterface(CPortBase* pPort, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_ResetInterface(pPort, pErrorInfo);
}

BOOL CInterface_RS232::I_SetInterfaceMode(WORD wModeIndex, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceMode(wModeIndex, pErrorInfo);
}

BOOL CInterface_RS232::I_GetInterfaceMode(WORD* pwModeIndex, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceMode(pwModeIndex, pErrorInfo);
}

void CInterface_RS232::InitJournalManager(CJournalManagerBase *pJournalManager)
{
    CInterfaceBase::InitJournalManager(pJournalManager);
}

void CInterface_RS232::ResetJournalManager()
{
    CInterfaceBase::ResetJournalManager();
}

BOOL CInterface_RS232::InitInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    if(!InitInterface(DRIVER_RS232, p_pErrorInfo)) oResult = FALSE;

    return oResult;
}

BOOL CInterface_RS232::InitInterface(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Port
    oResult = InitPort(p_DriverName, p_pErrorInfo);

    return oResult;
}

BOOL CInterface_RS232::InitPort(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CPortBase* pPort(0);

    //Creation
    pPort = new CPort_RS232();

    //Initialisation
    if(pPort)
    {
        pPort->InitInfoHandling(m_pInfoHandling);
        pPort->InitInterfacePortName(m_strInterfaceName);
        if(pPort->InitPort(p_DriverName, m_pGateway, p_pErrorInfo))
        {
            m_PortList.push_back(pPort);
            oResult = TRUE;
        }
        else
        {
            delete pPort;
        }
    }

    return oResult;
}

BOOL CInterface_RS232::InitParameterSet()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = TRUE;
        if(oResult) oResult &= m_pGateway->ResetParameterSet();
    }

    return oResult;
}

BOOL CInterface_RS232::InitErrorHandling()
{
    CErrorProducer errorProducer;
    CStdString strClassName = _T("Interface_RS232");

    if(m_pErrorHandling)
    {
        //Init ErrorProducer
        errorProducer.Init(INTERFACE_LAYER, strClassName);
        m_pErrorHandling->InitErrorProducer(&errorProducer);
        return TRUE;
    }

    return FALSE;
}

BOOL CInterface_RS232::InitInfoHandling(CDeviceInfoHandlingBase* p_pDeviceInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CRs232DeviceInfoHandling*>(p_pDeviceInfoHandling);

    return oResult;
}