// Interface_USB.cpp: Implementierung der Klasse CInterface_USB.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <CommunicationModel/Interface/Classes/Interfaces/Interface_USB.h>
#include <CommunicationModel/Interface/Classes/Interfaces/Factory/InterfaceUsbFactory.h>

#include <CommunicationModel/Interface/Classes/Ports/Port_USB.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////
CInterface_USB::CInterface_USB()
{
    Init();

    InitErrorHandling();

    m_strInterfaceName = INTERFACE_USB;
    m_pJournalManager = 0;
    m_pInfoHandling = 0;
}

CInterface_USB::CInterface_USB(const CInterface_USB& rObject):CInterfaceBase(rObject)
{
    InitErrorHandling();

    //JournalManager
    InitJournalManager(rObject.m_pJournalManager);

    //UsbDeviceInfoHandling
    m_pInfoHandling = rObject.m_pInfoHandling;
}

CInterface_USB::~CInterface_USB()
{
}

CInterfaceBase* CInterface_USB::Clone()
{
    CInterfaceBase* pInterfaceBase = new CInterface_USB(*this);
    return pInterfaceBase;
}

BOOL CInterface_USB::InitInterface(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    std::list<CStdString> aIdentifiers;
    oResult = CInterfaceUsbFactory::GetAllIdentifiers(aIdentifiers);

    if(oResult)
    {
        for(std::list<CStdString>::iterator it=aIdentifiers.begin(); it!=aIdentifiers.end() && oResult; it++)
        {
            if(!InitInterface(*it, p_pErrorInfo))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CInterface_USB::InitInterface(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Port
    oResult = InitPort(p_DriverName, p_pErrorInfo);

    return oResult;
}

BOOL CInterface_USB::InitPort(CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CPortBase* pPort(0);

    //Creation
    pPort = new CPort_USB();

    //Initialisation
    if(pPort)
    {
        pPort->InitInfoHandling(m_pInfoHandling);
        pPort->InitInterfacePortName(m_strInterfaceName);
        if(pPort->InitPort(p_DriverName, m_pGateway, p_pErrorInfo))
        {
            m_PortList.push_back(pPort);
            oResult = TRUE;
        }
        else
        {
            delete pPort;
        }
    }

    return oResult;
}

BOOL CInterface_USB::I_OpenInterface(CErrorInfo* pErrorInfo)
{
    //No Interface to open
    return TRUE;
}

BOOL CInterface_USB::I_OpenInterfacePort(CPortBase* pPort, CStdString strPortName, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_OpenInterfacePort(pPort, strPortName, pErrorInfo);
}

BOOL CInterface_USB::I_CloseInterface(CErrorInfo* pErrorInfo)
{
    //No Interface to close
    return TRUE;
}

BOOL CInterface_USB::I_CloseInterfacePort(CPortBase* pPort, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_CloseInterfacePort(pPort, pErrorInfo);
}

BOOL CInterface_USB::I_SetInterfaceSettings(CPortBase* pPort, DWORD dBaudrate, DWORD dTimeout, BOOL oChangeOnly, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceSettings(pPort, dBaudrate, dTimeout, oChangeOnly, pErrorInfo);
}

BOOL CInterface_USB::I_GetInterfaceSettings(CPortBase* pPort, DWORD* pdBaudrate, DWORD* pdTimeout, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceSettings(pPort, pdBaudrate, pdTimeout, pErrorInfo);
}

BOOL CInterface_USB::I_ResetInterface(CPortBase* pPort, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_ResetInterface(pPort, pErrorInfo);
}

BOOL CInterface_USB::I_SetInterfaceMode(WORD wModeIndex, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_SetInterfaceMode(wModeIndex, pErrorInfo);
}

BOOL CInterface_USB::I_GetInterfaceMode(WORD* pwModeIndex, CErrorInfo* pErrorInfo)
{
    return CInterfaceBase::I_GetInterfaceMode(pwModeIndex, pErrorInfo);
}

void CInterface_USB::InitJournalManager(CJournalManagerBase *pJournalManager)
{
    CInterfaceBase::InitJournalManager(pJournalManager);
}

void CInterface_USB::ResetJournalManager()
{
    CInterfaceBase::ResetJournalManager();
}

BOOL CInterface_USB::InitParameterSet()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = TRUE;
        if(oResult) oResult &= m_pGateway->ResetParameterSet();
    }

    return oResult;
}

BOOL CInterface_USB::InitErrorHandling()
{
    CErrorProducer errorProducer;
    CStdString strClassName = _T("Interface_USB");

    if(m_pErrorHandling)
    {
        //Init ErrorProducer
        errorProducer.Init(INTERFACE_LAYER, strClassName);
        m_pErrorHandling->InitErrorProducer(&errorProducer);
        return TRUE;
    }

    return FALSE;
}

BOOL CInterface_USB::InitInfoHandling(CDeviceInfoHandlingBase* pDeviceInfoHandling)
{
    BOOL oResult = TRUE;

    m_pInfoHandling = dynamic_cast<CUsbDeviceInfoHandling*>(pDeviceInfoHandling);

    return oResult;
}

BOOL CInterface_USB::I_ResetPortEnumeration(CErrorInfo* pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pInfoHandling)
    {
        //Init
        if(m_pInfoHandling->ResetDeviceInfos())
        {
            oResult = TRUE;

            for(std::list<CPortBase*>::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
            {
                CPortBase* pPort = (*it);
                if(!pPort->ResetPortEnumeration(pErrorInfo))
                {
                    oResult = FALSE;
                    break;
                }
            }
        }
    }

    return oResult;
}
