#include "stdafx.h"
#include <Log/MmcLog/MmcLog.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/BaseClasses/Command_I.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/GatewayIToDrv.h>
#include <CommunicationModel/Interface/Classes/Gateway/BaseClasses/DeviceInfoHandlingBase.h>

#include <CommunicationModel/Interface/Classes/Ports/BaseClasses/PortBase.h>

CPortBase::CPortBase()
{
    m_strInterfacePortName = _T("");
    m_strPortName = _T("");
    m_oReopenPending = FALSE;

    m_pJournalManager = 0;
    m_pGateway = 0;

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CPortBase::CPortBase(const CPortBase& p_rObject)
{
    //Names
    m_strInterfacePortName = p_rObject.m_strInterfacePortName;
    m_strPortName = p_rObject.m_strPortName;
    m_oReopenPending = p_rObject.m_oReopenPending;

    //Journal manager
    m_pJournalManager = p_rObject.m_pJournalManager;

    //Gateway
    if(p_rObject.m_pGateway)
    {
        m_pGateway = (CGatewayIToDrv*)p_rObject.m_pGateway->Clone();
    }
    else
    {
        m_pGateway = 0;
    }

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CPortBase::~CPortBase()
{
    DeleteGateway();
    DeleteErrorHandling();
}

BOOL CPortBase::InitErrorHandling()
{
    BOOL oResult = TRUE;

    DeleteErrorHandling();
    m_pErrorHandling = new CErrorHandling();

    return oResult;
}

BOOL CPortBase::DeleteErrorHandling()
{
    BOOL oResult = TRUE;

    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = 0;
    }

    return oResult;
}

CPortBase* CPortBase::Clone()
{
    return 0;
}

BOOL CPortBase::IsEqual(CStdString p_PortName)
{
    BOOL oResult = TRUE;

    if(oResult && (m_strPortName != p_PortName))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CPortBase::InitGateway(CGatewayIToDrv* p_pGateway)
{
    return TRUE;
}

BOOL CPortBase::InitInfoHandling(CDeviceInfoHandlingBase* p_pInfoHandling)
{
    return TRUE;
}

BOOL CPortBase::InitPort(CStdString p_DriverName, CGatewayIToDrv* p_pGateway, CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CPortBase::InitBaudrateSelection(CStdDWordArray& p_rulBaudrateSel)
{
    BOOL oResult = FALSE;

    //Init Baudrate Selection of Gateway
    if(m_pGateway)
    {
        oResult = m_pGateway->InitBaudrateSelection(p_rulBaudrateSel);
    }

    return oResult;
}

BOOL CPortBase::InitDefaultPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
    BOOL oResult = FALSE;

    //Init Baudrate Selection of Gateway
    if(m_pGateway)
    {
        oResult = m_pGateway->InitDefaultPortSettings(p_ulBaudrate, p_ulTimeout);
    }

    return oResult;
}

BOOL CPortBase::InitDefaultPortMode(WORD p_usPortMode)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->InitDefaultPortMode(p_usPortMode);
    }

    return oResult;
}

BOOL CPortBase::IsPortNameSupported(CStdString p_PortName)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->IsPortNameSupported(p_PortName);
    }

    return oResult;
}

void CPortBase::DeleteGateway()
{
    if(m_pGateway)
    {
        delete m_pGateway;
        m_pGateway = 0;
    }
}

//Bsp.:Der interfacePortName kann RS232, IXXAT, Vector sein
BOOL CPortBase::GetInterfacePortName(CStdString* p_pInterfacePortName)
{
    BOOL oResult = FALSE;

    if(p_pInterfacePortName)
    {
        *p_pInterfacePortName = m_strInterfacePortName;
        oResult = TRUE;
    }

    return oResult;
}

//Bsp.:Der PortName kann COM1, COM2, CAN1... sein
BOOL CPortBase::GetPortName(CStdString* p_pPortName)
{
    BOOL oResult = FALSE;

    if(p_pPortName)
    {
        *p_pPortName = m_strPortName;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CPortBase::SetPortName(CStdString p_PortName)
{
    BOOL oResult = TRUE;

    m_strPortName = p_PortName;

    return oResult;
}

void CPortBase::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;
}

void CPortBase::ResetJournalManager()
{
    m_pJournalManager = 0;
}

BOOL CPortBase::InitJournalManager(HANDLE p_hHandle, CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;

    return FALSE;
}

BOOL CPortBase::ResetJournalManager(HANDLE p_hHandle)
{
    m_pJournalManager = 0;

    return FALSE;
}

CJournalManagerBase* CPortBase::GetJournalManager()
{
    return m_pJournalManager;
}

BOOL CPortBase::ResetPortEnumeration(CErrorInfo* p_pErrorInfo)
{
    if (m_pErrorHandling)
    {
        m_pErrorHandling->GetError(ERROR_FUNCTION_NOT_SUPPORTED, p_pErrorInfo);
    }

    return FALSE;
}

BOOL CPortBase::UpdateDeviceInfos(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->UpdateDeviceInfos(p_rOpenPortList);
    }

    return oResult;
}

BOOL CPortBase::UpdatePortNameSelection(tPortList& p_rOpenPortList, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->UpdatePortList(p_rOpenPortList);
    }

    return oResult;
}

BOOL CPortBase::OpenPort(CStdString p_PortName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        if(m_pGateway->OpenPort(p_PortName, p_pErrorInfo))
        {
            m_strPortName = p_PortName;
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CPortBase::SetReopenFlag()
{
    m_oReopenPending = TRUE;

    LogHotPlugInReopenMessage(TRUE);

    return TRUE;
}

BOOL CPortBase::ResetReopenFlag()
{
    m_oReopenPending = FALSE;

    LogHotPlugInReopenMessage(FALSE);

    return TRUE;
}

BOOL CPortBase::IsReopenPending()
{
    return m_oReopenPending;
}

BOOL CPortBase::ReloadDriver()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->ReloadDriver();
    }

    return oResult;
}

BOOL CPortBase::ResetPort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->ResetPort(p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::ClosePort(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        if(m_pGateway->ClosePort(p_pErrorInfo))
        {
            DeleteGateway();
            oResult = TRUE;
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetPortSettings(p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, BOOL p_oChangeOnly, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetPortSettings(p_ulBaudrate, p_ulTimeout, p_oChangeOnly, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetDefaultPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetDefaultPortSettings(p_pulBaudrate, p_pulTimeout, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::SetDefaultPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetDefaultPortSettings(p_ulBaudrate, p_ulTimeout, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetPortMode(WORD* p_pusPortMode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetPortMode(p_pusPortMode, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::SetPortMode(WORD p_usPortMode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetPortMode(p_usPortMode, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetPortNameSelection(CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetPortNameSelection(p_pPortSel, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetBaudrateSelection(CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetBaudrateSelection(p_pulBaudrateSel, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::GetPortModeSelection(CStdStringArray* p_pPortModeSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetPortModeSelection(p_pPortModeSel, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::InitInterfacePortName(CStdString p_Name)
{
    BOOL oResult(TRUE);

    m_strInterfacePortName = p_Name;

    if(m_pGateway)
    {
        m_pGateway->InitInterfacePortName(p_Name);
    }

    return oResult;
}

BOOL CPortBase::ExecuteCommand(CCommandRoot* p_pCommand, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CLayerManagerBase* pLayerManager = 0;
    HANDLE hHandle = 0;

    if(p_pCommand)
    {
        //Init references
        p_pCommand->InitGateway(m_pGateway);
        p_pCommand->InitJournalManager(m_pJournalManager);

        //Execute
        oResult = p_pCommand->Execute(pLayerManager, hHandle, p_hTransactionHandle);

        //Reset references
        p_pCommand->ResetGateway();
        p_pCommand->ResetJournalManager();
    }

    return oResult;
}

BOOL CPortBase::GetCommands(CStdString* p_pCommandInfo)
{
    return FALSE;
}

BOOL CPortBase::EnableTracing(CStdString p_FileName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->EnableTracing(p_FileName, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling)
        {
            m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        }
    }

    return oResult;
}

BOOL CPortBase::DisableTracing(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->DisableTracing(p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    }

    return oResult;
}

BOOL CPortBase::AreParameterEqual(CPortBase* p_pPort)
{
    BOOL oResult = FALSE;

    if(p_pPort && m_pGateway)
    {
        oResult = m_pGateway->AreParameterEqual(p_pPort->m_pGateway);
    }

    return oResult;
}

BOOL CPortBase::SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CPortBase::SetParameter(CStdString p_Name, CStdString p_Value)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_Name, p_Value);
    }

    return oResult;
}

BOOL CPortBase::GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CPortBase::GetParameter(CStdString p_Name, CStdString& p_rValue)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_rValue);
    }

    return oResult;
}

BOOL CPortBase::SetParameterSet(CParameterSet& p_rParameterSet)
{
    BOOL oResult = FALSE;

    if (m_pGateway)
    {
        oResult = m_pGateway->SetParameterSet(p_rParameterSet);
    }

    return oResult;
}

BOOL CPortBase::GetParameterSet(CParameterSet& p_rParameterSet)
{
    BOOL oResult = FALSE;

    if (m_pGateway)
    {
        oResult = m_pGateway->GetParameterSet(p_rParameterSet);
    }

    return oResult;
}

BOOL CPortBase::Lock(DWORD p_ulTimeout)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->Lock(p_ulTimeout);
    }

    return oResult;
}

BOOL CPortBase::Unlock()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->Unlock();
    }

    return oResult;
}

BOOL CPortBase::IsLocked()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->IsLocked();
    }

    return oResult;
}

BOOL CPortBase::GetDriverInfos(CStdString& p_rDriverName, CStdString& p_rDriverParameters)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->GetDriverInfos(p_rDriverName, p_rDriverParameters);
    }

    return oResult;
}

void CPortBase::LogHotPlugInReopenMessage(BOOL p_oSetFlag)
{
    if(CMmcLog::IsActive(LMT_INFO, CUserDefinedMessageLevel(_T("HotPlugIn"))))
    {
        CStdString message(_T(""));

        if(p_oSetFlag)
        {
            message.Format(_T("SetReopen Flag: Port %s"), m_strPortName);
        }
        else
        {
            message.Format(_T("ResetReopen Flag: Port %s"), m_strPortName);
        }

        CMmcLog::Log(LMT_INFO, CUserDefinedMessageLevel(_T("HotPlugIn")), message);
    }
}
