#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/Command_I_CAN.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/GatewayCANToSocketCAN.h>
#include <CommunicationModel/Interface/Classes/Gateway/CAN/Classes/CANDeviceInfoHandling.h>

#include <CommunicationModel/Interface/Classes/Ports/Port_CAN.h>

CPort_CAN::CPort_CAN()
{
    InitErrorHandling();

    m_strInterfacePortName = INTERFACE_CAN;

    m_pCommand_TransmitCanFrame = NULL;
    m_pCommand_ReceiveCanFrame = NULL;
    m_pCommand_ReceiveFilteredCanFrame = NULL;
    m_pInfoHandling = NULL;
    InitCommands();

    m_pJournalManager = NULL;

    SetDefault();
}

CPort_CAN::CPort_CAN(CPort_CAN& p_rObject):CPortBase(p_rObject)
{
    InitErrorHandling();

    m_strInterfacePortName = p_rObject.m_strInterfacePortName;
    m_pInfoHandling = p_rObject.m_pInfoHandling;

    p_rObject.ResetPortMembers();

    m_pCommand_TransmitCanFrame = NULL;
    m_pCommand_ReceiveCanFrame = NULL;
    m_pCommand_ReceiveFilteredCanFrame = NULL;
    InitCommands();

    InitJournalManager(p_rObject.m_pJournalManager);
}

CPort_CAN::~CPort_CAN()
{
    DeleteCommands();
}

CPortBase* CPort_CAN::Clone()
{
    CPortBase* pPortBase = new CPort_CAN(*this);

    return pPortBase;
}

void CPort_CAN::SetDefault()
{
    m_strPortName = "";
}

void CPort_CAN::ResetPortMembers()
{
    m_strPortName = "";
}

BOOL CPort_CAN::I_TransmitCanFrame(HANDLE p_hTransactionHandle, DWORD p_ulCobId, BOOL p_oRtr, BYTE p_ubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_DATA_LENGTH = 8;

    CLayerManagerBase* pLayerManager = NULL;
    HANDLE hHandle = 0;
    DWORD ulDataLength;
    BOOL oResult = FALSE;

    if(m_pCommand_TransmitCanFrame)
    {
        //Limit DataLength
        ulDataLength = p_ubDlc;
        if(ulDataLength > MAX_DATA_LENGTH) ulDataLength = MAX_DATA_LENGTH;
        if(ulDataLength > p_ulDataBufferLength) ulDataLength = p_ulDataBufferLength;

        //Set parameter data
        m_pCommand_TransmitCanFrame->ResetStatus();
        m_pCommand_TransmitCanFrame->SetParameterData(0, &p_ulCobId, sizeof(p_ulCobId));
        m_pCommand_TransmitCanFrame->SetParameterData(1, &p_oRtr, sizeof(p_oRtr));
        m_pCommand_TransmitCanFrame->SetParameterData(2, &p_ubDlc, sizeof(p_ubDlc));
        m_pCommand_TransmitCanFrame->SetParameterData(3, p_pDataBuffer, ulDataLength);

        //Execute command
        oResult = m_pCommand_TransmitCanFrame->Execute(pLayerManager, hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_TransmitCanFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CPort_CAN::I_ReceiveCanFrame(HANDLE p_hTransactionHandle, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_DATA_LENGTH = 8;

    CLayerManagerBase* pLayerManager = NULL;
    HANDLE hHandle = 0;
    DWORD ulDataLength;
    BOOL oResult = FALSE;

    if(m_pCommand_ReceiveCanFrame)
    {
        //Limit dataLength
        if(p_pubDlc) ulDataLength = *p_pubDlc; else ulDataLength = MAX_DATA_LENGTH;
        if(ulDataLength > MAX_DATA_LENGTH) ulDataLength = MAX_DATA_LENGTH;
        if(ulDataLength > p_ulDataBufferLength) ulDataLength = p_ulDataBufferLength;

        //Set parameter data
        m_pCommand_ReceiveCanFrame->ResetStatus();

        //Execute command
        oResult = m_pCommand_ReceiveCanFrame->Execute(pLayerManager, hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_ReceiveCanFrame->GetReturnParameterData(0, p_pulCobId, sizeof(*p_pulCobId));
        m_pCommand_ReceiveCanFrame->GetReturnParameterData(1, p_poRtr, sizeof(*p_poRtr));
        m_pCommand_ReceiveCanFrame->GetReturnParameterData(2, p_pubDlc, sizeof(*p_pubDlc));
        m_pCommand_ReceiveCanFrame->GetReturnParameterData(3, p_pDataBuffer, ulDataLength);

        //Get error code
        m_pCommand_ReceiveCanFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CPort_CAN::I_ReceiveFilteredCanFrame(HANDLE p_hTransactionHandle, DWORD p_ulCobIdFilter, BOOL p_oRtrFilter, DWORD* p_pulCobId, BOOL* p_poRtr, BYTE* p_pubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_DATA_LENGTH = 8;

    CLayerManagerBase* pLayerManager = NULL;
    HANDLE hHandle = 0;
    DWORD ulDataLength;
    BOOL oResult = FALSE;

    if(m_pCommand_ReceiveFilteredCanFrame)
    {
        //Limit dataLength
        if(p_pubDlc) ulDataLength = *p_pubDlc; else ulDataLength = MAX_DATA_LENGTH;
        if(ulDataLength > MAX_DATA_LENGTH) ulDataLength = MAX_DATA_LENGTH;
        if(ulDataLength > p_ulDataBufferLength) ulDataLength = p_ulDataBufferLength;

        //Set parameter data
        m_pCommand_ReceiveFilteredCanFrame->ResetStatus();
        m_pCommand_ReceiveFilteredCanFrame->SetParameterData(0, &p_ulCobIdFilter, sizeof(p_ulCobIdFilter));
        m_pCommand_ReceiveFilteredCanFrame->SetParameterData(1, &p_oRtrFilter, sizeof(p_oRtrFilter));

        //Execute command
        oResult = m_pCommand_ReceiveFilteredCanFrame->Execute(pLayerManager, hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_ReceiveFilteredCanFrame->GetReturnParameterData(0, p_pulCobId, sizeof(*p_pulCobId));
        m_pCommand_ReceiveFilteredCanFrame->GetReturnParameterData(1, p_poRtr, sizeof(*p_poRtr));
        m_pCommand_ReceiveFilteredCanFrame->GetReturnParameterData(2, p_pubDlc, sizeof(*p_pubDlc));
        m_pCommand_ReceiveFilteredCanFrame->GetReturnParameterData(3, p_pDataBuffer, ulDataLength);

        //Get error code
        m_pCommand_ReceiveFilteredCanFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CPort_CAN::InitCommands()
{
    DeleteCommands();

    //Init command TransmitCanFrame
    m_pCommand_TransmitCanFrame = new CCommand_I_CAN();
    m_pCommand_TransmitCanFrame->InitCommand(CAN_TRANSMIT_CAN_FRAME);

    //Init command ReceiveCanFrame
    m_pCommand_ReceiveCanFrame = new CCommand_I_CAN();
    m_pCommand_ReceiveCanFrame->InitCommand(CAN_RECEIVE_FILTERED_CAN_FRAME);

    //Init command ReceiveFilteredCanFrame
    m_pCommand_ReceiveFilteredCanFrame = new CCommand_I_CAN();
    m_pCommand_ReceiveFilteredCanFrame->InitCommand(CAN_RECEIVE_FILTERED_CAN_FRAME);
}

BOOL CPort_CAN::GetCommands(CStdString* p_pCommandInfo)
{
    BOOL oResult(FALSE);
    CXXMLFile xmlFile;
    CXXMLFile::CElementPart* pElementPart;
    CXXMLFile::CElement* pElement;

    if(p_pCommandInfo)
    {
        //Root
        pElementPart = xmlFile.Root();
        if(xmlFile.IsElement(pElementPart))
        {
            oResult = TRUE;

            //CommandInfo elements
            pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElementPart);
            xmlFile.SetText(pElement, "CommandInfo");

            //CommandGroup elements
            pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElement);
            xmlFile.SetText(pElement, "CommandGroup");
            pElement->SetAt("Name", m_strPortName);

            //Command elements
            if(oResult && m_pCommand_TransmitCanFrame && !m_pCommand_TransmitCanFrame->StoreToXMLFile(&xmlFile, pElement, TRUE)) oResult = FALSE;
            if(oResult && m_pCommand_ReceiveCanFrame && !m_pCommand_ReceiveCanFrame->StoreToXMLFile(&xmlFile, pElement, TRUE)) oResult = FALSE;
            if(oResult && m_pCommand_ReceiveFilteredCanFrame && !m_pCommand_ReceiveFilteredCanFrame->StoreToXMLFile(&xmlFile, pElement, TRUE)) oResult = FALSE;

            //Write to string
            if(oResult)
            {
                xmlFile.WriteToString(p_pCommandInfo);
            }
        }
    }

    return oResult;
}

void CPort_CAN::DeleteCommands()
{
    if(m_pCommand_TransmitCanFrame)
    {
        delete m_pCommand_TransmitCanFrame;
        m_pCommand_TransmitCanFrame = NULL;
    }

    if(m_pCommand_ReceiveCanFrame)
    {
        delete m_pCommand_ReceiveCanFrame;
        m_pCommand_ReceiveCanFrame = NULL;
    }

    if(m_pCommand_ReceiveFilteredCanFrame)
    {
        delete m_pCommand_ReceiveFilteredCanFrame;
        m_pCommand_ReceiveFilteredCanFrame = NULL;
    }
}

void CPort_CAN::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    CPortBase::InitJournalManager(p_pJournalManager);
    if(m_pCommand_TransmitCanFrame) m_pCommand_TransmitCanFrame->InitJournalManager(p_pJournalManager);
    if(m_pCommand_ReceiveCanFrame) m_pCommand_ReceiveCanFrame->InitJournalManager(p_pJournalManager);
    if(m_pCommand_ReceiveFilteredCanFrame) m_pCommand_ReceiveFilteredCanFrame->InitJournalManager(p_pJournalManager);
}

void CPort_CAN::ResetJournalManager()
{
    CPortBase::ResetJournalManager();
    if(m_pCommand_TransmitCanFrame) m_pCommand_TransmitCanFrame->ResetJournalManager();
    if(m_pCommand_ReceiveCanFrame) m_pCommand_ReceiveCanFrame->ResetJournalManager();
    if(m_pCommand_ReceiveFilteredCanFrame) m_pCommand_ReceiveFilteredCanFrame->ResetJournalManager();
}

void CPort_CAN::InitCommands(CGatewayIToDrv* p_pGateway)
{
    if(m_pCommand_TransmitCanFrame) m_pCommand_TransmitCanFrame->InitGateway(p_pGateway);
    if(m_pCommand_ReceiveCanFrame) m_pCommand_ReceiveCanFrame->InitGateway(p_pGateway);
    if(m_pCommand_ReceiveFilteredCanFrame) m_pCommand_ReceiveFilteredCanFrame->InitGateway(p_pGateway);
}

BOOL CPort_CAN::InitParameterSet()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = TRUE;
        if(oResult) oResult &= m_pGateway->ResetParameterSet();
    }

    return oResult;
}

BOOL CPort_CAN::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("Port_CAN"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(INTERFACE_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CPort_CAN::InitPort(CStdString p_DriverName, CGatewayIToDrv* p_pGateway, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Gateway
    if(InitGateway(p_DriverName, p_pGateway))
    {
        InitParameterSet();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CPort_CAN::InitGateway(CStdString p_DriverName, CGatewayIToDrv* p_pGateway)
{
    BOOL oRetVal(FALSE);
    CGatewayIToDrv* pGateway(0);

    //Creation
    DeleteGateway();
    if(p_pGateway)
    {
        //Clone gateway prototype
        pGateway = (CGatewayIToDrv*)p_pGateway->Clone();
    }
    else if(DRIVER_XLIB == p_DriverName)
    {
        //Create gateway
        pGateway = new CGatewayCANToSocketCAN();
    }

    //Initialisation
    if(pGateway)
    {
        oRetVal = TRUE;
        if(!pGateway->InitInfoHandling(m_pInfoHandling)) oRetVal = FALSE;
        if(!pGateway->InitDriverInfos(p_DriverName, _T(""))) oRetVal = FALSE;
        if(!pGateway->InitInterfacePortName(m_strInterfacePortName)) oRetVal = FALSE;
        if(!pGateway->InitPortList()) oRetVal = FALSE;

        if(oRetVal)
        {
            m_pGateway = pGateway;
        }
        else
        {
            delete pGateway;
        }
    }

    return oRetVal;
}

BOOL CPort_CAN::InitInfoHandling(CDeviceInfoHandlingBase* p_pDeviceInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CCANDeviceInfoHandling*>(p_pDeviceInfoHandling);

    return oResult;
}

BOOL CPort_CAN::InitGateway(CGatewayIToDrv* p_pGateway)
{
    BOOL oResult(FALSE);

    DeleteGateway();

    if(p_pGateway)
    {
        m_pGateway = (CGatewayIToDrv*)p_pGateway->Clone();

        oResult = TRUE;
    }

    return oResult;
}
