// Port_USB.cpp: Implementierung der Klasse CPort_USB.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <string.h>
#include <algorithm>

#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/BaseClasses/Command_I.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Interface/Command_I_USB.h>
#include <CommunicationModel/Interface/Classes/Interfaces/Factory/InterfaceUsbFactory.h>

#include <CommunicationModel/Interface/Classes/Ports/Port_USB.h>

CPort_USB::CPort_USB()
{
    InitErrorHandling();

    m_strInterfacePortName = INTERFACE_USB;

    m_pCommand_WriteData = 0;
    m_pCommand_ReadData = 0;
    m_pInfoHandling = 0;
    InitCommands();

    SetDefault();
}

CPort_USB::CPort_USB(CPort_USB& rObject):CPortBase(rObject)
{
    InitErrorHandling();

    m_strInterfacePortName = rObject.m_strInterfacePortName;
    rObject.ResetPortMembers();

    m_pCommand_WriteData = 0;
    m_pCommand_ReadData = 0;
    m_pInfoHandling = rObject.m_pInfoHandling;

    InitCommands();
    InitJournalManager(rObject.m_pJournalManager);
}

CPort_USB::~CPort_USB()
{
    DeleteCommands();
}

CPortBase* CPort_USB::Clone()
{
    CPortBase* pPortBase = new CPort_USB(*this);
    return pPortBase;
}

void CPort_USB::SetDefault()
{
    m_strPortName = _T("");
}

BOOL CPort_USB::operator==(const CPort_USB& rPort)
{
    CStdString strPortName1 = m_strPortName;
    CStdString strPortName2 = rPort.m_strPortName;

    return (strPortName1.CompareNoCase(strPortName2) == 0);
}

CPort_USB& CPort_USB::operator=(CPort_USB& rSetting)
{
    if(this != &rSetting)
    {
        m_strPortName = rSetting.m_strPortName;
        m_pInfoHandling = rSetting.m_pInfoHandling;
    }

    return *this;
}

void CPort_USB::ResetPortMembers()
{
    m_strPortName = _T("");
}

BOOL CPort_USB::I_WriteData(HANDLE hTransactionHandle, void* pData, DWORD dNumberOfBytesToWrite, DWORD* pdNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    CLayerManagerBase* pLayerManager = 0;
    HANDLE hHandle = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_WriteData)
    {
        //Set Parameter Data
        m_pCommand_WriteData->ResetStatus();
        m_pCommand_WriteData->SetParameterData(0, pData, dNumberOfBytesToWrite);

        //Execute Command
        oResult = m_pCommand_WriteData->Execute(pLayerManager, hHandle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_WriteData->GetReturnParameterData(0, pdNumberOfBytesWritten, sizeof(*pdNumberOfBytesWritten));

        //Get ErrorCode
        m_pCommand_WriteData->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CPort_USB::I_ReadData(HANDLE hTransactionHandle, void* pData, DWORD dNumberOfBytesToRead, DWORD* pdNumberOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    CLayerManagerBase* pLayerManager = 0;
    HANDLE hHandle = 0;
    BOOL oResult = FALSE;
    DWORD dataLength;

    if(m_pCommand_ReadData)
    {
        //Set Parameter Data
        m_pCommand_ReadData->ResetStatus();
        m_pCommand_ReadData->SetParameterData(0, &dNumberOfBytesToRead, sizeof(dNumberOfBytesToRead));

        //Execute Command
        oResult = m_pCommand_ReadData->Execute(pLayerManager, hHandle, hTransactionHandle);

        //Limit Parameter Length
        dataLength = m_pCommand_ReadData->GetReturnParameterLength(0);
        if(dataLength > dNumberOfBytesToRead) dataLength = dNumberOfBytesToRead;

        //Get ReturnParameter Data
        m_pCommand_ReadData->GetReturnParameterData(0, pData, dataLength);
        if(pdNumberOfBytesRead) *pdNumberOfBytesRead = dataLength;

        //Get ErrorCode
        m_pCommand_ReadData->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CPort_USB::InitCommands()
{
    DeleteCommands();

    //Init Command WriteData
    m_pCommand_WriteData = new CCommand_I_USB();
    m_pCommand_WriteData->InitCommand(USB_WRITE_DATA);

    //Init Command ReadData
    m_pCommand_ReadData = new CCommand_I_USB();
    m_pCommand_ReadData->InitCommand(USB_READ_DATA);
}

BOOL CPort_USB::GetCommands(CStdString* pCommandInfo)
{
    CXXMLFile xmlFile;
    CXXMLFile::CElementPart* pElementPart;
    CXXMLFile::CElement* pElement;
    BOOL oResult = TRUE;

    if(pCommandInfo)
    {
        //Root
        pElementPart = xmlFile.Root();
        if(!xmlFile.IsElement(pElementPart)) return FALSE;

        //CommandInfo Elements
        pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElementPart);
        xmlFile.SetText(pElement, "CommandInfo");

        //CommandGroup Elements
        pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElement);
        xmlFile.SetText(pElement, "CommandGroup");
        pElement->SetAt("Name", m_strPortName);

        //Command Elements
        if(m_pCommand_ReadData && !m_pCommand_ReadData->StoreToXMLFile(&xmlFile, pElement, TRUE)) return FALSE;
        if(m_pCommand_WriteData && !m_pCommand_WriteData->StoreToXMLFile(&xmlFile, pElement, TRUE)) return FALSE;

        //Write to string
        xmlFile.WriteToString(pCommandInfo);
        return oResult;
    }

    return FALSE;
}

void CPort_USB::DeleteCommands()
{
    if(m_pCommand_ReadData)
    {
        delete m_pCommand_ReadData;
        m_pCommand_ReadData = 0;
    }

    if(m_pCommand_WriteData)
    {
        delete m_pCommand_WriteData;
        m_pCommand_WriteData = 0;
    }
}

void CPort_USB::InitJournalManager(CJournalManagerBase *pJournalManager)
{
    CPortBase::InitJournalManager(pJournalManager);
    if(m_pCommand_WriteData) m_pCommand_WriteData->InitJournalManager(pJournalManager);
    if(m_pCommand_ReadData) m_pCommand_ReadData->InitJournalManager(pJournalManager);
}

void CPort_USB::ResetJournalManager()
{
    CPortBase::ResetJournalManager();
    if(m_pCommand_WriteData) m_pCommand_WriteData->ResetJournalManager();
    if(m_pCommand_ReadData) m_pCommand_ReadData->ResetJournalManager();
}

void CPort_USB::InitCommands(CGatewayIToDrv* pGateway)
{
    if(m_pCommand_WriteData) m_pCommand_WriteData->InitGateway(pGateway);
    if(m_pCommand_ReadData) m_pCommand_ReadData->InitGateway(pGateway);
}

BOOL CPort_USB::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className = _T("Port_USB");

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(INTERFACE_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CPort_USB::InitInfoHandling(CDeviceInfoHandlingBase* p_pDeviceInfoHandling)
{
    BOOL oResult(TRUE);

    m_pInfoHandling = dynamic_cast<CUsbDeviceInfoHandling*>(p_pDeviceInfoHandling);

    return oResult;
}

BOOL CPort_USB::InitPort(CStdString p_DriverName, CGatewayIToDrv* p_pGateway, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    //Gateway
    if(InitGateway(p_DriverName, p_pGateway))
    {
        InitParameterSet();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CPort_USB::InitGateway(CStdString p_DriverName, CGatewayIToDrv* p_pGateway)
{
    BOOL oRetVal(FALSE);
    CGatewayIToDrv* pGateway(0);
    CStdString driverName(_T(""));
    CStdString driverParameters(_T(""));

    if(ExtractDriverInfos(p_DriverName, driverName, driverParameters))
    {
        //Creation
        DeleteGateway();
        if(p_pGateway)
        {
            //Clone gateway prototype
            pGateway = (CGatewayIToDrv*)p_pGateway->Clone();
        }
        else
        {
            //Create gateway
            CInterfaceUsbFactory::CreateInstance(driverName, pGateway);
        }

        //Initialisation
        if(pGateway)
        {
            oRetVal = TRUE;
            if(!pGateway->InitInfoHandling(m_pInfoHandling)) oRetVal = FALSE;
            if(!pGateway->InitDriverInfos(driverName, driverParameters)) oRetVal = FALSE;
            if(!pGateway->InitInterfacePortName(m_strInterfacePortName)) oRetVal = FALSE;
            if(!pGateway->InitPortList()) oRetVal = FALSE;

            if(oRetVal)
            {
                m_pGateway = pGateway;
            }
            else
            {
                delete pGateway;
            }
        }
    }

    return oRetVal;
}

BOOL CPort_USB::ExtractDriverInfos(CStdString p_DriverName, CStdString& p_rDriverName, CStdString& p_rDriverParameters)
{
    BOOL oResult(FALSE);
    int lIndex(0);

    if(p_DriverName.size() != 0)
    {
        //Expected Format: "DriverName DriverParameters"
        p_DriverName = p_DriverName.Trim();
        lIndex = p_DriverName.Find(_T(" "));
        if(lIndex != -1)
        {
            p_rDriverName = p_DriverName.Left(lIndex);
            p_rDriverParameters = p_DriverName.Right((int)p_DriverName.length() - lIndex -1);
            oResult = TRUE;
        }
        else
        {
            p_rDriverName = p_DriverName;
            p_rDriverParameters = _T("");
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CPort_USB::InitParameterSet()
{
    const DWORD DEFAULT_WRITE_INHIBIT_TIME(0);

    BOOL oResult(FALSE);

    if (m_pGateway)
    {
        oResult = m_pGateway->ResetParameterSet();

        if (oResult)
        {
            oResult = m_pGateway->AddParameter(_T("WriteInhibitTimeInMicroSeconds"), (BYTE*)&DEFAULT_WRITE_INHIBIT_TIME, sizeof(DEFAULT_WRITE_INHIBIT_TIME));
        }
    }

    return oResult;
}

BOOL CPort_USB::ResetPortEnumeration(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = m_pGateway->InitPortList();
    }

    return oResult;
}
