#include "stdafx.h"
#include <malloc.h>
#include <memory.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Epos.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEposDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/BaseClasses/Command_PS.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/Command_PS_CANopen.h>
#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <Log/MmcLog/MmcLog.h>

#include <CommunicationModel/ProtocolStack/Classes/GatewayToDevice/CANopen/GatewayCANopenToEpos.h>

//Constants
const DWORD MAX_RS232_DATA_SIZE(7);

CGatewayCANopenToEpos::CGatewayCANopenToEpos()
{
    m_pCommand_WriteObject = NULL;
    m_pCommand_InitiateSegmentedWrite = NULL;
    m_pCommand_SegmentedWrite = NULL;
    m_pCommand_ReadObject = NULL;
    m_pCommand_InitiateSegmentedRead = NULL;
    m_pCommand_SegmentedRead = NULL;
    m_pCommand_AbortSegmentedTransfer = NULL;
    m_pCommand_SendNMTService = NULL;
    m_pCommand_RequestCANFrame = NULL;
    m_pCommand_SendCANFrame = NULL;

    m_pBufferedData = NULL;
    m_dBufferedDataSize = 0;
    m_dBufferedDataRead = 0;
    m_oNoMoreSegments = FALSE;
    m_oToggle = FALSE;
    m_WriteSegmentSize = 0;
    ResetBufferedData();

    InitCommands();
}

CGatewayCANopenToEpos::~CGatewayCANopenToEpos()
{
    ResetBufferedData();
    DeleteCommands();
}

CGateway* CGatewayCANopenToEpos::Clone()
{
    CGatewayCANopenToEpos* pClonedGateway;

    pClonedGateway = new CGatewayCANopenToEpos();
    *pClonedGateway = *this;

    return pClonedGateway;
}

void CGatewayCANopenToEpos::InitCommands()
{
    DeleteCommands();

    //WriteObject
    m_pCommand_WriteObject = new CCommand_DCS_Epos();
    m_pCommand_WriteObject->InitCommand(EPOS_WRITE_OBJECT);
    //ReadObject
    m_pCommand_ReadObject = new CCommand_DCS_Epos();
    m_pCommand_ReadObject->InitCommand(EPOS_READ_OBJECT);
    //InitiateSegmentedWrite
    m_pCommand_InitiateSegmentedWrite = new CCommand_DCS_Epos();
    m_pCommand_InitiateSegmentedWrite->InitCommand(EPOS_INITIATE_SEGMENT_WRITE);
    //InitiateSegmentedRead
    m_pCommand_InitiateSegmentedRead = new CCommand_DCS_Epos();
    m_pCommand_InitiateSegmentedRead->InitCommand(EPOS_INITIATE_SEGMENT_READ);
    //SegmentedWrite
    m_pCommand_SegmentedWrite = new CCommand_DCS_Epos();
    m_pCommand_SegmentedWrite->InitCommand(EPOS_SEGMENT_WRITE);
    //SegmentedRead
    m_pCommand_SegmentedRead = new CCommand_DCS_Epos();
    m_pCommand_SegmentedRead->InitCommand(EPOS_SEGMENT_READ);
    //AbortSegmentedTransfer
    m_pCommand_AbortSegmentedTransfer = new CCommand_DCS_Epos();
    m_pCommand_AbortSegmentedTransfer->InitCommand(EPOS_ABORT_SEGMENT_TRANSFER);
    //Init Command SendNMTService
    m_pCommand_SendNMTService = new CCommand_DCS_Epos();
    m_pCommand_SendNMTService->InitCommand(EPOS_SEND_NMT_SERVICE);
    //Init Command Request
    m_pCommand_RequestCANFrame = new CCommand_DCS_Epos();
    m_pCommand_RequestCANFrame->InitCommand(EPOS_REQUEST_CAN_FRAME);
    //Init Command Send
    m_pCommand_SendCANFrame = new CCommand_DCS_Epos();
    m_pCommand_SendCANFrame->InitCommand(EPOS_SEND_CAN_FRAME);
}

void CGatewayCANopenToEpos::DeleteCommands()
{
    if(m_pCommand_WriteObject)
    {
        delete m_pCommand_WriteObject;
        m_pCommand_WriteObject = NULL;
    }
    if(m_pCommand_InitiateSegmentedWrite)
    {
        delete m_pCommand_InitiateSegmentedWrite;
        m_pCommand_InitiateSegmentedWrite = NULL;
    }
    if(m_pCommand_SegmentedWrite)
    {
        delete m_pCommand_SegmentedWrite;
        m_pCommand_SegmentedWrite = NULL;
    }
    if(m_pCommand_ReadObject)
    {
        delete m_pCommand_ReadObject;
        m_pCommand_ReadObject = NULL;
    }
    if(m_pCommand_InitiateSegmentedRead)
    {
        delete m_pCommand_InitiateSegmentedRead;
        m_pCommand_InitiateSegmentedRead = NULL;
    }
    if(m_pCommand_SegmentedRead)
    {
        delete m_pCommand_SegmentedRead;
        m_pCommand_SegmentedRead = NULL;
    }
    if(m_pCommand_AbortSegmentedTransfer)
    {
        delete m_pCommand_AbortSegmentedTransfer;
        m_pCommand_AbortSegmentedTransfer = NULL;
    }
    if(m_pCommand_SendNMTService)
    {
        delete m_pCommand_SendNMTService;
        m_pCommand_SendNMTService = NULL;
    }
    if(m_pCommand_RequestCANFrame)
    {
        delete m_pCommand_RequestCANFrame;
        m_pCommand_RequestCANFrame = NULL;
    }
    if(m_pCommand_SendCANFrame)
    {
        delete m_pCommand_SendCANFrame;
        m_pCommand_SendCANFrame = NULL;
    }
}

CGatewayCANopenToEpos& CGatewayCANopenToEpos::operator=(CGatewayCANopenToEpos& other)
{
    if(this != &other)
    {
        *((CGatewayPStoDCS*)this) = *((CGatewayPStoDCS*)&other);
    }

    return *this;
}

BOOL CGatewayCANopenToEpos::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* pManager, HANDLE h_Handle, HANDLE hTransactionHandle)
{
    CCommand_PS* pCommand_PS;
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager;
    BOOL oResult = FALSE;

    if(CGateway::ProcessCommand(p_pCommand, pManager, h_Handle, hTransactionHandle))
    {
        if(CheckLayers(p_pCommand, pManager))
        {
            pCommand_PS = (CCommand_PS*)p_pCommand;
            pDeviceCommandSetManager = (CDeviceCommandSetManagerBase*)pManager;

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayCANopenToEpos")));
            }

            switch(p_pCommand->GetCommandId())
            {
                case CANOPEN_INITIATE_SDO_DOWNLOAD: oResult = Process_InitiateSDODownload(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_DOWNLOAD_SDO_SEGMENT: oResult = Process_DownloadSDOSegment(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_INITIATE_SDO_UPLOAD: oResult = Process_InitiateSDOUpload(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_UPLOAD_SDO_SEGMENT: oResult = Process_UploadSDOSegment(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_NETWORK_INDICATION: oResult = Process_NetworkIndication(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_ABORT_SDO_TRANSFER: oResult = Process_AbortSDOTransfer(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_NMT_SERVICE: oResult = Process_SendNMTService(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_CAN_FRAME: oResult = Process_SendCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_READ_CAN_FRAME: oResult = Process_ReadCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_REQUEST_CAN_FRAME: oResult = Process_RequestCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_LSS_FRAME: oResult = Process_SendLSSFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_READ_LSS_FRAME: oResult = Process_ReadLSSFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
            }

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayCANopenToEpos"), TRUE));
            }
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::Process_InitiateSDODownload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_DATA_SIZE = 4;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_EXPEDITED_TRANSFER = 2;
    const int PARAMETER_INDEX_SIZE_INDICATED = 3;
    const int PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 4;
    const int PARAMETER_INDEX_INDEX = 5;
    const int PARAMETER_INDEX_SUB_INDEX = 6;
    const int PARAMETER_INDEX_DATA = 7;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_INDEX = 1;
    const int RETURN_PARAMETER_INDEX_SUB_INDEX = 2;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 3;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oExpeditedTransfer = 0;
    BOOL oSizeIndicated = FALSE;
    BYTE uNonValidNbOfBytes = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;
    DWORD dObjectLength = 0;

    //ReturnParameter CANopen
    DWORD dAbortCode = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdClientServer,sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_EXPEDITED_TRANSFER,&oExpeditedTransfer,sizeof(oExpeditedTransfer));
        pCommand->GetParameterData(PARAMETER_INDEX_SIZE_INDICATED,&oSizeIndicated,sizeof(oSizeIndicated));
        pCommand->GetParameterData(PARAMETER_INDEX_NON_VALID_NB_OF_BYTES,&uNonValidNbOfBytes,sizeof(uNonValidNbOfBytes));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX,&wIndex,sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX,&uSubIndex,sizeof(uSubIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA,pDataBuffer,dDataBufferLength);

        //Check DataBuffer Length
        dDataBufferLength = CheckDataBufferLength(oSizeIndicated,uNonValidNbOfBytes,dDataBufferLength,MAX_DATA_SIZE);

        //Translate Command
        if(oExpeditedTransfer)
        {
            //Expedited Transfer
            oResult = DCS_WriteObject(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),wIndex,uSubIndex,(BYTE*)pDataBuffer,dDataBufferLength,&errorInfo);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else
        {
            //NonExpedited Transfer
            if(oSizeIndicated && (dDataBufferLength >= MAX_DATA_SIZE))
            {
                dObjectLength = *((DWORD*)pDataBuffer);
                oResult = DCS_InitiateSegmentedWrite(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),wIndex,uSubIndex,dObjectLength,&errorInfo);
                dAbortCode = errorInfo.GetErrorCode();
                m_oToggle = FALSE;

                m_WriteSegmentSize = MAX_RS232_DATA_SIZE;
            }
            else
            {
                oResult = FALSE;
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL,&errorInfo);
                dAbortCode = errorInfo.GetErrorCode();
            }
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_INDEX,&wIndex,sizeof(wIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SUB_INDEX,&uSubIndex,sizeof(uSubIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE,&dAbortCode,sizeof(dAbortCode));

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);
        ResetBufferedData();

        //Unlock critical section
        if(oExpeditedTransfer || !oResult)
        {
            Unlock();
        }

        return oResult;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_AbortSDOTransfer(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //*Constants PS*
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_INDEX = 1;
    const int PARAMETER_INDEX_SUB_INDEX = 2;
    const int PARAMETER_INDEX_ABORT_CODE = 3;

    //*Variables PS*
    //Parameter
    DWORD dCobIdClientServer = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;
    DWORD dAbortCode = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdClientServer,sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX,&wIndex,sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX,&uSubIndex,sizeof(uSubIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_ABORT_CODE,&dAbortCode,sizeof(dAbortCode));

        //Execute command
        oResult = DCS_AbortSegmentedTransfer(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, GetNodeId(dCobIdClientServer), wIndex, (WORD)uSubIndex, dAbortCode, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult,&errorInfo);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::Process_NetworkIndication(CCommand_PS* p_pCommand, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    //Not supported
    return TRUE;
}

BOOL CGatewayCANopenToEpos::Process_DownloadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_CANOPEN_DATA_SIZE = 7;

    //Constants PS CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_TOGGLE = 2;
    const int PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 3;
    const int PARAMETER_INDEX_NO_MORE_SEGMENTS = 4;
    const int PARAMETER_INDEX_SEG_DATA = 5;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_TOGGLE = 1;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 2;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oToggle = 0;
    BYTE uNonValidNbOfBytes = 0;
    BOOL oNoMoreSegments = FALSE;
    void* pSegDataBuffer = NULL;
    DWORD dSegDataBufferLength = 0;

    //ReturnParameter CANopen
    DWORD dAbortCode = 0;

    //ReturnParameter Esam
    BOOL oRetToggle = FALSE;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;
    DWORD dDataLengthRead = 0;
    DWORD dDataLengthWritten = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Check CriticalSection
        if(!IsLocked(pCommand)) return FALSE;

        //Prepare DataBuffer
        dSegDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_SEG_DATA);
        if(dSegDataBufferLength > 0) pSegDataBuffer = malloc(dSegDataBufferLength);
        dDataBufferLength = m_WriteSegmentSize;
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdClientServer,sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_TOGGLE,&oToggle,sizeof(oToggle));
        pCommand->GetParameterData(PARAMETER_INDEX_NON_VALID_NB_OF_BYTES,&uNonValidNbOfBytes,sizeof(uNonValidNbOfBytes));
        pCommand->GetParameterData(PARAMETER_INDEX_NO_MORE_SEGMENTS,&oNoMoreSegments,sizeof(oNoMoreSegments));
        pCommand->GetParameterData(PARAMETER_INDEX_SEG_DATA,pSegDataBuffer,dSegDataBufferLength);

        //Check DataBuffer Length
        dSegDataBufferLength = CheckDataBufferLength(TRUE,uNonValidNbOfBytes,dSegDataBufferLength,MAX_CANOPEN_DATA_SIZE);

        //Translate Command
        oResult = AddBufferedData(pSegDataBuffer,dSegDataBufferLength,oNoMoreSegments);
        if(oResult)
        {
            while(GetBufferedData(pDataBuffer,dDataBufferLength,&dDataLengthRead,&oNoMoreSegments))
            {
                //AbortCommand
                if(m_oAbortCommands)
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER,&errorInfo);
                    oResult = FALSE;
                    break;
                }

                oResult = DCS_SegmentedWrite(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),!oNoMoreSegments,m_oToggle,pDataBuffer,dDataLengthRead,&oRetToggle,&dDataLengthWritten,&errorInfo);
                m_oToggle = !m_oToggle;

                //Update SegmentLength
                if(dDataLengthWritten < dDataLengthRead)
                {
                    ReverseBufferedData(dDataLengthRead-dDataLengthWritten);
                    oNoMoreSegments = FALSE;
                    if(pDataBuffer)
                    {
                        free(pDataBuffer);
                        pDataBuffer = NULL;

                        m_WriteSegmentSize = dDataLengthWritten;
                        dDataBufferLength = dDataLengthWritten;

                        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);
                    }
                }

                if(oNoMoreSegments) break;
            }
        }
        dAbortCode = errorInfo.GetErrorCode();

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TOGGLE,&oToggle,sizeof(oToggle));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE,&dAbortCode,sizeof(dAbortCode));

        //Free DataBuffer
        if(pSegDataBuffer) free(pSegDataBuffer);
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        if(oNoMoreSegments || !oResult)
        {
            ResetBufferedData();
            Unlock();
        }

        return oResult;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_InitiateSDOUpload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_DATA_SIZE = 4;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_INDEX = 2;
    const int PARAMETER_INDEX_SUB_INDEX = 3;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_EXPEDITED_TRANSFER = 1;
    const int RETURN_PARAMETER_INDEX_SIZE_INDICATED = 2;
    const int RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 3;
    const int RETURN_PARAMETER_INDEX_INDEX = 4;
    const int RETURN_PARAMETER_INDEX_SUB_INDEX = 5;
    const int RETURN_PARAMETER_INDEX_DATA = 6;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 7;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;

    //ReturnParameter CANopen
    BOOL oRetExpeditedTransfer = FALSE;
    BOOL oRetSizeIndicated = FALSE;
    BYTE uRetNonValidNbOfBytes = 0;
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;
    DWORD dAbortCode = 0;

    //ReturnParameter
    DWORD dObjectLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Prepare DataBuffer
        dRetDataBufferLength = MAX_DATA_SIZE;
        pRetDataBuffer = malloc(dRetDataBufferLength);
        memset(pRetDataBuffer,0,dRetDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdClientServer,sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX,&wIndex,sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX,&uSubIndex,sizeof(uSubIndex));

        //Translate Command
        oResult = DCS_ReadObject(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),wIndex,uSubIndex,(BYTE*)pRetDataBuffer,dRetDataBufferLength,&dObjectLength,&errorInfo);

        if (oResult)
        {
            //Expedited Transfer
            oRetExpeditedTransfer = TRUE;
            oRetSizeIndicated = FALSE;
            uRetNonValidNbOfBytes = 0;
            dAbortCode = errorInfo.GetErrorCode();
        }
        else
        {
            oResult = DCS_InitiateSegmentedRead(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),wIndex,uSubIndex,&errorInfo);

            if (oResult)
            {
                //Non Expedited Transfer
                oRetExpeditedTransfer = FALSE;
                oRetSizeIndicated = FALSE;
                uRetNonValidNbOfBytes = 0;
                *((DWORD*)pRetDataBuffer) = 0;
                dAbortCode = errorInfo.GetErrorCode();
                m_oToggle = FALSE;

            }
            else
            {
                //Failed
                dAbortCode = errorInfo.GetErrorCode();
            }
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_EXPEDITED_TRANSFER,&oRetExpeditedTransfer,sizeof(oRetExpeditedTransfer));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SIZE_INDICATED,&oRetSizeIndicated,sizeof(oRetSizeIndicated));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES,&uRetNonValidNbOfBytes,sizeof(uRetNonValidNbOfBytes));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_INDEX,&wIndex,sizeof(wIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SUB_INDEX,&uSubIndex,sizeof(uSubIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA,pRetDataBuffer,dRetDataBufferLength);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE,&dAbortCode,sizeof(dAbortCode));

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);
        ResetBufferedData();

        //Unlock critical section
        if(oRetExpeditedTransfer || !oResult)
        {
            Unlock();
        }

        return oResult;
    }

    return FALSE;

}

BOOL CGatewayCANopenToEpos::Process_UploadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_CANOPEN_DATA_SIZE = 7;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_TOGGLE = 2;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_TOGGLE = 1;
    const int RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 2;
    const int RETURN_PARAMETER_INDEX_NO_MORE_SEGMENTS = 3;
    const int RETURN_PARAMETER_INDEX_SEG_DATA = 4;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 5;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oToggle = FALSE;

    //ReturnParameter CANopen
    BYTE uRetNonValidNbOfBytes = 0;
    BOOL oRetNoMoreSegments = FALSE;
    void* pRetSegDataBuffer = NULL;
    DWORD dRetSegDataBufferLength = 0;
    DWORD dSegmentLengthRead = 0;
    DWORD dAbortCode = 0;

    //ReturnParameter Esam
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Check CriticalSection
        if(!IsLocked(pCommand)) return FALSE;

        //Prepare DataBuffer
        dRetSegDataBufferLength = MAX_CANOPEN_DATA_SIZE;
        pRetSegDataBuffer = malloc(dRetSegDataBufferLength);
        memset(pRetSegDataBuffer,0,dRetSegDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdClientServer,sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_TOGGLE,&oToggle,sizeof(oToggle));

        //Translate Command
        oResult = GetBufferedData(pRetSegDataBuffer,dRetSegDataBufferLength,&dSegmentLengthRead,&oRetNoMoreSegments);
        if(!oResult)
        {
            oResult = DCS_SegmentedRead(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,GetNodeId(dCobIdClientServer),m_oToggle,&oRetNoMoreSegments,&pDataBuffer,&dDataBufferLength,&errorInfo);
            if(oResult)
            {
                m_oToggle = !m_oToggle;
                AddBufferedData(pDataBuffer,dDataBufferLength,oRetNoMoreSegments);
                oResult = GetBufferedData(pRetSegDataBuffer,dRetSegDataBufferLength,&dSegmentLengthRead,&oRetNoMoreSegments);
            }
        }

        //CheckSize
        if(oResult && (dSegmentLengthRead <= MAX_CANOPEN_DATA_SIZE))
        {
            uRetNonValidNbOfBytes = (BYTE)(MAX_CANOPEN_DATA_SIZE-dSegmentLengthRead);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else if(oResult)
        {
            oResult = FALSE;
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL,&errorInfo);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else
        {
            uRetNonValidNbOfBytes = MAX_CANOPEN_DATA_SIZE;
            dAbortCode = errorInfo.GetErrorCode();
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER,&dCobIdServerClient,sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TOGGLE,&oToggle,sizeof(oToggle));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES,&uRetNonValidNbOfBytes,sizeof(uRetNonValidNbOfBytes));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NO_MORE_SEGMENTS,&oRetNoMoreSegments,sizeof(oRetNoMoreSegments));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SEG_DATA,pRetSegDataBuffer,dSegmentLengthRead);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE,&dAbortCode,sizeof(dAbortCode));

        //Free DataBuffer
        if(pRetSegDataBuffer) free(pRetSegDataBuffer);
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        if(oRetNoMoreSegments || !oResult)
        {
            ResetBufferedData();
            Unlock();
        }

        return oResult;
    }

    return FALSE;

}

BOOL CGatewayCANopenToEpos::Process_SendNMTService(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COMMAND_SPECIFIER = 0;
    const int PARAMETER_INDEX_NODE_ID = 1;

    //Parameter CANopen
    BYTE uCommandSpecifier = 0;
    BYTE uNodeId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COMMAND_SPECIFIER,&uCommandSpecifier,sizeof(uCommandSpecifier));
        pCommand->GetParameterData(PARAMETER_INDEX_NODE_ID,&uNodeId,sizeof(uNodeId));

        //Translate Command
        oResult = DCS_SendNMTService(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,(WORD)uNodeId,(WORD)uCommandSpecifier,&errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);

        //Unlock critical section
        Unlock();

        return oResult;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_SendCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_LENGTH = 1;
    const int PARAMETER_INDEX_DATA = 2;

    //Parameter CANopen
    DWORD dCobId = 0;
    BYTE uLength = 0;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand))
        {
            return FALSE;
        }

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID,&dCobId,sizeof(dCobId));
        pCommand->GetParameterData(PARAMETER_INDEX_LENGTH,&uLength,sizeof(uLength));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA,pDataBuffer,dDataBufferLength);

        //Check Length
        if(uLength > dDataBufferLength) uLength = (BYTE)dDataBufferLength;

        //Translate Command
        oResult = DCS_SendCANFrame(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,(WORD)dCobId,(WORD)uLength,pDataBuffer,&errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();

        return oResult;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_ReadCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Not supported
    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_RequestCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_LENGTH = 1;

    const int RETURN_PARAMETER_INDEX_DATA = 0;

    //Parameter CANopen
    DWORD dCobId = 0;
    BYTE uLength = 0;

    //ReturnParameter CANopen
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID,&dCobId,sizeof(dCobId));
        pCommand->GetParameterData(PARAMETER_INDEX_LENGTH,&uLength,sizeof(uLength));

        //Prepare DataBuffer
        dRetDataBufferLength = uLength;
        if(dRetDataBufferLength > 0)
        {
            pRetDataBuffer = malloc(dRetDataBufferLength);
            memset(pRetDataBuffer,0,dRetDataBufferLength);
        }

        //Translate Command
        oResult = DCS_RequestCANFrame(pDeviceCommandSetManager,hDCS_Handle,hTransactionHandle,(WORD)dCobId,(WORD)uLength,pRetDataBuffer,&errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult,&errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA,pRetDataBuffer,dRetDataBufferLength);

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        Unlock();

        return oResult;
    }

    return FALSE;

}

BOOL CGatewayCANopenToEpos::Process_SendLSSFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Not supported
    return FALSE;
}

BOOL CGatewayCANopenToEpos::Process_ReadLSSFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Not supported
    return FALSE;
}

BOOL CGatewayCANopenToEpos::DCS_ReadObject(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wNodeId,WORD wIndex,BYTE uSubIndex,BYTE* pObjectBuffer,DWORD dObjectBufferLength,DWORD* pdObjectLengthRead,CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    DWORD dObjectLength;
    BOOL oResult = FALSE;
    BYTE uNodeId = (BYTE)wNodeId;

    if(m_pCommand_ReadObject && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_ReadObject->ResetStatus();
        m_pCommand_ReadObject->SetParameterData(0,&wIndex,sizeof(wIndex));
        m_pCommand_ReadObject->SetParameterData(1,&uSubIndex,sizeof(uSubIndex));
        m_pCommand_ReadObject->SetParameterData(2,&uNodeId,sizeof(uNodeId));
        m_pCommand_ReadObject->SetParameterData(3,&dObjectBufferLength,sizeof(dObjectBufferLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_ReadObject, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Length
        dObjectLength = m_pCommand_ReadObject->GetReturnParameterLength(1);
        if(dObjectBufferLength < dObjectLength) dObjectLength = dObjectBufferLength;
        if(pdObjectLengthRead) *pdObjectLengthRead = dObjectLength;

        //Get ReturnParameter Data
        m_pCommand_ReadObject->GetReturnParameterData(0,&dDeviceErrorCode,sizeof(dDeviceErrorCode));
        m_pCommand_ReadObject->GetReturnParameterData(1,pObjectBuffer,dObjectLength);

        //Get ErrorCode
        m_pCommand_ReadObject->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_InitiateSegmentedRead(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD wNodeId, WORD wIndex, BYTE uSubIndex, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    BYTE uNodeId = (BYTE)wNodeId;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedRead && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_InitiateSegmentedRead->ResetStatus();
        m_pCommand_InitiateSegmentedRead->SetParameterData(0, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedRead->SetParameterData(1, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_InitiateSegmentedRead->SetParameterData(2, &uNodeId, sizeof(uNodeId));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_InitiateSegmentedRead, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_InitiateSegmentedRead->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_InitiateSegmentedRead->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_InitiateSegmentedWrite(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD wNodeId, WORD wIndex, BYTE uSubIndex, DWORD dObjectLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    BYTE uNodeId = (BYTE)wNodeId;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedWrite && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_InitiateSegmentedWrite->ResetStatus();
        m_pCommand_InitiateSegmentedWrite->SetParameterData(0, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(1, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(2, &uNodeId, sizeof(uNodeId));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(3, &dObjectLength, sizeof(dObjectLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_InitiateSegmentedWrite, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_InitiateSegmentedWrite->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_InitiateSegmentedWrite->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_SegmentedRead(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wNodeId,BOOL oToggle,BOOL* poLastDataSegment,void** ppSegmentBuffer,DWORD* pdSegmentBufferLength,CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    UEposControlByte controlByte;
    BYTE uDummyByte = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedRead && pDeviceCommandSetManager && ppSegmentBuffer && pdSegmentBufferLength)
    {
        //Prepare ControlByte
        controlByte.bValue = 0;
        controlByte.structure.bToggle = oToggle,

        //Set Parameter Data
        m_pCommand_SegmentedRead->ResetStatus();
        m_pCommand_SegmentedRead->SetParameterData(0,&wNodeId,sizeof(wNodeId));
        m_pCommand_SegmentedRead->SetParameterData(1,&controlByte,sizeof(controlByte));
        m_pCommand_SegmentedRead->SetParameterData(2,&uDummyByte,sizeof(uDummyByte));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SegmentedRead, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SegmentedRead->GetReturnParameterData(0,&dDeviceErrorCode,sizeof(dDeviceErrorCode));
        m_pCommand_SegmentedRead->GetReturnParameterData(1,&controlByte,sizeof(controlByte));

        //Prepare DataBufer
        *pdSegmentBufferLength = m_pCommand_SegmentedRead->GetReturnParameterLength(2);
        if(controlByte.structure.bLength < *pdSegmentBufferLength) *pdSegmentBufferLength = controlByte.structure.bLength;
        if(*pdSegmentBufferLength > 0)
        {
            *ppSegmentBuffer = malloc(*pdSegmentBufferLength);
            memset(*ppSegmentBuffer,0,*pdSegmentBufferLength);
        }
        m_pCommand_SegmentedRead->GetReturnParameterData(2,*ppSegmentBuffer,*pdSegmentBufferLength);

        //Get ErrorCode
        m_pCommand_SegmentedRead->GetErrorInfo(pErrorInfo);

        //Restore ControlByte
        if(poLastDataSegment) *poLastDataSegment = !controlByte.structure.bMoreSegments;

    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_SegmentedWrite(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wNodeId,BOOL oMoreSegments,BOOL oToggle,void* pSegmentBuffer,DWORD dSegmentLength,BOOL* poRetToggle,DWORD* pdSegmentLengthWritten,CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    UEposControlByte controlByte;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedWrite && pDeviceCommandSetManager)
    {
        //Prepare ControlByte
        controlByte.bValue = 0;
        controlByte.structure.bLength = dSegmentLength;
        controlByte.structure.bToggle = oToggle;
        controlByte.structure.bMoreSegments = oMoreSegments;

        //Set Parameter Data
        m_pCommand_SegmentedWrite->ResetStatus();
        m_pCommand_SegmentedWrite->SetParameterData(0,&wNodeId,sizeof(wNodeId));
        m_pCommand_SegmentedWrite->SetParameterData(1,&controlByte,sizeof(controlByte));
        m_pCommand_SegmentedWrite->SetParameterData(2,pSegmentBuffer,dSegmentLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SegmentedWrite, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SegmentedWrite->GetReturnParameterData(0,&dDeviceErrorCode,sizeof(dDeviceErrorCode));
        m_pCommand_SegmentedWrite->GetReturnParameterData(1,&controlByte,sizeof(controlByte));

        //Restore ControlByte
        if(poRetToggle) *poRetToggle = controlByte.structure.bToggle;
        if(pdSegmentLengthWritten) *pdSegmentLengthWritten = controlByte.structure.bLength;

        //Get ErrorCode
        m_pCommand_SegmentedWrite->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_AbortSegmentedTransfer(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD wNodeId, WORD wIndex, WORD wSubIndex, DWORD dAbortCode, CErrorInfo* pErrorInfo)
{
    BYTE uSubIndex = (BYTE)wSubIndex;
    BOOL oResult = FALSE;

    if(m_pCommand_AbortSegmentedTransfer && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_AbortSegmentedTransfer->ResetStatus();
        m_pCommand_AbortSegmentedTransfer->SetParameterData(0, &wNodeId, sizeof(wNodeId));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(3, &dAbortCode, sizeof(dAbortCode));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_AbortSegmentedTransfer, hDCS_Handle, hTransactionHandle);

        //Get ErrorCode
        m_pCommand_AbortSegmentedTransfer->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_WriteObject(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wNodeId,WORD wIndex,BYTE uSubIndex,BYTE* pObjectBuffer,DWORD dObjectLength,CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    BOOL oResult = FALSE;
    BYTE uNodeId = (BYTE)wNodeId;

    if(m_pCommand_WriteObject && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_WriteObject->ResetStatus();
        m_pCommand_WriteObject->SetParameterData(0,&wIndex,sizeof(wIndex));
        m_pCommand_WriteObject->SetParameterData(1,&uSubIndex,sizeof(uSubIndex));
        m_pCommand_WriteObject->SetParameterData(2,&uNodeId,sizeof(uNodeId));
        m_pCommand_WriteObject->SetParameterData(3,pObjectBuffer,dObjectLength);
        m_pCommand_WriteObject->SetParameterData(4,&dObjectLength,sizeof(dObjectLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_WriteObject, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_WriteObject->GetReturnParameterData(0,&dDeviceErrorCode,sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_WriteObject->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_SendNMTService(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wNodeId,WORD wCommandSpecifier,CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SendNMTService && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_SendNMTService->ResetStatus();
        m_pCommand_SendNMTService->SetParameterData(0,&wNodeId,sizeof(wNodeId));
        m_pCommand_SendNMTService->SetParameterData(1,&wCommandSpecifier,sizeof(wCommandSpecifier));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SendNMTService, hDCS_Handle, hTransactionHandle);

        //Get ErrorCode
        m_pCommand_SendNMTService->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_SendCANFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wCobID,WORD wLength,void* pData,CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SendCANFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_SendCANFrame->ResetStatus();
        m_pCommand_SendCANFrame->SetParameterData(0,&wCobID,sizeof(wCobID));
        m_pCommand_SendCANFrame->SetParameterData(1,&wLength,sizeof(wLength));
        m_pCommand_SendCANFrame->SetParameterData(2,pData,wLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SendCANFrame, hDCS_Handle, hTransactionHandle);

        //Get ErrorCode
        m_pCommand_SendCANFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::DCS_RequestCANFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle,HANDLE hTransactionHandle,WORD wCobID,WORD wLength,void* pData,CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_RequestCANFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_RequestCANFrame->ResetStatus();
        m_pCommand_RequestCANFrame->SetParameterData(0,&wCobID,sizeof(wCobID));
        m_pCommand_RequestCANFrame->SetParameterData(1,&wLength,sizeof(wLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_RequestCANFrame, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_RequestCANFrame->GetReturnParameterData(0,&dDeviceErrorCode,sizeof(dDeviceErrorCode));
        m_pCommand_RequestCANFrame->GetReturnParameterData(1,pData,wLength);

        //Get ErrorCode
        m_pCommand_RequestCANFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

WORD CGatewayCANopenToEpos::GetNodeId(DWORD dDefaultSdoCobIdClientServer)
{
    const DWORD MIN_TX_SDO        = 0x600;
    const DWORD MAX_TX_SDO        = 0x67F;
    const DWORD TX_SDO_OFFSET    = 0x600;

    if((dDefaultSdoCobIdClientServer >= MIN_TX_SDO) && (dDefaultSdoCobIdClientServer <= MAX_TX_SDO))
    {
        return (WORD)(dDefaultSdoCobIdClientServer - TX_SDO_OFFSET);
    }

    return 0;
}

DWORD CGatewayCANopenToEpos::CheckDataBufferLength(BOOL oSizeIndicated,BYTE uNonValidNbOfBytes,DWORD dDataBufferLength,DWORD dMaxDataBufferLength)
{
    if(oSizeIndicated)
    {
        //Size Indicated
        if(dDataBufferLength > (dMaxDataBufferLength - uNonValidNbOfBytes)) dDataBufferLength = (dMaxDataBufferLength - uNonValidNbOfBytes);
    }
    else
    {
        //Size not Indicated
        if(dDataBufferLength > dMaxDataBufferLength) dDataBufferLength = dMaxDataBufferLength;
    }

    return dDataBufferLength;
}

BOOL CGatewayCANopenToEpos::ResetBufferedData()
{
    if(m_pBufferedData) free(m_pBufferedData);
    m_pBufferedData = NULL;
    m_dBufferedDataSize = 0;
    m_dBufferedDataRead = 0;
    m_oNoMoreSegments = FALSE;

    return TRUE;
}

BOOL CGatewayCANopenToEpos::AddBufferedData(void* pDataBuffer,DWORD dDataBufferSize,BOOL oNoMoreSegments)
{
    DWORD size;
    BYTE* p;

    if(dDataBufferSize > 0)
    {
        //Allocate Memory
        size = m_dBufferedDataSize + dDataBufferSize;
        m_pBufferedData = realloc(m_pBufferedData,size);
        p = (BYTE*)m_pBufferedData + m_dBufferedDataSize;
        memcpy(p,pDataBuffer,dDataBufferSize);
        m_dBufferedDataSize += dDataBufferSize;

        //NoMoreSegments
        m_oNoMoreSegments = oNoMoreSegments;

        return TRUE;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::IsBufferedDataAvailable(DWORD dSize,DWORD* pdSizeAvailable,BOOL* poNoMoreSegments)
{
    DWORD dTotalSizeAvailable;

    if(m_dBufferedDataSize > m_dBufferedDataRead)
    {
        dTotalSizeAvailable = m_dBufferedDataSize-m_dBufferedDataRead;
        if(dTotalSizeAvailable > dSize)
        {
            if(poNoMoreSegments) *poNoMoreSegments = FALSE;
            if(pdSizeAvailable) *pdSizeAvailable = dSize;
            return TRUE;
        }
        else if(dTotalSizeAvailable == dSize)
        {
            if(poNoMoreSegments) *poNoMoreSegments = m_oNoMoreSegments;
            if(pdSizeAvailable) *pdSizeAvailable = dTotalSizeAvailable;
            return TRUE;
        }
        else if(dTotalSizeAvailable < dSize)
        {
            if(m_oNoMoreSegments)
            {
                if(poNoMoreSegments) *poNoMoreSegments = TRUE;
                if(pdSizeAvailable) *pdSizeAvailable = dTotalSizeAvailable;
                return TRUE;
            }
        }
    }

    if(poNoMoreSegments) *poNoMoreSegments = FALSE;
    return FALSE;
}

BOOL CGatewayCANopenToEpos::ReverseBufferedData(DWORD dReverseSize)
{
    if(m_dBufferedDataRead >= dReverseSize)
    {
        m_dBufferedDataRead -= dReverseSize;
        return TRUE;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::GetBufferedData(void* pDataBuffer,DWORD dDataBufferSize,DWORD* pdDataRead,BOOL* poNoMoreSegments)
{
    DWORD dSizeAvailable;
    BYTE* p;

    if(pDataBuffer && pdDataRead && poNoMoreSegments)
    {
        if(IsBufferedDataAvailable(dDataBufferSize,&dSizeAvailable,poNoMoreSegments))
        {
            if(dSizeAvailable > dDataBufferSize) dSizeAvailable = dDataBufferSize;
            p = (BYTE*)m_pBufferedData + m_dBufferedDataRead;
            memcpy(pDataBuffer,p,dSizeAvailable);
            m_dBufferedDataRead += dSizeAvailable;
            *pdDataRead = dSizeAvailable;

            return TRUE;
        }
    }

    return FALSE;
}

BOOL CGatewayCANopenToEpos::InitLayerParameterStack(CCommandRoot* p_pCommand)
{
    CLayerParameterStack layerParameterStack;
    CLayerParameterSet layerParameterSet;
    BOOL oResult = FALSE;

    if(p_pCommand)
    {
        if(p_pCommand->GetLayerParameterStack(layerParameterStack))
        {
            //Pop PS Layer Parameter Set
            layerParameterStack.PopLayer(PROTOCOL_STACK_LAYER, layerParameterSet);

            //Set I Layer Commands
            if(m_pCommand_WriteObject) m_pCommand_WriteObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedWrite) m_pCommand_InitiateSegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedWrite) m_pCommand_SegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadObject) m_pCommand_ReadObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedRead) m_pCommand_InitiateSegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedRead) m_pCommand_SegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_AbortSegmentedTransfer) m_pCommand_AbortSegmentedTransfer->SetLayerParameterStack(layerParameterStack);

            if(m_pCommand_SendCANFrame) m_pCommand_SendCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_RequestCANFrame) m_pCommand_RequestCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SendNMTService) m_pCommand_SendNMTService->SetLayerParameterStack(layerParameterStack);

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD p_ulTimeout)
{
    WORD usTimeout(0);
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    HANDLE hTransactionHandle(NULL);
    BYTE ubNodeId(0);
    CStdString interfaceName(_T(""));

    if(p_pManager)
    {
        if(p_pManager->DCS_GetInterfaceName(p_hHandle, &interfaceName))
        {
            //Init
            usTimeout = (WORD)p_ulTimeout;

            if(GetAddressParameter(p_pManager, p_hHandle, ubNodeId, &errorInfo))
            {
                if(interfaceName == INTERFACE_RS232)
                {
                    //RS232 Frame Timeout
                       oResult = DCS_WriteObject(p_pManager, p_hHandle, hTransactionHandle, ubNodeId, INDEX_RS232_FRAME_TIMEOUT, SUBINDEX_RS232_FRAME_TIMEOUT, (BYTE*)&usTimeout, sizeof(usTimeout), &errorInfo);
                }
            }
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEpos::GetAddressParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo)
{
    const CStdString NODE_ID = _T("NodeId");

    BOOL oResult(FALSE);

    if(p_pManager)
    {
        //Get Address Parameter from Device
        oResult = p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_rNodeId, sizeof(p_rNodeId), p_pErrorInfo);
    }

    return oResult;
}

