#include "stdafx.h"
#include <malloc.h>
#include <memory.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Esam2.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEsam2Def.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/BaseClasses/Command_PS.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/Command_PS_CANopen.h>
#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>

#include <CommunicationModel/ProtocolStack/Classes/GatewayToDevice/CANopen/GatewayCANopenToEsam2.h>

//Constants
const DWORD MAX_ESAM2_DATA_SIZE(255);

CGatewayCANopenToEsam2::CGatewayCANopenToEsam2()
{
    m_pCommand_WriteObject = NULL;
    m_pCommand_InitiateSegmentedWrite = NULL;
    m_pCommand_SegmentedWrite = NULL;
    m_pCommand_ReadObject = NULL;
    m_pCommand_InitiateSegmentedRead = NULL;
    m_pCommand_SegmentedRead = NULL;
    m_pCommand_SendNMTService = NULL;
    m_pCommand_RequestCANFrame = NULL;
    m_pCommand_SendCANFrame = NULL;
    m_pCommand_ReadCANFrame = NULL;
    m_pCommand_SendLSSFrame = NULL;
    m_pCommand_ReadLSSFrame = NULL;
    m_pCommand_AbortSegmentedTransfer = NULL;

    m_pBufferedData = NULL;
    m_dBufferedDataSize = 0;
    m_dBufferedDataRead  = 0;
    m_oNoMoreSegments = FALSE;
    m_oToggle = FALSE;
    m_WriteSegmentSize = 0;
    ResetBufferedData();

    m_ulNetworkIndicationThreadId = 0;
    m_wTargetNetworkId = 0;
    m_uTargetNodeId = 0;

    InitCommands();
}

CGatewayCANopenToEsam2::~CGatewayCANopenToEsam2()
{
    ResetBufferedData();
    DeleteCommands();
}

CGateway* CGatewayCANopenToEsam2::Clone()
{
    CGatewayCANopenToEsam2* pClonedGateway;

    pClonedGateway = new CGatewayCANopenToEsam2();
    *pClonedGateway = *this;

    return pClonedGateway;
}

void CGatewayCANopenToEsam2::InitCommands()
{
    DeleteCommands();

    //WriteObject
    m_pCommand_WriteObject = new CCommand_DCS_Esam2();
    m_pCommand_WriteObject->InitCommand(ESAM2_WRITE_OBJECT);
    //ReadObject
    m_pCommand_ReadObject = new CCommand_DCS_Esam2();
    m_pCommand_ReadObject->InitCommand(ESAM2_READ_OBJECT);
    //InitiateSegmentedWrite
    m_pCommand_InitiateSegmentedWrite = new CCommand_DCS_Esam2();
    m_pCommand_InitiateSegmentedWrite->InitCommand(ESAM2_INITIATE_SEGMENT_WRITE);
    //InitiateSegmentedRead
    m_pCommand_InitiateSegmentedRead = new CCommand_DCS_Esam2();
    m_pCommand_InitiateSegmentedRead->InitCommand(ESAM2_INITIATE_SEGMENT_READ);
    //SegmentedWrite
    m_pCommand_SegmentedWrite = new CCommand_DCS_Esam2();
    m_pCommand_SegmentedWrite->InitCommand(ESAM2_SEGMENT_WRITE);
    //SegmentedRead
    m_pCommand_SegmentedRead = new CCommand_DCS_Esam2();
    m_pCommand_SegmentedRead->InitCommand(ESAM2_SEGMENT_READ);
    //AbortSegmentedTransfer
    m_pCommand_AbortSegmentedTransfer = new CCommand_DCS_Esam2();
    m_pCommand_AbortSegmentedTransfer->InitCommand(ESAM2_ABORT_SEGMENT_TRANSFER);
    //Init Command SendNMTService
    m_pCommand_SendNMTService = new CCommand_DCS_Esam2();
    m_pCommand_SendNMTService->InitCommand(ESAM2_SEND_NMT_SERVICE);
    //Init Command Request
    m_pCommand_RequestCANFrame = new CCommand_DCS_Esam2();
    m_pCommand_RequestCANFrame->InitCommand(ESAM2_REQUEST_CAN_FRAME);
    //Init Command Send
    m_pCommand_SendCANFrame = new CCommand_DCS_Esam2();
    m_pCommand_SendCANFrame->InitCommand(ESAM2_SEND_CAN_FRAME);
    //Init Command Read
    m_pCommand_ReadCANFrame = new CCommand_DCS_Esam2();
    m_pCommand_ReadCANFrame->InitCommand(ESAM2_READ_CAN_FRAME);
    //SendLSSFrame
    m_pCommand_SendLSSFrame = new CCommand_DCS_Esam2();
    m_pCommand_SendLSSFrame->InitCommand(ESAM2_SEND_LSS_FRAME);
    //ReadLSSFrame
    m_pCommand_ReadLSSFrame = new CCommand_DCS_Esam2();
    m_pCommand_ReadLSSFrame->InitCommand(ESAM2_READ_LSS_FRAME);
}

void CGatewayCANopenToEsam2::DeleteCommands()
{
    //WriteObject
    if(m_pCommand_WriteObject)
    {
        delete m_pCommand_WriteObject;
        m_pCommand_WriteObject = NULL;
    }
    //InitiateSegmentedWrite
    if(m_pCommand_InitiateSegmentedWrite)
    {
        delete m_pCommand_InitiateSegmentedWrite;
        m_pCommand_InitiateSegmentedWrite = NULL;
    }
    //SegmentedWrite
    if(m_pCommand_SegmentedWrite)
    {
        delete m_pCommand_SegmentedWrite;
        m_pCommand_SegmentedWrite = NULL;
    }
    //ReadObject
    if(m_pCommand_ReadObject)
    {
        delete m_pCommand_ReadObject;
        m_pCommand_ReadObject = NULL;
    }
    //AbortSegmentedTransfer
    if(m_pCommand_AbortSegmentedTransfer)
    {
        delete m_pCommand_AbortSegmentedTransfer;
        m_pCommand_AbortSegmentedTransfer = NULL;
    }
    //InitateSegmentedRead
    if(m_pCommand_InitiateSegmentedRead)
    {
        delete m_pCommand_InitiateSegmentedRead;
        m_pCommand_InitiateSegmentedRead = NULL;
    }
    //SegmentedRead
    if(m_pCommand_SegmentedRead)
    {
        delete m_pCommand_SegmentedRead;
        m_pCommand_SegmentedRead = NULL;
    }
    //NMTService
    if(m_pCommand_SendNMTService)
    {
        delete m_pCommand_SendNMTService;
        m_pCommand_SendNMTService = NULL;
    }
    //RequestCANFrame
    if(m_pCommand_RequestCANFrame)
    {
        delete m_pCommand_RequestCANFrame;
        m_pCommand_RequestCANFrame = NULL;
    }
    //SendCANFrame
    if(m_pCommand_SendCANFrame)
    {
        delete m_pCommand_SendCANFrame;
        m_pCommand_SendCANFrame = NULL;
    }
    //ReadCANFrame
    if(m_pCommand_ReadCANFrame)
    {
        delete m_pCommand_ReadCANFrame;
        m_pCommand_ReadCANFrame = NULL;
    }
    //SendLSSFrame
    if(m_pCommand_SendLSSFrame)
    {
        delete m_pCommand_SendLSSFrame;
        m_pCommand_SendLSSFrame = NULL;
    }
    //ReadLSSFrame
    if(m_pCommand_ReadLSSFrame)
    {
        delete m_pCommand_ReadLSSFrame;
        m_pCommand_ReadLSSFrame = NULL;
    }
}

CGatewayCANopenToEsam2& CGatewayCANopenToEsam2::operator=(CGatewayCANopenToEsam2& other)
{
    if(this != &other)
    {
        *((CGatewayPStoDCS*)this) = *((CGatewayPStoDCS*)&other);
    }

    return *this;
}

BOOL CGatewayCANopenToEsam2::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* pManager, HANDLE h_Handle, HANDLE hTransactionHandle)
{
    CCommand_PS* pCommand_PS;
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager;
    BOOL oResult = FALSE;

    if(CGateway::ProcessCommand(p_pCommand, pManager, h_Handle, hTransactionHandle))
    {
        if(CheckLayers(p_pCommand, pManager))
        {
            pCommand_PS = (CCommand_PS*)p_pCommand;
            pDeviceCommandSetManager = (CDeviceCommandSetManagerBase*)pManager;

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayCANopenToEsam2")));
            }

            switch(p_pCommand->GetCommandId())
            {
                case CANOPEN_INITIATE_SDO_DOWNLOAD: oResult = Process_InitiateSDODownload(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_DOWNLOAD_SDO_SEGMENT: oResult = Process_DownloadSDOSegment(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_INITIATE_SDO_UPLOAD: oResult = Process_InitiateSDOUpload(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_UPLOAD_SDO_SEGMENT: oResult = Process_UploadSDOSegment(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_NETWORK_INDICATION: oResult = Process_NetworkIndication(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_ABORT_SDO_TRANSFER: oResult = Process_AbortSDOTransfer(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_NMT_SERVICE: oResult = Process_SendNMTService(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_CAN_FRAME: oResult = Process_SendCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_READ_CAN_FRAME: oResult = Process_ReadCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_REQUEST_CAN_FRAME: oResult = Process_RequestCANFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_SEND_LSS_FRAME: oResult = Process_SendLSSFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
                case CANOPEN_READ_LSS_FRAME: oResult = Process_ReadLSSFrame(pCommand_PS, pDeviceCommandSetManager, h_Handle, hTransactionHandle); break;
            }

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayCANopenToEsam2")));
            }
        }
    }

    return oResult;
}
BOOL CGatewayCANopenToEsam2::Process_InitiateSDODownload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_DATA_SIZE = 4;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_EXPEDITED_TRANSFER = 2;
    const int PARAMETER_INDEX_SIZE_INDICATED = 3;
    const int PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 4;
    const int PARAMETER_INDEX_INDEX = 5;
    const int PARAMETER_INDEX_SUB_INDEX = 6;
    const int PARAMETER_INDEX_DATA = 7;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_INDEX = 1;
    const int RETURN_PARAMETER_INDEX_SUB_INDEX = 2;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 3;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oExpeditedTransfer = FALSE;
    BOOL oSizeIndicated = FALSE;
    BYTE uNonValidNbOfBytes = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;
    DWORD dObjectLength = 0;

    //ReturnParameter CANopen
    DWORD dAbortCode = 0;

    //Address Parameter
    WORD wNetworkId = 0;
    BYTE uNodeId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(IsNetworkIndicationThread())
        {
            //Already locked by NetworkIndication
            if(!IsLocked(pCommand)) return FALSE;
        }
        else
        {
            //No NetworkIndication received
            if(!Lock(pCommand)) return FALSE;
        }

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_EXPEDITED_TRANSFER, &oExpeditedTransfer, sizeof(oExpeditedTransfer));
        pCommand->GetParameterData(PARAMETER_INDEX_SIZE_INDICATED, &oSizeIndicated, sizeof(oSizeIndicated));
        pCommand->GetParameterData(PARAMETER_INDEX_NON_VALID_NB_OF_BYTES, &uNonValidNbOfBytes, sizeof(uNonValidNbOfBytes));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX, &wIndex, sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX, &uSubIndex, sizeof(uSubIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, dDataBufferLength);

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);
        if(oResult) oResult = GetNodeId(dCobIdClientServer, uNodeId, &errorInfo);

        //Check DataBuffer Length
        dDataBufferLength = CheckDataBufferLength(oSizeIndicated, uNonValidNbOfBytes, dDataBufferLength, MAX_DATA_SIZE);

        //Translate Command
        if(oExpeditedTransfer)
        {
            //Expedited Transfer
            if(oResult) oResult = DCS_WriteObject(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, uNodeId, wIndex, uSubIndex, (BYTE*)pDataBuffer, dDataBufferLength, &errorInfo);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else
        {
            //NonExpedited Transfer
            if(oSizeIndicated && (dDataBufferLength >= MAX_DATA_SIZE))
            {
                dObjectLength = *((DWORD*)pDataBuffer);
                if(oResult) oResult = DCS_InitiateSegmentedWrite(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, uNodeId, wIndex, uSubIndex, dObjectLength, &errorInfo);
                dAbortCode = errorInfo.GetErrorCode();
                m_oToggle = FALSE;
                m_WriteSegmentSize = MAX_ESAM2_DATA_SIZE;
            }
            else
            {
                oResult = FALSE;
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, &errorInfo);
                dAbortCode = errorInfo.GetErrorCode();
            }
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_INDEX, &wIndex, sizeof(wIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SUB_INDEX, &uSubIndex, sizeof(uSubIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);
        ResetBufferedData();

        //Unlock critical section
        if(oExpeditedTransfer || !oResult)
        {
            ResetNetworkIndication();
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_AbortSDOTransfer(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //*Constants PS*
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_INDEX = 1;
    const int PARAMETER_INDEX_SUB_INDEX = 2;
    const int PARAMETER_INDEX_ABORT_CODE = 3;

    //*Variables PS*
    //Parameter
    DWORD dCobIdClientServer = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;
    DWORD dAbortCode = 0;

    //Address Parameter
    WORD wNetworkId = 0;
    BYTE uNodeId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Init
        oResult = TRUE;

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX, &wIndex, sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX, &uSubIndex, sizeof(uSubIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);
        if(oResult) oResult = GetNodeId(dCobIdClientServer, uNodeId, &errorInfo);

        //Execute command
        if(oResult) oResult = DCS_AbortSegmentedTransfer(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, uNodeId, wIndex, uSubIndex, dAbortCode, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);

        //Unlock critical section
        ResetNetworkIndication();
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_NetworkIndication(CCommand_PS* p_pCommand, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    //*Constants PS*
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_TARGET_NETWORK_ID = 2;
    const int PARAMETER_INDEX_TARGET_NODE_ID = 3;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_TARGET_NETWORK_ID = 1;
    const int RETURN_PARAMETER_INDEX_TARGET_NODE_ID = 2;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 3;

    //*Variables PS*
    //Parameter
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    WORD wTargetNetworkId = 0;
    BYTE uTargetNodeId = 0;

    //ReturnParameter
    DWORD dAbortCode = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Lock critical section
        if(!Lock(p_pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Get PS Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        p_pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT, &dCobIdServerClient, sizeof(dCobIdServerClient));
        p_pCommand->GetParameterData(PARAMETER_INDEX_TARGET_NETWORK_ID, &wTargetNetworkId, sizeof(wTargetNetworkId));
        p_pCommand->GetParameterData(PARAMETER_INDEX_TARGET_NODE_ID, &uTargetNodeId, sizeof(uTargetNodeId));

        //No Corresonding Command -> Save Information for SDO Service
        if(oResult) oResult = SetNetworkIndication(wTargetNetworkId, uTargetNodeId);

        //Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdServerClient, sizeof(dCobIdServerClient));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TARGET_NETWORK_ID, &wTargetNetworkId, sizeof(wTargetNetworkId));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TARGET_NODE_ID, &uTargetNodeId, sizeof(uTargetNodeId));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Unlock critical section
        if(!oResult)
        {
            ResetNetworkIndication();
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_DownloadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_CANOPEN_DATA_SIZE = 7;

    //Constants PS CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_TOGGLE = 2;
    const int PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 3;
    const int PARAMETER_INDEX_NO_MORE_SEGMENTS = 4;
    const int PARAMETER_INDEX_SEG_DATA = 5;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_TOGGLE = 1;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 2;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oToggle = FALSE;
    BYTE uNonValidNbOfBytes = 0;
    BOOL oNoMoreSegments = FALSE;
    void* pSegDataBuffer = NULL;
    DWORD dSegDataBufferLength = 0;

    //ReturnParameter CANopen
    DWORD dAbortCode = 0;

    //ReturnParameter Esam2
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;
    DWORD dDataLengthRead = 0;
    DWORD dDataLengthWritten = 0;

    //Address Parameter
    WORD wNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Check CriticalSection
        if(!IsLocked(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dSegDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_SEG_DATA);
        if(dSegDataBufferLength > 0) pSegDataBuffer = malloc(dSegDataBufferLength);
        dDataBufferLength = m_WriteSegmentSize;
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_TOGGLE, &oToggle, sizeof(oToggle));
        pCommand->GetParameterData(PARAMETER_INDEX_NON_VALID_NB_OF_BYTES, &uNonValidNbOfBytes, sizeof(uNonValidNbOfBytes));
        pCommand->GetParameterData(PARAMETER_INDEX_NO_MORE_SEGMENTS, &oNoMoreSegments, sizeof(oNoMoreSegments));
        pCommand->GetParameterData(PARAMETER_INDEX_SEG_DATA, pSegDataBuffer, dSegDataBufferLength);

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);

        //Check DataBuffer Length
        dSegDataBufferLength = CheckDataBufferLength(TRUE, uNonValidNbOfBytes, dSegDataBufferLength, MAX_CANOPEN_DATA_SIZE);

        //Translate Command
        oResult = AddBufferedData(pSegDataBuffer, dSegDataBufferLength, oNoMoreSegments);
        if(oResult)
        {
            while(GetBufferedData(pDataBuffer, dDataBufferLength, &dDataLengthRead, &oNoMoreSegments))
            {
                //AbortCommand
                if(m_oAbortCommands)
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER, &errorInfo);
                    oResult = FALSE;
                    break;
                }

                oResult = DCS_SegmentedWrite(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, m_oToggle, oNoMoreSegments, pDataBuffer, dDataLengthRead, &dDataLengthWritten, &errorInfo);
                m_oToggle = !m_oToggle;

                //Update SegmentLength
                if(dDataLengthWritten < dDataLengthRead)
                {
                    ReverseBufferedData(dDataLengthRead-dDataLengthWritten);
                    oNoMoreSegments = FALSE;
                    if(pDataBuffer)
                    {
                        free(pDataBuffer);
                        pDataBuffer = NULL;

                        m_WriteSegmentSize = dDataLengthWritten;
                        dDataBufferLength = dDataLengthWritten;

                        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);
                    }
                }

                if(oNoMoreSegments) break;
            }
        }
        dAbortCode = errorInfo.GetErrorCode();

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TOGGLE, &oToggle, sizeof(oToggle));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Free DataBuffer
        if(pSegDataBuffer) free(pSegDataBuffer);
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        if(oNoMoreSegments || !oResult)
        {
            ResetBufferedData();
            ResetNetworkIndication();
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_InitiateSDOUpload(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_DATA_SIZE = 4;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_INDEX = 2;
    const int PARAMETER_INDEX_SUB_INDEX = 3;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_EXPEDITED_TRANSFER = 1;
    const int RETURN_PARAMETER_INDEX_SIZE_INDICATED = 2;
    const int RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 3;
    const int RETURN_PARAMETER_INDEX_INDEX = 4;
    const int RETURN_PARAMETER_INDEX_SUB_INDEX = 5;
    const int RETURN_PARAMETER_INDEX_DATA = 6;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 7;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    WORD wIndex = 0;
    BYTE uSubIndex = 0;

    //ReturnParameter CANopen
    BOOL oRetExpeditedTransfer = FALSE;
    BOOL oRetSizeIndicated = FALSE;
    BYTE uRetNonValidNbOfBytes = 0;
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;
    DWORD dAbortCode = 0;

    //Address Parameter
    WORD wNetworkId = 0;
    BYTE uNodeId = 0;

    //ReturnParameter
    DWORD dObjectLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(IsNetworkIndicationThread())
        {
            //Already locked by NetworkIndication
            if(!IsLocked(pCommand)) return FALSE;
        }
        else
        {
            //No NetworkIndication received
            if(!Lock(pCommand)) return FALSE;
        }

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dRetDataBufferLength = MAX_DATA_SIZE;
        pRetDataBuffer = malloc(dRetDataBufferLength);
        memset(pRetDataBuffer, 0, dRetDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_INDEX, &wIndex, sizeof(wIndex));
        pCommand->GetParameterData(PARAMETER_INDEX_SUB_INDEX, &uSubIndex, sizeof(uSubIndex));

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);
        if(oResult) oResult = GetNodeId(dCobIdClientServer, uNodeId, &errorInfo);

        //Translate Command
        if(oResult)
        {
            oResult = DCS_ReadObject(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, uNodeId, wIndex, uSubIndex, (BYTE*)pRetDataBuffer, dRetDataBufferLength, &dObjectLength, &errorInfo);

            if (oResult)
            {
                //Expedited Transfer
                oRetExpeditedTransfer = TRUE;
                oRetSizeIndicated = FALSE;
                uRetNonValidNbOfBytes = 0;
                dAbortCode = errorInfo.GetErrorCode();
            }
            else if(errorInfo.GetErrorCode() == ERROR_DEVICE_ESAM2_SEGMENTED_TRANSFER_REQUIRED_INIT_DONE)
            {
                //Non Expedited Transfer -> Initialization already done
                oRetExpeditedTransfer = FALSE;
                oRetSizeIndicated = TRUE;
                uRetNonValidNbOfBytes = 0;
                dAbortCode = 0;
                oResult = TRUE;
                m_oToggle = FALSE;
            }
            else
            {
                oResult = DCS_InitiateSegmentedRead(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, uNodeId, wIndex, uSubIndex, &dObjectLength, &errorInfo);

                if (oResult)
                {
                    //Non Expedited Transfer
                    oRetExpeditedTransfer = FALSE;
                    oRetSizeIndicated = TRUE;
                    uRetNonValidNbOfBytes = 0;
                    *((DWORD*)pRetDataBuffer) = dObjectLength;
                    dAbortCode = errorInfo.GetErrorCode();
                    m_oToggle = FALSE;
                }
                else
                {
                    //Failed
                    dAbortCode = errorInfo.GetErrorCode();
                }
            }
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_EXPEDITED_TRANSFER, &oRetExpeditedTransfer, sizeof(oRetExpeditedTransfer));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SIZE_INDICATED, &oRetSizeIndicated, sizeof(oRetSizeIndicated));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES, &uRetNonValidNbOfBytes, sizeof(uRetNonValidNbOfBytes));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_INDEX, &wIndex, sizeof(wIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SUB_INDEX, &uSubIndex, sizeof(uSubIndex));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, dRetDataBufferLength);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);
        ResetBufferedData();

        //Unlock critical section
        if(oRetExpeditedTransfer || !oResult)
        {
            ResetNetworkIndication();
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_UploadSDOSegment(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_CANOPEN_DATA_SIZE = 7;

    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int PARAMETER_INDEX_COB_ID_SERVER_CLIENT = 1;
    const int PARAMETER_INDEX_TOGGLE = 2;

    const int RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER = 0;
    const int RETURN_PARAMETER_INDEX_TOGGLE = 1;
    const int RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES = 2;
    const int RETURN_PARAMETER_INDEX_NO_MORE_SEGMENTS = 3;
    const int RETURN_PARAMETER_INDEX_SEG_DATA = 4;
    const int RETURN_PARAMETER_INDEX_ABORT_CODE = 5;

    //Parameter CANopen
    DWORD dCobIdClientServer = 0;
    DWORD dCobIdServerClient = 0;
    BOOL oToggle = FALSE;

    //ReturnParameter CANopen
    BYTE uRetNonValidNbOfBytes = 0;
    BOOL oRetNoMoreSegments = FALSE;
    void* pRetSegDataBuffer = NULL;
    DWORD dRetSegDataBufferLength = 0;
    DWORD dSegmentLengthRead = 0;
    DWORD dAbortCode = 0;

    //Address Parameter
    WORD wNetworkId = 0;

    //ReturnParameter Esam
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Check CriticalSection
        if(!IsLocked(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dRetSegDataBufferLength = MAX_CANOPEN_DATA_SIZE;
        pRetSegDataBuffer = malloc(dRetSegDataBufferLength);
        memset(pRetSegDataBuffer, 0, dRetSegDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdClientServer, sizeof(dCobIdClientServer));
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID_SERVER_CLIENT, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->GetParameterData(PARAMETER_INDEX_TOGGLE, &oToggle, sizeof(oToggle));

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);

        //Translate Command
        oResult = GetBufferedData(pRetSegDataBuffer, dRetSegDataBufferLength, &dSegmentLengthRead, &oRetNoMoreSegments);
        if(!oResult)
        {
            oResult = DCS_SegmentedRead(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, m_oToggle, &oRetNoMoreSegments, &pDataBuffer, &dDataBufferLength, &errorInfo);
            if(oResult)
            {
                m_oToggle = !m_oToggle;
                AddBufferedData(pDataBuffer, dDataBufferLength, oRetNoMoreSegments);
                oResult = GetBufferedData(pRetSegDataBuffer, dRetSegDataBufferLength, &dSegmentLengthRead, &oRetNoMoreSegments);
            }
        }

        //CheckSize
        if(oResult && (dSegmentLengthRead <= MAX_CANOPEN_DATA_SIZE))
        {
            uRetNonValidNbOfBytes = (BYTE)(MAX_CANOPEN_DATA_SIZE - dSegmentLengthRead);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else if(oResult)
        {
            oResult = FALSE;
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, &errorInfo);
            dAbortCode = errorInfo.GetErrorCode();
        }
        else
        {
            uRetNonValidNbOfBytes = MAX_CANOPEN_DATA_SIZE;
            dAbortCode = errorInfo.GetErrorCode();
        }

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_COB_ID_CLIENT_SERVER, &dCobIdServerClient, sizeof(dCobIdServerClient));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_TOGGLE, &oToggle, sizeof(oToggle));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NON_VALID_NB_OF_BYTES, &uRetNonValidNbOfBytes, sizeof(uRetNonValidNbOfBytes));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_NO_MORE_SEGMENTS, &oRetNoMoreSegments, sizeof(oRetNoMoreSegments));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_SEG_DATA, pRetSegDataBuffer, dSegmentLengthRead);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_ABORT_CODE, &dAbortCode, sizeof(dAbortCode));

        //Free DataBuffer
        if(pRetSegDataBuffer) free(pRetSegDataBuffer);
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        if(oRetNoMoreSegments || !oResult)
        {
            ResetBufferedData();
            ResetNetworkIndication();
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_SendNMTService(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COMMAND_SPECIFIER = 0;
    const int PARAMETER_INDEX_NODE_ID = 1;

    //Parameter CANopen
    BYTE uCommandSpecifier = 0;
    BYTE uNodeId = 0;

    //Address Parameter
    WORD wNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COMMAND_SPECIFIER, &uCommandSpecifier, sizeof(uCommandSpecifier));
        pCommand->GetParameterData(PARAMETER_INDEX_NODE_ID, &uNodeId, sizeof(uNodeId));

        //Get Address Parameter
        if(oResult) oResult = GetNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wNetworkId, &errorInfo);

        //Translate Command
        if(oResult) oResult = DCS_SendNMTService(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wNetworkId, (WORD)uNodeId, (WORD)uCommandSpecifier, &errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_SendCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_LENGTH = 1;
    const int PARAMETER_INDEX_DATA = 2;

    //Parameter CANopen
    DWORD dCobId = 0;
    BYTE uLength = 0;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;

    //Address Parameter
    WORD wRemoteNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID, &dCobId, sizeof(dCobId));
        pCommand->GetParameterData(PARAMETER_INDEX_LENGTH, &uLength, sizeof(uLength));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, dDataBufferLength);

        //Get Address Parameter
        if(oResult) oResult = GetRemoteNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wRemoteNetworkId, &errorInfo);

        //Check Length
        if(uLength > dDataBufferLength) uLength = (BYTE)dDataBufferLength;

        //Translate Command
        if(oResult) oResult = DCS_SendCANFrame(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wRemoteNetworkId, (WORD)dCobId, (WORD)uLength, pDataBuffer, &errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_ReadCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_LENGTH = 1;
    const int PARAMETER_INDEX_TIMEOUT = 2;

    const int RETURN_PARAMETER_INDEX_DATA = 0;

    //Parameter CANopen
    DWORD ulCobId = 0;
    BYTE ubLength = 0;
    DWORD ulTimeout = 0;

    //ReturnParameter CANopen
    void* pRetDataBuffer = NULL;
    DWORD ulRetDataBufferLength = 0;

    //Address parameter
    WORD usRemoteNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(Lock(pCommand))
        {
            //Init
            oResult = TRUE;

            //Get ParameterData CANopen
            pCommand->GetParameterData(PARAMETER_INDEX_COB_ID, &ulCobId, sizeof(ulCobId));
            pCommand->GetParameterData(PARAMETER_INDEX_LENGTH, &ubLength, sizeof(ubLength));
            pCommand->GetParameterData(PARAMETER_INDEX_TIMEOUT, &ulTimeout, sizeof(ulTimeout));

            //Get Address Parameter
            if(oResult) oResult = GetRemoteNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, usRemoteNetworkId, &errorInfo);

            //Prepare DataBuffer
            ulRetDataBufferLength = ubLength;
            if(ulRetDataBufferLength > 0)
            {
                pRetDataBuffer = malloc(ulRetDataBufferLength);
                memset(pRetDataBuffer, 0, ulRetDataBufferLength);
            }

            //Translate command
            oResult = DCS_ReadCANFrame(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, usRemoteNetworkId, (WORD)ulCobId, (WORD)ubLength, pRetDataBuffer, ulTimeout, &errorInfo);

            //Set ReturnParameterData CANopen
            pCommand->SetStatus(oResult, &errorInfo);
            pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, ulRetDataBufferLength);

            //Free DataBuffer
            if(pRetDataBuffer) free(pRetDataBuffer);

            //Unlock critical section
            Unlock();
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_RequestCANFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //Constants CANopen
    const int PARAMETER_INDEX_COB_ID = 0;
    const int PARAMETER_INDEX_LENGTH = 1;

    const int RETURN_PARAMETER_INDEX_DATA = 0;

    //Parameter CANopen
    DWORD dCobId = 0;
    BYTE uLength = 0;

    //ReturnParameter CANopen
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    //Address Parameter
    WORD wRemoteNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Get ParameterData CANopen
        pCommand->GetParameterData(PARAMETER_INDEX_COB_ID, &dCobId, sizeof(dCobId));
        pCommand->GetParameterData(PARAMETER_INDEX_LENGTH, &uLength, sizeof(uLength));

        //Get Address Parameter
        if(oResult) oResult = GetRemoteNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wRemoteNetworkId, &errorInfo);

        //Prepare DataBuffer
        dRetDataBufferLength = uLength;
        if(dRetDataBufferLength > 0)
        {
            pRetDataBuffer = malloc(dRetDataBufferLength);
            memset(pRetDataBuffer, 0, dRetDataBufferLength);
        }

        //Translate Command
        oResult = DCS_RequestCANFrame(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wRemoteNetworkId, (WORD)dCobId, (WORD)uLength, pRetDataBuffer, &errorInfo);

        //Set ReturnParameterData CANopen
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, dRetDataBufferLength);

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_SendLSSFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //*Constants PS*
    const int PARAMETER_INDEX_DATA = 0;

    //*Variables PS*
    //Parameter
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength = 0;

    //Address Parameter
    WORD wRemoteNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, dDataBufferLength);

        //Get Address Parameter
        if(oResult) oResult = GetRemoteNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wRemoteNetworkId, &errorInfo);

        //Translate Command
        if(oResult) oResult = DCS_SendLSSFrame(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wRemoteNetworkId, pDataBuffer, dDataBufferLength, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::Process_ReadLSSFrame(CCommand_PS* pCommand, CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle)
{
    //*Constants PS*
    const int FRAME_SIZE = 8;
    const int PARAMETER_INDEX_TIMEOUT = 0;
    const int RETURN_PARAMETER_INDEX_DATA = 0;

    //*Variables PS*
    //Parameter
    WORD wTimeout = 0;

    //ReturnParameter
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    //Address Parameter
    WORD wRemoteNetworkId = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Init
        oResult = TRUE;

        //Prepare RetDataBuffer
        dRetDataBufferLength = FRAME_SIZE;
        if(dRetDataBufferLength > 0) pRetDataBuffer = malloc(dRetDataBufferLength);

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_TIMEOUT, &wTimeout, sizeof(wTimeout));

        //Get Address Parameter
        if(oResult) oResult = GetRemoteNetworkId(pCommand, pDeviceCommandSetManager, hDCS_Handle, wRemoteNetworkId, &errorInfo);

        //Translate Command
        oResult = DCS_ReadLSSFrame(pDeviceCommandSetManager, hDCS_Handle, hTransactionHandle, wRemoteNetworkId, wTimeout, pRetDataBuffer, dRetDataBufferLength, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, dRetDataBufferLength);

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_ReadObject(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pObjectBuffer, DWORD dObjectBufferLength, DWORD* pdObjectLengthRead, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    DWORD dObjectLength = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadObject && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_ReadObject->ResetStatus();
        m_pCommand_ReadObject->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_ReadObject->SetParameterData(1, &uNodeId, sizeof(uNodeId));
        m_pCommand_ReadObject->SetParameterData(2, &wIndex, sizeof(wIndex));
        m_pCommand_ReadObject->SetParameterData(3, &uSubIndex, sizeof(uSubIndex));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_ReadObject, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Length
        dObjectLength = m_pCommand_ReadObject->GetReturnParameterLength(1);
        if(dObjectBufferLength < dObjectLength) dObjectLength = dObjectBufferLength;
        if(pdObjectLengthRead) *pdObjectLengthRead = dObjectLength;

        //Get ReturnParameter Data
        m_pCommand_ReadObject->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_ReadObject->GetReturnParameterData(1, pObjectBuffer, dObjectLength);

        //Get ErrorCode
        m_pCommand_ReadObject->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_InitiateSegmentedRead(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD* pdObjectLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedRead && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_InitiateSegmentedRead->ResetStatus();
        m_pCommand_InitiateSegmentedRead->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_InitiateSegmentedRead->SetParameterData(1, &uNodeId, sizeof(uNodeId));
        m_pCommand_InitiateSegmentedRead->SetParameterData(2, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedRead->SetParameterData(3, &uSubIndex, sizeof(uSubIndex));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_InitiateSegmentedRead, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_InitiateSegmentedRead->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_InitiateSegmentedRead->GetReturnParameterData(1, pdObjectLength, sizeof(*pdObjectLength));

        //Get ErrorCode
        m_pCommand_InitiateSegmentedRead->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_SegmentedRead(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BOOL oToggle, BOOL* poLastDataSegment, void** ppSegmentBuffer, DWORD* pdSegmentBufferLength, CErrorInfo* pErrorInfo)
{
    UEsam2ControlByte controlByte;
    BYTE uDummyByte = 0;
    DWORD dDeviceErrorCode;
    BYTE uLength = 0;
    UEsam2ControlByte retControlByte;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedRead && pDeviceCommandSetManager && ppSegmentBuffer && pdSegmentBufferLength)
    {
        //Prepare Parameter
        controlByte.ubValue = 0;
        controlByte.structure.ubToggle = oToggle;

        //Set Parameter Data
        m_pCommand_SegmentedRead->ResetStatus();
        m_pCommand_SegmentedRead->SetParameterData(0, &controlByte, sizeof(controlByte));
        m_pCommand_SegmentedRead->SetParameterData(1, &uDummyByte, sizeof(uDummyByte));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SegmentedRead, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SegmentedRead->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_SegmentedRead->GetReturnParameterData(1, &uLength, sizeof(uLength));
        m_pCommand_SegmentedRead->GetReturnParameterData(2, &retControlByte, sizeof(retControlByte));

        //Prepare DataBufer
        *pdSegmentBufferLength = m_pCommand_SegmentedRead->GetReturnParameterLength(3);
        if(uLength < *pdSegmentBufferLength) *pdSegmentBufferLength = uLength;
        if(*pdSegmentBufferLength > 0)
        {
            *ppSegmentBuffer = malloc(*pdSegmentBufferLength);
            memset(*ppSegmentBuffer, 0, *pdSegmentBufferLength);
        }
        m_pCommand_SegmentedRead->GetReturnParameterData(3, *ppSegmentBuffer, *pdSegmentBufferLength);

        //Get ErrorCode
        m_pCommand_SegmentedRead->GetErrorInfo(pErrorInfo);

        //Restore ControlByte
        if(poLastDataSegment) *poLastDataSegment = retControlByte.structure.ubLastDataSegment;
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_InitiateSegmentedWrite(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD dObjectLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedWrite && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_InitiateSegmentedWrite->ResetStatus();
        m_pCommand_InitiateSegmentedWrite->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(1, &uNodeId, sizeof(uNodeId));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(2, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(3, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(4, &dObjectLength, sizeof(dObjectLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_InitiateSegmentedWrite, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_InitiateSegmentedWrite->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_InitiateSegmentedWrite->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_SegmentedWrite(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BOOL oToggle, BOOL oLastDataSegment, void* pSegmentBuffer, DWORD dSegmentLength, DWORD* pdSegmentLengthWritten, CErrorInfo* pErrorInfo)
{
    BYTE uLength = 0;
    UEsam2ControlByte controlByte;
    DWORD dDeviceErrorCode = 0;
    BYTE uLengthWritten = 0;
    UEsam2ControlByte retControlByte;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedWrite && pDeviceCommandSetManager)
    {
        //Prepare Parameters
        uLength = (BYTE)dSegmentLength;
        controlByte.ubValue = 0;
        controlByte.structure.ubToggle = oToggle;
        controlByte.structure.ubLastDataSegment = oLastDataSegment;

        //Set Parameter Data
        m_pCommand_SegmentedWrite->ResetStatus();
        m_pCommand_SegmentedWrite->SetParameterData(0, &uLength, sizeof(uLength));
        m_pCommand_SegmentedWrite->SetParameterData(1, &controlByte, sizeof(controlByte));
        m_pCommand_SegmentedWrite->SetParameterData(2, pSegmentBuffer, dSegmentLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SegmentedWrite, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SegmentedWrite->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_SegmentedWrite->GetReturnParameterData(1, &uLengthWritten, sizeof(uLengthWritten));
        m_pCommand_SegmentedWrite->GetReturnParameterData(2, &retControlByte, sizeof(retControlByte));

        //Restore Parameters
        if(pdSegmentLengthWritten) *pdSegmentLengthWritten = uLengthWritten;

        //Get ErrorCode
        m_pCommand_SegmentedWrite->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_AbortSegmentedTransfer(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD dAbortCode, CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_AbortSegmentedTransfer && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_AbortSegmentedTransfer->ResetStatus();
        m_pCommand_AbortSegmentedTransfer->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(1, &uNodeId, sizeof(uNodeId));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(2, &wIndex, sizeof(wIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(3, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(4, &dAbortCode, sizeof(dAbortCode));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_AbortSegmentedTransfer, hDCS_Handle, hTransactionHandle);

        //Get ErrorCode
        m_pCommand_AbortSegmentedTransfer->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_WriteObject(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pObjectBuffer, DWORD dObjectLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_WriteObject && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_WriteObject->ResetStatus();
        m_pCommand_WriteObject->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_WriteObject->SetParameterData(1, &uNodeId, sizeof(uNodeId));
        m_pCommand_WriteObject->SetParameterData(2, &wIndex, sizeof(wIndex));
        m_pCommand_WriteObject->SetParameterData(3, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_WriteObject->SetParameterData(4, pObjectBuffer, dObjectLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_WriteObject, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_WriteObject->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_WriteObject->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_SendNMTService(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, WORD wNodeId, WORD wCommandSpecifier, CErrorInfo* pErrorInfo)
{
    BOOL oResult = FALSE;
    DWORD dDeviceErrorCode = 0;

    if(m_pCommand_SendNMTService && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_SendNMTService->ResetStatus();
        m_pCommand_SendNMTService->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_SendNMTService->SetParameterData(1, &wNodeId, sizeof(wNodeId));
        m_pCommand_SendNMTService->SetParameterData(2, &wCommandSpecifier, sizeof(wCommandSpecifier));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SendNMTService, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SendNMTService->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_SendNMTService->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_SendCANFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, WORD wCobID, WORD wLength, void* pData, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SendCANFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_SendCANFrame->ResetStatus();
        m_pCommand_SendCANFrame->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_SendCANFrame->SetParameterData(1, &wCobID, sizeof(wCobID));
        m_pCommand_SendCANFrame->SetParameterData(2, &wLength, sizeof(wLength));
        m_pCommand_SendCANFrame->SetParameterData(3, pData, wLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SendCANFrame, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SendCANFrame->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_SendCANFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_ReadCANFrame(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNetworkId, WORD p_usCobID, WORD p_usLength, void* p_pData, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode(0);
    BOOL oResult(FALSE);

    if(m_pCommand_ReadCANFrame && p_pDeviceCommandSetManager)
    {
        //Set parameter data
        m_pCommand_ReadCANFrame->ResetStatus();
        m_pCommand_ReadCANFrame->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_ReadCANFrame->SetParameterData(1, &p_usCobID, sizeof(p_usCobID));
        m_pCommand_ReadCANFrame->SetParameterData(2, &p_usLength, sizeof(p_usLength));
        m_pCommand_ReadCANFrame->SetParameterData(3, &p_ulTimeout, sizeof(p_ulTimeout));

        //Execute command
        oResult = p_pDeviceCommandSetManager->ExecuteCommand(m_pCommand_ReadCANFrame, p_hDCS_Handle, p_hTransactionHandle);

        //Get returnParameter data
        m_pCommand_ReadCANFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_ReadCANFrame->GetReturnParameterData(1, p_pData, p_usLength);

        //Get error code
        m_pCommand_ReadCANFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_RequestCANFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, WORD wCobID, WORD wLength, void* pData, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_RequestCANFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_RequestCANFrame->ResetStatus();
        m_pCommand_RequestCANFrame->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_RequestCANFrame->SetParameterData(1, &wCobID, sizeof(wCobID));
        m_pCommand_RequestCANFrame->SetParameterData(2, &wLength, sizeof(wLength));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_RequestCANFrame, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_RequestCANFrame->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_RequestCANFrame->GetReturnParameterData(1, pData, wLength);

        //Get ErrorCode
        m_pCommand_RequestCANFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_SendLSSFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SendLSSFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_SendLSSFrame->ResetStatus();
        m_pCommand_SendLSSFrame->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_SendLSSFrame->SetParameterData(1, pDataBuffer, dDataBufferLength);

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_SendLSSFrame, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_SendLSSFrame->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));

        //Get ErrorCode
        m_pCommand_SendLSSFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::DCS_ReadLSSFrame(CDeviceCommandSetManagerBase* pDeviceCommandSetManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, WORD p_usNetworkId, WORD wTimeout, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* pErrorInfo)
{
    DWORD dDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadLSSFrame && pDeviceCommandSetManager)
    {
        //Set Parameter Data
        m_pCommand_ReadLSSFrame->ResetStatus();
        m_pCommand_ReadLSSFrame->SetParameterData(0, &p_usNetworkId, sizeof(p_usNetworkId));
        m_pCommand_ReadLSSFrame->SetParameterData(1, &wTimeout, sizeof(wTimeout));

        //Execute command
        oResult = pDeviceCommandSetManager->ExecuteCommand(m_pCommand_ReadLSSFrame, hDCS_Handle, hTransactionHandle);

        //Get ReturnParameter Data
        m_pCommand_ReadLSSFrame->GetReturnParameterData(0, &dDeviceErrorCode, sizeof(dDeviceErrorCode));
        m_pCommand_ReadLSSFrame->GetReturnParameterData(1, pDataBuffer, dDataBufferLength);

        //Get ErrorCode
        m_pCommand_ReadLSSFrame->GetErrorInfo(pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::GetNodeId(DWORD dDefaultSdoCobIdClientServer, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo)
{
    const DWORD MIN_TX_SDO = 0x600;
    const DWORD MAX_TX_SDO = 0x67F;
    const DWORD TX_SDO_OFFSET = 0x600;

    BOOL oResult = FALSE;

    if(IsNetworkIndicationActive())
    {
        //NetworkIndication Active
        p_rNodeId = m_uTargetNodeId;
        oResult = TRUE;
    }
    else
    {
        //No NetworkIndication Active
        if((dDefaultSdoCobIdClientServer >= MIN_TX_SDO) && (dDefaultSdoCobIdClientServer <= MAX_TX_SDO))
        {
            p_rNodeId = (BYTE)(dDefaultSdoCobIdClientServer - TX_SDO_OFFSET);
            oResult = TRUE;
        }
        else
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PARAMETER, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::GetNetworkId(CCommand_PS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, WORD& p_rRemoteNetworkId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(IsNetworkIndicationActive())
    {
        //NetworkIndication Active
        p_rRemoteNetworkId = m_wTargetNetworkId;
        oResult = TRUE;
    }
    else
    {
        //No NetworkIndication Active
        oResult = GetRemoteNetworkId(p_pCommand, p_pManager, p_hDCS_Handle, p_rRemoteNetworkId, p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::GetRemoteNetworkId(CCommand_PS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, WORD& p_rRemoteNetworkId, CErrorInfo* p_pErrorInfo)
{
    const CStdString REMOTE_NETWORK_ID = _T("RemoteNetworkId");

    CErrorInfo errorInfo;
    BOOL oResult = FALSE;

    //Get RemoteNetworkId from Command
    if(!oResult) oResult = p_pCommand->GetLayerParameter(DEVICE_COMMAND_SET_LAYER, REMOTE_NETWORK_ID, (BYTE*)&p_rRemoteNetworkId, sizeof(p_rRemoteNetworkId), &errorInfo);

    //Get RemoteNetworkId from Device
    if(!oResult) oResult = p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, REMOTE_NETWORK_ID, (BYTE*)&p_rRemoteNetworkId, sizeof(p_rRemoteNetworkId), &errorInfo);

    //ErrorHandling
    if(!oResult && p_pErrorInfo) *p_pErrorInfo = errorInfo;

    return oResult;
}

DWORD CGatewayCANopenToEsam2::CheckDataBufferLength(BOOL oSizeIndicated, BYTE uNonValidNbOfBytes, DWORD dDataBufferLength, DWORD dMaxDataBufferLength)
{
    if(oSizeIndicated)
    {
        //Size Indicated
        if(dDataBufferLength > (dMaxDataBufferLength - uNonValidNbOfBytes)) dDataBufferLength = (dMaxDataBufferLength - uNonValidNbOfBytes);
    }
    else
    {
        //Size not Indicated
        if(dDataBufferLength > dMaxDataBufferLength) dDataBufferLength = dMaxDataBufferLength;
    }

    return dDataBufferLength;
}

BOOL CGatewayCANopenToEsam2::ResetBufferedData()
{
    if(m_pBufferedData) free(m_pBufferedData);
    m_pBufferedData = NULL;
    m_dBufferedDataSize = 0;
    m_dBufferedDataRead = 0;
    m_oNoMoreSegments = FALSE;

    return TRUE;
}

BOOL CGatewayCANopenToEsam2::AddBufferedData(void* pDataBuffer, DWORD dDataBufferSize, BOOL oNoMoreSegments)
{
    DWORD size;
    BYTE* p;

    if(dDataBufferSize > 0)
    {
        //Allocate Memory
        size = m_dBufferedDataSize + dDataBufferSize;
        m_pBufferedData = realloc(m_pBufferedData, size);
        p = (BYTE*)m_pBufferedData + m_dBufferedDataSize;
        memcpy(p, pDataBuffer, dDataBufferSize);
        m_dBufferedDataSize += dDataBufferSize;

        //NoMoreSegments
        m_oNoMoreSegments = oNoMoreSegments;

        return TRUE;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEsam2::IsBufferedDataAvailable(DWORD dSize, DWORD* pdSizeAvailable, BOOL* poNoMoreSegments)
{
    DWORD dTotalSizeAvailable;

    if(m_dBufferedDataSize > m_dBufferedDataRead)
    {
        dTotalSizeAvailable = m_dBufferedDataSize-m_dBufferedDataRead;
        if(dTotalSizeAvailable > dSize)
        {
            if(poNoMoreSegments) *poNoMoreSegments = FALSE;
            if(pdSizeAvailable) *pdSizeAvailable = dSize;
            return TRUE;
        }
        else if(dTotalSizeAvailable == dSize)
        {
            if(poNoMoreSegments) *poNoMoreSegments = m_oNoMoreSegments;
            if(pdSizeAvailable) *pdSizeAvailable = dTotalSizeAvailable;
            return TRUE;
        }
        else if(dTotalSizeAvailable < dSize)
        {
            if(m_oNoMoreSegments)
            {
                if(poNoMoreSegments) *poNoMoreSegments = TRUE;
                if(pdSizeAvailable) *pdSizeAvailable = dTotalSizeAvailable;
                return TRUE;
            }
        }
    }

    if(poNoMoreSegments) *poNoMoreSegments = FALSE;
    return FALSE;
}

BOOL CGatewayCANopenToEsam2::GetBufferedData(void* pDataBuffer, DWORD dDataBufferSize, DWORD* pdDataRead, BOOL* poNoMoreSegments)
{
    DWORD dSizeAvailable;
    BYTE* p;

    if(pDataBuffer && pdDataRead && poNoMoreSegments)
    {
        if(IsBufferedDataAvailable(dDataBufferSize, &dSizeAvailable, poNoMoreSegments))
        {
            if(dSizeAvailable > dDataBufferSize) dSizeAvailable = dDataBufferSize;
            p = (BYTE*)m_pBufferedData + m_dBufferedDataRead;
            memcpy(pDataBuffer, p, dSizeAvailable);
            m_dBufferedDataRead += dSizeAvailable;
            *pdDataRead = dSizeAvailable;

            return TRUE;
        }
    }

    return FALSE;
}

BOOL CGatewayCANopenToEsam2::ReverseBufferedData(DWORD dReverseSize)
{
    if(m_dBufferedDataRead >= dReverseSize)
    {
        m_dBufferedDataRead -= dReverseSize;
        return TRUE;
    }

    return FALSE;
}

BOOL CGatewayCANopenToEsam2::SetNetworkIndication(WORD p_wTargetNetworkId, BYTE p_uTargetNodeId)
{
    BOOL oResult = FALSE;

    m_ulNetworkIndicationThreadId = MmcGetCurrentThreadId();
    m_wTargetNetworkId = p_wTargetNetworkId;
    m_uTargetNodeId = p_uTargetNodeId;
    oResult = TRUE;

    return oResult;
}

BOOL CGatewayCANopenToEsam2::ResetNetworkIndication()
{
    BOOL oResult = FALSE;

    m_ulNetworkIndicationThreadId = 0;
    m_wTargetNetworkId = 0;
    m_uTargetNodeId = 0;
    oResult = TRUE;

    return oResult;
}

BOOL CGatewayCANopenToEsam2::IsNetworkIndicationActive()
{
    return (m_ulNetworkIndicationThreadId != 0);
}

BOOL CGatewayCANopenToEsam2::IsNetworkIndicationThread()
{
    BOOL oResult = FALSE;

    if(IsNetworkIndicationActive())
    {
        oResult = (MmcGetCurrentThreadId() == m_ulNetworkIndicationThreadId);
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::InitLayerParameterStack(CCommandRoot* p_pCommand)
{
    CLayerParameterStack layerParameterStack;
    CLayerParameterSet layerParameterSet;
    BOOL oResult = FALSE;

    if(p_pCommand)
    {
        if(p_pCommand->GetLayerParameterStack(layerParameterStack))
        {
            //Pop PS Layer Parameter Set
            layerParameterStack.PopLayer(PROTOCOL_STACK_LAYER, layerParameterSet);

            //Set I Layer Commands
            if(m_pCommand_WriteObject) m_pCommand_WriteObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedWrite) m_pCommand_InitiateSegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedWrite) m_pCommand_SegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadObject) m_pCommand_ReadObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedRead) m_pCommand_InitiateSegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedRead) m_pCommand_SegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_AbortSegmentedTransfer) m_pCommand_AbortSegmentedTransfer->SetLayerParameterStack(layerParameterStack);

            if(m_pCommand_SendCANFrame) m_pCommand_SendCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadCANFrame) m_pCommand_ReadCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_RequestCANFrame) m_pCommand_RequestCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SendNMTService) m_pCommand_SendNMTService->SetLayerParameterStack(layerParameterStack);

            if(m_pCommand_SendLSSFrame) m_pCommand_SendLSSFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadLSSFrame) m_pCommand_ReadLSSFrame->SetLayerParameterStack(layerParameterStack);
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD p_ulTimeout)
{
    WORD usTimeout(0);
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    HANDLE hTransactionHandle(NULL);
    WORD usNetworkId(0);
    BYTE ubNodeId(0);
    CStdString protocolStackName(_T(""));
    CStdString interfaceName(_T(""));

    if(p_pManager)
    {
        if(p_pManager->DCS_GetProtocolStackName(p_hHandle, &protocolStackName) && p_pManager->DCS_GetInterfaceName(p_hHandle, &interfaceName))
        {
            //Init
            usTimeout = (WORD)p_ulTimeout;

            if(GetAddressParameter(p_pManager, p_hHandle, ubNodeId, &errorInfo))
            {
                if(protocolStackName == PROTOCOL_CAN_OPEN)
                {
                    //CAN Frame Timeout
                    oResult = DCS_WriteObject(p_pManager, p_hHandle, hTransactionHandle, usNetworkId, ubNodeId, INDEX_CAN_FRAME_TIMEOUT, SUBINDEX_CAN_FRAME_TIMEOUT, (BYTE*)&usTimeout, sizeof(usTimeout), &errorInfo);
                }
                else if(protocolStackName == PROTOCOL_MAXON_SERIAL_V2)
                {
                    if(interfaceName == INTERFACE_RS232)
                    {
                        //RS232 Frame Timeout
                           oResult = DCS_WriteObject(p_pManager, p_hHandle, hTransactionHandle, usNetworkId, ubNodeId, INDEX_RS232_FRAME_TIMEOUT, SUBINDEX_RS232_FRAME_TIMEOUT, (BYTE*)&usTimeout, sizeof(usTimeout), &errorInfo);
                    }
                    else if(interfaceName == INTERFACE_USB)
                    {
                        //USB Frame Timeout
                        oResult = DCS_WriteObject(p_pManager, p_hHandle, hTransactionHandle, usNetworkId, ubNodeId, INDEX_USB_FRAME_TIMEOUT, SUBINDEX_USB_FRAME_TIMEOUT, (BYTE*)&usTimeout, sizeof(usTimeout), &errorInfo);
                    }
                }
            }
        }
    }

    return oResult;
}

BOOL CGatewayCANopenToEsam2::GetAddressParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo)
{
    const CStdString NODE_ID = _T("NodeId");

    BOOL oResult(FALSE);

    if(p_pManager)
    {
        //Get Address Parameter from Device
        oResult = p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_rNodeId, sizeof(p_rNodeId), p_pErrorInfo);
    }

    return oResult;
}
