#include "stdafx.h"
#include <malloc.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/BaseClasses/Command_PS.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/Command_PS_InfoteamSerial.h>
#include <CommunicationModel/Interface/BaseClasses/InterfaceManagerBase.h>
#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/InfoteamSerial/Frame/InfoteamSerialReceivingFrame.h>
#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/InfoteamSerial/Frame/InfoteamSerialSendingFrame.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>

#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/InfoteamSerial/GatewayInfoteamSerialToI.h>

CGatewayInfoteamSerialToI::CGatewayInfoteamSerialToI()
{
    m_pSendingFrame = new CInfoteamSerialSendingFrame();
    m_pReceivingFrame = new CInfoteamSerialReceivingFrame();
}

CGatewayInfoteamSerialToI::~CGatewayInfoteamSerialToI()
{
    if(m_pSendingFrame) delete m_pSendingFrame;
    if(m_pReceivingFrame) delete m_pReceivingFrame;
}

BOOL CGatewayInfoteamSerialToI::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* pManager, HANDLE h_Handle, HANDLE hTransactionHandle)
{
    CCommand_PS* pCommand_PS;
    CInterfaceManagerBase* pInterfaceManager;
    BOOL oResult = FALSE;

    if(CGateway::ProcessCommand(p_pCommand, pManager, h_Handle, hTransactionHandle))
    {
        if(CheckLayers(p_pCommand, pManager))
        {
            pCommand_PS = (CCommand_PS*)p_pCommand;
            pInterfaceManager = (CInterfaceManagerBase*)pManager;

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayEtherCATToI")));
            }

            switch(p_pCommand->GetCommandId())
            {
                case INFOTEAM_SERIAL_PROCESS_PROTOCOL       : oResult = Process_ProcessProtocol(pCommand_PS, pInterfaceManager, h_Handle, hTransactionHandle); break;
                case INFOTEAM_SERIAL_PROCESS_PROTOCOL_MAXON : oResult = Process_ProcessProtocolMaxon(pCommand_PS, pInterfaceManager, h_Handle, hTransactionHandle); break;
                case INFOTEAM_SERIAL_ABORT_PROTOCOL_MAXON   : oResult = Process_AbortProtocolMaxon(pCommand_PS, pInterfaceManager, h_Handle, hTransactionHandle); break;
                case INFOTEAM_SERIAL_SEND_CHUNK             : oResult = Process_SendChunk(pCommand_PS, pInterfaceManager, h_Handle, hTransactionHandle); break;
                case INFOTEAM_SERIAL_RECEIVE_DATA           : oResult = Process_ReceiveData(pCommand_PS, pInterfaceManager, h_Handle, hTransactionHandle); break;
            }

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayEtherCATToI"), TRUE));
            }
        }
    }

    return oResult;
}

BOOL CGatewayInfoteamSerialToI::Process_ProcessProtocol(CCommand_PS* pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_PACKAGE_SIZE_NO_LIMIT = 0xFFFFFFFF;

    //*Constants PS*
    const int PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int PARAMETER_INDEX_CHUNK_SIZE = 1;
    const int PARAMETER_INDEX_LAST_CHUNK_FLAG = 2;
    const int PARAMETER_INDEX_CHECK_SUM = 3;
    const int PARAMETER_INDEX_DATA = 4;

    const int RETURN_PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int RETURN_PARAMETER_INDEX_CHECK_SUM = 1;
    const int RETURN_PARAMETER_INDEX_DATA = 2;

    //*Variables PS*
    //Parameter
    DWORD dPackageSize;
    WORD wChunkSize;
    BYTE uLastChunkFlag;
    DWORD dChecksum;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength;

    //ReturnParameter
    DWORD dRetPackageSize;
    DWORD dRetChecksum;
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    DWORD dTimeout;

    if(pCommand && p_pInterfaceManager)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_PACKAGE_SIZE, &dPackageSize, sizeof(dPackageSize));
        pCommand->GetParameterData(PARAMETER_INDEX_CHUNK_SIZE, &wChunkSize, sizeof(wChunkSize));
        pCommand->GetParameterData(PARAMETER_INDEX_LAST_CHUNK_FLAG, &uLastChunkFlag, sizeof(uLastChunkFlag));
        pCommand->GetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, dDataBufferLength);

        //Execute command
        dTimeout = pCommand->GetTimeout();
        oResult = SendFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, dPackageSize, wChunkSize, uLastChunkFlag, &dChecksum, pDataBuffer, dDataBufferLength, &errorInfo);
        if(oResult)
        {
            oResult = ReceiveFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, MAX_PACKAGE_SIZE_NO_LIMIT, &dRetPackageSize, &dRetChecksum, &pRetDataBuffer, &dRetDataBufferLength, dTimeout, &errorInfo);
        }

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_PACKAGE_SIZE, &dRetPackageSize, sizeof(dRetPackageSize));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_CHECK_SUM, &dRetChecksum, sizeof(dRetChecksum));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, dRetDataBufferLength);

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayInfoteamSerialToI::Process_ProcessProtocolMaxon(CCommand_PS* pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_PACKAGE_SIZE_MAXON = 256;

    //*Constants PS*
    const int PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int PARAMETER_INDEX_CHUNK_SIZE = 1;
    const int PARAMETER_INDEX_LAST_CHUNK_FLAG = 2;
    const int PARAMETER_INDEX_CHECK_SUM = 3;
    const int PARAMETER_INDEX_PACKAGE_TYPE = 4;
    const int PARAMETER_INDEX_OPCODE = 5;
    const int PARAMETER_INDEX_DATA = 6;
    const int PARAMETER_INDEX_KEEP_LOCK = 7;

    const int RETURN_PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int RETURN_PARAMETER_INDEX_CHECK_SUM = 1;
    const int RETURN_PARAMETER_INDEX_PACKAGE_TYPE = 2;
    const int RETURN_PARAMETER_INDEX_DATA = 3;

    //*Variables PS*
    //Parameter
    DWORD dPackageSize;
    WORD wChunkSize;
    BYTE uLastChunkFlag;
    DWORD dChecksum;
    BYTE uPackageType = 0;
    BYTE uOpCode = 0;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength;
    BYTE ubKeepLock = 0;

    BYTE* p = NULL;
    DWORD dSize;

    //ReturnParameter
    DWORD dRetPackageSize;
    DWORD dRetChecksum;
    BYTE uRetPackageType = 0;
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    DWORD dTimeout;

    if(pCommand && p_pInterfaceManager)
    {
        //Lock critical section
        if(!IsLocked(pCommand))
        {
            if(!Lock(pCommand)) return FALSE;
        }

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA) + sizeof(uPackageType) + sizeof(uOpCode);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_PACKAGE_SIZE, &dPackageSize, sizeof(dPackageSize));
        pCommand->GetParameterData(PARAMETER_INDEX_CHUNK_SIZE, &wChunkSize, sizeof(wChunkSize));
        pCommand->GetParameterData(PARAMETER_INDEX_LAST_CHUNK_FLAG, &uLastChunkFlag, sizeof(uLastChunkFlag));
        pCommand->GetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));
        pCommand->GetParameterData(PARAMETER_INDEX_KEEP_LOCK, &ubKeepLock, sizeof(ubKeepLock));

        p = (BYTE*)pDataBuffer;
        pCommand->GetParameterData(PARAMETER_INDEX_PACKAGE_TYPE, p, sizeof(uPackageType));p+=sizeof(uPackageType);
        pCommand->GetParameterData(PARAMETER_INDEX_OPCODE, p, sizeof(uOpCode));p+=sizeof(uOpCode);
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, p, pCommand->GetParameterLength(PARAMETER_INDEX_DATA));

        //Execute command
        dTimeout = pCommand->GetTimeout();
        oResult = SendFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, dPackageSize, wChunkSize, uLastChunkFlag, &dChecksum, pDataBuffer, dDataBufferLength, &errorInfo);
        if(oResult)
        {
            oResult = ReceiveFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, MAX_PACKAGE_SIZE_MAXON, &dRetPackageSize, &dRetChecksum, &pRetDataBuffer, &dRetDataBufferLength, dTimeout, &errorInfo);
        }

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_PACKAGE_SIZE, &dRetPackageSize, sizeof(dRetPackageSize));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_CHECK_SUM, &dRetChecksum, sizeof(dRetChecksum));

        if(dRetDataBufferLength > 0)
        {
            p = (BYTE*)pRetDataBuffer;
            dSize = dRetDataBufferLength;
            pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_PACKAGE_TYPE, p, sizeof(uRetPackageType)); p+=sizeof(uPackageType); dSize -= sizeof(uPackageType);
            pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, p, dSize);
        }

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        if(!ubKeepLock) Unlock();
    }

    return oResult;
}

BOOL CGatewayInfoteamSerialToI::Process_AbortProtocolMaxon(CCommand_PS* p_pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle)
{
    CErrorInfo errorInfo;
    BOOL oResult = FALSE;

    if(p_pCommand)
    {
        //Unlock critical section
        Unlock();
        oResult = TRUE;

        //Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayInfoteamSerialToI::Process_SendChunk(CCommand_PS* pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle)
{
    //*Constants PS*
    const int PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int PARAMETER_INDEX_CHUNK_SIZE = 1;
    const int PARAMETER_INDEX_LAST_CHUNK_FLAG = 2;
    const int PARAMETER_INDEX_CHECK_SUM = 3;
    const int PARAMETER_INDEX_DATA = 4;

    //*Variables PS*
    //Parameter
    DWORD dPackageSize;
    WORD wChunkSize;
    BYTE uLastChunkFlag;
    DWORD dChecksum;
    void* pDataBuffer = NULL;
    DWORD dDataBufferLength;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(pCommand && p_pInterfaceManager)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Prepare DataBuffer
        dDataBufferLength = pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(dDataBufferLength > 0) pDataBuffer = malloc(dDataBufferLength);

        //Get PS Parameter Data
        pCommand->GetParameterData(PARAMETER_INDEX_PACKAGE_SIZE, &dPackageSize, sizeof(dPackageSize));
        pCommand->GetParameterData(PARAMETER_INDEX_CHUNK_SIZE, &wChunkSize, sizeof(wChunkSize));
        pCommand->GetParameterData(PARAMETER_INDEX_LAST_CHUNK_FLAG, &uLastChunkFlag, sizeof(uLastChunkFlag));
        pCommand->GetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));
        pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, dDataBufferLength);

        //Execute command
        oResult = SendFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, dPackageSize, wChunkSize, uLastChunkFlag, &dChecksum, pDataBuffer, dDataBufferLength, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetParameterData(PARAMETER_INDEX_CHECK_SUM, &dChecksum, sizeof(dChecksum));

        //Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayInfoteamSerialToI::Process_ReceiveData(CCommand_PS* pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle)
{
    const DWORD MAX_PACKAGE_SIZE_NO_LIMIT = 0xFFFFFFFF;

    //*Constants PS*
    const int RETURN_PARAMETER_INDEX_PACKAGE_SIZE = 0;
    const int RETURN_PARAMETER_INDEX_CHECK_SUM = 1;
    const int RETURN_PARAMETER_INDEX_DATA = 2;

    //*Variables PS*
    //ReturnParameter
    DWORD dRetPackageSize;
    DWORD dRetChecksum;
    void* pRetDataBuffer = NULL;
    DWORD dRetDataBufferLength = 0;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    DWORD dTimeout;

    if(pCommand && p_pInterfaceManager)
    {
        //Lock critical section
        if(!Lock(pCommand)) return FALSE;

        //Execute command
        dTimeout = pCommand->GetTimeout();
        oResult = ReceiveFrameRepeated(p_pInterfaceManager, hI_Handle, hTransactionHandle, MAX_PACKAGE_SIZE_NO_LIMIT, &dRetPackageSize, &dRetChecksum, &pRetDataBuffer, &dRetDataBufferLength, dTimeout, &errorInfo);

        //Set PS ReturnParameter Data
        pCommand->SetStatus(oResult, &errorInfo);
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_PACKAGE_SIZE, &dRetPackageSize, sizeof(dRetPackageSize));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_CHECK_SUM, &dRetChecksum, sizeof(dRetChecksum));
        pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, dRetDataBufferLength);

        //Free DataBuffer
        if(pRetDataBuffer) free(pRetDataBuffer);

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

CGateway* CGatewayInfoteamSerialToI::Clone()
{
    CGatewayInfoteamSerialToI* pClonedGateway;

    pClonedGateway = new CGatewayInfoteamSerialToI();
    *pClonedGateway = *this;

    return pClonedGateway;
}

CGatewayInfoteamSerialToI& CGatewayInfoteamSerialToI::operator=(CGatewayInfoteamSerialToI& other)
{
    if(this != &other)
    {
        *((CGatewayPStoI*)this) = *((CGatewayPStoI*)&other);
    }

    return *this;
}

BOOL CGatewayInfoteamSerialToI::SendFrameRepeated(CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle, DWORD dPackageSize, WORD wChunkSize, BYTE uLastChunkFlag, DWORD* pdChecksum, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_SENDING_COUNT = 3;

    CErrorInfo errorInfo;
    DWORD dSendingCount = 0;

    while(dSendingCount < MAX_SENDING_COUNT)
    {
        dSendingCount++;
        if(SendFrame(p_pInterfaceManager, hI_Handle, hTransactionHandle, dPackageSize, wChunkSize, uLastChunkFlag, pdChecksum, pDataBuffer, dDataBufferLength, &errorInfo))
        {
            return TRUE;
        }
        else
        {
            if(errorInfo.GetErrorCode() != ERROR_INFOTEAM_SERIAL_REP_ACK_RECEIVED)
            {
                if(p_pErrorInfo) *p_pErrorInfo = errorInfo;
                return FALSE;
            }
        }
    }

    if(p_pErrorInfo) *p_pErrorInfo = errorInfo;
    return FALSE;
}

BOOL CGatewayInfoteamSerialToI::SendFrame(CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle, DWORD dPackageSize, WORD wChunkSize, BYTE uLastChunkFlag, DWORD* pdChecksum, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BYTE uAck;
    DWORD dWritten;
    DWORD dRead;

    if(p_pInterfaceManager && hI_Handle && m_pSendingFrame)
    {
        if(p_pErrorInfo) p_pErrorInfo->Reset();

        //Check CunkSize
        if(wChunkSize > MAX_CHUNK_SIZE)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INFOTEAM_SERIAL_CHUNK_SIZE_TOO_HIGH, p_pErrorInfo);
            return FALSE;
        }

        //Initialization
        p_pInterfaceManager->I_ResetInterface(hI_Handle);
        if(!m_pSendingFrame->PrepareSendFrame(dPackageSize, wChunkSize, uLastChunkFlag, pdChecksum, pDataBuffer, dDataBufferLength))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        //Send Header & Data
        if(!I_WriteData(p_pInterfaceManager, hI_Handle, hTransactionHandle, m_pSendingFrame->GetFramePointer(), m_pSendingFrame->GetFrameSize(), &dWritten, p_pErrorInfo)) return FALSE;
        if(dWritten != m_pSendingFrame->GetFrameSize())
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        //Wait for acknowledge
        if(!I_ReadData(p_pInterfaceManager, hI_Handle, hTransactionHandle, &uAck, sizeof(uAck), &dRead, p_pErrorInfo))
        {
            return FALSE;
        }

        //Check acknowledge
        if(dRead != 0)
        {
            switch(uAck)
            {
                case OK_ACK:
                    {
                        return TRUE;
                    } break;

                case REP_ACK:
                    {
                        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INFOTEAM_SERIAL_REP_ACK_RECEIVED, p_pErrorInfo);
                        return FALSE;
                    } break;

                default:
                    {
                        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INFOTEAM_SERIAL_BAD_ACK_RECEIVED, p_pErrorInfo);
                        return FALSE;
                    };
            }
        }
        else
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayInfoteamSerialToI::ReceiveFrameRepeated(CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle, DWORD p_ulMaxPackageSize, DWORD* pdRetPackageSize, DWORD* pdRetChecksum, void** ppRetDataBuffer, DWORD* pdRetDataBufferLength, DWORD dTimeout, CErrorInfo* p_pErrorInfo)
{
    const DWORD MAX_RECEIVING_COUNT = 3;

    CErrorInfo errorInfo;
    DWORD dReceivingCount = 0;
    BOOL oSendRepAck = TRUE;

    while(dReceivingCount < MAX_RECEIVING_COUNT)
    {
        dReceivingCount++;
        if(dReceivingCount == MAX_RECEIVING_COUNT) oSendRepAck = FALSE;
        if(ReceiveFrame(p_pInterfaceManager, hI_Handle, hTransactionHandle, p_ulMaxPackageSize, pdRetPackageSize, pdRetChecksum, ppRetDataBuffer, pdRetDataBufferLength, dTimeout, oSendRepAck, &errorInfo))
        {
            return TRUE;
        }
        else
        {
            if(!oSendRepAck || (errorInfo.GetErrorCode() != ERROR_INFOTEAM_SERIAL_BAD_DATA_SIZE_RECEIVED) || (errorInfo.GetErrorCode() != ERROR_INFOTEAM_SERIAL_BAD_CRC_RECEIVED))
            {
                if(p_pErrorInfo) *p_pErrorInfo = errorInfo;
                return FALSE;
            }
        }
    }

    if(p_pErrorInfo) *p_pErrorInfo = errorInfo;
    return FALSE;
}

BOOL CGatewayInfoteamSerialToI::ReceiveFrame(CInterfaceManagerBase* p_pInterfaceManager, HANDLE hI_Handle, HANDLE hTransactionHandle, DWORD p_ulMaxPackageSize, DWORD* pdRetPackageSize, DWORD* pdRetChecksum, void** ppRetDataBuffer, DWORD* pdRetDataBufferLength, DWORD dTimeout, BOOL oSendRepAck, CErrorInfo* p_pErrorInfo)
{
    DWORD startTime, time;
    BOOL oFirstDataReceived;
    DWORD dPackageSize = 0;
    BYTE uAck;

    DWORD dWritten; //number of bytes written
    DWORD dRead;    //number of bytes read
    BOOL oCrcOkay = FALSE;
    BOOL oDataOkay = TRUE;

    if(p_pInterfaceManager && hI_Handle && m_pReceivingFrame)
    {
        if(p_pErrorInfo) p_pErrorInfo->Reset();

        //Read PackageSize (FirstData / Including Timeout Handling)
        oFirstDataReceived = FALSE;
        time = 0;

        startTime = MmcGetTickCount();

        while((!oFirstDataReceived && (time < dTimeout)) || (!oFirstDataReceived && (dTimeout == 0)))
        {
            if(I_ReadData(p_pInterfaceManager, hI_Handle, hTransactionHandle, &dPackageSize, sizeof(dPackageSize), &dRead, p_pErrorInfo))
            {
                if(dRead == sizeof(dPackageSize))
                {
                    oFirstDataReceived = TRUE;
                }
            }

            if(dTimeout != 0) time = MmcGetTickCount()-startTime;

        }
        if(!oFirstDataReceived)
        {
            return FALSE;
        }

        //Prepare Receive Frame
        if(!m_pReceivingFrame->PrepareReceiveFrame(dPackageSize, p_ulMaxPackageSize))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        //Read Checksum, Data
        if(!I_ReadData(p_pInterfaceManager, hI_Handle, hTransactionHandle, m_pReceivingFrame->GetChecksumPointer(), m_pReceivingFrame->GetChecksumSize()+m_pReceivingFrame->GetDataSize(), &dRead, p_pErrorInfo))
        {
            oDataOkay = FALSE;
        }
        if(dRead != (m_pReceivingFrame->GetChecksumSize()+m_pReceivingFrame->GetDataSize()))
        {
            oDataOkay = FALSE;
        }

        //Check CRC
        if(oDataOkay) oCrcOkay = m_pReceivingFrame->CheckCRC();
        if(oDataOkay && oCrcOkay)
        {
            //Send Okay Acknowledge
            uAck = OK_ACK;
        }
        else
        {
            if(oSendRepAck)
            {
                //Send Repeat Acknowledge
                p_pInterfaceManager->I_ResetInterface(hI_Handle);
                uAck = REP_ACK;
            }
            else
            {
                //Send Okay Acknowledge and handle frame as failed
                uAck = OK_ACK;
            }
        }

        //Send Acknowledge
        if(!I_WriteData(p_pInterfaceManager, hI_Handle, hTransactionHandle, &uAck, sizeof(uAck), &dWritten, p_pErrorInfo))
        {
            return FALSE;
        }
        if(dWritten != sizeof(uAck))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        //Copy received data to destination
        if(oDataOkay && oCrcOkay)
        {
            if(!m_pReceivingFrame->CopyReceivedData(pdRetPackageSize, pdRetChecksum, ppRetDataBuffer, pdRetDataBufferLength))
            {
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
                return FALSE;
            }
            return TRUE;
        }
        else //REP_ACK
        {
            if(!oDataOkay)
            {
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INFOTEAM_SERIAL_BAD_DATA_SIZE_RECEIVED, p_pErrorInfo);
            }
            else if(!oCrcOkay)
            {
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INFOTEAM_SERIAL_BAD_CRC_RECEIVED, p_pErrorInfo);
            }
            return FALSE;
        }
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayInfoteamSerialToI::I_ReadData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, void* p_pData, DWORD p_ulNumberOfBytesToRead, DWORD* p_pulNumberOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    //Has to be implementeded by child class
    return oResult;
}

BOOL CGatewayInfoteamSerialToI::I_WriteData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, void* p_pData, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    //Has to be implementeded by child class
    return oResult;
}
