#include "stdafx.h"
#include <malloc.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/BaseClasses/Command_PS.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/Command_PS_MaxonSerialV2.h>
#include <CommunicationModel/Interface/BaseClasses/InterfaceManagerBase.h>
#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/MaxonSerialV2/Frame/MaxonSerialV2Frame.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>

#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/MaxonSerialV2/GatewayMaxonSerialV2ToI.h>

CGatewayMaxonSerialV2ToI::CGatewayMaxonSerialV2ToI()
    : m_pBlockBuffer(NULL)
    , m_ulBlockBufferSize(0)
    , m_ulGetBlockBufferIndex(0)
{
    m_pSendingFrame = new CMaxonSerialV2Frame();
    m_pReceivingFrame = new CMaxonSerialV2Frame();
}

CGatewayMaxonSerialV2ToI::~CGatewayMaxonSerialV2ToI()
{
    DeleteBlockBuffer();
    if(m_pSendingFrame) delete m_pSendingFrame;
    if(m_pReceivingFrame) delete m_pReceivingFrame;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_h_Handle, HANDLE p_hTransactionHandle)
{
    CCommand_PS* pCommand_PS;
    CInterfaceManagerBase* pInterfaceManager;
    BOOL oResult = FALSE;

    if(CGateway::ProcessCommand(p_pCommand, p_pManager, p_h_Handle, p_hTransactionHandle))
    {
        if(CheckLayers(p_pCommand, p_pManager))
        {
            pCommand_PS = (CCommand_PS*)p_pCommand;
            pInterfaceManager = (CInterfaceManagerBase*)p_pManager;

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayMaxonSerialV2ToI")));
            }

            switch(p_pCommand->GetCommandId())
            {
                case MAXON_SERIAL_V2_PROCESS_PROTOCOL:
                    {
                        oResult = Process_ProcessProtocol(pCommand_PS, pInterfaceManager, p_h_Handle, p_hTransactionHandle);
                        break;
                    }
                case MAXON_SERIAL_V2_ABORT_PROTOCOL:
                    {
                        oResult = Process_AbortProtocol(pCommand_PS, pInterfaceManager, p_h_Handle, p_hTransactionHandle);
                        break;
                    }
            }

            if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_PS)))
            {
                CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayMaxonSerialV2ToI"), TRUE));
            }
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::Process_ProcessProtocol(CCommand_PS* p_pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle)
{
    // *Constants PS*
    const int PARAMETER_INDEX_OPCODE(0);
    const int PARAMETER_INDEX_LEN(1);
    const int PARAMETER_INDEX_DATA(2);
    const int PARAMETER_INDEX_CRC(3);
    const int PARAMETER_INDEX_KEEP_LOCK(4);
    const int RETURN_PARAMETER_INDEX_OPCODE(0);
    const int RETURN_PARAMETER_INDEX_LEN(1);
    const int RETURN_PARAMETER_INDEX_DATA(2);
    const int RETURN_PARAMETER_INDEX_CRC(3);

    // *Variables PS*
    // Parameter
    BYTE ubOpCode(0);
    BYTE ubLen(0);
    void* pDataBuffer(NULL);
    DWORD ulDataBufferLength(0);
    WORD usCrc(0);
    BYTE ubKeepLock(0);

    // ReturnParameter
    BYTE ubRetOpCode(0);
    BYTE ubRetLen(0);
    void* pRetDataBuffer(NULL);
    DWORD ulRetDataBufferLength(0);
    WORD usRetCrc(0);

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    DWORD ulTimeout;

    if(p_pCommand && p_pInterfaceManager)
    {
        // Lock CriticalSection
        if(!IsLocked(p_pCommand))
        {
            if(!Lock(p_pCommand)) return FALSE;
        }

        // Prepare DataBuffer
        ulDataBufferLength = p_pCommand->GetParameterLength(PARAMETER_INDEX_DATA);
        if(ulDataBufferLength > 0) pDataBuffer = malloc(ulDataBufferLength);

        // Prepare Return DataBuffer
        ulRetDataBufferLength = p_pCommand->GetReturnParameterLength(RETURN_PARAMETER_INDEX_DATA);
        if(ulRetDataBufferLength > 0) pRetDataBuffer = malloc(ulRetDataBufferLength);

        // Get PS Parameter Data
        p_pCommand->GetParameterData(PARAMETER_INDEX_OPCODE, &ubOpCode, sizeof(ubOpCode));
        p_pCommand->GetParameterData(PARAMETER_INDEX_LEN, &ubLen, sizeof(ubLen));
        p_pCommand->GetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);
        p_pCommand->GetParameterData(PARAMETER_INDEX_CRC, &usCrc, sizeof(usCrc));
        p_pCommand->GetParameterData(PARAMETER_INDEX_KEEP_LOCK, &ubKeepLock, sizeof(ubKeepLock));

        // Execute Command
        ulTimeout = p_pCommand->GetTimeout();

        if(CMmcLog::IsActive(LMT_DEBUG, CCommunicationMessageLevel(CML_PS)))
        {
            CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayMaxonSerialV2ToI")));
        }

        if(SendFrame(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubOpCode, &ubLen, &pDataBuffer, &ulDataBufferLength, &usCrc, &errorInfo))
        {
            oResult = ReceiveFrame(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, &ubRetOpCode, &ubRetLen, &pRetDataBuffer, &ulRetDataBufferLength, &usRetCrc, ulTimeout, &errorInfo);
        }

        // Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetParameterData(PARAMETER_INDEX_LEN, &ubLen, sizeof(ubLen));
        p_pCommand->SetParameterData(PARAMETER_INDEX_DATA, pDataBuffer, ulDataBufferLength);
        p_pCommand->SetParameterData(PARAMETER_INDEX_CRC, &usCrc, sizeof(usCrc));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_OPCODE, &ubRetOpCode, sizeof(ubRetOpCode));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_LEN, &ubRetLen, sizeof(ubRetLen));
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_DATA, pRetDataBuffer, ulRetDataBufferLength);
        p_pCommand->SetReturnParameterData(RETURN_PARAMETER_INDEX_CRC, &usRetCrc, sizeof(usRetCrc));

        if(CMmcLog::IsActive(LMT_DEBUG, CCommunicationMessageLevel(CML_PS)))
        {
            CMmcLog::Log(LMT_DEBUG, CCommunicationMessageLevel(CML_PS), p_pCommand->GetLogMessage(_T("CGatewayMaxonSerialV2ToI"), TRUE));
        }

        // Free DataBuffer
        if(pDataBuffer) free(pDataBuffer);
        if(pRetDataBuffer) free(pRetDataBuffer);

        // Unlock CriticalSection
        if(!ubKeepLock) Unlock();
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::Process_AbortProtocol(CCommand_PS* p_pCommand, CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        // Unlock CriticalSection
        Unlock();
        oResult = TRUE;

        // Set PS ReturnParameter Data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

CGateway* CGatewayMaxonSerialV2ToI::Clone()
{
    CGatewayMaxonSerialV2ToI* pClonedGateway;

    pClonedGateway = new CGatewayMaxonSerialV2ToI();
    *pClonedGateway = *this;

    return pClonedGateway;
}

CGatewayMaxonSerialV2ToI& CGatewayMaxonSerialV2ToI::operator=(CGatewayMaxonSerialV2ToI& other)
{
    if(this != &other)
    {
        *((CGatewayPStoI*)this) = *((CGatewayPStoI*)&other);
    }

    return *this;
}

BOOL CGatewayMaxonSerialV2ToI::SendFrame(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, BYTE p_ubOpCode, BYTE* p_puLen, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, WORD* p_pusCrc, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    DWORD ulToWrite(0);
    DWORD ulWritten(0);    // Number of bytes written

    if(p_pInterfaceManager && p_hI_Handle && m_pSendingFrame)
    {
        // Reset
        oResult = TRUE;
        if(p_pErrorInfo) p_pErrorInfo->Reset();

        // 0. Initialization
        p_pInterfaceManager->I_ResetInterface(p_hI_Handle);
        if(oResult && !m_pSendingFrame->PrepareStuffedSendFrame(p_ubOpCode, p_puLen, p_ppDataBuffer, p_pulDataBufferLength, p_pusCrc))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            oResult = FALSE;
        }

        // 1. Send DLE, STX, OpCode, Len, Data, Crc
        ulToWrite = m_pSendingFrame->GetFrameSize();
        if(oResult && !I_WriteData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, m_pSendingFrame->GetFramePointer(), ulToWrite, &ulWritten, p_pErrorInfo))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_SEND_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
        if(oResult && (ulWritten != ulToWrite))
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_BAD_DATA_SIZE_WRITTEN, p_pErrorInfo);
            oResult = FALSE;
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ReceiveFrame(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, BYTE* p_pubOpCode, BYTE* p_pubLen, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength, WORD* p_pusCrc, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oEscapeState(FALSE);
    BOOL oResult(FALSE);
    BYTE uDle(0);
    BYTE ubLen(0);
    BYTE ubOpCode(0);
    BYTE uStx(0);
    BYTE* pData(NULL);
    DWORD ulNbOfDataBytesToRead(0);
    DWORD ulReadBlockSize(0);
    DWORD ulStartTime(0);
    DWORD ulTime(0);
    EReceiveFrameState receiveFrameState(RFS_WAITING_FOR_DLE);

    if(p_pInterfaceManager && p_hI_Handle && p_ppDataBuffer && p_pulDataBufferLength && m_pReceivingFrame)
    {
        // 1. Reset
        oResult = TRUE;
        DeleteBlockBuffer();
        if(p_pErrorInfo) p_pErrorInfo->Reset();

        // Init
        ulStartTime = MmcGetTickCount();

        ulTime = 0;
        InitWaitingForDleState(receiveFrameState, uDle, ulReadBlockSize);

        // 2. Read Frame
        while(oResult && (receiveFrameState != RFS_FRAME_RECEIVED))
        {
            if(receiveFrameState == RFS_WAITING_FOR_DLE)
            {
                // Waiting for DLE
                if(ulTime < p_ulTimeout)
                {
                    // Timeout not elapsed
                    if(ProcessWaitingForDle(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, receiveFrameState, uDle, ulReadBlockSize, p_pErrorInfo))
                    {
                        // Init Next States
                        if(receiveFrameState == RFS_WAITING_FOR_STX)
                        {
                            InitWaitingForStxState(receiveFrameState, uStx, ulReadBlockSize);
                        }
                    }

                    // Update Time
                    ulTime = MmcGetTickCount() - ulStartTime;
                }
                else
                {
                    // Timeout elapsed
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
            else if(receiveFrameState == RFS_WAITING_FOR_STX)
            {
                // Waiting for STX
                oResult = ProcessWaitingForStx(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, receiveFrameState, uStx, ulReadBlockSize, p_pErrorInfo);

                if (oResult)
                {
                    // Init Next States
                    if(receiveFrameState == RFS_WAITING_FOR_DLE) InitWaitingForDleState(receiveFrameState, uDle, ulReadBlockSize);
                    if(receiveFrameState == RFS_WAITING_FOR_OPCODE) InitWaitingForOpCodeState(receiveFrameState, ubOpCode, ulReadBlockSize, oEscapeState);
                }
            }
            else if(receiveFrameState == RFS_WAITING_FOR_OPCODE)
            {
                // Waiting for OpCode
                oResult = ProcessWaitingForOpCode(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, receiveFrameState, ubOpCode, ulReadBlockSize, oEscapeState, p_pErrorInfo);

                if (oResult)
                {
                    // Init Next States
                    if(receiveFrameState == RFS_WAITING_FOR_LEN) InitWaitingForLenState(receiveFrameState, ubLen, ulReadBlockSize, oEscapeState);
                }
            }
            else if(receiveFrameState == RFS_WAITING_FOR_LEN)
            {
                // Waiting for Len
                oResult = ProcessWaitingForLen(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, receiveFrameState, ubLen, ulReadBlockSize, oEscapeState, p_pErrorInfo);

                if (oResult)
                {
                    // Init Next States
                    if(receiveFrameState == RFS_WAITING_FOR_OPCODE) InitWaitingForOpCodeState(receiveFrameState, ubOpCode, ulReadBlockSize, oEscapeState);
                    if(receiveFrameState == RFS_WAITING_FOR_DATA_AND_CRC) InitWaitingForDataAndCrcState(receiveFrameState, m_pReceivingFrame, ubOpCode, ubLen, *p_pulDataBufferLength, pData, ulNbOfDataBytesToRead, ulReadBlockSize, oEscapeState);
                }
            }
            else if(receiveFrameState == RFS_WAITING_FOR_DATA_AND_CRC)
            {
                // Waiting for Data and Crc
                oResult = ProcessWaitingForDataAndCrc(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, receiveFrameState, pData, ulNbOfDataBytesToRead, ulReadBlockSize, oEscapeState, p_pErrorInfo);

                if (oResult)
                {
                    // Init Next States
                    if(receiveFrameState == RFS_WAITING_FOR_OPCODE) InitWaitingForOpCodeState(receiveFrameState, ubOpCode, ulReadBlockSize, oEscapeState);
                }
            }
        }

        // 3. Check CRC
        if(oResult)
        {
            if(m_pReceivingFrame->CheckCRC())
            {
                // CRC okay
                if(!m_pReceivingFrame->CopyReceivedData(p_pubOpCode, p_pubLen, p_ppDataBuffer, p_pulDataBufferLength, p_pusCrc))
                {
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
            else
            {
                // CRC bad
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_BAD_CRC_RECEIVED, p_pErrorInfo);
                oResult = FALSE;
            }
        }
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitWaitingForDleState(EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruDle, DWORD& p_rulReadBlockSize)
{
    BOOL oResult(FALSE);

    if(m_pReceivingFrame)
    {
        oResult = TRUE;
        p_rReceiveFrameState = RFS_WAITING_FOR_DLE;
        p_ruDle = 0;
        p_rulReadBlockSize = m_pReceivingFrame->GetDleSize() + m_pReceivingFrame->GetStxSize() + m_pReceivingFrame->GetOpCodeSize() + m_pReceivingFrame->GetLenSize();
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitWaitingForStxState(EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruStx, DWORD& p_rulReadBlockSize)
{
    BOOL oResult(FALSE);

    if(m_pReceivingFrame)
    {
        oResult = TRUE;
        p_rReceiveFrameState = RFS_WAITING_FOR_STX;
        p_ruStx = 0;
        p_rulReadBlockSize = m_pReceivingFrame->GetStxSize() + m_pReceivingFrame->GetOpCodeSize() + m_pReceivingFrame->GetLenSize();
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitWaitingForOpCodeState(EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruOpCode, DWORD& p_rulReadBlockSize, BOOL& p_roEscapeState)
{
    BOOL oResult(FALSE);

    if(m_pReceivingFrame)
    {
        oResult = TRUE;
        p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
        p_ruOpCode = 0;
        p_rulReadBlockSize = m_pReceivingFrame->GetOpCodeSize() + m_pReceivingFrame->GetLenSize();
        p_roEscapeState = FALSE;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitWaitingForLenState(EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruLen, DWORD& p_rulReadBlockSize, BOOL p_roEscapeState)
{
    BOOL oResult(FALSE);

    if(m_pReceivingFrame)
    {
        oResult = TRUE;
        p_rReceiveFrameState = RFS_WAITING_FOR_LEN;
        p_ruLen = 0;
        p_rulReadBlockSize = m_pReceivingFrame->GetLenSize();
        p_roEscapeState = FALSE;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitWaitingForDataAndCrcState(EReceiveFrameState& p_rReceiveFrameState, CMaxonSerialV2Frame* p_pReceivingFrame, BYTE p_uOpCode, BYTE p_ubLen, DWORD p_ulRetDataBufferSize, BYTE*& p_rpData, DWORD& p_rulNbOfDataBytesToRead, DWORD& p_rulReadBlockSize, BOOL& p_roEscapeState)
{
    BOOL oResult(FALSE);

    if(p_pReceivingFrame && p_pReceivingFrame->PrepareReceiveFrame(p_uOpCode, p_ubLen, p_ulRetDataBufferSize))
    {
        oResult = TRUE;
        p_rpData = (BYTE*)p_pReceivingFrame->GetDataPointer();
        p_rulNbOfDataBytesToRead = p_pReceivingFrame->GetDataSize() + p_pReceivingFrame->GetCrcSize();
        p_rulReadBlockSize = p_pReceivingFrame->GetDataSize() + p_pReceivingFrame->GetCrcSize();
        p_roEscapeState = FALSE;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessWaitingForDle(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruDle, DWORD& p_rulReadBlockSize, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubData(0);

    if(m_pReceivingFrame)
    {
        if(GetReceivedData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubData, p_rulReadBlockSize))
        {
            if(ubData == m_pReceivingFrame->GetDleCharacter())
            {
                // Goto Waiting for STX
                p_rReceiveFrameState = RFS_WAITING_FOR_STX;
                p_ruDle = ubData;
                oResult = TRUE;
            }
            else
            {
                // Goto Waiting for DLE
                p_rReceiveFrameState = RFS_WAITING_FOR_DLE;
                p_ruDle = 0;
                oResult = TRUE;
            }
        }
        else
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessWaitingForStx(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruStx, DWORD& p_rulReadBlockSize, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubData(0);

    if(m_pReceivingFrame)
    {
        if(GetReceivedData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubData, p_rulReadBlockSize))
        {
            if(ubData == m_pReceivingFrame->GetStxCharacter())
            {
                // Goto Waiting for OpCode
                p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
                p_ruStx = ubData;
                oResult = TRUE;
            }
            else if(ubData == m_pReceivingFrame->GetDleCharacter())
            {
                // Goto Waiting for STX
                p_rReceiveFrameState = RFS_WAITING_FOR_STX;
                p_ruStx = 0;
                oResult = TRUE;
            }
        }
        else
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessWaitingForOpCode(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruOpCode, DWORD& p_rulReadBlockSize, BOOL& p_roEscapeState, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubData(0);

    if(m_pReceivingFrame)
    {
        if(GetReceivedData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubData, p_rulReadBlockSize))
        {
            if(p_roEscapeState)
            {
                // Already in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Goto Waiting for Len
                    p_rReceiveFrameState = RFS_WAITING_FOR_LEN;
                    p_roEscapeState = FALSE;
                    p_ruOpCode = ubData;
                    oResult = TRUE;
                }
                else if(ubData == m_pReceivingFrame->GetStxCharacter())
                {
                    // Goto Waiting for OpCode
                    p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
                    p_roEscapeState = FALSE;
                    p_ruOpCode = 0;
                    oResult = TRUE;
                }
                else
                {
                    // Stuffing Error
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_DESTUFFING, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
            else
            {
                // Not in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Goto Escape OpCode
                    p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
                    p_roEscapeState = TRUE;
                    p_ruOpCode = 0;
                    oResult = TRUE;
                }
                else
                {
                    // Goto Waiting for Len
                    p_rReceiveFrameState = RFS_WAITING_FOR_LEN;
                    p_roEscapeState = FALSE;
                    p_ruOpCode = ubData;
                    oResult = TRUE;
                }
            }
        }
        else
        {
            // Failed reading Data
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessWaitingForLen(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, EReceiveFrameState& p_rReceiveFrameState, BYTE& p_ruLen, DWORD& p_rulReadBlockSize, BOOL& p_roEscapeState, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubData(0);

    if(m_pReceivingFrame)
    {
        if(GetReceivedData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubData, p_rulReadBlockSize))
        {
            if(p_roEscapeState)
            {
                // Already in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Goto Waiting for Data and Crc
                    p_rReceiveFrameState = RFS_WAITING_FOR_DATA_AND_CRC;
                    p_roEscapeState = FALSE;
                    p_ruLen = ubData;
                    oResult = TRUE;
                }
                else if(ubData == m_pReceivingFrame->GetStxCharacter())
                {
                    // Goto Waiting for OpCode
                    p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
                    p_roEscapeState = FALSE;
                    p_ruLen = 0;
                    oResult = TRUE;
                }
                else
                {
                    // Stuffing Error
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_DESTUFFING, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
            else
            {
                // Not in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Goto Escape Len
                    p_rReceiveFrameState = RFS_WAITING_FOR_LEN;
                    p_roEscapeState = TRUE;
                    p_ruLen = 0;
                    oResult = TRUE;
                }
                else
                {
                    // Goto Waiting for Data and Crc
                    p_rReceiveFrameState = RFS_WAITING_FOR_DATA_AND_CRC;
                    p_roEscapeState = FALSE;
                    p_ruLen = ubData;
                    oResult = TRUE;
                }
            }
        }
        else
        {
            // Failed reading Data
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::ProcessWaitingForDataAndCrc(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, EReceiveFrameState& p_rReceiveFrameState, BYTE*& p_rpData, DWORD& p_rulNbOfDataBytesToRead, DWORD& p_rulReadBlockSize, BOOL& p_roEscapeState, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubData(0);

    if(m_pReceivingFrame && p_rpData)
    {
        if(GetReceivedData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, ubData, p_rulReadBlockSize))
        {
            if(p_roEscapeState)
            {
                // Already in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Copy Data
                    *p_rpData = ubData;
                    p_rpData++;
                    p_rulNbOfDataBytesToRead--;
                    p_rulReadBlockSize--;
                    p_roEscapeState = FALSE;
                    oResult = TRUE;

                    if(p_rulNbOfDataBytesToRead == 0)
                    {
                        // Goto Frame Received
                        p_rReceiveFrameState = RFS_FRAME_RECEIVED;
                    }
                }
                else if(ubData == m_pReceivingFrame->GetStxCharacter())
                {
                    // Goto Waiting for OpCode
                    p_rReceiveFrameState = RFS_WAITING_FOR_OPCODE;
                    p_roEscapeState = FALSE;
                    oResult = TRUE;
                }
                else
                {
                    // Stuffing Error
                    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_DESTUFFING, p_pErrorInfo);
                    oResult = FALSE;
                }
            }
            else
            {
                // Not in Escape State
                if(ubData == m_pReceivingFrame->GetDleCharacter())
                {
                    // Goto Escape Len
                    p_rReceiveFrameState = RFS_WAITING_FOR_DATA_AND_CRC;
                    p_roEscapeState = TRUE;
                    oResult = TRUE;
                }
                else
                {
                    // Copy Data
                    *p_rpData = ubData;
                    p_rpData++;
                    p_rulNbOfDataBytesToRead--;
                    p_rulReadBlockSize--;
                    p_roEscapeState = FALSE;
                    oResult = TRUE;

                    if(p_rulNbOfDataBytesToRead == 0)
                    {
                        // Goto Frame Received
                        p_rReceiveFrameState = RFS_FRAME_RECEIVED;
                    }
                }
            }
        }
        else
        {
            // Failed reading Data
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_RECEIVE_FRAME, p_pErrorInfo);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::GetReceivedData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, BYTE& p_ruData, DWORD p_ulReadBlockSize)
{
    BOOL oResult(FALSE);

    if(GetDataFromBlockBuffer(p_ruData))
    {
        // Buffered Data from BlockBuffer
        oResult = TRUE;
    }
    else
    {
        // Update BlockBuffer
        if(UpdateBlockBuffer(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, p_ulReadBlockSize))
        {
            oResult = GetDataFromBlockBuffer(p_ruData);
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::InitBlockBuffer(DWORD p_ulReadBlockSize)
{
    BOOL oResult(TRUE);

    DeleteBlockBuffer();
    if(!m_pBlockBuffer)
    {
        m_pBlockBuffer = (BYTE*)malloc(p_ulReadBlockSize);
        m_ulBlockBufferSize = p_ulReadBlockSize;
        m_ulGetBlockBufferIndex = 0;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::UpdateBlockBuffer(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, DWORD p_ulReadBlockSize)
{
    DWORD ulNumberOfBytesRead(0);
    CErrorInfo errorInfo;
    BOOL oResult(FALSE);

    if(InitBlockBuffer(p_ulReadBlockSize))
    {
        if(I_ReadData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, m_pBlockBuffer, m_ulBlockBufferSize, &ulNumberOfBytesRead, &errorInfo))
        {
            if(m_ulBlockBufferSize == ulNumberOfBytesRead)
            {
                oResult = TRUE;
            }
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::DeleteBlockBuffer()
{
    BOOL oResult(TRUE);

    if(m_pBlockBuffer)
    {
        free(m_pBlockBuffer);
        m_pBlockBuffer = NULL;
        m_ulBlockBufferSize = 0;
        m_ulGetBlockBufferIndex = 0;
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::GetDataFromBlockBuffer(BYTE& p_ruData)
{
    BOOL oResult(FALSE);

    if(m_pBlockBuffer)
    {
        if(m_ulGetBlockBufferIndex < m_ulBlockBufferSize)
        {
            p_ruData = *(m_pBlockBuffer+m_ulGetBlockBufferIndex);
            m_ulGetBlockBufferIndex++;
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::I_ReadData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, void* p_pData, DWORD p_ulNumberOfBytesToRead, DWORD* p_pulNumberOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    // Has to be implementeded by child class
    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::I_WriteData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, void* p_pData, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    // Has to be implementeded by child class
    return oResult;
}

BOOL CGatewayMaxonSerialV2ToI::I_WriteStuffedData(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle, HANDLE p_hTransactionHandle, void* p_pData, DWORD p_ulNumberOfBytesToWrite, DWORD* p_pulNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    DWORD ulNumberOfStuffedBytes(0);
    DWORD ulNumberOfStuffedBytesToWrite(0);
    DWORD ulNumberOfStuffedBytesWritten(0);
    void* pStuffedData(NULL);

    // Stuffing Data
    if(CMaxonSerialV2Frame::StuffingData((BYTE*)p_pData, p_ulNumberOfBytesToWrite, (BYTE*&)pStuffedData, ulNumberOfStuffedBytesToWrite, ulNumberOfStuffedBytes))
    {
        // Write Data
        if(I_WriteData(p_pInterfaceManager, p_hI_Handle, p_hTransactionHandle, pStuffedData, ulNumberOfStuffedBytesToWrite, &ulNumberOfStuffedBytesWritten, p_pErrorInfo))
        {
            if(p_pulNumberOfBytesWritten) *p_pulNumberOfBytesWritten = ulNumberOfStuffedBytesWritten - ulNumberOfStuffedBytes;
            oResult = TRUE;
        }

        if(pStuffedData) free(pStuffedData);
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_MAXON_SERIAL_V2_STUFFING, p_pErrorInfo);
        oResult = FALSE;
    }

    return oResult;
}

bool CGatewayMaxonSerialV2ToI::StartTime(double* p_pdStartCount)
{
    // Start timer
    if(p_pdStartCount)
    {
        *p_pdStartCount = m_timer.start();
    }

    return true;
}

DWORD CGatewayMaxonSerialV2ToI::StopTime(double p_dStartCount)
{
    // Stop timer
    m_timer.stop();

    return (DWORD)m_timer.getElapsedTimeInMilliSec();
}
