#include "stdafx.h"
#include <algorithm>

#include <Process/MmcProcess.h>
#include <Thread/MmcSemaphore.h>
#include <Thread/MmcSingleLock.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/ProtocolStack/BaseClasses/Command_PS.h>
#include <CommunicationModel/ProtocolStack/Classes/GatewayToInterface/BaseClasses/GatewayPStoI.h>
#include <CommunicationModel/ProtocolStack/Classes/GatewayToDevice/BaseClasses/GatewayPStoDCS.h>

#include <CommunicationModel/Interface//InterfaceManager.h>
#include <CommunicationModel/DeviceCommandSet/DeviceCommandSetManager.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInManager.h>

#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/BaseClasses/ProtocolStackBase.h>

using namespace std;

CProtocolStackBase::CProtocolStackBase()
{
    m_lInstanceValue = 0;
    m_strProtocolStackName = _T("");

    m_pJournalManager = 0;
    m_pGatewayToInterface = 0;
    m_pGatewayToDevice = 0;
    m_dTimeout = 0;
    m_ulBaudrate = 0;

    m_pPlugInManager = 0;
    m_pReopenSemaphore = 0;
    m_pErrorHandling = 0;
    InitErrorHandling();
}

CProtocolStackBase::CProtocolStackBase(const CProtocolStackBase& p_rObject)
{
    //Instance Value
    m_lInstanceValue = p_rObject.m_lInstanceValue;

    //Timeout
    m_dTimeout = p_rObject.m_dTimeout;
    m_ulBaudrate = p_rObject.m_ulBaudrate;

    //Protokolstackname kopieren
    m_strProtocolStackName = p_rObject.m_strProtocolStackName;

    //JournalManager
    m_pJournalManager = p_rObject.m_pJournalManager;

    //Gateways
    if(p_rObject.m_pGatewayToInterface)
    {
        m_pGatewayToInterface = (CGatewayPStoI*)p_rObject.m_pGatewayToInterface->Clone();
    }
    else
    {
        m_pGatewayToInterface = 0;
    }

    if(p_rObject.m_pGatewayToDevice)
    {
        m_pGatewayToDevice = (CGatewayPStoDCS*)p_rObject.m_pGatewayToDevice->Clone();
    }
    else
    {
        m_pGatewayToDevice = 0;
    }

    m_pPlugInManager = 0;
    m_pReopenSemaphore = 0;
    m_pErrorHandling = 0;
    InitErrorHandling();
}

BOOL CProtocolStackBase::CreateReopenSemaphore(CStdString p_interfaceName, CStdString p_portName)
{
    const long NB_OF_ALLOWED_CLIENTS = 1;

    BOOL oResult = FALSE;
    CStdString name(_T(""));

    if(!m_pReopenSemaphore)
    {
        name.Format(_T("Process-0x%.8X_%s_%s"), MmcGetCurrentProcessId(), p_interfaceName, p_portName);

        m_pReopenSemaphore = new CMmcSemaphore(NB_OF_ALLOWED_CLIENTS, NB_OF_ALLOWED_CLIENTS, name);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::DeleteReopenSemaphore()
{
    BOOL oResult(FALSE);

    if(m_pReopenSemaphore)
    {
        delete m_pReopenSemaphore;
        m_pReopenSemaphore = 0;

        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::CreatePlugInManager()
{
    BOOL oResult(FALSE);

    DeletePlugInManager();
    if(!m_pPlugInManager)
    {
        m_pPlugInManager = new CInterfacePlugInManager();
        m_pPlugInManager->Init(this);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::DeletePlugInManager()
{
    BOOL oResult(FALSE);

    if(m_pPlugInManager)
    {
        delete m_pPlugInManager;
        m_pPlugInManager = 0;
        oResult = TRUE;
    }

    return oResult;
}

CProtocolStackBase* CProtocolStackBase::Clone()
{
    return 0;
}

BOOL CProtocolStackBase::IsEqual(CProtocolStackBase* p_pProtocolStack)
{
    BOOL oResult(FALSE);

    if(p_pProtocolStack)
    {
        oResult = TRUE;

        if(oResult && (m_strProtocolStackName != p_pProtocolStack->m_strProtocolStackName))
        {
            oResult = FALSE;
        }

        if(oResult && (m_lInstanceValue != p_pProtocolStack->m_lInstanceValue))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

CProtocolStackBase::~CProtocolStackBase()
{
    DeletePlugInManager();
    DeleteReopenSemaphore();
    DeleteErrorHandling();
    DeleteGatewayToInterface();
    DeleteGatewayToDevice();
    DeleteInterfaceManagerList();
    DeleteDeviceCommandSetManagerList();
}

BOOL CProtocolStackBase::InitInstanceValue(int p_lInstanceValue)
{
    BOOL oResult(TRUE);

    m_lInstanceValue = p_lInstanceValue;

    if(m_pPlugInManager)
    {
        m_pPlugInManager->InitInstanceValue(p_lInstanceValue);
    }

    return oResult;
}

BOOL CProtocolStackBase::InitErrorHandling()
{
    DeleteErrorHandling();

    m_pErrorHandling = new CErrorHandling();

    return TRUE;
}

BOOL CProtocolStackBase::DeleteErrorHandling()
{
    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = 0;
    }

    return TRUE;
}

BOOL CProtocolStackBase::ExecuteCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pLayerManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oTimeoutValid = FALSE;
    BOOL oResult(FALSE);
    CMmcSingleLock* pLock(0);

    if(p_pCommand)
    {
        //Lock
        DoLockReopen(pLock);

        //Init References
        if(m_pGatewayToInterface) p_pCommand->InitGateway(m_pGatewayToInterface);
        if(m_pGatewayToDevice) p_pCommand->InitGateway(m_pGatewayToDevice);
        p_pCommand->InitJournalManager(m_pJournalManager);

        //Timeout
        oTimeoutValid = p_pCommand->IsTimeoutValid();
        if(!oTimeoutValid) p_pCommand->SetTimeout(m_dTimeout);

        //Execute
        oResult = p_pCommand->Execute(p_pLayerManager, p_hHandle, p_hTransactionHandle);

        //Timeout
        if(!oTimeoutValid) p_pCommand->ResetTimeout();

        //Reset references
        p_pCommand->ResetGateway();
        p_pCommand->ResetJournalManager();

        //Unlock
        DoUnlockReopen(pLock);
    }

    return oResult;
}


void CProtocolStackBase::DoLockReopen(CMmcSingleLock*& p_rLock)
{
    if(m_pReopenSemaphore != 0)
    {
        p_rLock = new CMmcSingleLock(m_pReopenSemaphore, FALSE);
        p_rLock->Lock();
    }
}

void CProtocolStackBase::DoUnlockReopen(CMmcSingleLock*& p_rLock)
{
    if(p_rLock != 0)
    {
        p_rLock->Unlock();
        delete p_rLock;
        p_rLock = 0;
    }
}

BOOL CProtocolStackBase::AbortCommands(BOOL p_oActive)
{
    BOOL oResult(TRUE);

    if(m_pGatewayToInterface)
    {
        if(!m_pGatewayToInterface->AbortCommands(p_oActive))
        {
            oResult = FALSE;
        }
    }

    if(m_pGatewayToDevice)
    {
        if(!m_pGatewayToDevice->AbortCommands(p_oActive))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::GetCommands(CStdString* p_pCommandInfo)
{
    return FALSE;
}

BOOL CProtocolStackBase::InitProtocolStack(CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CProtocolStackBase::InitProtocolStack(CStdString p_InterfaceName, CErrorInfo* p_pErrorInfo)
{
    CInterfaceManagerBase* pManager = 0;
    BOOL oNewCreated = FALSE;
    BOOL oResult = FALSE;

    if(p_InterfaceName.IsEmpty())
    {
        //Don't create an interface manager
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_InterfaceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CInterfaceManagerBase*) new CInterfaceManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Klassenstruktur für Interface aufbauen
        if(VerifyGatewayToInterface(p_InterfaceName))
        {
            if(pManager->I_InitInterface(p_InterfaceName, p_pErrorInfo))
            {
                //Interface already existing
                oResult = TRUE;
            }
            else
            {
                //Interface not existing
                oResult = TRUE;
            }

            //Init PlugIn manager
            if(m_pPlugInManager)
            {
                m_pPlugInManager->RegisterInterface(p_InterfaceName);
            }

            if(oNewCreated) m_InterfaceManagerList.push_back(pManager);
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::InitProtocolStackEx(CStdString p_InterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    CInterfaceManagerBase* pManager = 0;
    BOOL oNewCreated = FALSE;
    BOOL oResult = FALSE;

    if(p_InterfaceName.IsEmpty())
    {
        //Don't create an interface manager
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_InterfaceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CInterfaceManagerBase*) new CInterfaceManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Klassenstruktur für Interface aufbauen
        if(VerifyGatewayToInterface(p_InterfaceName))
        {
            if(pManager->I_InitInterface(p_InterfaceName, p_DriverName, p_pErrorInfo))
            {
                //Interface already existing
                oResult = TRUE;
            }
            else
            {
                //Interface not existing
                oResult = TRUE;
            }

            //Init PlugIn manager
            if(m_pPlugInManager)
            {
                m_pPlugInManager->RegisterInterface(p_InterfaceName, p_DriverName);
            }

            if(oNewCreated)
            {
                m_InterfaceManagerList.push_back(pManager);
            }
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::InitProtocolStack(CStdString p_deviceName, CStdString p_protocolStackName, CErrorInfo* p_pErrorInfo)
{
    CDeviceCommandSetManagerBase* pManager = 0;
    BOOL oNewCreated = FALSE;
    BOOL oResult = FALSE;

    if(p_deviceName.IsEmpty())
    {
        //Don't create an device manager
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_deviceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CDeviceCommandSetManagerBase*) new CDeviceCommandSetManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Klassenstruktur für device aufbauen
        if((VerifyGatewayToDevice(p_deviceName)) && (pManager->DCS_InitDevice(p_deviceName, p_protocolStackName, _T(""), p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_DeviceCommandSetManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if(FindCorrectManager(p_strInterfaceName, &pManager) || GetFirstManager(&pManager))
    {
        //Update
        oResult = pManager->I_UpdateInterfaceNameSelection(p_strInterfaceName, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling)
        {
            m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_UpdateInterfaceNameSelection(CStdString p_strInterfaceName, CStdString p_DriverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if(FindCorrectManager(p_strInterfaceName, &pManager) || GetFirstManager(&pManager))
    {
        //Update
        oResult = pManager->I_UpdateInterfaceNameSelection(p_strInterfaceName, p_DriverName, p_pErrorInfo);
    }
    else
    {
        if(m_pErrorHandling)
        {
            m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_ResetPortEnumeration(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_strInterfaceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Enumeration
    if(oResult && !pManager->I_ResetPortEnumeration(p_strInterfaceName, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_UpdatePortNameSelection(CStdString p_strInterfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;
    CInterfaceManagerBase* pManager = 0;

    //Is manager already existing
    if(FindCorrectManager(p_strInterfaceName, &pManager) || GetFirstManager(&pManager))
    {
        if(VerifyGatewayToInterface(p_strInterfaceName))
        {
            if(pManager && pManager->I_UpdatePortNameSelection(p_strInterfaceName, p_pErrorInfo))
            {
                //Interface updated
                oResult = TRUE;
            }
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::AddPort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        pManager = (*it);
        if(pManager)
        {
            if(pManager->I_IsInterfaceNameSupported(p_InterfaceName))
            {
                oResult = TRUE;
                break;
            }
        }
    }

    if(oResult)
    {
        if(VerifyGatewayToInterface(p_InterfaceName))
        {
            if(pManager)
            {
                oResult = pManager->I_AddInterfacePort(p_InterfaceName, p_PortName, p_pErrorInfo);
            }
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::DeletePort(CStdString p_InterfaceName, CStdString p_PortName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        pManager = (*it);
        if(pManager)
        {
            if(pManager->I_IsInterfaceNameSupported(p_InterfaceName))
            {
                oResult = TRUE;
                break;
            }
        }
    }

    if(oResult)
    {
        if(VerifyGatewayToInterface(p_InterfaceName))
        {
            if(pManager)
            {
                oResult = pManager->I_DeleteInterfacePort(p_InterfaceName, p_PortName, p_pErrorInfo);
            }
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::InitGatewayToInterface(CStdString p_InterfaceName)
{
    return FALSE;
}

BOOL CProtocolStackBase::InitGatewayToDevice(CStdString p_deviceName)
{
    return FALSE;
}

BOOL CProtocolStackBase::InitBaudrateSelection(CStdString p_InterfaceName, CStdDWordArray& p_rulBaudrateSel)
{
    BOOL oResult(TRUE);

    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pInterfaceManager = (*it);
        if(pInterfaceManager)
        {
            if(!pInterfaceManager->I_InitBaudrateSelection(p_InterfaceName, p_rulBaudrateSel))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::InitDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
    BOOL oResult(TRUE);

    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pInterfaceManager = (*it);
        if(pInterfaceManager)
        {
            if(!pInterfaceManager->I_InitDefaultInterfaceSettings(p_InterfaceName, p_ulBaudrate, p_ulTimeout))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::AddManager(CInterfaceManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_InterfaceManagerList.push_back(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::RemoveManager(CInterfaceManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_InterfaceManagerList.remove(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::AddManager(CDeviceCommandSetManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_DeviceCommandSetManagerList.push_back(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::GetFirstManager(CInterfaceManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pListManager = (*it);
        if(pListManager)
        {
            if(p_ppManager)
            {
                *p_ppManager = pListManager;
            }

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::GetFirstManager(CDeviceCommandSetManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        CDeviceCommandSetManagerBase* pListManager = (*it);
        if(pListManager)
        {
            if(p_ppManager)
            {
                *p_ppManager = pListManager;
            }

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::FindCorrectManager(CStdString p_InterfaceName, CInterfaceManagerBase** p_ppManager)
{
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pListManager = (*it);
        if(pListManager)
        {
            if(pListManager->I_IsInterfaceNameSupported(p_InterfaceName))
            {
                if(p_ppManager)
                {
                    *p_ppManager = pListManager;
                }

                return TRUE;
            }
        }
    }

    return FindCorrectLibrary(p_InterfaceName, p_ppManager);
}

BOOL CProtocolStackBase::FindCorrectManager(CStdString p_deviceName, CDeviceCommandSetManagerBase** p_ppManager)
{
    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        CDeviceCommandSetManagerBase* pListManager = (*it);
        if(pListManager)
        {
            if(pListManager->DCS_IsDeviceNameSupported(p_deviceName))
            {
                if(p_ppManager)
                {
                    *p_ppManager = pListManager;
                }

                return TRUE;
            }
        }
    }

    return FindCorrectLibrary(p_deviceName, p_ppManager);
}

BOOL CProtocolStackBase::FindCorrectLibrary(CStdString p_InterfaceName, CInterfaceManagerBase** p_ppManager)
{
    if(p_ppManager)
    {
        *p_ppManager = 0;
    }

    return FALSE;
}

BOOL CProtocolStackBase::FindCorrectLibrary(CStdString p_deviceName, CDeviceCommandSetManagerBase** p_ppManager)
{
    if(p_ppManager)
    {
        *p_ppManager = 0;
    }

    return FALSE;
}

void CProtocolStackBase::DeleteGatewayToInterface()
{
    if(m_pGatewayToInterface)
    {
        delete m_pGatewayToInterface;
        m_pGatewayToInterface = 0;
    }
}

void CProtocolStackBase::DeleteGatewayToDevice()
{
    if(m_pGatewayToDevice)
    {
        delete m_pGatewayToDevice;
        m_pGatewayToDevice = 0;
    }
}

void CProtocolStackBase::DeleteInterfaceManagerList()
{
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pManager = (*it);
        delete pManager;
    }

    m_InterfaceManagerList.clear();
}

void CProtocolStackBase::DeleteDeviceCommandSetManagerList()
{
    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end();it++)
    {
        CDeviceCommandSetManagerBase* pManager = (*it);
        delete pManager;
    }

    m_DeviceCommandSetManagerList.clear();
}

BOOL CProtocolStackBase::PS_GetProtocolStackName(CStdString* p_pProtocolStackName)
{
    BOOL oResult(FALSE);

    if(p_pProtocolStackName)
    {
        *p_pProtocolStackName = m_strProtocolStackName;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_GetInterfaceNameSelection(CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CStdStringArray selArray;

    if(p_pInterfaceNameSel)
    {
        //Force PlugIn manager Update
        if(m_pPlugInManager)
        {
            //Update
            m_pPlugInManager->UpdateDevice();
        }

        //Enumerated update interfaces
        p_pInterfaceNameSel->clear();

        for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
        {
            CInterfaceManagerBase* pManager = (*it);
            if(pManager)
            {
                if(pManager->I_GetInterfaceNameSelection(&selArray))
                {
                    AppendSelectionArray(p_pInterfaceNameSel, &selArray);
                }
            }
        }

        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_GetDeviceNameSelection(CStdStringArray* p_pDeviceNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CStdStringArray selArray;

    if(p_pDeviceNameSel)
    {
        p_pDeviceNameSel->clear();

        for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
        {
            CDeviceCommandSetManagerBase* pManager = (*it);
            if(pManager)
            {
                if(pManager->DCS_GetDeviceNameSelection(&selArray))
                {
                    AppendSelectionArray(p_pDeviceNameSel, &selArray);
                }
            }
        }

        oResult = TRUE;
    }
    else
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_GetProtocolStackNameSelection(CStdString p_deviceName, CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_deviceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_DEVICE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get ProtocolStack Name Selection
    if(oResult && !pManager->DCS_GetProtocolStackNameSelection(p_deviceName, p_pProtocolStackNameSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray)
{
    BOOL oResult(FALSE);
    CStdString strDestName;
    CStdString strSourceName;
    BOOL oFound;

    if(p_pDestArray && p_pSourceArray)
    {
        for(size_t i = 0; i < p_pSourceArray->size(); i++)
        {
            oFound = FALSE;
            strSourceName = p_pSourceArray->at(i);
            for(size_t j = 0; j < p_pDestArray->size(); j++)
            {
                strDestName = p_pDestArray->at(j);

                if(strSourceName.CompareNoCase(strDestName) == 0)
                {
                    oFound = TRUE;
                    break;
                }
            }

            if(!oFound)
            {
                p_pDestArray->push_back(strSourceName);
            }
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CProtocolStackBase::VerifyGatewayToInterface(CStdString p_InterfaceName)
{
    return FALSE;
}

BOOL CProtocolStackBase::VerifyGatewayToDevice(CStdString p_deviceName)
{
    return FALSE;
}

BOOL CProtocolStackBase::PS_GetPortNameSelection(CStdString p_InterfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_InterfaceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //GetPortNameSelection
    if(oResult && !pManager->I_GetPortNameSelection(p_InterfaceName, p_pPortSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_GetBaudrateSelection(CStdString p_InterfaceName, CStdString p_PortName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_InterfaceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //GetBaudrateSelection
    if(oResult && !pManager->I_GetBaudrateSelection(p_InterfaceName, p_PortName, p_pulBaudrateSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_GetProtocolStackModeSelection(CStdStringArray* p_pProtocolStackModeSel, CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CProtocolStackBase::PS_GetProtocolStackMode(WORD* p_pusModeIndex, CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CProtocolStackBase::PS_SetProtocolStackMode(WORD p_usModeIndex, CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

void CProtocolStackBase::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    m_pJournalManager = p_pJournalManager;

    //Interface
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pInterface = (*it);
        pInterface->InitJournalManager(p_pJournalManager);
    }

    //Device
    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        CDeviceCommandSetManagerBase* pDevice = (*it);
        pDevice->InitJournalManager(p_pJournalManager);
    }
}

void CProtocolStackBase::ResetJournalManager()
{
    m_pJournalManager = 0;

    //Interface
    for(list<CInterfaceManagerBase*>::iterator it = m_InterfaceManagerList.begin(); it != m_InterfaceManagerList.end(); it++)
    {
        CInterfaceManagerBase* pInterface = (*it);
        pInterface->ResetJournalManager();
    }

    //Device
    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        CDeviceCommandSetManagerBase* pDevice = (*it);
        pDevice->ResetJournalManager();
    }
}

BOOL CProtocolStackBase::PS_GetDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD* p_pulBaudrate, DWORD* p_pulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_InterfaceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //GetDefaultInterfaceSettings
    if(oResult && !pManager->I_GetDefaultInterfaceSettings(p_InterfaceName, p_pulBaudrate, p_pulTimeout, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::PS_SetDefaultProtocolStackSettings(CStdString p_InterfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CInterfaceManagerBase* pManager = 0;

    //Find manager
    if((oResult && (!FindCorrectManager(p_InterfaceName, &pManager))) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_INTERFACE_NAME, p_pErrorInfo);
        oResult = FALSE;
    }

    //SetDefaultInterfaceSettings
    if(oResult && !pManager->I_SetDefaultInterfaceSettings(p_InterfaceName, p_ulBaudrate, p_ulTimeout, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CProtocolStackBase::UpdateProtocolStackSettings(CInterfaceManagerBase* p_pInterfaceManager, HANDLE p_hI_Handle)
{
    BOOL oResult(FALSE);

    if(p_pInterfaceManager)
    {
        oResult = p_pInterfaceManager->I_GetInterfaceSettings(p_hI_Handle, &m_ulBaudrate, &m_dTimeout);
    }

    return oResult;
}

BOOL CProtocolStackBase::UpdateProtocolStackSettings(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManagerManager, HANDLE p_hDCS_Handle)
{
    BOOL oResult(FALSE);

    if(p_pDeviceCommandSetManagerManager)
    {
        oResult = p_pDeviceCommandSetManagerManager->DCS_GetProtocolStackSettings(p_hDCS_Handle, &m_ulBaudrate, &m_dTimeout);
    }

    return oResult;
}

BOOL CProtocolStackBase::SetTimeout(DWORD p_ulTimeout)
{
    m_dTimeout = p_ulTimeout;

    return TRUE;
}

BOOL CProtocolStackBase::SetDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    //Device timeout
    if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->SetDeviceTimeout(p_pManager, p_hHandle, p_ulTimeout);
    }

    return oResult;
}

DWORD CProtocolStackBase::GetTimeout()
{
    return m_dTimeout;
}

DWORD CProtocolStackBase::GetBaudrate()
{
    return m_ulBaudrate;
}

DWORD CProtocolStackBase::GetEvaluatedTimeout(DWORD p_ulCommandTimeout)
{
    DWORD ulTimeout(0);

    //Evaluated if commandTimeout or ProtocolStack Timeout is valid
    if(p_ulCommandTimeout == 0)
    {
        ulTimeout = m_dTimeout;
    }
    else
    {
        ulTimeout = p_ulCommandTimeout;
    }

    return ulTimeout;
}

BOOL CProtocolStackBase::AreParameterEqual(CProtocolStackBase* p_pProtocolStack)
{
    BOOL oResult(FALSE);

    if(p_pProtocolStack)
    {
        if(m_pGatewayToInterface && p_pProtocolStack->m_pGatewayToInterface)
        {
            oResult = m_pGatewayToInterface->AreParameterEqual(p_pProtocolStack->m_pGatewayToInterface);
        }
        else if(m_pGatewayToDevice && p_pProtocolStack->m_pGatewayToDevice)
        {
            oResult = m_pGatewayToDevice->AreParameterEqual(p_pProtocolStack->m_pGatewayToDevice);
        }
    }

    return oResult;
}

BOOL CProtocolStackBase::SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->SetParameter(p_Name, p_pValue, p_ulSize);
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->SetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CProtocolStackBase::SetParameter(CStdString p_Name, CStdString p_Value)
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->SetParameter(p_Name, p_Value);
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->SetParameter(p_Name, p_Value);
    }

    return oResult;
}

BOOL CProtocolStackBase::GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->GetParameter(p_Name, p_pValue, p_ulSize);
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->GetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CProtocolStackBase::GetParameter(CStdString p_Name, CStdString& p_rValue)
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->GetParameter(p_Name, p_rValue);
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->GetParameter(p_Name, p_rValue);
    }

    return oResult;
}

BOOL CProtocolStackBase::Lock(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->Lock(p_ulTimeout);
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->Lock(p_ulTimeout);
    }

    return oResult;
}

BOOL CProtocolStackBase::Unlock()
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->Unlock();
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->Unlock();
    }

    return oResult;
}

BOOL CProtocolStackBase::IsLocked()
{
    BOOL oResult(FALSE);

    if(m_pGatewayToInterface)
    {
        oResult = m_pGatewayToInterface->IsLocked();
    }
    else if(m_pGatewayToDevice)
    {
        oResult = m_pGatewayToDevice->IsLocked();
    }

    return oResult;
}

BOOL CProtocolStackBase::WriteToLogFile(CStdString p_Action)
{
    BOOL oResult(FALSE);

    if(m_pPlugInManager)
    {
        oResult = m_pPlugInManager->WriteToLogFile(p_Action);
    }

    return oResult;
}
