#include "stdafx.h"

#include <Thread/MmcSingleLock.h>
#include <Thread/MmcSemaphore.h>

#include <Log/MmcLogger.h>
#include <Drv/Helper/UsbTree/UsbTree.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/BaseClasses/ProtocolStackBase.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInThread.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInManager.h>

using namespace std;

CInterfacePlugInManager::CInterfacePlugInManager(void):
    m_pProtocolStack(0),
    m_pPlugInThread(0),
    m_ulNotificationDelay(100),
    m_lInstanceValue(0),
    m_pUpdateSemaphore(0)
{
    CreateUpdateSemaphore();
    CreatePlugInThread();
}

CInterfacePlugInManager::~CInterfacePlugInManager(void)
{
    StopPlugInMonitoring();
    DeletePlugInThread();
    DeleteUpdateSemaphore();
}

BOOL CInterfacePlugInManager::CreatePlugInThread()
{
    BOOL oResult = FALSE;

    DeletePlugInThread();
    if(!m_pPlugInThread)
    {
        m_pPlugInThread = new CInterfacePlugInThread();
        m_pPlugInThread->Init(this);

        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::DeletePlugInThread()
{
    BOOL oResult = FALSE;

    if(m_pPlugInThread)
    {
        StopPlugInMonitoring();

        delete m_pPlugInThread;
        m_pPlugInThread = NULL;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::Init(CProtocolStackBase* p_pProtocolStack)
{
    BOOL oResult = TRUE;

    m_pProtocolStack = p_pProtocolStack;

    return oResult;
}

BOOL CInterfacePlugInManager::InitInstanceValue(int p_lInstanceValue)
{
    m_lInstanceValue = p_lInstanceValue;

    return TRUE;
}

BOOL CInterfacePlugInManager::RegisterInterface(CStdString p_InterfaceName)
{
    BOOL oResult = FALSE;
    int lIndex = 0;

    if(FindRegisteredInterface(p_InterfaceName, lIndex))
    {
        //Already registered
        oResult = TRUE;
    }
    else
    {
        SPluginManagerInterface pluginInterface;

        pluginInterface.m_InterfaceName = p_InterfaceName;

        //Add Interface
        m_RegisteredInterfaces.push_back(pluginInterface);

        //Start PlugInThread if first Interface
        oResult = CheckPlugInThread();
    }

    return oResult;
}

BOOL CInterfacePlugInManager::ExtractDriverInfos(CStdString p_DriverName, CStdString& p_rDriverName, CStdString& p_rDriverParameters)
{
    BOOL oResult(FALSE);
    int lIndex(0);

    if(p_DriverName.size() != 0)
    {
        //Expected Format: "DriverName DriverParameters"
        p_DriverName = p_DriverName.Trim();
        lIndex = p_DriverName.Find(_T(" "));
        if(lIndex != -1)
        {
            p_rDriverName = p_DriverName.Left(lIndex);
            p_rDriverParameters = p_DriverName.Right((int)p_DriverName.length() - lIndex -1);
            oResult = TRUE;
        }
        else
        {
            p_rDriverName = p_DriverName;
            p_rDriverParameters = _T("");
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::ExtractVendorProductIDs(CStdString& p_rDriverParameters, WORD& p_rVendorId, WORD& p_rProductId)
{
    BOOL oResult(FALSE);
    int lStartIndex(0);
    int lEndIndex(0);
    CStdString vendorProductIDs(_T(""));

    //Init
    p_rDriverParameters = p_rDriverParameters.Trim();
    lStartIndex = p_rDriverParameters.Find(_T("VID_"));
    if(lStartIndex != -1)
    {
        lEndIndex = p_rDriverParameters.Find(_T(" "));
        if(lEndIndex != -1)
        {
            //Parameters following
            vendorProductIDs = p_rDriverParameters.Mid(lStartIndex, lEndIndex - lStartIndex);
            p_rDriverParameters = (p_rDriverParameters.Left(lStartIndex) + p_rDriverParameters.Right((int)p_rDriverParameters.length() - lEndIndex));
        }
        else
        {
            //Last Ids
            vendorProductIDs = p_rDriverParameters.Right((int)p_rDriverParameters.length() - lStartIndex);
            p_rDriverParameters = p_rDriverParameters.Left(lStartIndex);
        }

        //Extract Values
        oResult = TRUE;
        if(oResult && !ExtractVendorIdValue(vendorProductIDs, p_rVendorId)) oResult = FALSE;
        if(oResult && !ExtractProductIdValue(vendorProductIDs, p_rProductId)) oResult = FALSE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::ExtractVendorIdValue(CStdString p_VendorProductIDs, WORD& p_rVendorId)
{
    const CStdString VID(_T("VID_"));
    const CStdString SEPERATOR(_T("&"));

    BOOL oResult(FALSE);
    int lStartIndex(0);
    int lEndIndex(0);
    CStdString id(_T(""));

    //Init (Format: VID_XXXX&PID_YYYY)
    p_VendorProductIDs = p_VendorProductIDs.Trim();

    //VendorId
    lStartIndex = p_VendorProductIDs.Find(VID);
    lEndIndex = p_VendorProductIDs.Find(SEPERATOR);
    if((lStartIndex != -1) && (lEndIndex != -1))
    {
        id = p_VendorProductIDs.Mid(lStartIndex + (int)VID.length(), lEndIndex - lStartIndex - (int)VID.length());
        p_rVendorId = (WORD)_tcstol(id, 0, 16);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::ExtractProductIdValue(CStdString p_VendorProductIDs, WORD& p_rProductId)
{
    const CStdString PID(_T("PID_"));

    BOOL oResult(FALSE);
    int lStartIndex(0);
    CStdString id(_T(""));

    //Init (Format: VID_XXXX&PID_YYYY)
    p_VendorProductIDs = p_VendorProductIDs.Trim();

    //VendorId
    lStartIndex = p_VendorProductIDs.Find(PID);
    if(lStartIndex != -1)
    {
        id = p_VendorProductIDs.Mid(lStartIndex + (int)PID.length(), (int)p_VendorProductIDs.length() - lStartIndex - (int)PID.length());
        p_rProductId = (WORD)_tcstol(id, 0, 16);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::RegisterInterface(CStdString p_InterfaceName, CStdString p_DriverName)
{
    BOOL oResult = FALSE;
    int lIndex = 0;

    if(FindRegisteredInterface(p_InterfaceName, lIndex))
    {
        //Already registered
        oResult = TRUE;
    }
    else
    {
        SPluginManagerInterface pluginInterface;
        CStdString driverName;
        CStdString driverParameters;

        pluginInterface.m_InterfaceName = p_InterfaceName;

        if(ExtractDriverInfos(p_DriverName, driverName, driverParameters))
        {
            pluginInterface.m_DriverName = driverName;
            pluginInterface.m_DriverParameters = driverParameters;
        }

        //Add Interface
        m_RegisteredInterfaces.push_back(pluginInterface);

        //Start PlugInThread if first Interface
        oResult = CheckPlugInThread();
    }

    return oResult;
}

BOOL CInterfacePlugInManager::UnregisterInterface(CStdString p_InterfaceName)
{
    BOOL oResult = FALSE;

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        SPluginManagerInterface pluginInterface = (*it);

        if(p_InterfaceName == pluginInterface.m_InterfaceName)
        {
            m_RegisteredInterfaces.erase(it);
            oResult = TRUE;
            break;
        }
    }

    if(oResult)
    {
        oResult = CheckPlugInThread();
    }

    return oResult;
}

BOOL CInterfacePlugInManager::IsDriverSupported(WORD p_usVendorId, WORD p_usProductId)
{
    BOOL oResult = FALSE;
    WORD usVendorId = 0;
    WORD usProductId = 0;

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        SPluginManagerInterface pluginInterface = (*it);

        if(ExtractVendorProductIDs(pluginInterface.m_DriverParameters, usVendorId, usProductId))
        {
            if(usVendorId == p_usVendorId && usProductId == p_usProductId)
            {
                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::FindRegisteredInterface(CStdString p_InterfaceName, int& p_rlIndex)
{
    BOOL oResult = FALSE;
    int lIndex = 0;

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        SPluginManagerInterface pluginInterface = (*it);

        if(p_InterfaceName == pluginInterface.m_InterfaceName)
        {
            p_rlIndex = (int)lIndex;
            oResult = TRUE;
            break;
        }
        lIndex++;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::StartPlugInMonitoring()
{
    BOOL oResult = FALSE;

    if(m_pPlugInThread)
    {
        if(!m_pPlugInThread->IsActive())
        {
            //Start
            oResult = m_pPlugInThread->StartThread();
        }
        else
        {
            //Already running
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::StopPlugInMonitoring()
{
    BOOL oResult = FALSE;

    if(m_pPlugInThread)
    {
        if(m_pPlugInThread->IsActive())
        {
            //Stop
            oResult = m_pPlugInThread->StopThread();
        }
        else
        {
            //Not running
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::CheckPlugInThread()
{
    BOOL oResult = FALSE;

    if(m_pPlugInThread)
    {
        if(m_RegisteredInterfaces.size() > 0)
        {
            //Start monitoring
            oResult = StartPlugInMonitoring();
        }
        else
        {
            //Stop monitoring
            oResult = StopPlugInMonitoring();
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::IsDriverSupported(CStdString p_DriverName)
{
    BOOL oResult = FALSE;

    CStdString driverName = _T("");

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        SPluginManagerInterface pluginInterface = (*it);

        if(pluginInterface.m_DriverName == p_DriverName)
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::IsDriverSupported(CStdString p_DriverName, CStdString p_DriverParameters)
{
    BOOL oResult = FALSE;

    CStdString driverName = _T("");

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        SPluginManagerInterface pluginInterface = (*it);

        if(pluginInterface.m_DriverName == p_DriverName && pluginInterface.m_DriverParameters == p_DriverParameters)
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::IsUsbInterfaceRegistered()
{
    BOOL oResult = FALSE;
    CStdString interfaceName = _T("");

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        interfaceName = (*it).m_InterfaceName;

        if(interfaceName == INTERFACE_USB)
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}

BOOL CInterfacePlugInManager::UpdateDevice()
{
    CStdString interfaceName = _T("");
    BOOL oResult = TRUE;

    ResetUsbEnumerationTree();

    for(list<SPluginManagerInterface>::iterator it = m_RegisteredInterfaces.begin(); it != m_RegisteredInterfaces.end(); it++)
    {
        interfaceName = (*it).m_InterfaceName;

        oResult = UpdateDevice(interfaceName);
    }

    return oResult;
}

CMmcSemaphore* CInterfacePlugInManager::GetUpdateSemaphore()
{
    return m_pUpdateSemaphore;
}

BOOL CInterfacePlugInManager::WriteToLogFile(CStdString p_Action)
{
    return CMmcLogger::Commit(m_lInstanceValue, p_Action);
}

BOOL CInterfacePlugInManager::UpdateDevice(CStdString p_InterfaceName)
{
    BOOL oResult = FALSE;

    if(m_pProtocolStack)
    {
        oResult = m_pProtocolStack->PS_UpdatePortNameSelection(p_InterfaceName);
    }

    return oResult;
}

BOOL CInterfacePlugInManager::SetNotificationDelay(DWORD p_ulNotificationDelay)
{
    BOOL oResult = FALSE;

    m_ulNotificationDelay = p_ulNotificationDelay;

    return oResult;
}

BOOL CInterfacePlugInManager::GetNotificationDelay(DWORD& p_rulNotificationDelay)
{
    BOOL oResult = FALSE;

    p_rulNotificationDelay = m_ulNotificationDelay;

    return oResult;
}

void CInterfacePlugInManager::ResetUsbEnumerationTree()
{
    CUsbTree* pUsbTree = CUsbTree::GetInstance();

    if(pUsbTree != 0)
    {
        // Force reloading of UsbTree. Without explicit reset the inhibit time is active
        pUsbTree->Reset();
    }
}

BOOL CInterfacePlugInManager::CreateUpdateSemaphore()
{
    const long NB_OF_ALLOWED_CLIENTS = 1;

    BOOL oResult = FALSE;
    CStdString name = _T("Maxon-InterfacePlugInManager-Update");

    if (!m_pUpdateSemaphore)
    {
        m_pUpdateSemaphore = new CMmcSemaphore(NB_OF_ALLOWED_CLIENTS, NB_OF_ALLOWED_CLIENTS, name);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInManager::DeleteUpdateSemaphore()
{
    BOOL oResult(FALSE);

    if (m_pUpdateSemaphore)
    {
        delete m_pUpdateSemaphore;
        m_pUpdateSemaphore = 0;

        oResult = TRUE;
    }

    return oResult;
}