#include "stdafx.h"

#include <Process/MmcProcess.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInManager.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInMonitor.h>

#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInThread.h>

CInterfacePlugInThread::CInterfacePlugInThread()
{
    m_pPlugInManager = NULL;
    m_pPlugInMonitor = NULL;
}

CInterfacePlugInThread::~CInterfacePlugInThread()
{
    DeletePlugInMonitor();
}

BOOL CInterfacePlugInThread::Init(CInterfacePlugInManager* p_pPlugInManager)
{
    BOOL oResult = FALSE;

    m_pPlugInManager = p_pPlugInManager;

    return oResult;
}

BOOL CInterfacePlugInThread::CreatePlugInMonitor()
{
    BOOL oResult = FALSE;

    DeletePlugInMonitor();

    if(!m_pPlugInMonitor)
    {
        m_pPlugInMonitor = new CInterfacePlugInMonitor();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInThread::DeletePlugInMonitor()
{
    BOOL oResult = FALSE;

    if(m_pPlugInMonitor)
    {
        delete m_pPlugInMonitor;
        m_pPlugInMonitor = 0;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CInterfacePlugInThread::WaitForPlugInMonitorCreation(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    DWORD ulTime(0);
    DWORD ulStart(MmcGetTickCount());

    while(ulTime < p_ulTimeout)
    {
        if(m_pPlugInMonitor)
        {
            if(m_pPlugInMonitor->IsReady())
            {
                oResult = TRUE;
                break;
            }
        }

        ulTime = MmcGetTickCount() - ulStart;
        MmcSleep(10);
    }

    return oResult;
}

BOOL CInterfacePlugInThread::WaitForPlugInMonitorDeletion(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);
    DWORD ulTime(0);
    DWORD ulStart(MmcGetTickCount());

    while(ulTime < p_ulTimeout)
    {
        if(m_pPlugInMonitor == 0)
        {
            oResult = TRUE;
            break;
        }

        ulTime = MmcGetTickCount() - ulStart;
        MmcSleep(10);
    }

    return oResult;
}

BOOL CInterfacePlugInThread::StartThread()
{
    const DWORD TIMEOUT = 100;

    BOOL oResult(FALSE);

    if(CMmcThread::Start())
    {
        oResult = WaitForPlugInMonitorCreation(TIMEOUT);
    }

    return oResult;
}

BOOL CInterfacePlugInThread::StopThread()
{
    const DWORD TIMEOUT = 250;

    BOOL oResult(FALSE);

    if(CMmcThread::Stop())
    {
        oResult = WaitForPlugInMonitorDeletion(TIMEOUT);
    }

    return oResult;
}

DWORD CInterfacePlugInThread::ThreadFunc()
{
    const unsigned int WAIT_TIMOUT = 100;

    SetThreadState(TBS_STARTED);

    CreatePlugInMonitor();

    if(m_pPlugInMonitor)
    {
        m_pPlugInMonitor->Init(m_pPlugInManager);

        //Process Messages
        while(!IsStopping())
        {
            m_pPlugInMonitor->UpdateHotPlugIn();
            
            if(!IsStopping()) MmcSleep(WAIT_TIMOUT);

            //Check if Notification is pending
            m_pPlugInMonitor->CheckNotification();
        }
    }

    MmcSleep(WAIT_TIMOUT);
    
    DeletePlugInMonitor();

    SetThreadState(TBS_STOPPED);

    return 0;
}
