
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_CanOpen.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/CanOpen/CommandSetCompleteAccess_VCS_CanOpen.h>

CCommandSetCompleteAccess_VCS_CanOpen::CCommandSetCompleteAccess_VCS_CanOpen()
{
    m_strCommandSetName = COMMAND_SET_CANOPEN_COMPLETE_ACCESS;

    m_pCommand_GetCompleteObject = NULL;

    InitCommands();
}

CCommandSetCompleteAccess_VCS_CanOpen::~CCommandSetCompleteAccess_VCS_CanOpen()
{
    DeleteCommands();
}

BOOL CCommandSetCompleteAccess_VCS_CanOpen::VCS_GetCompleteObject(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usObjectIndex, void* p_pData, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetCompleteObject)
    {
        //Set parameter data
        m_pCommand_GetCompleteObject->ResetStatus();
        m_pCommand_GetCompleteObject->SetParameterData(0, &p_usObjectIndex, sizeof(p_usObjectIndex));
        m_pCommand_GetCompleteObject->SetParameterData(1, &p_ulNbOfBytesToRead, sizeof(p_ulNbOfBytesToRead));

        //Execute command
        oResult = m_pCommand_GetCompleteObject->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetCompleteObject->GetReturnParameterData(0, p_pData, p_ulNbOfBytesToRead);
        m_pCommand_GetCompleteObject->GetReturnParameterData(1, p_pulNbOfBytesRead, sizeof(p_pulNbOfBytesRead));

        //Get error code
        m_pCommand_GetCompleteObject->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetCompleteAccess_VCS_CanOpen::InitCommands()
{
    DeleteCommands();

    m_pCommand_GetCompleteObject = new CCommand_VCS_CanOpen();
    m_pCommand_GetCompleteObject->InitCommand(VCS_CANOPEN_GET_COMPLETE_OBJECT);
}

void CCommandSetCompleteAccess_VCS_CanOpen::DeleteCommands()
{
    if(m_pCommand_GetCompleteObject)
    {
        delete m_pCommand_GetCompleteObject;
        m_pCommand_GetCompleteObject = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetCompleteAccess_VCS_CanOpen::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetCompleteObject) m_pCommand_GetCompleteObject->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetCompleteAccess_VCS_CanOpen::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetCompleteObject) m_pCommand_GetCompleteObject->InitJournalManager(p_pJournalManager);
}

void CCommandSetCompleteAccess_VCS_CanOpen::ResetJournalManager()
{
    if(m_pCommand_GetCompleteObject) m_pCommand_GetCompleteObject->ResetJournalManager();
}

BOOL CCommandSetCompleteAccess_VCS_CanOpen::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetCompleteObject && !m_pCommand_GetCompleteObject->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
