
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Configuration.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Configuration/CommandSetControllerGain_VCS_Configuration.h>

CCommandSetControllerGain_VCS_Configuration::CCommandSetControllerGain_VCS_Configuration()
{
    m_strCommandSetName = COMMAND_SET_CONFIGURATION_REGULATION;

    m_pCommand_GetControllerGain = 0;
    m_pCommand_SetControllerGain = 0;

    InitCommands();
}

CCommandSetControllerGain_VCS_Configuration::~CCommandSetControllerGain_VCS_Configuration()
{
    DeleteCommands();
}

BOOL CCommandSetControllerGain_VCS_Configuration::VCS_SetControllerGain(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usController, WORD p_usGain, __uint64 p_ullValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetControllerGain)
    {
        //Set parameter data
        m_pCommand_SetControllerGain->ResetStatus();
        m_pCommand_SetControllerGain->SetParameterData(0, &p_usController, sizeof(p_usController));
        m_pCommand_SetControllerGain->SetParameterData(1, &p_usGain, sizeof(p_usGain));
        m_pCommand_SetControllerGain->SetParameterData(2, &p_ullValue, sizeof(p_ullValue));

        //Execute command
        oResult = m_pCommand_SetControllerGain->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetControllerGain->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetControllerGain_VCS_Configuration::VCS_GetControllerGain(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usController, WORD p_usGain, __uint64* p_pullValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetControllerGain)
    {
        //Set parameter data
        m_pCommand_GetControllerGain->ResetStatus();
        m_pCommand_SetControllerGain->SetParameterData(0, &p_usController, sizeof(p_usController));
        m_pCommand_SetControllerGain->SetParameterData(1, &p_usGain, sizeof(p_usGain));

        //Execute command
        oResult = m_pCommand_GetControllerGain->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetControllerGain->GetReturnParameterData(0, p_pullValue, sizeof(*p_pullValue));

        //Get error code
        m_pCommand_GetControllerGain->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetControllerGain_VCS_Configuration::InitCommands()
{
    DeleteCommands();

    m_pCommand_GetControllerGain = new CCommand_VCS_Configuration();
    m_pCommand_GetControllerGain->InitCommand(CONF_GET_CONTROLLER_GAIN);

    m_pCommand_SetControllerGain = new CCommand_VCS_Configuration();
    m_pCommand_SetControllerGain->InitCommand(CONF_SET_CONTROLLER_GAIN);
}

void CCommandSetControllerGain_VCS_Configuration::DeleteCommands()
{
    if(m_pCommand_GetControllerGain)
    {
        delete m_pCommand_GetControllerGain;
        m_pCommand_GetControllerGain = NULL;
    }

    if(m_pCommand_SetControllerGain)
    {
        delete m_pCommand_SetControllerGain;
        m_pCommand_SetControllerGain = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetControllerGain_VCS_Configuration::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetControllerGain) m_pCommand_GetControllerGain->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetControllerGain) m_pCommand_SetControllerGain->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetControllerGain_VCS_Configuration::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetControllerGain) m_pCommand_GetControllerGain->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetControllerGain) m_pCommand_SetControllerGain->InitJournalManager(p_pJournalManager);
}

void CCommandSetControllerGain_VCS_Configuration::ResetJournalManager()
{
    if(m_pCommand_GetControllerGain) m_pCommand_GetControllerGain->ResetJournalManager();
    if(m_pCommand_SetControllerGain) m_pCommand_SetControllerGain->ResetJournalManager();
}

BOOL CCommandSetControllerGain_VCS_Configuration::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetControllerGain && !m_pCommand_GetControllerGain->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetControllerGain && !m_pCommand_SetControllerGain->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
