
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Configuration.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Configuration/CommandSetUnits_VCS_Configuration.h>

CCommandSetUnits_VCS_Configuration::CCommandSetUnits_VCS_Configuration()
{
    m_strCommandSetName = COMMAND_SET_CONFIGURATION_UNITS;

    m_pCommand_GetVelocityUnits = 0;
    m_pCommand_SetVelocityUnits = 0;

    InitCommands();
}

CCommandSetUnits_VCS_Configuration::~CCommandSetUnits_VCS_Configuration()
{
    DeleteCommands();
}

BOOL CCommandSetUnits_VCS_Configuration::VCS_GetVelocityUnits(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE* p_pubVelocityDimension, char* p_pbVelocityNotation, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetVelocityUnits)
    {
        //Set parameter data
        m_pCommand_GetVelocityUnits->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetVelocityUnits->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetVelocityUnits->GetReturnParameterData(0, p_pubVelocityDimension, sizeof(*p_pubVelocityDimension));
        m_pCommand_GetVelocityUnits->GetReturnParameterData(1, p_pbVelocityNotation, sizeof(*p_pbVelocityNotation));

        //Get error code
        m_pCommand_GetVelocityUnits->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetUnits_VCS_Configuration::VCS_SetVelocityUnits(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubVelocityDimension, char p_bVelocityNotation, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetVelocityUnits)
    {
        //Set parameter data
        m_pCommand_SetVelocityUnits->ResetStatus();
        m_pCommand_SetVelocityUnits->SetParameterData(0, &p_ubVelocityDimension, sizeof(p_ubVelocityDimension));
        m_pCommand_SetVelocityUnits->SetParameterData(1, &p_bVelocityNotation, sizeof(p_bVelocityNotation));

        //Execute command
        oResult = m_pCommand_SetVelocityUnits->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetVelocityUnits->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetUnits_VCS_Configuration::InitCommands()
{
    DeleteCommands();

    //Init GetVelocityUnits
    m_pCommand_GetVelocityUnits = new CCommand_VCS_Configuration();
    m_pCommand_GetVelocityUnits->InitCommand(CONF_GET_VELOCITY_UNITS);
    //Init SetVelocityUnits
    m_pCommand_SetVelocityUnits = new CCommand_VCS_Configuration();
    m_pCommand_SetVelocityUnits->InitCommand(CONF_SET_VELOCITY_UNITS);
}

void CCommandSetUnits_VCS_Configuration::DeleteCommands()
{
    if(m_pCommand_GetVelocityUnits)
    {
        delete m_pCommand_GetVelocityUnits;
        m_pCommand_GetVelocityUnits = NULL;
    }
    if(m_pCommand_SetVelocityUnits)
    {
        delete m_pCommand_SetVelocityUnits;
        m_pCommand_SetVelocityUnits = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetUnits_VCS_Configuration::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetVelocityUnits) m_pCommand_GetVelocityUnits->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetVelocityUnits) m_pCommand_SetVelocityUnits->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetUnits_VCS_Configuration::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetVelocityUnits) m_pCommand_GetVelocityUnits->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetVelocityUnits) m_pCommand_GetVelocityUnits->InitJournalManager(p_pJournalManager);
}

void CCommandSetUnits_VCS_Configuration::ResetJournalManager()
{
    if(m_pCommand_GetVelocityUnits) m_pCommand_GetVelocityUnits->ResetJournalManager();
    if(m_pCommand_SetVelocityUnits) m_pCommand_SetVelocityUnits->ResetJournalManager();
}

BOOL CCommandSetUnits_VCS_Configuration::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetVelocityUnits && !m_pCommand_GetVelocityUnits->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetVelocityUnits && !m_pCommand_SetVelocityUnits->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
