
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Device.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Device/CommandSetInfos_VCS_Device.h>

CCommandSetInfos_VCS_Device::CCommandSetInfos_VCS_Device()
{
    m_strCommandSetName = COMMAND_SET_DEVICE_INFOS;

    m_pCommand_GetVersion = NULL;
    m_pCommand_GetDeviceType = NULL;

    InitCommands();
}

CCommandSetInfos_VCS_Device::~CCommandSetInfos_VCS_Device()
{
    DeleteCommands();
}

BOOL CCommandSetInfos_VCS_Device::VCS_GetVersion(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD* p_pulHardwareVersion, WORD* p_pulSoftwareVersion, WORD* p_pulApplicationNumber, WORD* p_pulApplicationVersion, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetVersion)
    {
        //Set parameter data
        m_pCommand_GetVersion->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetVersion->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetVersion->GetReturnParameterData(0, p_pulHardwareVersion, sizeof(*p_pulHardwareVersion));
        m_pCommand_GetVersion->GetReturnParameterData(1, p_pulSoftwareVersion, sizeof(*p_pulSoftwareVersion));
        m_pCommand_GetVersion->GetReturnParameterData(2, p_pulApplicationNumber, sizeof(*p_pulApplicationNumber));
        m_pCommand_GetVersion->GetReturnParameterData(3, p_pulApplicationVersion, sizeof(*p_pulApplicationVersion));

        //Get error code
        m_pCommand_GetVersion->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetInfos_VCS_Device::VCS_GetDeviceType(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD* p_pulDeviceType, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetDeviceType)
    {
        //Set parameter data
        m_pCommand_GetDeviceType->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetDeviceType->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetDeviceType->GetReturnParameterData(0, p_pulDeviceType, sizeof(*p_pulDeviceType));

        //Get error code
        m_pCommand_GetDeviceType->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetInfos_VCS_Device::InitCommands()
{
    DeleteCommands();

    //Init GetVersion
    m_pCommand_GetVersion = new CCommand_VCS_Device();
    m_pCommand_GetVersion->InitCommand(DEVICE_GET_VERSION);
    //Init GetDeviceType
    m_pCommand_GetDeviceType = new CCommand_VCS_Device();
    m_pCommand_GetDeviceType->InitCommand(DEVICE_GET_TYPE);
}

void CCommandSetInfos_VCS_Device::DeleteCommands()
{
    if(m_pCommand_GetVersion)
    {
        delete m_pCommand_GetVersion;
        m_pCommand_GetVersion = NULL;
    }

    if(m_pCommand_GetDeviceType)
    {
        delete m_pCommand_GetDeviceType;
        m_pCommand_GetDeviceType = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetInfos_VCS_Device::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetVersion) m_pCommand_GetVersion->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetDeviceType) m_pCommand_GetDeviceType->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetInfos_VCS_Device::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetVersion) m_pCommand_GetVersion->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetDeviceType) m_pCommand_GetDeviceType->InitJournalManager(p_pJournalManager);
}

void CCommandSetInfos_VCS_Device::ResetJournalManager()
{
    if(m_pCommand_GetVersion) m_pCommand_GetVersion->ResetJournalManager();
    if(m_pCommand_GetDeviceType) m_pCommand_GetDeviceType->ResetJournalManager();
}

BOOL CCommandSetInfos_VCS_Device::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetVersion && !m_pCommand_GetVersion->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetDeviceType && !m_pCommand_GetDeviceType->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
