
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_ErrorHandling.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/ErrorHandling/CommandSetErrorHistory_VCS_ErrorHandling.h>

CCommandSetErrorHistory_VCS_ErrorHandling::CCommandSetErrorHistory_VCS_ErrorHandling()
{
    m_strCommandSetName = COMMAND_SET_ERROR_HANDLING_ERROR_HISTORY;

    m_pCommand_ClearDeviceErrors = NULL;
    m_pCommand_GetNbOfDeviceErrors = NULL;
    m_pCommand_GetDeviceErrorCode = NULL;

    InitCommands();
}

CCommandSetErrorHistory_VCS_ErrorHandling::~CCommandSetErrorHistory_VCS_ErrorHandling()
{
    DeleteCommands();
}

BOOL CCommandSetErrorHistory_VCS_ErrorHandling::VCS_ClearDeviceErrors(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ClearDeviceErrors)
    {
        //Set parameter data
        m_pCommand_ClearDeviceErrors->ResetStatus();

        //Execute command
        oResult = m_pCommand_ClearDeviceErrors->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ClearDeviceErrors->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetErrorHistory_VCS_ErrorHandling::VCS_GetNbOfDeviceErrors(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE* p_pubNbOfDeviceErrors, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetNbOfDeviceErrors)
    {
        //Set parameter data
        m_pCommand_GetNbOfDeviceErrors->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetNbOfDeviceErrors->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetNbOfDeviceErrors->GetReturnParameterData(0, p_pubNbOfDeviceErrors, sizeof(*p_pubNbOfDeviceErrors));

        //Get error code
        m_pCommand_GetNbOfDeviceErrors->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetErrorHistory_VCS_ErrorHandling::VCS_GetDeviceErrorCode(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubErrorNumber, DWORD* p_pulDeviceErrorCode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetDeviceErrorCode)
    {
        //Set parameter data
        m_pCommand_GetDeviceErrorCode->ResetStatus();
        m_pCommand_GetDeviceErrorCode->SetParameterData(0, &p_ubErrorNumber, sizeof(p_ubErrorNumber));

        //Execute command
        oResult = m_pCommand_GetDeviceErrorCode->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetDeviceErrorCode->GetReturnParameterData(0, p_pulDeviceErrorCode, sizeof(*p_pulDeviceErrorCode));

        //Get error code
        m_pCommand_GetDeviceErrorCode->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetErrorHistory_VCS_ErrorHandling::InitCommands()
{
    DeleteCommands();

    //Init ClearDeviceErrors
    m_pCommand_ClearDeviceErrors = new CCommand_VCS_ErrorHandling();
    m_pCommand_ClearDeviceErrors->InitCommand(EH_CLEAR_DEVICE_ERRORS);
    //Init GetNbOfDeviceErrors
    m_pCommand_GetNbOfDeviceErrors = new CCommand_VCS_ErrorHandling();
    m_pCommand_GetNbOfDeviceErrors->InitCommand(EH_GET_NB_OF_DEVICE_ERRORS);
    //Init GetDeviceErrorCode
    m_pCommand_GetDeviceErrorCode = new CCommand_VCS_ErrorHandling();
    m_pCommand_GetDeviceErrorCode->InitCommand(EH_GET_DEVICE_ERROR_CODE);
}

void CCommandSetErrorHistory_VCS_ErrorHandling::DeleteCommands()
{
    if(m_pCommand_ClearDeviceErrors)
    {
        delete m_pCommand_ClearDeviceErrors;
        m_pCommand_ClearDeviceErrors = NULL;
    }
    if(m_pCommand_GetNbOfDeviceErrors)
    {
        delete m_pCommand_GetNbOfDeviceErrors;
        m_pCommand_GetNbOfDeviceErrors = NULL;
    }
    if(m_pCommand_GetDeviceErrorCode)
    {
        delete m_pCommand_GetDeviceErrorCode;
        m_pCommand_GetDeviceErrorCode = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetErrorHistory_VCS_ErrorHandling::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_ClearDeviceErrors) m_pCommand_ClearDeviceErrors->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetNbOfDeviceErrors) m_pCommand_GetNbOfDeviceErrors->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetDeviceErrorCode) m_pCommand_GetDeviceErrorCode->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetErrorHistory_VCS_ErrorHandling::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_ClearDeviceErrors) m_pCommand_ClearDeviceErrors->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetNbOfDeviceErrors) m_pCommand_GetNbOfDeviceErrors->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetDeviceErrorCode) m_pCommand_GetDeviceErrorCode->InitJournalManager(p_pJournalManager);
}

void CCommandSetErrorHistory_VCS_ErrorHandling::ResetJournalManager()
{
    if(m_pCommand_ClearDeviceErrors) m_pCommand_ClearDeviceErrors->ResetJournalManager();
    if(m_pCommand_GetNbOfDeviceErrors) m_pCommand_GetNbOfDeviceErrors->ResetJournalManager();
    if(m_pCommand_GetDeviceErrorCode) m_pCommand_GetDeviceErrorCode->ResetJournalManager();
}

BOOL CCommandSetErrorHistory_VCS_ErrorHandling::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_ClearDeviceErrors && !m_pCommand_ClearDeviceErrors->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetNbOfDeviceErrors && !m_pCommand_GetNbOfDeviceErrors->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetDeviceErrorCode && !m_pCommand_GetDeviceErrorCode->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
