
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_ErrorHandling.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/ErrorHandling/CommandSetReset_VCS_ErrorHandling.h>

CCommandSetReset_VCS_ErrorHandling::CCommandSetReset_VCS_ErrorHandling()
{
    m_strCommandSetName = COMMAND_SET_ERROR_HANDLING_RESET;

    m_pCommand_ResetDevice = 0;
    m_pCommand_WaitForReboot = 0;

    InitCommands();
}

CCommandSetReset_VCS_ErrorHandling::~CCommandSetReset_VCS_ErrorHandling()
{
    DeleteCommands();
}

BOOL CCommandSetReset_VCS_ErrorHandling::VCS_ResetDevice(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL p_oWaitForReboot, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ResetDevice)
    {
        //Set parameter data
        m_pCommand_ResetDevice->ResetStatus();
        m_pCommand_ResetDevice->SetParameterData(0, &p_oWaitForReboot, sizeof(p_oWaitForReboot));

        //Execute command
        oResult = m_pCommand_ResetDevice->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ResetDevice->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetReset_VCS_ErrorHandling::VCS_ResetDevice(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL p_oWaitForReboot, DWORD p_ulRebootTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ResetDevice)
    {
        //Set parameter data
        m_pCommand_ResetDevice->ResetStatus();
        m_pCommand_ResetDevice->SetParameterData(0, &p_oWaitForReboot, sizeof(p_oWaitForReboot));
        m_pCommand_ResetDevice->SetParameterData(1, &p_ulRebootTimeout, sizeof(p_ulRebootTimeout));

        //Execute command
        oResult = m_pCommand_ResetDevice->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ResetDevice->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetReset_VCS_ErrorHandling::VCS_WaitForReboot(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_WaitForReboot)
    {
        //Set parameter data
        m_pCommand_WaitForReboot->ResetStatus();

        //Execute command
        oResult = m_pCommand_WaitForReboot->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_WaitForReboot->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetReset_VCS_ErrorHandling::VCS_WaitForReboot(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulRebootTimeout, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_WaitForReboot)
    {
        //Set parameter data
        m_pCommand_WaitForReboot->ResetStatus();
        m_pCommand_WaitForReboot->SetParameterData(0, &p_ulRebootTimeout, sizeof(p_ulRebootTimeout));

        //Execute command
        oResult = m_pCommand_WaitForReboot->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_WaitForReboot->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetReset_VCS_ErrorHandling::InitCommands()
{
    DeleteCommands();

    m_pCommand_ResetDevice = new CCommand_VCS_ErrorHandling();
    m_pCommand_ResetDevice->InitCommand(EH_RESET_DEVICE);

    m_pCommand_WaitForReboot = new CCommand_VCS_ErrorHandling();
    m_pCommand_WaitForReboot->InitCommand(EH_WAIT_FOR_REBOOT);
}

void CCommandSetReset_VCS_ErrorHandling::DeleteCommands()
{
    if(m_pCommand_ResetDevice)
    {
        delete m_pCommand_ResetDevice;
        m_pCommand_ResetDevice = NULL;
    }
    if(m_pCommand_WaitForReboot)
    {
        delete m_pCommand_WaitForReboot;
        m_pCommand_WaitForReboot = 0;
    }
}

CXXMLFile::CElementPart* CCommandSetReset_VCS_ErrorHandling::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_ResetDevice) m_pCommand_ResetDevice->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetReset_VCS_ErrorHandling::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_ResetDevice) m_pCommand_ResetDevice->InitJournalManager(p_pJournalManager);
    if(m_pCommand_WaitForReboot) m_pCommand_WaitForReboot->InitJournalManager(p_pJournalManager);
}

void CCommandSetReset_VCS_ErrorHandling::ResetJournalManager()
{
    if(m_pCommand_ResetDevice) m_pCommand_ResetDevice->ResetJournalManager();
    if(m_pCommand_WaitForReboot) m_pCommand_WaitForReboot->ResetJournalManager();
}

BOOL CCommandSetReset_VCS_ErrorHandling::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_ResetDevice && !m_pCommand_ResetDevice->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_WaitForReboot && !m_pCommand_WaitForReboot->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
