
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_ErrorHandling.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/ErrorHandling/CommandSetSafety_VCS_ErrorHandling.h>

CCommandSetSafety_VCS_ErrorHandling::CCommandSetSafety_VCS_ErrorHandling()
{
    m_strCommandSetName = COMMAND_SET_ERROR_HANDLING_SAFETY;

    m_pCommand_GotoSafeState = 0;

    InitCommands();
}

CCommandSetSafety_VCS_ErrorHandling::~CCommandSetSafety_VCS_ErrorHandling()
{
    DeleteCommands();
}

BOOL CCommandSetSafety_VCS_ErrorHandling::VCS_GotoSafeState(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GotoSafeState)
    {
        //Set parameter data
        m_pCommand_GotoSafeState->ResetStatus();

        //Execute command
        oResult = m_pCommand_GotoSafeState->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_GotoSafeState->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetSafety_VCS_ErrorHandling::InitCommands()
{
    DeleteCommands();

    //Init GotoSafeState
    m_pCommand_GotoSafeState = new CCommand_VCS_ErrorHandling();
    m_pCommand_GotoSafeState->InitCommand(EH_GOTO_SAFE_STATE);
}

void CCommandSetSafety_VCS_ErrorHandling::DeleteCommands()
{
    if(m_pCommand_GotoSafeState)
    {
        delete m_pCommand_GotoSafeState;
        m_pCommand_GotoSafeState = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetSafety_VCS_ErrorHandling::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GotoSafeState) m_pCommand_GotoSafeState->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetSafety_VCS_ErrorHandling::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GotoSafeState) m_pCommand_GotoSafeState->InitJournalManager(p_pJournalManager);
}

void CCommandSetSafety_VCS_ErrorHandling::ResetJournalManager()
{
    if(m_pCommand_GotoSafeState) m_pCommand_GotoSafeState->ResetJournalManager();
}

BOOL CCommandSetSafety_VCS_ErrorHandling::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GotoSafeState && !m_pCommand_GotoSafeState->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
