
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Firmware.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Firmware/CommandSetDownload_VCS_Firmware.h>

CCommandSetDownload_VCS_Firmware::CCommandSetDownload_VCS_Firmware()
{
    m_strCommandSetName = COMMAND_SET_FIRMWARE_DOWNLOAD;

    m_pCommand_GetBaseSectorVersion = 0;
    m_pCommand_GetSerialNumber = 0;
    m_pCommand_ProgramHardwareInfos = 0;

    InitCommands();
}

CCommandSetDownload_VCS_Firmware::~CCommandSetDownload_VCS_Firmware()
{
    DeleteCommands();
}

BOOL CCommandSetDownload_VCS_Firmware::VCS_GetBaseSectorVersion(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD* p_pusBaseSectorVersion, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetBaseSectorVersion)
    {
        //Set parameter data
        m_pCommand_GetBaseSectorVersion->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetBaseSectorVersion->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetBaseSectorVersion->GetReturnParameterData(0, p_pusBaseSectorVersion, sizeof(*p_pusBaseSectorVersion));

        //Get error code
        m_pCommand_GetBaseSectorVersion->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDownload_VCS_Firmware::VCS_GetSerialNumber(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, __uint64* p_pullSerialNumber, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetSerialNumber)
    {
        //Set parameter data
        m_pCommand_GetSerialNumber->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetSerialNumber->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetSerialNumber->GetReturnParameterData(0, p_pullSerialNumber, sizeof(*p_pullSerialNumber));

        //Get error code
        m_pCommand_GetSerialNumber->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDownload_VCS_Firmware::VCS_ProgramHardwareInfos(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usHardwareVersion, __uint64 p_ullSerialNumber, DWORD p_ulControlWord, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_ProgramHardwareInfos)
    {
        //Set parameter data
        m_pCommand_ProgramHardwareInfos->ResetStatus();
        m_pCommand_ProgramHardwareInfos->SetParameterData(0, &p_usHardwareVersion, sizeof(p_usHardwareVersion));
        m_pCommand_ProgramHardwareInfos->SetParameterData(1, &p_ullSerialNumber, sizeof(p_ullSerialNumber));
        m_pCommand_ProgramHardwareInfos->SetParameterData(2, &p_ulControlWord, sizeof(p_ulControlWord));

        //Execute command
        oResult = m_pCommand_ProgramHardwareInfos->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ProgramHardwareInfos->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetDownload_VCS_Firmware::InitCommands()
{
    DeleteCommands();

    m_pCommand_GetBaseSectorVersion = new CCommand_VCS_Firmware();
    m_pCommand_GetBaseSectorVersion->InitCommand(FIRMWARE_GET_BASE_SECTOR_VERSION);

    m_pCommand_GetSerialNumber = new CCommand_VCS_Firmware();
    m_pCommand_GetSerialNumber->InitCommand(FIRMWARE_GET_SERIAL_NUMBER);

    m_pCommand_ProgramHardwareInfos = new CCommand_VCS_Firmware();
    m_pCommand_ProgramHardwareInfos->InitCommand(FIRMWARE_PROGRAM_HARDWARE_INFOS);
}

void CCommandSetDownload_VCS_Firmware::DeleteCommands()
{
    if(m_pCommand_GetBaseSectorVersion)
    {
        delete m_pCommand_GetBaseSectorVersion;
        m_pCommand_GetBaseSectorVersion = NULL;
    }
    if(m_pCommand_GetSerialNumber)
    {
        delete m_pCommand_GetSerialNumber;
        m_pCommand_GetSerialNumber = NULL;
    }
    if(m_pCommand_ProgramHardwareInfos)
    {
        delete m_pCommand_ProgramHardwareInfos;
        m_pCommand_ProgramHardwareInfos = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetDownload_VCS_Firmware::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetBaseSectorVersion) m_pCommand_GetBaseSectorVersion->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetSerialNumber) m_pCommand_GetSerialNumber->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_ProgramHardwareInfos) m_pCommand_ProgramHardwareInfos->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetDownload_VCS_Firmware::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetBaseSectorVersion) m_pCommand_GetBaseSectorVersion->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetSerialNumber) m_pCommand_GetSerialNumber->InitJournalManager(p_pJournalManager);
    if(m_pCommand_ProgramHardwareInfos) m_pCommand_ProgramHardwareInfos->InitJournalManager(p_pJournalManager);
}

void CCommandSetDownload_VCS_Firmware::ResetJournalManager()
{
    if(m_pCommand_GetBaseSectorVersion) m_pCommand_GetBaseSectorVersion->ResetJournalManager();
    if(m_pCommand_GetSerialNumber) m_pCommand_GetSerialNumber->ResetJournalManager();
    if(m_pCommand_ProgramHardwareInfos) m_pCommand_ProgramHardwareInfos->ResetJournalManager();
}

BOOL CCommandSetDownload_VCS_Firmware::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetBaseSectorVersion && !m_pCommand_GetBaseSectorVersion->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetSerialNumber && !m_pCommand_GetSerialNumber->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_ProgramHardwareInfos && !m_pCommand_ProgramHardwareInfos->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
