
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/InputsOutputs/CommandSetAnalogInputs_VCS_InputsOutputs.h>

CCommandSetAnalogInputs_VCS_InputsOutputs::CCommandSetAnalogInputs_VCS_InputsOutputs()
{
    m_strCommandSetName = COMMAND_SET_INPUTS_OUTPUTS_ANALOG_INPUTS;

    m_pCommand_AnalogInputConfiguration = NULL;
    m_pCommand_GetAnalogInput = NULL;
    m_pCommand_GetAnalogInputVoltage = NULL;

    InitCommands();
}

CCommandSetAnalogInputs_VCS_InputsOutputs::~CCommandSetAnalogInputs_VCS_InputsOutputs()
{
    DeleteCommands();
}

BOOL CCommandSetAnalogInputs_VCS_InputsOutputs::VCS_AnalogInputConfiguration(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usInputNumber, WORD p_usConfiguration, BOOL p_oExecutionMask, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_AnalogInputConfiguration)
    {
        //Set parameter data
        m_pCommand_AnalogInputConfiguration->ResetStatus();
        m_pCommand_AnalogInputConfiguration->SetParameterData(0, &p_usInputNumber, sizeof(p_usInputNumber));
        m_pCommand_AnalogInputConfiguration->SetParameterData(1, &p_usConfiguration, sizeof(p_usConfiguration));
        m_pCommand_AnalogInputConfiguration->SetParameterData(2, &p_oExecutionMask, sizeof(p_oExecutionMask));

        //Execute command
        oResult = m_pCommand_AnalogInputConfiguration->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_AnalogInputConfiguration->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetAnalogInputs_VCS_InputsOutputs::VCS_GetAnalogInput(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usInputNumber, WORD* p_pusAnalogValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetAnalogInput)
    {
        //Set parameter data
        m_pCommand_GetAnalogInput->ResetStatus();
        m_pCommand_GetAnalogInput->SetParameterData(0, &p_usInputNumber, sizeof(p_usInputNumber));

        //Execute command
        oResult = m_pCommand_GetAnalogInput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetAnalogInput->GetReturnParameterData(0, p_pusAnalogValue, sizeof(*p_pusAnalogValue));

        //Get error code
        m_pCommand_GetAnalogInput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetAnalogInputs_VCS_InputsOutputs::VCS_GetAnalogInputVoltage(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usInputNumber, long* p_plAnalogValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if (m_pCommand_GetAnalogInputVoltage)
    {
        //Set parameter data
        m_pCommand_GetAnalogInputVoltage->ResetStatus();
        m_pCommand_GetAnalogInputVoltage->SetParameterData(0, &p_usInputNumber, sizeof(p_usInputNumber));

        //Execute command
        oResult = m_pCommand_GetAnalogInputVoltage->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetAnalogInputVoltage->GetReturnParameterData(0, p_plAnalogValue, sizeof(*p_plAnalogValue));

        //Get error code
        m_pCommand_GetAnalogInputVoltage->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetAnalogInputs_VCS_InputsOutputs::InitCommands()
{
    DeleteCommands();

    //Init AnalogInputConfiguration
    m_pCommand_AnalogInputConfiguration = new CCommand_VCS_InputsOutputs();
    m_pCommand_AnalogInputConfiguration->InitCommand(IO_ANALOG_INPUT_CONFIGURATION);

    //Init GetAnalogInput
    m_pCommand_GetAnalogInput = new CCommand_VCS_InputsOutputs();
    m_pCommand_GetAnalogInput->InitCommand(IO_GET_ANALOG_INPUT);

    //Init GetAnalogInput
    m_pCommand_GetAnalogInputVoltage = new CCommand_VCS_InputsOutputs();
    m_pCommand_GetAnalogInputVoltage->InitCommand(IO_GET_ANALOG_INPUT_VOLTAGE);
}

void CCommandSetAnalogInputs_VCS_InputsOutputs::DeleteCommands()
{
    if(m_pCommand_AnalogInputConfiguration)
    {
        delete m_pCommand_AnalogInputConfiguration;
        m_pCommand_AnalogInputConfiguration = NULL;
    }

    if (m_pCommand_GetAnalogInput)
    {
        delete m_pCommand_GetAnalogInput;
        m_pCommand_GetAnalogInput = NULL;
    }

    if (m_pCommand_GetAnalogInputVoltage)
    {
        delete m_pCommand_GetAnalogInputVoltage;
        m_pCommand_GetAnalogInputVoltage = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetAnalogInputs_VCS_InputsOutputs::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_AnalogInputConfiguration) m_pCommand_AnalogInputConfiguration->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetAnalogInput) m_pCommand_GetAnalogInput->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetAnalogInputVoltage) m_pCommand_GetAnalogInputVoltage->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetAnalogInputs_VCS_InputsOutputs::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_AnalogInputConfiguration) m_pCommand_AnalogInputConfiguration->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetAnalogInput) m_pCommand_GetAnalogInput->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetAnalogInputVoltage) m_pCommand_GetAnalogInputVoltage->InitJournalManager(p_pJournalManager);
}

void CCommandSetAnalogInputs_VCS_InputsOutputs::ResetJournalManager()
{
    if(m_pCommand_AnalogInputConfiguration) m_pCommand_AnalogInputConfiguration->ResetJournalManager();
    if(m_pCommand_GetAnalogInput) m_pCommand_GetAnalogInput->ResetJournalManager();
    if(m_pCommand_GetAnalogInputVoltage) m_pCommand_GetAnalogInputVoltage->ResetJournalManager();
}

BOOL CCommandSetAnalogInputs_VCS_InputsOutputs::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_AnalogInputConfiguration && !m_pCommand_AnalogInputConfiguration->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetAnalogInput && !m_pCommand_GetAnalogInput->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetAnalogInputVoltage && !m_pCommand_GetAnalogInputVoltage->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
