
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/InputsOutputs/CommandSetAnalogOutputs_VCS_InputsOutputs.h>

CCommandSetAnalogOutputs_VCS_InputsOutputs::CCommandSetAnalogOutputs_VCS_InputsOutputs()
{
    m_strCommandSetName = COMMAND_SET_INPUTS_OUTPUTS_ANALOG_OUTPUTS;

    m_pCommand_AnalogOutputConfiguration = 0;
    m_pCommand_SetAnalogOutput = 0;
    m_pCommand_SetAnalogOutputVoltage = 0;
    m_pCommand_SetAnalogOutputState = 0;

    InitCommands();
}

CCommandSetAnalogOutputs_VCS_InputsOutputs::~CCommandSetAnalogOutputs_VCS_InputsOutputs()
{
    DeleteCommands();
}

BOOL CCommandSetAnalogOutputs_VCS_InputsOutputs::VCS_AnalogOutputConfiguration(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usOutputNumber, WORD p_usConfiguration, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if (m_pCommand_AnalogOutputConfiguration)
    {
        //Set parameter data
        m_pCommand_AnalogOutputConfiguration->ResetStatus();
        m_pCommand_AnalogOutputConfiguration->SetParameterData(0, &p_usOutputNumber, sizeof(p_usOutputNumber));
        m_pCommand_AnalogOutputConfiguration->SetParameterData(1, &p_usConfiguration, sizeof(p_usConfiguration));

        //Execute command
        oResult = m_pCommand_AnalogOutputConfiguration->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_AnalogOutputConfiguration->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetAnalogOutputs_VCS_InputsOutputs::VCS_SetAnalogOutput(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usOutputNumber, WORD* p_pusAnalogValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetAnalogOutput)
    {
        //Set parameter data
        m_pCommand_SetAnalogOutput->ResetStatus();
        m_pCommand_SetAnalogOutput->SetParameterData(0, &p_usOutputNumber, sizeof(p_usOutputNumber));
        m_pCommand_SetAnalogOutput->SetParameterData(1, &p_pusAnalogValue, sizeof(p_pusAnalogValue));

        //Execute command
        oResult = m_pCommand_SetAnalogOutput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetAnalogOutput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetAnalogOutputs_VCS_InputsOutputs::VCS_SetAnalogOutputVoltage(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usOutputNumber, long* p_plAnalogValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if (m_pCommand_SetAnalogOutput)
    {
        //Set parameter data
        m_pCommand_SetAnalogOutput->ResetStatus();
        m_pCommand_SetAnalogOutput->SetParameterData(0, &p_usOutputNumber, sizeof(p_usOutputNumber));
        m_pCommand_SetAnalogOutput->SetParameterData(1, &p_plAnalogValue, sizeof(p_plAnalogValue));

        //Execute command
        oResult = m_pCommand_SetAnalogOutput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetAnalogOutput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetAnalogOutputs_VCS_InputsOutputs::VCS_SetAnalogOutputState(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usConfiguration, long* p_plStateValue, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if (m_pCommand_SetAnalogOutput)
    {
        //Set parameter data
        m_pCommand_SetAnalogOutput->ResetStatus();
        m_pCommand_SetAnalogOutput->SetParameterData(0, &p_usConfiguration, sizeof(p_usConfiguration));
        m_pCommand_SetAnalogOutput->SetParameterData(1, &p_plStateValue, sizeof(p_plStateValue));

        //Execute command
        oResult = m_pCommand_SetAnalogOutput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetAnalogOutput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetAnalogOutputs_VCS_InputsOutputs::InitCommands()
{
    DeleteCommands();

    m_pCommand_AnalogOutputConfiguration = new CCommand_VCS_InputsOutputs();
    m_pCommand_AnalogOutputConfiguration->InitCommand(IO_ANALOG_OUTPUT_CONFIGURATION);

    m_pCommand_SetAnalogOutput = new CCommand_VCS_InputsOutputs();
    m_pCommand_SetAnalogOutput->InitCommand(IO_SET_ANALOG_OUTPUT);

    m_pCommand_SetAnalogOutputVoltage = new CCommand_VCS_InputsOutputs();
    m_pCommand_SetAnalogOutputVoltage->InitCommand(IO_SET_ANALOG_OUTPUT_VOLTAGE);

    m_pCommand_SetAnalogOutputState = new CCommand_VCS_InputsOutputs();
    m_pCommand_SetAnalogOutputState->InitCommand(IO_SET_ANALOG_OUTPUT_STATE);
}

void CCommandSetAnalogOutputs_VCS_InputsOutputs::DeleteCommands()
{
    if (m_pCommand_AnalogOutputConfiguration)
    {
        delete m_pCommand_AnalogOutputConfiguration;
        m_pCommand_AnalogOutputConfiguration = NULL;
    }

    if (m_pCommand_SetAnalogOutput)
    {
        delete m_pCommand_SetAnalogOutput;
        m_pCommand_SetAnalogOutput = NULL;
    }

    if (m_pCommand_SetAnalogOutputVoltage)
    {
        delete m_pCommand_SetAnalogOutputVoltage;
        m_pCommand_SetAnalogOutputVoltage = NULL;
    }

    if (m_pCommand_SetAnalogOutputState)
    {
        delete m_pCommand_SetAnalogOutputState;
        m_pCommand_SetAnalogOutputState = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetAnalogOutputs_VCS_InputsOutputs::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if (m_pCommand_AnalogOutputConfiguration) m_pCommand_AnalogOutputConfiguration->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if (m_pCommand_SetAnalogOutput) m_pCommand_SetAnalogOutput->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if (m_pCommand_SetAnalogOutputVoltage) m_pCommand_SetAnalogOutputVoltage->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if (m_pCommand_SetAnalogOutputState) m_pCommand_SetAnalogOutputState->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetAnalogOutputs_VCS_InputsOutputs::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if (m_pCommand_AnalogOutputConfiguration) m_pCommand_AnalogOutputConfiguration->InitJournalManager(p_pJournalManager);
    if (m_pCommand_SetAnalogOutput) m_pCommand_SetAnalogOutput->InitJournalManager(p_pJournalManager);
    if (m_pCommand_SetAnalogOutputVoltage) m_pCommand_SetAnalogOutputVoltage->InitJournalManager(p_pJournalManager);
    if (m_pCommand_SetAnalogOutputState) m_pCommand_SetAnalogOutputState->InitJournalManager(p_pJournalManager);
}

void CCommandSetAnalogOutputs_VCS_InputsOutputs::ResetJournalManager()
{
    if (m_pCommand_AnalogOutputConfiguration) m_pCommand_AnalogOutputConfiguration->ResetJournalManager();
    if (m_pCommand_SetAnalogOutput) m_pCommand_SetAnalogOutput->ResetJournalManager();
    if (m_pCommand_SetAnalogOutputVoltage) m_pCommand_SetAnalogOutputVoltage->ResetJournalManager();
    if (m_pCommand_SetAnalogOutputState) m_pCommand_SetAnalogOutputState->ResetJournalManager();
}

BOOL CCommandSetAnalogOutputs_VCS_InputsOutputs::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if (oResult && m_pCommand_AnalogOutputConfiguration && !m_pCommand_AnalogOutputConfiguration->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_SetAnalogOutput && !m_pCommand_SetAnalogOutput->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_SetAnalogOutputVoltage && !m_pCommand_SetAnalogOutputVoltage->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_SetAnalogOutputState && !m_pCommand_SetAnalogOutputState->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
