
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/InputsOutputs/CommandSetDigitalInputs_VCS_InputsOutputs.h>

CCommandSetDigitalInputs_VCS_InputsOutputs::CCommandSetDigitalInputs_VCS_InputsOutputs()
{
    m_strCommandSetName = COMMAND_SET_INPUTS_OUTPUTS_DIGITAL_INPUTS;

    m_pCommand_DigitalInputConfiguration = 0;
    m_pCommand_GetAllDigitalInputs = NULL;

    InitCommands();
}

CCommandSetDigitalInputs_VCS_InputsOutputs::~CCommandSetDigitalInputs_VCS_InputsOutputs()
{
    DeleteCommands();
}

BOOL CCommandSetDigitalInputs_VCS_InputsOutputs::VCS_DigitalInputConfiguration(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usDigitalInputNb, WORD p_usConfiguration, BOOL p_oMask, BOOL p_oPolarity, BOOL p_oExecutionMask, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DigitalInputConfiguration)
    {
        //Set parameter data
        m_pCommand_DigitalInputConfiguration->ResetStatus();
        m_pCommand_DigitalInputConfiguration->SetParameterData(0, &p_usDigitalInputNb, sizeof(p_usDigitalInputNb));
        m_pCommand_DigitalInputConfiguration->SetParameterData(1, &p_usConfiguration, sizeof(p_usConfiguration));
        m_pCommand_DigitalInputConfiguration->SetParameterData(2, &p_oMask, sizeof(p_oMask));
        m_pCommand_DigitalInputConfiguration->SetParameterData(3, &p_oPolarity, sizeof(p_oPolarity));
        m_pCommand_DigitalInputConfiguration->SetParameterData(4, &p_oExecutionMask, sizeof(p_oExecutionMask));

        //Execute command
        oResult = m_pCommand_DigitalInputConfiguration->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DigitalInputConfiguration->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDigitalInputs_VCS_InputsOutputs::VCS_GetAllDigitalInputs(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD* p_pusInputs, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetAllDigitalInputs)
    {
        //Set parameter data
        m_pCommand_GetAllDigitalInputs->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetAllDigitalInputs->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetAllDigitalInputs->GetReturnParameterData(0, p_pusInputs, sizeof(*p_pusInputs));

        //Get error code
        m_pCommand_GetAllDigitalInputs->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetDigitalInputs_VCS_InputsOutputs::InitCommands()
{
    DeleteCommands();

    m_pCommand_DigitalInputConfiguration = new CCommand_VCS_InputsOutputs();
    m_pCommand_DigitalInputConfiguration->InitCommand(IO_DIGITAL_INPUT_CONFIGURATION);

    m_pCommand_GetAllDigitalInputs = new CCommand_VCS_InputsOutputs();
    m_pCommand_GetAllDigitalInputs->InitCommand(IO_GET_ALL_DIGITAL_INPUTS);
}

void CCommandSetDigitalInputs_VCS_InputsOutputs::DeleteCommands()
{
    if(m_pCommand_DigitalInputConfiguration)
    {
        delete m_pCommand_DigitalInputConfiguration;
        m_pCommand_DigitalInputConfiguration = NULL;
    }
    if(m_pCommand_GetAllDigitalInputs)
    {
        delete m_pCommand_GetAllDigitalInputs;
        m_pCommand_GetAllDigitalInputs = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetDigitalInputs_VCS_InputsOutputs::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_DigitalInputConfiguration) m_pCommand_DigitalInputConfiguration->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetAllDigitalInputs) m_pCommand_GetAllDigitalInputs->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetDigitalInputs_VCS_InputsOutputs::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_DigitalInputConfiguration) m_pCommand_DigitalInputConfiguration->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetAllDigitalInputs) m_pCommand_GetAllDigitalInputs->InitJournalManager(p_pJournalManager);
}

void CCommandSetDigitalInputs_VCS_InputsOutputs::ResetJournalManager()
{
    if(m_pCommand_DigitalInputConfiguration) m_pCommand_DigitalInputConfiguration->ResetJournalManager();
    if(m_pCommand_GetAllDigitalInputs) m_pCommand_GetAllDigitalInputs->ResetJournalManager();
}

BOOL CCommandSetDigitalInputs_VCS_InputsOutputs::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_DigitalInputConfiguration && !m_pCommand_DigitalInputConfiguration->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetAllDigitalInputs && !m_pCommand_GetAllDigitalInputs->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
