
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/InputsOutputs/CommandSetDigitalOutputs_VCS_InputsOutputs.h>

CCommandSetDigitalOutputs_VCS_InputsOutputs::CCommandSetDigitalOutputs_VCS_InputsOutputs()
{
    m_strCommandSetName = COMMAND_SET_INPUTS_OUTPUTS_DIGITAL_OUTPUTS;

    m_pCommand_DigitalOutputConfiguration = NULL;
    m_pCommand_GetAllDigitalOutputs = NULL;
    m_pCommand_SetAllDigitalOutputs = NULL;

    InitCommands();
}

CCommandSetDigitalOutputs_VCS_InputsOutputs::~CCommandSetDigitalOutputs_VCS_InputsOutputs()
{
    DeleteCommands();
}

BOOL CCommandSetDigitalOutputs_VCS_InputsOutputs::VCS_DigitalOutputConfiguration(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usDigitalOutputNb, WORD p_usConfiguration, BOOL p_oState, BOOL p_oMask, BOOL p_oPolarity, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DigitalOutputConfiguration)
    {
        //Set parameter data
        m_pCommand_DigitalOutputConfiguration->ResetStatus();
        m_pCommand_DigitalOutputConfiguration->SetParameterData(0, &p_usDigitalOutputNb, sizeof(p_usDigitalOutputNb));
        m_pCommand_DigitalOutputConfiguration->SetParameterData(1, &p_usConfiguration, sizeof(p_usConfiguration));
        m_pCommand_DigitalOutputConfiguration->SetParameterData(2, &p_oState, sizeof(p_oState));
        m_pCommand_DigitalOutputConfiguration->SetParameterData(3, &p_oMask, sizeof(p_oMask));
        m_pCommand_DigitalOutputConfiguration->SetParameterData(4, &p_oPolarity, sizeof(p_oPolarity));

        //Execute command
        oResult = m_pCommand_DigitalOutputConfiguration->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DigitalOutputConfiguration->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDigitalOutputs_VCS_InputsOutputs::VCS_GetAllDigitalOutputs(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD* p_pusOutputs, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetAllDigitalOutputs)
    {
        //Set parameter data
        m_pCommand_GetAllDigitalOutputs->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetAllDigitalOutputs->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetAllDigitalOutputs->GetReturnParameterData(0, p_pusOutputs, sizeof(*p_pusOutputs));

        //Get error code
        m_pCommand_GetAllDigitalOutputs->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetDigitalOutputs_VCS_InputsOutputs::VCS_SetAllDigitalOutputs(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usOutputs, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetAllDigitalOutputs)
    {
        //Set parameter data
        m_pCommand_SetAllDigitalOutputs->ResetStatus();
        m_pCommand_SetAllDigitalOutputs->SetParameterData(0, &p_usOutputs, sizeof(p_usOutputs));

        //Execute command
        oResult = m_pCommand_SetAllDigitalOutputs->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetAllDigitalOutputs->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetDigitalOutputs_VCS_InputsOutputs::InitCommands()
{
    DeleteCommands();

    //Init DigitalOutputConfiguration
    m_pCommand_DigitalOutputConfiguration = new CCommand_VCS_InputsOutputs();
    m_pCommand_DigitalOutputConfiguration->InitCommand(IO_DIGITAL_OUTPUT_CONFIGURATION);
    //Init GetAllDigitalOutputs
    m_pCommand_GetAllDigitalOutputs = new CCommand_VCS_InputsOutputs();
    m_pCommand_GetAllDigitalOutputs->InitCommand(IO_GET_ALL_DIGITAL_OUTPUTS);
    //Init SetAllDigitalOutputs
    m_pCommand_SetAllDigitalOutputs = new CCommand_VCS_InputsOutputs();
    m_pCommand_SetAllDigitalOutputs->InitCommand(IO_SET_ALL_DIGITAL_OUTPUTS);
}

void CCommandSetDigitalOutputs_VCS_InputsOutputs::DeleteCommands()
{
    if(m_pCommand_DigitalOutputConfiguration)
    {
        delete m_pCommand_DigitalOutputConfiguration;
        m_pCommand_DigitalOutputConfiguration = NULL;
    }
    if(m_pCommand_GetAllDigitalOutputs)
    {
        delete m_pCommand_GetAllDigitalOutputs;
        m_pCommand_GetAllDigitalOutputs = NULL;
    }
    if(m_pCommand_SetAllDigitalOutputs)
    {
        delete m_pCommand_SetAllDigitalOutputs;
        m_pCommand_SetAllDigitalOutputs = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetDigitalOutputs_VCS_InputsOutputs::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_DigitalOutputConfiguration) m_pCommand_DigitalOutputConfiguration->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetAllDigitalOutputs) m_pCommand_GetAllDigitalOutputs->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetAllDigitalOutputs) m_pCommand_SetAllDigitalOutputs->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetDigitalOutputs_VCS_InputsOutputs::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_DigitalOutputConfiguration) m_pCommand_DigitalOutputConfiguration->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetAllDigitalOutputs) m_pCommand_GetAllDigitalOutputs->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetAllDigitalOutputs) m_pCommand_SetAllDigitalOutputs->InitJournalManager(p_pJournalManager);
}

void CCommandSetDigitalOutputs_VCS_InputsOutputs::ResetJournalManager()
{
    if(m_pCommand_DigitalOutputConfiguration) m_pCommand_DigitalOutputConfiguration->ResetJournalManager();
    if(m_pCommand_GetAllDigitalOutputs) m_pCommand_GetAllDigitalOutputs->ResetJournalManager();
    if(m_pCommand_SetAllDigitalOutputs) m_pCommand_SetAllDigitalOutputs->ResetJournalManager();
}

BOOL CCommandSetDigitalOutputs_VCS_InputsOutputs::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_DigitalOutputConfiguration && !m_pCommand_DigitalOutputConfiguration->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetAllDigitalOutputs && !m_pCommand_GetAllDigitalOutputs->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetAllDigitalOutputs && !m_pCommand_SetAllDigitalOutputs->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
