
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/OperationMode/CommandSetCurrentModeAnalog_VCS_OperationMode.h>

CCommandSetCurrentModeAnalog_VCS_OperationMode::CCommandSetCurrentModeAnalog_VCS_OperationMode()
{
    m_strCommandSetName = COMMAND_SET_OPERATION_MODE_CURRENT_MODE;

    m_pCommand_ActivateAnalogCurrentSetpoint = NULL;
    m_pCommand_DeactivateAnalogCurrentSetpoint = NULL;
    m_pCommand_EnableAnalogCurrentSetpoint = NULL;
    m_pCommand_DisableAnalogCurrentSetpoint = NULL;

    InitCommands();
}

CCommandSetCurrentModeAnalog_VCS_OperationMode::~CCommandSetCurrentModeAnalog_VCS_OperationMode()
{
    DeleteCommands();
}

BOOL CCommandSetCurrentModeAnalog_VCS_OperationMode::VCS_ActivateAnalogCurrentSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, float fScaling, short sOffset, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ActivateAnalogCurrentSetpoint)
    {
        //Set parameter data
        m_pCommand_ActivateAnalogCurrentSetpoint->ResetStatus();
        m_pCommand_ActivateAnalogCurrentSetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));
        m_pCommand_ActivateAnalogCurrentSetpoint->SetParameterData(1, &fScaling, sizeof(fScaling));
        m_pCommand_ActivateAnalogCurrentSetpoint->SetParameterData(2, &sOffset, sizeof(sOffset));

        //Execute command
        oResult = m_pCommand_ActivateAnalogCurrentSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ActivateAnalogCurrentSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetCurrentModeAnalog_VCS_OperationMode::VCS_DeactivateAnalogCurrentSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DeactivateAnalogCurrentSetpoint)
    {
        //Set parameter data
        m_pCommand_DeactivateAnalogCurrentSetpoint->ResetStatus();
        m_pCommand_DeactivateAnalogCurrentSetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));

        //Execute command
        oResult = m_pCommand_DeactivateAnalogCurrentSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DeactivateAnalogCurrentSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetCurrentModeAnalog_VCS_OperationMode::VCS_EnableAnalogCurrentSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_EnableAnalogCurrentSetpoint)
    {
        //Set parameter data
        m_pCommand_EnableAnalogCurrentSetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_EnableAnalogCurrentSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_EnableAnalogCurrentSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetCurrentModeAnalog_VCS_OperationMode::VCS_DisableAnalogCurrentSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DisableAnalogCurrentSetpoint)
    {
        //Set parameter data
        m_pCommand_DisableAnalogCurrentSetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_DisableAnalogCurrentSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DisableAnalogCurrentSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetCurrentModeAnalog_VCS_OperationMode::InitCommands()
{
    DeleteCommands();

    //Init ActivateAnalogCurrentSetpoint
    m_pCommand_ActivateAnalogCurrentSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_ActivateAnalogCurrentSetpoint->InitCommand(MODE_CUM_ACTIVATE_ANALOG_CURRENT_SETPOINT);
    //Init DeactivateAnalogCurrentSetpoint
    m_pCommand_DeactivateAnalogCurrentSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DeactivateAnalogCurrentSetpoint->InitCommand(MODE_CUM_DEACTIVATE_ANALOG_CURRENT_SETPOINT);
    //Init EnableAnalogCurrentSetpoint
    m_pCommand_EnableAnalogCurrentSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_EnableAnalogCurrentSetpoint->InitCommand(MODE_CUM_ENABLE_ANALOG_CURRENT_SETPOINT);
    //Init DisableAnalogCurrentSetpoint
    m_pCommand_DisableAnalogCurrentSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DisableAnalogCurrentSetpoint->InitCommand(MODE_CUM_DISABLE_ANALOG_CURRENT_SETPOINT);
}

void CCommandSetCurrentModeAnalog_VCS_OperationMode::DeleteCommands()
{
    if(m_pCommand_ActivateAnalogCurrentSetpoint)
    {
        delete m_pCommand_ActivateAnalogCurrentSetpoint;
        m_pCommand_ActivateAnalogCurrentSetpoint = NULL;
    }
    if(m_pCommand_DeactivateAnalogCurrentSetpoint)
    {
        delete m_pCommand_DeactivateAnalogCurrentSetpoint;
        m_pCommand_DeactivateAnalogCurrentSetpoint = NULL;
    }
    if(m_pCommand_EnableAnalogCurrentSetpoint)
    {
        delete m_pCommand_EnableAnalogCurrentSetpoint;
        m_pCommand_EnableAnalogCurrentSetpoint = NULL;
    }
    if(m_pCommand_DisableAnalogCurrentSetpoint)
    {
        delete m_pCommand_DisableAnalogCurrentSetpoint;
        m_pCommand_DisableAnalogCurrentSetpoint = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetCurrentModeAnalog_VCS_OperationMode::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_ActivateAnalogCurrentSetpoint) m_pCommand_ActivateAnalogCurrentSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DeactivateAnalogCurrentSetpoint) m_pCommand_DeactivateAnalogCurrentSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_EnableAnalogCurrentSetpoint) m_pCommand_EnableAnalogCurrentSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DisableAnalogCurrentSetpoint) m_pCommand_DisableAnalogCurrentSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetCurrentModeAnalog_VCS_OperationMode::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_ActivateAnalogCurrentSetpoint) m_pCommand_ActivateAnalogCurrentSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DeactivateAnalogCurrentSetpoint) m_pCommand_DeactivateAnalogCurrentSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_EnableAnalogCurrentSetpoint) m_pCommand_EnableAnalogCurrentSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DisableAnalogCurrentSetpoint) m_pCommand_DisableAnalogCurrentSetpoint->InitJournalManager(p_pJournalManager);
}

void CCommandSetCurrentModeAnalog_VCS_OperationMode::ResetJournalManager()
{
    if(m_pCommand_ActivateAnalogCurrentSetpoint) m_pCommand_ActivateAnalogCurrentSetpoint->ResetJournalManager();
    if(m_pCommand_DeactivateAnalogCurrentSetpoint) m_pCommand_DeactivateAnalogCurrentSetpoint->ResetJournalManager();
    if(m_pCommand_EnableAnalogCurrentSetpoint) m_pCommand_EnableAnalogCurrentSetpoint->ResetJournalManager();
    if(m_pCommand_DisableAnalogCurrentSetpoint) m_pCommand_DisableAnalogCurrentSetpoint->ResetJournalManager();
}

BOOL CCommandSetCurrentModeAnalog_VCS_OperationMode::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_ActivateAnalogCurrentSetpoint && !m_pCommand_ActivateAnalogCurrentSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DeactivateAnalogCurrentSetpoint && !m_pCommand_DeactivateAnalogCurrentSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_EnableAnalogCurrentSetpoint && !m_pCommand_EnableAnalogCurrentSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DisableAnalogCurrentSetpoint && !m_pCommand_DisableAnalogCurrentSetpoint->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
