#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/OperationMode/CommandSetHomingMode_VCS_OperationMode.h>

CCommandSetHomingMode_VCS_OperationMode::CCommandSetHomingMode_VCS_OperationMode()
{
    m_strCommandSetName = COMMAND_SET_OPERATION_MODE_HOMING_MODE;

    m_pCommand_FindHome = NULL;
    m_pCommand_GetHomingParameter = NULL;
    m_pCommand_SetHomingParameter = NULL;
    m_pCommand_StopHoming = NULL;
    m_pCommand_DefinePosition = NULL;
    m_pCommand_GetHomingState = NULL;

    InitCommands();
}

CCommandSetHomingMode_VCS_OperationMode::~CCommandSetHomingMode_VCS_OperationMode()
{
    DeleteCommands();
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_GetHomingParameter(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD* p_pulHomingAcceleration, DWORD* p_pulSpeedSwitch, DWORD* p_pulSpeedIndex, int* p_plHomeOffset, WORD* p_pusCurrentThreshold, int* p_plHomePosition, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetHomingParameter)
    {
        //Set parameter data
        m_pCommand_GetHomingParameter->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetHomingParameter->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetHomingParameter->GetReturnParameterData(0, p_pulHomingAcceleration, sizeof(*p_pulHomingAcceleration));
        m_pCommand_GetHomingParameter->GetReturnParameterData(1, p_pulSpeedSwitch, sizeof(*p_pulSpeedSwitch));
        m_pCommand_GetHomingParameter->GetReturnParameterData(2, p_pulSpeedIndex, sizeof(*p_pulSpeedIndex));
        m_pCommand_GetHomingParameter->GetReturnParameterData(3, p_plHomeOffset, sizeof(*p_plHomeOffset));
        m_pCommand_GetHomingParameter->GetReturnParameterData(4, p_pusCurrentThreshold, sizeof(*p_pusCurrentThreshold));
        m_pCommand_GetHomingParameter->GetReturnParameterData(5, p_plHomePosition, sizeof(*p_plHomePosition));

        //Get error code
        m_pCommand_GetHomingParameter->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_SetHomingParameter(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulHomingAcceleration, DWORD p_ulSpeedSwitch, DWORD p_ulSpeedIndex, int p_lHomeOffset, WORD p_usCurrentThreshold, int p_lHomePosition, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetHomingParameter)
    {
        //Set parameter data
        m_pCommand_SetHomingParameter->ResetStatus();
        m_pCommand_SetHomingParameter->SetParameterData(0, &p_ulHomingAcceleration, sizeof(p_ulHomingAcceleration));
        m_pCommand_SetHomingParameter->SetParameterData(1, &p_ulSpeedSwitch, sizeof(p_ulSpeedSwitch));
        m_pCommand_SetHomingParameter->SetParameterData(2, &p_ulSpeedIndex, sizeof(p_ulSpeedIndex));
        m_pCommand_SetHomingParameter->SetParameterData(3, &p_lHomeOffset, sizeof(p_lHomeOffset));
        m_pCommand_SetHomingParameter->SetParameterData(4, &p_usCurrentThreshold, sizeof(p_usCurrentThreshold));
        m_pCommand_SetHomingParameter->SetParameterData(5, &p_lHomePosition, sizeof(p_lHomePosition));

        //Execute command
        oResult = m_pCommand_SetHomingParameter->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetHomingParameter->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_FindHome(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, __int8 p_bHomingMethod, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_FindHome)
    {
        //Set parameter data
        m_pCommand_FindHome->ResetStatus();
        m_pCommand_FindHome->SetParameterData(0, &p_bHomingMethod, sizeof(p_bHomingMethod));

        //Execute command
        oResult = m_pCommand_FindHome->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_FindHome->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_StopHoming(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_StopHoming)
    {
        //Set parameter data
        m_pCommand_StopHoming->ResetStatus();

        //Execute command
        oResult = m_pCommand_StopHoming->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_StopHoming->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_DefinePosition(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, int p_lHomePosition, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DefinePosition)
    {
        //Set parameter data
        m_pCommand_DefinePosition->ResetStatus();
        m_pCommand_DefinePosition->SetParameterData(0, &p_lHomePosition, sizeof(p_lHomePosition));

        //Execute command
        oResult = m_pCommand_DefinePosition->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DefinePosition->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetHomingMode_VCS_OperationMode::VCS_GetHomingState(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL* p_poHomingAttained, BOOL* p_poHomingError, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if (m_pCommand_GetHomingState)
    {
        //Set parameter data
        m_pCommand_GetHomingState->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetHomingState->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetHomingState->GetReturnParameterData(0, p_poHomingAttained, sizeof(*p_poHomingAttained));
        m_pCommand_GetHomingState->GetReturnParameterData(1, p_poHomingError, sizeof(*p_poHomingError));

        //Get error code
        m_pCommand_GetHomingState->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetHomingMode_VCS_OperationMode::InitCommands()
{
    DeleteCommands();

    //Init FindHome command
    m_pCommand_FindHome = new CCommand_VCS_OperationMode();
    m_pCommand_FindHome->InitCommand(MODE_HM_FIND_HOME);

    //Init GetHomingParameter command
    m_pCommand_GetHomingParameter = new CCommand_VCS_OperationMode();
    m_pCommand_GetHomingParameter->InitCommand(MODE_HM_GET_HOMING_PARAMETER);

    //Init SetHomingParameter command
    m_pCommand_SetHomingParameter = new CCommand_VCS_OperationMode();
    m_pCommand_SetHomingParameter->InitCommand(MODE_HM_SET_HOMING_PARAMETER);

    //Init stop homing
    m_pCommand_StopHoming = new CCommand_VCS_OperationMode();
    m_pCommand_StopHoming->InitCommand(MODE_HM_STOP_HOMING);

    //Init Define Position
    m_pCommand_DefinePosition = new CCommand_VCS_OperationMode();
    m_pCommand_DefinePosition->InitCommand(MODE_HM_DEFINE_POSITION);

    //Init GetHomingState
    m_pCommand_GetHomingState = new CCommand_VCS_OperationMode();
    m_pCommand_GetHomingState->InitCommand(MODE_HM_GET_HOMING_STATE);
}

void CCommandSetHomingMode_VCS_OperationMode::DeleteCommands()
{
    if(m_pCommand_FindHome)
    {
        delete m_pCommand_FindHome;
        m_pCommand_FindHome = NULL;
    }

    if(m_pCommand_GetHomingParameter)
    {
        delete m_pCommand_GetHomingParameter;
        m_pCommand_GetHomingParameter = NULL;
    }

    if(m_pCommand_SetHomingParameter)
    {
        delete m_pCommand_SetHomingParameter;
        m_pCommand_SetHomingParameter = NULL;
    }

    if(m_pCommand_StopHoming)
    {
        delete m_pCommand_StopHoming;
        m_pCommand_StopHoming = NULL;
    }

    if(m_pCommand_DefinePosition)
    {
        delete m_pCommand_DefinePosition;
        m_pCommand_DefinePosition = NULL;
    }

    if (m_pCommand_GetHomingState)
    {
        delete m_pCommand_GetHomingState;
        m_pCommand_GetHomingState = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetHomingMode_VCS_OperationMode::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetHomingParameter) m_pCommand_GetHomingParameter->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetHomingParameter) m_pCommand_SetHomingParameter->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_FindHome) m_pCommand_FindHome->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_StopHoming) m_pCommand_StopHoming->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DefinePosition) m_pCommand_DefinePosition->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if (m_pCommand_GetHomingState) m_pCommand_GetHomingState->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetHomingMode_VCS_OperationMode::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_FindHome) m_pCommand_FindHome->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetHomingParameter) m_pCommand_GetHomingParameter->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetHomingParameter) m_pCommand_SetHomingParameter->InitJournalManager(p_pJournalManager);
    if(m_pCommand_StopHoming) m_pCommand_StopHoming->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DefinePosition) m_pCommand_DefinePosition->InitJournalManager(p_pJournalManager);
    if (m_pCommand_GetHomingState) m_pCommand_GetHomingState->InitJournalManager(p_pJournalManager);
}

void CCommandSetHomingMode_VCS_OperationMode::ResetJournalManager()
{
    if (m_pCommand_FindHome) m_pCommand_FindHome->ResetJournalManager();
    if (m_pCommand_GetHomingParameter) m_pCommand_GetHomingParameter->ResetJournalManager();
    if (m_pCommand_SetHomingParameter) m_pCommand_SetHomingParameter->ResetJournalManager();
    if (m_pCommand_StopHoming) m_pCommand_StopHoming->ResetJournalManager();
    if (m_pCommand_DefinePosition) m_pCommand_DefinePosition->ResetJournalManager();
    if (m_pCommand_GetHomingState) m_pCommand_GetHomingState->ResetJournalManager();
}

BOOL CCommandSetHomingMode_VCS_OperationMode::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if (oResult && m_pCommand_FindHome && !m_pCommand_FindHome->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_GetHomingParameter && !m_pCommand_GetHomingParameter->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_SetHomingParameter && !m_pCommand_SetHomingParameter->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_StopHoming && !m_pCommand_StopHoming->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_DefinePosition && !m_pCommand_DefinePosition->InitGateway(p_pGateway)) oResult = FALSE;
    if (oResult && m_pCommand_GetHomingState && !m_pCommand_GetHomingState->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
