
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/OperationMode/CommandSetInitialisation_VCS_OperationMode.h>

CCommandSetInitialisation_VCS_OperationMode::CCommandSetInitialisation_VCS_OperationMode()
{
    m_strCommandSetName = COMMAND_SET_OPERATION_MODE_INITIALISATION;

    m_pCommand_GetOperationMode = 0;
    m_pCommand_SetOperationMode = NULL;

    InitCommands();
}

CCommandSetInitialisation_VCS_OperationMode::~CCommandSetInitialisation_VCS_OperationMode()
{
    DeleteCommands();
}

BOOL CCommandSetInitialisation_VCS_OperationMode::VCS_GetOperationMode(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, __int8* p_pbMode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_GetOperationMode)
    {
        //Set parameter data
        m_pCommand_GetOperationMode->ResetStatus();

        //Execute command
        oResult = m_pCommand_GetOperationMode->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetOperationMode->GetReturnParameterData(0, p_pbMode, sizeof(*p_pbMode));

        //Get error code
        m_pCommand_GetOperationMode->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetInitialisation_VCS_OperationMode::VCS_SetOperationMode(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, __int8 p_bMode, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_SetOperationMode)
    {
        //Set Parameterdata
        m_pCommand_SetOperationMode->ResetStatus();
        m_pCommand_SetOperationMode->SetParameterData(0, &p_bMode, sizeof(p_bMode));

        //Execute command
        oResult = m_pCommand_SetOperationMode->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetOperationMode->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetInitialisation_VCS_OperationMode::InitCommands()
{
    DeleteCommands();

    //Init GetOperationMode
    m_pCommand_GetOperationMode = new CCommand_VCS_OperationMode();
    m_pCommand_GetOperationMode->InitCommand(MODE_GET_OPERATION_MODE);
    //Init SetOperationMode
    m_pCommand_SetOperationMode = new CCommand_VCS_OperationMode();
    m_pCommand_SetOperationMode->InitCommand(MODE_SET_OPERATION_MODE);
}

void CCommandSetInitialisation_VCS_OperationMode::DeleteCommands()
{
    if(m_pCommand_GetOperationMode)
    {
        delete m_pCommand_GetOperationMode;
        m_pCommand_GetOperationMode = NULL;
    }
    if(m_pCommand_SetOperationMode)
    {
        delete m_pCommand_SetOperationMode;
        m_pCommand_SetOperationMode = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetInitialisation_VCS_OperationMode::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetOperationMode) m_pCommand_GetOperationMode->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetOperationMode) m_pCommand_SetOperationMode->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetInitialisation_VCS_OperationMode::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetOperationMode) m_pCommand_GetOperationMode->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetOperationMode) m_pCommand_SetOperationMode->InitJournalManager(p_pJournalManager);
}

void CCommandSetInitialisation_VCS_OperationMode::ResetJournalManager()
{
    if(m_pCommand_GetOperationMode) m_pCommand_GetOperationMode->ResetJournalManager();
    if(m_pCommand_SetOperationMode) m_pCommand_SetOperationMode->ResetJournalManager();
}

BOOL CCommandSetInitialisation_VCS_OperationMode::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetOperationMode && !m_pCommand_GetOperationMode->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetOperationMode && !m_pCommand_SetOperationMode->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
