
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/OperationMode/CommandSetPositionModeAnalog_VCS_OperationMode.h>

CCommandSetPositionModeAnalog_VCS_OperationMode::CCommandSetPositionModeAnalog_VCS_OperationMode()
{
    m_strCommandSetName = COMMAND_SET_OPERATION_MODE_POSITION_MODE_ANALOG;

    m_pCommand_ActivateAnalogPositionSetpoint = NULL;
    m_pCommand_DeactivateAnalogPositionSetpoint = NULL;
    m_pCommand_EnableAnalogPositionSetpoint = NULL;
    m_pCommand_DisableAnalogPositionSetpoint = NULL;

    InitCommands();
}

CCommandSetPositionModeAnalog_VCS_OperationMode::~CCommandSetPositionModeAnalog_VCS_OperationMode()
{
    DeleteCommands();
}

BOOL CCommandSetPositionModeAnalog_VCS_OperationMode::VCS_ActivateAnalogPositionSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, float fScaling, short sOffset, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ActivateAnalogPositionSetpoint)
    {
        //Set parameter data
        m_pCommand_ActivateAnalogPositionSetpoint->ResetStatus();
        m_pCommand_ActivateAnalogPositionSetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));
        m_pCommand_ActivateAnalogPositionSetpoint->SetParameterData(1, &fScaling, sizeof(fScaling));
        m_pCommand_ActivateAnalogPositionSetpoint->SetParameterData(2, &sOffset, sizeof(sOffset));

        //Execute command
        oResult = m_pCommand_ActivateAnalogPositionSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ActivateAnalogPositionSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPositionModeAnalog_VCS_OperationMode::VCS_DeactivateAnalogPositionSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DeactivateAnalogPositionSetpoint)
    {
        //Set parameter data
        m_pCommand_DeactivateAnalogPositionSetpoint->ResetStatus();
        m_pCommand_DeactivateAnalogPositionSetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));

        //Execute command
        oResult = m_pCommand_DeactivateAnalogPositionSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DeactivateAnalogPositionSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPositionModeAnalog_VCS_OperationMode::VCS_EnableAnalogPositionSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_EnableAnalogPositionSetpoint)
    {
        //Set parameter data
        m_pCommand_EnableAnalogPositionSetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_EnableAnalogPositionSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_EnableAnalogPositionSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPositionModeAnalog_VCS_OperationMode::VCS_DisableAnalogPositionSetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DisableAnalogPositionSetpoint)
    {
        //Set parameter data
        m_pCommand_DisableAnalogPositionSetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_DisableAnalogPositionSetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DisableAnalogPositionSetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetPositionModeAnalog_VCS_OperationMode::InitCommands()
{
    DeleteCommands();

    //Init ActivateAnalogPositionSetpoint
    m_pCommand_ActivateAnalogPositionSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_ActivateAnalogPositionSetpoint->InitCommand(MODE_POM_ACTIVATE_ANALOG_POSITION_SETPOINT);
    //Init DeactivateAnalogPositionSetpoint
    m_pCommand_DeactivateAnalogPositionSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DeactivateAnalogPositionSetpoint->InitCommand(MODE_POM_DEACTIVATE_ANALOG_POSITION_SETPOINT);
    //Init EnableAnalogPositionSetpoint
    m_pCommand_EnableAnalogPositionSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_EnableAnalogPositionSetpoint->InitCommand(MODE_POM_ENABLE_ANALOG_POSITION_SETPOINT);
    //Init DisableAnalogPositionSetpoint
    m_pCommand_DisableAnalogPositionSetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DisableAnalogPositionSetpoint->InitCommand(MODE_POM_DISABLE_ANALOG_POSITION_SETPOINT);
}

void CCommandSetPositionModeAnalog_VCS_OperationMode::DeleteCommands()
{
    if(m_pCommand_ActivateAnalogPositionSetpoint)
    {
        delete m_pCommand_ActivateAnalogPositionSetpoint;
        m_pCommand_ActivateAnalogPositionSetpoint = NULL;
    }
    if(m_pCommand_DeactivateAnalogPositionSetpoint)
    {
        delete m_pCommand_DeactivateAnalogPositionSetpoint;
        m_pCommand_DeactivateAnalogPositionSetpoint = NULL;
    }
    if(m_pCommand_EnableAnalogPositionSetpoint)
    {
        delete m_pCommand_EnableAnalogPositionSetpoint;
        m_pCommand_EnableAnalogPositionSetpoint = NULL;
    }
    if(m_pCommand_DisableAnalogPositionSetpoint)
    {
        delete m_pCommand_DisableAnalogPositionSetpoint;
        m_pCommand_DisableAnalogPositionSetpoint = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetPositionModeAnalog_VCS_OperationMode::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_ActivateAnalogPositionSetpoint) m_pCommand_ActivateAnalogPositionSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DeactivateAnalogPositionSetpoint) m_pCommand_DeactivateAnalogPositionSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_EnableAnalogPositionSetpoint) m_pCommand_EnableAnalogPositionSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DisableAnalogPositionSetpoint) m_pCommand_DisableAnalogPositionSetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetPositionModeAnalog_VCS_OperationMode::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_ActivateAnalogPositionSetpoint) m_pCommand_ActivateAnalogPositionSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DeactivateAnalogPositionSetpoint) m_pCommand_DeactivateAnalogPositionSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_EnableAnalogPositionSetpoint) m_pCommand_EnableAnalogPositionSetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DisableAnalogPositionSetpoint) m_pCommand_DisableAnalogPositionSetpoint->InitJournalManager(p_pJournalManager);
}

void CCommandSetPositionModeAnalog_VCS_OperationMode::ResetJournalManager()
{
    if(m_pCommand_ActivateAnalogPositionSetpoint) m_pCommand_ActivateAnalogPositionSetpoint->ResetJournalManager();
    if(m_pCommand_DeactivateAnalogPositionSetpoint) m_pCommand_DeactivateAnalogPositionSetpoint->ResetJournalManager();
    if(m_pCommand_EnableAnalogPositionSetpoint) m_pCommand_EnableAnalogPositionSetpoint->ResetJournalManager();
    if(m_pCommand_DisableAnalogPositionSetpoint) m_pCommand_DisableAnalogPositionSetpoint->ResetJournalManager();
}

BOOL CCommandSetPositionModeAnalog_VCS_OperationMode::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_ActivateAnalogPositionSetpoint && !m_pCommand_ActivateAnalogPositionSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DeactivateAnalogPositionSetpoint && !m_pCommand_DeactivateAnalogPositionSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_EnableAnalogPositionSetpoint && !m_pCommand_EnableAnalogPositionSetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DisableAnalogPositionSetpoint && !m_pCommand_DisableAnalogPositionSetpoint->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
