
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/OperationMode/CommandSetVelocityModeAnalog_VCS_OperationMode.h>

CCommandSetVelocityModeAnalog_VCS_OperationMode::CCommandSetVelocityModeAnalog_VCS_OperationMode()
{
    m_strCommandSetName = COMMAND_SET_OPERATION_MODE_VELOCITY_MODE_ANALOG;

    m_pCommand_ActivateAnalogVelocitySetpoint = NULL;
    m_pCommand_DeactivateAnalogVelocitySetpoint = NULL;
    m_pCommand_EnableAnalogVelocitySetpoint = NULL;
    m_pCommand_DisableAnalogVelocitySetpoint = NULL;

    InitCommands();
}

CCommandSetVelocityModeAnalog_VCS_OperationMode::~CCommandSetVelocityModeAnalog_VCS_OperationMode()
{
    DeleteCommands();
}

BOOL CCommandSetVelocityModeAnalog_VCS_OperationMode::VCS_ActivateAnalogVelocitySetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, float fScaling, short sOffset, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_ActivateAnalogVelocitySetpoint)
    {
        //Set parameter data
        m_pCommand_ActivateAnalogVelocitySetpoint->ResetStatus();
        m_pCommand_ActivateAnalogVelocitySetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));
        m_pCommand_ActivateAnalogVelocitySetpoint->SetParameterData(1, &fScaling, sizeof(fScaling));
        m_pCommand_ActivateAnalogVelocitySetpoint->SetParameterData(2, &sOffset, sizeof(sOffset));

        //Execute command
        oResult = m_pCommand_ActivateAnalogVelocitySetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ActivateAnalogVelocitySetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetVelocityModeAnalog_VCS_OperationMode::VCS_DeactivateAnalogVelocitySetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usAnalogInputNumber, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DeactivateAnalogVelocitySetpoint)
    {
        //Set parameter data
        m_pCommand_DeactivateAnalogVelocitySetpoint->ResetStatus();
        m_pCommand_DeactivateAnalogVelocitySetpoint->SetParameterData(0, &p_usAnalogInputNumber, sizeof(p_usAnalogInputNumber));

        //Execute command
        oResult = m_pCommand_DeactivateAnalogVelocitySetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DeactivateAnalogVelocitySetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetVelocityModeAnalog_VCS_OperationMode::VCS_EnableAnalogVelocitySetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_EnableAnalogVelocitySetpoint)
    {
        //Set parameter data
        m_pCommand_EnableAnalogVelocitySetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_EnableAnalogVelocitySetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_EnableAnalogVelocitySetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetVelocityModeAnalog_VCS_OperationMode::VCS_DisableAnalogVelocitySetpoint(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    if(m_pCommand_DisableAnalogVelocitySetpoint)
    {
        //Set parameter data
        m_pCommand_DisableAnalogVelocitySetpoint->ResetStatus();

        //Execute command
        oResult = m_pCommand_DisableAnalogVelocitySetpoint->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_DisableAnalogVelocitySetpoint->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetVelocityModeAnalog_VCS_OperationMode::InitCommands()
{
    DeleteCommands();

    //Init ActivateAnalogVelocitySetpoint
    m_pCommand_ActivateAnalogVelocitySetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_ActivateAnalogVelocitySetpoint->InitCommand(MODE_VEM_ACTIVATE_ANALOG_VELOCITY_SETPOINT);
    //Init DeactivateAnalogVelocitySetpoint
    m_pCommand_DeactivateAnalogVelocitySetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DeactivateAnalogVelocitySetpoint->InitCommand(MODE_VEM_DEACTIVATE_ANALOG_VELOCITY_SETPOINT);
    //Init EnableAnalogVelocitySetpoint
    m_pCommand_EnableAnalogVelocitySetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_EnableAnalogVelocitySetpoint->InitCommand(MODE_VEM_ENABLE_ANALOG_VELOCITY_SETPOINT);
    //Init DisableAnalogVelocitySetpoint
    m_pCommand_DisableAnalogVelocitySetpoint = new CCommand_VCS_OperationMode();
    m_pCommand_DisableAnalogVelocitySetpoint->InitCommand(MODE_VEM_DISABLE_ANALOG_VELOCITY_SETPOINT);
}

void CCommandSetVelocityModeAnalog_VCS_OperationMode::DeleteCommands()
{
    if(m_pCommand_ActivateAnalogVelocitySetpoint)
    {
        delete m_pCommand_ActivateAnalogVelocitySetpoint;
        m_pCommand_ActivateAnalogVelocitySetpoint = NULL;
    }
    if(m_pCommand_DeactivateAnalogVelocitySetpoint)
    {
        delete m_pCommand_DeactivateAnalogVelocitySetpoint;
        m_pCommand_DeactivateAnalogVelocitySetpoint = NULL;
    }
    if(m_pCommand_EnableAnalogVelocitySetpoint)
    {
        delete m_pCommand_EnableAnalogVelocitySetpoint;
        m_pCommand_EnableAnalogVelocitySetpoint = NULL;
    }
    if(m_pCommand_DisableAnalogVelocitySetpoint)
    {
        delete m_pCommand_DisableAnalogVelocitySetpoint;
        m_pCommand_DisableAnalogVelocitySetpoint = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetVelocityModeAnalog_VCS_OperationMode::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pParentElement;

        //Command elements
        if(m_pCommand_ActivateAnalogVelocitySetpoint) m_pCommand_ActivateAnalogVelocitySetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DeactivateAnalogVelocitySetpoint) m_pCommand_DeactivateAnalogVelocitySetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_EnableAnalogVelocitySetpoint) m_pCommand_EnableAnalogVelocitySetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_DisableAnalogVelocitySetpoint) m_pCommand_DisableAnalogVelocitySetpoint->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetVelocityModeAnalog_VCS_OperationMode::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_ActivateAnalogVelocitySetpoint) m_pCommand_ActivateAnalogVelocitySetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DeactivateAnalogVelocitySetpoint) m_pCommand_DeactivateAnalogVelocitySetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_EnableAnalogVelocitySetpoint) m_pCommand_EnableAnalogVelocitySetpoint->InitJournalManager(p_pJournalManager);
    if(m_pCommand_DisableAnalogVelocitySetpoint) m_pCommand_DisableAnalogVelocitySetpoint->InitJournalManager(p_pJournalManager);
}

void CCommandSetVelocityModeAnalog_VCS_OperationMode::ResetJournalManager()
{
    if(m_pCommand_ActivateAnalogVelocitySetpoint) m_pCommand_ActivateAnalogVelocitySetpoint->ResetJournalManager();
    if(m_pCommand_DeactivateAnalogVelocitySetpoint) m_pCommand_DeactivateAnalogVelocitySetpoint->ResetJournalManager();
    if(m_pCommand_EnableAnalogVelocitySetpoint) m_pCommand_EnableAnalogVelocitySetpoint->ResetJournalManager();
    if(m_pCommand_DisableAnalogVelocitySetpoint) m_pCommand_DisableAnalogVelocitySetpoint->ResetJournalManager();
}

BOOL CCommandSetVelocityModeAnalog_VCS_OperationMode::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_ActivateAnalogVelocitySetpoint && !m_pCommand_ActivateAnalogVelocitySetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DeactivateAnalogVelocitySetpoint && !m_pCommand_DeactivateAnalogVelocitySetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_EnableAnalogVelocitySetpoint && !m_pCommand_EnableAnalogVelocitySetpoint->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_DisableAnalogVelocitySetpoint && !m_pCommand_DisableAnalogVelocitySetpoint->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
