
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Parameter.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Parameter/CommandSetPersistence_VCS_Parameter.h>

CCommandSetPersistence_VCS_Parameter::CCommandSetPersistence_VCS_Parameter()
{
    m_strCommandSetName = COMMAND_SET_PARAMETER_PERSISTENCE;

    m_pCommand_Store = 0;
    m_pCommand_Restore = 0;
    m_pCommand_RestoreFactory = 0;
    m_pCommand_Store_All = 0;
    m_pCommand_Restore_All = 0;

    InitCommands();
}

CCommandSetPersistence_VCS_Parameter::~CCommandSetPersistence_VCS_Parameter()
{
    DeleteCommands();
}

BOOL CCommandSetPersistence_VCS_Parameter::VCS_Store(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_Store)
    {
        //Set parameter data
        m_pCommand_Store->ResetStatus();

        //Execute command
        oResult = m_pCommand_Store->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_Store->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPersistence_VCS_Parameter::VCS_Restore(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_Restore)
    {
        //Set parameter data
        m_pCommand_Restore->ResetStatus();

        //Execute command
        oResult = m_pCommand_Restore->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_Restore->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPersistence_VCS_Parameter::VCS_RestoreFactory(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_RestoreFactory)
    {
        //Set parameter data
        m_pCommand_RestoreFactory->ResetStatus();

        //Execute command
        oResult = m_pCommand_RestoreFactory->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_RestoreFactory->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPersistence_VCS_Parameter::VCS_StoreAll(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_Store_All)
    {
        //Set parameter data
        m_pCommand_Store_All->ResetStatus();

        //Execute command
        oResult = m_pCommand_Store_All->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_Store_All->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetPersistence_VCS_Parameter::VCS_RestoreAll(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_Restore_All)
    {
        //Set parameter data
        m_pCommand_Restore_All->ResetStatus();

        //Execute command
        oResult = m_pCommand_Restore_All->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_Restore_All->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetPersistence_VCS_Parameter::InitCommands()
{
    DeleteCommands();

    //Init store
    m_pCommand_Store = new CCommand_VCS_Parameter();
    m_pCommand_Store->InitCommand(PARAMETER_STORE);
    //Init restore
    m_pCommand_Restore = new CCommand_VCS_Parameter();
    m_pCommand_Restore->InitCommand(PARAMETER_RESTORE);
    //Init restore factory
    m_pCommand_RestoreFactory = new CCommand_VCS_Parameter();
    m_pCommand_RestoreFactory->InitCommand(PARAMETER_RESTORE_FACTORY);
    //Init store all
    m_pCommand_Store_All = new CCommand_VCS_Parameter();
    m_pCommand_Store_All->InitCommand(PARAMETER_STORE_ALL);
    //Init restore all
    m_pCommand_Restore_All = new CCommand_VCS_Parameter();
    m_pCommand_Restore_All->InitCommand(PARAMETER_RESTORE_ALL);
}

void CCommandSetPersistence_VCS_Parameter::DeleteCommands()
{
    if(m_pCommand_Store)
    {
        delete m_pCommand_Store;
        m_pCommand_Store = 0;
    }
    if(m_pCommand_Restore)
    {
        delete m_pCommand_Restore;
        m_pCommand_Restore = 0;
    }
    if(m_pCommand_RestoreFactory)
    {
        delete m_pCommand_RestoreFactory;
        m_pCommand_RestoreFactory = 0;
    }
    if(m_pCommand_Store_All)
    {
        delete m_pCommand_Store_All;
        m_pCommand_Store_All = 0;
    }
    if(m_pCommand_Restore_All)
    {
        delete m_pCommand_Restore_All;
        m_pCommand_Restore_All = 0;
    }
}

CXXMLFile::CElementPart* CCommandSetPersistence_VCS_Parameter::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_Store) m_pCommand_Store->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_Restore) m_pCommand_Restore->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_RestoreFactory) m_pCommand_RestoreFactory->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_Store_All) m_pCommand_Store_All->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_Restore_All) m_pCommand_Restore_All->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetPersistence_VCS_Parameter::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_Store) m_pCommand_Store->InitJournalManager(p_pJournalManager);
    if(m_pCommand_Restore) m_pCommand_Restore->InitJournalManager(p_pJournalManager);
    if(m_pCommand_RestoreFactory) m_pCommand_RestoreFactory->InitJournalManager(p_pJournalManager);
    if(m_pCommand_Store_All) m_pCommand_Store_All->InitJournalManager(p_pJournalManager);
    if(m_pCommand_Restore_All) m_pCommand_Restore_All->InitJournalManager(p_pJournalManager);
}

void CCommandSetPersistence_VCS_Parameter::ResetJournalManager()
{
    if(m_pCommand_Store) m_pCommand_Store->ResetJournalManager();
    if(m_pCommand_Restore) m_pCommand_Restore->ResetJournalManager();
    if(m_pCommand_RestoreFactory) m_pCommand_RestoreFactory->ResetJournalManager();
    if(m_pCommand_Store_All) m_pCommand_Store_All->ResetJournalManager();
    if(m_pCommand_Restore_All) m_pCommand_Restore_All->ResetJournalManager();
}

BOOL CCommandSetPersistence_VCS_Parameter::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_Store && !m_pCommand_Store->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_Restore && !m_pCommand_Restore->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_RestoreFactory && !m_pCommand_RestoreFactory->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_Store_All && !m_pCommand_Store_All->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_Restore_All && !m_pCommand_Restore_All->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
