
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Parameter.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Parameter/CommandSetValues_VCS_Parameter.h>

CCommandSetValues_VCS_Parameter::CCommandSetValues_VCS_Parameter()
{
    m_strCommandSetName = COMMAND_SET_PARAMETER_VALUES;

    m_pCommand_GetValue = 0;
    m_pCommand_SetValue = 0;

    InitCommands();
}

CCommandSetValues_VCS_Parameter::~CCommandSetValues_VCS_Parameter()
{
    DeleteCommands();
}

BOOL CCommandSetValues_VCS_Parameter::VCS_GetValue(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, void* p_pData, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetValue)
    {
        //Set parameter data
        m_pCommand_GetValue->ResetStatus();
        m_pCommand_GetValue->SetParameterData(0, &p_usObjectIndex, sizeof(p_usObjectIndex));
        m_pCommand_GetValue->SetParameterData(1, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
        m_pCommand_GetValue->SetParameterData(2, &p_ulNbOfBytesToRead, sizeof(p_ulNbOfBytesToRead));

        //Execute command
        oResult = m_pCommand_GetValue->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetValue->GetReturnParameterData(0, p_pData, p_ulNbOfBytesToRead);
        m_pCommand_GetValue->GetReturnParameterData(1, p_pulNbOfBytesRead, sizeof(*p_pulNbOfBytesRead));

        //Get error code
        m_pCommand_GetValue->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetValues_VCS_Parameter::VCS_SetValue(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, void* p_pData, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_SetValue)
    {
        //Set parameter data
        m_pCommand_SetValue->ResetStatus();
        m_pCommand_SetValue->SetParameterData(0, &p_usObjectIndex, sizeof(p_usObjectIndex));
        m_pCommand_SetValue->SetParameterData(1, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
        m_pCommand_SetValue->SetParameterData(2, p_pData, p_ulNbOfBytesToWrite);
        m_pCommand_SetValue->SetParameterData(3, &p_ulNbOfBytesToWrite, sizeof(p_ulNbOfBytesToWrite));

        //Execute command
        oResult = m_pCommand_SetValue->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_SetValue->GetReturnParameterData(0, p_pulNbOfBytesWritten, sizeof(*p_pulNbOfBytesWritten));

        //Get error code
        m_pCommand_SetValue->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetValues_VCS_Parameter::InitCommands()
{
    DeleteCommands();

    m_pCommand_GetValue = new CCommand_VCS_Parameter();
    m_pCommand_GetValue->InitCommand(PARAMETER_GET_VALUE);

    m_pCommand_SetValue = new CCommand_VCS_Parameter();
    m_pCommand_SetValue->InitCommand(PARAMETER_SET_VALUE);
}

void CCommandSetValues_VCS_Parameter::DeleteCommands()
{
    if(m_pCommand_GetValue)
    {
        delete m_pCommand_GetValue;
        m_pCommand_GetValue = NULL;
    }
    if(m_pCommand_SetValue)
    {
        delete m_pCommand_SetValue;
        m_pCommand_SetValue = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetValues_VCS_Parameter::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetValue) m_pCommand_GetValue->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetValue) m_pCommand_SetValue->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetValues_VCS_Parameter::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetValue) m_pCommand_GetValue->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetValue) m_pCommand_SetValue->InitJournalManager(p_pJournalManager);
}

void CCommandSetValues_VCS_Parameter::ResetJournalManager()
{
    if(m_pCommand_GetValue) m_pCommand_GetValue->ResetJournalManager();
    if(m_pCommand_SetValue) m_pCommand_SetValue->ResetJournalManager();
}

BOOL CCommandSetValues_VCS_Parameter::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetValue && !m_pCommand_GetValue->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetValue && !m_pCommand_SetValue->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
