
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Scada/CommandSetInitialisation_VCS_Scada.h>

CCommandSetInitialisation_VCS_Scada::CCommandSetInitialisation_VCS_Scada()
{
    m_strCommandSetName = COMMAND_SET_SCADA_INITIALISATION;

    m_pCommand_GetBootupBehavior = 0;
    m_pCommand_SetBootupBehavior = 0;

    InitCommands();
}

CCommandSetInitialisation_VCS_Scada::~CCommandSetInitialisation_VCS_Scada()
{
    DeleteCommands();
}

BOOL CCommandSetInitialisation_VCS_Scada::VCS_SetBootupBehavior(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usProgramControl, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_SetBootupBehavior)
    {
        //Set parameter data
        m_pCommand_SetBootupBehavior->ResetStatus();
        m_pCommand_SetBootupBehavior->SetParameterData(0, &p_usProgramControl, sizeof(p_usProgramControl));

        //Execute command
        oResult = m_pCommand_SetBootupBehavior->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetBootupBehavior->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetInitialisation_VCS_Scada::VCS_GetBootupBehavior(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD* p_pusProgramControl, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetBootupBehavior)
    {
        //Execute command
        oResult = m_pCommand_GetBootupBehavior->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetBootupBehavior->GetReturnParameterData(0, p_pusProgramControl, sizeof(*p_pusProgramControl));

        //Get error code
        m_pCommand_GetBootupBehavior->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetInitialisation_VCS_Scada::InitCommands()
{
    DeleteCommands();

    //Init GetBootupBehavior
    m_pCommand_GetBootupBehavior = new CCommand_VCS_Scada();
    m_pCommand_GetBootupBehavior->InitCommand(SCADA_GET_BOOTUP_BEHAVIOUR);
    //Init SetBootupBehavior
    m_pCommand_SetBootupBehavior = new CCommand_VCS_Scada();
    m_pCommand_SetBootupBehavior->InitCommand(SCADA_SET_BOOTUP_BEHAVIOUR);
}

void CCommandSetInitialisation_VCS_Scada::DeleteCommands()
{
    if(m_pCommand_GetBootupBehavior)
    {
        delete m_pCommand_GetBootupBehavior;
        m_pCommand_GetBootupBehavior = NULL;
    }
    if(m_pCommand_SetBootupBehavior)
    {
        delete m_pCommand_SetBootupBehavior;
        m_pCommand_SetBootupBehavior = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetInitialisation_VCS_Scada::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetBootupBehavior) m_pCommand_GetBootupBehavior->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetBootupBehavior) m_pCommand_SetBootupBehavior->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetInitialisation_VCS_Scada::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetBootupBehavior) m_pCommand_GetBootupBehavior->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetBootupBehavior) m_pCommand_SetBootupBehavior->InitJournalManager(p_pJournalManager);
}

void CCommandSetInitialisation_VCS_Scada::ResetJournalManager()
{
    if(m_pCommand_GetBootupBehavior) m_pCommand_GetBootupBehavior->ResetJournalManager();
    if(m_pCommand_SetBootupBehavior) m_pCommand_SetBootupBehavior->ResetJournalManager();
}

BOOL CCommandSetInitialisation_VCS_Scada::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetBootupBehavior && !m_pCommand_GetBootupBehavior->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetBootupBehavior && !m_pCommand_SetBootupBehavior->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
