
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Scada/CommandSetProcessImage_VCS_Scada.h>

CCommandSetProcessImage_VCS_Scada::CCommandSetProcessImage_VCS_Scada()
{
    m_strCommandSetName = COMMAND_SET_SCADA_PROCESS_IMAGE;

    m_pCommand_GetProcessOutputImage = NULL;
    m_pCommand_SetProcessInputImage = NULL;

    InitCommands();
}

CCommandSetProcessImage_VCS_Scada::~CCommandSetProcessImage_VCS_Scada()
{
    DeleteCommands();
}

BOOL CCommandSetProcessImage_VCS_Scada::VCS_SetProcessInputImage(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulSelectionStart, DWORD p_ulSelectionLength, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_SetProcessInputImage)
    {
        //Set parameter data
        m_pCommand_SetProcessInputImage->ResetStatus();
        m_pCommand_SetProcessInputImage->SetParameterData(0, &p_ulSelectionStart, sizeof(p_ulSelectionStart));
        m_pCommand_SetProcessInputImage->SetParameterData(1, &p_ulSelectionLength, sizeof(p_ulSelectionLength));
        m_pCommand_SetProcessInputImage->SetParameterData(2, p_pDataBuffer, p_ulDataBufferLength);

        //Execute command
        oResult = m_pCommand_SetProcessInputImage->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetProcessInputImage->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProcessImage_VCS_Scada::VCS_GetProcessOutputImage(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, DWORD p_ulSelectionStart, DWORD p_ulSelectionLength, void* p_pDataBuffer, DWORD p_ulDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetProcessOutputImage)
    {
        //Set parameter data
        m_pCommand_GetProcessOutputImage->ResetStatus();
        m_pCommand_GetProcessOutputImage->SetParameterData(0, &p_ulSelectionStart, sizeof(p_ulSelectionStart));
        m_pCommand_GetProcessOutputImage->SetParameterData(1, &p_ulSelectionLength, sizeof(p_ulSelectionLength));

        //Execute command
        oResult = m_pCommand_GetProcessOutputImage->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetProcessOutputImage->GetReturnParameterData(0, p_pDataBuffer, p_ulDataBufferLength);

        //Get error code
        m_pCommand_GetProcessOutputImage->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetProcessImage_VCS_Scada::InitCommands()
{
    DeleteCommands();

    //Init GetProcessOutputImage
    m_pCommand_GetProcessOutputImage = new CCommand_VCS_Scada();
    m_pCommand_GetProcessOutputImage->InitCommand(SCADA_GET_PROCESS_OUTPUT_IMAGE);
    //Init SetProcessInputImage
    m_pCommand_SetProcessInputImage = new CCommand_VCS_Scada();
    m_pCommand_SetProcessInputImage->InitCommand(SCADA_SET_PROCESS_INPUT_IMAGE);
}

void CCommandSetProcessImage_VCS_Scada::DeleteCommands()
{
    if(m_pCommand_GetProcessOutputImage)
    {
        delete m_pCommand_GetProcessOutputImage;
        m_pCommand_GetProcessOutputImage = NULL;
    }
    if(m_pCommand_SetProcessInputImage)
    {
        delete m_pCommand_SetProcessInputImage;
        m_pCommand_SetProcessInputImage = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetProcessImage_VCS_Scada::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_SetProcessInputImage) m_pCommand_SetProcessInputImage->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetProcessOutputImage) m_pCommand_GetProcessOutputImage->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetProcessImage_VCS_Scada::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_SetProcessInputImage) m_pCommand_SetProcessInputImage->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetProcessOutputImage) m_pCommand_GetProcessOutputImage->InitJournalManager(p_pJournalManager);
}

void CCommandSetProcessImage_VCS_Scada::ResetJournalManager()
{
    if(m_pCommand_SetProcessInputImage) m_pCommand_SetProcessInputImage->ResetJournalManager();
    if(m_pCommand_GetProcessOutputImage) m_pCommand_GetProcessOutputImage->ResetJournalManager();
}

BOOL CCommandSetProcessImage_VCS_Scada::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_SetProcessInputImage && !m_pCommand_SetProcessInputImage->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetProcessOutputImage && !m_pCommand_GetProcessOutputImage->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
