
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Scada/CommandSetProcessInputOutput_VCS_Scada.h>

CCommandSetProcessInputOutput_VCS_Scada::CCommandSetProcessInputOutput_VCS_Scada()
{
    m_strCommandSetName = COMMAND_SET_SCADA_PROCESS_INPUT_OUTPUT;

    m_pCommand_GetProcessOutput = NULL;
    m_pCommand_SetProcessInput = NULL;
    m_pCommand_GetProcessOutputBit = NULL;
    m_pCommand_SetProcessInputBit = NULL;

    InitCommands();
}

CCommandSetProcessInputOutput_VCS_Scada::~CCommandSetProcessInputOutput_VCS_Scada()
{
    DeleteCommands();
}

BOOL CCommandSetProcessInputOutput_VCS_Scada::VCS_SetProcessInput(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usProcessInputType, BYTE p_ubElementNumber, void* p_pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_SetProcessInput)
    {
        //Set parameter data
        m_pCommand_SetProcessInput->ResetStatus();
        m_pCommand_SetProcessInput->SetParameterData(0, &p_usProcessInputType, sizeof(p_usProcessInputType));
        m_pCommand_SetProcessInput->SetParameterData(1, &p_ubElementNumber, sizeof(p_ubElementNumber));
        m_pCommand_SetProcessInput->SetParameterData(2, &p_ulNbOfBytesToWrite, sizeof(p_ulNbOfBytesToWrite));
        m_pCommand_SetProcessInput->SetParameterData(3, p_pDataBuffer, p_ulNbOfBytesToWrite);

        //Execute command
        oResult = m_pCommand_SetProcessInput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_SetProcessInput->GetReturnParameterData(0, p_pulNbOfBytesWritten, sizeof(*p_pulNbOfBytesWritten));

        //Get error code
        m_pCommand_SetProcessInput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProcessInputOutput_VCS_Scada::VCS_GetProcessOutput(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usProcessOutputType, BYTE p_ubElementNumber, void* p_pDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetProcessOutput)
    {
        //Set parameter data
        m_pCommand_GetProcessOutput->ResetStatus();
        m_pCommand_GetProcessOutput->SetParameterData(0, &p_usProcessOutputType, sizeof(p_usProcessOutputType));
        m_pCommand_GetProcessOutput->SetParameterData(1, &p_ubElementNumber, sizeof(p_ubElementNumber));
        m_pCommand_GetProcessOutput->SetParameterData(2, &p_ulNbOfBytesToRead, sizeof(p_ulNbOfBytesToRead));

        //Execute command
        oResult = m_pCommand_GetProcessOutput->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetProcessOutput->GetReturnParameterData(0, p_pulNbOfBytesRead, sizeof(*p_pulNbOfBytesRead));
        m_pCommand_GetProcessOutput->GetReturnParameterData(1, p_pDataBuffer, p_ulNbOfBytesToRead);

        //Get error code
        m_pCommand_GetProcessOutput->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProcessInputOutput_VCS_Scada::VCS_SetProcessInputBit(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usProcessInputType, BYTE p_ubElementNumber, BYTE p_ubBitNumber, BYTE p_ubBitState, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_SetProcessInputBit)
    {
        //Set parameter data
        m_pCommand_SetProcessInputBit->ResetStatus();
        m_pCommand_SetProcessInputBit->SetParameterData(0, &p_usProcessInputType, sizeof(p_usProcessInputType));
        m_pCommand_SetProcessInputBit->SetParameterData(1, &p_ubElementNumber, sizeof(p_ubElementNumber));
        m_pCommand_SetProcessInputBit->SetParameterData(2, &p_ubBitNumber, sizeof(p_ubBitNumber));
        m_pCommand_SetProcessInputBit->SetParameterData(2, &p_ubBitState, sizeof(p_ubBitState));

        //Execute command
        oResult = m_pCommand_SetProcessInputBit->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_SetProcessInputBit->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProcessInputOutput_VCS_Scada::VCS_GetProcessOutputBit(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, WORD p_usProcessOutputType, BYTE p_ubElementNumber, BYTE p_ubBitNumber, BYTE* p_pubBitState, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetProcessOutputBit)
    {
        //Set parameter data
        m_pCommand_GetProcessOutputBit->ResetStatus();
        m_pCommand_GetProcessOutputBit->SetParameterData(0, &p_usProcessOutputType, sizeof(p_usProcessOutputType));
        m_pCommand_GetProcessOutputBit->SetParameterData(1, &p_ubElementNumber, sizeof(p_ubElementNumber));
        m_pCommand_GetProcessOutputBit->SetParameterData(2, &p_ubBitNumber, sizeof(p_ubBitNumber));

        //Execute command
        oResult = m_pCommand_GetProcessOutputBit->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetProcessOutputBit->GetReturnParameterData(0, p_pubBitState, sizeof(*p_pubBitState));

        //Get error code
        m_pCommand_GetProcessOutputBit->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetProcessInputOutput_VCS_Scada::InitCommands()
{
    DeleteCommands();

    //Init GetProcessOutput
    m_pCommand_GetProcessOutput = new CCommand_VCS_Scada();
    m_pCommand_GetProcessOutput->InitCommand(SCADA_GET_PROCESS_OUTPUT);

    //Init SetProcessInput
    m_pCommand_SetProcessInput = new CCommand_VCS_Scada();
    m_pCommand_SetProcessInput->InitCommand(SCADA_SET_PROCESS_INPUT);

    //Init GetProcessOutputBit
    m_pCommand_GetProcessOutputBit = new CCommand_VCS_Scada();
    m_pCommand_GetProcessOutputBit->InitCommand(SCADA_GET_PROCESS_OUTPUT_BIT);

    //Init SetProcessInputBit
    m_pCommand_SetProcessInputBit = new CCommand_VCS_Scada();
    m_pCommand_SetProcessInputBit->InitCommand(SCADA_SET_PROCESS_INPUT_BIT);
}

void CCommandSetProcessInputOutput_VCS_Scada::DeleteCommands()
{
    if(m_pCommand_GetProcessOutput)
    {
        delete m_pCommand_GetProcessOutput;
        m_pCommand_GetProcessOutput = NULL;
    }
    if(m_pCommand_SetProcessInput)
    {
        delete m_pCommand_SetProcessInput;
        m_pCommand_SetProcessInput = NULL;
    }
    if(m_pCommand_GetProcessOutputBit)
    {
        delete m_pCommand_GetProcessOutputBit;
        m_pCommand_GetProcessOutputBit = NULL;
    }
    if(m_pCommand_SetProcessInputBit)
    {
        delete m_pCommand_SetProcessInputBit;
        m_pCommand_SetProcessInputBit = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetProcessInputOutput_VCS_Scada::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_SetProcessInput) m_pCommand_SetProcessInput->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetProcessOutput) m_pCommand_GetProcessOutput->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetProcessInputBit) m_pCommand_SetProcessInputBit->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetProcessOutputBit) m_pCommand_GetProcessOutputBit->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetProcessInputOutput_VCS_Scada::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_SetProcessInput) m_pCommand_SetProcessInput->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetProcessOutput) m_pCommand_GetProcessOutput->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetProcessInputBit) m_pCommand_SetProcessInputBit->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetProcessOutputBit) m_pCommand_GetProcessOutputBit->InitJournalManager(p_pJournalManager);
}

void CCommandSetProcessInputOutput_VCS_Scada::ResetJournalManager()
{
    if(m_pCommand_SetProcessInput) m_pCommand_SetProcessInput->ResetJournalManager();
    if(m_pCommand_GetProcessOutput) m_pCommand_GetProcessOutput->ResetJournalManager();
    if(m_pCommand_SetProcessInputBit) m_pCommand_SetProcessInputBit->ResetJournalManager();
    if(m_pCommand_GetProcessOutputBit) m_pCommand_GetProcessOutputBit->ResetJournalManager();
}

BOOL CCommandSetProcessInputOutput_VCS_Scada::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_SetProcessInput && !m_pCommand_SetProcessInput->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetProcessOutput && !m_pCommand_GetProcessOutput->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetProcessInputBit && !m_pCommand_SetProcessInputBit->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetProcessOutputBit && !m_pCommand_GetProcessOutputBit->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
