
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Scada/CommandSetProgramStatus_VCS_Scada.h>

CCommandSetProgramStatus_VCS_Scada::CCommandSetProgramStatus_VCS_Scada()
{
    m_strCommandSetName = COMMAND_SET_SCADA_PROGRAM_STATUS;

    m_pCommand_GetPlcStatus = NULL;
    m_pCommand_ColdstartPlc = NULL;
    m_pCommand_WarmstartPlc = NULL;
    m_pCommand_HotstartPlc = NULL;
    m_pCommand_StopPlc = NULL;

    InitCommands();
}

CCommandSetProgramStatus_VCS_Scada::~CCommandSetProgramStatus_VCS_Scada()
{
    DeleteCommands();
}

BOOL CCommandSetProgramStatus_VCS_Scada::VCS_GetPlcStatus(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BOOL* p_poIsProgramRunning, BOOL* p_poIsProgramAvailable, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_GetPlcStatus)
    {
        //Execute command
        oResult = m_pCommand_GetPlcStatus->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetPlcStatus->GetReturnParameterData(0, p_poIsProgramRunning, sizeof(*p_poIsProgramRunning));
        m_pCommand_GetPlcStatus->GetReturnParameterData(1, p_poIsProgramAvailable, sizeof(*p_poIsProgramAvailable));

        //Get error code
        m_pCommand_GetPlcStatus->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramStatus_VCS_Scada::VCS_ColdstartPlc(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_ColdstartPlc)
    {
        //Set parameter data
        m_pCommand_ColdstartPlc->ResetStatus();

        //Execute command
        oResult = m_pCommand_ColdstartPlc->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_ColdstartPlc->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramStatus_VCS_Scada::VCS_WarmstartPlc(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_WarmstartPlc)
    {
        //Set parameter data
        m_pCommand_WarmstartPlc->ResetStatus();

        //Execute command
        oResult = m_pCommand_WarmstartPlc->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_WarmstartPlc->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramStatus_VCS_Scada::VCS_HotstartPlc(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_HotstartPlc)
    {
        //Set parameter data
        m_pCommand_HotstartPlc->ResetStatus();

        //Execute command
        oResult = m_pCommand_HotstartPlc->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_HotstartPlc->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramStatus_VCS_Scada::VCS_StopPlc(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_StopPlc)
    {
        //Set parameter data
        m_pCommand_StopPlc->ResetStatus();

        //Execute command
        oResult = m_pCommand_StopPlc->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_StopPlc->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetProgramStatus_VCS_Scada::InitCommands()
{
    DeleteCommands();

    //Init GetPlcStatus
    m_pCommand_GetPlcStatus = new CCommand_VCS_Scada();
    m_pCommand_GetPlcStatus->InitCommand(SCADA_GET_PLC_STATUS);
    //Init ColdstartPlc
    m_pCommand_ColdstartPlc = new CCommand_VCS_Scada();
    m_pCommand_ColdstartPlc->InitCommand(SCADA_COLDSTART_PLC);
    //Init WarmstartPlc
    m_pCommand_WarmstartPlc = new CCommand_VCS_Scada();
    m_pCommand_WarmstartPlc->InitCommand(SCADA_WARMSTART_PLC);
    //Init HotstartPlc
    m_pCommand_HotstartPlc = new CCommand_VCS_Scada();
    m_pCommand_HotstartPlc->InitCommand(SCADA_HOTSTART_PLC);
    //Init StopPlc
    m_pCommand_StopPlc = new CCommand_VCS_Scada();
    m_pCommand_StopPlc->InitCommand(SCADA_STOP_PLC);
}

void CCommandSetProgramStatus_VCS_Scada::DeleteCommands()
{
    if(m_pCommand_GetPlcStatus)
    {
        delete m_pCommand_GetPlcStatus;
        m_pCommand_GetPlcStatus = NULL;
    }
    if(m_pCommand_ColdstartPlc)
    {
        delete m_pCommand_ColdstartPlc;
        m_pCommand_ColdstartPlc = NULL;
    }
    if(m_pCommand_WarmstartPlc)
    {
        delete m_pCommand_WarmstartPlc;
        m_pCommand_WarmstartPlc = NULL;
    }
    if(m_pCommand_HotstartPlc)
    {
        delete m_pCommand_HotstartPlc;
        m_pCommand_HotstartPlc = NULL;
    }
    if(m_pCommand_StopPlc)
    {
        delete m_pCommand_StopPlc;
        m_pCommand_StopPlc = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetProgramStatus_VCS_Scada::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_GetPlcStatus) m_pCommand_GetPlcStatus->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_ColdstartPlc) m_pCommand_ColdstartPlc->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_WarmstartPlc) m_pCommand_WarmstartPlc->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_HotstartPlc) m_pCommand_HotstartPlc->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_StopPlc) m_pCommand_StopPlc->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetProgramStatus_VCS_Scada::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_GetPlcStatus) m_pCommand_GetPlcStatus->InitJournalManager(p_pJournalManager);
    if(m_pCommand_ColdstartPlc) m_pCommand_ColdstartPlc->InitJournalManager(p_pJournalManager);
    if(m_pCommand_WarmstartPlc) m_pCommand_WarmstartPlc->InitJournalManager(p_pJournalManager);
    if(m_pCommand_HotstartPlc) m_pCommand_HotstartPlc->InitJournalManager(p_pJournalManager);
    if(m_pCommand_StopPlc) m_pCommand_StopPlc->InitJournalManager(p_pJournalManager);
}

void CCommandSetProgramStatus_VCS_Scada::ResetJournalManager()
{
    if(m_pCommand_GetPlcStatus) m_pCommand_GetPlcStatus->ResetJournalManager();
    if(m_pCommand_ColdstartPlc) m_pCommand_ColdstartPlc->ResetJournalManager();
    if(m_pCommand_WarmstartPlc) m_pCommand_WarmstartPlc->ResetJournalManager();
    if(m_pCommand_HotstartPlc) m_pCommand_HotstartPlc->ResetJournalManager();
    if(m_pCommand_StopPlc) m_pCommand_StopPlc->ResetJournalManager();
}

BOOL CCommandSetProgramStatus_VCS_Scada::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_GetPlcStatus && !m_pCommand_GetPlcStatus->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_ColdstartPlc && !m_pCommand_ColdstartPlc->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_WarmstartPlc && !m_pCommand_WarmstartPlc->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_HotstartPlc && !m_pCommand_HotstartPlc->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_StopPlc && !m_pCommand_StopPlc->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
