
#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scada.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandSets/Scada/CommandSetProgramVariable_VCS_Scada.h>

CCommandSetProgramVariable_VCS_Scada::CCommandSetProgramVariable_VCS_Scada()
{
    m_strCommandSetName = COMMAND_SET_SCADA_PROGRAM_VARIABLE;

    m_pCommand_InitVariableTable = 0;
    m_pCommand_GetVariable = NULL;
    m_pCommand_SetVariable = NULL;

    InitCommands();
}

CCommandSetProgramVariable_VCS_Scada::~CCommandSetProgramVariable_VCS_Scada()
{
    DeleteCommands();
}

BOOL CCommandSetProgramVariable_VCS_Scada::VCS_InitVariableTable(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, char* p_pProjectPathName, WORD p_usLength, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if(m_pCommand_InitVariableTable)
    {
        //Set parameter data
        m_pCommand_InitVariableTable->ResetStatus();
        m_pCommand_InitVariableTable->SetParameterData(0, &p_usLength, sizeof(p_usLength));
        m_pCommand_InitVariableTable->SetParameterData(1, p_pProjectPathName, p_usLength);

        //Execute command
        oResult = m_pCommand_InitVariableTable->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get error code
        m_pCommand_InitVariableTable->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramVariable_VCS_Scada::VCS_SetVariable(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CStdString p_VariableName, void* p_pDataBuffer, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulNbOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    DWORD ulNameLength(p_VariableName.GetLength());

    if(m_pCommand_SetVariable)
    {
        //Set parameter data
        m_pCommand_SetVariable->ResetStatus();
        m_pCommand_SetVariable->SetParameterData(0, &ulNameLength, sizeof(ulNameLength));
        m_pCommand_SetVariable->SetParameterData(1, &p_VariableName, ulNameLength);
        m_pCommand_SetVariable->SetParameterData(2, &p_ulNbOfBytesToWrite, sizeof(p_ulNbOfBytesToWrite));
        m_pCommand_SetVariable->SetParameterData(3, p_pDataBuffer, p_ulNbOfBytesToWrite);

        //Execute command
        oResult = m_pCommand_SetVariable->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_SetVariable->GetReturnParameterData(0, p_pulNbOfBytesWritten, sizeof(*p_pulNbOfBytesWritten));

        //Get error code
        m_pCommand_SetVariable->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CCommandSetProgramVariable_VCS_Scada::VCS_GetVariable(CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, CStdString p_VariableName, void* p_pDataBuffer, DWORD p_ulNbOfBytesToRead, DWORD* p_pulNbOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    DWORD ulNameLength(p_VariableName.GetLength());

    if(m_pCommand_GetVariable)
    {
        //Set parameter data
        m_pCommand_GetVariable->ResetStatus();
        m_pCommand_SetVariable->SetParameterData(0, &ulNameLength, sizeof(ulNameLength));
        m_pCommand_GetVariable->SetParameterData(1, &p_VariableName, ulNameLength);
        m_pCommand_GetVariable->SetParameterData(2, &p_ulNbOfBytesToRead, sizeof(p_ulNbOfBytesToRead));

        //Execute command
        oResult = m_pCommand_GetVariable->Execute(p_pManager, p_hHandle, p_hTransactionHandle);

        //Get return parameter data
        m_pCommand_GetVariable->GetReturnParameterData(0, p_pulNbOfBytesRead, sizeof(*p_pulNbOfBytesRead));
        m_pCommand_GetVariable->GetReturnParameterData(1, p_pDataBuffer, p_ulNbOfBytesToRead);

        //Get error code
        m_pCommand_GetVariable->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

void CCommandSetProgramVariable_VCS_Scada::InitCommands()
{
    DeleteCommands();

    m_pCommand_InitVariableTable = new CCommand_VCS_Scada();
    m_pCommand_InitVariableTable->InitCommand(SCADA_INIT_VARIABLE_TABLE);
    m_pCommand_GetVariable = new CCommand_VCS_Scada();
    m_pCommand_GetVariable->InitCommand(SCADA_GET_VARIABLE);
    m_pCommand_SetVariable = new CCommand_VCS_Scada();
    m_pCommand_SetVariable->InitCommand(SCADA_SET_VARIABLE);
}

void CCommandSetProgramVariable_VCS_Scada::DeleteCommands()
{
    if(m_pCommand_InitVariableTable)
    {
        delete m_pCommand_InitVariableTable;
        m_pCommand_InitVariableTable = NULL;
    }
    if(m_pCommand_GetVariable)
    {
        delete m_pCommand_GetVariable;
        m_pCommand_GetVariable = NULL;
    }
    if(m_pCommand_SetVariable)
    {
        delete m_pCommand_SetVariable;
        m_pCommand_SetVariable = NULL;
    }
}

CXXMLFile::CElementPart* CCommandSetProgramVariable_VCS_Scada::StoreToXMLFile(CXXMLFile* p_pFile, CXXMLFile::CElementPart* p_pParentElement)
{
    CXXMLFile::CElement* pElement(0);

    if(p_pFile && p_pParentElement)
    {
        //CommandSet elements
        pElement = (CXXMLFile::CElement*)p_pFile->AddElement(p_pParentElement);
        p_pFile->SetText(pElement, "CommandSet");
        pElement->SetAt("Name", m_strCommandSetName);

        //Command elements
        if(m_pCommand_InitVariableTable) m_pCommand_InitVariableTable->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_SetVariable) m_pCommand_SetVariable->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
        if(m_pCommand_GetVariable) m_pCommand_GetVariable->StoreToXMLFile(p_pFile, pElement, CHECK_NOT_VISIBILITY, CHECK_GATEWAY_SUPPORT);
    }

    return pElement;
}

void CCommandSetProgramVariable_VCS_Scada::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    if(m_pCommand_InitVariableTable) m_pCommand_InitVariableTable->InitJournalManager(p_pJournalManager);
    if(m_pCommand_SetVariable) m_pCommand_SetVariable->InitJournalManager(p_pJournalManager);
    if(m_pCommand_GetVariable) m_pCommand_GetVariable->InitJournalManager(p_pJournalManager);
}

void CCommandSetProgramVariable_VCS_Scada::ResetJournalManager()
{
    if(m_pCommand_InitVariableTable) m_pCommand_InitVariableTable->ResetJournalManager();
    if(m_pCommand_SetVariable) m_pCommand_SetVariable->ResetJournalManager();
    if(m_pCommand_GetVariable) m_pCommand_GetVariable->ResetJournalManager();
}

BOOL CCommandSetProgramVariable_VCS_Scada::InitGateway(CGateway* p_pGateway)
{
    BOOL oResult(TRUE);

    if(oResult && m_pCommand_InitVariableTable && !m_pCommand_InitVariableTable->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_SetVariable && !m_pCommand_SetVariable->InitGateway(p_pGateway)) oResult = FALSE;
    if(oResult && m_pCommand_GetVariable && !m_pCommand_GetVariable->InitGateway(p_pGateway)) oResult = FALSE;

    return oResult;
}
