#include "stdafx.h"
#include <memory.h>

#include <MmcTypeDefinition.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/BaseClasses/Cia402StateMachine.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>

CCia402StateMachine::CCia402StateMachine()
{
}

CCia402StateMachine::~CCia402StateMachine()
{
}

BOOL  CCia402StateMachine::IsDecelerationRelevantForTransition(WORD p_usTransition)
{
    BOOL oResult(FALSE);

    switch (p_usTransition)
    {
        case 8:
        case 9:
        case 5:
        case 11:
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}
    
BOOL  CCia402StateMachine::IsStandstillWindowRelevantForTransition(WORD p_usTransition)
{
    BOOL oResult(FALSE);

    switch (p_usTransition)
    {
        case 8:
        case 9:
        case 5:
        case 11:
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}
    
BOOL  CCia402StateMachine::IsBrakeRiseTimeRelevantForTransition(WORD p_usTransition)
{
    BOOL oResult(FALSE);

    switch (p_usTransition)
    {
        case 8:
        case 9:
        case 5:
        case 12:
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}
    
BOOL  CCia402StateMachine::IsBrakeFallTimeRelevantForTransition(WORD p_usTransition)
{
    BOOL oResult(FALSE);

    switch (p_usTransition)
    {
        case 4:
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}

void CCia402StateMachine::FindTransition(EDeviceState p_ActualState, EDeviceState p_TargetState, WORD* p_pusTransition)
{
    if (p_ActualState == DS_OPERATION_ENABLED)
    {
        FindTransitionFromOperationEnabledTo(p_TargetState, p_pusTransition);
    }
    else if(p_ActualState == DS_QUICK_STOP_ACTIV)
    {
        FindTransitionFromQuickStopTo(p_TargetState, p_pusTransition);
    }
    else if(p_ActualState == DS_READY_TO_SWITCH_ON)
    {
        FindTransitionFromReadyToSwitchOnTo(p_TargetState, p_pusTransition);
    }
    else if(p_ActualState == DS_SWITCHED_ON)
    {
        FindTransitionFromSwitchedOnTo(p_TargetState, p_pusTransition);
    }
}

void CCia402StateMachine::FindTransitionFromOperationEnabledTo(EDeviceState p_TargetState, WORD* p_pusTransition)
{
    switch (p_TargetState)
    {
        case DS_SWITCH_ON_DISABLED:
        {
            *p_pusTransition = 9;
            break;
        }
        case DS_READY_TO_SWITCH_ON:
        {
            *p_pusTransition = 8;
            break;
        }
        case DS_SWITCHED_ON:
        {
            *p_pusTransition = 5;
            break;
        }
        case DS_QUICK_STOP_ACTIV:
        {
            *p_pusTransition = 11;
            break;
        }
        default:
        {
            *p_pusTransition = 0;
        }
    }
}
   
void CCia402StateMachine::FindTransitionFromQuickStopTo(EDeviceState p_TargetState, WORD* p_pusTransition)
{
    switch (p_TargetState)
    {
        case DS_SWITCH_ON_DISABLED:
        {
            *p_pusTransition = 12;
            break;
        }
        case DS_OPERATION_ENABLED:
        {
            *p_pusTransition = 16;
            break;
        }
        default:
        {
            *p_pusTransition = 0;
        }
    }
}
    
void CCia402StateMachine::FindTransitionFromSwitchedOnTo(EDeviceState p_TargetState, WORD* p_pusTransition)
{
    switch (p_TargetState)
    {
        case DS_READY_TO_SWITCH_ON:
        {
            *p_pusTransition = 6;
            break;
        }
        case DS_SWITCH_ON_DISABLED:
        {
            *p_pusTransition = 10;
            break;
        }
        case DS_OPERATION_ENABLED:
        {
            *p_pusTransition = 4;
            break;
        }
        default:
        {
            *p_pusTransition = 0;
        }
    }
}

void CCia402StateMachine::FindTransitionFromReadyToSwitchOnTo(EDeviceState p_TargetState, WORD* p_pusTransition)
{
    switch (p_TargetState)
    {
        case DS_SWITCH_ON_DISABLED:
        {
            *p_pusTransition = 7;
            break;
        }
        case DS_SWITCHED_ON:
        {
            *p_pusTransition = 3;
            break;
        }
        case DS_OPERATION_ENABLED:
        {
            *p_pusTransition = 4;
            break;
        }
        default:
        {
            *p_pusTransition = 0;
        }
    }
}

void CCia402StateMachine::FindRelevantOptionCode(WORD p_usTransition, WORD* p_pusOptionCodeIndex, WORD* p_pusOptionCodeSubIndex)
{
    switch (p_usTransition)
    {
        case 8:
        case 9:
        {
            *p_pusOptionCodeIndex = INDEX_SHUTDOWN_OPTION_CODE;
            *p_pusOptionCodeSubIndex = SUBINDEX_SHUTDOWN_OPTION_CODE;
            break;
        }
        case 5:
        {
            *p_pusOptionCodeIndex = INDEX_DISABLE_OPERATION_OPTION_CODE;
            *p_pusOptionCodeSubIndex = SUBINDEX_DISABLE_OPERATION_OPTION_CODE;
            break;
        }
        case 11:
        {
            *p_pusOptionCodeIndex = INDEX_QUICK_STOP_OPTION_CODE;
            *p_pusOptionCodeSubIndex = SUBINDEX_QUICK_STOP_OPTION_CODE;
            break;
        }
    }
}

void CCia402StateMachine::FindRelevantDecelerationProfile(WORD p_usOptionCode, WORD* p_pusDecelerationIndex, WORD* p_pusDecelerationSubIndex)
{
    const WORD OPTION_CODE_SLOWDOWN_RAMP = 1;
    const WORD OPTION_CODE_QUICKSTOP_RAMP = 2;
    const WORD OPTION_CODE_QUICKSTOP_RAMP_AND_STAY_ENABLED = 6;

    switch (p_usOptionCode)
    {
        case OPTION_CODE_SLOWDOWN_RAMP:
        {
            *p_pusDecelerationIndex = INDEX_PROFILE_DECELERATION;
            *p_pusDecelerationSubIndex = SUBINDEX_PROFILE_DECELERATION;
        }
        case OPTION_CODE_QUICKSTOP_RAMP:
        case OPTION_CODE_QUICKSTOP_RAMP_AND_STAY_ENABLED:
        {
            *p_pusDecelerationIndex = INDEX_QUICK_STOP_DECELERATION;
            *p_pusDecelerationSubIndex = SUBINDEX_QUICK_STOP_DECELERATION;
        }
    }
}

