#include "stdafx.h"

#include <Log/MmcLog/MmcLog.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/CommandRoot.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/DataRecorder/RecorderChannelObjects.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/BaseClasses/GatewayVCStoDCS.h>

CGatewayVCStoDCS::CGatewayVCStoDCS() :
    m_pRecorderChannelObjects(0)
{
    m_eInputLayer = VIRTUAL_COMMAND_SET_LAYER;
    m_eOutputLayer = DEVICE_COMMAND_SET_LAYER;
}

CGatewayVCStoDCS::~CGatewayVCStoDCS()
{
    DeleteRecorderChannelObjects();
}

BOOL CGatewayVCStoDCS::InitGateway()
{
    return TRUE;
}

CGateway* CGatewayVCStoDCS::Clone()
{
    return NULL;
}

CGatewayVCStoDCS& CGatewayVCStoDCS::operator=(CGatewayVCStoDCS& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGateway*)this) = *((CGateway*)&p_rOther);
    }

    return *this;
}

BOOL CGatewayVCStoDCS::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    return FALSE;
}

WORD CGatewayVCStoDCS::GetNewBitMask(WORD p_usConfiguration, WORD p_usOldMask, BOOL p_oState)
{
    WORD usNewMask;

    if(p_oState)
    {//bit set to 1
        usNewMask = 1;

        for(int i = 0; i < p_usConfiguration; i++)
        {
            usNewMask <<= 1;
        }

        usNewMask |= p_usOldMask;
    }
    else
    {//bit set to 0
        usNewMask = 0xFFFF;

        for(int i = 0; i < p_usConfiguration; i++)
        {
            usNewMask <<= 1;
        }

        usNewMask--;
        usNewMask &= p_usOldMask;
    }

    return usNewMask;
}

BOOL CGatewayVCStoDCS::CopyBuffer(char* p_pBuffer, DWORD p_ulSize, CStdString* p_pValue)
{
    BOOL oResult(FALSE);

    if(p_pBuffer && p_pValue)
    {
        CStdString str(_T(""));
        char* pBuffer(p_pBuffer);

        for(DWORD i = 0; i < p_ulSize; i++)
        {
            str += *pBuffer++;
        }

        *p_pValue = str;

        oResult = TRUE;
    }

    return oResult;
}

BOOL CGatewayVCStoDCS::SetGatewayDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD p_ulGatewayTimeout)
{
    return FALSE;
}

CRecorderChannelObjects* CGatewayVCStoDCS::GetRecorderChannelObjects()
{
    return m_pRecorderChannelObjects;
}

BOOL CGatewayVCStoDCS::LogCmdMessageIn(CCommandRoot* p_pCommand, BOOL p_oLogMessageActive, CStdString p_label)
{
    BOOL oResult = FALSE;

    if(p_pCommand)
    {
        if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_VCS)) && p_oLogMessageActive)
        {
            CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_VCS), p_pCommand->GetLogMessage(p_label));

            p_pCommand->DeactivateLogMessage();
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CGatewayVCStoDCS::LogCmdMessageOut(CCommandRoot* p_pCommand, BOOL p_oLogMessageActive, CStdString p_label)
{
    BOOL oResult = FALSE;

    if(CMmcLog::IsActive(LMT_INFO, CCommunicationMessageLevel(CML_VCS)) && p_oLogMessageActive)
    {
        p_pCommand->ActivateLogMessage();

        CMmcLog::Log(LMT_INFO, CCommunicationMessageLevel(CML_VCS), p_pCommand->GetLogMessage(p_label, TRUE));

        oResult = TRUE;
    }

    return oResult;
}

void CGatewayVCStoDCS::CreateRecorderChannelObjects(WORD p_usMaxNumberOfRecorderVariables)
{
    DeleteRecorderChannelObjects();

    m_pRecorderChannelObjects = new CRecorderChannelObjects(p_usMaxNumberOfRecorderVariables);
}

void CGatewayVCStoDCS::DeleteRecorderChannelObjects()
{
    if (m_pRecorderChannelObjects)
    {
        delete m_pRecorderChannelObjects;
        m_pRecorderChannelObjects = 0;
    }
}
