#include "stdafx.h"
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/BaseClasses/GatewayVCStoEsam.h>

#include <memory.h>
#include <Process/MmcProcess.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>

#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Esam.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEsamDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenCommunicationDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>

CGatewayVCStoEsam::CGatewayVCStoEsam()
{
    m_pCommand_WriteObject = NULL;
    m_pCommand_InitiateSegmentedWrite = NULL;
    m_pCommand_SegmentedWrite = NULL;
    m_pCommand_ReadObject = NULL;
    m_pCommand_InitiateSegmentedRead = NULL;
    m_pCommand_SegmentedRead = NULL;
    m_pCommand_AbortSegmentedTransfer = NULL;
    InitCommands_ObjectDictionary();

    m_pCommand_RequestCANFrame = NULL;
    m_pCommand_SendCANFrame = NULL;
    m_pCommand_ReadCANFrame = NULL;
    InitCommands_GeneralGateway();

    m_pCommand_SendNMTService = NULL;
    InitCommands_NetworkManagement();

    m_pCommand_SendLSSFrame = NULL;
    m_pCommand_ReadLSSFrame = NULL;
    InitCommands_LayerSettingServices();
}

CGatewayVCStoEsam::~CGatewayVCStoEsam()
{
    DeleteCommands_ObjectDictionary();
    DeleteCommands_NetworkManagement();
    DeleteCommands_GeneralGateway();
    DeleteCommands_LayerSettingServices();
}

BOOL CGatewayVCStoEsam::ReadODObject(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pDataBuffer, DWORD dMaxNumberOfBytesToRead, DWORD* pdNumberOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    const WORD MAX_LENGTH_NON_SEGMENTED_TRANSFER = 4;

    BOOL oResult;
    BOOL oLastDataSegment;
    BYTE* pSegmentBuffer;
    DWORD segmentBufferLength;
    DWORD segmentLengthRead;
    DWORD dTotalNumberOfBytesRead;
    DWORD dTotalNumberOfBytesToRead;
    DWORD ulNumberOfBytesRead;
    DWORD ulAbortCode;

    if(pManager)
    {
        if(dMaxNumberOfBytesToRead <= MAX_LENGTH_NON_SEGMENTED_TRANSFER)
        {
            //Non Segmented Transfer
            if(pdNumberOfBytesRead) *pdNumberOfBytesRead = 0;

            if(DCS_ReadObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, pDataBuffer, dMaxNumberOfBytesToRead, &ulNumberOfBytesRead, p_pErrorInfo))
            {
                if(pdNumberOfBytesRead) *pdNumberOfBytesRead = ulNumberOfBytesRead;
                return TRUE;
            }

            return FALSE;
        }
        else
        {
            //Don't allow new segmented Transfer
            if(m_oAbortCommands)
            {
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER, p_pErrorInfo);
                return FALSE;
            }

            //Segmented Transfer
            if(pdNumberOfBytesRead) *pdNumberOfBytesRead = 0;
            if(DCS_InitiateSegmentedRead(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, &dTotalNumberOfBytesToRead, p_pErrorInfo))
            {
                //NumberOfBytesToRead
                if(dTotalNumberOfBytesToRead > dMaxNumberOfBytesToRead) dTotalNumberOfBytesToRead = dMaxNumberOfBytesToRead;

                //Initialize variables
                dTotalNumberOfBytesRead = 0;
                oLastDataSegment = FALSE;
                segmentBufferLength = dTotalNumberOfBytesToRead;
                pSegmentBuffer = (BYTE*)pDataBuffer;

                oResult = TRUE;
                while(oResult && !oLastDataSegment && (segmentBufferLength > 0))
                {
                    //Abort Transfer
                    if(m_oAbortCommands)
                    {
                        ulAbortCode = (DWORD)-1;
                        if(DCS_AbortSegmentedTransfer(pManager, hDCS_Handle, hTransactionHandle, uNodeId, wIndex, uSubIndex, ulAbortCode))
                        {
                            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER, p_pErrorInfo);
                            return FALSE;
                        }
                    }

                    //Do Transfer
                    oResult = DCS_SegmentedRead(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, &oLastDataSegment, pSegmentBuffer, segmentBufferLength, &segmentLengthRead, p_pErrorInfo);
                    if(oResult)
                    {
                        //Prepare next segment
                        dTotalNumberOfBytesRead += segmentLengthRead;
                        pSegmentBuffer += segmentLengthRead;
                        if(segmentLengthRead <= segmentBufferLength) segmentBufferLength -= segmentLengthRead; else segmentBufferLength = 0;
                    }
                }

                if(pdNumberOfBytesRead) *pdNumberOfBytesRead = dTotalNumberOfBytesRead;
                return oResult;
            }

            return FALSE;
        }
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::WriteODObject(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pDataBuffer, DWORD ulNumberOfBytesToWrite, DWORD* pdNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    const WORD MAX_LENGTH_NON_SEGMENTED_TRANSFER = 4;
    const WORD MAX_SEGMENT_LENGTH = 127;

    DWORD lTotalNumberOfBytesToWrite;
    DWORD lTotalNumberOfBytesWritten;
    BYTE* pSegmentBuffer;
    DWORD lSegmentLength;
    DWORD lSegmentLengthWritten;
    BOOL oResult;
    DWORD ulAbortCode;

    if(pManager)
    {
        if(ulNumberOfBytesToWrite <= MAX_LENGTH_NON_SEGMENTED_TRANSFER)
        {
            //Non Segmented Transfer
            if(pdNumberOfBytesWritten) *pdNumberOfBytesWritten = 0;
            if(DCS_WriteObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, pDataBuffer, ulNumberOfBytesToWrite, p_pErrorInfo))
            {
                if(pdNumberOfBytesWritten) *pdNumberOfBytesWritten = ulNumberOfBytesToWrite;
                return TRUE;
            }
            return FALSE;
        }
        else
        {
            //Don't allow new segmented transfer
            if(m_oAbortCommands)
            {
                if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER, p_pErrorInfo);
                return FALSE;
            }

            //Segmented Transfer
            if(pdNumberOfBytesWritten) *pdNumberOfBytesWritten = 0;
            if(DCS_InitiateSegmentedWrite(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, ulNumberOfBytesToWrite, p_pErrorInfo))
            {
                //Initialize variables
                lTotalNumberOfBytesWritten = 0;
                lTotalNumberOfBytesToWrite = ulNumberOfBytesToWrite;
                pSegmentBuffer = (BYTE*)pDataBuffer;
                if(ulNumberOfBytesToWrite < MAX_SEGMENT_LENGTH) lSegmentLength = ulNumberOfBytesToWrite; else lSegmentLength = MAX_SEGMENT_LENGTH;

                do
                {
                    //Abort Transfer
                    if(m_oAbortCommands)
                    {
                        ulAbortCode = (DWORD)-1;
                        if(DCS_AbortSegmentedTransfer(pManager, hDCS_Handle, hTransactionHandle, uNodeId, wIndex, uSubIndex, ulAbortCode))
                        {
                            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_COMMAND_ABORTED_BY_USER, p_pErrorInfo);
                            return FALSE;
                        }
                    }

                    //Do Transfer
                    oResult = DCS_SegmentedWrite(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, pSegmentBuffer, lSegmentLength, p_pErrorInfo);
                    if(oResult)
                    {
                        //No Feedback of written bytes
                        lSegmentLengthWritten = lSegmentLength;

                        //Prepare next segment
                        lTotalNumberOfBytesWritten += lSegmentLengthWritten;
                        ulNumberOfBytesToWrite -= (WORD)lSegmentLengthWritten;
                        pSegmentBuffer += lSegmentLengthWritten;
                        if(ulNumberOfBytesToWrite < lSegmentLengthWritten) lSegmentLength = ulNumberOfBytesToWrite; else lSegmentLength = lSegmentLengthWritten;
                        if(lSegmentLength > MAX_SEGMENT_LENGTH) lSegmentLength = MAX_SEGMENT_LENGTH;
                    }

                } while(oResult && (lTotalNumberOfBytesWritten < lTotalNumberOfBytesToWrite));

                if(pdNumberOfBytesWritten) *pdNumberOfBytesWritten = lTotalNumberOfBytesWritten;
                return oResult;
            }

            return FALSE;
        }
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadInt8Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, __int8* pbValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(pbValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)pbValue, sizeof(*pbValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*pbValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadInt16Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, short* piValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(piValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)piValue, sizeof(*piValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*piValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadInt32Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, int* plValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(plValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)plValue, sizeof(*plValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*plValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadUInt8Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* puValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(puValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)puValue, sizeof(*puValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*puValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadUInt16Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, WORD* pwValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(pwValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)pwValue, sizeof(*pwValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*pwValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadUInt32Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD* pdValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;

    if(pdValue)
    {
        if(!ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)pdValue, sizeof(*pdValue), &ulNumberOfBytesRead, p_pErrorInfo)) return FALSE;
        if(sizeof(*pdValue) != ulNumberOfBytesRead)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
            return FALSE;
        }

        return TRUE;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::ReadStringObject(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE ubPortNumber, BYTE ubNodeId, WORD usIndex, BYTE ubSubIndex, CStdString* pValue, CErrorInfo* p_pErrorInfo)
{
    const DWORD EXPEDITED_DATA_BUFFER_SIZE = 4;
    const DWORD NON_EXPEDITED_DATA_BUFFER_SIZE = 256;

    char expeditedDataBuffer[EXPEDITED_DATA_BUFFER_SIZE + 1];
    char nonExpeditedDataBuffer[NON_EXPEDITED_DATA_BUFFER_SIZE + 1];
    DWORD ulNumberOfBytesRead(0);
    BOOL oResult = FALSE;

    if(pValue)
    {
        //Init
        memset(expeditedDataBuffer, 0, EXPEDITED_DATA_BUFFER_SIZE + 1);
        memset(nonExpeditedDataBuffer, 0, NON_EXPEDITED_DATA_BUFFER_SIZE + 1);

        //Read Object
        if((ReadDataBufferObject(pManager, hDCS_Handle, hTransactionHandle, ubPortNumber, ubNodeId, usIndex, ubSubIndex, (void*)nonExpeditedDataBuffer, NON_EXPEDITED_DATA_BUFFER_SIZE, &ulNumberOfBytesRead, p_pErrorInfo) && (NON_EXPEDITED_DATA_BUFFER_SIZE >= ulNumberOfBytesRead)))
        {
            *pValue = nonExpeditedDataBuffer;
            oResult = TRUE;
        }
        else if((ReadDataBufferObject(pManager, hDCS_Handle, hTransactionHandle, ubPortNumber, ubNodeId, usIndex, ubSubIndex, (void*)expeditedDataBuffer, EXPEDITED_DATA_BUFFER_SIZE, &ulNumberOfBytesRead, p_pErrorInfo) && (EXPEDITED_DATA_BUFFER_SIZE >= ulNumberOfBytesRead)))
        {
            *pValue = expeditedDataBuffer;
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::ReadDataBufferObject(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, void* pData, DWORD ulNumberOfBytesToRead, DWORD* pdNumberOfBytesRead, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesRead;
    BOOL oResult;

    if(pData)
    {
        oResult = ReadODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)pData, ulNumberOfBytesToRead, &ulNumberOfBytesRead, p_pErrorInfo);

        if(pdNumberOfBytesRead) *pdNumberOfBytesRead = ulNumberOfBytesRead;
        return oResult;
    }

    if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
    return FALSE;
}

BOOL CGatewayVCStoEsam::WriteUInt8Object(CDeviceCommandSetManagerBase *pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE uValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, &uValue, sizeof(uValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(uValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteUInt16Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, WORD usValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)&usValue, sizeof(usValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(usValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteUInt32Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD ulValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)&ulValue, sizeof(ulValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(ulValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteInt8Object(CDeviceCommandSetManagerBase *pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, __int8 bValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)&bValue, sizeof(bValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(bValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteInt16Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, short iValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)&iValue, sizeof(iValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(iValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteInt32Object(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, int lValue, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;

    if(!WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)&lValue, sizeof(lValue), &ulNumberOfBytesWritten, p_pErrorInfo)) return FALSE;
    if(sizeof(lValue) != ulNumberOfBytesWritten)
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        return FALSE;
    }

    return TRUE;
}

BOOL CGatewayVCStoEsam::WriteDataBufferObject(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, void* pData, DWORD ulNumberOfBytesToWrite, DWORD* pdNumberOfBytesWritten, CErrorInfo* p_pErrorInfo)
{
    DWORD ulNumberOfBytesWritten;
    BOOL oResult = FALSE;

    if(pData)
    {
        oResult = WriteODObject(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, uNodeId, wIndex, uSubIndex, (BYTE*)pData, ulNumberOfBytesToWrite, &ulNumberOfBytesWritten, p_pErrorInfo);

        if(pdNumberOfBytesWritten) *pdNumberOfBytesWritten = ulNumberOfBytesWritten;
    }

    return oResult;
}

CGatewayVCStoEsam& CGatewayVCStoEsam::operator=(CGatewayVCStoEsam& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayVCStoDCS*)this) = *((CGatewayVCStoDCS*)&p_rOther);
    }

    return *this;
}

void CGatewayVCStoEsam::InitCommands_ObjectDictionary()
{
    DeleteCommands_ObjectDictionary();

    //WriteObject
    m_pCommand_WriteObject = new CCommand_DCS_Esam();
    m_pCommand_WriteObject->InitCommand(ESAM_WRITE_OBJECT);

    //ReadObject
    m_pCommand_ReadObject = new CCommand_DCS_Esam();
    m_pCommand_ReadObject->InitCommand(ESAM_READ_OBJECT);

    //InitiateSegmentedWrite
    m_pCommand_InitiateSegmentedWrite = new CCommand_DCS_Esam();
    m_pCommand_InitiateSegmentedWrite->InitCommand(ESAM_INITIATE_SEGMENT_WRITE);

    //InitiateSegmentedRead
    m_pCommand_InitiateSegmentedRead = new CCommand_DCS_Esam();
    m_pCommand_InitiateSegmentedRead->InitCommand(ESAM_INITIATE_SEGMENT_READ);

    //SegmentedWrite
    m_pCommand_SegmentedWrite = new CCommand_DCS_Esam();
    m_pCommand_SegmentedWrite->InitCommand(ESAM_SEGMENT_WRITE);

    //SegmentedRead
    m_pCommand_SegmentedRead = new CCommand_DCS_Esam();
    m_pCommand_SegmentedRead->InitCommand(ESAM_SEGMENT_READ);

    //AbortSegmentedTransfer
    m_pCommand_AbortSegmentedTransfer = new CCommand_DCS_Esam();
    m_pCommand_AbortSegmentedTransfer->InitCommand(ESAM_ABORT_SEGMENT_TRANSFER);
}

void CGatewayVCStoEsam::DeleteCommands_ObjectDictionary()
{
    //WriteObject
    if(m_pCommand_WriteObject)
    {
        delete m_pCommand_WriteObject;
        m_pCommand_WriteObject = NULL;
    }

    //InitiateSegmentedWrite
    if(m_pCommand_InitiateSegmentedWrite)
    {
        delete m_pCommand_InitiateSegmentedWrite;
        m_pCommand_InitiateSegmentedWrite = NULL;
    }

    //SegmentedWrite
    if(m_pCommand_SegmentedWrite)
    {
        delete m_pCommand_SegmentedWrite;
        m_pCommand_SegmentedWrite = NULL;
    }

    //ReadObject
    if(m_pCommand_ReadObject)
    {
        delete m_pCommand_ReadObject;
        m_pCommand_ReadObject = NULL;
    }

    //InitateSegmentedRead
    if(m_pCommand_InitiateSegmentedRead)
    {
        delete m_pCommand_InitiateSegmentedRead;
        m_pCommand_InitiateSegmentedRead = NULL;
    }

    //SegmentedRead
    if(m_pCommand_SegmentedRead)
    {
        delete m_pCommand_SegmentedRead;
        m_pCommand_SegmentedRead = NULL;
    }

    //AbortSegmentedTransfer
    if(m_pCommand_AbortSegmentedTransfer)
    {
        delete m_pCommand_AbortSegmentedTransfer;
        m_pCommand_AbortSegmentedTransfer = NULL;
    }
}

void CGatewayVCStoEsam::InitCommands_NetworkManagement()
{
    DeleteCommands_NetworkManagement();

    //Init Command SendNMTService
    m_pCommand_SendNMTService = new CCommand_DCS_Esam();
    m_pCommand_SendNMTService->InitCommand(ESAM_SEND_NMT_SERVICE);
}

void CGatewayVCStoEsam::DeleteCommands_NetworkManagement()
{
    if(m_pCommand_SendNMTService)
    {
        delete m_pCommand_SendNMTService;
        m_pCommand_SendNMTService = NULL;
    }
}

void CGatewayVCStoEsam::InitCommands_GeneralGateway()
{
    DeleteCommands_GeneralGateway();

    //Init Command Request
    m_pCommand_RequestCANFrame = new CCommand_DCS_Esam();
    m_pCommand_RequestCANFrame->InitCommand(ESAM_REQUEST_CAN_FRAME);

    //Init Command Send
    m_pCommand_SendCANFrame = new CCommand_DCS_Esam();
    m_pCommand_SendCANFrame->InitCommand(ESAM_SEND_CAN_FRAME);

    //Init Command Read
    m_pCommand_ReadCANFrame = new CCommand_DCS_Esam();
    m_pCommand_ReadCANFrame->InitCommand(ESAM_READ_CAN_FRAME);
}

void CGatewayVCStoEsam::InitCommands_LayerSettingServices()
{
    DeleteCommands_LayerSettingServices();

    //Init Command Send
    m_pCommand_SendLSSFrame = new CCommand_DCS_Esam();
    m_pCommand_SendLSSFrame->InitCommand(ESAM_SEND_LSS_FRAME);

    //Init Command Read
    m_pCommand_ReadLSSFrame = new CCommand_DCS_Esam();
    m_pCommand_ReadLSSFrame->InitCommand(ESAM_SEND_LSS_FRAME);
}

void CGatewayVCStoEsam::DeleteCommands_GeneralGateway()
{
    if(m_pCommand_RequestCANFrame)
    {
        delete m_pCommand_RequestCANFrame;
        m_pCommand_RequestCANFrame = NULL;
    }

    if(m_pCommand_SendCANFrame)
    {
        delete m_pCommand_SendCANFrame;
        m_pCommand_SendCANFrame = NULL;
    }

    if(m_pCommand_ReadCANFrame)
    {
        delete m_pCommand_ReadCANFrame;
        m_pCommand_ReadCANFrame = NULL;
    }
}

void CGatewayVCStoEsam::DeleteCommands_LayerSettingServices()
{
    if(m_pCommand_SendLSSFrame)
    {
        delete m_pCommand_SendLSSFrame;
        m_pCommand_SendLSSFrame = NULL;
    }

    if(m_pCommand_ReadLSSFrame)
    {
        delete m_pCommand_ReadLSSFrame;
        m_pCommand_ReadLSSFrame = NULL;
    }
}

BOOL CGatewayVCStoEsam::DCS_SendCANFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wCobID, WORD usLength, void* pData, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SendCANFrame && pManager)
    {
        //Set parameter data
        m_pCommand_SendCANFrame->ResetStatus();
        m_pCommand_SendCANFrame->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_SendCANFrame->SetParameterData(1, &wCobID, sizeof(wCobID));
        m_pCommand_SendCANFrame->SetParameterData(2, &usLength, sizeof(usLength));
        m_pCommand_SendCANFrame->SetParameterData(3, pData, usLength);

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_SendCANFrame, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SendCANFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_SendCANFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_ReadCANFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wCobID, WORD usLength, void* pData, DWORD p_ulTimeout, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadCANFrame && pManager)
    {
        //Set parameter data
        m_pCommand_ReadCANFrame->ResetStatus();
        m_pCommand_ReadCANFrame->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_ReadCANFrame->SetParameterData(1, &wCobID, sizeof(wCobID));
        m_pCommand_ReadCANFrame->SetParameterData(2, &usLength, sizeof(usLength));
        m_pCommand_ReadCANFrame->SetParameterData(3, &p_ulTimeout, sizeof(p_ulTimeout));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_ReadCANFrame, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_ReadCANFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_ReadCANFrame->GetReturnParameterData(1, pData, usLength);

        //Get error code
        m_pCommand_ReadCANFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_RequestCANFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wCobID, WORD usLength, void* pData, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_RequestCANFrame && pManager)
    {
        //Set parameter data
        m_pCommand_RequestCANFrame->ResetStatus();
        m_pCommand_RequestCANFrame->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_RequestCANFrame->SetParameterData(1, &wCobID, sizeof(wCobID));
        m_pCommand_RequestCANFrame->SetParameterData(2, &usLength, sizeof(usLength));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_RequestCANFrame, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_RequestCANFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_RequestCANFrame->GetReturnParameterData(1, pData, usLength);

        //Get error code
        m_pCommand_RequestCANFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_SendNMTService(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wNodeId, WORD wCommandSpecifier, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;
    DWORD ulDeviceErrorCode = 0;

    if(m_pCommand_SendNMTService && pManager)
    {
        //Set parameter data
        m_pCommand_SendNMTService->ResetStatus();
        m_pCommand_SendNMTService->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_SendNMTService->SetParameterData(1, &wNodeId, sizeof(wNodeId));
        m_pCommand_SendNMTService->SetParameterData(2, &wCommandSpecifier, sizeof(wCommandSpecifier));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_SendNMTService, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SendNMTService->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_SendNMTService->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_SendLSSFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_SendLSSFrame && pManager)
    {
        //Set parameter data
        m_pCommand_SendLSSFrame->ResetStatus();
        m_pCommand_SendLSSFrame->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_SendLSSFrame->SetParameterData(1, pDataBuffer, dDataBufferLength);

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_SendLSSFrame, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SendLSSFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_SendLSSFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_ReadLSSFrame(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wTimeout, void* pDataBuffer, DWORD dDataBufferLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode = 0;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadLSSFrame && pManager)
    {
        //Set parameter data
        m_pCommand_ReadLSSFrame->ResetStatus();
        m_pCommand_ReadLSSFrame->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_ReadLSSFrame->SetParameterData(1, &wTimeout, sizeof(wTimeout));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_ReadLSSFrame, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_ReadLSSFrame->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_ReadLSSFrame->GetReturnParameterData(1, pDataBuffer, dDataBufferLength);

        //Get error code
        m_pCommand_ReadLSSFrame->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_InitiateSegmentedRead(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD* pdObjectLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedRead && pManager)
    {
        //Set parameter data
        m_pCommand_InitiateSegmentedRead->ResetStatus();
        m_pCommand_InitiateSegmentedRead->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_InitiateSegmentedRead->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedRead->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_InitiateSegmentedRead->SetParameterData(3, &uNodeId, sizeof(uNodeId));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_InitiateSegmentedRead, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_InitiateSegmentedRead->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_InitiateSegmentedRead->GetReturnParameterData(1, pdObjectLength, sizeof(*pdObjectLength));

        //Get error code
        m_pCommand_InitiateSegmentedRead->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_InitiateSegmentedWrite(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, DWORD dObjectLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    BOOL oResult = FALSE;

    if(m_pCommand_InitiateSegmentedWrite && pManager)
    {
        //Set parameter data
        m_pCommand_InitiateSegmentedWrite->ResetStatus();
        m_pCommand_InitiateSegmentedWrite->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(3, &uNodeId, sizeof(uNodeId));
        m_pCommand_InitiateSegmentedWrite->SetParameterData(4, &dObjectLength, sizeof(dObjectLength));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_InitiateSegmentedWrite, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_InitiateSegmentedWrite->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_InitiateSegmentedWrite->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_ReadObject(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pObjectBuffer, DWORD dObjectBufferLength, DWORD* pdObjectLengthRead, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    DWORD dObjectLength;
    BOOL oResult = FALSE;

    if(m_pCommand_ReadObject && pManager)
    {
        //Set parameter data
        m_pCommand_ReadObject->ResetStatus();
        m_pCommand_ReadObject->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_ReadObject->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_ReadObject->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_ReadObject->SetParameterData(3, &uNodeId, sizeof(uNodeId));
        m_pCommand_ReadObject->SetParameterData(4, &dObjectLength, sizeof(dObjectLength));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_ReadObject, hHandle, hTransactionHandle);

        //Get ReturnParameter Length
        dObjectLength = m_pCommand_ReadObject->GetReturnParameterLength(1);
        if(dObjectBufferLength < dObjectLength) dObjectLength = dObjectBufferLength;
        if(pdObjectLengthRead) *pdObjectLengthRead = dObjectLength;

        //Get return parameter data
        m_pCommand_ReadObject->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_ReadObject->GetReturnParameterData(1, pObjectBuffer, dObjectLength);

        //Get error code
        m_pCommand_ReadObject->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_SegmentedRead(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BOOL* poLastDataSegment, BYTE* pSegmentBuffer, DWORD dSegmentBufferLength, DWORD* pdSegmentLengthRead, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    UEsamControlByte controlByte;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedRead && pManager)
    {
        //Set parameter data
        m_pCommand_SegmentedRead->ResetStatus();
        m_pCommand_SegmentedRead->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_SegmentedRead, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SegmentedRead->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
        m_pCommand_SegmentedRead->GetReturnParameterData(1, &controlByte, sizeof(controlByte));
        m_pCommand_SegmentedRead->GetReturnParameterData(2, pSegmentBuffer, dSegmentBufferLength);

        //Get error code
        m_pCommand_SegmentedRead->GetErrorInfo(p_pErrorInfo);

        //Restore ControlByte
        if(pdSegmentLengthRead) *pdSegmentLengthRead = controlByte.structure.ubNbOfDataBytes;
        if(poLastDataSegment) *poLastDataSegment = controlByte.structure.ubLastDataSegment;
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_SegmentedWrite(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE* pSegmentBuffer, DWORD dSegmentLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    UEsamControlByte controlByte;
    BOOL oResult = FALSE;

    if(m_pCommand_SegmentedWrite && pManager)
    {
        //Prepare ControlByte
        controlByte.ubValue = 0;
        controlByte.structure.ubNbOfDataBytes = dSegmentLength;

        //Set parameter data
        m_pCommand_SegmentedWrite->ResetStatus();
        m_pCommand_SegmentedWrite->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_SegmentedWrite->SetParameterData(1, &controlByte, sizeof(controlByte));
        m_pCommand_SegmentedWrite->SetParameterData(2, pSegmentBuffer, dSegmentLength);

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_SegmentedWrite, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_SegmentedWrite->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_SegmentedWrite->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_WriteObject(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, BYTE uPortNumber, BYTE uNodeId, WORD wIndex, BYTE uSubIndex, BYTE* pObjectBuffer, DWORD dObjectLength, CErrorInfo* p_pErrorInfo)
{
    DWORD ulDeviceErrorCode;
    BOOL oResult = FALSE;

    if(m_pCommand_WriteObject && pManager)
    {
        //Set parameter data
        m_pCommand_WriteObject->ResetStatus();
        m_pCommand_WriteObject->SetParameterData(0, &uPortNumber, sizeof(uPortNumber));
        m_pCommand_WriteObject->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_WriteObject->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_WriteObject->SetParameterData(3, &uNodeId, sizeof(uNodeId));
        m_pCommand_WriteObject->SetParameterData(4, pObjectBuffer, dObjectLength);

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_WriteObject, hHandle, hTransactionHandle);

        //Get return parameter data
        m_pCommand_WriteObject->GetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));

        //Get error code
        m_pCommand_WriteObject->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::DCS_AbortSegmentedTransfer(CLayerManagerBase* pManager, HANDLE hHandle, HANDLE hTransactionHandle, WORD wNodeId, WORD wIndex, WORD wSubIndex, DWORD ulAbortCode, CErrorInfo* p_pErrorInfo)
{
    BYTE uSubIndex = (BYTE)wSubIndex;
    BOOL oResult = FALSE;

    if(m_pCommand_AbortSegmentedTransfer)
    {
        //Set parameter data
        m_pCommand_AbortSegmentedTransfer->ResetStatus();
        m_pCommand_AbortSegmentedTransfer->SetParameterData(0, &wNodeId, sizeof(wNodeId));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(1, &wIndex, sizeof(wIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(2, &uSubIndex, sizeof(uSubIndex));
        m_pCommand_AbortSegmentedTransfer->SetParameterData(3, &ulAbortCode, sizeof(ulAbortCode));

        //Execute command
        oResult = pManager->ExecuteCommand(m_pCommand_AbortSegmentedTransfer, hHandle, hTransactionHandle);

        //Get error code
        m_pCommand_AbortSegmentedTransfer->GetErrorInfo(p_pErrorInfo);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::SetDisableState(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wNodeId, CErrorInfo* p_pErrorInfo)
{
    WORD usIndex = 0;
    BYTE ubSubIndex = 0;
    WORD usDeviceState = 0;
    BOOL oResult = FALSE;

    //Read Device State
    oResult = ReadDeviceState(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, wNodeId, &usDeviceState, p_pErrorInfo);
    if(oResult)
    {
        usIndex = INDEX_CONTROL_WORD;
        ubSubIndex = SUBINDEX_CONTROL_WORD;
        switch(usDeviceState)
        {
            case DS_FAULT:
            case DS_SWITCH_ON_DISABLED: oResult = TRUE; break;
            case DS_OPERATION_ENABLED:
            case DS_QUICK_STOP_ACTIV:
            case DS_SWITCHED_ON:
            case DS_READY_TO_SWITCH_ON:
                if(oResult)
                    oResult = WriteUInt16Object(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, (BYTE)wNodeId, usIndex, ubSubIndex, DISABLE_VOLTAGE, p_pErrorInfo);
                if(oResult)
                    oResult = CheckDeviceState(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, wNodeId, DS_SWITCH_ON_DISABLED, p_pErrorInfo);
                break;
            default:
                oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::ReadDeviceState(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wNodeId, WORD* pwDeviceState, CErrorInfo* p_pErrorInfo)
{
    const WORD STATE_BIT5_MASK = 0x0020;
    const WORD STATE5_BIT_MASK = 0x004F;
    const WORD STATE6_BIT_MASK = 0x006F;

    WORD wIndex;
    BYTE ubSubIndex;
    WORD wDeviceState;

    //Read Device State
    wIndex = INDEX_STATUS_WORD;
    ubSubIndex = SUBINDEX_STATUS_WORD;
    if(!ReadUInt16Object(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, (BYTE)wNodeId, wIndex, ubSubIndex, &wDeviceState, p_pErrorInfo))
    {
        return FALSE;
    }
    switch(wDeviceState & STATE5_BIT_MASK)
    {
        case DS_NOT_READY_TO_SWITCH_ON:
        case DS_SWITCH_ON_DISABLED:
        case DS_FAULT_REACTION_ACTIV:
        case DS_FAULT:
            {
                wDeviceState &= ~STATE_BIT5_MASK;
            }; break;
    }
    wDeviceState &= STATE6_BIT_MASK;
    if(pwDeviceState) *pwDeviceState = wDeviceState;
    return TRUE;
}

BOOL CGatewayVCStoEsam::CheckDeviceState(CDeviceCommandSetManagerBase* pManager, HANDLE hDCS_Handle, HANDLE hTransactionHandle, BYTE uPortNumber, WORD wNodeId, WORD wState, CErrorInfo* p_pErrorInfo)
{
    DWORD ulTime = 0;
    DWORD dTimeoutDeviceState = 1000;
    DWORD dStart = MmcGetTickCount();
    WORD wDeviceState = 0;

    while((ulTime < dTimeoutDeviceState) && (wDeviceState != wState))
    {
        if(!ReadDeviceState(pManager, hDCS_Handle, hTransactionHandle, uPortNumber, wNodeId, &wDeviceState, p_pErrorInfo)) return FALSE;

        ulTime = MmcGetTickCount()-dStart;

        if(ulTime > dTimeoutDeviceState)
        {
            if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_TIMEOUT_ELAPSED, p_pErrorInfo);
            return FALSE;
        }
    }
    return TRUE;
}

BOOL CGatewayVCStoEsam::InitLayerParameterStack(CCommandRoot* p_pCommand)
{
    CLayerParameterStack layerParameterStack;
    CLayerParameterSet layerParameterSet;
    BOOL oResult = FALSE;

    if(p_pCommand)
    {
        if(p_pCommand->GetLayerParameterStack(layerParameterStack))
        {
            //Pop VCS Layer Parameter Set
            layerParameterStack.PopLayer(VIRTUAL_COMMAND_SET_LAYER, layerParameterSet);

            //Set DCS Layer Commands
            if(m_pCommand_WriteObject) m_pCommand_WriteObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedWrite) m_pCommand_InitiateSegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedWrite) m_pCommand_SegmentedWrite->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadObject) m_pCommand_ReadObject->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_InitiateSegmentedRead) m_pCommand_InitiateSegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SegmentedRead) m_pCommand_SegmentedRead->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_AbortSegmentedTransfer) m_pCommand_AbortSegmentedTransfer->SetLayerParameterStack(layerParameterStack);

            if(m_pCommand_SendCANFrame) m_pCommand_SendCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_RequestCANFrame) m_pCommand_RequestCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadCANFrame) m_pCommand_ReadCANFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_SendNMTService) m_pCommand_SendNMTService->SetLayerParameterStack(layerParameterStack);

            if(m_pCommand_SendLSSFrame) m_pCommand_SendLSSFrame->SetLayerParameterStack(layerParameterStack);
            if(m_pCommand_ReadLSSFrame) m_pCommand_ReadLSSFrame->SetLayerParameterStack(layerParameterStack);

            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::StoreParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubPortNumber, BYTE p_ubNodeId, CErrorInfo* p_pErrorInfo)
{
    const DWORD TIMEOUT = 5000;

    DWORD ulPassword = 0;
    DWORD ulTransferRate = 0;
    DWORD ulBackupTimeout = 0;
    BOOL oResult = FALSE;

    if(p_pManager)
    {
        //Init
        oResult = TRUE;

        //Backup Timeout
        p_pManager->DCS_GetProtocolStackSettings(p_hHandle, &ulTransferRate, &ulBackupTimeout);

        //Increase timeout
        p_pManager->DCS_SetProtocolStackSettings(p_hHandle, ulTransferRate, TIMEOUT, TRUE);

        //Write save all parameters
        ulPassword = 'e';
        ulPassword *= 256;
        ulPassword += 'v';
        ulPassword *= 256;
        ulPassword += 'a';
        ulPassword *= 256;
        ulPassword += 's';

        if(oResult) oResult = WriteUInt32Object(p_pManager, p_hHandle, p_hTransactionHandle, p_ubPortNumber, p_ubNodeId, INDEX_STORE_PARAMETERS, SUBINDEX_SAVE_ALL_PARAMETERS, ulPassword, p_pErrorInfo);

        //Restore timeout
        p_pManager->DCS_SetProtocolStackSettings(p_hHandle, ulTransferRate, ulBackupTimeout, TRUE);
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::ResetDevice(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubPortNumber, BYTE p_ubNodeId, BOOL p_oWaitForReboot, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    //SendNMTService
    if(DCS_SendNMTService(p_pManager, p_hHandle, p_hTransactionHandle, p_ubPortNumber, p_ubNodeId, (WORD)NSI_RESET_NODE, p_pErrorInfo))
    {
        //Init
        oResult = TRUE;

        if(p_oWaitForReboot)
        {
            //WaitForReboot
            WaitForReboot(p_pManager, p_hHandle, 0, p_ubPortNumber, p_ubNodeId, 0);
        }
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::WaitForReboot(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubPortNumber, BYTE p_ubNodeId, CErrorInfo* p_pErrorInfo)
{
    const DWORD REBOOT_TIMEOUT = 8000;

    BYTE uErrorRegister = 0;
    DWORD ulStartTime = 0;
    DWORD ulTime = 0;
    DWORD ulSleepTime = 10;
    BOOL oResult = FALSE;

    //Start Time
    ulStartTime = MmcGetTickCount();
    ulTime = 0;

    while(ulTime < REBOOT_TIMEOUT)
    {
        //Reset
        if(p_pErrorInfo) p_pErrorInfo->Reset();

        //Check Communication
        if(ReadUInt8Object(p_pManager, p_hHandle, p_hTransactionHandle, p_ubPortNumber, p_ubNodeId, INDEX_ERROR_REGISTER, SUBINDEX_ERROR_REGISTER, &uErrorRegister, p_pErrorInfo))
        {
            oResult = TRUE;
            break;
        }
        else
        {
            //Update Time
            ulTime = MmcGetTickCount()-ulStartTime;
            MmcSleep(ulSleepTime);
        }
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::GetAddressParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rPortNumber, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);

    //NodeId
    if(oResult && !GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, p_rNodeId, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    //PortNumber
    if(oResult && !GetPortNumber(p_pCommand, p_pManager, p_hDCS_Handle, p_rPortNumber, p_pErrorInfo))
    {
        if(p_rNodeId == 0) p_rPortNumber = 0; else p_rPortNumber = 1;
    }

    return oResult;
}

BOOL CGatewayVCStoEsam::GetPortNumber(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rPortNumber, CErrorInfo* p_pErrorInfo)
{
    const CStdString PORT_NUMBER = _T("PortNumber");

    BOOL oResult = FALSE;

    //Get port number from command
    if(!oResult && p_pCommand) oResult = p_pCommand->GetLayerParameter(DEVICE_COMMAND_SET_LAYER, PORT_NUMBER, (BYTE*)&p_rPortNumber, sizeof(p_rPortNumber), p_pErrorInfo);

    //Get port number from device
    if(!oResult && p_pManager) oResult = p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, PORT_NUMBER, (BYTE*)&p_rPortNumber, sizeof(p_rPortNumber), p_pErrorInfo);

    return oResult;
}

BOOL CGatewayVCStoEsam::GetNodeId(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rNodeId, CErrorInfo* p_pErrorInfo)
{
    const CStdString NODE_ID = _T("NodeId");

    BOOL oResult = FALSE;

    //Get node id from command
    if(!oResult && p_pCommand) oResult = p_pCommand->GetLayerParameter(DEVICE_COMMAND_SET_LAYER, NODE_ID, (BYTE*)&p_rNodeId, sizeof(p_rNodeId), p_pErrorInfo);

    //Get node id from Device
    if(!oResult && p_pManager) oResult = p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_rNodeId, sizeof(p_rNodeId), p_pErrorInfo);

    return oResult;
}

BOOL CGatewayVCStoEsam::RestoreParameter(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle, BYTE p_ubPortNumber, BYTE p_ubNodeId, CErrorInfo* p_pErrorInfo)
{
    const DWORD TIMEOUT = 5000;

    DWORD ulPassword = 0;
    DWORD ulTransferRate = 0;
    DWORD ulBackupTimeout = 0;
    BOOL oResult = FALSE;

    if(p_pManager)
    {
        //Init
        oResult = TRUE;

        //Backup Timeout
        p_pManager->DCS_GetProtocolStackSettings(p_hHandle, &ulTransferRate, &ulBackupTimeout);

        //Increase timeout
        p_pManager->DCS_SetProtocolStackSettings(p_hHandle, ulTransferRate, TIMEOUT, TRUE);

        //Write load all parameters
        ulPassword = 'd';
        ulPassword *= 256;
        ulPassword += 'a';
        ulPassword *= 256;
        ulPassword += 'o';
        ulPassword *= 256;
        ulPassword += 'l';

        if(oResult)
        {
            oResult = WriteUInt32Object(p_pManager, p_hHandle, p_hTransactionHandle, p_ubPortNumber, p_ubNodeId, INDEX_RESTORE_DEFAULT_PARAMETERS, SUBINDEX_RESTORE_ALL_DEFAULT_PARAMETERS, ulPassword, p_pErrorInfo);
        }

        //Restore timeout
        p_pManager->DCS_SetProtocolStackSettings(p_hHandle, ulTransferRate, ulBackupTimeout, TRUE);
    }

    return oResult;
}

