#include "stdafx.h"

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/DataRecorder/RecorderChannelObject.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/DataRecorder/RecorderChannelObjects.h>

using namespace std;

// Constants
const BYTE TRIGGER_DISABLED(0);

CRecorderChannelObjects::CRecorderChannelObjects(WORD p_usMaxNumberOfRecorderVariables)
    : m_usMaxNumberOfRecorderVariables(p_usMaxNumberOfRecorderVariables)
{
    CreateChannels();
    ResetTrigger();
}

CRecorderChannelObjects::~CRecorderChannelObjects()
{
    DeleteChannels();
}

CRecorderChannelObject* CRecorderChannelObjects::GetChannel(int p_lChannelIndex)
{
    return m_RecorderChannelObjects.at(p_lChannelIndex);
}

BOOL CRecorderChannelObjects::SetChannel(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex, BYTE p_ubSize)
{
    BOOL oResult(FALSE);
    CRecorderChannelObject* pRecorderChannelObject(m_RecorderChannelObjects.at(p_lChannelIndex));

    if (pRecorderChannelObject)
    {
        pRecorderChannelObject->SetParameter(p_usIndex, p_ubSubIndex, p_ubSize);

        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderChannelObjects::GetChannel(int p_lChannelIndex, WORD& p_rusIndex, BYTE& p_rubSubIndex, BYTE& p_rubSize)
{
    BOOL oResult(FALSE);
    CRecorderChannelObject* pRecorderChannelObject(m_RecorderChannelObjects.at(p_lChannelIndex));

    if (pRecorderChannelObject)
    {
        pRecorderChannelObject->GetParameter(p_rusIndex, p_rubSubIndex, p_rubSize);

        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderChannelObjects::IsEqual(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex)
{
    BOOL oResult(FALSE);
    CRecorderChannelObject* pRecorderChannelObject(m_RecorderChannelObjects.at(p_lChannelIndex));

    if (pRecorderChannelObject)
    {
        oResult = pRecorderChannelObject->IsEqual(p_usIndex, p_ubSubIndex);
    }

    return oResult;
}

BOOL CRecorderChannelObjects::IsSizeAvailable(int p_lChannelIndex)
{
    return (GetSizeInWord(p_lChannelIndex) != 0);
}

WORD CRecorderChannelObjects::GetSizeInWord(int p_lChannelIndex)
{
    WORD usResult(0);
    CRecorderChannelObject* pRecorderChannelObject(m_RecorderChannelObjects.at(p_lChannelIndex));

    if (pRecorderChannelObject)
    {
        usResult = pRecorderChannelObject->SizeInWord();
    }

    return usResult;
}

WORD CRecorderChannelObjects::GetTotalSizeInWord()
{
    WORD usResult(0);

    for (DWORD i = 0; i < m_RecorderChannelObjects.size(); i++)
    {
        if (IsSizeAvailable(i))
        {
            usResult += m_RecorderChannelObjects.at(i)->SizeInWord();
        }
    }

    return usResult;
}

int CRecorderChannelObjects::GetNbOfConfiguredChannels()
{
    int lResult(0);

    for (DWORD i = 0; i < m_RecorderChannelObjects.size(); i++)
    {
        if (IsSizeAvailable(i))
        {
            lResult++;
        }
    }

    return lResult;
}

void CRecorderChannelObjects::ResetTrigger()
{
    m_ubTriggerType = TRIGGER_DISABLED;
}

void CRecorderChannelObjects::SetTrigger(BYTE p_ubTriggerType)
{
    m_ubTriggerType = p_ubTriggerType;
}

BYTE CRecorderChannelObjects::GetTrigger()
{
    return m_ubTriggerType;
}

void CRecorderChannelObjects::CreateChannels()
{
    for (int i = 0; i < m_usMaxNumberOfRecorderVariables; i++)
    {
        CreateChannel();
    }
}

void CRecorderChannelObjects::CreateChannel()
{
    CRecorderChannelObject* pRecorderChannelObject(new CRecorderChannelObject());

    m_RecorderChannelObjects.push_back(pRecorderChannelObject);
}

void CRecorderChannelObjects::DeleteChannels()
{
    vector<CRecorderChannelObject*>::iterator it = m_RecorderChannelObjects.begin();

    while (it != m_RecorderChannelObjects.end())
    {
        CRecorderChannelObject* pChannel = (*it);
        if (pChannel)
        {
            m_RecorderChannelObjects.erase(it);
            delete pChannel;

            it = m_RecorderChannelObjects.begin();
        }
    }

    m_RecorderChannelObjects.clear();
}
