#include "stdafx.h"

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/DiagnosisHistory/DiagnosisHistory.h>

using namespace std;

CDiagnosisHistory::CDiagnosisHistory(void)
{
    Reset();
}

CDiagnosisHistory::~CDiagnosisHistory(void)
{
}

void CDiagnosisHistory::Reset()
{
    m_ubMaximumMessages = 0;
    m_ubNewestMessage = 0;
    m_ubNewestAckMessage = 0;

    m_MessageList.clear();
}

BOOL CDiagnosisHistory::InitDiagnosisHistory(BYTE p_ubMaximumMessages, BYTE p_ubNewestMessage, BYTE p_ubNewestAckMessage)
{
    Reset();

    m_ubMaximumMessages = p_ubMaximumMessages;
    m_ubNewestMessage = p_ubNewestMessage;
    m_ubNewestAckMessage = p_ubNewestAckMessage;

    return TRUE;
}

BOOL CDiagnosisHistory::AddDiagnoseMessage(SDiagnosisMessage p_diagnosisMessage)
{
    m_MessageList.push_back(p_diagnosisMessage);

    return TRUE;
}

BOOL CDiagnosisHistory::AddDiagnoseMessage(SDiagnosisLongMessage p_diagnosisMessage)
{
    BOOL oResult(FALSE);
    SDiagnosisMessage message;

    memcpy(&message, &p_diagnosisMessage, sizeof(message));

    oResult = AddDiagnoseMessage(message);

    return oResult;
}

BYTE CDiagnosisHistory::GetNbOfDiagnoseMessages()
{
    BYTE ubResult(0);
    SDiagnosisMessage diagnosisMessage;

    for(list<SDiagnosisMessage>::iterator it = m_MessageList.begin(); it != m_MessageList.end(); it++)
    {
        diagnosisMessage = (SDiagnosisMessage&)(*it);

        if(diagnosisMessage.ulDiagCode_ErrorCode != 0)
        {
            ubResult++;
        }
    }

    return ubResult;
}

BOOL CDiagnosisHistory::GetDiagnoseMessage(BYTE p_ubMessageNumber, SDiagnosisMessage& p_rsMessage)
{
    const BYTE INDEX_OFFSET(1);

    BOOL oResult(FALSE);
    int lRingBufferIndex(0);
    int lIndex(0);

    if(p_ubMessageNumber > 0)
    {
        if(GetRingBufferIndex(p_ubMessageNumber - INDEX_OFFSET, lRingBufferIndex))
        {
            if((lRingBufferIndex >= 0) && (lRingBufferIndex < (int)m_MessageList.size()))
            {
                for(list<SDiagnosisMessage>::iterator it = m_MessageList.begin(); it != m_MessageList.end(); it++, lIndex++)
                {
                    if(lIndex == lRingBufferIndex)
                    {
                        p_rsMessage = (SDiagnosisMessage&)(*it);

                        oResult = TRUE;
                        break;
                    }
                }
            }
        }
    }

    return oResult;
}

BYTE CDiagnosisHistory::GetNbOfDiagnoseMessages(EDiagSource p_eSource, WORD p_usId)
{
    BYTE ubResult(0);
    BYTE ubTotalNbOfDiagnoseMessage(0);
    SDiagnosisMessage message;

    if(p_usId > 0)
    {
        //Init
        ubTotalNbOfDiagnoseMessage = GetNbOfDiagnoseMessages();

        //Filter Source
        for(BYTE i = 0; i < ubTotalNbOfDiagnoseMessage; i++)
        {
            if(GetDiagnoseMessage(i, message))
            {
                if(message.ulDiagCode_Source == p_eSource + (p_usId - 1))
                {
                    ubResult++;
                }
            }
            else
            {
                ubResult = 0;
                break;
            }
        }
    }

    return ubResult;
}

BOOL CDiagnosisHistory::GetDiagnoseMessage(BYTE p_ubMessageNumber, EDiagSource p_eSource, WORD p_usId, SDiagnosisMessage& p_rsMessage)
{
    BOOL oResult(FALSE);
    BYTE ubTotalNbOfDiagnoseMessage(0);
    BYTE ubIndex(0);
    SDiagnosisMessage message;

    if((p_ubMessageNumber > 0) && (p_usId > 0))
    {
        //Init
        ubTotalNbOfDiagnoseMessage = GetNbOfDiagnoseMessages();

        //Filter Source
        for(BYTE i = 0; i < ubTotalNbOfDiagnoseMessage; i++)
        {
            if(GetDiagnoseMessage(i, message))
            {
                if(message.ulDiagCode_Source == p_eSource + (p_usId - 1))
                {
                    if(ubIndex == p_ubMessageNumber -1)
                    {
                        p_rsMessage = message;
                        oResult = TRUE;
                        break;
                    }
                    ubIndex++;
                }
            }
            else
            {
                oResult = FALSE;
                break;
            }
        }
    }

    return oResult;
}

BOOL CDiagnosisHistory::GetMaxNbOfMessages(int& p_rlMaxNb)
{
    BOOL oResult(TRUE);

    p_rlMaxNb = (int)m_ubMaximumMessages;

    return oResult;
}

BOOL CDiagnosisHistory::GetNewestMessageIndex(int& p_rlIndex)
{
    const BYTE INDEX_OFFSET(1);

    BOOL oResult(FALSE);

    if((m_ubNewestMessage >= SUBINDEX_DIAGNOSIS_HISTORY_FIRST_MESSAGE) && (m_ubNewestMessage <= (SUBINDEX_DIAGNOSIS_HISTORY_FIRST_MESSAGE - INDEX_OFFSET + m_ubMaximumMessages)))
    {
        p_rlIndex = (int)(m_ubNewestMessage - SUBINDEX_DIAGNOSIS_HISTORY_FIRST_MESSAGE);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CDiagnosisHistory::GetRingBufferIndex(int p_lMsgIndex, int& p_rlRingBufferIndex)
{
    BOOL oResult(FALSE);
    int lMaxNbOfMessages(0);
    int lNewestMessageIndex(0);

    if(GetMaxNbOfMessages(lMaxNbOfMessages) && GetNewestMessageIndex(lNewestMessageIndex))
    {
        p_rlRingBufferIndex = lNewestMessageIndex - p_lMsgIndex;

        if(p_rlRingBufferIndex < 0)
        {
            p_rlRingBufferIndex += lMaxNbOfMessages;
        }

        oResult = TRUE;
    }

    return oResult;
}
