#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEposDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/Helper/ControllerGainsEpos.h>

CControllerGainsEpos::CControllerGainsEpos(void)
{
}

CControllerGainsEpos::~CControllerGainsEpos(void)
{
}

BOOL CControllerGainsEpos::GetControllerGainIndex(WORD p_usController, WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(FALSE);

    switch (p_usController)
    {
        case PI_CURRENT_CONTROLLER:
        {
            oResult = GetCurrentControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rErrorInfo);
            break;
        }
        case PI_VELOCITY_CONTROLLER:
        {
            oResult = GetVelocityControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rErrorInfo);
            break;
        }
        case PID_POSITION_CONTROLLER:
        {
            oResult = GetPositionControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rErrorInfo);
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos::GetCurrentControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set index
    p_rusIndex = INDEX_CURRENT_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PICC_P_GAIN:
        {
            p_rusSubIndex = SUBINDEX_CURRENT_P_GAIN;
            break;
        }
        case PICC_I_GAIN:
        {
            p_rusSubIndex = SUBINDEX_CURRENT_I_GAIN;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos::GetVelocityControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set index
    p_rusIndex = INDEX_VELOCITY_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PIVC_P_GAIN:
        {
            p_rusSubIndex = SUBINDEX_VELOCITY_P_GAIN;
            break;
        }
        case PIVC_I_GAIN:
        {
            p_rusSubIndex = SUBINDEX_VELOCITY_I_GAIN;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos::GetPositionControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set index
    p_rusIndex = INDEX_POSITION_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PIDPC_P_GAIN:
        {
            p_rusSubIndex = SUBINDEX_POSITION_P_GAIN;
            break;
        }
        case PIDPC_I_GAIN:
        {
            p_rusSubIndex = SUBINDEX_POSITION_I_GAIN;
            break;
        }
        case PIDPC_D_GAIN:
        {
            p_rusSubIndex = SUBINDEX_POSITION_D_GAIN;
            break;
        }
        case PIDPC_FEED_FORWARD_VELOCITY_GAIN:
        {
            p_rusSubIndex = SUBINDEX_POSITION_VELOCITY_FEED_FORWARD;
            break;
        }
        case PIDPC_FEED_FORWARD_ACCELERATION_GAIN:
        {
            p_rusSubIndex = SUBINDEX_POSITION_ACCELERATION_FEED_FORWARD;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}
