#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos4Def.h>
#include <CommunicationModel/CommonLayer/ErrorHandling/ErrorHandling.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/Helper/ControllerGainsEpos4.h>

CControllerGainsEpos4::CControllerGainsEpos4(void)
{
}

CControllerGainsEpos4::~CControllerGainsEpos4(void)
{
}

BOOL CControllerGainsEpos4::GetControllerGainIndex(WORD p_usController, WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(FALSE);

    switch (p_usController)
    {
        case PI_CURRENT_CONTROLLER:
        {
            oResult = GetCurrentControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rDataType, p_rErrorInfo);
            break;
        }
        case PI_VELOCITY_CONTROLLER:
        {
            oResult = GetVelocityControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rDataType, p_rErrorInfo);
            break;
        }
        case PI_VELOCITY_CONTROLLER_WITH_OBSERVER:
        {
            oResult = GetVelocityControllerGainWithObserverIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rDataType, p_rErrorInfo);
            break;
        }
        case PID_POSITION_CONTROLLER:
        {
            oResult = GetPositionControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rDataType, p_rErrorInfo);
            break;
        }
        case DUAL_LOOP_POSITION_CONTROLLER:
        {
            oResult = GetDualLoopPositionControllerGainIndex(p_usGain, p_rusIndex, p_rusSubIndex, p_rDataType, p_rErrorInfo);
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos4::GetCurrentControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set data type
    p_rDataType = ODT_UINT32;

    // Set index
    p_rusIndex = EPOS4_INDEX_CURRENT_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PICC_P_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_CURRENT_P_GAIN;
            break;
        }
        case PICC_I_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_CURRENT_I_GAIN;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos4::GetVelocityControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set data type
    p_rDataType = ODT_UINT32;

    // Set index
    p_rusIndex = EPOS4_INDEX_VELOCITY_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PIVC_P_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_P_GAIN;
            break;
        }
        case PIVC_I_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_I_GAIN;
            break;
        }
        case PIVC_FEED_FORWARD_VELOCITY_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_VELOCITY_FEED_FORWARD;
            break;
        }
        case PIVC_FEED_FORWARD_ACCELERATION_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_ACCELERATION_FEED_FORWARD;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos4::GetVelocityControllerGainWithObserverIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set data type
    p_rDataType = ODT_UINT32;

    switch (p_usGain)
    {
        case PIVCWO_P_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_P_GAIN;
            break;
        }
        case PIVCWO_I_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_I_GAIN;
            break;
        }
        case PIVCWO_FEED_FORWARD_VELOCITY_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_VELOCITY_FEED_FORWARD;
            break;
        }
        case PIVCWO_FEED_FORWARD_ACCELERATION_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_ACCELERATION_FEED_FORWARD;
            break;
        }
        case PIVCWO_OBSERVER_THETA_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_OBSERVER_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_OBSERVER_POS_CORR_GAIN;
            break;
        }
        case PIVCWO_OBSERVER_OMEGA_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_OBSERVER_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_OBSERVER_VEL_CORR_GAIN;
            break;
        }
        case PIVCWO_OBSERVER_TAU_GAIN:
        {
            p_rusIndex = EPOS4_INDEX_VELOCITY_OBSERVER_GAIN;
            p_rusSubIndex = EPOS4_SUBINDEX_VELOCITY_OBSERVER_LOAD_CORR_GAIN;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos4::GetPositionControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set data type
    p_rDataType = ODT_UINT32;

    // Set index
    p_rusIndex = EPOS4_INDEX_POSITION_GAIN;

    // Set sub index
    switch (p_usGain)
    {
        case PIDPC_P_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_POSITION_P_GAIN;
            break;
        }
        case PIDPC_I_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_POSITION_I_GAIN;
            break;
        }
        case PIDPC_D_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_POSITION_D_GAIN;
            break;
        }
        case PIDPC_FEED_FORWARD_VELOCITY_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_POSITION_VELOCITY_FEED_FORWARD;
            break;
        }
        case PIDPC_FEED_FORWARD_ACCELERATION_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_POSITION_ACCELERATION_FEED_FORWARD;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CControllerGainsEpos4::GetDualLoopPositionControllerGainIndex(WORD p_usGain, WORD& p_rusIndex, WORD& p_rusSubIndex, EObjectDataType& p_rDataType, CErrorInfo& p_rErrorInfo)
{
    BOOL oResult(TRUE);

    // Set index
    p_rusIndex = EPOS4_INDEX_DUAL_LOOP_CONTROL_PARAMETER;

    // Set sub index
    switch (p_usGain)
    {
        case DLPC_AUXILIARY_LOOP_P_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_P_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_I_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_I_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_FEED_FORWARD_VELOCITY_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_FF_VELOCITY_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_FEED_FORWARD_ACCELERATION_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_CONTROLLER_FF_ACCELERATION_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_OBSERVER_THETA_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_POSITION_CORRECTION_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_OBSERVER_OMEGA_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_VELOCITY_CORRECTION_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_AUXILIARY_LOOP_OBSERVER_TAU_GAIN:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_INNER_LOOP_OBSERVER_LOAD_CORRECTION_GAIN;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_P_GAIN_HIGH:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_P_GAIN_HIGH_BANDWIDTH;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_P_GAIN_LOW:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_P_GAIN_LOW_BANDWIDTH;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_GAIN_SCHEDULING_WEIGHT:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_CONTROLLER_GAIN_SCHEDULING_WEIGHT;
            p_rDataType = ODT_UINT16;
            break;
        }
        case DLPC_MAIN_LOOP_FILTER_COEFFICIENT_A:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_A;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_FILTER_COEFFICIENT_B:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_B;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_FILTER_COEFFICIENT_C:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_C;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_FILTER_COEFFICIENT_D:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_D;
            p_rDataType = ODT_UINT32;
            break;
        }
        case DLPC_MAIN_LOOP_FILTER_COEFFICIENT_E:
        {
            p_rusSubIndex = EPOS4_SUBINDEX_OUTER_LOOP_SECOND_ORDER_FILTER_COEFFICIENT_E;
            p_rDataType = ODT_UINT32;
            break;
        }
        default:
        {
            p_rErrorInfo.Init(ERROR_VIRTUAL_DEVICE_CONTROLLER_GAIN_NOT_SUPPORTED);
            oResult = FALSE;
        }
    }

    return oResult;
}
