
#include "stdafx.h"

#include <Log/MmcLog/MmcLog.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenCommunicationDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_DataRecorder.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive2/GatewayDrive2ToEpos.h>

CGatewayDrive2ToEpos::CGatewayDrive2ToEpos()
{
    InitErrorHandling();
}

CGatewayDrive2ToEpos::~CGatewayDrive2ToEpos()
{
}

CGateway* CGatewayDrive2ToEpos::Clone()
{
    CGatewayDrive2ToEpos* pClonedGateway;

    pClonedGateway = new CGatewayDrive2ToEpos();
    *pClonedGateway = *this;

    return (CGatewayVCStoDCS*)pClonedGateway;
}

CGatewayDrive2ToEpos& CGatewayDrive2ToEpos::operator=(CGatewayDrive2ToEpos& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayVCStoEpos*)this) = *((CGatewayVCStoEpos*)&p_rOther);
    }

    return *this;
}

BOOL CGatewayDrive2ToEpos::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(CheckLayers(p_pCommand, p_pManager))
    {
        BOOL oLogMessageActive = p_pCommand->IsLogMessageActive();

        LogCmdMessageIn(p_pCommand, oLogMessageActive, _T("CGatewayDrive2ToEpos"));
        
        oResult = ProcessDrive2Command(p_pCommand, p_pManager, p_hHandle, p_hTransactionHandle);

        if(p_pCommand->IsIdle())
        {
            oResult = CGatewayDriveToEpos::ProcessCommand(p_pCommand, p_pManager, p_hHandle, p_hTransactionHandle);
        }

        LogCmdMessageOut(p_pCommand, oLogMessageActive, _T("CGatewayDrive2ToEpos"));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::ProcessDrive2Command(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CCommand_VCS* pCommand_VCS = (CCommand_VCS*)p_pCommand;
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager = (CDeviceCommandSetManagerBase*)p_pManager;

    //Lock critical section
    if(Lock(p_pCommand))
    {
        //Process Command
        switch(p_pCommand->GetCommandId())
        {
            //DataRecorder Setup
            case REC_SET_RECORDER_PARAMETER: oResult = Process_SetDataRecordingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_RECORDER_PARAMETER: oResult = Process_GetDataRecordingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_ENABLE_TRIGGER: oResult = Process_EnableTrigger(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_DISABLE_ALL_TRIGGERS: oResult = Process_DisableAllTriggers(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_ACTIVATE_CHANNEL: oResult = Process_ActivateChannel(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_DEACTIVATE_ALL_CHANNELS: oResult = Process_DeactivateAllChannels(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Status
            case REC_START_RECORDER: oResult = Process_StartRecorder(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_STOP_RECORDER: oResult = Process_StopRecorder(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_FORCE_TRIGGER: oResult = Process_ForceTrigger(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_IS_RECORDER_RUNNING: oResult = Process_IsRecorderRunning(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_IS_RECORDER_TRIGGERED: oResult = Process_IsRecorderTriggered(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Data
            case REC_READ_CHANNEL_VECTOR_SIZE: oResult = Process_ReadChannelVectorSize(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Advanced Functions
            case REC_READ_DATA_BUFFER: oResult = Process_ReadDataBuffer(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
        }
        
        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("GatewayDrive2ToEpos"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(VIRTUAL_COMMAND_SET_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_SetDataRecordingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usSamplingPeriod(0);
    WORD usNbOfPrecedingSamples(0);

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
        p_pCommand->GetParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));

        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Sampling period
        if(oResult)
        {
            usIndex = INDEX_SAMPLING_PERIOD;
            usSubIndex = SUBINDEX_SAMPLING_PERIOD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usSamplingPeriod, &errorInfo);
        }

        //Number of preceding samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usNbOfPrecedingSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_GetDataRecordingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usSamplingPeriod(0);
    WORD usNbOfPrecedingSamples(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Sampling period
        if(oResult)
        {
            usIndex = INDEX_SAMPLING_PERIOD;
            usSubIndex = SUBINDEX_SAMPLING_PERIOD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSamplingPeriod, &errorInfo);
        }

        //Number of preceding samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfPrecedingSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
        p_pCommand->SetReturnParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_EnableTrigger(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubTriggerType(0);
    WORD usConfiguration(0);

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubTriggerType, sizeof(ubTriggerType));

        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder configuration
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                switch(ubTriggerType)
                {
                    case RTT_MOVEMENT_START_TRIGGER:    usConfiguration |= 0x0001; break;
                    case RTT_ERROR_TRIGGER:             usConfiguration |= 0x0002; break;
                    case RTT_DIGITAL_INPUT_TRIGGER:     usConfiguration |= 0x0004; break;
                    case RTT_MOVEMENT_END_TRIGGER:      usConfiguration |= 0x0008; break;
                }

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_DisableAllTriggers(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD TRIGGER_CONFIGURATION_OFF = 0xFFF0;
    const WORD TRIGGER_CONTROL_OFF = 0xFFFD;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usConfiguration(0);
    WORD usControl(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder configuration
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                usConfiguration &= TRIGGER_CONFIGURATION_OFF;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usControl, &errorInfo);

            if(oResult)
            {
                usControl &= TRIGGER_CONTROL_OFF;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usControl, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_ActivateChannel(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubChannelNumber;
    WORD usObjectIndex;
    BYTE ubObjectSubIndex;
    BYTE ubObjectSize;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));
        p_pCommand->GetParameterData(1, &usObjectIndex, sizeof(usObjectIndex));
        p_pCommand->GetParameterData(2, &ubObjectSubIndex, sizeof(ubObjectSubIndex));
        p_pCommand->GetParameterData(3, &ubObjectSize, sizeof(ubObjectSize));

        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Channel object index
        if(oResult)
        {
            usIndex = INDEX_VARIABLE_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usObjectIndex, &errorInfo);
        }

        //Channel object subindex
        if(oResult)
        {
            usIndex = INDEX_VARIABLE_SUB_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubObjectSubIndex, &errorInfo);
        }

        //Activate number of channels
        if(oResult)
        {
            usIndex = INDEX_NB_OF_SAMPLING_VARIABLES;
            usSubIndex = SUBINDEX_NB_OF_SAMPLING_VARIABLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubChannelNumber, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_DeactivateAllChannels(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usDefaultValue(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Number of sampling variables
        if(oResult)
        {
            usIndex = INDEX_NB_OF_SAMPLING_VARIABLES;
            usSubIndex = SUBINDEX_NB_OF_SAMPLING_VARIABLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
        }

        for(BYTE ubChannelNumber = 0; ubChannelNumber < MAX_NB_OF_DATA_RECORDER_VARIABLES; ubChannelNumber++)
        {
            //Channel object index
            if(oResult)
            {
                usIndex = INDEX_VARIABLE_INDEX;
                usSubIndex = ubChannelNumber + 1;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
            }

            //Channel object subindex
            if(oResult)
            {
                usIndex = INDEX_VARIABLE_SUB_INDEX;
                usSubIndex = ubChannelNumber + 1;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_StartRecorder(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD START_RECORDER = 0x0001;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = START_RECORDER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_StopRecorder(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD STOP_RECORDER = 0x0000;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = STOP_RECORDER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_ForceTrigger(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD FORCE_TRIGGER = 0x0002;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = FORCE_TRIGGER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_IsRecorderRunning(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD RECORDER_RUNNING = 0x0001;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BOOL oRecorderRunning = FALSE;
    WORD usRecorderStatus;

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder status
        if(oResult)
        {
            usIndex = INDEX_RECORDER_STATUS;
            usSubIndex = SUBINDEX_RECORDER_STATUS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderStatus, &errorInfo);

            if(oResult)
            {
                if(usRecorderStatus & RECORDER_RUNNING)
                {
                    oRecorderRunning = TRUE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oRecorderRunning, sizeof(oRecorderRunning));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_IsRecorderTriggered(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD RECORDER_TRIGGERED = 0x0002;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BOOL oRecorderTriggered = FALSE;
    WORD usRecorderStatus;

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder status
        if(oResult)
        {
            usIndex = INDEX_RECORDER_STATUS;
            usSubIndex = SUBINDEX_RECORDER_STATUS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderStatus, &errorInfo);

            if(oResult)
            {
                if(usRecorderStatus & RECORDER_TRIGGERED)
                {
                    oRecorderTriggered = TRUE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oRecorderTriggered, sizeof(oRecorderTriggered));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_ReadChannelVectorSize(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usVectorSize(0);
    DWORD ulVectorSize(0);

    if(p_pCommand)
    {
        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Max number of samples
        if(oResult)
        {
            usIndex = INDEX_MAX_NB_OF_SAMPLES;
            usSubIndex = SUBINDEX_MAX_NB_OF_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVectorSize, &errorInfo);
        }

        ulVectorSize = usVectorSize;

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulVectorSize, sizeof(ulVectorSize));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos::Process_ReadDataBuffer(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    void* pubDataBuffer(NULL);
    DWORD ulBufferSizeToRead;
    DWORD ulBufferSizeRead(0);
    WORD usVectorStartOffset(0);
    WORD usMaxNbOfSamples(0);
    WORD usNbOfRecordedSamples(0);

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulBufferSizeToRead, sizeof(ulBufferSizeToRead));

        //Get Address Parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read data buffer
        if(oResult)
        {
            pubDataBuffer = malloc(ulBufferSizeToRead);
            usIndex = INDEX_DATA_BUFFER;
            usSubIndex = SUBINDEX_DATA_BUFFER;
            oResult = ReadDataBufferObject(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, pubDataBuffer, ulBufferSizeToRead, &ulBufferSizeRead, &errorInfo);
        }

        //Vector start offset
        if(oResult)
        {
            usIndex = INDEX_VECTOR_START_OFFSET;
            usSubIndex = SUBINDEX_VECTOR_START_OFFSET;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVectorStartOffset, &errorInfo);
        }

        //Max number of samples
        if(oResult)
        {
            usIndex = INDEX_MAX_NB_OF_SAMPLES;
            usSubIndex = SUBINDEX_MAX_NB_OF_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMaxNbOfSamples, &errorInfo);
        }

        //Number of recorded samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_RECORDED_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_RECORDED_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfRecordedSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, pubDataBuffer, ulBufferSizeToRead);
        p_pCommand->SetReturnParameterData(1, &ulBufferSizeRead, sizeof(ulBufferSizeRead));
        p_pCommand->SetReturnParameterData(2, &usVectorStartOffset, sizeof(usVectorStartOffset));
        p_pCommand->SetReturnParameterData(3, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
        p_pCommand->SetReturnParameterData(4, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));

        if(pubDataBuffer) free(pubDataBuffer);
    }

    return oResult;
}
