#include "stdafx.h"

#include <Classes/MmcDataConversion.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenCommunicationDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos2Def.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Configuration.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_DataRecorder.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>
#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/DataRecorder/RecorderChannelObjects.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive2/GatewayDrive2ToEpos2.h>

CGatewayDrive2ToEpos2::CGatewayDrive2ToEpos2()
{
    CreateRecorderChannelObjects(MAX_NB_OF_DATA_RECORDER_VARIABLES);

    InitErrorHandling();
}

CGatewayDrive2ToEpos2::~CGatewayDrive2ToEpos2()
{
}

CGateway* CGatewayDrive2ToEpos2::Clone()
{
    CGatewayDrive2ToEpos2* pClonedGateway;

    pClonedGateway = new CGatewayDrive2ToEpos2();
    *pClonedGateway = *this;

    return (CGatewayVCStoDCS*)pClonedGateway;
}

CGatewayDrive2ToEpos2& CGatewayDrive2ToEpos2::operator=(CGatewayDrive2ToEpos2& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayVCStoEpos2*)this) = *((CGatewayVCStoEpos2*)&p_rOther);
    }

    return *this;
}

BOOL CGatewayDrive2ToEpos2::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("GatewayDrive2ToEpos2"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(VIRTUAL_COMMAND_SET_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(CheckLayers(p_pCommand, p_pManager))
    {
        BOOL oLogMessageActive = p_pCommand->IsLogMessageActive();

        LogCmdMessageIn(p_pCommand, oLogMessageActive, _T("CGatewayDrive2ToEpos2"));

        oResult = ProcessDrive2Command(p_pCommand, p_pManager, p_hHandle, p_hTransactionHandle);

        if(p_pCommand->IsIdle())
        {
            oResult = CGatewayDriveToEpos2::ProcessCommand(p_pCommand, p_pManager, p_hHandle, p_hTransactionHandle);
        }

        LogCmdMessageOut(p_pCommand, oLogMessageActive, _T("CGatewayDrive2ToEpos2"));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::ProcessDrive2Command(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CCommand_VCS* pCommand_VCS = (CCommand_VCS*)p_pCommand;
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager = (CDeviceCommandSetManagerBase*)p_pManager;

    //Lock critical section
    if(Lock(p_pCommand))
    {
        //Process command
        switch(p_pCommand->GetCommandId())
        {
            //Units
            case CONF_GET_VELOCITY_UNITS: oResult = Process_GetVelocityUnits(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_VELOCITY_UNITS: oResult = Process_SetVelocityUnits(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //Interpolated position mode
            case MODE_IPM_SET_IPM_BUFFER_PARAMETER: oResult = Process_SetIpmBufferParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_GET_IPM_BUFFER_PARAMETER: oResult = Process_GetIpmBufferParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_CLEAR_IPM_BUFFER: oResult = Process_ClearIpmBuffer(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_GET_FREE_IPM_BUFFER_SIZE: oResult = Process_GetFreeIpmBufferSize(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_ADD_PVT_VALUE_TO_IPM_BUFFER: oResult = Process_AddPvtValueToIpmBuffer(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_START_IPM_TRAJECTORY: oResult = Process_StartIpmTrajectory(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_STOP_IPM_TRAJECTORY: oResult = Process_StopIpmTrajectory(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_IPM_GET_IPM_STATUS: oResult = Process_GetIpmStatus(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //Master encoder mode
            case MODE_MEM_SET_MASTER_ENCODER_PARAMETER: oResult = Process_SetMasterEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_MEM_GET_MASTER_ENCODER_PARAMETER: oResult = Process_GetMasterEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //StepDirectionMode
            case MODE_STM_SET_STEP_DIRECTION_PARAMETER: oResult = Process_SetStepDirectionParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_STM_GET_STEP_DIRECTION_PARAMETER: oResult = Process_GetStepDirectionParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //I/O Position compare
            case IO_SET_POSITION_COMPARE_PARAMETER: oResult = Process_SetPositionCompareParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_POSITION_COMPARE_PARAMETER : oResult = Process_GetPositionCompareParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_ACTIVATE_POSITION_COMPARE: oResult = Process_ActivatePositionCompare(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_DEACTIVATE_POSITION_COMPARE: oResult = Process_DeactivatePositionCompare(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_ENABLE_POSITION_COMPARE: oResult = Process_EnablePositionCompare(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_DISABLE_POSITION_COMPARE: oResult = Process_DisablePositionCompare(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_SET_POSITION_COMPARE_REFERENCE_POSITION: oResult = Process_SetPositionCompareReferencePosition(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Setup
            case REC_SET_RECORDER_PARAMETER: oResult = Process_SetDataRecordingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_RECORDER_PARAMETER: oResult = Process_GetDataRecordingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_ENABLE_TRIGGER: oResult = Process_EnableTrigger(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_DISABLE_ALL_TRIGGERS: oResult = Process_DisableAllTriggers(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_ACTIVATE_CHANNEL: oResult = Process_ActivateChannel(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_ACTIVE_CHANNEL: oResult = Process_ActiveChannel(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_DEACTIVATE_ALL_CHANNELS: oResult = Process_DeactivateAllChannels(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_VARIABLE: oResult = Process_GetVariable(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_MAX_NUMBER_OF_SAMPLES: oResult = Process_GetMaxNbOfSamples(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_NUMBER_OF_RECORDED_SAMPLES: oResult = Process_GetNbOfRecordedSamples(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_SET_NUMBER_OF_PRECEDING_SAMPLES: oResult = Process_SetNbOfPrecedingSamples(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_NUMBER_OF_PRECEDING_SAMPLES: oResult = Process_GetNbOfPrecedingSamples(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_NUMBER_OF_SAMPLING_VARIABLES: oResult = Process_GetNbOfSamplingVariables(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_SAMPLING_PERIOD: oResult = Process_GetSamplingPeriod(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Status
            case REC_START_RECORDER: oResult = Process_StartRecorder(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_STOP_RECORDER: oResult = Process_StopRecorder(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_FORCE_TRIGGER: oResult = Process_ForceTrigger(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_IS_RECORDER_RUNNING: oResult = Process_IsRecorderRunning(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_IS_RECORDER_TRIGGERED: oResult = Process_IsRecorderTriggered(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_RECORDER_STATUS: oResult = Process_GetRecorderStatus(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_SET_RECORDER_CONFIGURATION: oResult = Process_SetRecorderConfiguration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_RECORDER_CONFIGURATION: oResult = Process_GetRecorderConfiguration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Data
            case REC_READ_CHANNEL_VECTOR_SIZE: oResult = Process_ReadChannelVectorSize(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case REC_GET_VECTOR_START_OFFSET: oResult = Process_GetVectorStartOffset(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //DataRecorder Advanced Functions
            case REC_READ_DATA_BUFFER: oResult = Process_ReadDataBuffer(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
        }

        //Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetPositionCompareParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    WORD usIndex;
    WORD usSubIndex;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubOperationalMode;
    BYTE ubIntervalMode;
    BYTE ubDirectionDependency;
    WORD usIntervalWidth;
    WORD usIntervalRepetitions;
    WORD usPulseWidth;
    WORD usConfiguration(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubOperationalMode, sizeof(ubOperationalMode));
        p_pCommand->GetParameterData(1, &ubIntervalMode, sizeof(ubIntervalMode));
        p_pCommand->GetParameterData(2, &ubDirectionDependency, sizeof(ubDirectionDependency));
        p_pCommand->GetParameterData(3, &usIntervalWidth, sizeof(usIntervalWidth));
        p_pCommand->GetParameterData(4, &usIntervalRepetitions, sizeof(usIntervalRepetitions));
        p_pCommand->GetParameterData(5, &usPulseWidth, sizeof(usPulseWidth));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set position compare configuration
        if(oResult)
        {
            ubDirectionDependency &= 0x03;
            usConfiguration += ubDirectionDependency;
            usConfiguration <<= 2;

            ubIntervalMode &= 0x03;
            usConfiguration += ubIntervalMode;
            usConfiguration <<= 2;

            ubOperationalMode &= 0x03;
            usConfiguration += ubOperationalMode;
            usConfiguration <<= 1;

            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_CONFIGURATION;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
        }

        //Set position compare interval width
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_INTERVAL_WIDTH;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usIntervalWidth, &errorInfo);
        }

        //Set position compare interval repetitions
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_INTERVAL_REPETITIONS;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usIntervalRepetitions, &errorInfo);
        }

        //Set position compare pulse width
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_PULSE_WIDTH;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPulseWidth, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetPositionCompareParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    WORD usIndex;
    WORD usSubIndex;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubOperationalMode;
    BYTE ubIntervalMode;
    BYTE ubDirectionDependency;
    WORD usIntervalWidth;
    WORD usIntervalRepetitions;
    WORD usPulseWidth;
    WORD usConfiguration(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set position compare configuration
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            usConfiguration >>= 1;
            ubOperationalMode = (BYTE)usConfiguration;
            ubOperationalMode &= 0x03;

            usConfiguration >>= 2;
            ubIntervalMode = (BYTE)usConfiguration;
            ubIntervalMode &= 0x03;

            usConfiguration >>= 2;
            ubDirectionDependency = (BYTE)usConfiguration;
            ubDirectionDependency &= 0x03;
        }

        //Set position compare interval width
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_INTERVAL_WIDTH;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usIntervalWidth, &errorInfo);
        }

        //Set position compare interval repetitions
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_INTERVAL_REPETITIONS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usIntervalRepetitions, &errorInfo);
        }

        //Set position compare pulse width
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_PULSE_WIDTH;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPulseWidth, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ubOperationalMode, sizeof(ubOperationalMode));
        p_pCommand->SetReturnParameterData(1, &ubIntervalMode, sizeof(ubIntervalMode));
        p_pCommand->SetReturnParameterData(2, &ubDirectionDependency, sizeof(ubDirectionDependency));
        p_pCommand->SetReturnParameterData(3, &usIntervalWidth, sizeof(usIntervalWidth));
        p_pCommand->SetReturnParameterData(4, &usIntervalRepetitions, sizeof(usIntervalRepetitions));
        p_pCommand->SetReturnParameterData(5, &usPulseWidth, sizeof(usPulseWidth));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ActivatePositionCompare(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_COMPARE = 1;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usDigitalOutputNumber;
    BOOL oPolarity;
    WORD usPolarity;
    WORD usMask;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalOutputNumber, sizeof(usDigitalOutputNumber));
        p_pCommand->GetParameterData(1, &oPolarity, sizeof(oPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital output functionalities
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_CONFIGURATION;
            usSubIndex = usDigitalOutputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, POSITION_COMPARE, &errorInfo);
        }

        //Set digital output polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPolarity, &errorInfo);

            if(oResult)
            {
                if(oPolarity)
                {
                    usPolarity |= 0x0002;
                }
                else
                {
                    usPolarity &= 0xFFFD;
                }

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPolarity, &errorInfo);
            }
        }

        //Set digital output mask
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMask, &errorInfo);

            if(oResult)
            {
                usMask |= 0x0002;

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMask, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_DeactivatePositionCompare(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usDigitalOutputNumber(0);
    WORD usConfiguration(15);
    WORD usMask(0);

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalOutputNumber, sizeof(usDigitalOutputNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set digital output configuration
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_CONFIGURATION;
            usSubIndex = usDigitalOutputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);

            while(!oResult && (usConfiguration != 0))
            {
                usConfiguration--;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set digital output mask
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMask, &errorInfo);

            if(oResult)
            {
                usMask &= 0xFFFD;

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMask, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_EnablePositionCompare(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_COMPARE_ENABLE = 0x0001;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usConfiguration = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set digital output mask
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                usConfiguration |= POSITION_COMPARE_ENABLE;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_DisablePositionCompare(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_COMPARE_DISABLE = 0xFFFE;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usConfiguration = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set digital output mask
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                usConfiguration &= POSITION_COMPARE_DISABLE;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetPositionCompareReferencePosition(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    int lReferencePosition = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &lReferencePosition, sizeof(lReferencePosition));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set position compare reference position
        if(oResult)
        {
            usIndex = INDEX_POSITION_COMPARE_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_COMPARE_REFERENCE_POSITION;
            oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lReferencePosition, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetVelocityUnits(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubVelocityDimension = 0;
    char bVelocityNotation = 0;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubVelocityDimension, sizeof(ubVelocityDimension));
        p_pCommand->GetParameterData(1, &bVelocityNotation, sizeof(bVelocityNotation));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Velocity dimension
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_DIMENSION;
            usSubIndex = SUBINDEX_VELOCITY_DIMENSION;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubVelocityDimension, &errorInfo);
        }

        //Velocity notation
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_NOTATION;
            usSubIndex = SUBINDEX_VELOCITY_NOTATION;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, bVelocityNotation, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetVelocityUnits(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubVelocityDimension = 0;
    __int8 bVelocityNotation = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Velocity dimension
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_DIMENSION;
            usSubIndex = SUBINDEX_VELOCITY_DIMENSION;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubVelocityDimension, &errorInfo);
        }

        //Velocity notation
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_NOTATION;
            usSubIndex = SUBINDEX_VELOCITY_NOTATION;
            oResult = ReadInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &bVelocityNotation, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ubVelocityDimension, sizeof(ubVelocityDimension));
        p_pCommand->SetReturnParameterData(1, &bVelocityNotation, sizeof(bVelocityNotation));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetIpmBufferParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usUnderflowWarning = 0;
    WORD usOverflowWarning = 0;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usUnderflowWarning, sizeof(usUnderflowWarning));
        p_pCommand->GetParameterData(1, &usOverflowWarning, sizeof(usOverflowWarning));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Underflow warning
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_BUFFER;
            usSubIndex = SUBINDEX_INTERPOLATION_BUFFER_UNDERFLOW_WARNING;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usUnderflowWarning, &errorInfo);
        }

        //Overflow warning
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_BUFFER;
            usSubIndex = SUBINDEX_INTERPOLATION_BUFFER_OVERFLOW_WARNING;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usOverflowWarning, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetIpmBufferParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usUnderflowWarning = 0;
    WORD usOverflowWarning = 0;
    DWORD ulMaxBufferSize = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Underflow warning
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_BUFFER;
            usSubIndex = SUBINDEX_INTERPOLATION_BUFFER_UNDERFLOW_WARNING;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usUnderflowWarning, &errorInfo);
        }

        //Overflow warning
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_BUFFER;
            usSubIndex = SUBINDEX_INTERPOLATION_BUFFER_OVERFLOW_WARNING;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usOverflowWarning, &errorInfo);
        }

        //Max buffer size
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_DATA_CONFIGURATION;
            usSubIndex = SUBINDEX_INTERPOLATION_DC_MAX_BUFFER_SIZE;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxBufferSize, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usUnderflowWarning, sizeof(usUnderflowWarning));
        p_pCommand->SetReturnParameterData(1, &usOverflowWarning, sizeof(usOverflowWarning));
        p_pCommand->SetReturnParameterData(2, &ulMaxBufferSize, sizeof(ulMaxBufferSize));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ClearIpmBuffer(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const BYTE BUFFER_CLEAR = 0;
    const BYTE BUFFER_ENABLE = 1;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Clear and disable data buffer
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_DATA_CONFIGURATION;
            usSubIndex = SUBINDEX_INTERPOLATION_DC_BUFFER_CLEAR;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, BUFFER_CLEAR, &errorInfo);
        }

        //Enable data buffer
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_DATA_CONFIGURATION;
            usSubIndex = SUBINDEX_INTERPOLATION_DC_BUFFER_CLEAR;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, BUFFER_ENABLE, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetFreeIpmBufferSize(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    DWORD ulFreeBufferSize = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Free buffer size
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_DATA_CONFIGURATION;
            usSubIndex = SUBINDEX_INTERPOLATION_DC_ACTUAL_BUFFER_SIZE;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulFreeBufferSize, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulFreeBufferSize, sizeof(ulFreeBufferSize));
    }

    return oResult;
}

void CGatewayDrive2ToEpos2::CreateInterpolatedDataBufferEntry(BYTE p_ubTime, int p_lVelocity, int p_lPosition, __uint64 &p_rllPvtReferencePoint)
{
    long lPosition = p_lPosition;
    long lVelocity = (long)p_lVelocity;
    BYTE ubTime = p_ubTime;

    //Security loop to delete values higher than 24-Bit
    lVelocity <<= 8;
    lVelocity >>= 8;

    //Add time
    p_rllPvtReferencePoint = ubTime;

    //Add velocity
    p_rllPvtReferencePoint <<= 24;
    if(lVelocity < 0)
    {
        lVelocity &= 0x00FFFFFF;
    }

    p_rllPvtReferencePoint += DWORD(lVelocity);

    //Add position
    p_rllPvtReferencePoint <<= 32;
    p_rllPvtReferencePoint += DWORD(lPosition);
}

BOOL CGatewayDrive2ToEpos2::Process_AddPvtValueToIpmBuffer(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    int lPosition = 0;
    int lVelocity = 0;
    BYTE ubTime = 0;
    __uint64 llPvtReferencePoint = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &lPosition, sizeof(lPosition));
        p_pCommand->GetParameterData(1, &lVelocity, sizeof(lVelocity));
        p_pCommand->GetParameterData(2, &ubTime, sizeof(ubTime));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Add PVT value
        if(oResult)
        {
            CreateInterpolatedDataBufferEntry(ubTime, lVelocity, lPosition, llPvtReferencePoint);

            //Write value to device
            usIndex = INDEX_INTERPOLATION_DATA_RECORD;
            usSubIndex = SUBINDEX_INTERPOLATION_DATA_RECORD;
            oResult = WriteUInt64Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, llPvtReferencePoint, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_StartIpmTrajectory(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Start IPM trajectory
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ENABLE_INTERPOLATED_POSITION_MODE, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_StopIpmTrajectory(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Stop IPM trajectory
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, STOP_PROFILE, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetIpmStatus(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD UNDERFLOW_WARNING = 0x0001;
    const WORD OVERFLOW_WARNING = 0x0002;
    const WORD VELOCITY_WARNING = 0x0004;
    const WORD ACCELERATION_WARNING = 0x0008;
    const WORD UNDERFLOW_ERROR = 0x0100;
    const WORD OVERFLOW_ERROR = 0x0200;
    const WORD VELOCITY_ERROR = 0x0400;
    const WORD ACCELERATION_ERROR = 0x0800;
    const WORD MODE_ACTIVE = 0x8000;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BOOL oTrajectoryRunning = FALSE;
    BOOL oIsUnderflowWarning = FALSE;
    BOOL oIsOverflowWarning = FALSE;
    BOOL oIsVelocityWarning = FALSE;
    BOOL oIsAccelerationWarning = FALSE;
    BOOL oIsUnderflowError = FALSE;
    BOOL oIsOverflowError = FALSE;
    BOOL oIsVelocityError = FALSE;
    BOOL oIsAccelerationError = FALSE;
    WORD usBufferStatus = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Buffer status
        if(oResult)
        {
            usIndex = INDEX_INTERPOLATION_BUFFER;
            usSubIndex = SUBINDEX_INTERPOLATION_BUFFER_STATUS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usBufferStatus, &errorInfo);
        }

        if(oResult)
        {
            if(MODE_ACTIVE & usBufferStatus) oTrajectoryRunning = TRUE;
            if(UNDERFLOW_WARNING & usBufferStatus) oIsUnderflowWarning = TRUE;
            if(OVERFLOW_WARNING & usBufferStatus) oIsOverflowWarning = TRUE;
            if(VELOCITY_WARNING & usBufferStatus) oIsVelocityWarning = TRUE;
            if(ACCELERATION_WARNING & usBufferStatus) oIsAccelerationWarning = TRUE;
            if(UNDERFLOW_ERROR & usBufferStatus) oIsUnderflowError = TRUE;
            if(OVERFLOW_ERROR & usBufferStatus) oIsOverflowError = TRUE;
            if(VELOCITY_ERROR & usBufferStatus) oIsVelocityError = TRUE;
            if(ACCELERATION_ERROR & usBufferStatus) oIsAccelerationError = TRUE;
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oTrajectoryRunning, sizeof(oTrajectoryRunning));
        p_pCommand->SetReturnParameterData(1, &oIsUnderflowWarning, sizeof(oIsUnderflowWarning));
        p_pCommand->SetReturnParameterData(2, &oIsOverflowWarning, sizeof(oIsOverflowWarning));
        p_pCommand->SetReturnParameterData(3, &oIsVelocityWarning, sizeof(oIsVelocityWarning));
        p_pCommand->SetReturnParameterData(4, &oIsAccelerationWarning, sizeof(oIsAccelerationWarning));
        p_pCommand->SetReturnParameterData(5, &oIsUnderflowError, sizeof(oIsUnderflowError));
        p_pCommand->SetReturnParameterData(6, &oIsOverflowError, sizeof(oIsOverflowError));
        p_pCommand->SetReturnParameterData(7, &oIsVelocityError, sizeof(oIsVelocityError));
        p_pCommand->SetReturnParameterData(8, &oIsAccelerationError, sizeof(oIsAccelerationError));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetMasterEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubPolarity(0);
    DWORD ulMaxAcceleration(0);
    DWORD ulMaxVelocity(0);
    WORD usScalingDenominator(0);
    WORD usScalingNumerator(0);

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &usScalingNumerator, sizeof(usScalingNumerator));
        p_pCommand->GetParameterData(1, &usScalingDenominator, sizeof(usScalingDenominator));
        p_pCommand->GetParameterData(2, &ubPolarity, sizeof(ubPolarity));
        p_pCommand->GetParameterData(3, &ulMaxVelocity, sizeof(ulMaxVelocity));
        p_pCommand->GetParameterData(4, &ulMaxAcceleration, sizeof(ulMaxAcceleration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital Position Scaling Numerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_NUMERATOR;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usScalingNumerator, &errorInfo);
        }

        //Digital Position Scaling Denomerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_DENOMINATOR;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usScalingDenominator, &errorInfo);
        }

        //Digital Position Polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_POLARITY;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPolarity, &errorInfo);
        }

        //Max Profile Velocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxVelocity, &errorInfo);
        }

        //Max Acceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetMasterEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubPolarity(0);
    DWORD ulMaxAcceleration(0);
    DWORD ulMaxVelocity(0);
    WORD usScalingDenominator(0);
    WORD usScalingNumerator(0);

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital Position Scaling Numerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_NUMERATOR;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usScalingNumerator, &errorInfo);
        }

        //Digital Position Scaling Denomerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_DENOMINATOR;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usScalingDenominator, &errorInfo);
        }

        //Digital Position Polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_POLARITY;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPolarity, &errorInfo);
        }

        //Max Profile Velocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxVelocity, &errorInfo);
        }

        //Max Acceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usScalingNumerator, sizeof(usScalingNumerator));
        p_pCommand->SetReturnParameterData(1, &usScalingDenominator, sizeof(usScalingDenominator));
        p_pCommand->SetReturnParameterData(2, &ubPolarity, sizeof(ubPolarity));
        p_pCommand->SetReturnParameterData(3, &ulMaxVelocity, sizeof(ulMaxVelocity));
        p_pCommand->SetReturnParameterData(4, &ulMaxAcceleration, sizeof(ulMaxAcceleration));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetStepDirectionParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubPolarity(0);
    DWORD ulMaxAcceleration(0);
    DWORD ulMaxVelocity(0);
    WORD usScalingDenominator(0);
    WORD usScalingNumerator(0);

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &usScalingNumerator, sizeof(usScalingNumerator));
        p_pCommand->GetParameterData(1, &usScalingDenominator, sizeof(usScalingDenominator));
        p_pCommand->GetParameterData(2, &ubPolarity, sizeof(ubPolarity));
        p_pCommand->GetParameterData(3, &ulMaxVelocity, sizeof(ulMaxVelocity));
        p_pCommand->GetParameterData(4, &ulMaxAcceleration, sizeof(ulMaxAcceleration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital Position Scaling Numerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_NUMERATOR;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usScalingNumerator, &errorInfo);
        }

        //Digital Position Scaling Denomerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_DENOMINATOR;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usScalingDenominator, &errorInfo);
        }

        //Digital Position Polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_POLARITY;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPolarity, &errorInfo);
        }

        //Max Profile Velocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxVelocity, &errorInfo);
        }

        //Max Acceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetStepDirectionParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubPolarity = 0;
    DWORD ulMaxAcceleration = 0;
    DWORD ulMaxVelocity = 0;
    WORD usScalingDenominator = 0;
    WORD usScalingNumerator = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital Position Scaling Numerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_NUMERATOR;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usScalingNumerator, &errorInfo);
        }

        //Digital Position Scaling Denomerator
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_SCALING_DENOMINATOR;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usScalingDenominator, &errorInfo);
        }

        //Digital Position Polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_POSITION_INPUT;
            usSubIndex = SUBINDEX_DIGITAL_POSITION_POLARITY;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPolarity, &errorInfo);
        }

        //Max Profile Velocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxVelocity, &errorInfo);
        }

        //Max Acceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usScalingNumerator, sizeof(usScalingNumerator));
        p_pCommand->SetReturnParameterData(1, &usScalingDenominator, sizeof(usScalingDenominator));
        p_pCommand->SetReturnParameterData(2, &ubPolarity, sizeof(ubPolarity));
        p_pCommand->SetReturnParameterData(3, &ulMaxVelocity, sizeof(ulMaxVelocity));
        p_pCommand->SetReturnParameterData(4, &ulMaxAcceleration, sizeof(ulMaxAcceleration));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetDataRecordingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usSamplingPeriod = 0;
    WORD usNbOfPrecedingSamples = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
        p_pCommand->GetParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Sampling period
        if(oResult)
        {
            usIndex = INDEX_SAMPLING_PERIOD;
            usSubIndex = SUBINDEX_SAMPLING_PERIOD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usSamplingPeriod, &errorInfo);
        }

        //Number of preceding samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usNbOfPrecedingSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetDataRecordingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usSamplingPeriod = 0;
    WORD usNbOfPrecedingSamples = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Sampling period
        if(oResult)
        {
            usIndex = INDEX_SAMPLING_PERIOD;
            usSubIndex = SUBINDEX_SAMPLING_PERIOD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSamplingPeriod, &errorInfo);
        }

        //Number of preceding samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfPrecedingSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
        p_pCommand->SetReturnParameterData(1, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_EnableTrigger(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubTriggerType = 0;
    WORD usConfiguration = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &ubTriggerType, sizeof(ubTriggerType));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder configuration
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                switch(ubTriggerType)
                {
                    case RTT_MOVEMENT_START_TRIGGER:    usConfiguration |= 0x0001; break;
                    case RTT_ERROR_TRIGGER:             usConfiguration |= 0x0002; break;
                    case RTT_DIGITAL_INPUT_TRIGGER:     usConfiguration |= 0x0004; break;
                    case RTT_MOVEMENT_END_TRIGGER:      usConfiguration |= 0x0008; break;
                }

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_DisableAllTriggers(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD TRIGGER_CONFIGURATION_OFF = 0xFFF0;
    const WORD TRIGGER_CONTROL_OFF = 0xFFFD;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usConfiguration = 0;
    WORD usControl = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder configuration
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usConfiguration, &errorInfo);

            if(oResult)
            {
                usConfiguration &= TRIGGER_CONFIGURATION_OFF;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usControl, &errorInfo);

            if(oResult)
            {
                usControl &= TRIGGER_CONTROL_OFF;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usControl, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ActivateChannel(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    CRecorderChannelObjects* pRecorderChannelObjects(GetRecorderChannelObjects());
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BYTE ubChannelNumber;
    WORD usObjectIndex;
    BYTE ubObjectSubIndex;
    BYTE ubObjectSize;

    if(p_pCommand && pRecorderChannelObjects)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));
        p_pCommand->GetParameterData(1, &usObjectIndex, sizeof(usObjectIndex));
        p_pCommand->GetParameterData(2, &ubObjectSubIndex, sizeof(ubObjectSubIndex));
        p_pCommand->GetParameterData(3, &ubObjectSize, sizeof(ubObjectSize));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Channel object index
        if(oResult)
        {
            usIndex = INDEX_VARIABLE_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, usIndex, usSubIndex, usObjectIndex, &errorInfo);
        }

        //Channel object subindex
        if(oResult)
        {
            usIndex = INDEX_VARIABLE_SUB_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, usIndex, usSubIndex, ubObjectSubIndex, &errorInfo);
        }

        //Activate number of channels
        if(oResult)
        {
            usIndex = INDEX_NB_OF_SAMPLING_VARIABLES;
            usSubIndex = SUBINDEX_NB_OF_SAMPLING_VARIABLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, usIndex, usSubIndex, ubChannelNumber, &errorInfo);
        }

        if (oResult)
        {
            oResult = pRecorderChannelObjects->SetChannel(ubChannelNumber - 1, usObjectIndex, ubObjectSubIndex, ubObjectSize);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ActiveChannel(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubChannelNumber;
    BYTE ubNodeId(0);
    BYTE ubObjectSize(0);
    BYTE ubObjectSubIndex;
    CErrorInfo errorInfo;
    CRecorderChannelObjects* pRecorderChannelObjects(GetRecorderChannelObjects());
    WORD usIndex;
    WORD usObjectIndex;
    WORD usObjectSubIndex;
    WORD usSubIndex;

    if (p_pCommand && pRecorderChannelObjects)
    {
        // Get parameter data
        p_pCommand->GetParameterData(0, &ubChannelNumber, sizeof(ubChannelNumber));

        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Channel object index
        if (oResult)
        {
            usIndex = INDEX_VARIABLE_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, usIndex, usSubIndex, &usObjectIndex, &errorInfo);
        }

        // Channel object subindex
        if (oResult)
        {
            usIndex = INDEX_VARIABLE_SUB_INDEX;
            usSubIndex = ubChannelNumber;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, usIndex, usSubIndex, &usObjectSubIndex, &errorInfo);
        }

        if (oResult)
        {
            ubObjectSubIndex = (BYTE)usObjectSubIndex;

            if(!pRecorderChannelObjects->IsEqual(ubChannelNumber - 1, usObjectIndex, ubObjectSubIndex))
            {
                oResult = pRecorderChannelObjects->SetChannel(ubChannelNumber - 1, usObjectIndex, ubObjectSubIndex, ubObjectSize);
            }
        }

        // Set return parameter data
        p_pCommand->SetParameterData(0, &usObjectIndex, sizeof(usObjectIndex));
        p_pCommand->SetParameterData(1, &ubObjectSubIndex, sizeof(ubObjectSubIndex));
        p_pCommand->SetParameterData(2, &ubObjectSize, sizeof(ubObjectSize));
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_DeactivateAllChannels(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    CRecorderChannelObjects* pRecorderChannelObjects(GetRecorderChannelObjects());
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usDefaultValue = 0;

    if (p_pCommand && pRecorderChannelObjects)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Number of sampling variables
        if(oResult)
        {
            usIndex = INDEX_NB_OF_SAMPLING_VARIABLES;
            usSubIndex = SUBINDEX_NB_OF_SAMPLING_VARIABLES;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
        }

        for(BYTE ubChannelIndex = 0; ubChannelIndex < MAX_NB_OF_DATA_RECORDER_VARIABLES; ubChannelIndex++)
        {
            //Channel object index
            if(oResult)
            {
                usIndex = INDEX_VARIABLE_INDEX;
                usSubIndex = ubChannelIndex + 1;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
            }

            //Channel object subindex
            if(oResult)
            {
                usIndex = INDEX_VARIABLE_SUB_INDEX;
                usSubIndex = ubChannelIndex + 1;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDefaultValue, &errorInfo);
            }

            if (oResult)
            {
                oResult = pRecorderChannelObjects->SetChannel(ubChannelIndex, usDefaultValue, (BYTE)usDefaultValue, (BYTE)usDefaultValue);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetVariable(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usObjectIndex(0);
    WORD usIndex;
    WORD usSubIndex;
    WORD usVariableIndex;
    WORD usVariableSubIndex;

    if (p_pCommand)
    {
        p_pCommand->GetParameterData(0, &usObjectIndex, sizeof(usObjectIndex));

        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_VARIABLE_INDEX;
            usSubIndex = SUBINDEX_VARIABLE_INDEX + usObjectIndex;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVariableIndex, &errorInfo);
        }

        if (oResult)
        {
            usIndex = INDEX_VARIABLE_SUB_INDEX;
            usSubIndex = SUBINDEX_VARIABLE_SUB_INDEX + usObjectIndex;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVariableSubIndex, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usVariableIndex, sizeof(usVariableIndex));
        p_pCommand->SetReturnParameterData(1, &usVariableSubIndex, sizeof(usVariableSubIndex));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetMaxNbOfSamples(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usMaxNbOfSamples(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_MAX_NB_OF_SAMPLES;
            usSubIndex = SUBINDEX_MAX_NB_OF_SAMPLES;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMaxNbOfSamples, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetNbOfRecordedSamples(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usNbOfRecordedSamples(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_NB_OF_RECORDED_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_RECORDED_SAMPLES;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfRecordedSamples, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetNbOfPrecedingSamples(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usNbOfPrecedingSamples(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        p_pCommand->GetParameterData(0, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));

        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;

            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usNbOfPrecedingSamples, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetNbOfPrecedingSamples(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usNbOfPrecedingSamples(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_NB_OF_PRECEDING_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_PRECEDING_SAMPLES;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfPrecedingSamples, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usNbOfPrecedingSamples, sizeof(usNbOfPrecedingSamples));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetNbOfSamplingVariables(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usNbOfSamplingVariables(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_NB_OF_SAMPLING_VARIABLES;
            usSubIndex = SUBINDEX_NB_OF_SAMPLING_VARIABLES;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfSamplingVariables, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usNbOfSamplingVariables, sizeof(usNbOfSamplingVariables));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetSamplingPeriod(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usSamplingPeriod(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_SAMPLING_PERIOD;
            usSubIndex = SUBINDEX_SAMPLING_PERIOD;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSamplingPeriod, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usSamplingPeriod, sizeof(usSamplingPeriod));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_StartRecorder(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD START_RECORDER = 0x0001;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = START_RECORDER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_StopRecorder(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD STOP_RECORDER = 0xFFFE;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = STOP_RECORDER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ForceTrigger(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD FORCE_TRIGGER = 0x0002;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usRecorderControl = 0;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder control
        if(oResult)
        {
            usIndex = INDEX_RECORDER_CONTROL;
            usSubIndex = SUBINDEX_RECORDER_CONTROL;
            usRecorderControl = FORCE_TRIGGER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderControl, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_IsRecorderRunning(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD RECORDER_RUNNING = 0x0001;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BOOL oRecorderRunning = FALSE;
    WORD usRecorderStatus;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder status
        if(oResult)
        {
            usIndex = INDEX_RECORDER_STATUS;
            usSubIndex = SUBINDEX_RECORDER_STATUS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderStatus, &errorInfo);

            if(oResult)
            {
                if(usRecorderStatus & RECORDER_RUNNING)
                {
                    oRecorderRunning = TRUE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oRecorderRunning, sizeof(oRecorderRunning));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_IsRecorderTriggered(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD RECORDER_TRIGGERED = 0x0002;

    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    BOOL oRecorderTriggered = FALSE;
    WORD usRecorderStatus;

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Recorder status
        if(oResult)
        {
            usIndex = INDEX_RECORDER_STATUS;
            usSubIndex = SUBINDEX_RECORDER_STATUS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderStatus, &errorInfo);

            if(oResult)
            {
                if(usRecorderStatus & RECORDER_TRIGGERED)
                {
                    oRecorderTriggered = TRUE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oRecorderTriggered, sizeof(oRecorderTriggered));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetRecorderStatus(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usRecorderStatus(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_RECORDER_STATUS;
            usSubIndex = SUBINDEX_RECORDER_STATUS;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderStatus, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usRecorderStatus, sizeof(usRecorderStatus));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_SetRecorderConfiguration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usRecorderConfiguration(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        p_pCommand->GetParameterData(0, &usRecorderConfiguration, sizeof(usRecorderConfiguration));

        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;

            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usRecorderConfiguration, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetRecorderConfiguration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usRecorderConfiguration(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_RECORDER_CONFIGURATION;
            usSubIndex = SUBINDEX_RECORDER_CONFIGURATION;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usRecorderConfiguration, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usRecorderConfiguration, sizeof(usRecorderConfiguration));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ReadChannelVectorSize(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult = FALSE;
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    WORD usVectorSize = 0;
    DWORD ulVectorSize = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Max number of samples
        if(oResult)
        {
            usIndex = INDEX_MAX_NB_OF_SAMPLES;
            usSubIndex = SUBINDEX_MAX_NB_OF_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVectorSize, &errorInfo);
        }

        ulVectorSize = usVectorSize;

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulVectorSize, sizeof(ulVectorSize));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_GetVectorStartOffset(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usVectorStartOffset(0);
    WORD usIndex;
    WORD usSubIndex;

    if (p_pCommand)
    {
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if (oResult)
        {
            usIndex = INDEX_VECTOR_START_OFFSET;
            usSubIndex = SUBINDEX_VECTOR_START_OFFSET;

            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVectorStartOffset, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usVectorStartOffset, sizeof(usVectorStartOffset));
    }

    return oResult;
}

BOOL CGatewayDrive2ToEpos2::Process_ReadDataBuffer(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usIndex;
    BYTE ubNodeId(0);
    WORD usSubIndex;
    void* pubDataBuffer(NULL);
    DWORD ulBufferSizeToRead;
    DWORD ulBufferSizeRead(0);
    WORD usVectorStartOffset(0);
    WORD usMaxNbOfSamples(0);
    WORD usNbOfRecordedSamples(0);

    if(p_pCommand)
    {
        //Init
        oResult = TRUE;

        //Get parameter data
        p_pCommand->GetParameterData(0, &ulBufferSizeToRead, sizeof(ulBufferSizeToRead));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read data buffer
        if(oResult)
        {
            pubDataBuffer = malloc(ulBufferSizeToRead);
            usIndex = INDEX_DATA_BUFFER;
            usSubIndex = SUBINDEX_DATA_BUFFER;
            oResult = ReadDataBufferObject(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, pubDataBuffer, ulBufferSizeToRead, &ulBufferSizeRead, &errorInfo);
        }

        //Vector start offset
        if(oResult)
        {
            usIndex = INDEX_VECTOR_START_OFFSET;
            usSubIndex = SUBINDEX_VECTOR_START_OFFSET;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVectorStartOffset, &errorInfo);
        }

        //Max number of samples
        if(oResult)
        {
            usIndex = INDEX_MAX_NB_OF_SAMPLES;
            usSubIndex = SUBINDEX_MAX_NB_OF_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMaxNbOfSamples, &errorInfo);
        }

        //Number of recorded samples
        if(oResult)
        {
            usIndex = INDEX_NB_OF_RECORDED_SAMPLES;
            usSubIndex = SUBINDEX_NB_OF_RECORDED_SAMPLES;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfRecordedSamples, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, pubDataBuffer, ulBufferSizeToRead);
        p_pCommand->SetReturnParameterData(1, &ulBufferSizeRead, sizeof(ulBufferSizeRead));
        p_pCommand->SetReturnParameterData(2, &usVectorStartOffset, sizeof(usVectorStartOffset));
        p_pCommand->SetReturnParameterData(3, &usMaxNbOfSamples, sizeof(usMaxNbOfSamples));
        p_pCommand->SetReturnParameterData(4, &usNbOfRecordedSamples, sizeof(usNbOfRecordedSamples));

        if(pubDataBuffer) free(pubDataBuffer);
    }

    return oResult;
}
