#include "stdafx.h"

#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <CommunicationModel/VirtualCommandSet/Classes/VirtualDevices/BaseClasses/VirtualDeviceBase.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Handle/HandleRegistration_VCS.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Handle/HandleRegistrationMap_VCS.h>

std::map<void*, void*>* CHandleRegistrationMap_VCS::m_pSingletonMap = 0;

CHandleRegistrationMap_VCS::CHandleRegistrationMap_VCS():
    m_pErrorHandling(0)
{
    InitErrorHandling();
} 

CHandleRegistrationMap_VCS::~CHandleRegistrationMap_VCS()
{
    DeleteErrorHandling();
    DeleteHandleRegistrationList();
}

BOOL CHandleRegistrationMap_VCS::InitErrorHandling()
{
    BOOL oResult = FALSE;
    CErrorProducer errorProducer;
    CStdString className = "HandleRegistrationMap_VCS";

    DeleteErrorHandling();
    m_pErrorHandling = new CErrorHandling();
    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(VIRTUAL_COMMAND_SET_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::DeleteErrorHandling()
{
    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = NULL;
    }

    return TRUE;
}

CHandleRegistrationMap_VCS* CHandleRegistrationMap_VCS::GetInstance(int p_lInstanceValue, void* p_pParent)
{
    CHandleRegistrationMap_VCS* pInstance = NULL;

    //SingletonMap
    if(!m_pSingletonMap)
        m_pSingletonMap = new std::map<void*, void*>();

    if(m_pSingletonMap)
    {
        std::map<void*, void*>::iterator it = m_pSingletonMap->find((void*)(intptr_t)p_lInstanceValue);

        CHandleRegistrationMap_VCS::CSingleton* pSingleton = 0;

        if(it == m_pSingletonMap->end())
        {
            pSingleton = new CHandleRegistrationMap_VCS::CSingleton();
            m_pSingletonMap->insert(std::pair<void*, void*>((void*)(intptr_t)p_lInstanceValue, (void*)pSingleton));
        }
        else
            pSingleton = (CHandleRegistrationMap_VCS::CSingleton*)(*it).second;

        //Get Instance
        if(pSingleton != NULL)
            pInstance = pSingleton->GetInstance(p_pParent);
    }

    return pInstance;
}

BOOL CHandleRegistrationMap_VCS::ReleaseInstance(int p_lInstanceValue, void* p_pParent)
{
    BOOL oResult = FALSE;

    if(m_pSingletonMap!=NULL)
    {
        CHandleRegistrationMap_VCS::CSingleton* pSingleton =
                (CHandleRegistrationMap_VCS::CSingleton*)(*m_pSingletonMap)[(void*)(intptr_t)p_lInstanceValue];

        if(pSingleton != NULL)
        {
            //Release Instance
            oResult = pSingleton->ReleaseInstance(p_pParent);

            //Delete Singleton
            if(pSingleton->GetParentCount() == 0)
            {
                m_pSingletonMap->erase((void*)(intptr_t)p_lInstanceValue);

                delete pSingleton;
                pSingleton = NULL;
            }

            //Delete Map
            if(m_pSingletonMap->size() == 0)
            {
                delete m_pSingletonMap;
                m_pSingletonMap = NULL;
            }
        }
    }

    return oResult;
}

void CHandleRegistrationMap_VCS::DeleteHandleRegistrationList()
{
    //Delete whole registration
    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);
        if(pHandleRegistration != NULL)
        {
            delete pHandleRegistration;
            pHandleRegistration = NULL;
        }
    }

    m_HandleRegistrationList.clear();
}

BOOL CHandleRegistrationMap_VCS::GetRegistrationValues(HANDLE p_hVCS_Handle, CVirtualDeviceBase** p_ppVirtualDevice, CDeviceCommandSetManagerBase** p_ppManager, HANDLE* p_phDeviceCommandSetHandle)
{
    BOOL oResult(FALSE);

    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);
        if(pHandleRegistration != NULL)
        {
            if(pHandleRegistration->GetKeyHandle() == p_hVCS_Handle)
            {
                oResult = pHandleRegistration->GetRegistrationValues(p_ppVirtualDevice, p_ppManager, p_phDeviceCommandSetHandle);
                break;
            }
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::GetRegistration(HANDLE p_hVCS_Handle, CHandleRegistration_VCS** p_ppRegistration)
{
    BOOL oResult(FALSE);

    if((p_ppRegistration) && (p_hVCS_Handle))
    {
        for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
        {
            CHandleRegistration_VCS* pHandleRegistration = (*it);
            if(pHandleRegistration != NULL)
            {
                if(pHandleRegistration->GetKeyHandle() == p_hVCS_Handle)
                {
                    *p_ppRegistration = pHandleRegistration;

                    oResult = TRUE;
                    break;
                }
            }
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::GetRegistrationList(tHandleRegistrationVCS_List** p_ppList)
{
    BOOL oResult(FALSE);

    if(p_ppList)
    {
        *p_ppList = &m_HandleRegistrationList;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::IsLastRegistration(HANDLE p_hVCS_Handle, HANDLE p_hDCS_Handle)
{
    int lCount = 0;

    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);
        if(pHandleRegistration != NULL)
        {
            HANDLE hDCS_Handle = NULL;
            HANDLE hVCS_Handle = pHandleRegistration->GetKeyHandle();
            pHandleRegistration->GetRegistrationValues(NULL, NULL, &hDCS_Handle);

            //Check
            if((p_hVCS_Handle != hVCS_Handle) && (p_hDCS_Handle == hDCS_Handle))
            {
                lCount++;
            }
        }
    }

    return (BOOL)(lCount == 0);
}

BOOL CHandleRegistrationMap_VCS::IsRegistrationMapEmpty()
{
    BOOL oResult(TRUE);

    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);
        if(pHandleRegistration != NULL && pHandleRegistration->GetKeyHandle())
        {
            oResult = FALSE;
            break;
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::DeleteMapRegistration(HANDLE p_hVCS_Handle)
{
    BOOL oResult(FALSE);

    //Werte aus der Liste loeschen. Position der Liste bleibt bestehen
    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);
        if(pHandleRegistration != NULL)
        {
            if(pHandleRegistration->GetKeyHandle() == p_hVCS_Handle)
            {
                pHandleRegistration->DeleteRegistration();

                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

HANDLE CHandleRegistrationMap_VCS::RegisterHandle(CVirtualDeviceBase* p_pVirtualDevice, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDeviceCommandSetHandle)
{
    HANDLE hVCS_Handle;

    //Always add new registration for virtual device
    hVCS_Handle = AddNewRegistration(p_pVirtualDevice, p_pManager, p_hDeviceCommandSetHandle);

    return hVCS_Handle;
}

HANDLE CHandleRegistrationMap_VCS::AddNewRegistration(CVirtualDeviceBase* p_pVirtualDevice, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDeviceCommandSetHandle)
{
    CHandleRegistration_VCS* pHandleRegistration = NULL;
    HANDLE hRegistration = 0;

    pHandleRegistration = GetNextFreeRegistration();

    if(pHandleRegistration)
    {
        pHandleRegistration->Init(p_pVirtualDevice, p_pManager, p_hDeviceCommandSetHandle);

        hRegistration = pHandleRegistration->GetKeyHandle();
    }

    return hRegistration;
}

CHandleRegistration_VCS* CHandleRegistrationMap_VCS::GetNextFreeRegistration()
{
    //In der Liste leere Positionen aus vorherige geloeschte Objekte suchen
    // Falls keine Vorhanden, neue Position einfuegen und handle zurueckgeben
    CHandleRegistration_VCS* pHandleRegistration = NULL;
    DWORD ulKeyHandleAddress = ((DWORD)VIRTUAL_COMMAND_SET_LAYER << 16) + 1;
    HANDLE hKeyHandle = 0;

    for(std::list<CHandleRegistration_VCS*>::iterator it = m_HandleRegistrationList.begin(); it != m_HandleRegistrationList.end(); it++)
    {
        CHandleRegistration_VCS* pHandleRegistration = (*it);

        if(pHandleRegistration != NULL && pHandleRegistration->IsEmpty())
        {
            hKeyHandle = (HANDLE)(intptr_t)ulKeyHandleAddress;

            pHandleRegistration->SetKeyHandle(hKeyHandle);

            return pHandleRegistration;
        }
        ulKeyHandleAddress++;
    }

    pHandleRegistration = new CHandleRegistration_VCS();
    
    hKeyHandle = (HANDLE)(intptr_t)ulKeyHandleAddress;
    pHandleRegistration->SetKeyHandle(hKeyHandle);
    
    m_HandleRegistrationList.push_back(pHandleRegistration);

    return pHandleRegistration;
}

BOOL CHandleRegistrationMap_VCS::SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo)
{
    CVirtualDeviceBase* pVirtualDevice = NULL;
    CDeviceCommandSetManagerBase* pManager = NULL;
    HANDLE hDCS_Handle = 0;
    CHandleRegistration_VCS* pRegistration = NULL;
    BOOL oResult = FALSE;

    //Get registration
    if(GetRegistration(p_hVCS_Handle, &pRegistration) && (pRegistration != NULL))
    {
        //Lock
        if(pRegistration->DoLock())
        {
            if(pRegistration->GetRegistrationValues(&pVirtualDevice, &pManager, &hDCS_Handle) && pVirtualDevice && pManager && hDCS_Handle)
            {
                if(PT_VIRTUAL_DEVICE == p_eParameterType)
                {
                    //Set virtual device parameter
                    oResult = pVirtualDevice->SetParameter(p_Name, p_pValue, p_ulSize);
                    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PARAMETER, p_pErrorInfo);
                }
                else if(PT_VIRTUAL_DEVICE > p_eParameterType)
                {
                    //Forward Parameter to next layer
                    oResult = pManager->SetParameter(p_eParameterType, hDCS_Handle, p_Name, p_pValue, p_ulSize, p_pErrorInfo);
                }
            }

            //Unlock
            if(pRegistration) pRegistration->DoUnlock();
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::SetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString p_Value, CErrorInfo* p_pErrorInfo)
{
    CVirtualDeviceBase* pVirtualDevice = NULL;
    CDeviceCommandSetManagerBase* pManager = NULL;
    HANDLE hDCS_Handle = 0;
    CHandleRegistration_VCS* pRegistration = NULL;
    BOOL oResult = FALSE;

    //Get registration
    if(GetRegistration(p_hVCS_Handle, &pRegistration) && (pRegistration != NULL))
    {
        //Lock
        if(pRegistration->DoLock())
        {
            if(pRegistration->GetRegistrationValues(&pVirtualDevice, &pManager, &hDCS_Handle) && pVirtualDevice && pManager && hDCS_Handle)
            {
                if(PT_VIRTUAL_DEVICE == p_eParameterType)
                {
                    //Set virtual device parameter
                    oResult = pVirtualDevice->SetParameter(p_Name, p_Value);
                    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PARAMETER, p_pErrorInfo);
                }
                else if(PT_VIRTUAL_DEVICE > p_eParameterType)
                {
                    //Forward Parameter to next layer
                    oResult = pManager->SetParameter(p_eParameterType, hDCS_Handle, p_Name, p_Value, p_pErrorInfo);
                }
            }

            //Unlock
            if(pRegistration) pRegistration->DoUnlock();
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize, CErrorInfo* p_pErrorInfo)
{
    CVirtualDeviceBase* pVirtualDevice = NULL;
    CDeviceCommandSetManagerBase* pManager = NULL;
    HANDLE hDCS_Handle = 0;
    CHandleRegistration_VCS* pRegistration = NULL;
    BOOL oIsLocked = FALSE;
    BOOL oResult = FALSE;

    //Get registration
    if(GetRegistration(p_hVCS_Handle, &pRegistration) && (pRegistration != NULL))
    {
        //Lock
        oIsLocked = pRegistration->IsLocked();
        if(oIsLocked || pRegistration->DoLock())
        {
            if(pRegistration->GetRegistrationValues(&pVirtualDevice, &pManager, &hDCS_Handle) && pVirtualDevice && pManager && hDCS_Handle)
            {
                if(PT_VIRTUAL_DEVICE == p_eParameterType)
                {
                    //Get virtual device parameter
                    oResult = pVirtualDevice->GetParameter(p_Name, p_pValue, p_ulSize);
                    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PARAMETER, p_pErrorInfo);
                }
                else if(PT_VIRTUAL_DEVICE > p_eParameterType)
                {
                    //Forward Parameter to next layer
                    oResult = pManager->GetParameter(p_eParameterType, hDCS_Handle, p_Name, p_pValue, p_ulSize, p_pErrorInfo);
                }
            }

            //Unlock
            if(pRegistration && !oIsLocked) pRegistration->DoUnlock();
        }
    }

    return oResult;
}

BOOL CHandleRegistrationMap_VCS::GetParameter(EParameterType p_eParameterType, HANDLE p_hVCS_Handle, CStdString p_Name, CStdString& p_rValue, CErrorInfo* p_pErrorInfo)
{
    CVirtualDeviceBase* pVirtualDevice = NULL;
    CDeviceCommandSetManagerBase* pManager = NULL;
    HANDLE hDCS_Handle = 0;
    CHandleRegistration_VCS* pRegistration = NULL;
    BOOL oIsLocked = FALSE;
    BOOL oResult = FALSE;

    //Get registration
    if(GetRegistration(p_hVCS_Handle, &pRegistration) && (pRegistration != NULL))
    {
        //Lock
        oIsLocked = pRegistration->IsLocked();
        if(oIsLocked || pRegistration->DoLock())
        {
            if(pRegistration->GetRegistrationValues(&pVirtualDevice, &pManager, &hDCS_Handle) && pVirtualDevice && pManager && hDCS_Handle)
            {
                if(PT_VIRTUAL_DEVICE == p_eParameterType)
                {
                    //Set virtual device parameter
                    oResult = pVirtualDevice->SetParameter(p_Name, p_rValue);
                    if(!oResult && m_pErrorHandling) m_pErrorHandling->GetError(ERROR_BAD_PARAMETER, p_pErrorInfo);
                }
                else if(PT_VIRTUAL_DEVICE > p_eParameterType)
                {
                    //Forward Parameter to next layer
                    oResult = pManager->SetParameter(p_eParameterType, hDCS_Handle, p_Name, p_rValue, p_pErrorInfo);
                }
            }

            //Unlock
            if(pRegistration && !oIsLocked) pRegistration->DoUnlock();
        }
    }

    return oResult;
}

/************************************************************************/
//Embedded Singleton Class
/************************************************************************/
CHandleRegistrationMap_VCS::CSingleton::CSingleton():
    m_pRegistrationMap(0)
{
    m_pRegistrationMap = new CHandleRegistrationMap_VCS();
}

CHandleRegistrationMap_VCS::CSingleton::~CSingleton()
{
    if(m_pRegistrationMap) delete m_pRegistrationMap;
}

CHandleRegistrationMap_VCS* CHandleRegistrationMap_VCS::CSingleton::GetInstance(void* p_pParent)
{
    bool found = false;

    for(std::list<void*>::iterator it = m_ParentRegistrationList.begin(); it != m_ParentRegistrationList.end(); it++)
    {
        void* pRegistration = (*it);
        if(pRegistration == p_pParent)
            found = true;
    }

    if(!found)
        m_ParentRegistrationList.push_back(p_pParent);

    //Instance
    return m_pRegistrationMap;
}

BOOL CHandleRegistrationMap_VCS::CSingleton::ReleaseInstance(void* p_pParent)
{
    BOOL oResult = FALSE;
    bool oFound = false;

    for(std::list<void*>::iterator it = m_ParentRegistrationList.begin();it != m_ParentRegistrationList.end(); it++)
    {
        void* pRegistration = (*it);
        if(pRegistration == p_pParent)
            oFound = true;
    }

    //Unregister Parent
    if(oFound)
    {
        m_ParentRegistrationList.remove(p_pParent);
        oResult = TRUE;
    }

    return oResult;
}

int CHandleRegistrationMap_VCS::CSingleton::GetParentCount()
{
    return (int)m_ParentRegistrationList.size();
}

