#include "stdafx.h"
#include <algorithm>

#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>
#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <CommunicationModel/DeviceCommandSet/DeviceCommandSetManager.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/BaseClasses/GatewayVCStoDCS.h>

#include <CommunicationModel/VirtualCommandSet/Classes/VirtualDevices/BaseClasses/VirtualDeviceBase.h>

using namespace std;

CVirtualDeviceBase::CVirtualDeviceBase()
{
    m_lInstanceValue = 0;
    m_strVirtualDeviceName = "";
    m_pGateway = 0;
    m_pJournalManager = 0;

    m_pErrorHandling = 0;
    InitErrorHandling();
}

CVirtualDeviceBase::CVirtualDeviceBase(const CVirtualDeviceBase& p_rObject)
{
    //Instance value
    m_lInstanceValue = p_rObject.m_lInstanceValue;

    //Copy device name
    m_strVirtualDeviceName = p_rObject.m_strVirtualDeviceName;

    //Gateway
    if(p_rObject.m_pGateway)
    {
        m_pGateway = (CGatewayVCStoDCS*)p_rObject.m_pGateway->Clone();
    }
    else
    {
        m_pGateway = 0;
    }

    //Journal manager
    m_pJournalManager = p_rObject.m_pJournalManager;

    m_pErrorHandling = 0;
    InitErrorHandling();
}

BOOL CVirtualDeviceBase::InitInstanceValue(int p_lInstanceValue)
{
    BOOL oResult(TRUE);

    m_lInstanceValue = p_lInstanceValue;

    return oResult;
}

BOOL CVirtualDeviceBase::InitErrorHandling()
{
    DeleteErrorHandling();
    m_pErrorHandling = new CErrorHandling();

    return TRUE;
}

BOOL CVirtualDeviceBase::DeleteErrorHandling()
{
    if(m_pErrorHandling)
    {
        delete m_pErrorHandling;
        m_pErrorHandling = 0;
    }

    return TRUE;
}

CVirtualDeviceBase::~CVirtualDeviceBase()
{
    DeleteErrorHandling();
    DeleteGateway();
    DeleteDCS_ManagerList();
}

BOOL CVirtualDeviceBase::InitVirtualDevice(CStdString p_deviceName, CErrorInfo* p_pErrorInfo)
{
    CDeviceCommandSetManagerBase* pManager(0);
    BOOL oNewCreated(FALSE);
    BOOL oResult(FALSE);

    if(p_deviceName.IsEmpty())
    {
        //Don't create a device
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_deviceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CDeviceCommandSetManagerBase*) new CDeviceCommandSetManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create class structure for device
        if((VerifyGateway(p_deviceName)) && (pManager->DCS_InitDevice(p_deviceName, p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_DeviceCommandSetManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)delete pManager;
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::InitVirtualDevice(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CErrorInfo* p_pErrorInfo)
{
    CDeviceCommandSetManagerBase* pManager(0);
    BOOL oNewCreated(FALSE);
    BOOL oResult(FALSE);

    if(p_deviceName.IsEmpty())
    {
        //Don't create a device
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_deviceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CDeviceCommandSetManagerBase*) new CDeviceCommandSetManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create class structure for device
        if((VerifyGateway(p_deviceName)) && (pManager->DCS_InitDevice(p_deviceName, p_protocolStackName, p_interfaceName, p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_DeviceCommandSetManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::InitVirtualDevice(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_driverName, CErrorInfo* p_pErrorInfo)
{
    BOOL oNewCreated(FALSE);
    BOOL oResult(FALSE);
    CDeviceCommandSetManagerBase* pManager(0);

    if(p_deviceName.size() == 0)
    {
        //Don't create a device
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_deviceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CDeviceCommandSetManagerBase*) new CDeviceCommandSetManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create class structure for device
        if((VerifyGateway(p_deviceName)) && (pManager->DCS_InitDevice(p_deviceName, p_protocolStackName, p_interfaceName, p_driverName, p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_DeviceCommandSetManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::InitVirtualGatewayDevice(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_gatewayDeviceName, CStdString p_gatewayProtocolStackName, CErrorInfo* p_pErrorInfo)
{
    BOOL oNewCreated(FALSE);
    BOOL oResult(FALSE);
    CDeviceCommandSetManagerBase* pManager(0);

    if(p_deviceName.size() == 0)
    {
        //Don't create a device
        oResult = TRUE;
    }
    else
    {
        //Is manager already existing
        if(!FindCorrectManager(p_deviceName, &pManager) && !GetFirstManager(&pManager))
        {
            //Create new manager
            pManager = (CDeviceCommandSetManagerBase*) new CDeviceCommandSetManager(m_lInstanceValue);
            oNewCreated = TRUE;
        }

        //Create class structure for device
        if((VerifyGateway(p_deviceName)) && (pManager->DCS_InitGatewayDevice(p_deviceName, p_protocolStackName, p_gatewayDeviceName, p_gatewayProtocolStackName, p_pErrorInfo)))
        {
            if(oNewCreated)
            {
                m_DeviceCommandSetManagerList.push_back(pManager);
            }

            oResult = TRUE;
        }
        else
        {
            if(oNewCreated)
            {
                delete pManager;
            }

            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::InitVirtualDevice(CErrorInfo* p_pErrorInfo)
{
    return FALSE;
}

BOOL CVirtualDeviceBase::VCS_GetVirtualDeviceName(CStdString* p_pVirtualDeviceName)
{
    BOOL oResult(FALSE);

    if(p_pVirtualDeviceName)
    {
        *p_pVirtualDeviceName = m_strVirtualDeviceName;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_ResetPortEnumeration(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if(!FindCorrectManager(p_deviceName, &pManager) || (pManager == 0))
    {
        if(m_pErrorHandling)
        {
            m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        }

        oResult = FALSE;
    }

    //Enumeration
    if(oResult && !pManager->DCS_ResetPortEnumeration(p_deviceName, p_protocolStackName, p_interfaceName, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

void CVirtualDeviceBase::DeleteDCS_ManagerList()
{
    CDeviceCommandSetManagerBase* pManager = 0;
    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        pManager = (*it);
        delete pManager;
    }
    m_DeviceCommandSetManagerList.clear();
}

void CVirtualDeviceBase::DeleteGateway()
{
    if(m_pGateway)
    {
        delete m_pGateway;
        m_pGateway = 0;
    }
}

BOOL CVirtualDeviceBase::AddManager(CDeviceCommandSetManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_DeviceCommandSetManagerList.push_back(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::RemoveManager(CDeviceCommandSetManagerBase* p_pManager)
{
    BOOL oResult(FALSE);

    if(p_pManager)
    {
        m_DeviceCommandSetManagerList.remove(p_pManager);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::GetFirstManager(CDeviceCommandSetManagerBase** p_ppManager)
{
    CDeviceCommandSetManagerBase* pListManager = 0;
    BOOL oResult(FALSE);

    list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin();

    if(it != m_DeviceCommandSetManagerList.end())
    {
        pListManager = (*it);
        if(pListManager)
        {
            if(p_ppManager) *p_ppManager = pListManager;
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::FindCorrectManager(CStdString p_deviceName, CDeviceCommandSetManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    if(p_ppManager)
    {
        for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
        {
            CDeviceCommandSetManager* pListManager = (CDeviceCommandSetManager*)(*it);
            if(pListManager)
            {
                if(pListManager->DCS_IsDeviceNameSupported(p_deviceName))
                {
                    if(p_ppManager) *p_ppManager = pListManager;
                    oResult = TRUE;
                    break;
                }
            }
        }

        if(!oResult) oResult = FindCorrectLibrary(p_deviceName, p_ppManager);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::FindCorrectLibrary(CStdString p_deviceName, CDeviceCommandSetManagerBase** p_ppManager)
{
    BOOL oResult(FALSE);

    if(p_ppManager)
    {
        *p_ppManager = 0;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_GetDeviceNameSelection(CStdStringArray* p_pDeviceNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CStdStringArray selArray;

    if(p_pDeviceNameSel)
    {
        p_pDeviceNameSel->clear();

        for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
        {
            CDeviceCommandSetManagerBase* pManager = (*it);
            if(pManager)
            {
                if(pManager->DCS_GetDeviceNameSelection(&selArray))
                {
                    AppendSelectionArray(p_pDeviceNameSel, &selArray);
                }
            }
        }

        oResult = TRUE;
    }

    if(!oResult && m_pErrorHandling)
    {
        m_pErrorHandling->GetError(ERROR_NULL_POINTER, p_pErrorInfo);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::AppendSelectionArray(CStdStringArray* p_pDestArray, CStdStringArray* p_pSourceArray)
{
    BOOL oResult(FALSE);
    CStdString strDestName;
    CStdString strSourceName;
    BOOL oFound;

    if(p_pDestArray && p_pSourceArray)
    {
        for(CStdStringArray::iterator it = p_pSourceArray->begin(); it != p_pSourceArray->end(); it++)
        {
            oFound = FALSE;
            strSourceName = (*it);
            for(CStdStringArray::iterator it2 = p_pDestArray->begin(); it2 != p_pDestArray->end(); it2++)
            {
                strDestName = (*it2);

                if(strSourceName.CompareNoCase(strDestName) == 0)
                {
                    oFound = TRUE;
                    break;
                }
            }

            if(!oFound)
            {
                p_pDestArray->push_back(strSourceName);
            }
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_GetProtocolStackNameSelection(CStdString p_deviceName, CStdStringArray* p_pProtocolStackNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if(!FindCorrectManager(p_deviceName, &pManager) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get protocol stack name selection
    if(oResult && !pManager->DCS_GetProtocolStackNameSelection(p_deviceName, p_pProtocolStackNameSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_GetInterfaceNameSelection(CStdString p_deviceName, CStdString p_protocolStackName, CStdStringArray* p_pInterfaceNameSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if(!FindCorrectManager(p_deviceName, &pManager) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get interface name selection
    if(oResult && !pManager->DCS_GetInterfaceNameSelection(p_deviceName, p_protocolStackName, p_pInterfaceNameSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_GetPortNameSelection(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdStringArray* p_pPortSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if(!FindCorrectManager(p_deviceName, &pManager) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get port name selection
    if(oResult && !pManager->DCS_GetPortNameSelection(p_deviceName, p_protocolStackName, p_interfaceName, p_pPortSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::VCS_GetBaudrateSelection(CStdString p_deviceName, CStdString p_protocolStackName, CStdString p_interfaceName, CStdString p_portName, CStdDWordArray* p_pulBaudrateSel, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pManager = 0;

    //Find manager
    if(!FindCorrectManager(p_deviceName, &pManager) || (pManager == 0))
    {
        if(m_pErrorHandling) m_pErrorHandling->GetError(ERROR_INTERNAL, p_pErrorInfo);
        oResult = FALSE;
    }

    //Get baudrate selection
    if(oResult && !pManager->DCS_GetBaudrateSelection(p_deviceName, p_protocolStackName, p_interfaceName, p_portName, p_pulBaudrateSel, p_pErrorInfo))
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::GetCommands(CStdString* p_pCommandInfo)
{
    return FALSE;
}

void CVirtualDeviceBase::InitJournalManager(CJournalManagerBase* p_pJournalManager)
{
    CDeviceCommandSetManagerBase* pManager = 0;

    m_pJournalManager = p_pJournalManager;

    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        pManager = (*it);
        pManager->InitJournalManager(p_pJournalManager);
    }
}

void CVirtualDeviceBase::ResetJournalManager()
{
    CDeviceCommandSetManagerBase* pManager = 0;

    m_pJournalManager = 0;

    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        pManager = (*it);
        pManager->ResetJournalManager();
    }
}

BOOL CVirtualDeviceBase::VerifyGateway(CStdString p_virtualDeviceName)
{
    return FALSE;
}

BOOL CVirtualDeviceBase::InitGateway(CStdString p_virtualDeviceName)
{
    return FALSE;
}

BOOL CVirtualDeviceBase::InitBaudrateSelection(CStdString p_deviceName, CStdString p_protocolStack, CStdString p_interfaceName, CStdDWordArray& p_ulBaudrateSel)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager = 0;

    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        pDeviceCommandSetManager = (*it);
        if(pDeviceCommandSetManager)
        {
            if(!pDeviceCommandSetManager->DCS_InitBaudrateSelection(p_deviceName, p_protocolStack, p_interfaceName, p_ulBaudrateSel))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CVirtualDeviceBase::InitDefaultProtocolStackSettings(CStdString p_deviceName, CStdString p_protocolStack, CStdString p_interfaceName, DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
    BOOL oResult(TRUE);
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager = 0;

    for(list<CDeviceCommandSetManagerBase*>::iterator it = m_DeviceCommandSetManagerList.begin(); it != m_DeviceCommandSetManagerList.end(); it++)
    {
        pDeviceCommandSetManager = (*it);
        if(pDeviceCommandSetManager)
        {
            if(!pDeviceCommandSetManager->DCS_InitDefaultProtocolStackSettings(p_deviceName, p_protocolStack, p_interfaceName, p_ulBaudrate, p_ulTimeout))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

CVirtualDeviceBase* CVirtualDeviceBase::Clone()
{
    return 0;
}

BOOL CVirtualDeviceBase::IsEqual(CVirtualDeviceBase* p_pVirtualDevice)
{
    BOOL oResult(FALSE);

    if(p_pVirtualDevice)
    {
        oResult = TRUE;
        if(oResult && (m_strVirtualDeviceName != p_pVirtualDevice->m_strVirtualDeviceName)) oResult = FALSE;
        if(oResult && (m_lInstanceValue != p_pVirtualDevice->m_lInstanceValue)) oResult = FALSE;
    }

    return oResult;
}

BOOL CVirtualDeviceBase::ExecuteCommand(CCommandRoot* p_pCommand, CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(p_pCommand)
    {
        //Init references
        p_pCommand->InitGateway(m_pGateway);
        p_pCommand->InitJournalManager(m_pJournalManager);

        //Execute
        oResult = p_pCommand->Execute(p_pDeviceCommandSetManager, p_hDCS_Handle, p_hTransactionHandle);

        //Reset references
        p_pCommand->ResetGateway();
        p_pCommand->ResetJournalManager();
    }

    return oResult;
}

BOOL CVirtualDeviceBase::AbortCommands(BOOL p_oActive)
{
    BOOL oResult(TRUE);

    if(m_pGateway)
    {
        oResult = m_pGateway->AbortCommands(p_oActive);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::AreParameterEqual(CVirtualDeviceBase* p_pVirtualDevice)
{
    BOOL oResult(FALSE);

    if(p_pVirtualDevice && m_pGateway)
    {
        oResult = m_pGateway->AreParameterEqual(p_pVirtualDevice->m_pGateway);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::SetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::SetParameter(CStdString p_Name, CStdString p_Value)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->SetParameter(p_Name, p_Value);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::GetParameter(CStdString p_Name, BYTE* p_pValue, DWORD p_ulSize)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_pValue, p_ulSize);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::GetParameter(CStdString p_Name, CStdString& p_rValue)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->GetParameter(p_Name, p_rValue);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::Lock(DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Lock(p_ulTimeout);
    }

    return oResult;
}

BOOL CVirtualDeviceBase::Unlock()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->Unlock();
    }

    return oResult;
}

BOOL CVirtualDeviceBase::IsLocked()
{
    BOOL oResult(FALSE);

    if(m_pGateway)
    {
        oResult = m_pGateway->IsLocked();
    }

    return oResult;
}

BOOL CVirtualDeviceBase::SetGatewayDeviceTimeout(CDeviceCommandSetManagerBase* p_pDeviceCommandSetManager, HANDLE p_hDCS_Handle, DWORD p_ulGatewayTimeout)
{
    BOOL oResult(FALSE);
    HANDLE hTransactionHandle(0);

    if (m_pGateway)
    {
        oResult = m_pGateway->SetGatewayDeviceTimeout(p_pDeviceCommandSetManager, p_hDCS_Handle, hTransactionHandle, p_ulGatewayTimeout);
    }

    return oResult;
}
