// DRIVE_Drive.cpp: Implementierung der Klasse CVirtualDevice_Drive.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <CommunicationModel/VirtualCommandSet/Classes/VirtualDevices/VirtualDevice_Drive.h>

#include <algorithm>

#include <CommunicationModel/DeviceCommandSet/Classes/Devices/Device_Epos.h>
#include <CommunicationModel/DeviceCommandSet/DeviceCommandSetManager.h>

#include <CommunicationModel/VirtualCommandSet/Classes/CommandGroups/Drive/CommandGroupStandard_VCS_Drive.h>
#include <CommunicationModel/VirtualCommandSet/Classes/VirtualDevices/Factory/VirtualDeviceDriveFactory.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CVirtualDevice_Drive::CVirtualDevice_Drive()
{
    InitErrorHandling();

    FillGroupList();
    m_strVirtualDeviceName = VIRTUAL_DEVICE_DRIVE;
}

CVirtualDevice_Drive::CVirtualDevice_Drive(const CVirtualDevice_Drive& rObject):CVirtualDeviceBase(rObject)
{
    InitErrorHandling();

    FillGroupList();
    m_strVirtualDeviceName = rObject.m_strVirtualDeviceName;

    InitGroupList(m_pGateway);
    InitJournalManager(rObject.m_pJournalManager);
}

CVirtualDevice_Drive::~CVirtualDevice_Drive()
{
    DeleteGroupList();
}

BOOL CVirtualDevice_Drive::InitVirtualDevice(CErrorInfo* p_pErrorInfo)
{
    BOOL oResult = FALSE;

    std::list<CStdString> aIdentifiers;
    oResult = CVirtualDeviceDriveFactory::GetAllIdentifiers(aIdentifiers);

    if(oResult)
    {
        for(std::list<CStdString>::iterator it=aIdentifiers.begin(); it!=aIdentifiers.end() && oResult; it++)
        {
            if(!CVirtualDeviceBase::InitVirtualDevice(*it, p_pErrorInfo))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

BOOL CVirtualDevice_Drive::VerifyGateway(CStdString p_DeviceName)
{
    std::list<CStdString> aIdentifiers;

    BOOL oResult = CVirtualDeviceDriveFactory::GetAllIdentifiers(aIdentifiers);

    if(oResult)
    {
        for(std::list<CStdString>::iterator it=aIdentifiers.begin(); it!=aIdentifiers.end() && oResult; it++)
        {
            if(p_DeviceName.CompareNoCase((*it)) == 0)
            {
                return TRUE;
            }
        }
    }

    return FALSE;
}

BOOL CVirtualDevice_Drive::InitErrorHandling()
{
    CErrorProducer errorProducer;

    if(m_pErrorHandling)
    {
        //Init ErrorProducer
        errorProducer.Init(VIRTUAL_COMMAND_SET_LAYER, VIRTUAL_DEVICE_DRIVE);
        m_pErrorHandling->InitErrorProducer(&errorProducer);
        return TRUE;
    }

    return FALSE;
}

BOOL CVirtualDevice_Drive::InitGateway(CStdString p_DeviceName)
{
    DeleteGateway();

    BOOL oResult = CVirtualDeviceDriveFactory::CreateInstance(p_DeviceName, m_pGateway);

    if(oResult)
    {
        if(m_pGateway->InitGateway())
        {
            InitParameterSet();
            InitGroupList(m_pGateway);
            return TRUE;
        }
        else
        {
            DeleteGateway();
            return FALSE;
        }
    }

    return FALSE;
}

BOOL CVirtualDevice_Drive::InitParameterSet()
{
    BOOL oResult = FALSE;

    if(m_pGateway)
    {
        oResult = TRUE;
        if(oResult) oResult &= m_pGateway->ResetParameterSet();
    }

    return oResult;
}

BOOL CVirtualDevice_Drive::InitGroupList(CGateway* pGateway)
{
    if(m_pCommandGroupStandard && !m_pCommandGroupStandard->InitGateway(pGateway)) return FALSE;
    return TRUE;
}

void CVirtualDevice_Drive::FillGroupList()
{
    m_pCommandGroupStandard = new CCommandGroupStandard_VCS_Drive();
}

void CVirtualDevice_Drive::DeleteGroupList()
{
    if(m_pCommandGroupStandard)
    {
        delete m_pCommandGroupStandard;
        m_pCommandGroupStandard = 0;
    }
}

BOOL CVirtualDevice_Drive::GetCommands(CStdString* pCommandInfo)
{
    BOOL oResult = TRUE;

    if(pCommandInfo)
    {

        CXXMLFile xmlFile;

        //Root
        CXXMLFile::CElementPart* pElementPart = xmlFile.Root();
        if(!xmlFile.IsElement(pElementPart)) return FALSE;

        //CommandInfo Elements
        CXXMLFile::CElement* pElement = (CXXMLFile::CElement*)xmlFile.AddElement(pElementPart);
        xmlFile.SetText(pElement, "CommandInfo");

        //CommandGroup Elements
        if(m_pCommandGroupStandard)
        {
            if(!m_pCommandGroupStandard->StoreToXMLFile(&xmlFile, pElement)) oResult = FALSE;
        }

        //Write to string
        xmlFile.WriteToString(pCommandInfo);
        //xmlFile.SetFile("C:\\Commands.xml");
        //xmlFile.Write();

        return oResult;
    }

    return FALSE;
}

void CVirtualDevice_Drive::InitJournalManager(CJournalManagerBase *pJournalManager)
{
    CVirtualDeviceBase::InitJournalManager(pJournalManager);
    if(m_pCommandGroupStandard) m_pCommandGroupStandard->InitJournalManager(pJournalManager);
}

void CVirtualDevice_Drive::ResetJournalManager()
{
    CVirtualDeviceBase::ResetJournalManager();
    if(m_pCommandGroupStandard) m_pCommandGroupStandard->ResetJournalManager();
}

CVirtualDeviceBase* CVirtualDevice_Drive::Clone()
{
    CVirtualDevice_Drive* pDevice;

    pDevice = new CVirtualDevice_Drive(*this);
    return pDevice;
}

