#include "stdafx.h"

#include <ObjectDictionary/ObjectEntry.h>

#include <RecorderCmdManager/Classes/RecorderCmdChannel.h>

CRecorderCmdChannel::CRecorderCmdChannel()
{
    m_oActive = FALSE;
    m_Scale = SCALE_LEFT;

    m_pSelObjectEntry = new CObjectEntry();
}

CRecorderCmdChannel::~CRecorderCmdChannel()
{
    if (m_pSelObjectEntry)
    {
        delete m_pSelObjectEntry;
    }
}

#ifdef WINVER
void CRecorderCmdChannel::Serialize(CArchive& ar)
{
    int lValue;

    if (ar.IsStoring())
    {
        ar << (int)m_Scale;
    }
    else
    {
        ar >> lValue;
        m_Scale = EScale(lValue);
    }
}
#endif

BOOL CRecorderCmdChannel::IsActive()
{
    return m_oActive;
}

void CRecorderCmdChannel::SetActive(BOOL p_oActive)
{
    m_oActive = p_oActive;
}

CStdString CRecorderCmdChannel::GetSelObjectDataTypeLongStr()
{
    CStdString result(_T(""));

    if(m_pSelObjectEntry)
    {
        result = m_pSelObjectEntry->GetDataTypeLongStr().c_str();
    }

    return result;
}

EDataType CRecorderCmdChannel::GetSelObjectDataType()
{
    EDataType eDataType(DT_UNKNOWN);
    EObjectDataType eObjectDataType(ODT_UNKNOWN);

    if(m_pSelObjectEntry)
    {
        eObjectDataType = m_pSelObjectEntry->GetDataType();
        switch(eObjectDataType)
        {
            case ODT_UNKNOWN:       eDataType = DT_UNKNOWN; break;
            case ODT_BOOLEAN:       eDataType = DT_UNKNOWN; break;
            case ODT_INT8:          eDataType = DT_INT; break;
            case ODT_INT16:         eDataType = DT_INT; break;
            case ODT_INT32:         eDataType = DT_LINT; break;
            case ODT_INT64:         eDataType = DT_UNKNOWN; break;

            case ODT_UINT8:         eDataType = DT_WORD; break;
            case ODT_UINT16:        eDataType = DT_WORD; break;
            case ODT_UINT32:        eDataType = DT_LWORD; break;
            case ODT_UINT64:        eDataType = DT_UNKNOWN; break;

            case ODT_FLOAT:         eDataType = DT_FLOAT; break;
            case ODT_STRING:        eDataType = DT_UNKNOWN; break;
            case ODT_DOMAIN:        eDataType = DT_UNKNOWN; break;
            case ODT_MEMORY_BLOCK:  eDataType = DT_UNKNOWN; break;

            default:                eDataType = DT_WORD;
        }
    }

   return eDataType;
}

BOOL CRecorderCmdChannel::IsLeftScale()
{
    return (m_Scale == SCALE_LEFT);
}

BOOL CRecorderCmdChannel::IsRightScale()
{
    return (m_Scale == SCALE_RIGHT);
}

void CRecorderCmdChannel::SetLeftScale()
{
    m_Scale = SCALE_LEFT;
}

void CRecorderCmdChannel::SetRightScale()
{
    m_Scale = SCALE_RIGHT;
}

BOOL CRecorderCmdChannel::SetSelObject(WORD p_usIndex, BYTE p_ubSubIndex, CStdString p_type)
{
    BOOL oResult(FALSE);

    if (m_pSelObjectEntry)
    {
        m_pSelObjectEntry->SetIndex(p_usIndex);
        m_pSelObjectEntry->SetSubIndex(p_ubSubIndex);
        m_pSelObjectEntry->SetDataTypeLongStr(p_type);

        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderCmdChannel::SetSelObject(WORD p_usIndex, BYTE p_ubSubIndex, WORD p_usType)
{
    BOOL oResult(FALSE);

    if (m_pSelObjectEntry)
    {
        m_pSelObjectEntry->SetIndex(p_usIndex);
        m_pSelObjectEntry->SetSubIndex(p_ubSubIndex);
        m_pSelObjectEntry->SetDataType(p_usType);

        oResult = TRUE;
    }

    return oResult;
}

CStdString CRecorderCmdChannel::GetSelObjectName()
{
    CStdString str(_T(""));

    if(m_pSelObjectEntry)
    {
        str = m_pSelObjectEntry->GetParameterNameStr();

        if(str == _T(""))
        {
            str.Format(_T("0x%.4X-%.2X"), GetSelObjectIndex(), GetSelObjectSubIndex());
        }
    }

    return str;
}

void CRecorderCmdChannel::SetSelObjectName(CStdString p_objectName)
{
    if(m_pSelObjectEntry)
    {
        m_pSelObjectEntry->SetParameterName(p_objectName);
    }
}

BOOL CRecorderCmdChannel::GetSelObject(WORD& p_rusIndex, BYTE& p_rubSubIndex, WORD& p_rusType)
{
    BOOL oResult(FALSE);

    if (m_pSelObjectEntry)
    {
        p_rusIndex = m_pSelObjectEntry->GetIndex();
        p_rubSubIndex = m_pSelObjectEntry->GetSubIndex();
        p_rusType = m_pSelObjectEntry->GetDataType();

        oResult = TRUE;
    }

    return oResult;
}

WORD CRecorderCmdChannel::GetSelObjectIndex()
{
    if(m_pSelObjectEntry)
    {
        return m_pSelObjectEntry->GetIndex();
    }

    return 0;
}

BYTE CRecorderCmdChannel::GetSelObjectSubIndex()
{
    if(m_pSelObjectEntry)
    {
        return m_pSelObjectEntry->GetSubIndex();
    }

    return 0;
}

WORD CRecorderCmdChannel::GetSelObjectSize()
{
    WORD usValue(0);

    if(m_pSelObjectEntry)
    {
        usValue = m_pSelObjectEntry->GetWordSize();
    }

    return usValue;
}

CRecorderCmdChannel* CRecorderCmdChannel::Clone()
{
    CRecorderCmdChannel* pRecorderChannel = NULL;

    pRecorderChannel = new CRecorderCmdChannel();
    *pRecorderChannel = *this;

    return pRecorderChannel;
}

CRecorderCmdChannel& CRecorderCmdChannel::operator=(CRecorderCmdChannel& p_other)
{
    if(this != &p_other)
    {
        m_oActive = p_other.m_oActive;

        if (m_pSelObjectEntry && p_other.m_pSelObjectEntry)
        {
            *m_pSelObjectEntry = *p_other.m_pSelObjectEntry;
        }

        m_Scale = p_other.m_Scale;
    }

    return *this;
}

void CRecorderCmdChannel::SetSelObjectDefault(int p_lChannelIndex)
{
    CStdString str;

    if(m_pSelObjectEntry)
    {
        m_pSelObjectEntry->Reset();

        str.Format(_T("Variable %i"), p_lChannelIndex + 1);
        SetSelObjectName(str);

        SetActive(FALSE);
        SetLeftScale();
    }
}
