#include "stdafx.h"

#include <Classes/MmcDataConversion.h>
#include <Classes/TypeDefinitions.h>
#include <ObjectDictionary/ObjectDictionary.h>
#include <ObjectDictionary/ObjectEntry.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenCommunicationDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos2Def.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_CanOpen.h>
#include <CommunicationModel/VirtualDeviceCommunicationModel.h>
#include <RecorderCmdManager/Classes/RecorderCmdChannel.h>
#include <RecorderCmdManager/RecorderCmdManager.h>

#include <RecorderCmdManager/Classes/RecorderCmdConfiguration.h>

using namespace std;

CRecorderCmdConfiguration::CRecorderCmdConfiguration()
    : MICRO_SECONDS_SAMPLING_CYCLE(100)
    , m_AquisitionMode(ContinuousAquisitionMode)
    , m_fPrecedingTimeFactor(0)
    , m_oTotalTimeMaster(FALSE)
    , m_pCommunicationModel(NULL)
    , m_pRecorderManager(0)
    , m_ulErrorCode(0)
    , m_ulMicroSeconds_SamplingPeriod(0)
    , m_ulMicroSeconds_TotalTime(0)
    , m_usMaxNbOfSamples(512)
    , m_usNbOfPrecedingSamples(0)
    , m_usNbOfSamples(0)
    , m_usSamplingPeriod(100)
{
    SetObjectMode();

    InitRecorderChannelList();
    InitSelections();
}

void CRecorderCmdConfiguration::InitSelections()
{
    //TotalTime Selection
    m_TotalTimeSelection.clear();
    m_TotalTimeSelection.push_back(10000);        //10ms
    m_TotalTimeSelection.push_back(20000);        //20ms
    m_TotalTimeSelection.push_back(30000);        //30ms
    m_TotalTimeSelection.push_back(40000);        //40ms
    m_TotalTimeSelection.push_back(50000);        //50ms
    m_TotalTimeSelection.push_back(60000);        //60ms
    m_TotalTimeSelection.push_back(70000);        //70ms
    m_TotalTimeSelection.push_back(80000);        //80ms
    m_TotalTimeSelection.push_back(90000);        //90ms
    m_TotalTimeSelection.push_back(100000);        //100ms
    m_TotalTimeSelection.push_back(200000);        //200ms
    m_TotalTimeSelection.push_back(300000);        //300ms
    m_TotalTimeSelection.push_back(400000);        //400ms
    m_TotalTimeSelection.push_back(500000);        //500ms
    m_TotalTimeSelection.push_back(600000);        //600ms
    m_TotalTimeSelection.push_back(700000);        //700ms
    m_TotalTimeSelection.push_back(800000);        //800ms
    m_TotalTimeSelection.push_back(900000);        //900ms
    m_TotalTimeSelection.push_back(1000000);        //1s
    m_TotalTimeSelection.push_back(2000000);        //2s
    m_TotalTimeSelection.push_back(3000000);        //3s
    m_TotalTimeSelection.push_back(4000000);        //4s
    m_TotalTimeSelection.push_back(5000000);        //5s
    m_TotalTimeSelection.push_back(6000000);        //6s
    m_TotalTimeSelection.push_back(7000000);        //7s
    m_TotalTimeSelection.push_back(8000000);        //8s
    m_TotalTimeSelection.push_back(9000000);        //9s
    m_TotalTimeSelection.push_back(10000000);        //10s

    //SamplingPeriod Selection
    m_SamplingPeriodSelection.clear();
    m_SamplingPeriodSelection.push_back(100);        //100us
    m_SamplingPeriodSelection.push_back(200);        //200us
    m_SamplingPeriodSelection.push_back(300);        //300us
    m_SamplingPeriodSelection.push_back(400);        //400us
    m_SamplingPeriodSelection.push_back(500);        //500us
    m_SamplingPeriodSelection.push_back(600);        //600us
    m_SamplingPeriodSelection.push_back(700);        //700us
    m_SamplingPeriodSelection.push_back(800);        //800us
    m_SamplingPeriodSelection.push_back(900);        //900us

    m_SamplingPeriodSelection.push_back(1000);    //1ms
    m_SamplingPeriodSelection.push_back(2000);    //1ms
    m_SamplingPeriodSelection.push_back(3000);    //1ms
    m_SamplingPeriodSelection.push_back(4000);    //1ms
    m_SamplingPeriodSelection.push_back(5000);    //1ms
    m_SamplingPeriodSelection.push_back(6000);    //1ms
    m_SamplingPeriodSelection.push_back(7000);    //1ms
    m_SamplingPeriodSelection.push_back(8000);    //1ms
    m_SamplingPeriodSelection.push_back(9000);    //5ms

    m_SamplingPeriodSelection.push_back(10000);    //10ms
    m_SamplingPeriodSelection.push_back(20000);    //20ms
    m_SamplingPeriodSelection.push_back(30000);    //30ms
    m_SamplingPeriodSelection.push_back(40000);    //40ms
    m_SamplingPeriodSelection.push_back(50000);    //50ms
    m_SamplingPeriodSelection.push_back(60000);    //60ms
    m_SamplingPeriodSelection.push_back(70000);    //70ms
    m_SamplingPeriodSelection.push_back(80000);    //80ms
    m_SamplingPeriodSelection.push_back(90000);    //90ms
    m_SamplingPeriodSelection.push_back(100000);    //100ms

    //VariableSize Selection
    m_VariableSizeSelection.clear();
    m_VariableSizeSelection.push_back("Unknown");
    m_VariableSizeSelection.push_back("Signed 8-Bit Integer");
    m_VariableSizeSelection.push_back("Unsigned 8-Bit Integer");
    m_VariableSizeSelection.push_back("Signed 16-Bit Integer");
    m_VariableSizeSelection.push_back("Unsigned 16-Bit Integer");
    m_VariableSizeSelection.push_back("Signed 32-Bit Integer");
    m_VariableSizeSelection.push_back("Unsigned 32-Bit Integer");
    m_VariableSizeSelection.push_back("32-bit Floating Point");
}

CRecorderCmdConfiguration::~CRecorderCmdConfiguration()
{
    DeleteRecorderChannelList();
}

void CRecorderCmdConfiguration::AddNewRecorderChannel()
{
    CRecorderCmdChannel* pRecorderChannel;

    pRecorderChannel = new CRecorderCmdChannel();

    m_RecorderChannelList.push_back(pRecorderChannel);
}

CRecorderCmdChannel* CRecorderCmdConfiguration::GetRecorderChannel(int p_lChannelIndex)
{
    CRecorderCmdChannel* pRecorderChannel(0);

    if((p_lChannelIndex >= 0) && (p_lChannelIndex < (int)m_RecorderChannelList.size()))
    {
        pRecorderChannel = m_RecorderChannelList.at(p_lChannelIndex);
    }

    return pRecorderChannel;
}

int CRecorderCmdConfiguration::GetNbOfRecorderChannels()
{
    return (int)m_RecorderChannelList.size();
}

int CRecorderCmdConfiguration::GetNbOfConfiguredChannels()
{
    int lResult(0);

    for (DWORD i = 0; i < m_RecorderChannelList.size(); i++)
    {
        if (IsSelObjectSizeAvailable(i))
        {
            lResult++;
        }
    }

    return lResult;
}

void CRecorderCmdConfiguration::DeleteRecorderChannelList()
{
    vector<CRecorderCmdChannel*>::iterator it = m_RecorderChannelList.begin();

    while (it != m_RecorderChannelList.end())
    {
        CRecorderCmdChannel* pChannel = (*it);
        if (pChannel)
        {
            m_RecorderChannelList.erase(it);
            delete pChannel;

            it = m_RecorderChannelList.begin();
        }
    }

    m_RecorderChannelList.clear();
}

void CRecorderCmdConfiguration::InitRecorderChannelList()
{
    for(int i = 0; i < MAX_NB_OF_DATA_RECORDER_VARIABLES; i++)
    {
        AddNewRecorderChannel();
    }
}

BOOL CRecorderCmdConfiguration::Init(CRecorderCmdManager* p_pRecorderManager)
{
    BOOL oResult(FALSE);

    if (p_pRecorderManager)
    {
        m_pRecorderManager = p_pRecorderManager;

        oResult = TRUE;
    }

    return oResult;
}

void CRecorderCmdConfiguration::InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel)
{
    m_pCommunicationModel = p_pCommunicationModel;
}

void CRecorderCmdConfiguration::Init(HANDLE p_hKeyHandle, CLayerParameterStack& p_rAddressParameter)
{
    m_hKeyHandle = p_hKeyHandle;
    m_AddressParameterStack = p_rAddressParameter;
}

BOOL CRecorderCmdConfiguration::UpdateData(BOOL p_oSaveAndValidate)
{
    BOOL oResult = UpdateDataSampling(p_oSaveAndValidate);

    if (oResult)
    {
        oResult = UpdateRecorderConfiguration(p_oSaveAndValidate);
    }

    if (oResult)
    {
        oResult = UpdateAllChannels(p_oSaveAndValidate);
    }

    if (oResult)
    {
        oResult = StartRecorder(p_oSaveAndValidate);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::UpdateAllChannels(BOOL p_oSaveAndValidate)
{
    for(int i = 0; i < MAX_NB_OF_DATA_RECORDER_VARIABLES; i++)
    {
        if(!UpdateChannel(p_oSaveAndValidate, i, GetRecorderChannel(i))) return FALSE;
    }

    if(!UpdateAllChannelActivation(p_oSaveAndValidate)) return FALSE;

    return TRUE;
}

BOOL CRecorderCmdConfiguration::UpdateAllChannelActivation(BOOL p_oSaveAndValidate)
{
    CRecorderCmdChannel* pRecorderChannel(NULL);
    WORD usNbOfVariables(0);

    if(m_pCommunicationModel)
    {
        if(p_oSaveAndValidate)
        {
            for(int i = 0; i < GetNbOfRecorderChannels(); i++)
            {
                pRecorderChannel = GetRecorderChannel(i);
                if(pRecorderChannel->IsActive()) usNbOfVariables++;
            }

            return SetObject(INDEX_NB_OF_SAMPLING_VARIABLES, SUBINDEX_NB_OF_SAMPLING_VARIABLES, (BYTE*)&usNbOfVariables, sizeof(usNbOfVariables), &m_ulErrorCode);
        }
        else
        {
            if(!GetObject(INDEX_NB_OF_SAMPLING_VARIABLES, SUBINDEX_NB_OF_SAMPLING_VARIABLES, (BYTE*)&usNbOfVariables, sizeof(usNbOfVariables), &m_ulErrorCode)) return FALSE;
            for(int i = 0; i < GetNbOfRecorderChannels(); i++)
            {
                pRecorderChannel = GetRecorderChannel(i);
                if(i < usNbOfVariables) pRecorderChannel->SetActive(TRUE); else pRecorderChannel->SetActive(FALSE);
            }

            return TRUE;
        }
    }

    return FALSE;
}

BOOL CRecorderCmdConfiguration::UpdateChannel(BOOL p_oSaveAndValidate, int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel)
{
    SRecorderConfiguration recorderConfiguration;

    if(m_pCommunicationModel)
    {
        if(p_oSaveAndValidate)
        {
            //Write Data to device
            return WriteChannelToDevice(p_lChannelIndex, p_pRecorderChannel);
        }
        else
        {
            //Read data from device
            if(!GetObject(INDEX_RECORDER_CONFIGURATION, SUBINDEX_RECORDER_CONFIGURATION, (BYTE*)&recorderConfiguration, sizeof(recorderConfiguration), &m_ulErrorCode))
            {
                p_pRecorderChannel->SetSelObjectDefault(p_lChannelIndex);

                return TRUE;
            }
            else
            {
                return ReadChannelFromDevice(p_lChannelIndex, p_pRecorderChannel);
            }
        }
    }

    return FALSE;
}

BOOL CRecorderCmdConfiguration::WriteChannelToDevice(int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel)
{
    BOOL oResult(FALSE);
    BYTE ubSubIndex;
    WORD usIndex;
    WORD usType;

    if(p_pRecorderChannel)
    {
        oResult = p_pRecorderChannel->GetSelObject(usIndex, ubSubIndex, usType);

        if (oResult)
        {
            oResult = WriteVariable((BYTE)p_lChannelIndex, usIndex, ubSubIndex, usType);
        }
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::ReadChannelFromDevice(int p_lChannelIndex, CRecorderCmdChannel* p_pRecorderChannel)
{
    BOOL oResult(FALSE);
    BYTE ubSubIndex(0);
    WORD usIndex;
    WORD usType(0);

    if(p_pRecorderChannel)
    {
        //Read variable data
        oResult = ReadVariable((BYTE)p_lChannelIndex, &usIndex, &ubSubIndex, &usType);

        if(oResult && (p_pRecorderChannel->GetSelObjectIndex() == 0
        || p_pRecorderChannel->GetSelObjectIndex() != usIndex
        || p_pRecorderChannel->GetSelObjectSubIndex() != ubSubIndex))
        {
            //Reset channel settings
            p_pRecorderChannel->SetSelObjectDefault(p_lChannelIndex);

            //Set channel settings
            p_pRecorderChannel->SetSelObject(usIndex, ubSubIndex, usType);
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::UpdateRecorderConfiguration(BOOL p_oSaveAndValidate)
{
    BOOL oResult = FALSE;

    if(m_pCommunicationModel)
    {
        if(p_oSaveAndValidate)
        {
            //Write data to device
            oResult = SetObject(INDEX_RECORDER_CONFIGURATION, SUBINDEX_RECORDER_CONFIGURATION, (BYTE*)&m_RecorderConfiguration, sizeof(m_RecorderConfiguration), &m_ulErrorCode);
        }
        else
        {
            //Read data from device
            oResult = GetObject(INDEX_RECORDER_CONFIGURATION, SUBINDEX_RECORDER_CONFIGURATION, (BYTE*)&m_RecorderConfiguration, sizeof(m_RecorderConfiguration), &m_ulErrorCode);
        }
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::UpdateDataSampling(BOOL p_oSaveAndValidate)
{
    BOOL oResult(FALSE);

    if(m_pCommunicationModel)
    {
        if(p_oSaveAndValidate)
        {
            //Write data to device
            if(!SetObject(INDEX_SAMPLING_PERIOD, SUBINDEX_SAMPLING_PERIOD, (BYTE*)&m_usSamplingPeriod, sizeof(m_usSamplingPeriod), &m_ulErrorCode)) return FALSE;
            if(!SetObject(INDEX_NB_OF_PRECEDING_SAMPLES, SUBINDEX_NB_OF_PRECEDING_SAMPLES, (BYTE*)&m_usNbOfPrecedingSamples, sizeof(m_usNbOfPrecedingSamples), &m_ulErrorCode)) return FALSE;

            oResult = TRUE;
        }
        else
        {
            //Read data from device
            if(!GetObject(INDEX_SAMPLING_PERIOD, SUBINDEX_SAMPLING_PERIOD, (BYTE*)&m_usSamplingPeriod, sizeof(m_usSamplingPeriod), &m_ulErrorCode)) return FALSE;
            if(!GetObject(INDEX_NB_OF_PRECEDING_SAMPLES, SUBINDEX_NB_OF_PRECEDING_SAMPLES, (BYTE*)&m_usNbOfPrecedingSamples, sizeof(m_usNbOfPrecedingSamples), &m_ulErrorCode)) return FALSE;
            if(!GetObject(INDEX_MAX_NB_OF_SAMPLES, SUBINDEX_MAX_NB_OF_SAMPLES, (BYTE*)&m_usMaxNbOfSamples, sizeof(m_usMaxNbOfSamples), &m_ulErrorCode)) return FALSE;

            //Update related variables
            SetMicroSecondsSamplingPeriod(m_usSamplingPeriod * MICRO_SECONDS_SAMPLING_CYCLE);
            SetPrecedingSamples(m_usNbOfPrecedingSamples);

            oResult = TRUE;
        }
    }

    return oResult;
}

CStdString CRecorderCmdConfiguration::GetSamplingPeriodStr()
{
    return GetTimeStr(m_ulMicroSeconds_SamplingPeriod);
}

CStdString CRecorderCmdConfiguration::GetTotalTimeStr()
{
    return GetTimeStr(m_ulMicroSeconds_TotalTime);
}

void CRecorderCmdConfiguration::SetMicroSecondsTotalTimeStr(CStdString p_strMicroSecondsTotalTime)
{
    DWORD ulTimeValue;

    ulTimeValue = GetTimeValue(p_strMicroSecondsTotalTime);
    SetMicroSecondsTotalTime(ulTimeValue);
}

void CRecorderCmdConfiguration::SetMicroSecondsTotalTime(DWORD p_ulMicroSecondsTotalTime)
{
    DWORD ulMicroSecondsSamplingPeriod;

    ulMicroSecondsSamplingPeriod = p_ulMicroSecondsTotalTime / m_usMaxNbOfSamples;

    //Round to next 100 microSeconds
    if(ulMicroSecondsSamplingPeriod % MICRO_SECONDS_SAMPLING_CYCLE)
    {
        ulMicroSecondsSamplingPeriod = ((ulMicroSecondsSamplingPeriod / MICRO_SECONDS_SAMPLING_CYCLE) + 1) * MICRO_SECONDS_SAMPLING_CYCLE;
    }
    else
    {
        ulMicroSecondsSamplingPeriod = (ulMicroSecondsSamplingPeriod / MICRO_SECONDS_SAMPLING_CYCLE) * MICRO_SECONDS_SAMPLING_CYCLE;
    }

    m_ulMicroSeconds_SamplingPeriod = ulMicroSecondsSamplingPeriod;
    m_usSamplingPeriod = (WORD)(ulMicroSecondsSamplingPeriod / MICRO_SECONDS_SAMPLING_CYCLE);
    m_ulMicroSeconds_TotalTime = p_ulMicroSecondsTotalTime;
    m_usNbOfSamples = (WORD)(m_ulMicroSeconds_TotalTime / m_ulMicroSeconds_SamplingPeriod);

    //Recalculate new NbOfPrecedingSamples;
    m_usNbOfPrecedingSamples = (WORD)(m_usNbOfSamples * m_fPrecedingTimeFactor);
}

void CRecorderCmdConfiguration::SetTotalTimeMaster(BOOL p_oTotalTimeMaster)
{
    m_oTotalTimeMaster = p_oTotalTimeMaster;
}

void CRecorderCmdConfiguration::SetMicroSecondsSamplingPeriod(DWORD p_ulMicroSecondsSamplingPeriod)
{
    m_ulMicroSeconds_SamplingPeriod = p_ulMicroSecondsSamplingPeriod;
    m_usSamplingPeriod = (WORD)(m_ulMicroSeconds_SamplingPeriod / MICRO_SECONDS_SAMPLING_CYCLE);
    m_ulMicroSeconds_TotalTime = m_ulMicroSeconds_SamplingPeriod * m_usMaxNbOfSamples;
    m_usNbOfSamples = m_usMaxNbOfSamples;
}

void CRecorderCmdConfiguration::SetMicroSecondsSamplingPeriodStr(CStdString p_strMicroSecondsSamplingPeriod)
{
    DWORD ulTimeValue;

    ulTimeValue = GetTimeValue(p_strMicroSecondsSamplingPeriod);
    SetMicroSecondsSamplingPeriod(ulTimeValue);
}

CStdString CRecorderCmdConfiguration::GetPrecedingTimeStr()
{
    return GetTimeStr(m_usNbOfPrecedingSamples*m_ulMicroSeconds_SamplingPeriod);
}

#ifdef WINVER
void CRecorderCmdConfiguration::Serialize(CArchive& ar)
{
    CRecorderCmdChannel* pRecorderChannel;
    int lCount;

    if(ar.IsStoring())
    {
        ar << m_RecorderChannelList.size();
        for (vector<CRecorderCmdChannel*>::iterator it = m_RecorderChannelList.begin(); it != m_RecorderChannelList.end(); it++)
        {
            pRecorderChannel = (*it);
            pRecorderChannel->Serialize(ar);
        }

        ar << m_oTotalTimeMaster;
        ar << m_AquisitionMode;
    }
    else
    {
        ar >> lCount;
        for(int i = 0; i < lCount; i++)
        {
            pRecorderChannel = m_RecorderChannelList.at(i);
            if(pRecorderChannel)
            {
                pRecorderChannel->Serialize(ar);
            }
            else
            {
                pRecorderChannel = new CRecorderCmdChannel();
                pRecorderChannel->Serialize(ar);
                delete pRecorderChannel;
            }
        }

        ar >> m_oTotalTimeMaster;
        ar >> lCount;
        switch(lCount)
        {
            case 0: m_AquisitionMode = ContinuousAquisitionMode; break;
            case 1: m_AquisitionMode = SingleTriggerMode; break;
        }
    }
}
#endif

CStdString CRecorderCmdConfiguration::GetMaxPrecedingTimeStr()
{
    CStdString str;

    str = GetTimeStr(m_usNbOfSamples * m_ulMicroSeconds_SamplingPeriod);
    str = str.Left(str.Find(_T(" "), 0));

    return str;
}

CStdString CRecorderCmdConfiguration::GetTimeStr(DWORD p_ulMicroSeconds)
{
    CStdString str;
    CStdString unitStr = _T("microSecond");

    if((p_ulMicroSeconds >= 1000) && ((p_ulMicroSeconds % 1000) == 0))
    {
        //Change to milliSecond
        unitStr = _T("milliSecond");
        p_ulMicroSeconds /= 1000;

        if((p_ulMicroSeconds >= 1000) && ((p_ulMicroSeconds % 1000) == 0))
        {
            //Change to second
            unitStr = _T("second");
            p_ulMicroSeconds /= 1000;
        }
    }

    if(p_ulMicroSeconds != 1) unitStr += "s";

    str.Format(_T("%i"), p_ulMicroSeconds);
    str = str + " " + unitStr;

    return str;
}

DWORD CRecorderCmdConfiguration::GetTimeValue(CStdString p_strTimeValue)
{
    CMmcDataConversion dataConversion;
    CStdString unitStr;
    CStdString valueStr;
    DWORD ulValue(0);

    unitStr = p_strTimeValue.Right(p_strTimeValue.GetLength() - p_strTimeValue.Find(_T(" "), 0)-1);
    valueStr = p_strTimeValue.Left(p_strTimeValue.Find(_T(" "), 0));

    if(dataConversion.DecDWordStr2DWord(valueStr, &ulValue, FALSE))
    {
        if(unitStr.Find(_T("microSecond"), 0) != -1)
        {
            ulValue = ulValue;
        }
        else if(unitStr.Find(_T("milliSecond"), 0) != -1)
        {
            ulValue *= 1000;
        }
        else if(unitStr.Find(_T("second"), 0) != -1)
        {
            ulValue *= 1000 * 1000;
        }
    }

    return ulValue;
}

BOOL CRecorderCmdConfiguration::IsMovementTriggerActive()
{
    return (m_RecorderConfiguration.usMovementTrigger);
}

BOOL CRecorderCmdConfiguration::IsEndOfProfileTriggerActive()
{
    return (m_RecorderConfiguration.usEndOfProfileTrigger);
}

BOOL CRecorderCmdConfiguration::IsDigInputTriggerActive()
{
    return (m_RecorderConfiguration.usDigInputTrigger);
}

BOOL CRecorderCmdConfiguration::IsErrorTriggerActive()
{
    return (m_RecorderConfiguration.usErrorTrigger);
}

void CRecorderCmdConfiguration::ActivateMovementTrigger(BOOL p_oActive)
{
    m_RecorderConfiguration.usMovementTrigger = p_oActive;
}

void CRecorderCmdConfiguration::ActivateEndOfProfileTrigger(BOOL p_oActive)
{
    m_RecorderConfiguration.usEndOfProfileTrigger = p_oActive;
}

void CRecorderCmdConfiguration::ActivateErrorTrigger(BOOL p_oActive)
{
    m_RecorderConfiguration.usErrorTrigger = p_oActive;
}

void CRecorderCmdConfiguration::ActivateDigInputTrigger(BOOL p_oActive)
{
    m_RecorderConfiguration.usDigInputTrigger = p_oActive;
}

WORD CRecorderCmdConfiguration::GetMaxPrecedingSamples()
{
    return m_usNbOfSamples;
}

WORD CRecorderCmdConfiguration::GetPrecedingSamples()
{
    return m_usNbOfPrecedingSamples;
}

CStdString CRecorderCmdConfiguration::GetNbOfSamplesStr()
{
    CStdString str;

    if(m_usNbOfSamples == m_usMaxNbOfSamples)
    {
        str.Format(_T("%i"), m_usNbOfSamples);
    }
    else
    {
        str.Format(_T("%i of %i"), m_usNbOfSamples, m_usMaxNbOfSamples);
    }

    return str;
}

CStdString CRecorderCmdConfiguration::GetMaxNbOfPrecedingSamplesStr()
{
    CStdString str;

    str.Format(_T("%i"), m_usNbOfSamples);

    return str;
}

CStdString CRecorderCmdConfiguration::GetNbOfPrecedingSamplesStr()
{
    CStdString str;

    str.Format(_T("%i"), m_usNbOfPrecedingSamples);

    return str;
}

void CRecorderCmdConfiguration::SetPrecedingSamples(WORD p_usNbOfPrecedingSamples)
{
    if(p_usNbOfPrecedingSamples > m_usMaxNbOfSamples) p_usNbOfPrecedingSamples = m_usMaxNbOfSamples;
    m_usNbOfPrecedingSamples = p_usNbOfPrecedingSamples;
    m_fPrecedingTimeFactor = float(m_usNbOfPrecedingSamples) / float(m_usMaxNbOfSamples);
}

void CRecorderCmdConfiguration::SetPrecedingTimeFactor(float p_fPrecedingTimeFactor)
{
    m_fPrecedingTimeFactor = p_fPrecedingTimeFactor;
    m_usNbOfPrecedingSamples = (WORD)(m_usNbOfSamples * m_fPrecedingTimeFactor);
}

void CRecorderCmdConfiguration::GetTotalTimeSelection(CStdStringArray* p_pTotalTimeSelection)
{
    if(p_pTotalTimeSelection)
    {
        p_pTotalTimeSelection->clear();
        for(int i = 0; i < (int)m_TotalTimeSelection.size(); i++)
        {
            p_pTotalTimeSelection->push_back(GetTimeStr(m_TotalTimeSelection[i]));
        }
    }
}

void CRecorderCmdConfiguration::GetSamplingPeriodSelection(CStdStringArray* p_pSamplingPeriodSelection)
{
    if(p_pSamplingPeriodSelection)
    {
        p_pSamplingPeriodSelection->clear();
        for(int i = 0; i < (int)m_SamplingPeriodSelection.size(); i++)
        {
            p_pSamplingPeriodSelection->push_back(GetTimeStr(m_SamplingPeriodSelection[i]));
        }
    }
}

void CRecorderCmdConfiguration::GetVariableSizeSelection(CStdStringArray* p_pVariableSizeSelection)
{
    if(p_pVariableSizeSelection)
    {
        p_pVariableSizeSelection->clear();
        for(int i = 0; i < (int)m_VariableSizeSelection.size(); i++)
        {
            p_pVariableSizeSelection->push_back(m_VariableSizeSelection[i]);
        }
    }
}

void CRecorderCmdConfiguration::SetObjectMode()
{
    m_RecorderConfiguration.usAddressMode = 0;
}

BOOL CRecorderCmdConfiguration::RecalculateMaxNbOfSamples()
{
    CRecorderCmdChannel* pRecorderChannel;
    DWORD ulTotalSampleSize;
    int lNbOfChannels;

    //Count total Sample Size
    lNbOfChannels = GetNbOfRecorderChannels();
    ulTotalSampleSize = 0;
    for(int i = 0; i < lNbOfChannels; i++)
    {
        pRecorderChannel = GetRecorderChannel(i);
        if(pRecorderChannel && pRecorderChannel->IsActive())
        {
            ulTotalSampleSize += pRecorderChannel->GetSelObjectSize();
        }
    }
    if(ulTotalSampleSize == 0) ulTotalSampleSize = 1;

    //Recalculate new MaxNbOfSamples
    m_usMaxNbOfSamples = (WORD)(MAX_DATA_RECORDER_BUFFER_SIZE / 2 / ulTotalSampleSize);

    //Recalculate new NbOfPrecedingSamples;
    m_usNbOfPrecedingSamples = (WORD)(m_usMaxNbOfSamples * m_fPrecedingTimeFactor);

    //Calculate related variables
    if(IsTotalTimeMaster())
    {
        SetMicroSecondsTotalTime(m_ulMicroSeconds_TotalTime);
    }
    else
    {
        SetMicroSecondsSamplingPeriod(m_usSamplingPeriod * MICRO_SECONDS_SAMPLING_CYCLE);
        return TRUE;
    }

    return FALSE;
}

BOOL CRecorderCmdConfiguration::IsTotalTimeMaster()
{
    return m_oTotalTimeMaster;
}

BOOL CRecorderCmdConfiguration::SetSelObject(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex, CStdString p_type)
{
    BOOL oResult(FALSE);
    CRecorderCmdChannel* pRecorderChannel(m_RecorderChannelList.at(p_lChannelIndex));

    if (pRecorderChannel)
    {
        oResult = pRecorderChannel->SetSelObject(p_usIndex, p_ubSubIndex, p_type);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::SetSelObject(int p_lChannelIndex, WORD p_usIndex, BYTE p_ubSubIndex, WORD p_usType)
{
    BOOL oResult(FALSE);
    CRecorderCmdChannel* pRecorderChannel(m_RecorderChannelList.at(p_lChannelIndex));

    if(pRecorderChannel)
    {
        oResult = pRecorderChannel->SetSelObject(p_usIndex, p_ubSubIndex, p_usType);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::GetSelObject(int p_lChannelIndex, WORD& p_rusIndex, BYTE& p_rubSubIndex, WORD& p_rusType)
{
    BOOL oResult(FALSE);
    CRecorderCmdChannel* pRecorderChannel(m_RecorderChannelList.at(p_lChannelIndex));

    if (pRecorderChannel)
    {
        oResult = pRecorderChannel->GetSelObject(p_rusIndex, p_rubSubIndex, p_rusType);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::StartRecorder(BOOL p_oSaveAndValidate)
{
    BOOL oResult(FALSE);

    if(m_pRecorderManager)
    {
        if (p_oSaveAndValidate)
        {
            oResult = m_pRecorderManager->StartRecorder(m_hKeyHandle, m_AddressParameterStack, &m_ulErrorCode);
        }
        else
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

EAquisitionMode CRecorderCmdConfiguration::GetAquisitionMode()
{
    return m_AquisitionMode;
}

void CRecorderCmdConfiguration::SetAquisitionMode(EAquisitionMode p_eAquisitionMode)
{
    m_AquisitionMode = p_eAquisitionMode;
}

CRecorderCmdConfiguration& CRecorderCmdConfiguration::operator=(CRecorderCmdConfiguration& other)
{
    CRecorderCmdChannel* pRecorderChannel = NULL;
    CRecorderCmdChannel* pClonedRecorderChannel = NULL;

    if(this != &other)
    {
        //Variables of Device
        m_RecorderConfiguration = other.m_RecorderConfiguration;
        m_usSamplingPeriod = other.m_usSamplingPeriod;
        m_usNbOfPrecedingSamples = other.m_usNbOfPrecedingSamples;
        m_usMaxNbOfSamples = other.m_usMaxNbOfSamples;

        //Recorder Channels
        DeleteRecorderChannelList();
        for (vector<CRecorderCmdChannel*>::iterator it = other.m_RecorderChannelList.begin(); it != other.m_RecorderChannelList.end(); it++)
        {
            pClonedRecorderChannel = (*it)->Clone();
            m_RecorderChannelList.push_back(pClonedRecorderChannel);
        }

        //Additional Variables
        m_oTotalTimeMaster = other.m_oTotalTimeMaster;
        m_ulMicroSeconds_TotalTime = other.m_ulMicroSeconds_TotalTime;
        m_ulMicroSeconds_SamplingPeriod = other.m_ulMicroSeconds_SamplingPeriod;
        m_usNbOfSamples = other.m_usNbOfSamples;
        m_fPrecedingTimeFactor = other.m_fPrecedingTimeFactor;

        m_AquisitionMode = other.m_AquisitionMode;

        //TotalTime Selection
        m_TotalTimeSelection.clear();
        m_TotalTimeSelection.insert(m_TotalTimeSelection.end(), other.m_TotalTimeSelection.begin(), other.m_TotalTimeSelection.end());

        //SamplingPeriod Selection
        m_SamplingPeriodSelection.clear();
        m_SamplingPeriodSelection.insert(m_SamplingPeriodSelection.end(), other.m_SamplingPeriodSelection.begin(), other.m_SamplingPeriodSelection.end());

        //VariableSize Selection
        m_VariableSizeSelection.clear();
        m_VariableSizeSelection.insert(m_VariableSizeSelection.end(), other.m_VariableSizeSelection.begin(), other.m_VariableSizeSelection.end());

        m_pCommunicationModel = other.m_pCommunicationModel;
    }

    return *this;
}

BOOL CRecorderCmdConfiguration::ReadVariable(BYTE p_ubChannelNumber, WORD* p_pusIndex, BYTE* p_pubSubIndex, WORD* p_pusObjectType)
{
    BOOL oResult(FALSE);

    if (m_pRecorderManager)
    {
        oResult = m_pRecorderManager->ActiveChannel(m_hKeyHandle, m_AddressParameterStack, p_ubChannelNumber, p_pusIndex, p_pubSubIndex, p_pusObjectType, &m_ulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::WriteVariable(BYTE p_ubChannelNumber, WORD p_usIndex, BYTE p_ubSubIndex, WORD p_usType)
{
    BOOL oResult(FALSE);

    if (m_pRecorderManager)
    {
        oResult = m_pRecorderManager->ActivateChannel(m_hKeyHandle, m_AddressParameterStack, p_ubChannelNumber, p_usIndex, p_ubSubIndex, p_usType, &m_ulErrorCode);
    }

    return oResult;
}

WORD CRecorderCmdConfiguration::GetSelObjectSize(int p_lChannelIndex)
{
    CRecorderCmdChannel* pRecorderChannel = m_RecorderChannelList.at(p_lChannelIndex);
    WORD usResult(0);

    if(pRecorderChannel)
    {
        usResult = pRecorderChannel->GetSelObjectSize();
    }

    return usResult;
}

BOOL CRecorderCmdConfiguration::IsSelObjectSizeAvailable(int p_lChannelIndex)
{
    BOOL oResult(FALSE);
    CRecorderCmdChannel* pRecorderChannel = m_RecorderChannelList.at(p_lChannelIndex);

    if(pRecorderChannel)
    {
        oResult = (DT_UNKNOWN != pRecorderChannel->GetSelObjectDataType());
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::GetObject(WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE* p_pData, DWORD p_ulNbOfBytesToRead, DWORD* p_pulErrorCode)
{
    BOOL oResult = FALSE;
    CCommand_VCS_CanOpen command(VCS_CANOPEN_GET_OBJECT);
    CLayerParameterStack addressParameter;
    DWORD ulNbOfBytesRead = 0;
    HANDLE hTransactionHandle = NULL;

    if(m_pCommunicationModel && p_pData)
    {
        //Set Parameter
        command.ResetStatus();
        command.SetParameterData(0, &p_usObjectIndex, sizeof(p_usObjectIndex));
        command.SetParameterData(1, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
        command.SetParameterData(2, &p_ulNbOfBytesToRead, sizeof(p_ulNbOfBytesToRead));

        //Address Parameter
        UpdateAddressParameter(addressParameter);
        command.SetLayerParameterStack(addressParameter);

        //ExecuteCommand
        oResult = m_pCommunicationModel->ExecuteCommand(&command, m_hKeyHandle, hTransactionHandle);

        //Get ReturnParameter Data
        command.GetReturnParameterData(0, p_pData, p_ulNbOfBytesToRead);
        command.GetReturnParameterData(1, &ulNbOfBytesRead, sizeof(ulNbOfBytesRead));

        //Get ErrorCode
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::SetObject(WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE* p_pData, DWORD p_ulNbOfBytesToWrite, DWORD* p_pulErrorCode)
{
    BOOL oResult = FALSE;
    CCommand_VCS_CanOpen command(VCS_CANOPEN_SET_OBJECT);
    CLayerParameterStack addressParameter;
    DWORD ulNbOfBytesWritten = 0;
    HANDLE hTransactionHandle = NULL;

    if(m_pCommunicationModel && p_pData)
    {
        //Set Parameter
        command.ResetStatus();
        command.SetParameterData(0, &p_usObjectIndex, sizeof(p_usObjectIndex));
        command.SetParameterData(1, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
        command.SetParameterData(2, p_pData, p_ulNbOfBytesToWrite);
        command.SetParameterData(3, &p_ulNbOfBytesToWrite, sizeof(p_ulNbOfBytesToWrite));

        //Address Parameter
        UpdateAddressParameter(addressParameter);
        command.SetLayerParameterStack(addressParameter);

        //ExecuteCommand
        oResult = m_pCommunicationModel->ExecuteCommand(&command, m_hKeyHandle, hTransactionHandle);

        //Get ReturnParameter Data
        command.GetReturnParameterData(0, &ulNbOfBytesWritten, sizeof(ulNbOfBytesWritten));

        //Get ErrorCode
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdConfiguration::UpdateAddressParameter(CLayerParameterStack& p_rAddressParameter)
{
    BOOL oResult = FALSE;

    if(m_pCommunicationModel)
    {
        if(m_AddressParameterStack.IsEmpty())
        {
            //Address Parameter defined by Communication Model
            oResult = m_pCommunicationModel->GetLayerParameterStack(p_rAddressParameter);
        }
        else
        {
            //Address Parameter already defined
            p_rAddressParameter = m_AddressParameterStack;
            oResult = TRUE;
        }
    }

    return oResult;
}
