#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/VirtualDeviceCommunicationModel.h>
#include <RecorderCmdManager/Classes/RecorderCmdConfiguration.h>

#include <RecorderCmdManager/Classes/RecorderConfigurationEntry.h>

CRecorderConfigurationEntry::CRecorderConfigurationEntry()
{
    m_KeyHandle = 0;
    m_pLayerParameterStack = NULL;
    m_pRecorderConfiguration = NULL;
}

CRecorderConfigurationEntry::~CRecorderConfigurationEntry()
{
    DeleteLayerParameterStack();
    DeleteRecorderConfiguration();
}

BOOL CRecorderConfigurationEntry::Init(CRecorderCmdManager* p_pRecorderManager)
{
    BOOL oResult = FALSE;

    if (m_pRecorderConfiguration)
    {
        m_pRecorderConfiguration->Init(p_pRecorderManager);

        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel)
{
    BOOL oResult = FALSE;

    if(m_pRecorderConfiguration)
    {
        m_pRecorderConfiguration->InitCommunicationModel(p_pCommunicationModel);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::Init(HANDLE p_KeyHandle, CLayerParameterStack& p_rParameterStack)
{
    BOOL oResult = FALSE;

    if(!m_pLayerParameterStack)
    {
        //Init
        oResult = TRUE;

        //KeyHandle
        m_KeyHandle = p_KeyHandle;

        //ParameterStack
        if(oResult) oResult = CreateLayerParameterStack(p_rParameterStack);

        //Recorder Control
        if(oResult) oResult = CreateRecorderConfiguration();
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::IsEqual(HANDLE p_KeyHandle, CLayerParameterStack& p_rParameterStack)
{
    BOOL oResult = FALSE;

    if(p_KeyHandle == m_KeyHandle)
    {
        if(m_pLayerParameterStack)
        {
            if(*m_pLayerParameterStack == p_rParameterStack)
            {
                oResult = TRUE;
            }
        }
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::GetRecorderConfiguration(CRecorderCmdConfiguration*& p_rpRecorderConfiguration)
{
    BOOL oResult = FALSE;

    if(m_pRecorderConfiguration)
    {
        p_rpRecorderConfiguration = m_pRecorderConfiguration;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::CreateLayerParameterStack(CLayerParameterStack& p_rParameterStack)
{
    CLayerParameterStack parameterStack;
    BOOL oResult = FALSE;

    if(!m_pLayerParameterStack)
    {
        //Init
        parameterStack = p_rParameterStack;
        oResult = TRUE;

        //Layer Parameter Stack
        m_pLayerParameterStack = new CLayerParameterStack();
        *m_pLayerParameterStack = p_rParameterStack;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::CreateRecorderConfiguration()
{
    BOOL oResult = FALSE;

    if(!m_pRecorderConfiguration && m_pLayerParameterStack)
    {
        m_pRecorderConfiguration = new CRecorderCmdConfiguration();
        m_pRecorderConfiguration->Init(m_KeyHandle, *m_pLayerParameterStack);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::DeleteLayerParameterStack()
{
    BOOL oResult = FALSE;

    if(m_pLayerParameterStack)
    {
        //Init
        oResult = TRUE;

        //Delete Stack
        delete m_pLayerParameterStack;
        m_pLayerParameterStack = NULL;
    }

    return oResult;
}

BOOL CRecorderConfigurationEntry::DeleteRecorderConfiguration()
{
    BOOL oResult = FALSE;

    if(m_pRecorderConfiguration)
    {
        delete m_pRecorderConfiguration;
        m_pRecorderConfiguration = NULL;
        oResult = TRUE;
    }

    return oResult;
}