#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterSet.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/VirtualDeviceCommunicationModel.h>
#include <RecorderCmdManager/Classes/RecorderCmdConfiguration.h>
#include <RecorderCmdManager/Classes/RecorderConfigurationEntry.h>

#include <RecorderCmdManager/Classes/RecorderConfigurationMap.h>

using namespace std;

CRecorderConfigurationMap::CRecorderConfigurationMap()
{
    m_pCommunicationModel = 0;
}

CRecorderConfigurationMap::~CRecorderConfigurationMap()
{
    ResetMap();
}

BOOL CRecorderConfigurationMap::Init(CRecorderCmdManager* p_pRecorderManager)
{
    BOOL oResult(FALSE);
        CRecorderConfigurationEntry* pEntry = 0;

    if (p_pRecorderManager)
    {
        for (list<CRecorderConfigurationEntry*>::iterator it = m_RecorderConfigurationEntryList.begin(); it != m_RecorderConfigurationEntryList.end(); it++)
        {
            pEntry = *it;

            if (pEntry)
            {
                oResult = pEntry->Init(p_pRecorderManager);
                break;
            }
        }
    }

    return oResult;
}

BOOL CRecorderConfigurationMap::InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel)
{
    BOOL oResult = TRUE;
    CRecorderConfigurationEntry* pEntry = 0;

    m_pCommunicationModel = p_pCommunicationModel;

    for (list<CRecorderConfigurationEntry*>::iterator it = m_RecorderConfigurationEntryList.begin(); it != m_RecorderConfigurationEntryList.end(); it++)
    {
        pEntry = *it;

        if (pEntry)
        {
            pEntry->InitCommunicationModel(m_pCommunicationModel);
        }
        else
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CRecorderConfigurationMap::GetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rParameterStack, CRecorderCmdConfiguration*& p_rpRecorderConfiguration)
{
    BOOL oResult = TRUE;
    CRecorderConfigurationEntry* pEntry = 0;

    if(!FindRecorderConfiguration(p_KeyHandle, p_rParameterStack, p_rpRecorderConfiguration))
    {
        oResult = FALSE;
        pEntry = new CRecorderConfigurationEntry();

        if(pEntry->Init(p_KeyHandle, p_rParameterStack))
        {
            pEntry->InitCommunicationModel(m_pCommunicationModel);
            m_RecorderConfigurationEntryList.push_back(pEntry);
            oResult = pEntry->GetRecorderConfiguration(p_rpRecorderConfiguration);
        }
        else
        {
            delete pEntry;
        }
    }

    return oResult;
}

BOOL CRecorderConfigurationMap::FindRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rParameterStack, CRecorderCmdConfiguration*& p_rpRecorderConfiguration)
{
    BOOL oResult = FALSE;
    CRecorderConfigurationEntry* pEntry = 0;

    for (list<CRecorderConfigurationEntry*>::iterator it = m_RecorderConfigurationEntryList.begin(); it != m_RecorderConfigurationEntryList.end(); it++)
    {
        pEntry = *it;

        if (pEntry && pEntry->IsEqual(p_KeyHandle, p_rParameterStack))
        {
            oResult = pEntry->GetRecorderConfiguration(p_rpRecorderConfiguration);
            break;
        }
    }

    return oResult;
}

BOOL CRecorderConfigurationMap::ResetMap()
{
    BOOL oResult = TRUE;
    list<CRecorderConfigurationEntry*>::iterator it = m_RecorderConfigurationEntryList.begin();

    while (it != m_RecorderConfigurationEntryList.end())
    {
        CRecorderConfigurationEntry* pEntry = (*it);
        if (pEntry)
        {
            m_RecorderConfigurationEntryList.remove(pEntry);
            delete pEntry;

            it = m_RecorderConfigurationEntryList.begin();
        }
    }

    m_RecorderConfigurationEntryList.clear();

    return oResult;
}
