#include "stdafx.h"

#include <CommunicationModel/CommonLayer/Classes/Commands/Command/LayerParameter/LayerParameterStack.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_DataRecorder.h>
#include <CommunicationModel/VirtualDeviceCommunicationModel.h>
#include <RecorderCmdManager/Classes/RecorderCmdConfiguration.h>
#include <RecorderCmdManager/Classes/RecorderConfigurationMap.h>

#include <RecorderCmdManager/RecorderCmdManager.h>

CRecorderCmdManager::CRecorderCmdManager(void) :
    m_pCommunicationModel(0)
{
    m_pRecorderConfigurationMap = new CRecorderConfigurationMap();
}

CRecorderCmdManager::~CRecorderCmdManager(void)
{
    if (m_pRecorderConfigurationMap)
    {
        delete m_pRecorderConfigurationMap;
        m_pRecorderConfigurationMap = NULL;
    }
}

BOOL CRecorderCmdManager::InitCommunicationModel(CVirtualDeviceCommunicationModel* p_pCommunicationModel)
{
    BOOL oResult(FALSE);

    if (p_pCommunicationModel)
    {
        m_pCommunicationModel = p_pCommunicationModel;

        oResult = TRUE;
    }

    return oResult;
}

CRecorderConfigurationMap* CRecorderCmdManager::GetRecorderConfigurationMap()
{
    return m_pRecorderConfigurationMap;
}

CVirtualDeviceCommunicationModel* CRecorderCmdManager::GetCommunicationModel()
{
    return m_pCommunicationModel;
}

BOOL CRecorderCmdManager::SetRecorderParameter(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usSamplingPeriod, WORD p_usNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_SET_RECORDER_PARAMETER);

    if (m_pCommunicationModel)
    {
        // Set Parameter
        command.SetParameterData(0, &p_usSamplingPeriod, sizeof(p_usSamplingPeriod));
        command.SetParameterData(1, &p_usNbOfPrecedingSamples, sizeof(p_usNbOfPrecedingSamples));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetRecorderParameter(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusSamplingPeriod, WORD* p_pusNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_RECORDER_PARAMETER);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusSamplingPeriod, sizeof(*p_pusSamplingPeriod));
        command.GetReturnParameterData(1, p_pusNbOfPrecedingSamples, sizeof(*p_pusNbOfPrecedingSamples));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::EnableTrigger(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubTriggerType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_ENABLE_TRIGGER);

    if (m_pCommunicationModel)
    {
        // Set Parameter
        command.SetParameterData(0, &p_ubTriggerType, sizeof(p_ubTriggerType));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::DisableAllTriggers(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_DISABLE_ALL_TRIGGERS);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ActivateChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, BYTE p_ubObjectSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_ACTIVATE_CHANNEL);
    CRecorderCmdConfiguration* pRecorderConfiguration(GetRecorderConfiguration(p_KeyHandle, p_rAddressParameter));

    if (m_pCommunicationModel && pRecorderConfiguration)
    {
        // Set Parameter
        command.SetParameterData(0, &p_ubChannelNumber, sizeof(p_ubChannelNumber));
        command.SetParameterData(1, &p_usObjectIndex, sizeof(p_usObjectIndex));
        command.SetParameterData(2, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
        command.SetParameterData(3, &p_ubObjectSize, sizeof(p_ubObjectSize));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        if (oResult)
        {
            oResult = pRecorderConfiguration->SetSelObject(p_ubChannelNumber - 1, p_usObjectIndex, p_ubObjectSubIndex, ConvertSizeToType(p_ubObjectSize));
        }

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ActivateChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD p_usObjectIndex, BYTE p_ubObjectSubIndex, WORD p_usObjectType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    BYTE ubObjectSize(0);
    CCommand_VCS_DataRecorder command(REC_ACTIVATE_CHANNEL);
    CRecorderCmdConfiguration* pRecorderConfiguration(GetRecorderConfiguration(p_KeyHandle, p_rAddressParameter));

    if (m_pCommunicationModel && pRecorderConfiguration)
    {
        if (pRecorderConfiguration->SetSelObject(p_ubChannelNumber - 1, p_usObjectIndex, p_ubObjectSubIndex, p_usObjectType))
        {
            ubObjectSize = (BYTE)pRecorderConfiguration->GetSelObjectSize(p_ubChannelNumber - 1);

            // Set Parameter
            command.SetParameterData(0, &p_ubChannelNumber, sizeof(p_ubChannelNumber));
            command.SetParameterData(1, &p_usObjectIndex, sizeof(p_usObjectIndex));
            command.SetParameterData(2, &p_ubObjectSubIndex, sizeof(p_ubObjectSubIndex));
            command.SetParameterData(3, &ubObjectSize, sizeof(ubObjectSize));

            // Address parameter
            UpdateAddressParameter(p_rAddressParameter);
            command.SetLayerParameterStack(p_rAddressParameter);

            // Execute command
            oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

            // Get return parameter
            command.GetErrorCode(p_pulErrorCode);
        }
    }

    return oResult;
}

BOOL CRecorderCmdManager::ActiveChannel(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, WORD* p_pusObjectIndex, BYTE* p_pubObjectSubIndex, WORD* p_pusObjectType, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    BYTE ubObjectSize(0);
    CCommand_VCS_DataRecorder command(REC_ACTIVE_CHANNEL);

    if (m_pCommunicationModel)
    {
        // Set Parameter
        command.SetParameterData(0, &p_ubChannelNumber, sizeof(p_ubChannelNumber));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusObjectIndex, sizeof(*p_pusObjectIndex));
        command.GetReturnParameterData(1, p_pubObjectSubIndex, sizeof(*p_pubObjectSubIndex));
        command.GetReturnParameterData(2, &ubObjectSize, sizeof(ubObjectSize));
        command.GetErrorCode(p_pulErrorCode);

        if (p_pusObjectType)
        {
            *p_pusObjectType = ConvertSizeToType(ubObjectSize);
        }
    }

    return oResult;
}

BOOL CRecorderCmdManager::DeactivateAllChannels(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_DEACTIVATE_ALL_CHANNELS);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetVariable(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usIndex, WORD* p_pusVarIndex, WORD* p_pusVarSubIndex, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_VARIABLE);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        command.SetParameterData(0, &p_usIndex, sizeof(p_usIndex));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusVarIndex, sizeof(*p_pusVarIndex));
        command.GetReturnParameterData(1, p_pusVarSubIndex, sizeof(*p_pusVarSubIndex));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetMaxNbOfSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusMaxNbOfSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_MAX_NUMBER_OF_SAMPLES);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusMaxNbOfSamples, sizeof(*p_pusMaxNbOfSamples));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetNbOfRecordedSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_NUMBER_OF_RECORDED_SAMPLES);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusNbOfRecordedSamples, sizeof(*p_pusNbOfRecordedSamples));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::SetNbOfPrecedingSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_SET_NUMBER_OF_PRECEDING_SAMPLES);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Set parameter
        command.SetParameterData(0, &p_usNbOfPrecedingSamples, sizeof(p_usNbOfPrecedingSamples));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetNbOfPrecedingSamples(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfPrecedingSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_NUMBER_OF_PRECEDING_SAMPLES);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusNbOfPrecedingSamples, sizeof(*p_pusNbOfPrecedingSamples));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetNbOfSamplingVariables(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusNbOfSamplingVariables, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_NUMBER_OF_SAMPLING_VARIABLES);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusNbOfSamplingVariables, sizeof(*p_pusNbOfSamplingVariables));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetSamplingPeriod(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusSamplingPeriod, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_SAMPLING_PERIOD);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusSamplingPeriod, sizeof(*p_pusSamplingPeriod));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::StartRecorder(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_START_RECORDER);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::StopRecorder(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_STOP_RECORDER);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ForceTrigger(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_FORCE_TRIGGER);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::IsRecorderRunning(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BOOL* p_oRunning, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_IS_RECORDER_RUNNING);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_oRunning, sizeof(*p_oRunning));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::IsRecorderTriggered(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BOOL* p_oTriggered, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_IS_RECORDER_TRIGGERED);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_oTriggered, sizeof(*p_oTriggered));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetRecorderStatus(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_ulRecorderStatus, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_RECORDER_STATUS);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_ulRecorderStatus, sizeof(*p_ulRecorderStatus));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::SetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD p_usRecorderConfiguration, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_SET_RECORDER_CONFIGURATION);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Set parameter
        command.SetParameterData(0, &p_usRecorderConfiguration, sizeof(p_usRecorderConfiguration));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusRecorderConfiguration, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_RECORDER_CONFIGURATION);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusRecorderConfiguration, sizeof(*p_pusRecorderConfiguration));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ReadChannelVectorSize(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, DWORD* p_pulVectorSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_READ_CHANNEL_VECTOR_SIZE);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_pulVectorSize, sizeof(*p_pulVectorSize));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ReadChannelDataVector(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, BYTE* p_pubDataVector, DWORD p_ulVectorSize, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    const DWORD WORD_TO_BYTE_FACTOR(2);

    BOOL oResult(FALSE);
    BYTE ubChannelCounter(1);
    BYTE* pubDataBuffer(NULL);
    CRecorderCmdConfiguration* pRecorderConfiguration(0);
    DWORD ulBufferSizeRead(0);
    DWORD ulMaxNbOfSamples(0);
    DWORD ulObjectSizeInWord(0);
    DWORD ulStartOffset(0);
    DWORD ulVectorStartOffset(0);
    WORD usMaxNbOfSamples(0);
    WORD usNbOfRecordedSamples(0);
    WORD usVectorStartOffset(0);

    pubDataBuffer = (BYTE*)malloc(MAX_DATA_RECORDER_BUFFER_SIZE);

    if (m_pRecorderConfigurationMap)
    {
        if (m_pRecorderConfigurationMap->GetRecorderConfiguration(p_KeyHandle, p_rAddressParameter, pRecorderConfiguration) && pRecorderConfiguration)
        {
            // Object size defined?
            if (pRecorderConfiguration->IsSelObjectSizeAvailable(p_ubChannelNumber - 1))
            {
                // Read whole data buffer
                if (ReadDataBuffer(p_KeyHandle, p_rAddressParameter, pubDataBuffer, MAX_DATA_RECORDER_BUFFER_SIZE, &ulBufferSizeRead, &usVectorStartOffset, &usMaxNbOfSamples, &usNbOfRecordedSamples, p_pulErrorCode, p_hTransactionHandle, p_oDoLock))
                {
                    // Calculate start point
                    while ((ubChannelCounter < p_ubChannelNumber) || (ubChannelCounter > MAX_NB_OF_DATA_RECORDER_VARIABLES))
                    {
                        ulObjectSizeInWord = pRecorderConfiguration->GetSelObjectSize(ubChannelCounter - 1);
                        ulStartOffset += (usMaxNbOfSamples * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR);

                        ubChannelCounter++;
                    }

                    ulObjectSizeInWord = pRecorderConfiguration->GetSelObjectSize(p_ubChannelNumber - 1);
                    ulMaxNbOfSamples = usMaxNbOfSamples * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR;
                    ulVectorStartOffset = usVectorStartOffset * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR;

                    if (ulMaxNbOfSamples > p_ulVectorSize)
                    {
                        ulMaxNbOfSamples = p_ulVectorSize;
                    }

                    // Copy data points
                    memmove(p_pubDataVector, pubDataBuffer + ulStartOffset + ulVectorStartOffset, ulMaxNbOfSamples - ulVectorStartOffset);
                    memmove(p_pubDataVector + (ulMaxNbOfSamples - ulVectorStartOffset), pubDataBuffer + ulStartOffset, ulVectorStartOffset);

                    oResult = TRUE;
                }
            }
        }
    }

    if (pubDataBuffer)
    {
        free(pubDataBuffer);
    }

    return oResult;
}

BOOL CRecorderCmdManager::GetVectorStartOffset(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, WORD* p_pusVectorStartOffset, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_GET_VECTOR_START_OFFSET);
    HANDLE hTransaction(0);

    if (m_pCommunicationModel)
    {
        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, hTransaction);

        // Get return parameter
        command.GetReturnParameterData(0, p_pusVectorStartOffset, sizeof(*p_pusVectorStartOffset));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ReadDataBuffer(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE* p_pubDataBuffer, DWORD p_ulBufferSizeToRead, DWORD* p_pulBufferSizeRead, WORD* p_pusVectorStartOffset, WORD* p_pusMaxNbOfSamples, WORD* p_pusNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    BOOL oResult(FALSE);
    CCommand_VCS_DataRecorder command(REC_READ_DATA_BUFFER);

    if (m_pCommunicationModel)
    {
        // Set parameter
        command.ResetStatus();
        command.SetParameterData(0, &p_ulBufferSizeToRead, sizeof(p_ulBufferSizeToRead));

        // Address parameter
        UpdateAddressParameter(p_rAddressParameter);
        command.SetLayerParameterStack(p_rAddressParameter);

        // Execute command
        oResult = m_pCommunicationModel->ExecuteCommand(&command, p_KeyHandle, NULL);

        // Get return parameter
        command.GetReturnParameterData(0, p_pubDataBuffer, p_ulBufferSizeToRead);
        command.GetReturnParameterData(1, p_pulBufferSizeRead, sizeof(*p_pulBufferSizeRead));
        command.GetReturnParameterData(2, p_pusVectorStartOffset, sizeof(*p_pusVectorStartOffset));
        command.GetReturnParameterData(3, p_pusMaxNbOfSamples, sizeof(*p_pusMaxNbOfSamples));
        command.GetReturnParameterData(4, p_pusNbOfRecordedSamples, sizeof(*p_pusNbOfRecordedSamples));
        command.GetErrorCode(p_pulErrorCode);
    }

    return oResult;
}

BOOL CRecorderCmdManager::ExtractChannelDataVector(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter, BYTE p_ubChannelNumber, BYTE* p_pubDataBuffer, DWORD p_ulBufferSize, BYTE* p_pubDataVector, DWORD p_ulVectorSize, WORD p_usVectorStartOffset, WORD p_usMaxNbOfSamples, WORD p_usNbOfRecordedSamples, DWORD* p_pulErrorCode, HANDLE p_hTransactionHandle, BOOL p_oDoLock)
{
    const DWORD WORD_TO_BYTE_FACTOR(2);

    BOOL oResult(FALSE);
    BYTE ubChannelCounter(1);
    CRecorderCmdConfiguration* pRecorderConfiguration(0);
    DWORD ulMaxNbOfSamples(0);
    DWORD ulObjectSizeInWord(0);
    DWORD ulStartOffset(0);
    DWORD ulVectorStartOffset(0);

    if (m_pRecorderConfigurationMap)
    {
        if (m_pRecorderConfigurationMap->GetRecorderConfiguration(p_KeyHandle, p_rAddressParameter, pRecorderConfiguration) && pRecorderConfiguration)
        {
            // Object size defined?
            if (pRecorderConfiguration->IsSelObjectSizeAvailable(p_ubChannelNumber - 1))
            {
                // Calculate start point
                while ((ubChannelCounter < p_ubChannelNumber) || (ubChannelCounter > MAX_NB_OF_DATA_RECORDER_VARIABLES))
                {
                    ulObjectSizeInWord = pRecorderConfiguration->GetSelObjectSize(ubChannelCounter - 1);
                    ulStartOffset += (p_usMaxNbOfSamples * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR);

                    ubChannelCounter++;
                }

                ulObjectSizeInWord = pRecorderConfiguration->GetSelObjectSize(p_ubChannelNumber - 1);
                ulMaxNbOfSamples = p_usMaxNbOfSamples * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR;
                ulVectorStartOffset = p_usVectorStartOffset * ulObjectSizeInWord * WORD_TO_BYTE_FACTOR;

                // Copy data points
                memmove(p_pubDataVector, p_pubDataBuffer + ulStartOffset + ulVectorStartOffset, ulMaxNbOfSamples - ulVectorStartOffset);
                memmove(p_pubDataVector + (ulMaxNbOfSamples - ulVectorStartOffset), p_pubDataBuffer + ulStartOffset, ulVectorStartOffset);

                oResult = TRUE;
            }
        }
    }

    return oResult;
}

CRecorderCmdConfiguration* CRecorderCmdManager::GetRecorderConfiguration(HANDLE p_KeyHandle, CLayerParameterStack& p_rAddressParameter)
{
    CRecorderCmdConfiguration* pRecorderConfiguration(0);

    if (m_pRecorderConfigurationMap)
    {
        m_pRecorderConfigurationMap->GetRecorderConfiguration(p_KeyHandle, p_rAddressParameter, pRecorderConfiguration);
    }

    return pRecorderConfiguration;
}

BOOL CRecorderCmdManager::UpdateAddressParameter(CLayerParameterStack& p_rAddressParameter)
{
    BOOL oResult = FALSE;

    if (m_pCommunicationModel)
    {
        if (p_rAddressParameter.IsEmpty())
        {
            // Address parameter defined by CommunicationModel
            oResult = m_pCommunicationModel->GetLayerParameterStack(p_rAddressParameter);
        }
        else
        {
            // Address parameter already defined
            oResult = TRUE;
        }
    }

    return oResult;
}

WORD CRecorderCmdManager::ConvertSizeToType(BYTE p_ubObjectSize)
{
    WORD usObjectType;

    switch (p_ubObjectSize)
    {
        case 0: usObjectType = ODT_UNKNOWN; break;
        case 1: usObjectType = ODT_INT8; break;
        case 2: usObjectType = ODT_INT16; break;
        case 4: usObjectType = ODT_INT32; break;
        case 8: usObjectType = ODT_INT64; break;
        default: usObjectType = ODT_UNKNOWN;
    }

    return usObjectType;
}
