#include "stdafx.h"
#include <Process/MmcProcess.h>
#include <CommunicationModel/Common/CommunicationModelDefinitions.h>
#include <Drv/Helper/UsbApi/UsbApiLinux.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInManager.h>
#include <CommunicationModel/ProtocolStack/Classes/ProtocolStacks/InterfacePlugInManager/InterfacePlugInMonitorLinux.h>

using namespace std;

CInterfacePlugInMonitorBase::CInterfacePlugInMonitorBase()
    : m_pPlugInManager(0)
	, m_HidHotplugHandle(0)
	, m_FdtiHotplugHandle(0)
	, m_oNotificationsRegistered(FALSE)
{
}

CInterfacePlugInMonitorBase::~CInterfacePlugInMonitorBase()
{
	DeletePlugInDevices();
	UnregisterNotifications();
}

int CInterfacePlugInMonitorBase::UsbHotplugNotification(libusb_context *p_pContext, libusb_device *p_pDevice, libusb_hotplug_event p_Event, void *p_pUserData)
{
	CInterfacePlugInMonitorBase* pPluginMonitor = (CInterfacePlugInMonitorBase*) p_pUserData;
	libusb_device_descriptor desc;

	if(pPluginMonitor && p_pDevice)
	{
		if (LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED == p_Event)
		{
			if(libusb_get_device_descriptor(p_pDevice, &desc) == 0)
			{
				if(pPluginMonitor->IsDriverSupported(desc.idVendor, desc.idProduct))
				{
					pPluginMonitor->DeviceAdded(p_pDevice);
				}
			}
		}
		else if (LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT == p_Event)
		{
			if(libusb_get_device_descriptor(p_pDevice, &desc) == 0)
			{
				if(pPluginMonitor->IsDriverSupported(desc.idVendor, desc.idProduct))
				{
					pPluginMonitor->DeviceRemoved(p_pDevice);
				}
			}
		}
	}

	return 0;
}

BOOL CInterfacePlugInMonitorBase::IsDriverSupported(WORD p_usVendorId, WORD p_usProductId)
{
	BOOL oResult = FALSE;

	if(m_pPlugInManager)
	{
		oResult = m_pPlugInManager->IsDriverSupported(p_usVendorId, p_usProductId);
	}

	return oResult;
}

void CInterfacePlugInMonitorBase::DeletePlugInDevices()
{
	list<CPlugInDevice*>::iterator it = m_PlugInDeviceList.begin();
	CPlugInDevice* pPlugInDevice = 0;

	while(it != m_PlugInDeviceList.end())
	{
		pPlugInDevice = (*it);

		m_PlugInDeviceList.erase(it);

		delete pPlugInDevice;

		it = m_PlugInDeviceList.begin();
	}
}

BOOL CInterfacePlugInMonitorBase::SearchPlugInDevice(uint8_t p_BusNumber, uint8_t p_PortNumber, CPlugInDevice*& p_pPlugInDevice)
{
	BOOL oResult = FALSE;

	for(list<CPlugInDevice*>::iterator it = m_PlugInDeviceList.begin(); it != m_PlugInDeviceList.end(); it++)
	{
		CPlugInDevice* pPlugInDevice = (*it);

		if(pPlugInDevice->m_ucBusNumber == p_BusNumber &&
		   pPlugInDevice->m_ucPortNumber == p_PortNumber)
		{
			p_pPlugInDevice = pPlugInDevice;
			oResult = TRUE;
			break;
		}
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::DeviceAdded(libusb_device *p_pDevice)
{
	BOOL oResult = FALSE;

	oResult = DevicePlugInStateChanged(p_pDevice, true);

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::DevicePlugInStateChanged(libusb_device *p_pDevice, bool p_oPluggedIn)
{
	BOOL oResult = FALSE;
	uint8_t busNumber = 0;
	uint8_t portNumber = 0;
	CPlugInDevice* pPlugInDevice = 0;

	if(m_pPlugInManager && p_pDevice)
	{
		busNumber = libusb_get_bus_number(p_pDevice);
		portNumber = libusb_get_port_number(p_pDevice);

		if(!SearchPlugInDevice(busNumber, portNumber, pPlugInDevice))
		{
			pPlugInDevice = new CPlugInDevice();
			m_PlugInDeviceList.push_back(pPlugInDevice);
		}

		if(pPlugInDevice)
		{
			pPlugInDevice->m_oPluggedIn = p_oPluggedIn;
			pPlugInDevice->m_ulTimestamp = MmcGetTickCount();
			pPlugInDevice->m_oUpdated = false;
		}

		oResult = TRUE;
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::DeviceRemoved(libusb_device *p_pDevice)
{
	BOOL oResult = FALSE;

	oResult = DevicePlugInStateChanged(p_pDevice, false);

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::Init(CInterfacePlugInManager* p_pPlugInManager)
{
    BOOL oResult(FALSE);

    if(p_pPlugInManager)
    {
		m_pPlugInManager = p_pPlugInManager;

		oResult = RegisterForNotifications();
    }

    return oResult;
}

BOOL CInterfacePlugInMonitorBase::RegisterForHidNotifications()
{
	BOOL oResult(TRUE);
	const unsigned short HID_VENDOR_ID = 0x24e7;
	const unsigned short HID_PRODUCT_ID = 0x3b01;

	int eventMask = LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED | LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT;

	int rc = libusb_hotplug_register_callback(UsbContext,
											(libusb_hotplug_event)eventMask,
											(libusb_hotplug_flag)0, HID_VENDOR_ID, HID_PRODUCT_ID,
											LIBUSB_HOTPLUG_MATCH_ANY, UsbHotplugNotification, (void *)this,
											&m_HidHotplugHandle);

	oResult = (LIBUSB_SUCCESS == rc);

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::UnregisterHidNotifications()
{
	BOOL oResult = TRUE;

	if(m_HidHotplugHandle)
	{
		libusb_hotplug_deregister_callback(UsbContext, m_HidHotplugHandle);
		m_HidHotplugHandle = 0;
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::RegisterForFtdiNotifications()
{
	BOOL oResult(TRUE);
	const unsigned short FTDI_VENDOR_ID = 0x0403;
	const unsigned short FTDI_PRODUCT_ID = 0xa8b0;

	int eventMask = LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED | LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT;

	int rc = libusb_hotplug_register_callback(UsbContext,
											(libusb_hotplug_event)eventMask,
											(libusb_hotplug_flag)0, FTDI_VENDOR_ID, FTDI_PRODUCT_ID,
											LIBUSB_HOTPLUG_MATCH_ANY, UsbHotplugNotification, (void *)this,
											&m_HidHotplugHandle);

	oResult = (LIBUSB_SUCCESS == rc);

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::UnregisterFtdiNotifications()
{
	BOOL oResult(TRUE);

	if(m_FdtiHotplugHandle)
	{
		libusb_hotplug_deregister_callback(UsbContext, m_FdtiHotplugHandle);
		m_FdtiHotplugHandle = 0;
	}

	return oResult;
}


BOOL CInterfacePlugInMonitorBase::RegisterForNotifications()
{
	BOOL oResult = FALSE;

	if(m_pPlugInManager)
	{
		BOOL oUsbRegistered = m_pPlugInManager->IsUsbInterfaceRegistered();

		if(oUsbRegistered)
		{
			if(m_pPlugInManager->IsDriverSupported(DRIVER_HID, USB_VENDOR_PRODUCT_ID_EPOS4))
			{
				oResult = RegisterForHidNotifications();
			}

			if(m_pPlugInManager->IsDriverSupported(DRIVER_FTD2XX, USB_VENDOR_PRODUCT_ID_EPOS2))
			{
				oResult = RegisterForFtdiNotifications();
			}

			m_oNotificationsRegistered = oResult;
		}
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::UnregisterNotifications()
{
	BOOL oResult = FALSE;

	if(m_pPlugInManager)
	{
		BOOL oUsbRegistered = m_pPlugInManager->IsUsbInterfaceRegistered();

		if(oUsbRegistered)
		{
			oResult = UnregisterHidNotifications();

			if(oResult)
			{
				oResult = UnregisterFtdiNotifications();
			}

			m_oNotificationsRegistered = !oResult;
		}
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::IsReady()
{
    return m_oNotificationsRegistered;
}

void CInterfacePlugInMonitorBase::ConvertMilisecondsToTimeval(unsigned int milliseconds, timeval& p_rTimeval)
{
	const int MILISEC_TO_SEC_FACTOR = 1000;

	p_rTimeval.tv_sec = milliseconds / MILISEC_TO_SEC_FACTOR;
	p_rTimeval.tv_usec = (milliseconds % MILISEC_TO_SEC_FACTOR) * MILISEC_TO_SEC_FACTOR;
}

BOOL CInterfacePlugInMonitorBase::UpdateHotPlugIn()
{
    BOOL oResult(TRUE);
    timeval timeout;
    int completed = 0;
    const int MIN_WAIT_TIME_IN_MS = 20;

    if(m_oNotificationsRegistered)
    {
    	ConvertMilisecondsToTimeval(MIN_WAIT_TIME_IN_MS, timeout);

    	if(libusb_try_lock_events(UsbContext) == 0)
    	{
    		if(!libusb_event_handling_ok(UsbContext))
			{
				oResult = FALSE;
			}
			else
			{
				if(libusb_handle_events_timeout_completed(UsbContext, &timeout, &completed) != LIBUSB_SUCCESS)
				{
					oResult = FALSE;
				}
			}

    		libusb_unlock_events(UsbContext);
    	}
    	else
    	{
    		libusb_lock_event_waiters(UsbContext);

			if (!libusb_event_handler_active(UsbContext))
			{
				libusb_unlock_event_waiters(UsbContext);
			}
			else
			{
				libusb_wait_for_event(UsbContext, &timeout);

				libusb_unlock_event_waiters(UsbContext);
			}

			oResult = FALSE;
    	}
    }

    return oResult;
}

BOOL CInterfacePlugInMonitorBase::GetLastNotificationTimestamp(list<CPlugInDevice*> p_plugInDeviceList, DWORD& p_rTimestamp)
{
	BOOL oResult = FALSE;
	CPlugInDevice* pPlugInDevice = 0;

	p_rTimestamp = 0;

	for(list<CPlugInDevice*>::iterator it = p_plugInDeviceList.begin(); it != p_plugInDeviceList.end(); it++)
	{
		pPlugInDevice = (*it);

		if(pPlugInDevice->m_ulTimestamp > p_rTimestamp && !pPlugInDevice->m_oUpdated)
		{
			p_rTimestamp = pPlugInDevice->m_ulTimestamp;
			oResult = TRUE;
		}
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::MarkAllDevicesAsUpdated(list<CPlugInDevice*> p_plugInDeviceList)
{
	BOOL oResult = TRUE;
	CPlugInDevice* pPlugInDevice = 0;

	for(list<CPlugInDevice*>::iterator it = p_plugInDeviceList.begin(); it != p_plugInDeviceList.end(); it++)
	{
		pPlugInDevice = (*it);
		pPlugInDevice->m_oUpdated = true;
	}

	return oResult;
}

BOOL CInterfacePlugInMonitorBase::CheckNotification()
{
	DWORD ulDelayTime = 0;
	DWORD ulLastNotificationTimestamp = 0;
	DWORD ulLastEventTimestamp = 0;
	BOOL oResult = FALSE;
	list<CPlugInDevice*> plugInDeviceList;

	if(m_pPlugInManager)
	{
		m_pPlugInManager->GetNotificationDelay(ulDelayTime);

		plugInDeviceList = m_PlugInDeviceList;

		if(GetLastNotificationTimestamp(plugInDeviceList, ulLastNotificationTimestamp))
		{
			ulLastEventTimestamp = MmcGetTickCount() - ulLastNotificationTimestamp;

			if(ulDelayTime < ulLastEventTimestamp)
			{
				m_pPlugInManager->UpdateDevice();

				MarkAllDevicesAsUpdated(plugInDeviceList);

				oResult = TRUE;
			}
		}
	}

	return oResult;
}
