#pragma once

#include <MmcTypeDefinition.h>
#include <Storage/MmcUserSettings.h>

class CUsbDeviceInfo;
typedef std::list<CUsbDeviceInfo*> tUsbDeviceInfoList;

class CUsbDeviceInfo
{
public:
    CUsbDeviceInfo(void);
    virtual ~CUsbDeviceInfo(void);

    CUsbDeviceInfo& operator=(CUsbDeviceInfo& p_rOther);
    virtual CUsbDeviceInfo* Clone();

    //Enums
    enum ELogInfoType
    {
        LIT_IDENTIFIER,
        LIT_STATE
    };

    enum ECompareMode
    {
        CM_PORT_CHAIN                       = 0,
        CM_DRV_PORT_INFOS                   = 1,
        CM_PORT_CHAIN_AND_DRV_PORT_INFOS    = 2
    };

    //Class Infos
    virtual void InitClassInfo() { m_strClassType = _T("-CUsbDeviceInfo") + m_strClassType;}
    BOOL IsKindOf(CStdString strClassType);
    CStdString GetClassType();
    
    //Initialisation
    virtual BOOL Init(CUsbDeviceInfo& p_rDeviceInfo);
    BOOL Init(CStdString p_PortChain, CStdString p_DevicePath, WORD p_usVendorId, WORD p_usProductId, CStdString p_SerialNumber);
    BOOL InitState(CUsbDeviceInfo* p_pDeviceInfo);
    BOOL InitPortName(CUsbDeviceInfo* p_pDeviceInfo);

    //Update
    virtual BOOL Update(CUsbDeviceInfo& p_rDeviceInfo);

    //Compare
    BOOL IsEqual(CUsbDeviceInfo& p_rDeviceInfo, ECompareMode p_eCompareMode = CM_PORT_CHAIN);
    BOOL IsPortChainEqual(CStdString p_PortChain);
    BOOL IsPortChainEmpty();
    BOOL IsDrvPortInfoEqual(CStdString p_PortChain);
    BOOL IsDevicePathEqual(CStdString p_DevicePath);
    BOOL IsSerialNumberEqual(CStdString p_SerialNumber);
    BOOL IsSerialNumberEmpty();
    BOOL IsPortNameEqual(CStdString p_PortName);
  
    //State Checks
    BOOL IsPluggedIn();
    BOOL IsOpened();
    BOOL IsOpenedBy(DWORD p_ulProcessId, DWORD m_ulInstanceValue);

    //State Infos
    BOOL SetOpenedByProcess(DWORD p_ulOpenedByProcess);
    BOOL SetOpenedByInstance(DWORD p_ulOpenedByInstance);
    BOOL SetOpenedBy(CStdString p_ulOpenedBy);
    BOOL SetPluggedIn(DWORD p_ulPluggedIn);
    DWORD GetOpenedByProcess();
    DWORD GetOpenedByInstance();
    CStdString GetOpenedBy();
    DWORD GetPluggedIn();

    //Log Infos
    virtual BOOL GetLogInfos(CStdString& p_rInfos, ELogInfoType p_eType);

    //Identifiers
    BOOL SetPortChain(CStdString p_PortChain);
    BOOL SetDrvPortInfo(CStdString p_DrvPortInfo);
    BOOL SetDevicePath(CStdString p_DevicePath);
    BOOL SetVendorId(CStdString p_VendorId);
    BOOL SetProductId(CStdString p_ProductId);
    BOOL SetSerialNumber(CStdString p_SerialNumber);
    BOOL SetPortName(CStdString p_PortName);
    CStdString GetPortChain();
    CStdString GetDrvPortInfo();
    CStdString GetDevicePath();
    CStdString GetPortName();
    CStdString GetVendorId();
    CStdString GetProductId();
    CStdString GetSerialNumber();
    BOOL GetPortName(CStdString& p_rPortName);
    BOOL GetPortNameIndex(int& p_rIndex);

    //Storage
    virtual BOOL WriteTo(CMmcUserSettings* p_pUserSettings);
    virtual BOOL ReadFrom(CMmcUserSettings* p_pUserSettings);

protected:
    //ClassInfos
    CStdString m_strClassType;

    //Registry Enumeration Infos
    CStdString m_PortChain;
    CStdString m_DrvPortInfo;
    CStdString m_DevicePath;
    WORD m_usVendorId;
    WORD m_usProductId;
    CStdString m_SerialNumber;
    CStdString m_PortName;
    DWORD m_ulPluggedIn;
    DWORD m_ulOpenedByProcess;
    DWORD m_ulOpenedByInstance;
    CStdString m_OpenedBy;
};

