#pragma once

#include <MmcTypeDefinition.h>

#include <initguid.h>
#include <WinIoCtl.h>
#include "win32/setupapi.h"
#include "win32/usbiodef.h"
#include "win32/usbioctl.h"
#include "win32/devpkey.h"
#include "win32/cfgmgr32.h"

typedef HDEVINFO (CALLBACK* t_SetupDiGetClassDevs) (LPGUID, PCWSTR, HWND, DWORD);
typedef BOOL (CALLBACK* t_SetupDiEnumDeviceInfo) (HDEVINFO, DWORD, PSP_DEVINFO_DATA);
typedef BOOL (CALLBACK* t_SetupDiEnumDeviceInterfaces) (HDEVINFO, PSP_DEVINFO_DATA, LPGUID, DWORD, PSP_DEVICE_INTERFACE_DATA);
typedef BOOL (CALLBACK* t_SetupDiGetDeviceInterfaceDetail) (HDEVINFO, PSP_DEVICE_INTERFACE_DATA, PSP_DEVICE_INTERFACE_DETAIL_DATA, DWORD, PDWORD, PSP_DEVINFO_DATA);
typedef BOOL (CALLBACK* t_SetupDiGetDeviceProperty) (HDEVINFO, PSP_DEVINFO_DATA, const DEVPROPKEY*, DEVPROPTYPE*, PBYTE, DWORD, PDWORD, DWORD);
typedef BOOL (CALLBACK* t_SetupDiClassGuidsFromName) (PCTSTR, LPGUID, DWORD, PDWORD);
typedef BOOL (CALLBACK* t_SetupDiGetDeviceRegistryProperty) (HDEVINFO, PSP_DEVINFO_DATA, DWORD, PDWORD, PBYTE, DWORD, PDWORD);
typedef BOOL (CALLBACK* t_SetupDiDestroyDeviceInfoList) (HDEVINFO);

typedef int (CALLBACK* t_CmGetDeviceInterfaceListSize) (PULONG, LPGUID, DEVINSTID, ULONG);
typedef int (CALLBACK* t_CmGetDeviceInterfaceList) (LPGUID, DEVINSTID, TCHAR*, ULONG, ULONG);
typedef int (CALLBACK* t_CmLocateDevNode) (PDEVINST, DEVINSTID, ULONG);
typedef int (CALLBACK* t_CmGetDevNodeRegistryProperty) (DEVINST, ULONG, PULONG, PVOID, PULONG, ULONG);
typedef int (CALLBACK* t_CmGetDevNodeProperty) (DEVINST, const DEVPROPKEY*, DEVPROPTYPE*, PBYTE, PULONG, ULONG);
typedef int (CALLBACK* t_CmGetChild) (PDEVINST, DEVINST, ULONG);
typedef int (CALLBACK* t_CmGetSibling) (PDEVINST, DEVINST, ULONG);
typedef int (CALLBACK* t_CmGetDeviceIdSize) (PULONG, DEVINST, ULONG);
typedef int (CALLBACK* t_CmGetDeviceId) (DEVINST, PWSTR, ULONG, ULONG);

class CUsbApiWin
{
public:
    CUsbApiWin(void);
    virtual ~CUsbApiWin(void);

    BOOL SetupDiGetClassDevs(GUID* p_pGuid, DWORD p_ulFlags, HDEVINFO& p_rhDevInfo);
    BOOL SetupDiGetClassDevs(GUID* p_pGuid, CStdString p_enumerator, DWORD p_ulFlags, HDEVINFO& p_rhDevInfo);
    BOOL SetupDiEnumDeviceInfo(HDEVINFO& p_rhDevInfo, DWORD p_ulIndex, SP_DEVINFO_DATA& p_rDeviceInfoData);
    BOOL SetupDiEnumDeviceInterfaces(HDEVINFO& p_rhDevInfo, GUID* p_pGuid, DWORD p_ulIndex, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData);
            
    BOOL SetupDiGetDeviceInterfaceDetail(HDEVINFO& p_rhDevInfo, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData, SP_INTERFACE_DEVICE_DETAIL_DATA*& p_rpInterfaceDeviceDetailData, SP_DEVINFO_DATA& p_rDeviceInfoData);
    BOOL SetupDiGetDeviceRegistryProperty(HDEVINFO& p_rhDevInfo, SP_DEVINFO_DATA& p_rDeviceInfoData, int p_lProperty, CStdString& p_rValue);
    BOOL SetupDiGetDeviceProperty(HDEVINFO& p_rhDevInfo, SP_DEVINFO_DATA& p_rDeviceInfoData, const DEVPROPKEY* p_pPropertyKey, CStdString& p_rValue);
    BOOL SetupDiGetDeviceProperty(HDEVINFO& p_rhDevInfo, SP_DEVINFO_DATA& p_rDeviceInfoData, const DEVPROPKEY* p_pPropertyKey, CStdStringArray& p_rValues);
    BOOL SetupDiGetDeviceProperty(HDEVINFO& p_rhDevInfo, SP_DEVINFO_DATA& p_rDeviceInfoData, const DEVPROPKEY* p_pPropertyKey, GUID& p_rGuid);
    BOOL SetupDiClassGuidFromName(CStdString p_ClassName, GUID& p_rGuid);
    BOOL SetupDiDeleteInterfaceDetailData(SP_INTERFACE_DEVICE_DETAIL_DATA*& p_rpInterfaceDeviceDetailData);
    BOOL SetupDiDestroyDeviceInfoList(HDEVINFO& p_rhDevInfo);

    BOOL CmGetDeviceInterfaceList(GUID* p_pGuid, DWORD p_ulFlags, CStdStringArray& p_rDevicePathes);
    BOOL CmLocateDevNode(CStdString p_deviceId, DEVINST& p_rdnDevInst);
    BOOL CmGetDevNodeRegistryProperty(DEVINST& p_rdnDevInst, int p_lProperty, CStdString& p_rValue);
    BOOL CmGetDevNodeProperty(DEVINST& p_rdnDevInst, const DEVPROPKEY* p_pPropertyKey, CStdString& p_rValue);
    BOOL CmGetChild(DEVINST p_dnDevInst, DEVINST& p_rdnChildDevInst);
    BOOL CmGetSibling(DEVINST p_dnDevInst, DEVINST& p_rdnSiblingDevInst);
    BOOL CmGetDeviceId(DEVINST p_dnDevInst, CStdString& p_rDeviceId);

    BOOL IoctlUsbGetRootHubName(CStdString p_devicePath, CStdString& p_rName);
    BOOL IoctlUsbGetNodeConnectionName(CStdString p_devicePath, DWORD p_ulPortNumber, CStdString& p_rName);
    BOOL IoctlUsbGetNodeConnectionDriverKeyName(CStdString p_devicePath, DWORD p_ulPortNumber, CStdString& p_rDriverKeyName);
    BOOL IoctlUsbGetNodeInformation(CStdString p_devicePath, USB_NODE_INFORMATION& p_rNodeInformation);
    BOOL IoctlUsbGetNodeConnectionInformationEx(CStdString p_devicePath, DWORD p_ulPortNumber, USB_NODE_CONNECTION_INFORMATION_EX& p_rNodeConnectionInformationEx);
    BOOL IoctlUsbGetDescriptorFromNodeConnection(CStdString p_devicePath, DWORD p_ulPortNumber, DWORD p_ulDescriptorId, CStdString& p_rDesciptor);

    void StartTimer();
    void StopTimer();
    double GetTimeInMilliseconds();

private:
    BOOL ResetSetupApiLibraryFunctions();
    BOOL LoadSetupApiLibraryFunctions();
    BOOL FreeSetupApiLibraryFunctions();
    BOOL AreSetupApiLibraryFunctionsLoaded();

    BOOL ResetCfgMgr32LibraryFunctions();
    BOOL LoadCfgMgr32LibraryFunctions();
    BOOL FreeCfgMgr32LibraryFunctions();
    BOOL AreCfgMgr32LibraryFunctionsLoaded();
    
    PTSTR WideStrToMultiStr (__in LPCWSTR WideStr);
    CStdString GetLastError();

private:
    HINSTANCE m_hSetupApiDLL;
    t_SetupDiGetClassDevs m_lpSetupDiGetClassDevs;
    t_SetupDiEnumDeviceInterfaces m_lpSetupDiEnumDeviceInterfaces;
    t_SetupDiEnumDeviceInfo m_lpSetupDiEnumDeviceInfo;
    t_SetupDiGetDeviceInterfaceDetail m_lpSetupDiGetDeviceInterfaceDetail;
    t_SetupDiGetDeviceRegistryProperty m_lpSetupDiGetDeviceRegistryProperty;
    t_SetupDiGetDeviceProperty m_lpSetupDiGetDeviceProperty;
    t_SetupDiClassGuidsFromName m_lpSetupDiClassGuidsFromName;
    t_SetupDiDestroyDeviceInfoList m_lpSetupDiDestroyDeviceInfoList;

    HINSTANCE m_hCfgMgr32DLL;
    t_CmGetDeviceInterfaceListSize m_lpCmGetDeviceInterfaceListSize;
    t_CmGetDeviceInterfaceList m_lpCmGetDeviceInterfaceList;
    t_CmLocateDevNode m_lpCmLocateDevNode;
    t_CmGetDevNodeRegistryProperty m_lpCmGetDevNodeRegistryProperty;
    t_CmGetDevNodeProperty m_lpCmGetDevNodeProperty;
    t_CmGetChild m_lpCmGetChild;
    t_CmGetSibling m_lpCmGetSibling;
    t_CmGetDeviceIdSize m_lpCmGetDeviceIdSize;
    t_CmGetDeviceId m_lpCmGetDeviceId;
};

