#pragma once

#include "UsbTreeFactoryDef.h"
#include <Drv/Helper/UsbApi/UsbApiWin.h>

class CUsbDevices;
class CUsbDevice;
class CUsbHub;
class CUsbHostController;

class CUsbTreeFactoryBase : public CUsbTreeFactoryDef
{
public:
    CUsbTreeFactoryBase();
    virtual ~CUsbTreeFactoryBase();

    BOOL LoadHostControllers(std::list<CUsbHostController*>& p_rHostControllers);
    BOOL LoadUsbDevices(std::list<CUsbDevice*>& p_rUsbDevices);

private:
    void CreateUsbApi();
    void DeleteUsbApi();

    // UsbDevices
    BOOL EnumerateDevices(std::list<CUsbDevice*>& p_rUsbDevices);
    BOOL LoadDevice(CUsbDevice* p_pDevice, CStdString p_devicePath);
    BOOL EnumerateChildDevices(CUsbDevice* p_pParentDevice, DEVINST& p_rDevInst);

    // HostControllers
    BOOL EnumerateHostControllers(std::list<CUsbHostController*>& p_rHostControllers);
    BOOL LoadHostController(CUsbHostController* p_pHostController, HDEVINFO& p_rhDevInfo, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData, DWORD p_ulIndex);
    BOOL EnumerateRootHub(CUsbHostController* p_pHostController);
    BOOL LoadRootHub(CUsbHostController* p_pHostController, CUsbHub* p_pHub);
    BOOL EnumerateHubPorts(CUsbHub* p_pHub);
    BOOL LoadHubPort(CUsbHub* p_pHub, DWORD p_ulPortIndex);
    BOOL EnumerateHub(CUsbHub* p_pParentHub, DWORD p_ulPortIndex);
    BOOL LoadHub(CUsbHub* p_pParentHub, CUsbHub* p_pHub, DWORD p_ulPortIndex);
    BOOL EnumerateDevice(CUsbHub* p_pParentHub, DWORD p_ulPortIndex);
    BOOL LoadDevice(CUsbHub* p_pParentHub, CUsbDevice* p_pDevice, DWORD p_ulPortIndex);
    BOOL EnumerateChildDevices(CUsbDevice* p_pParentDevice);
    BOOL LoadChildDevice(CUsbDevice* p_pParentDevice, CUsbDevice* p_pDevice, DWORD p_ulPortIndex);

    // Api
    BOOL SetupDiGetClassDevs(GUID* p_prGuid, DWORD p_ulFlags, HDEVINFO& p_rhDevInfo);
    BOOL SetupDiGetClassDevs(GUID* p_prGuid, CStdString p_enumerator, DWORD p_ulFlags, HDEVINFO& p_rhDevInfo);
    BOOL SetupDiEnumDeviceInfo(HDEVINFO& p_rhDevInfo, DWORD p_ulIndex, SP_DEVINFO_DATA& p_rDeviceInfoData);
    BOOL SetupDiEnumDeviceInterfaces(HDEVINFO& p_rhDevInfo, GUID* p_pGuid, DWORD p_ulIndex, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData);
    BOOL SetupDiGetDeviceInterfaceDetail(HDEVINFO& p_rhDevInfo, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData, SP_INTERFACE_DEVICE_DETAIL_DATA*& p_rpInterfaceDeviceDetailData, SP_DEVINFO_DATA& p_rDeviceInfoData);
    BOOL SetupDiGetDeviceRegistryProperty(HDEVINFO& p_rhDevInfo, SP_DEVINFO_DATA& p_rDeviceInfoData, int p_lProperty, CStdString& p_rValue);
    BOOL SetupDiDeleteInterfaceDetailData(SP_INTERFACE_DEVICE_DETAIL_DATA*& p_rpInterfaceDeviceDetailData);
    BOOL SetupDiDestroyDeviceInfoList(HDEVINFO& p_rhDevInfo);
    
    // Infos
    BOOL GetDevicePath(HDEVINFO& p_rhDevInfo, SP_DEVICE_INTERFACE_DATA& p_rDeviceInterfaceData, CStdString& p_rDevicePath);
    BOOL GetRootHubDevicePath(CUsbHostController* p_pHostController, CStdString& p_rDevicePath);
    BOOL GetHubDevicePath(CUsbHub* p_pParentHub, DWORD p_ulPortIndex, CStdString& p_rDevicePath);
    BOOL GetHubPortCount(CStdString p_DevicePath, DWORD& p_rulPortCount);
    BOOL GetRootHubName(CUsbHostController* p_pHostController, CStdString& p_rName);
    BOOL GetNodeInformation(CStdString p_DevicePath, USB_NODE_INFORMATION& p_rNodeInformation);
    BOOL GetNodeConnectionName(CStdString p_DevicePath, DWORD p_ulPortIndex, CStdString& p_rName);
    BOOL GetNodeConnectionDriverKeyName(CStdString p_DevicePath, DWORD p_ulPortIndex, CStdString& p_rDriverKeyName);
    BOOL GetNodeConnectionInformationEx(CStdString p_DevicePath, DWORD p_ulPortIndex, USB_NODE_CONNECTION_INFORMATION_EX& p_rNodeConnectionInformationEx);
    BOOL GetDescriptorFromNodeConnection(CStdString p_DevicePath, DWORD p_ulPortIndex, DWORD p_ulDescriptorId, CStdString& p_rDescriptor);
    BOOL GetUsbDeviceByDriverKeyName(CStdString p_driverKeyName, CUsbDevice*& p_rpUsbDevice);
    BOOL GetDeviceIdentifiers(CUsbHub* p_pParentHub, DWORD p_ulPortIndex, WORD& p_rusVendorId, WORD& p_rusProductId, WORD& p_rusSerialNumberId, CStdString& p_rSerialNumber);
    BOOL GetDevicePathByDeviceId(CStdString p_deviceId, CStdString& p_rDevicePath);
    BOOL GetDeviceIdByDevicePath(CStdString p_devicePath, CStdString& p_rDeviceId);

    // Helper
    CStdString ExtractClassNameFromDrviceId(CStdString p_deviceId);
    CStdString ConvertPortNameToDevicePath(CStdString p_portName);
    CStdString ConvertDeviceIdToDevicePath(CStdString p_deviceId, CStdString p_guid);
    CStdString ConvertDevicePathToDeviceId(CStdString p_devicePath);
    CStdString ConvertGuidToString(GUID& p_rGuid);
    BOOL GetGuidFromClassName(CStdString p_className, GUID& p_rGuid);
    BOOL GetGuidFromClassName(CStdString p_className, CStdString& p_rGuid);
    
    // Debugging
    void StartTimer();
    void StopTimer();
    double GetTimeInMilliseconds();
    void ShowTimeDebugMessage(CStdString p_Message);
    void ShowDebugMessage(CStdString p_Message);

private:
    CUsbApiWin* m_pUsbApi;

    LARGE_INTEGER m_frequency;
    LARGE_INTEGER m_startCount;
    LARGE_INTEGER m_endCount;
};
