#pragma once

#include <afx.h>
#define SS_ANSI
#include <drv/hid/MmcHidHndlDef.h>
#include <StdString.h>

#include "win32/hid.h"

//**** Interface functions for HID driver ******************
typedef BOOLEAN (CALLBACK* t_FlushQueue) (HANDLE);
typedef BOOLEAN (CALLBACK* t_FreePreparsedData) (PHIDP_PREPARSED_DATA);
typedef BOOLEAN (CALLBACK* t_GetAttributes) (HANDLE, PHIDD_ATTRIBUTES);
typedef BOOLEAN (CALLBACK* t_GetConfiguration) (HANDLE, PHIDD_CONFIGURATION, ULONG);
typedef BOOLEAN (CALLBACK* t_GetFeature) (HANDLE, PVOID*, ULONG);
typedef BOOLEAN (CALLBACK* t_GetHidGuid) (LPGUID);
typedef BOOLEAN (CALLBACK* t_GetManufacturerString) (HANDLE, PVOID, ULONG);
typedef BOOLEAN (CALLBACK* t_GetNumInputBuffers) (HANDLE, PULONG);
typedef BOOLEAN (CALLBACK* t_GetPhysicalDescriptor) (HANDLE, PVOID, ULONG);
typedef BOOLEAN (CALLBACK* t_GetPreparsedData) (HANDLE, PHIDP_PREPARSED_DATA*);
typedef BOOLEAN (CALLBACK* t_GetSerialNumberString) (HANDLE, PVOID*, ULONG);
typedef BOOLEAN (CALLBACK* t_SetConfiguration) (HANDLE, PHIDD_CONFIGURATION, ULONG);
typedef NTSTATUS (CALLBACK* t_GetCaps) (PHIDP_PREPARSED_DATA, PHIDP_CAPS);
//**** Interface functions for HID driver ******************

class CMmcHidHndlBase : public CMmcHidHndlDef
{
public:
    CMmcHidHndlBase();
    virtual ~CMmcHidHndlBase();

public:
    bool Init();
    virtual CStdString GetDeviceClassType();
    virtual bool GetDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, unsigned short p_usVendorId, unsigned short p_usProductId, BOOL p_oIsHidDevice);
    bool OpenPort(CHidDeviceInfo& p_rInfo);
    bool ClosePort();
    bool IsValid();
    bool IsOpened();
    bool PurgeBuffer();
    bool WaitForOutReportData(unsigned long p_ulTimeout);
    unsigned long WaitForOverlapped(unsigned long p_ulTimeout);
    bool UpdateOutReport();
    bool ReadFile(void* p_pBuffer, unsigned long p_ulNumberOfBytesToRead, unsigned long* p_pulNumberOfBytesRead);
    bool WriteFile(void* p_pBuffer, unsigned long p_ulNumberOfBytesToWrite, unsigned long* p_pulNumberOfBytesWritten);
    bool CancelIo();
    bool GetOverlappedResult(DWORD* p_pulNumberOfBytesTransferred, BOOL p_oWait);
    CStdString GetLastError();
    bool ResetOutReport();
    int GetInputReportByteLength();
    int GetOutputReportByteLength();
    bool SetBaudrate(DWORD p_ulBaudrate);

protected:
    bool LoadLibraryFunctions();
    bool FreeLibraryFunctions();
    bool AreLibraryFunctionsLoaded();

private:
    // Logging
    void LogDeviceInfosFromUsbTree(tUsbDeviceInfoList& p_rDeviceInfoList);
    void LogDeviceInfos(unsigned short p_vendorId, unsigned short p_productId, BOOL p_oIsHidDevice);

    bool IsValidBaudrate(unsigned int p_ulBaudrate);

private:
    GUID m_GuidHid;
    HANDLE m_hEventObject;
    HANDLE m_hDeviceObject;
    HIDP_CAPS m_Capabilities;
    OVERLAPPED m_Overlapped;

    // Function Pointers HID
    HINSTANCE m_hHidDrvDLL;
    t_FlushQueue m_lpFlushQueue;
    t_FreePreparsedData m_lpFreePreparsedData;
    t_GetAttributes m_lpGetAttributes;
    t_GetConfiguration m_lpGetConfiguration;
    t_GetHidGuid m_lpGetHidGuid;
    t_GetPreparsedData m_lpGetPreparsedData;
    t_GetCaps m_lpGetCaps;
};
