#pragma once

#include <StdString.h>
#include <list>
#include <map>
#include <Drv/SocketCAN/MmcSocketCANHndlDef.h>
#include <Drv/SocketCAN/SocketCANDevice.h>
#include <Drv/SocketCAN/SocketCANPort.h>

typedef std::list<CSocketCANPort*> tCanPortList;

class CMmcSocketCANHndlBase : public CMmcSocketCANHndlDef
{
public:
    CMmcSocketCANHndlBase();
    virtual ~CMmcSocketCANHndlBase();

public:
    bool GetDeviceInfos(tSocketCANDeviceInfoList& p_rDeviceInfoList);
    bool OpenInterface(CSocketCANDeviceInfo& p_rInfo);
    bool CloseInterface();
    bool OpenPort(CStdString p_portName);
    bool ClosePort();
    bool GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout);
    bool SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout);
    bool CheckStatusAndReset();
    bool ResetPort();

    bool ReceiveFilteredCanFrame(DWORD p_ulCobIdFilter, int p_lRtrFilter, DWORD p_ulTimeout, DWORD* p_pulCobId, int* p_plRtr, unsigned char* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength);
    bool ReceiveCanFrame(DWORD p_ulTimeout, DWORD* p_pulCobId, int* p_plRtr, unsigned char* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength);
    bool TransmitCanFrame(DWORD p_ulCobId, int p_lRtr, unsigned char p_ubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength);

    CStdString GetInterfaceName();

private:
    bool CreateSocketCanPorts(std::list<CSocketCANPort*>& p_rCanPortList);
    bool EnumAvailableCanPorts(std::list<CStdString>& p_rCanPortList);
    bool ClearCanPorts(std::list<CSocketCANPort*>& p_rCanPortList);
    bool GetCanPortState(CStdString p_CanPort, CStdString& p_rState, int& p_rBitrate);
    bool GetCanPortState(CStdString p_CanPort, CStdString& p_rState);
    bool SetCanPortState(CStdString p_PortName, CStdString p_State);
    bool SetCanPortBitrate(CStdString p_PortName);
    bool SetCanFilter(DWORD p_ulCobIdFilter, int p_lRtrFilter);
    bool IsCanPortAvailable(CStdString p_CanPort);
    bool ReadCanPortOperState(CStdString p_CanDir, CStdString& p_rOperstate);
    bool ConvertToCanDeviceInfoList(tCanPortList& p_rCanPortList, tSocketCANDeviceInfoList& p_rCANDeviceInfoList);
    bool SearchDeviceInfoByAddress(tSocketCANDeviceInfoList& p_rCANDeviceInfoList, CStdString p_Address, CSocketCANDeviceInfo*& p_rpCanDeviceInfo);
    bool UserHasSufficientAccessRights();
    bool IsCanPortUp(CStdString p_PortName);
    bool GetDeviceCountByDriverName(tSocketCANDeviceInfoList& p_rCANDeviceInfoList, CStdString p_DriverName, DWORD& p_rDeviceCount);

private:
    CStdString m_interfaceName;
    CStdString m_ActivePortName;
    int m_lChannelHandle;
    DWORD m_ulBaudrate;
    DWORD m_ulTimeout;
    DWORD m_ulCobIdFilter;
    int m_lRtrFilter;
};
