#pragma once

#include <StdString.h>
#include "Message/LogMessageType.h"

class CLogMessage;
class CLogFilter;
class CLogOutput;
class CLogMessageLevel;
class CMmcHiResTimer;

class CLog
{
public:
    CLog();
    virtual ~CLog();

    bool Log(CStdString p_Msg);
    bool Log(CStdString p_Msg, double p_Timestamp);
    bool Log(ELogMessageType p_eType, CStdString p_Msg);
    bool Log(CLogMessageLevel* p_pLevel, CStdString p_Msg);
    bool Log(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg);

    bool LogTimingStart(CStdString p_Msg);
    bool LogTimingStop(CStdString p_Msg);

    bool Write(CStdString p_Msg);
    bool Write(CStdString p_Msg, double p_Timestamp);
    bool Write(ELogMessageType p_eType, CStdString p_Msg);
    bool Write(CLogMessageLevel* p_pLevel, CStdString p_Msg);
    bool Write(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg);

    bool IsActive(ELogMessageType p_eMessageType);
    bool IsActive(CLogMessageLevel& p_rMsgLevel);
    bool IsActive(ELogMessageType p_eMessageType, CLogMessageLevel& p_rMsgLevel);

    void InitFilter(CLogFilter* p_pLogFilter);
    void InitMessage(CLogMessage* p_pLogMessage);
    void InitOutput(CLogOutput* p_pLogOutput);

private:
    void DeleteFilter();
    void DeleteMessage();
    void DeleteOutput();

    bool WriteMessage(CStdString p_Msg);
    bool WriteMessage(ELogMessageType p_eType, CStdString p_Msg);
    bool WriteMessage(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg);
    bool WriteMessage(CLogMessageLevel* p_pLevel, CStdString p_Msg);
    bool WriteMessage(CStdString p_Msg, double p_Timestamp);

    void CreateTimer();
    void DeleteTimer();

private:
    CLogFilter* m_pLogFilter;
    CLogMessage* m_pLogMessage;
    CLogOutput* m_pLogOutput;
    CMmcHiResTimer* m_pTimer;
};
