#ifndef _MMCUSERSETIINGS
#define _MMCUSERSETIINGS

#include <Storage/MmcUserSettingsDef.h>
#include <list>
#include <StdString.h>
#include <Xml/rapidxml.hpp>

class CMmcUserSettingsBase : public CMmcUserSettingsDef
{
public:
    CMmcUserSettingsBase();
    CMmcUserSettingsBase(CStdString p_Key, bool p_oCreateKey);
    CMmcUserSettingsBase(CMmcUserSettingsBase *p_pParent, CStdString p_Key, bool p_oCreateKey);
    virtual ~CMmcUserSettingsBase();

public:
    unsigned long   ReadULONG(CStdString p_Key);
    CStdString      ReadString(CStdString p_Key);
    bool            Write(CStdString p_Key, CStdString p_Value);
    bool            Write(CStdString p_Key, unsigned long p_ulValue);
    bool            Delete(CStdString p_Key, bool deep = true);
    std::list<CStdString>   GetChildrenNames();

private:
    bool Init();
    bool Reset();
    bool CreateDocument(CStdString p_strFileName);
    bool LoadDocument(CStdString p_strFileName);
    bool LoadFromString(CStdStringA* p_pSource);
    bool FileExists(CStdString p_FileName);
    bool Open();
    bool Open(CStdString p_Key, bool p_oCreateKey);
    bool CreateKey(CStdString p_Key);
    bool CreateKeyRecursive(CStdString p_Key);
    bool OpenKey(CStdString p_Key);
    bool Save();
    void GoToRoot();
    CStdString ToValidXmlNodeName(CStdString p_NodeName);
    bool GetSubkeyList(CStdString p_Key, std::list<CStdString>& p_rSubkeyList, bool p_oValidate = true);
    bool CreateSubkeysIterative(std::list<CStdString> p_SubkeyList);
    bool IsParsed();
    bool ChildNodeExists(CStdString p_Key, bool& p_roExists);
    bool GetChildNode(CStdString p_NodeName, rapidxml::xml_node<char>*& p_rpChildNode);
    CStdString GetKeyFullPath();
    bool EnsureMmcDirectoryExists();
    CStdString GetRegistryFileFullPath();
    bool ValidateRootNode(rapidxml::xml_node<char>* p_pRootNode);

    rapidxml::xml_node<char>* GetRootNode();
    rapidxml::xml_node<char>* SelectSingleNode(CStdString p_XPathNode, int p_lNodeIndex = 0);

private:
    rapidxml::xml_document<char> m_Doc;    
    rapidxml::xml_node<char>* m_pCurrentNode;
    bool m_oCreateKey;    
    bool m_oParsed;
    CStdString m_MmcDir;
    CStdString m_MmcRegistryFileName;
    CStdString m_RootNodeName;
};

#endif //_MMCUSERSETIINGS
