#pragma once

#include <MmcTypeDefinition.h>
#include <Thread/Definitions/ThreadBaseDefinitions.h>
#include <Thread/MmcCriticalSection.h>

class CIThreadBaseEvents;
class CThreadBaseEventsSource;

class CMmcThreadBase
{
public:
    CMmcThreadBase();
    virtual ~CMmcThreadBase();

    int RegisterEvents(CIThreadBaseEvents* p_pIEvents);
    int UnregisterEvents(CIThreadBaseEvents* p_pIEvents);

public:
    virtual int Start() = 0;
    virtual int Stop() = 0;

    int IsActive();
    int IsStopping();
    int IsStarted();
    int IsStopped();

    virtual int SetHighPriority() = 0;

    EThreadBaseState GetThreadState();

protected:
    int CanStart();
    void NotifyPriorityChange(int p_lNewPriority);

    virtual void SetThreadState(EThreadBaseState p_eState);

    virtual int IsStillActive() = 0;

private:
    void CreateEventsSource();
    void DeleteEventsSource();

protected:
    CMmcCriticalSection m_Sync;

private:
    EThreadBaseState m_ThreadState;
    CThreadBaseEventsSource* m_pEventsSource;
};
