#include "stdafx.h"
#include <stdlib.h>
#include <MmcTypeDefinition.h>
#include <Text/MmcTextConvert.h>
#include <Drv/BaseClasses/UsbDeviceInfo.h>

CUsbDeviceInfo::CUsbDeviceInfo(void)
    : m_strClassType(_T("-"))
    , m_PortChain(_T(""))
    , m_DrvPortInfo(_T(""))
    , m_DevicePath(_T(""))
    , m_usVendorId(0)
    , m_usProductId(0)
    , m_SerialNumber(_T(""))
    , m_PortName(_T(""))
    , m_ulPluggedIn(0)
    , m_ulOpenedByProcess(0)
    , m_ulOpenedByInstance(0)
    , m_OpenedBy(_T(""))
{
    InitClassInfo();
}

CUsbDeviceInfo::~CUsbDeviceInfo(void)
{
}

CUsbDeviceInfo* CUsbDeviceInfo::Clone()
{
    CUsbDeviceInfo* pDeviceInfo = 0;

    pDeviceInfo = new CUsbDeviceInfo();
    *pDeviceInfo = *this;

    return pDeviceInfo;
}

BOOL CUsbDeviceInfo::IsKindOf(CStdString p_classType)
{
    BOOL oResult(FALSE);

    //Init
    p_classType = _T("-") + p_classType + _T("-");

    //Check class type
    if(m_strClassType.Find(p_classType) != -1)
    {
        oResult = TRUE;
    }

    return oResult;
}

CStdString CUsbDeviceInfo::GetClassType()
{
    int lIndex = 1;
    CStdString classType(_T(""));

    if(!m_strClassType.IsEmpty())
    {
        lIndex = m_strClassType.Find(_T("-"), lIndex);
        if(lIndex != -1)
        {
            classType = m_strClassType.substr(0, lIndex);
            classType = classType.TrimLeft(_T("-"));
            classType = classType.TrimRight(_T("-"));
        }
    }

    return classType;
}

CUsbDeviceInfo& CUsbDeviceInfo::operator=(CUsbDeviceInfo& p_rOther)
{
    if(this != &p_rOther)
    {
        m_PortChain = p_rOther.m_PortChain;
        m_DrvPortInfo = p_rOther.m_DrvPortInfo;
        m_DevicePath = p_rOther.m_DevicePath;
        m_usVendorId = p_rOther.m_usVendorId;
        m_usProductId = p_rOther.m_usProductId;
        m_SerialNumber = p_rOther.m_SerialNumber;

        m_PortName = p_rOther.m_PortName;
        m_ulPluggedIn = p_rOther.m_ulPluggedIn;
        m_ulOpenedByProcess = p_rOther.m_ulOpenedByProcess;
        m_ulOpenedByInstance = p_rOther.m_ulOpenedByInstance;
        m_OpenedBy = p_rOther.m_OpenedBy;
    }

    return *this;
}

BOOL CUsbDeviceInfo::GetLogInfos(CStdString& p_rInfos, CUsbDeviceInfo::ELogInfoType p_eType)
{
    BOOL oResult(FALSE);

    switch(p_eType)
    {
        case LIT_IDENTIFIER: p_rInfos.Format(_T("%s PortChain *%s* SerNb %s; "), m_PortName, m_PortChain, m_SerialNumber); oResult = TRUE; break;
        case LIT_STATE: p_rInfos.Format(_T("%s PortChain *%s* SerNb %s PluggedIn %i OpenedByProcess %i OpenedByInstance %i OpenedBy %s; "), m_PortName, m_PortChain, m_SerialNumber, IsPluggedIn(), GetOpenedByProcess(), GetOpenedByInstance(), GetOpenedBy()); oResult = TRUE; break;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::Init(CUsbDeviceInfo& p_rDeviceInfo)
{
    BOOL oResult(FALSE);

    if(p_rDeviceInfo.IsKindOf(_T("CUsbDeviceInfo")))
    {
        CUsbDeviceInfo* pDeviceInfo = (CUsbDeviceInfo*)&p_rDeviceInfo;
        m_PortChain = pDeviceInfo->m_PortChain;
        m_DrvPortInfo = pDeviceInfo->m_DrvPortInfo;
        m_DevicePath = pDeviceInfo->m_DevicePath;
        m_usVendorId = pDeviceInfo->m_usVendorId;
        m_usProductId = pDeviceInfo->m_usProductId;
        m_SerialNumber = pDeviceInfo->m_SerialNumber;
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::Init(CStdString p_PortChain, CStdString p_DevicePath, WORD p_usVendorId, WORD p_usProductId, CStdString p_SerialNumber)
{
    m_PortChain = p_PortChain;
    m_DevicePath = p_DevicePath;
    m_usVendorId = p_usVendorId;
    m_usProductId = p_usProductId;
    m_SerialNumber = p_SerialNumber;

    return TRUE;
}

BOOL CUsbDeviceInfo::Update(CUsbDeviceInfo& p_rDeviceInfo)
{
    BOOL oResult(FALSE);

    if(p_rDeviceInfo.IsKindOf(_T("CUsbDeviceInfo")))
    {
        CUsbDeviceInfo* pDeviceInfo = (CUsbDeviceInfo*)&p_rDeviceInfo;
        if(!pDeviceInfo->m_SerialNumber.size() == 0)
        {
            m_SerialNumber = pDeviceInfo->m_SerialNumber;
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsEqual(CUsbDeviceInfo& p_rDeviceInfo, ECompareMode p_eCompareMode)
{
    BOOL oResult(FALSE);

    switch(p_eCompareMode)
    {
        case CM_PORT_CHAIN:
            {
                oResult = IsPortChainEqual(p_rDeviceInfo.GetPortChain());
            }
            break;

        case CM_DRV_PORT_INFOS:
            {
                oResult = IsDrvPortInfoEqual(p_rDeviceInfo.GetDrvPortInfo());
            }
            break;

        case CM_PORT_CHAIN_AND_DRV_PORT_INFOS:
            {
                oResult = IsPortChainEqual(p_rDeviceInfo.GetPortChain()) && IsDrvPortInfoEqual(p_rDeviceInfo.GetDrvPortInfo());
            }
            break;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsPortChainEqual(CStdString p_PortChain)
{
    BOOL oResult = FALSE;

    if(m_PortChain == p_PortChain)
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsPortChainEmpty()
{
    return (m_PortChain == _T(""));
}

BOOL CUsbDeviceInfo::IsDrvPortInfoEqual(CStdString p_DrvPortInfo)
{
    BOOL oResult = FALSE;

    if(m_DrvPortInfo == p_DrvPortInfo)
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsDevicePathEqual(CStdString p_DevicePath)
{
    BOOL oResult = FALSE;

    if(m_DevicePath == p_DevicePath)
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsSerialNumberEqual(CStdString p_SerialNumber)
{
    BOOL oResult = FALSE;

    if(m_SerialNumber == p_SerialNumber)
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsSerialNumberEmpty()
{
    return (m_SerialNumber == _T(""));
}

BOOL CUsbDeviceInfo::IsPortNameEqual(CStdString p_PortName)
{
    BOOL oResult = FALSE;

    if(m_PortName == p_PortName)
    {
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::IsPluggedIn()
{
    return (m_ulPluggedIn > 0);
}

BOOL CUsbDeviceInfo::IsOpened()
{
    return (m_ulOpenedByProcess > 0) && (m_ulOpenedByInstance > 0);
}

BOOL CUsbDeviceInfo::IsOpenedBy(DWORD p_ulProcessId, DWORD p_ulInstanceValue)
{
    return ((m_ulOpenedByProcess == p_ulProcessId) && (m_ulOpenedByInstance == p_ulInstanceValue));
}

BOOL CUsbDeviceInfo::SetPortChain(CStdString p_PortChain)
{
    BOOL oResult = TRUE;

    m_PortChain = p_PortChain;

    return oResult;
}

BOOL CUsbDeviceInfo::SetDrvPortInfo(CStdString p_DrvPortInfo)
{
    BOOL oResult = TRUE;

    m_DrvPortInfo = p_DrvPortInfo;

    return oResult;
}

BOOL CUsbDeviceInfo::SetDevicePath(CStdString p_DevicePath)
{
    BOOL oResult = TRUE;

    m_DevicePath = p_DevicePath;

    return oResult;
}

BOOL CUsbDeviceInfo::SetVendorId(CStdString p_VendorId)
{
    BOOL oResult = TRUE;
    unsigned int ullValue;

    CStdString textValue = p_VendorId.Right((int)p_VendorId.length() - (int)_tcslen(_T("0x")));
    ullValue = _tcstol(textValue, 0, 16);
    m_usVendorId = (WORD)ullValue;

    return oResult;
}

BOOL CUsbDeviceInfo::SetProductId(CStdString p_ProductId)
{
    BOOL oResult = TRUE;
    unsigned int ullValue;

    CStdString textValue = p_ProductId.Right((int)p_ProductId.length() - (int)_tcslen(_T("0x")));
    ullValue = _tcstol(textValue, 0, 16);
    m_usProductId = (WORD)ullValue;

    return oResult;
}

BOOL CUsbDeviceInfo::SetSerialNumber(CStdString p_SerialNumber)
{
    BOOL oResult = TRUE;

    m_SerialNumber = p_SerialNumber;

    return oResult;
}

BOOL CUsbDeviceInfo::SetPortName(CStdString p_PortName)
{
    BOOL oResult = TRUE;

    m_PortName = p_PortName;

    return oResult;
}

BOOL CUsbDeviceInfo::InitPortName(CUsbDeviceInfo* p_pDeviceInfo)
{
    BOOL oResult(FALSE);

    if(p_pDeviceInfo)
    {
        m_PortName = p_pDeviceInfo->GetPortName();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::InitState(CUsbDeviceInfo* p_pDeviceInfo)
{
    BOOL oResult(FALSE);

    if(p_pDeviceInfo)
    {
        m_ulOpenedByProcess = p_pDeviceInfo->GetOpenedByProcess();
        m_ulOpenedByInstance = p_pDeviceInfo->GetOpenedByInstance();
        m_ulPluggedIn = p_pDeviceInfo->GetPluggedIn();
        m_OpenedBy = p_pDeviceInfo->GetOpenedBy();
        oResult = TRUE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::SetOpenedByProcess(DWORD p_ulOpenedByProcess)
{
    m_ulOpenedByProcess = p_ulOpenedByProcess;
    return TRUE;
}

BOOL CUsbDeviceInfo::SetOpenedByInstance(DWORD p_ulOpenedByInstance)
{
    m_ulOpenedByInstance = p_ulOpenedByInstance;
    return TRUE;
}

BOOL CUsbDeviceInfo::SetOpenedBy(CStdString p_OpenedBy)
{
    m_OpenedBy = p_OpenedBy;
    return TRUE;
}

BOOL CUsbDeviceInfo::SetPluggedIn(DWORD p_ulPluggedIn)
{
    m_ulPluggedIn = p_ulPluggedIn;
    return TRUE;
}

DWORD CUsbDeviceInfo::GetOpenedByProcess()
{
    return m_ulOpenedByProcess;
}

DWORD CUsbDeviceInfo::GetOpenedByInstance()
{
    return m_ulOpenedByInstance;
}

CStdString CUsbDeviceInfo::GetOpenedBy()
{
    return m_OpenedBy;
}

DWORD CUsbDeviceInfo::GetPluggedIn()
{
    return m_ulPluggedIn;
}

BOOL CUsbDeviceInfo::GetPortName(CStdString& p_rPortName)
{
    BOOL oResult = TRUE;

    p_rPortName = m_PortName;

    return oResult;
}

CStdString CUsbDeviceInfo::GetPortName()
{
    return m_PortName;
}

BOOL CUsbDeviceInfo::GetPortNameIndex(int& p_rIndex)
{
    BOOL oResult(FALSE);
    CStdString portName(m_PortName);
    CStdString index = portName.TrimLeft(_T("ABCDEFGHIJKLMNOPQRSTUVWXYZ"));

    if(!index.IsEmpty())
    {
        p_rIndex = ConvertStringToInt(index);

        oResult = TRUE;
    }

    return oResult;
}

CStdString CUsbDeviceInfo::GetPortChain()
{
    return m_PortChain;
}

CStdString CUsbDeviceInfo::GetDrvPortInfo()
{
    return m_DrvPortInfo;
}

CStdString CUsbDeviceInfo::GetDevicePath()
{
    return m_DevicePath;
}

CStdString CUsbDeviceInfo::GetProductId()
{
    CStdString productId(_T(""));

    productId.Format(_T("0x%.4X"), m_usProductId);

    return productId;
}

CStdString CUsbDeviceInfo::GetVendorId()
{
    CStdString vendorId(_T(""));

    vendorId.Format(_T("0x%.4X"), m_usVendorId);

    return vendorId;
}

CStdString CUsbDeviceInfo::GetSerialNumber()
{
    return m_SerialNumber;
}

BOOL CUsbDeviceInfo::WriteTo(CMmcUserSettings* p_pUserSettings)
{
    BOOL oResult = FALSE;
    CStdString portChain = _T("");
    CStdString devicePath = _T("");
    CStdString vendorId = _T("");
    CStdString productId = _T("");
    CStdString serialNumber = _T("");
    CStdString portName = _T("");
    DWORD pluggedIn = 0;
    DWORD openedByProcess = 0;
    DWORD openedByInstance = 0;
    CStdString openedBy;

    if(p_pUserSettings)
    {
        //Init
        oResult = TRUE;
        portChain = GetPortChain();
        devicePath = GetDevicePath();
        vendorId = GetVendorId();
        productId = GetProductId();
        serialNumber = GetSerialNumber();
        portName = GetPortName();
        pluggedIn = GetPluggedIn();
        openedByProcess = GetOpenedByProcess();
        openedByInstance = GetOpenedByInstance();
        openedBy = GetOpenedBy();

        //Write
        if(oResult && !p_pUserSettings->Write("", portName)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("PortChain", portChain)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("DevicePath", devicePath)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("VendorId", vendorId)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("ProductId", productId)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("SerialNumber", serialNumber)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("PluggedIn", pluggedIn)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("OpenedByProcess", openedByProcess)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("OpenedByInstance", openedByInstance)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("OpenedBy", openedBy)) oResult = FALSE;
    }

    return oResult;
}

BOOL CUsbDeviceInfo::ReadFrom(CMmcUserSettings* p_pUserSettings)
{
    BOOL oResult(FALSE);

    if(p_pUserSettings)
    {
        //Read
        CStdString portName = p_pUserSettings->ReadString("");
        CStdString portChain = p_pUserSettings->ReadString("PortChain");
        CStdString devicePath = p_pUserSettings->ReadString("DevicePath");
        CStdString vendorId = p_pUserSettings->ReadString("VendorId");
        CStdString productId = p_pUserSettings->ReadString("ProductId");
        CStdString serialNumber = p_pUserSettings->ReadString("SerialNumber");
        CStdString openedBy = p_pUserSettings->ReadString("OpenedBy");
        DWORD pluggedIn = p_pUserSettings->ReadULONG("PluggedIn");
        DWORD openedByProcess = p_pUserSettings->ReadULONG("OpenedByProcess");
        DWORD openedByInstance = p_pUserSettings->ReadULONG("OpenedByInstance");

        //Return Values
        SetPortChain(portChain);
        SetDevicePath(devicePath);
        SetVendorId(vendorId);
        SetProductId(productId);
        SetPortName(portName);
        SetSerialNumber(serialNumber);
        SetPluggedIn(pluggedIn);
        SetOpenedByProcess(openedByProcess);
        SetOpenedByInstance(openedByInstance);
        SetOpenedBy(openedBy);
        oResult = TRUE;
    }

    return oResult;
}
