#include "stdafx.h"
#include <Drv/Ftd2xx/Ftd2xxDeviceInfo.h>
#include <MmcTypeDefinition.h>

CFtd2xxDeviceInfo::CFtd2xxDeviceInfo(void):
    m_ulLocationId(0)
    , m_ulDrvFlags(0)
{
    InitClassInfo();
}

CFtd2xxDeviceInfo::~CFtd2xxDeviceInfo(void)
{
}

CUsbDeviceInfo* CFtd2xxDeviceInfo::Clone()
{
    CFtd2xxDeviceInfo* pDeviceInfo = 0;

    pDeviceInfo = new CFtd2xxDeviceInfo();
    *pDeviceInfo = *this;

    return pDeviceInfo;
}

CFtd2xxDeviceInfo& CFtd2xxDeviceInfo::operator=(CFtd2xxDeviceInfo& other)
{
    if(this != &other)
    {
        *((CUsbDeviceInfo*)(this)) = *((CUsbDeviceInfo*)&other);
        m_ulLocationId = other.m_ulLocationId;
        m_ulDrvFlags = other.m_ulDrvFlags;

    }

    return *this;
}

BOOL CFtd2xxDeviceInfo::GetLogInfos(CStdString& p_rInfos, CUsbDeviceInfo::ELogInfoType p_eType)
{
    BOOL oResult(FALSE);

    switch(p_eType)
    {
        case LIT_IDENTIFIER: p_rInfos.Format(_T("%s PortChain *%s* LocId %.8X SerNb %s Flags %.8X; "), m_PortName, m_PortChain, m_ulLocationId, m_SerialNumber, m_ulDrvFlags); oResult = TRUE; break;
        case LIT_STATE: p_rInfos.Format(_T("%s PortChain *%s* LocId %.8X SerNb %s PluggedIn %i OpenedByProcess %i OpenedByInstance %i OpenedBy %s; "), m_PortName, m_PortChain, m_ulLocationId, m_SerialNumber, IsPluggedIn(), GetOpenedByProcess(), GetOpenedByInstance(), GetOpenedBy()); oResult = TRUE; break;
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::Init(CStdString p_PortChain, CStdString p_DevicePath, WORD p_usVendorId, WORD p_usProductId, CStdString p_SerialNumber, DWORD p_ulLocId, DWORD p_ulFlags)
{
    BOOL oResult(FALSE);

    if(CUsbDeviceInfo::Init(p_PortChain, p_DevicePath, p_usVendorId, p_usProductId, p_SerialNumber))
    {
        SetLocationId(p_ulLocId);
        SetDrvFlags(p_ulFlags);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::InitDrvPortInfo(DWORD p_ulLocationId)
{
    BOOL oResult(FALSE);
    CStdString drvPortInfo(_T(""));

    if(p_ulLocationId != 0)
    {
        drvPortInfo.Format(_T("Ftd2xx-LocId-%i"), p_ulLocationId);
        oResult = SetDrvPortInfo(drvPortInfo);
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::Init(CUsbDeviceInfo& p_rDeviceInfo)
{
    BOOL oResult(FALSE);

    oResult = CUsbDeviceInfo::Init(p_rDeviceInfo);
    if(p_rDeviceInfo.IsKindOf(_T("CFtd2xxDeviceInfo")))
    {
        CFtd2xxDeviceInfo* pDeviceInfo = (CFtd2xxDeviceInfo*)&p_rDeviceInfo;
        SetLocationId(pDeviceInfo->m_ulLocationId);
        SetDrvFlags(pDeviceInfo->m_ulDrvFlags);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::Update(CUsbDeviceInfo& p_rDeviceInfo)
{
    BOOL oResult(FALSE);

    if(p_rDeviceInfo.IsKindOf(_T("CFtd2xxDeviceInfo")))
    {
        CUsbDeviceInfo::Update(p_rDeviceInfo);

        CFtd2xxDeviceInfo* pDeviceInfo = (CFtd2xxDeviceInfo*)&p_rDeviceInfo;
        SetLocationId(pDeviceInfo->m_ulLocationId);
        oResult = TRUE;
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::SetLocationId(DWORD p_ulLocationId)
{
    InitDrvPortInfo(p_ulLocationId);
    m_ulLocationId = p_ulLocationId;

    return TRUE;
}

BOOL CFtd2xxDeviceInfo::SetDrvFlags(DWORD p_ulFlags)
{
    m_ulDrvFlags = p_ulFlags;
    return TRUE;
}

DWORD CFtd2xxDeviceInfo::GetLocationId()
{
    return m_ulLocationId;
}

DWORD CFtd2xxDeviceInfo::GetDrvFlags()
{
    return m_ulDrvFlags;
}

BOOL CFtd2xxDeviceInfo::WriteTo(CMmcUserSettings* p_pUserSettings)
{
    BOOL oResult = FALSE;
    CStdString serialNumber = _T("");
    DWORD locationId = 0;

    if(p_pUserSettings)
    {
        //Init
        oResult = TRUE;
        locationId = GetLocationId();

        //Write
        if(oResult && !CUsbDeviceInfo::WriteTo(p_pUserSettings)) oResult = FALSE;
        if(oResult && !p_pUserSettings->Write("LocationId", locationId)) oResult = FALSE;
    }

    return oResult;
}

BOOL CFtd2xxDeviceInfo::ReadFrom(CMmcUserSettings* p_pUserSettings)
{
    BOOL oResult(FALSE);

    if(p_pUserSettings)
    {
        //Read
        if(CUsbDeviceInfo::ReadFrom(p_pUserSettings))
        {
            //Read
            DWORD locationId = p_pUserSettings->ReadULONG("LocationId");

            //Return Values
            SetLocationId(locationId);
            oResult = TRUE;
        }
    }

    return oResult;
}
