#include "stdafx.h"

#include <Drv/Helper/UsbTree/UsbTree.h>
#include <Drv/Helper/UsbTree/UsbDevices.h>

#include <Drv/Helper/UsbTree/UsbDeviceBase.h>

using namespace std;

CUsbDeviceBase::CUsbDeviceBase()
    : m_ulIndex(0)
    , m_DevicePath(_T(""))
    , m_PortChain(_T(""))
    , m_pParent(0)
{
}

CUsbDeviceBase::~CUsbDeviceBase()
{
    DeleteChildDevices();
}

CUsbDeviceBase* CUsbDeviceBase::Clone()
{
    CUsbDeviceBase* pUsbDeviceBase = new CUsbDeviceBase();
    *pUsbDeviceBase = *this;

    return pUsbDeviceBase;
}

CUsbDeviceBase& CUsbDeviceBase::operator=(CUsbDeviceBase& p_rOther)
{
    if(this != &p_rOther)
    {
        m_ulIndex = p_rOther.m_ulIndex;
        m_DevicePath = p_rOther.m_DevicePath;
        m_PortChain = p_rOther.m_PortChain;

        DeleteChildDevices();

        for(list<CUsbDeviceBase*>::iterator it = p_rOther.m_ChildDevices.begin(); it !=  p_rOther.m_ChildDevices.end(); it++)
        {
            CUsbDeviceBase* pUsbDeviceBase = (*it);
            CUsbDeviceBase* pClonedDeviceBase = pUsbDeviceBase->Clone();
            AddChild(pClonedDeviceBase);
        }
    }

    return *this;
}

void CUsbDeviceBase::InitParent(CUsbDeviceBase* p_pParent)
{
    m_pParent = p_pParent;
}

void CUsbDeviceBase::InitIndex(DWORD p_ulIndex)
{
    m_ulIndex = p_ulIndex;
}

void CUsbDeviceBase::InitDevicePath(CStdString p_devicePath)
{
    m_DevicePath = p_devicePath;
}

void CUsbDeviceBase::AddChild(CUsbDeviceBase* p_pDeviceBase)
{
    if(p_pDeviceBase != 0)
    {
        p_pDeviceBase->InitParent(this);
        m_ChildDevices.push_back(p_pDeviceBase);
    }
}

void CUsbDeviceBase::DeleteChildDevices()
{
    list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin();

    while(it != m_ChildDevices.end())
    {
        CUsbDeviceBase* pChildDevice = (*it);

        delete pChildDevice;

        m_ChildDevices.erase(it);

        it = m_ChildDevices.begin();
    }

    m_ChildDevices.clear();
}

CUsbDeviceBase* CUsbDeviceBase::GetParent()
{
    return m_pParent;
}

list<CUsbDeviceBase*>* CUsbDeviceBase::GetChildren()
{
    return &m_ChildDevices;
}

CStdString CUsbDeviceBase::GetParentDevicePath()
{
    CStdString devicePath(_T(""));

    if(m_pParent != 0)
    {
        devicePath = m_pParent->GetDevicePath();
    }

    return devicePath;
}

CStdString CUsbDeviceBase::GetChildDevicePath(int p_lIndex)
{
    CStdString childDevicePath(_T(""));
    int lIndex(0);

    for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
    {
        if(lIndex == p_lIndex)
        {
            CUsbDeviceBase* pDevice = *(it);
            childDevicePath = pDevice->GetDevicePath();
            break;
        }

        lIndex++;
    }

    return childDevicePath;
}

DWORD CUsbDeviceBase::GetIndex()
{
    return m_ulIndex;
}

CStdString CUsbDeviceBase::GetDevicePath()
{
    return m_DevicePath;
}

CStdString CUsbDeviceBase::GetPortChain()
{
    return m_PortChain;
}

CStdString CUsbDeviceBase::BuildPortChain()
{
    CStdString portChain(_T(""));
    CStdString digit;

    if(m_pParent != 0)
    {
        portChain = m_pParent->GetPortChain() + _T("-");
    }

    digit.Format(_T("%i"), GetIndex() + 1);
    portChain +=  digit;

    return portChain;
}

void CUsbDeviceBase::InitPortChains()
{
    m_PortChain = BuildPortChain();

    for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
    {
        CUsbDeviceBase* pDeviceBase = *(it);
        pDeviceBase->InitPortChains();
    }
}

BOOL CUsbDeviceBase::GetPortChain(CStdString p_devicePath, CStdString& p_rPortChain)
{
    BOOL oResult(FALSE);

    if(GetDevicePath() == p_devicePath)
    {
        p_rPortChain = GetPortChain();
        oResult = TRUE;
    }
    else
    {
        for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
        {
            CUsbDeviceBase* pDeviceBase = *(it);
            if(pDeviceBase->GetPortChain(p_devicePath, p_rPortChain))
            {
                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

BOOL CUsbDeviceBase::GetPortChain(WORD p_usVendorId, WORD p_usProductId, CStdString p_serialNumber, CStdString& p_rPortChain)
{
    BOOL oResult(FALSE);
    CStdString devicePath(_T(""));
    WORD usVendorId(0);
    WORD usProductId(0);
    CStdString serialNumber(_T(""));

    if(GetInfos(devicePath, usVendorId, usProductId, serialNumber))
    {
        if((usVendorId == p_usVendorId) && (usProductId == p_usProductId) && (serialNumber == p_serialNumber))
        {
            p_rPortChain = GetPortChain();
            oResult = TRUE;
        }
    }

    if(!oResult)
    {
        for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
        {
            CUsbDeviceBase* pDeviceBase = *(it);
            if(pDeviceBase->GetPortChain(p_usVendorId, p_usProductId, p_serialNumber, p_rPortChain))
            {
                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

BOOL CUsbDeviceBase::GetPortChains(WORD p_usVendorId, WORD p_usProductId, CStdStringArray& p_rPortChains)
{
    BOOL oResult(TRUE);
    CStdString devicePath(_T(""));
    WORD usVendorId(0);
    WORD usProductId(0);
    CStdString serialNumber(_T(""));

    if(GetInfos(devicePath, usVendorId, usProductId, serialNumber))
    {
        if((usVendorId == p_usVendorId) && (usProductId == p_usProductId))
        {
            p_rPortChains.push_back(GetPortChain());
        }
    }

    for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
    {
        CUsbDeviceBase* pDeviceBase = *(it);
        if(!pDeviceBase->GetPortChains(p_usVendorId, p_usProductId, p_rPortChains))
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CUsbDeviceBase::GetInfos(CStdString p_portChain, CStdString& p_rDevicePath, WORD& p_rusVendorId, WORD& p_rusProductId, CStdString& p_rSerialNumber)
{
    BOOL oResult(FALSE);

    if(GetPortChain() == p_portChain)
    {
        oResult = GetInfos(p_rDevicePath, p_rusVendorId, p_rusProductId, p_rSerialNumber);
    }

    if(!oResult)
    {
        for(list<CUsbDeviceBase*>::iterator it = m_ChildDevices.begin(); it != m_ChildDevices.end(); it++)
        {
            CUsbDeviceBase* pDeviceBase = *(it);
            if(pDeviceBase->GetInfos(p_portChain, p_rDevicePath, p_rusVendorId, p_rusProductId, p_rSerialNumber))
            {
                oResult = TRUE;
                break;
            }
        }
    }

    return oResult;
}

BOOL CUsbDeviceBase::GetInfos(CStdString& p_rDevicePath, WORD& p_rusVendorId, WORD& p_rusProductId, CStdString& p_rSerialNumber)
{
    return FALSE;
}
