#include "stdafx.h"

#include <Drv/Helper/UsbTree/UsbDevice.h>
#include <Drv/Helper/UsbTree/UsbTreeFactory.h>

#include <Drv/Helper/UsbTree/UsbDevices.h>

using namespace std;

CUsbDevices::CUsbDevices()
{
}

CUsbDevices::~CUsbDevices(void)
{
    DeleteUsbDevices();
}

BOOL CUsbDevices::Load()
{
    BOOL oResult(FALSE);

    oResult = LoadUsbDevices();

    return oResult;
}

BOOL CUsbDevices::Reset()
{
    DeleteUsbDevices();

    return TRUE;
}

BOOL CUsbDevices::LoadUsbDevices()
{
    BOOL oResult(FALSE);
    CUsbTreeFactory factory;

    DeleteUsbDevices();

    if(factory.LoadUsbDevices(m_UsbDevices))
    {
        oResult = TRUE;
    }

    return oResult;
}

list<CUsbDevice*> CUsbDevices::GetDeviceList()
{
    return m_UsbDevices;
}

void CUsbDevices::DeleteUsbDevices()
{
    for(list<CUsbDevice*>::iterator it = m_UsbDevices.begin(); it != m_UsbDevices.end(); it++)
    {
        CUsbDevice* pUsbDevice = *(it);
        delete pUsbDevice;
    }

    m_UsbDevices.clear();
}

BOOL CUsbDevices::GetPortChains(WORD p_usVendorId, WORD p_usProductId, CStdStringArray& p_rPortChains)
{
    BOOL oResult(TRUE);

    p_rPortChains.clear();
    for(list<CUsbDevice*>::iterator it = m_UsbDevices.begin(); it != m_UsbDevices.end(); it++)
    {
        CUsbDevice* pUsbDevice = *(it);
        if(pUsbDevice->GetVendorId() == p_usVendorId)
        {
            if(pUsbDevice->GetProductId() == p_usProductId)
            {
                p_rPortChains.push_back(pUsbDevice->GetPortChain());
            }
        }
    }

    return oResult;
}

BOOL CUsbDevices::GetInfos(CStdString p_portChain, CStdString& p_rDevicePath, WORD& p_rusVendorId, WORD& p_rusProductId, CStdString& p_rSerialNumber)
{
    BOOL oResult(TRUE);

    for(list<CUsbDevice*>::iterator it = m_UsbDevices.begin(); it != m_UsbDevices.end(); it++)
    {
        CUsbDevice* pUsbDevice = *(it);
        if(pUsbDevice->GetPortChain() == p_portChain)
        {
            oResult = pUsbDevice->GetInfos(p_rDevicePath, p_rusVendorId, p_rusProductId, p_rSerialNumber);
        }
    }

    return oResult;
}

BOOL CUsbDevices::GetUsbDeviceByDriverKeyName(CStdString p_driverKeyName, CUsbDevice*& p_rpUsbDevice)
{
    BOOL oResult(FALSE);

    for(list<CUsbDevice*>::iterator it = m_UsbDevices.begin(); it != m_UsbDevices.end(); it++)
    {
        CUsbDevice* pUsbDevice = *(it);
        if(pUsbDevice->GetDriverKeyName() == p_driverKeyName)
        {
            p_rpUsbDevice = pUsbDevice;
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}
