#include "stdafx.h"
#include <MmcTypeDefinition.h>
#include <Drv/SocketCAN/SocketCANDeviceInfo.h>

CSocketCANDeviceInfo::CSocketCANDeviceInfo(void)
    : m_classType(_T("-"))
    , m_InterfaceName(_T(""))
	, m_DeviceName(_T(""))
	, m_DriverName(_T(""))
	, m_DeviceAddress(_T(""))
	, m_lInterfaceIndex(0)
{
    InitClassInfo();
}

CSocketCANDeviceInfo::~CSocketCANDeviceInfo(void)
{
}

CSocketCANDeviceInfo& CSocketCANDeviceInfo::operator=(CSocketCANDeviceInfo& p_rOther)
{
    if(this != &p_rOther)
    {
        m_InterfaceName = p_rOther.m_InterfaceName;
        m_DeviceName = p_rOther.m_DeviceName;
        m_DriverName = p_rOther.m_DriverName;
        m_DeviceAddress = p_rOther.m_DeviceAddress;
        m_PortList = p_rOther.m_PortList;
        m_lInterfaceIndex = p_rOther.m_lInterfaceIndex;
    }

    return *this;
}

CSocketCANDeviceInfo* CSocketCANDeviceInfo::Clone()
{
    CSocketCANDeviceInfo* pDeviceInfo = 0;

    pDeviceInfo = new CSocketCANDeviceInfo();
    *pDeviceInfo = *this;

    return pDeviceInfo;
}

BOOL CSocketCANDeviceInfo::IsKindOf(CStdString p_classType)
{
    BOOL oResult(FALSE);

    //Init
    p_classType = _T("-") + p_classType + _T("-");

    //Check class type
    if(m_classType.Find(p_classType) != -1)
    {
        oResult = TRUE;
    }

    return oResult;
}

CStdString CSocketCANDeviceInfo::GetClassType()
{
    int lIndex = 1;
    CStdString classType(_T(""));

    if(!m_classType.IsEmpty())
    {
        lIndex = m_classType.Find(_T("-"), lIndex);
        if(lIndex != -1)
        {
            classType = m_classType.substr(0, lIndex);
            classType = classType.TrimLeft(_T("-"));
            classType = classType.TrimRight(_T("-"));
        }
    }

    return classType;
}

BOOL CSocketCANDeviceInfo::SetInterfaceName(CStdString p_InterfaceName)
{
    BOOL oResult(TRUE);

    m_InterfaceName = p_InterfaceName;

    return oResult;
}

BOOL CSocketCANDeviceInfo::SetDeviceName(CStdString p_DeviceName)
{
    BOOL oResult(TRUE);

    m_DeviceName = p_DeviceName;

    return oResult;
}

BOOL CSocketCANDeviceInfo::SetDriverName(CStdString p_DriverName)
{
    BOOL oResult(TRUE);

    m_DriverName = p_DriverName;

    return oResult;
}

BOOL CSocketCANDeviceInfo::SetDeviceAddress(CStdString p_DeviceAddress)
{
    BOOL oResult(TRUE);

    m_DeviceAddress = p_DeviceAddress;

    return oResult;
}

BOOL CSocketCANDeviceInfo::SetInterfaceIndex(int p_lInterfaceIndex)
{
    BOOL oResult(TRUE);

    m_lInterfaceIndex = p_lInterfaceIndex;

    return oResult;
}

int CSocketCANDeviceInfo::GetInterfaceIndex()
{
	return m_lInterfaceIndex;
}

CStdString CSocketCANDeviceInfo::GetDeviceName()
{
	return m_DeviceName;
}

CStdString CSocketCANDeviceInfo::GetDriverName()
{
	return m_DriverName;
}

CStdString CSocketCANDeviceInfo::GetDeviceAddress()
{
	return m_DeviceAddress;
}

CStdString CSocketCANDeviceInfo::GetVirtualPortName()
{
	CStdString virtualPortName;

	virtualPortName.Format("CAN%d", GetPortCount());

	return virtualPortName;
}

BOOL CSocketCANDeviceInfo::AddPort(CStdString p_PhysicalPortName)
{
    m_PortList.push_back(tStdStringPair(p_PhysicalPortName, GetVirtualPortName()));

    return TRUE;
}

CStdString CSocketCANDeviceInfo::GetInterfaceName()
{
	CStdString interfaceName(_T(""));

	if(m_lInterfaceIndex != -1)
	{
		interfaceName.Format(_T("%s %i"), m_InterfaceName, m_lInterfaceIndex);
	}
	else
	{
		interfaceName = m_InterfaceName;
	}

	return interfaceName;
}

int CSocketCANDeviceInfo::GetPortCount()
{
	return m_PortList.size();
}

CStdStringArray CSocketCANDeviceInfo::GetPortNames()
{
	CStdStringArray portNames;

	for(tStdStringPairList::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
	{
		portNames.push_back((*it).second);
	}

    return portNames;
}

CStdString CSocketCANDeviceInfo::GetPhysicalPortName(CStdString p_VirtualPortName)
{
	CStdString portName;

	for(tStdStringPairList::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
	{
		if((*it).second ==  p_VirtualPortName)
		{
			portName = (*it).first;
			break;
		}
	}

	return portName;
}

CStdString CSocketCANDeviceInfo::GetVirtualPortName(CStdString p_PhysicalPortName)
{
	CStdString portName;

	for(tStdStringPairList::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
	{
		if((*it).first ==  p_PhysicalPortName)
		{
			portName = (*it).second;
			break;
		}
	}

	return portName;
}

CStdStringArray CSocketCANDeviceInfo::GetPhysicalPortNames()
{
	CStdStringArray portNames;

	for(tStdStringPairList::iterator it = m_PortList.begin(); it != m_PortList.end(); it++)
	{
		portNames.push_back((*it).first);
	}

	return portNames;
}
