
#include "stdafx.h"
#include <Events/IEventsBase.h>
#include <Events/EventsSourceBase.h>

CEventsSourceBase::CEventsSourceBase()
{
}

CEventsSourceBase::~CEventsSourceBase(void)
{
    if(m_mutex.Lock(0))
    {
        DoUnregistration();
    }

    m_mutex.Unlock();
}

BOOL CEventsSourceBase::RegisterEvents(CIEventsBase* p_pIEvents)
{
    BOOL oResult(FALSE);

    if(m_mutex.Lock(0))
    {
        if(!IsEventsExisting(p_pIEvents))
        {
            //Reverse Registration for Unregistration in desctructor
            p_pIEvents->RegisterEventsSource(this);

            //Register
            m_EventsList.push_back(p_pIEvents);
            oResult = TRUE;
        }
        else
        {
            //Already registered
            oResult = TRUE;
        }
    }
    
    m_mutex.Unlock();

    return oResult;
}

BOOL CEventsSourceBase::UnregisterEvents(CIEventsBase* p_pIEvents)
{
    BOOL oResult(FALSE);

    if(m_mutex.Lock(0))
    {
        if(IsEventsExisting(p_pIEvents))
        {
            //Reverse Unregistration for Unregistration in desctructor
            p_pIEvents->UnregisterEventsSource(this);

            //Unregister
            m_EventsList.remove(p_pIEvents);
            oResult = TRUE;
        }
        else
        {
            //Not registered
            oResult = TRUE;
        }
    }
    
    m_mutex.Unlock();

    return oResult;
}

BOOL CEventsSourceBase::IsEventsExisting(CIEventsBase* p_pIEvents)
{
    BOOL oResult(FALSE);

    for(std::list<CIEventsBase*>::iterator it = m_EventsList.begin(); it != m_EventsList.end(); it++)
    {
        if(p_pIEvents == *(it))
        {
            oResult = TRUE;
            break;
        }
    }

    return oResult;
}

BOOL CEventsSourceBase::DoUnregistration()
{
    BOOL oResult(TRUE);
    CIEventsBase* p_pEvents(0);

    for(std::list<CIEventsBase*>::iterator it = m_EventsList.begin(); it != m_EventsList.end() && oResult; it++)
    {
        p_pEvents = *(it);
        if(p_pEvents)
        {
            if(!p_pEvents->UnregisterEventsSource(this))
            {
                oResult = FALSE;
            }
        }
    }

    return oResult;
}

