#include "stdafx.h"
#include <Log/MmcLog/Log/Message/LogMessageLevel.h>
#include <Log/MmcLog/Log/Message/AllMessageLevel.h>
#include <Log/MmcLog/Log/Message/UserDefinedMessageLevel.h>
#include <Process/MmcProcess.h>

#include <Log/MmcLog/Log/Filter/LogFilter.h>

using namespace std;

const unsigned int ALL_PIDS = -1;
const unsigned int ALL_TIDS = -1;

CLogFilter::CLogFilter()
{
}

CLogFilter::~CLogFilter()
{
    DeleteLevelLists();
}

void CLogFilter::DeleteLevelLists()
{
    DeleteLevelList(m_PositiveLevelList);    
}

void CLogFilter::DeleteLevelList(std::list<CLogMessageLevel*>& p_rLevelList)
{
    list<CLogMessageLevel*>::iterator it = p_rLevelList.begin();

    while(it != p_rLevelList.end())
    {
        CLogMessageLevel* pLevel = (*it);

        p_rLevelList.erase(it);

        delete pLevel;

        it = p_rLevelList.begin();
    }
}

void CLogFilter::AddType(ELogMessageType p_eType)
{
    m_PositiveTypeList.push_back(p_eType);
}

void CLogFilter::RemoveType(ELogMessageType p_eType)
{
    m_NegativeTypeList.push_back(p_eType);    
}

void CLogFilter::RemoveMessageLevel(CStdString p_Level)
{    
    m_NegativeLevelList.push_back(p_Level);    
}

int CLogFilter::GetTypesCount()
{
    return ((int)(m_PositiveTypeList.size() - m_NegativeTypeList.size()));
}

void CLogFilter::AddMessageLevel(CLogMessageLevel* p_pMessageLevel)
{
    m_PositiveLevelList.push_back(p_pMessageLevel);
}

void CLogFilter::AddTid(unsigned int p_lTid)
{
    m_TidList.push_back(p_lTid);
}

void CLogFilter::AddPid(unsigned int p_lPid)
{
    m_PidList.push_back(p_lPid);
}

bool CLogFilter::IsMatchingProcess()
{
    bool oResult = false;
    unsigned long ulCurrentProcessId = MmcGetCurrentProcessId();

    for(list<unsigned int>::iterator it = m_PidList.begin(); it != m_PidList.end(); it++)
    {
        if((*it) == ALL_PIDS)
        {
            oResult = true;
            break;
        }
        else if(ulCurrentProcessId == (*it))
        {
            oResult = true;
            break;
        }
    }

    return oResult;
}

bool CLogFilter::IsMatchingThread()
{
    bool oResult = false;

    unsigned long ulCurrentThreadId = MmcGetCurrentThreadId();

    for(list<unsigned int>::iterator it = m_TidList.begin(); it != m_TidList.end(); it++)
    {
        if((*it) == ALL_TIDS)
        {
            oResult = true;
            break;
        }
        else if(ulCurrentThreadId == (*it))
        {
            oResult = true;
            break;
        }
    }

    return oResult;
}

bool CLogFilter::IsMatching()
{
    bool oResult = false;

    if(IsMatchingProcess())
    {
        oResult = IsMatchingThread();
    }

    return oResult;
}

bool CLogFilter::IsTypeInList(std::list<ELogMessageType>& p_TypeList, ELogMessageType p_eType)
{
    bool oResult = false;

    for(list<ELogMessageType>::iterator it = p_TypeList.begin(); it != p_TypeList.end(); it++)
    {
        if(*it == p_eType)
        {
            oResult = true;
            break;
        }
    }

    return oResult;
}

bool CLogFilter::IsMatching(ELogMessageType p_eType)
{
    bool oResult = true;

    if(p_eType == LMT_DEBUG)
    {
#ifdef _DEBUG
        oResult = true;
#else
        oResult = false;
#endif 
    }

    if(oResult)
    {
        oResult = IsMatching();

        if(oResult)
        {
            oResult = IsTypeInList(m_PositiveTypeList, p_eType);

            if(oResult)
            {
                oResult = !IsTypeInList(m_NegativeTypeList, p_eType);
            }
        }
    }
        
    return oResult;
}

bool CLogFilter::IsLevelInPositiveList(CLogMessageLevel* p_pMessageLevel)
{
    bool oResult = false;

    for(list<CLogMessageLevel*>::iterator it = m_PositiveLevelList.begin(); it != m_PositiveLevelList.end(); it++)
    {
        CLogMessageLevel* pLevel = (*it);
        CAllMessageLevel* pAllMessageLevel = dynamic_cast<CAllMessageLevel*>(pLevel); 

        if(pAllMessageLevel || (pLevel->GetUniqueId() == p_pMessageLevel->GetUniqueId()))
        {
            oResult = true;
            break;
        }
    }

    return oResult;
}

bool CLogFilter::IsLevelInNegativeList(CStdString p_LevelUniqueId)
{
    bool oResult = false;

    for(list<CStdString>::iterator it = m_NegativeLevelList.begin(); it!= m_NegativeLevelList.end(); it++)
    {
        if((*it) == p_LevelUniqueId.Trim())
        {
            oResult = true;
            break;
        }
    }

    return oResult;
}

bool CLogFilter::IsMatching(CLogMessageLevel* p_pMessageLevel)
{
    bool oResult = IsMatching();

    if(oResult)
    {
        oResult = IsLevelInPositiveList(p_pMessageLevel);

        if(oResult)
        {
            oResult = !IsLevelInNegativeList(p_pMessageLevel->GetUniqueId());
        }
    }

    return oResult;
}

bool CLogFilter::IsMatching(ELogMessageType p_eType, CLogMessageLevel* p_pMessageLevel)
{
    bool oResult = IsMatching(p_eType);

    if(oResult)
    {
        oResult = IsMatching(p_pMessageLevel);
    }

    return oResult;
}
