#include "stdafx.h"
#include <Log/MmcLog/Log/Filter/LogFilter.h>
#include <Log/MmcLog/Log/Message/LogMessage.h>
#include <Log/MmcLog/Log/Output/LogOutput.h>
#include <Time/MmcHiResTimer.h>

#include <Log/MmcLog/Log/Log.h>

CLog::CLog()
    : m_pLogFilter(0)
    , m_pLogMessage(0)
    , m_pLogOutput(0)
{
}

CLog::~CLog()
{
    DeleteFilter();
    DeleteMessage();
    DeleteOutput();
}

void CLog::DeleteFilter()
{
    if(m_pLogFilter)
    {
        delete m_pLogFilter;
        m_pLogFilter = 0;
    }
}

void CLog::DeleteMessage()
{
    if(m_pLogMessage)
    {
        delete m_pLogMessage;
        m_pLogMessage = 0;
    }
}

void CLog::DeleteOutput()
{
    if(m_pLogOutput)
    {
        delete m_pLogOutput;
        m_pLogOutput = 0;
    }
}

void CLog::InitFilter(CLogFilter* p_pLogFilter)
{
    m_pLogFilter = p_pLogFilter;
}

void CLog::InitMessage(CLogMessage* p_pLogMessage)
{
    if(p_pLogMessage)
    {
        m_pLogMessage = p_pLogMessage;
    }
}

void CLog::InitOutput(CLogOutput* p_pLogOutput)
{
    m_pLogOutput = p_pLogOutput;
}

bool CLog::WriteMessage(CStdString p_Msg)
{
    bool oResult = false;
    CStdString outputMessage;

    if(m_pLogMessage)
    {
        m_pLogMessage->ResetType();
        m_pLogMessage->ResetLevel();

        outputMessage = m_pLogMessage->Format(p_Msg);
    }

    if(m_pLogOutput)
    {
        oResult = m_pLogOutput->Write(outputMessage);
    }

    return oResult;
}

bool CLog::WriteMessage(ELogMessageType p_eType, CStdString p_Msg)
{
    bool oResult = false;
    CStdString outputMessage;

    if(m_pLogMessage)
    {
        m_pLogMessage->SetType(p_eType);
        m_pLogMessage->ResetLevel();

        outputMessage = m_pLogMessage->Format(p_Msg);
    }

    if(m_pLogOutput)
    {
        oResult = m_pLogOutput->Write(outputMessage);
    }

    return oResult;
}

bool CLog::WriteMessage(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = false;
    CStdString outputMessage;

    if(m_pLogMessage)
    {
        m_pLogMessage->SetType(p_eType);
        m_pLogMessage->SetLevel(p_pLevel);

        outputMessage = m_pLogMessage->Format(p_Msg);
    }

    if(m_pLogOutput)
    {
        oResult = m_pLogOutput->Write(outputMessage);
    }

    return oResult;
}

bool CLog::WriteMessage(CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = false;
    CStdString outputMessage;

    if(m_pLogMessage)
    {
        m_pLogMessage->SetLevel(p_pLevel);
        m_pLogMessage->ResetType();

        outputMessage = m_pLogMessage->Format(p_Msg);
    }

    if(m_pLogOutput)
    {
        oResult = m_pLogOutput->Write(outputMessage);
    }

    return oResult;
}

bool CLog::WriteMessage(CStdString p_Msg, double p_Timestamp)
{
    bool oResult = false;
    CStdString outputMessage;

    if(m_pLogMessage)
    {
        m_pLogMessage->ResetType();
        m_pLogMessage->ResetLevel();

        outputMessage = m_pLogMessage->Format(p_Msg, p_Timestamp);
    }

    if(m_pLogOutput)
    {
        oResult = m_pLogOutput->Write(outputMessage);
    }

    return oResult;
}

bool CLog::Write(CStdString p_Msg)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching();
    }

    if(oResult)
    {
        WriteMessage(p_Msg);
    }

    return oResult;
}

bool CLog::Write(CStdString p_Msg, double p_Timestamp)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching();
    }

    if(oResult)
    {
        WriteMessage(p_Msg, p_Timestamp);
    }

    return oResult;
}

bool CLog::Write(ELogMessageType p_eType, CStdString p_Msg)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(p_eType);
    }

    if(oResult)
    {
        WriteMessage(p_eType, p_Msg);
    }

    return oResult;
}

bool CLog::Write(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(p_eType, p_pLevel);
    }

    if(oResult)
    {
        WriteMessage(p_eType, p_pLevel, p_Msg);
    }

    return oResult;
}

bool CLog::Write(CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(p_pLevel);
    }

    if(oResult)
    {
        WriteMessage(p_pLevel, p_Msg);
    }

    return oResult;
}

bool CLog::IsActive(ELogMessageType p_eMessageType)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(p_eMessageType);
    }

    return oResult;
}

bool CLog::IsActive(CLogMessageLevel& p_rMsgLevel)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(&p_rMsgLevel);
    }

    return oResult;
}

bool CLog::IsActive(ELogMessageType p_eMessageType, CLogMessageLevel& p_rMsgLevel)
{
    bool oResult = true;

    if(m_pLogFilter)
    {
        oResult = m_pLogFilter->IsMatching(p_eMessageType, &p_rMsgLevel);
    }

    return oResult;
}

bool CLog::Log(CStdString p_Msg)
{
    bool oResult = false;

    oResult = Write(p_Msg);

    return oResult;
}

bool CLog::Log(CStdString p_Msg, double p_Timing)
{
    bool oResult = false;

    oResult = Write(p_Msg, p_Timing);

    return oResult;
}

bool CLog::Log(ELogMessageType p_eType, CStdString p_Msg)
{
    bool oResult = false;

    oResult = Write(p_eType, p_Msg);

    return oResult;
}

bool CLog::Log(CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = false;

    oResult = Write(p_pLevel, p_Msg);

    return oResult;
}

bool CLog::Log(ELogMessageType p_eType, CLogMessageLevel* p_pLevel, CStdString p_Msg)
{
    bool oResult = false;

    oResult = Write(p_eType, p_pLevel, p_Msg);

    return oResult;
}

void CLog::CreateTimer()
{
    DeleteTimer();

    m_pTimer = new CMmcHiResTimer();
}

void CLog::DeleteTimer()
{
    if(m_pTimer)
    {
        delete m_pTimer;
        m_pTimer = 0;
    }
}

bool CLog::LogTimingStart(CStdString p_Msg)
{
    bool oResult = false;

    CreateTimer();

    if(m_pTimer)
    {
        m_pTimer->start();

        oResult = Log(p_Msg);
    }

    return oResult;
}

bool CLog::LogTimingStop(CStdString p_Msg)
{
    bool oResult = false;

    if(m_pTimer)
    {
        m_pTimer->stop();

        oResult = Log(p_Msg, m_pTimer->getElapsedTimeInMilliSec());
    }

    return oResult;
}
