#include "stdafx.h"

#include <iostream>
#include <iomanip>
#include <sstream>
#include <Process/MmcProcess.h>
#include <Thread/MmcSingleLock.h>

#include <Log/MmcLog/Log/Message/Format/TimespanLogFormatMaskType.h>

using namespace std;

tTimespanEntryList* CTimespanLogFormatMaskType::m_pThreadTimespanEntries = 0;
CMmcCriticalSection* CTimespanLogFormatMaskType::m_pLockObject = 0;

CTimespanLogFormatMaskType::CTimespanLogFormatMaskType()
{
    m_pThreadTimespanEntries = new tTimespanEntryList();
    m_pLockObject = new CMmcCriticalSection();

    SetMaskType(FMT_TIMESPAN);
}

CTimespanLogFormatMaskType::~CTimespanLogFormatMaskType()
{
}

void CTimespanLogFormatMaskType::AddLeadingZeros(CStdString& p_rMsg)
{
    const int MIN_FIELD_SIZE = 10;

    while(p_rMsg.GetLength() < MIN_FIELD_SIZE)
    {
        p_rMsg.Insert(0, _T("0"));
    }
}

void CTimespanLogFormatMaskType::Cleanup()
{
    if(m_pLockObject && m_pThreadTimespanEntries)
    {
        CMmcSingleLock lock(m_pLockObject, TRUE);

        tTimespanEntryList::iterator it = m_pThreadTimespanEntries->begin();

        while(it != m_pThreadTimespanEntries->end())
        {
            CTimespanEntry* pTimespanEntry = (*it);
            delete pTimespanEntry;
            m_pThreadTimespanEntries->erase(it);
            it = m_pThreadTimespanEntries->begin();
        }

        delete m_pThreadTimespanEntries;
        m_pThreadTimespanEntries = 0;
    }

    if(m_pLockObject)
    {
        delete m_pLockObject;
        m_pLockObject = 0;
    }
}

bool CTimespanLogFormatMaskType::FindTimespanEntry(unsigned long p_ulThreadId, CTimespanEntry*& p_rpTimespanEntry)
{
    bool oResult = false;

    if(m_pThreadTimespanEntries)
    {
        for(tTimespanEntryList::iterator it = m_pThreadTimespanEntries->begin(); it != m_pThreadTimespanEntries->end(); it++)
        {
            if((*it)->GetThreadId() == p_ulThreadId)
            {
                p_rpTimespanEntry = (*it);
                oResult = true;
                break;
            }
        }
    }

    return oResult;
}

CStdString CTimespanLogFormatMaskType::ToString()
{
    CStdString result;
    CTimespanEntry* pTimespanEntry = 0;

    if(m_pLockObject && m_pThreadTimespanEntries)
    {
        if(!FindTimespanEntry(MmcGetCurrentThreadId(), pTimespanEntry))
        {
            CMmcSingleLock lock(m_pLockObject, TRUE);
            pTimespanEntry = new CTimespanEntry();
            pTimespanEntry->Start();
            m_pThreadTimespanEntries->push_back(pTimespanEntry);
        }
    }

    if(pTimespanEntry)
    {
        result.Format(_T("%.3f"), pTimespanEntry->GetTimespan());
        AddLeadingZeros(result);
        pTimespanEntry->Reset();
    }

    return result;
}
