#include "stdafx.h"
#include <Log/MmcLog/Log/Message/Format/LogFormat.h>
#include <Log/MmcLog/Log/Message/Format/TimingLogFormat.h>
#include <Log/MmcLog/Log/Message/Format/TimespanLogFormatMaskType.h>
#include <Log/MmcLog/Log/Message/LogMessage.h>

CLogMessage::CLogMessage()
    : m_pLogMessageLevel(0)
    , m_eType(LMT_UNKNOWN)
{
}

CLogMessage::~CLogMessage()
{
    DeleteLogFormatList();

    CTimespanLogFormatMaskType::Cleanup();
}

void CLogMessage::DeleteLogFormatList()
{
    std::list<CLogFormat*>::iterator it = m_LogFormatList.begin();

    while(it != m_LogFormatList.end())
    {
        CLogFormat* pLogFormat = (*it);

        m_LogFormatList.erase(it);

        delete pLogFormat;

        it = m_LogFormatList.begin();
    }
}

void CLogMessage::AddFormat(CLogFormat* p_pLogFormat)
{
    m_LogFormatList.push_back(p_pLogFormat);
}

void CLogMessage::ResetLevel()
{
    m_pLogMessageLevel = 0;
}

void CLogMessage::ResetType()
{
    m_eType = LMT_UNKNOWN;
}

CLogMessageLevel* CLogMessage::GetLevel()
{
    return m_pLogMessageLevel;
}

CStdString CLogMessage::GetMessage()
{
    return m_Message;
}

ELogMessageType CLogMessage::GetType()
{
    return m_eType;
}

void CLogMessage::SetLevel(CLogMessageLevel* p_LogMessageLevel)
{
    m_pLogMessageLevel = p_LogMessageLevel;
}

void CLogMessage::SetType(ELogMessageType p_eType)
{
    m_eType = p_eType;
}

CStdString CLogMessage::Format(CStdString p_Msg)
{
    CStdString result;

    m_Message = p_Msg;

    for(std::list<CLogFormat*>::iterator it = m_LogFormatList.begin(); it != m_LogFormatList.end(); it++)
    {
        CLogFormat* pLogFormat = (*it);
        CTimingLogFormat* pTimingLogFormat = dynamic_cast<CTimingLogFormat*>(*it);

        if(pLogFormat && pTimingLogFormat == 0)
        {
            result = pLogFormat->Format(p_Msg);
        }
    }

    return result;
}

CStdString CLogMessage::Format(CStdString p_Msg, double p_Timestamp)
{
    CStdString result;

    for(std::list<CLogFormat*>::iterator it = m_LogFormatList.begin(); it != m_LogFormatList.end(); it++)
    {
        CTimingLogFormat* pLogFormat = dynamic_cast<CTimingLogFormat*>(*it);

        if(pLogFormat)
        {
            result = pLogFormat->Format(p_Msg, p_Timestamp);
        }
    }

    return result;
}
