#include "stdafx.h"
#include <Log/MmcLog/MmcLog.h>

using namespace std;

bool CMmcLog::m_oLogInitialized = false;
auto_ptr<CLog> CMmcLog::m_Log(new CLog());
auto_ptr<CMmcCriticalSection> CMmcLog::m_LockObject(new CMmcCriticalSection());

void CMmcLog::InitLog()
{
    CMmcSingleLock lock(m_LockObject.get());
    CMmcLogFactory factory;

    Lock(lock);

    m_Log->InitFilter(factory.CreateFilter());
    m_Log->InitMessage(factory.CreateMessage());
    m_Log->InitOutput(factory.CreateOutput());

    m_oLogInitialized = true;

    Unlock(lock);
}

void CMmcLog::Lock(CMmcSingleLock& p_rSingleLock)
{
    p_rSingleLock.Lock();
}

void CMmcLog::Unlock(CMmcSingleLock& p_rSingleLock)
{
    p_rSingleLock.Unlock();
}

bool CMmcLog::Log(CStdString p_msg)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->Log(p_msg);

    Unlock(lock);

    return oResult;
}

bool CMmcLog::Log(ELogMessageType p_eMessageType, CStdString p_msg)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->Log(p_eMessageType, p_msg);

    Unlock(lock);

    return oResult;
}

bool CMmcLog::Log(CLogMessageLevel p_rMsgLevel, CStdString p_msg)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->Log(&p_rMsgLevel, p_msg);

    Unlock(lock);

    return oResult;
}

void CMmcLog::Cleanup()
{
    if(m_Log.get())
    {
        CLog* pLog = m_Log.release();
        delete pLog;
    }

    if(m_LockObject.get())
    {
        CMmcSingleLock lock(m_LockObject.get());
        CMmcCriticalSection* pLockObject = m_LockObject.release();

        delete pLockObject;
    }
}

bool CMmcLog::Log(ELogMessageType p_eMessageType, CLogMessageLevel p_rMsgLevel, CStdString p_msg)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->Log(p_eMessageType, &p_rMsgLevel, p_msg);

    Unlock(lock);

    return oResult;
}

bool CMmcLog::IsActive(ELogMessageType p_eMessageType)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->IsActive(p_eMessageType);

    Unlock(lock);

    return oResult;
}

bool CMmcLog::IsActive(CLogMessageLevel p_rMsgLevel)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->IsActive(p_rMsgLevel);

    Unlock(lock);

    return oResult;
}

bool CMmcLog::IsActive(ELogMessageType p_eMessageType, CLogMessageLevel p_rMsgLevel)
{
    bool oResult = false;

    if(!m_oLogInitialized)
    {
        InitLog();
    }

    CMmcSingleLock lock(m_LockObject.get());

    Lock(lock);

    oResult = m_Log->IsActive(p_eMessageType, p_rMsgLevel);

    Unlock(lock);

    return oResult;
}
