#include "stdafx.h"
#include <Thread/Events/IThreadBaseEvents.h>
#include <Thread/Events/ThreadBaseEventsSource.h>
#include <Thread/MmcSingleLock.h>
#include <Thread/MmcThreadBase.h>

CMmcThreadBase::CMmcThreadBase()
    : m_pEventsSource(0)
    , m_ThreadState(TBS_STOPPED)
{
    CreateEventsSource();
}

CMmcThreadBase::~CMmcThreadBase()
{
    DeleteEventsSource();
}

void CMmcThreadBase::CreateEventsSource()
{
    if (m_pEventsSource == 0)
    {
        m_pEventsSource = new CThreadBaseEventsSource();
        m_pEventsSource->InitThreadBase(this);
    }
}

void CMmcThreadBase::DeleteEventsSource()
{
    if (m_pEventsSource != 0)
    {
        delete m_pEventsSource;
        m_pEventsSource = 0;
    }
}

BOOL CMmcThreadBase::RegisterEvents(CIThreadBaseEvents* p_pIEvents)
{
    BOOL oResult(FALSE);

    if (m_pEventsSource)
    {
        oResult = m_pEventsSource->RegisterEvents(p_pIEvents);
    }

    return oResult;
}

BOOL CMmcThreadBase::UnregisterEvents(CIThreadBaseEvents* p_pIEvents)
{
    BOOL oResult(FALSE);

    if (m_pEventsSource)
    {
        oResult = m_pEventsSource->UnregisterEvents(p_pIEvents);
    }

    return oResult;
}

void CMmcThreadBase::SetThreadState(EThreadBaseState p_eState)
{
    CMmcSingleLock lock(&m_Sync);
    bool oChanged = false;

    lock.Lock();

    if (m_ThreadState != p_eState)
    {
        m_ThreadState = p_eState;
        oChanged = true;
    }

    lock.Unlock();

    if (m_pEventsSource && oChanged)
    {
        m_pEventsSource->SendEventStateChanged(this, p_eState);
    }
}

EThreadBaseState CMmcThreadBase::GetThreadState()
{
    return m_ThreadState;
}

BOOL CMmcThreadBase::CanStart()
{
    CMmcSingleLock lock(&m_Sync, true);

    return (m_ThreadState == TBS_STOPPED);
}

BOOL CMmcThreadBase::IsStopping()
{
    CMmcSingleLock lock(&m_Sync, true);

    return (m_ThreadState == TBS_STOPPING);
}

BOOL CMmcThreadBase::IsActive()
{
    CMmcSingleLock lock(&m_Sync, true);

    return (m_ThreadState == TBS_STARTING || m_ThreadState == TBS_STARTED || m_ThreadState == TBS_STOPPING);
}

BOOL CMmcThreadBase::IsStarted()
{
    CMmcSingleLock lock(&m_Sync, true);

    return (m_ThreadState == TBS_STARTED);
}

int CMmcThreadBase::IsStopped()
{
    CMmcSingleLock lock(&m_Sync, true);

    return (m_ThreadState == TBS_STOPPED);
}

void CMmcThreadBase::NotifyPriorityChange(int p_lNewPriority)
{
    if (m_pEventsSource)
    {
        m_pEventsSource->SendEventPriorityChanged(this, p_lNewPriority);
    }
}
