#ifndef CLINUXUSBDEVICE_H
#define CLINUXUSBDEVICE_H

#include <list>
#include "StdString.h"
#include "../../../../include/Drv/Ftd2xx/ftd2xx.h"

class CLinuxUsbDirectory;
class CUsbDeviceDescription;
class CLinuxUsbTree;

class CLinuxUsbDevice
{
public:
    CLinuxUsbDevice();
    virtual ~CLinuxUsbDevice();
    
    virtual void Reset();
    
    void Init(CLinuxUsbTree* p_pUsbTree, std::list<CLinuxUsbDirectory*> p_DirectoryList, std::list<CUsbDeviceDescription*> p_DeviceInfoList, CLinuxUsbDirectory* p_pDirectory);
    
    virtual bool Load();
        
    int GetBusNum();
    int GetPortNum();
    int GetDevNum();
    CStdString GetDevPath();
    CStdString GetDevicePath();
    int GetProductId();
    int GetVendorId();
    CStdString GetManufacturer();
    CStdString GetProduct();
    CStdString GetVersion();
    CStdString GetSerialNumber();
    int GetSerialNumberId();
    libusb_class_code GetDeviceClass();
    
    CLinuxUsbDirectory* GetDirectory();            
    std::list<CLinuxUsbDevice*> GetChildren();
    
    bool DeviceExists();
    bool DeviceExists(int p_lBusNum, int p_lPortNum, CStdString p_DevicePath, int p_lIdProduct, int p_lIdVendor);

protected:
    bool ReadIntValue(CStdString p_Identifier, int& p_rlValue);
    bool ReadStringValue(CStdString p_Identifier, CStdString& p_rValue);
    bool GetChildDirectories(std::list<CLinuxUsbDirectory*>& p_rChildDirs);
    bool LoadChildren();
    
private:
    void DeleteChildren();
    bool UpdateBusNum();
    bool UpdatePortNum();
    bool UpdateDevNum();
    bool UpdateDevPath();
    bool UpdateProductId();
    bool UpdateVendorId();
    bool UpdateManufacturer();
    bool UpdateProduct();
    bool UpdateVersion();
    bool UpdateDeviceClass();
    bool UpdateSerialNumber();
    bool UpdateSerialNumberId();
    bool UpdateDevicePath();
    bool FindMatchingDeviceDescription(CUsbDeviceDescription*& p_pDeviceDescription);
    
private:
    int m_lBusNum;
    int m_lPortNum;
    int m_lDevNum;
    CStdString m_DevPath;
    CStdString m_DevicePath;
    int m_lIdProduct;
    int m_lIdVendor;
    CStdString m_Manufacturer;
    CStdString m_Product;
    CStdString m_Version;
    CStdString m_SerialNumber;
    int m_SerialNumberId;
    libusb_class_code m_eDeviceClass;
    CLinuxUsbDirectory* m_pDirectory;
    std::list<CLinuxUsbDirectory*> m_DirectoryList;
    std::list<CUsbDeviceDescription*> m_DeviceInfoList;
    std::list<CLinuxUsbDevice*> m_Children;
    CLinuxUsbTree* m_pUsbTree;
};

#endif // CLINUXUSBDEVICE_H
