#pragma once

#include "HidDeviceDescription.h"
#include <wchar.h>
#include <list>

#include "HidDeviceLinux.h"

class CHidApi
{
public:
	static int Init();
	static int Exit();

	static CHidDevice* Open(unsigned short p_usVendorId, unsigned short p_usProductId, const wchar_t *p_pSerial_number);
	static CHidDevice* OpenPath(const char *p_pPath);
	static void Close(CHidDevice *p_pDevice);

	static bool Enumerate(unsigned short p_usVendorId, unsigned short p_usProductId, std::list<CHidDeviceDescription*>& p_rDeviceInfoList);
	static void FreeEnumeration(std::list<CHidDeviceDescription*>& p_rDeviceInfoList);

	static int  Read(CHidDevice *p_pDevice, unsigned char *p_pData, size_t p_Length);
	static int  ReadTimeout(CHidDevice *p_pDevice, unsigned char *p_pData, size_t p_Length, int p_lMilliseconds);
	static int  Write(CHidDevice *p_pDevice, const unsigned char *p_pData, size_t p_lLength);

	static int  SetNonblocking(CHidDevice *p_pDevice, int p_lNonblock);
	static int  SendFeatureReport(CHidDevice *p_pDevice, const unsigned char *p_pData, size_t p_Length);
	static int  GetFeatureReport(CHidDevice *p_pDevice, unsigned char *p_pData, size_t p_Length);

	static int  GetManufacturerString(CHidDevice *p_pDevice, wchar_t *p_pString, size_t p_Maxlen);
	static int  GetProductString(CHidDevice *p_pDevice, wchar_t *p_pString, size_t p_Maxlen);
	static int  GetSerialNumberString(CHidDevice *p_pDevice, wchar_t *p_pString, size_t p_Maxlen);

private:
	static CHidDevice *NewDevice();
	static void FreeDevice(CHidDevice *p_pDevice);
	static int ReturnData(CHidDevice *p_Device, unsigned char *p_pData, size_t p_Length);
	static void ReadCallback(struct libusb_transfer *p_pTransfer);
	static void *ReadThread(void *p_pParam);
	static void CleanupMutex(void *p_pParam);
	static int  GetIndexedString(CHidDevice *p_pDevice, int p_lString_index, wchar_t *p_pString, size_t p_Maxlen);
};
