#include <memory.h>
#include <string.h>
#include <stdio.h>

#include <MmcTypeDefinition.h>
#include <Drv/Helper/UsbTree/UsbTree.h>

#include <Drv/Ftd2xx/MmcFtd2xxHndlLinux.h>
#include "../../../include/Drv/Ftd2xx/ftd2xx.h"

CMmcFtd2xxHndlBase::CMmcFtd2xxHndlBase()
    : m_Handle(0)
{
    FT_SetVIDPID(0x403,0xa8b0);
}

CMmcFtd2xxHndlBase::~CMmcFtd2xxHndlBase()
{
}

CStdString CMmcFtd2xxHndlBase::GetFtStatusDescription(unsigned long status)
{
    CStdString desc("Unknown");

    switch(status)
    {
        case FT_OK: desc = "FT_OK"; break;
        case FT_INVALID_HANDLE: desc = "FT_INVALID_HANDLE"; break;
        case FT_DEVICE_NOT_FOUND: desc = "FT_DEVICE_NOT_FOUND"; break;
        case FT_DEVICE_NOT_OPENED: desc = "FT_DEVICE_NOT_OPENED"; break;
        case FT_IO_ERROR: desc = "FT_IO_ERROR"; break;
        case FT_INSUFFICIENT_RESOURCES: desc = "FT_INSUFFICIENT_RESOURCES"; break;
        case FT_INVALID_PARAMETER: desc = "FT_INVALID_PARAMETER"; break;
        case FT_INVALID_BAUD_RATE: desc = "FT_INVALID_BAUD_RATE"; break;
        case FT_DEVICE_NOT_OPENED_FOR_ERASE: desc = "FT_DEVICE_NOT_OPENED_FOR_ERASE"; break;
        case FT_DEVICE_NOT_OPENED_FOR_WRITE: desc = "FT_DEVICE_NOT_OPENED_FOR_WRITE"; break;
        case FT_FAILED_TO_WRITE_DEVICE: desc = "FT_FAILED_TO_WRITE_DEVICE"; break;
        case FT_EEPROM_READ_FAILED: desc = "FT_EEPROM_READ_FAILED"; break;
        case FT_EEPROM_WRITE_FAILED: desc = "FT_EEPROM_WRITE_FAILED"; break;
        case FT_EEPROM_ERASE_FAILED: desc = "FT_EEPROM_ERASE_FAILED"; break;
        case FT_EEPROM_NOT_PRESENT: desc = "FT_EEPROM_NOT_PRESENT"; break;
        case FT_EEPROM_NOT_PROGRAMMED: desc = "FT_EEPROM_NOT_PROGRAMMED"; break;
        case FT_INVALID_ARGS: desc = "FT_INVALID_ARGS"; break;
        case FT_NOT_SUPPORTED: desc = "FT_NOT_SUPPORTED"; break;
        case FT_OTHER_ERROR: desc = "FT_OTHER_ERROR"; break;
        case FT_DEVICE_LIST_NOT_READY: desc = "FT_DEVICE_LIST_NOT_READY"; break;
        default: desc.Format("FT_Unknown - error code=%ld", status);
    }

    return desc;
}

bool CMmcFtd2xxHndlBase::GetDeviceInfoDetail(unsigned int dDeviceIndex,
    unsigned int *pFlags,
    unsigned int *pType,
    unsigned int *pID,
    unsigned int *pLocId,
    char* strSerialNumberBuffer,
    char* strDescriptionBuffer)
{

    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_HANDLE ftHandleTemp;

    FT_STATUS ftStatus = FT_GetDeviceInfoDetail((DWORD)dDeviceIndex,
        (DWORD*)pFlags,
        (DWORD*)pType,
        (DWORD*)pID,
        (DWORD*)pLocId,
        strSerialNumberBuffer,
        strDescriptionBuffer,
        &ftHandleTemp);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_GetDeviceInfoDetail";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::CreateDeviceInfoList(unsigned int *pdNbOfDevices)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    *pdNbOfDevices = 0;

    FT_STATUS ftStatus = FT_CreateDeviceInfoList((DWORD*)pdNbOfDevices);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_CreateDeviceInfoList";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::GetQueueStatus(unsigned int *pdAmountRxQueue)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_GetQueueStatus(m_Handle, (DWORD*)pdAmountRxQueue);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_GetQueueStatus";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::Reload(unsigned short usVendorId, unsigned short usProductId)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    perror("Reload - not implemented");

    return true;
}

bool CMmcFtd2xxHndlBase::Rescan()
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    return true;
}

CStdString CMmcFtd2xxHndlBase::GetDeviceClassType()
{
    CFtd2xxDeviceInfo deviceInfo;

    return deviceInfo.GetClassType();
}

bool CMmcFtd2xxHndlBase::GetDeviceInfos(tUsbDeviceInfoList& p_rDeviceInfoList, unsigned short p_vendorId, unsigned short p_productId, BOOL p_oIsFtdDevice)
{
    bool oResult(false);
    tUsbDeviceInfoList ftdiDeviceInfoList;
    tUsbDeviceInfoList usbTreeDeviceInfoList;

    if(GetDeviceInfosFromUsbTree(usbTreeDeviceInfoList, p_vendorId, p_productId))
    {
        if(p_oIsFtdDevice)
        {
            oResult = GetDeviceInfosFromFtdi(ftdiDeviceInfoList, p_vendorId, p_productId);

            if(oResult)
            {
                oResult = MergeDeviceInfos(p_rDeviceInfoList, usbTreeDeviceInfoList, ftdiDeviceInfoList);

                DeleteDeviceInfoList(ftdiDeviceInfoList);
            }
        }
        else
        {
            oResult = MergeDeviceInfos(p_rDeviceInfoList, usbTreeDeviceInfoList, ftdiDeviceInfoList);
        }

        DeleteDeviceInfoList(usbTreeDeviceInfoList);
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::MergeDeviceInfos(tUsbDeviceInfoList& p_rTargetDeviceInfoList, tUsbDeviceInfoList& p_rUsbTreeDeviceInfoList, tUsbDeviceInfoList& p_rFtdiDeviceInfoList)
{
    bool oResult(true);
    CFtd2xxDeviceInfo* pFtdiDeviceInfo(0);

    for(tUsbDeviceInfoList::iterator it = p_rUsbTreeDeviceInfoList.begin(); it != p_rUsbTreeDeviceInfoList.end(); it++)
    {
        CUsbDeviceInfo* pDeviceInfo = (CUsbDeviceInfo*)(*it);

        if(FindDeviceInfo(p_rFtdiDeviceInfoList, pDeviceInfo->GetVendorId(), pDeviceInfo->GetProductId(), pDeviceInfo->GetSerialNumber(), pFtdiDeviceInfo))
        {
            CFtd2xxDeviceInfo* pMergedFtdiDeviceInfo = new CFtd2xxDeviceInfo();
            pMergedFtdiDeviceInfo->Init(*pDeviceInfo);
            pMergedFtdiDeviceInfo->SetLocationId(pFtdiDeviceInfo->GetLocationId());
            pMergedFtdiDeviceInfo->SetDrvFlags(pFtdiDeviceInfo->GetDrvFlags());
            p_rTargetDeviceInfoList.push_back(pMergedFtdiDeviceInfo);
        }
        else
        {
            CUsbDeviceInfo* pClonedDeviceInfo = new CUsbDeviceInfo();
            pClonedDeviceInfo->Init(*pDeviceInfo);
            p_rTargetDeviceInfoList.push_back(pClonedDeviceInfo);
        }
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::FindDeviceInfo(tUsbDeviceInfoList& p_rDeviceInfoList, CStdString p_vendorId, CStdString p_productId, CStdString p_serialNumber, CFtd2xxDeviceInfo*& p_rpDeviceInfo)
{
    bool oResult(false);

    for(tUsbDeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CUsbDeviceInfo* pDeviceInfo = (CUsbDeviceInfo*)(*it);

        if(pDeviceInfo->GetVendorId() == p_vendorId)
        {
            if(pDeviceInfo->GetProductId() == p_productId)
            {
                if(pDeviceInfo->GetSerialNumber() == p_serialNumber)
                {
                    p_rpDeviceInfo = (CFtd2xxDeviceInfo*)pDeviceInfo;
                    oResult = true;
                    break;
                }
            }
        }
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::GetDeviceInfosFromUsbTree(tUsbDeviceInfoList& p_rDeviceInfoList, unsigned short p_vendorId, unsigned short p_productId)
{
    bool oResult(false);
    CUsbTree* pUsbTree = CUsbTree::GetInstance();
    CStdStringArray portChains;
    CStdString devicePath(_T(""));
    WORD usVendorId(0);
    WORD usProductId(0);
    CStdString serialNumber(_T(""));
    CUsbDeviceInfo* pDeviceInfo(0);

    if(pUsbTree != 0)
    {
        pUsbTree->Load();
        if(pUsbTree->GetPortChains(p_vendorId, p_productId, portChains))
        {
            oResult = true;
            for(CStdStringArray::iterator it = portChains.begin(); it != portChains.end(); it++)
            {
                CStdString portChain = *(it);

                //Init
                pUsbTree->GetInfos(portChain, devicePath, usVendorId, usProductId, serialNumber);

                //Add DeviceInfo
                pDeviceInfo = new CUsbDeviceInfo();
                pDeviceInfo->Init(portChain, devicePath, usVendorId, usProductId, serialNumber);

                //Add to List
                p_rDeviceInfoList.push_back(pDeviceInfo);
            }
        }
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::DeleteDeviceInfoList(tUsbDeviceInfoList& p_rDeviceInfoList)
{
    bool oResult(true);

    for(tUsbDeviceInfoList::iterator it = p_rDeviceInfoList.begin(); it != p_rDeviceInfoList.end(); it++)
    {
        CUsbDeviceInfo* pDeviceInfo = (CUsbDeviceInfo*)(*it);
        delete pDeviceInfo;
    }

    p_rDeviceInfoList.clear();

    return oResult;
}

bool CMmcFtd2xxHndlBase::GetDeviceInfosFromFtdi(tUsbDeviceInfoList& p_rDeviceInfoList, unsigned short p_vendorId, unsigned short p_productId)
{
    const int BUFFER_SIZE(100);
    bool oResult(false);
    char strSerialNumberBuffer[BUFFER_SIZE];
    char strDescriptionBuffer[BUFFER_SIZE];
    unsigned int ulNbOfDevices(0);
    DWORD ulDeviceIndex(0);
    unsigned int Flags(0);
    unsigned int Type(0);
    unsigned int ID(0);
    unsigned int LocId(0);
    CFtd2xxDeviceInfo* pDeviceInfo(0);
    DWORD ulVendorProductId(0);

    //Init
    ulVendorProductId = (p_vendorId << 16) + p_productId;

    //Enum Devices
    if (CreateDeviceInfoList(&ulNbOfDevices))
    {
        oResult = true;

        if(ulNbOfDevices>0)
        {
            for(ulDeviceIndex=0; ulDeviceIndex<ulNbOfDevices; ulDeviceIndex++)
            {
                if (GetDeviceInfoDetail(ulDeviceIndex, &Flags, &Type, &ID, &LocId, strSerialNumberBuffer, strDescriptionBuffer))
                {
                    if(ulVendorProductId == ID)
                    {
                        //Add DeviceInfo
                        pDeviceInfo = new CFtd2xxDeviceInfo();
                        pDeviceInfo->Init(_T(""), _T(""), p_vendorId, p_productId, strSerialNumberBuffer, LocId, Flags);

                        //Add to List
                        p_rDeviceInfoList.push_back(pDeviceInfo);
                    }
                }
            }
        }
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::OpenPort(CFtd2xxDeviceInfo& p_rInfo)
{
    CStdString serialNumber(_T(""));

    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    //Init
    serialNumber = p_rInfo.GetSerialNumber();

    //Open
    FT_STATUS ftStatus = FT_OpenEx((void*)serialNumber.GetBuffer(), FT_OPEN_BY_SERIAL_NUMBER, &m_Handle);

    serialNumber.ReleaseBuffer();

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_OpenEx";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::ClosePort()
{
    FT_STATUS ftStatus = FT_OK;

    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    if(m_Handle)
    {
        ftStatus = FT_Close(m_Handle);

        if( ftStatus != FT_OK )
        {
            std::string errormsg = GetFtStatusDescription(ftStatus);
            errormsg += ":FT_Close";
            perror(errormsg.c_str());
        }
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::Reload()
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    return true;
}

bool CMmcFtd2xxHndlBase::Configure()
{

    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    bool oResult = true;
    const BYTE LATENCY_TIMER(2);
    const DWORD INPUT_TRANSFER_SIZE(128);
    const DWORD OUTPUT_TRANSFER_SIZE(128);
    FT_STATUS ftStatus = FT_OK;

    //Data Characteristics:ByteSize, StopBit, Parity
    if(FT_OK != (ftStatus = FT_SetDataCharacteristics(m_Handle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE)))
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_SetDataCharacteristics";
        perror(errormsg.c_str());
        oResult = false;
    }

    //FlowControl: Dtr, Rts, Xon, Xoff
    if(FT_OK != (ftStatus = FT_SetFlowControl(m_Handle, FT_FLOW_NONE, 0, 0)))
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_SetFlowControl";
        perror(errormsg.c_str());
        oResult = false;
    }

    //LatencyTimer
    if(FT_OK != (ftStatus = FT_SetLatencyTimer(m_Handle, LATENCY_TIMER)))
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_SetLatencyTimer";
        perror(errormsg.c_str());
        oResult = false;
    }

    //TransferSize
    if(FT_OK != (ftStatus = FT_SetUSBParameters(m_Handle, INPUT_TRANSFER_SIZE, OUTPUT_TRANSFER_SIZE)))
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_SetUSBParameters";
        perror(errormsg.c_str());
        oResult = false;
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::SetTimeouts(unsigned int dReadTimeout, unsigned int dWriteTimeout)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_SetTimeouts(m_Handle, dReadTimeout, dWriteTimeout);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_GetStatus";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::SetBaudrate(unsigned int ulBaudrate)
{
    bool oResult(true);

    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        oResult = false;
    }

    if (oResult)
    {
        switch (ulBaudrate)
        {
            case USB_FTDI_BAUDRATE_1M:
                {
                    oResult = true;
                    break;
                }

            default: oResult = false;
        }
    }

    if (oResult)
    {
        FT_STATUS ftStatus = FT_SetBaudRate(m_Handle, ulBaudrate);

        if (ftStatus != FT_OK)
        {
            std::string errormsg = GetFtStatusDescription(ftStatus);
            errormsg += ":FT_GetStatus";
            perror(errormsg.c_str());
        }

        oResult = (FT_OK == ftStatus);
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::GetStatus(unsigned int *pAmountInRxQueue, unsigned int *pAmountInTxQueue, unsigned int *pEventStatus)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_GetStatus(m_Handle, (DWORD*)pAmountInRxQueue, (DWORD*)pAmountInTxQueue, (DWORD*)pEventStatus);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_GetStatus";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::ReadByte( unsigned char *pData, unsigned int *pDataRead)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_Read(m_Handle, pData, 1, (DWORD*)pDataRead);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_SetUSBParameters";
        perror(errormsg.c_str());
    }

    return (FT_OK == ftStatus);
}

bool CMmcFtd2xxHndlBase::Write(void *pDataBuffer, unsigned int dNumberOfBytesToWrite, unsigned int *pdNumberOfBytesWritten)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_Write(m_Handle, pDataBuffer, dNumberOfBytesToWrite, (DWORD*)pdNumberOfBytesWritten);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_Write";
        perror(errormsg.c_str());
    }

    if( *pdNumberOfBytesWritten <= 0 )
    {
        perror("Write processed successfully, but number of bytes written is 0");
        return false;
    }

    return true;
}

bool CMmcFtd2xxHndlBase::Read(void *pDataBuffer, unsigned int dNumberOfBytesToRead, unsigned int *pdNumberOfBytesRead)
{
    if( !AreLibraryFunctionsLoaded() )
    {
        perror("Library not loaded");
        return false;
    }

    FT_STATUS ftStatus = FT_Read(m_Handle, pDataBuffer, dNumberOfBytesToRead, (DWORD*)pdNumberOfBytesRead);

    if( ftStatus != FT_OK )
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_Read";
        perror(errormsg.c_str());
        return false;
    }

    if( *pdNumberOfBytesRead <= 0 )
    {
        //perror("Read processed successfully, but number of read bytes is 0");
        return false;
    }

    return true;
}

bool CMmcFtd2xxHndlBase::EE_Read(
    unsigned short& p_rusVendorId,
    unsigned short& p_rusProductId,
    char*& p_rpManufacturer,
    unsigned int& p_rulManufacturerLength,
    char*& p_rpDescription,
    unsigned int& p_rulDescriptionLength,
    char*& p_rpSerialNumber,
    unsigned int& p_rulSerialNumberLength,
    bool& p_roUseVcpDriver)
{
    const int BUFFER_SIZE(64);

    FT_PROGRAM_DATA ftData;
    TCHAR ManufacturerBuf[BUFFER_SIZE];
    TCHAR ManufacturerIdBuf[BUFFER_SIZE];
    TCHAR DescriptionBuf[BUFFER_SIZE];
    TCHAR SerialNumberBuf[BUFFER_SIZE];
    BOOL oResult(FALSE);
    FT_STATUS ftStatus = FT_OK;

    //ResetErrorInfo
    //m_ErrorInfo.Reset();

    //Prepare Structure
    ftData.Signature1 = 0x00000000;
    ftData.Signature2 = 0xffffffff;
    ftData.Version = 0x00000002; // EEPROM structure with FT232R extensions
    ftData.Manufacturer = ManufacturerBuf;
    ftData.ManufacturerId = ManufacturerIdBuf;
    ftData.Description = DescriptionBuf;
    ftData.SerialNumber = SerialNumberBuf;

    if(FT_OK == (ftStatus = FT_EE_Read(m_Handle, &ftData)))
    {
        //VendorId
        p_rusVendorId = ftData.VendorId;

        //ProductId
        p_rusProductId = ftData.ProductId;

        //Manufacturer
        p_rulManufacturerLength = strlen(ManufacturerBuf)+1;
        p_rpManufacturer = (char*)malloc(p_rulManufacturerLength*sizeof(char));
        memset(p_rpManufacturer, 0, p_rulManufacturerLength*sizeof(char));
        strcpy(p_rpManufacturer, ManufacturerBuf);

        //Description
        p_rulDescriptionLength = strlen(DescriptionBuf)+1;
        p_rpDescription = (char*)malloc(p_rulDescriptionLength*sizeof(char));
        memset(p_rpDescription, 0, p_rulDescriptionLength*sizeof(char));
        strcpy(p_rpDescription, DescriptionBuf);

        //SerialNumber
        p_rulSerialNumberLength = strlen(SerialNumberBuf)+1;
        p_rpSerialNumber = (char*)malloc(p_rulSerialNumberLength*sizeof(char));
        memset(p_rpSerialNumber, 0, p_rulSerialNumberLength*sizeof(char));
        strcpy(p_rpSerialNumber, SerialNumberBuf);

        //UseVcpDriver
        if(ftData.RIsD2XX) p_roUseVcpDriver = FALSE; else p_roUseVcpDriver = TRUE;

        oResult = TRUE;
    }
    else
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_EE_Read";
        perror(errormsg.c_str());

        oResult = FALSE;
    }

    return oResult;
}

bool CMmcFtd2xxHndlBase::EE_Program(
    unsigned short p_usVendorId,
    unsigned short p_usProductId,
    char* p_pManufacturer,
    unsigned int p_ulManufacturerLength,
    char* p_pDescription,
    unsigned int p_ulDescriptionLength,
    char* p_pSerialNumber,
    unsigned int p_ulSerialNumberLength,
    bool p_oUseVcpDriver)
{
    const unsigned int BUFFER_SIZE(64);

    FT_PROGRAM_DATA ftData;
    TCHAR ManufacturerBuf[BUFFER_SIZE];
    TCHAR ManufacturerIdBuf[BUFFER_SIZE];
    TCHAR DescriptionBuf[BUFFER_SIZE];
    TCHAR SerialNumberBuf[BUFFER_SIZE];
    BOOL oResult(FALSE);
    FT_STATUS ftStatus = FT_OK;

    //ResetErrorInfo
    //m_ErrorInfo.Reset();

    //Prepare Structure
    ftData.Signature1 = 0x00000000;
    ftData.Signature2 = 0xffffffff;
    ftData.Version = 0x00000002; // EEPROM structure with FT232R extensions
    ftData.Manufacturer = ManufacturerBuf;
    ftData.ManufacturerId = ManufacturerIdBuf;
    ftData.Description = DescriptionBuf;
    ftData.SerialNumber = SerialNumberBuf;

    //Read EE
    if(FT_OK == (ftStatus = FT_EE_Read(m_Handle, &ftData)))
    {
        //VendorId
        ftData.VendorId = p_usVendorId;

        //ProductId
        ftData.ProductId = p_usProductId;

        //Manufacturer
        if(p_ulManufacturerLength < BUFFER_SIZE)
        {
            memset(ManufacturerBuf, 0, BUFFER_SIZE*sizeof(char));
            if(p_pManufacturer) strcpy(ManufacturerBuf, p_pManufacturer);
        }

        //Description
        if(p_ulDescriptionLength < BUFFER_SIZE)
        {
            memset(DescriptionBuf, 0, BUFFER_SIZE*sizeof(TCHAR));
            if(p_pDescription) strcpy(DescriptionBuf, p_pDescription);
        }

        //SerialNumber
        if(p_ulSerialNumberLength < BUFFER_SIZE)
        {
            memset(SerialNumberBuf, 0, BUFFER_SIZE*sizeof(TCHAR));
            if(p_pSerialNumber) strcpy(SerialNumberBuf, p_pSerialNumber);
        }

        //UseVcpDriver
        if(p_oUseVcpDriver) ftData.RIsD2XX = 0; else ftData.RIsD2XX = 1;

        //Program EE
        if(FT_OK == (ftStatus = FT_EE_Program(m_Handle, &ftData)))
            oResult = TRUE;
        else
        {
            std::string errormsg = GetFtStatusDescription(ftStatus);
            errormsg += ":FT_EE_Program";
            perror(errormsg.c_str());

            oResult = FALSE;
        }
    }
    else
    {
        std::string errormsg = GetFtStatusDescription(ftStatus);
        errormsg += ":FT_EE_Read";
        perror(errormsg.c_str());

        oResult = FALSE;
    }

    return oResult;
}
