#include <fstream>
#include <sstream>
#include "../../../../include/Drv/Helper/UsbTree/LinuxUsbDirectory.h"
#include "../../../../include/Drv/Helper/UsbTree/LinuxUsbDevice.h"
#include "../../../../include/Drv/Helper/UsbTree/LinuxUsbTree.h"
#include <Drv/Helper/UsbApi/UsbDeviceDescription.h>

CLinuxUsbDevice::CLinuxUsbDevice()
{
    Reset();
}

CLinuxUsbDevice::~CLinuxUsbDevice()
{
    DeleteChildren();
}

void CLinuxUsbDevice::Reset()
{
	m_pUsbTree = 0;
    m_pDirectory = 0;
    m_DirectoryList.clear();
    m_lBusNum = 0;
    m_lPortNum = 0;
    m_lDevNum = 0;
    m_DevPath = "";
    m_lIdProduct = 0;
    m_lIdVendor = 0;
    m_Manufacturer = "";
    m_Product = "";
    m_Version = "";
    m_SerialNumber = "";
    m_SerialNumberId = 0;
    m_DevicePath = "";
    
    DeleteChildren();
}

void CLinuxUsbDevice::Init(CLinuxUsbTree* p_pUsbTree, std::list<CLinuxUsbDirectory*> p_DirectoryList, std::list<CUsbDeviceDescription*> p_DeviceInfoList, CLinuxUsbDirectory* p_pDirectory)
{
	m_pUsbTree = p_pUsbTree;
    m_pDirectory = p_pDirectory;
    m_DirectoryList = p_DirectoryList;
    m_DeviceInfoList = p_DeviceInfoList;
}

bool CLinuxUsbDevice::Load()
{
    bool oResult = false;
    
    oResult = UpdateDeviceClass();
    
    if(oResult)
    {
        oResult = UpdateBusNum();    
    }
    if(oResult)
    {
        oResult = UpdateDevNum();
    }
    if(oResult)
    {
        oResult = UpdateDevPath();
    }
    if(oResult)
    {
        oResult = UpdateProductId();    
    }
    if(oResult)
    {
        oResult = UpdateVendorId();    
    }
    if(oResult)
    {
        oResult = UpdateManufacturer();    
    }
    if(oResult)
    {
        oResult = UpdateProduct();    
    }
        
    if(oResult)
    {
        oResult = UpdateVersion();    
    }

    if(oResult)
	{
		oResult = UpdateSerialNumber();
	}

    if(oResult)
	{
		oResult = UpdateSerialNumberId();
	}

    if(oResult)
	{
		oResult = UpdatePortNum();
	}

    if(oResult)
	{
		oResult = UpdateDevicePath();
	}

    if(oResult && GetDeviceClass() == LIBUSB_CLASS_HUB)
    {
        oResult = LoadChildren();
    }    
    
    if(oResult)
    {
        m_pDirectory->SetParsed(true);    
    }
    
    return oResult;
}
    
std::list<CLinuxUsbDevice*> CLinuxUsbDevice::GetChildren()
{
    return m_Children;
}
    
void CLinuxUsbDevice::DeleteChildren()
{
    std::list<CLinuxUsbDevice*>::iterator it = m_Children.begin();
    
    while(it!=m_Children.end())
    {
        CLinuxUsbDevice* pDevice = (*it);
        delete pDevice;
        m_Children.erase(it);
        it = m_Children.begin();
    }
}
    
bool CLinuxUsbDevice::LoadChildren()
{
    bool oResult = true;
    std::list<CLinuxUsbDirectory*> childDirs;
        
    oResult = GetChildDirectories(childDirs);
    
    for(std::list<CLinuxUsbDirectory*>::iterator it=childDirs.begin(); it!=childDirs.end(); it++)
    {
        CLinuxUsbDirectory* pDirectory = (*it);
        
        if(!pDirectory->IsParsed())
        {        
            CLinuxUsbDevice* pDevice = new CLinuxUsbDevice();
        
            pDevice->Init(m_pUsbTree, childDirs, m_DeviceInfoList, pDirectory);
            
            if(pDevice->Load() && !pDevice->DeviceExists())
            {            
                m_Children.push_back(pDevice);
            }
            else
            {      
                delete pDevice;
            }
        }
    }
    
    return oResult;
}

CLinuxUsbDirectory* CLinuxUsbDevice::GetDirectory()
{
    return m_pDirectory;
}

bool CLinuxUsbDevice::UpdateDeviceClass()
{    
    libusb_class_code eResult = LIBUSB_CLASS_VENDOR_SPEC;
    int lDeviceClass = LIBUSB_CLASS_VENDOR_SPEC;
    bool oResult = false;
    
    if(ReadIntValue("bDeviceClass", lDeviceClass))
    {    
        eResult = (libusb_class_code)lDeviceClass;        
        m_eDeviceClass = eResult;
        oResult = true;
    }    
    
    return oResult;
}

libusb_class_code CLinuxUsbDevice::GetDeviceClass()
{    
    return m_eDeviceClass;
}

bool CLinuxUsbDevice::ReadIntValue(CStdString p_Identifier, int& p_rlValue)
{
    bool oResult = false;
    CStdString fileName;
    std::ifstream file;
    CLinuxUsbDirectory* pDirectory = GetDirectory();
        
    fileName.Format("%s/%s", pDirectory->GetFileName(), p_Identifier);    
    
    file.open(fileName);
    
    if(file.is_open())
    {
        file >> p_rlValue;    
        oResult = true;
    }
    
    return oResult;
}

bool CLinuxUsbDevice::ReadStringValue(CStdString p_Identifier, CStdString& p_rValue)
{
    bool oResult = false;
    CStdString fileName;
    std::ifstream file;
    std::string value;
    
    fileName.Format("%s/%s", GetDirectory()->GetFileName(), p_Identifier);    
    
    file.open(fileName);
    
    if(file.is_open())
    {
        std::getline(file, value);
        p_rValue = value;
        oResult = true;
    }
    
    return oResult;
}

int CLinuxUsbDevice::GetBusNum()
{
    return m_lBusNum;
}

int CLinuxUsbDevice::GetPortNum()
{
    return m_lPortNum;
}

bool CLinuxUsbDevice::UpdateBusNum()
{
    bool oResult = false;
    int lValue = 0;
    
    if(ReadIntValue("busnum", lValue))
    {
        m_lBusNum = lValue;
        oResult = true;
    }
    
    return oResult;
}

bool CLinuxUsbDevice::UpdatePortNum()
{
    CStdString value;
	bool oResult = true; //optional
	CUsbDeviceDescription* pDeviceDescription = 0;

	if(FindMatchingDeviceDescription(pDeviceDescription) && pDeviceDescription)
	{
		m_lPortNum = pDeviceDescription->port_number;
		oResult = true;
	}

	return oResult;
}

bool CLinuxUsbDevice::UpdateDevNum()
{
    bool oResult = false;
    int lValue = 0;
    
    if(ReadIntValue("devnum", lValue))
    {
        m_lDevNum = lValue;
        oResult = true;
    }
    
    return oResult;
}

int CLinuxUsbDevice::GetDevNum()
{
    return m_lDevNum;
}

bool CLinuxUsbDevice::UpdateDevPath()
{
    CStdString value;
    bool oResult = false;
    
    if(ReadStringValue("devpath", value))
    {
        m_DevPath = value;
        oResult = true;
    }
    
    return oResult;
}

bool CLinuxUsbDevice::FindMatchingDeviceDescription(CUsbDeviceDescription*& p_rpDeviceDescription)
{
	bool oResult = false;

	for(std::list<CUsbDeviceDescription*>::iterator it=m_DeviceInfoList.begin(); it!=m_DeviceInfoList.end(); it++)
	{
		CUsbDeviceDescription* pDeviceDecription = (*it);

		if(pDeviceDecription->bus_number == m_lBusNum && pDeviceDecription->device_id == m_lDevNum)
		{
			p_rpDeviceDescription = pDeviceDecription;
			oResult = true;
			break;
		}
	}

	return oResult;
}

bool CLinuxUsbDevice::UpdateDevicePath()
{
    CStdString value;
    bool oResult = true;

    CUsbDeviceDescription* pDeviceDescription = 0;

	if(FindMatchingDeviceDescription(pDeviceDescription) && pDeviceDescription)
	{
		m_DevicePath = pDeviceDescription->path;
		oResult = true;
	}

    return oResult;
}

bool CLinuxUsbDevice::UpdateSerialNumber()
{
    bool oResult = true; //optional
    CUsbDeviceDescription* pDeviceDescription = 0;

    if(FindMatchingDeviceDescription(pDeviceDescription) && pDeviceDescription)
	{
    	m_SerialNumber = pDeviceDescription->serial_number_string;
    	oResult = true;
	}

    return oResult;
}

bool CLinuxUsbDevice::UpdateSerialNumberId()
{
    bool oResult = true; //optional
    CUsbDeviceDescription* pDeviceDescription = 0;

    if(FindMatchingDeviceDescription(pDeviceDescription) && pDeviceDescription)
	{
    	m_SerialNumberId = pDeviceDescription->serial_number;
    	oResult = true;
	}

    return oResult;
}


CStdString CLinuxUsbDevice::GetDevPath()
{
    return m_DevPath;
}

CStdString CLinuxUsbDevice::GetDevicePath()
{
    return m_DevicePath;
}

int CLinuxUsbDevice::GetProductId()
{    
    return m_lIdProduct;
}

bool CLinuxUsbDevice::UpdateProductId()
{    
    bool oResult = false;
    CStdString value;
    std::stringstream convert;
    
    if(ReadStringValue("idProduct", value))
    {        
        convert << std::hex << value;
        convert >> m_lIdProduct;        
        oResult = true;
    }
    
    return oResult;
}

int CLinuxUsbDevice::GetVendorId()
{
    return m_lIdVendor;
}

bool CLinuxUsbDevice::UpdateVendorId()
{
    bool oResult = false;
    CStdString value;
    std::stringstream convert;
    
    if(ReadStringValue("idVendor", value))
    {        
        convert << std::hex << value;
        convert >> m_lIdVendor;
        oResult = true;
    }
    
    return oResult;
}

CStdString CLinuxUsbDevice::GetManufacturer()
{    
    return m_Manufacturer;
}

CStdString CLinuxUsbDevice::GetSerialNumber()
{
    return m_SerialNumber;
}

int CLinuxUsbDevice::GetSerialNumberId()
{
    return m_SerialNumberId;
}

bool CLinuxUsbDevice::UpdateManufacturer()
{   
    bool oResult = true; //optional
    CStdString value;
    
    if(ReadStringValue("manufacturer", value))
    {
        m_Manufacturer = value;
        oResult = true;
    }
    
    return oResult;
}

CStdString CLinuxUsbDevice::GetProduct()
{
    return m_Product;
}

bool CLinuxUsbDevice::UpdateProduct()
{
    bool oResult = true; //optional
    CStdString value;
    
    if(ReadStringValue("product", value))
    {
        m_Product = value;
        oResult = true;
    }
    
    return oResult;
}

CStdString CLinuxUsbDevice::GetVersion()
{
    return m_Version;
}

bool CLinuxUsbDevice::UpdateVersion()
{
    bool oResult = false;
    CStdString value;
    
    if(ReadStringValue("version", value))
    {
        m_Version = value;
        oResult = true;
    }
            
    return oResult;
}

bool CLinuxUsbDevice::DeviceExists()
{
	bool oResult = false;

	if(m_pUsbTree)
	{
		oResult = m_pUsbTree->DeviceExists(m_lBusNum, m_lPortNum, m_DevicePath, m_lIdProduct, m_lIdVendor);
	}

	return oResult;
}

bool CLinuxUsbDevice::DeviceExists(int p_lBusNum, int p_lPortNum, CStdString p_DevicePath, int p_lIdProduct, int p_lIdVendor)
{
	bool oResult = false;

	if(p_lBusNum == m_lBusNum &&
		p_lPortNum == m_lPortNum &&
		p_DevicePath == m_DevicePath &&
		p_lIdProduct == m_lIdProduct &&
		p_lIdVendor == m_lIdVendor)
	{
		oResult = true;
	}
	else
	{
		for(std::list<CLinuxUsbDevice*>::iterator it = m_Children.begin(); it!=m_Children.end(); it++)
		{
			CLinuxUsbDevice* pChild = (*it);

			oResult = pChild->DeviceExists(p_lBusNum, p_lPortNum, p_DevicePath, p_lIdProduct, p_lIdVendor);

			if(oResult)
			{
				break;
			}
		}
	}

	return oResult;
}

bool CLinuxUsbDevice::GetChildDirectories(std::list<CLinuxUsbDirectory*>& p_rChildDirs)
{
    bool oResult = true;
    
    CStdString directoryName = m_pDirectory->GetFileName();
    CStdString directoryTargetName = m_pDirectory->GetTargetFileName();
    
    for(std::list<CLinuxUsbDirectory*>::iterator it=m_DirectoryList.begin(); it!=m_DirectoryList.end(); it++)
    {
        CLinuxUsbDirectory* pUsbDir = (*it);        
        
        if(pUsbDir->GetFileName().Find(directoryName) != -1 && 
            pUsbDir->GetFileName() != directoryName)
        {
            p_rChildDirs.push_back(pUsbDir);
        }
        else if(pUsbDir->GetTargetFileName().Find(directoryTargetName) != -1 && 
                pUsbDir->GetTargetFileName() != directoryTargetName)
        {
            p_rChildDirs.push_back(pUsbDir);
        }
    }
    
    return oResult;
}
