#include "stdafx.h"
#include <Drv/Helper/UsbTree/UsbHostController.h>
#include <Drv/Helper/UsbTree/UsbDevices.h>
#include <Drv/Helper/UsbTree/UsbDevice.h>
#include <Drv/Helper/UsbTree/UsbDeviceBase.h>
#include <Drv/Helper/UsbTree/UsbTree.h>
#include <Drv/Helper/UsbApi/UsbApiLinux.h>
#include <Drv/Helper/UsbTree/UsbTreeFactoryLinux.h>
#include "../../../../include/Drv/Helper/UsbTree/LinuxRootHub.h"
#include "../../../../include/Drv/Helper/UsbTree/LinuxUsbTree.h"
#include "../../../../include/Drv/Helper/UsbTree/LinuxUsbHub.h"

CUsbTreeFactoryBase::CUsbTreeFactoryBase()
{
}

CUsbTreeFactoryBase::~CUsbTreeFactoryBase()
{
	DeleteUsbDevices();
}

BOOL CUsbTreeFactoryBase::LoadHostControllers(std::list<CUsbHostController*>& p_rHostControllers)
{
	BOOL oResult(FALSE);

	oResult = EnumerateHostControllers(p_rHostControllers);

	return oResult;
}

BOOL CUsbTreeFactoryBase::EnumerateHostControllers(std::list<CUsbHostController*>& p_rHostControllers)
{
    BOOL oResult(TRUE);

	CUsbHostController* pUsbHostController = new CUsbHostController();

	oResult = EnumerateRootHubs(pUsbHostController);

	if(oResult)
	{
		p_rHostControllers.push_back(pUsbHostController);
	}
	else
	{
		delete pUsbHostController;
	}

    return oResult;
}

BOOL CUsbTreeFactoryBase::EnumerateRootHubs(CUsbHostController* p_pHostController)
{
    BOOL oResult(FALSE);
    CLinuxUsbHub* pUsbHub(0);
    CLinuxUsbTree tree;
	std::list<CLinuxRootHub*> rootHubs;

	tree.GetRootHubs(rootHubs);

	for(std::list<CLinuxRootHub*>::iterator it=rootHubs.begin(); it!=rootHubs.end(); it++)
	{
		CLinuxRootHub* pRootHub = (*it);

		if(LoadRootHub(p_pHostController, pRootHub, pUsbHub) && pUsbHub)
		{
			p_pHostController->AddChild(pUsbHub);
			oResult = TRUE;
		}
	}

    return oResult;
}

BOOL CUsbTreeFactoryBase::LoadChildDevices(CUsbDeviceBase* p_pParent, CLinuxUsbDevice* p_pUsbDeviceInfo)
{
	BOOL oResult(TRUE);

	std::list<CLinuxUsbDevice*> childNodes;

	if(p_pUsbDeviceInfo)
	{
		childNodes = p_pUsbDeviceInfo->GetChildren();

		for(std::list<CLinuxUsbDevice*>::iterator it=childNodes.begin(); it!=childNodes.end(); it++)
		{
			CLinuxUsbDevice* pDeviceInfo = (*it);

			if(pDeviceInfo->GetDeviceClass() == LIBUSB_CLASS_HUB)
			{
				CLinuxUsbHub* pUsbHub = new CLinuxUsbHub(FALSE);
				CStdString devicePath = pDeviceInfo->GetDevicePath();
				int lPortNum = pDeviceInfo->GetPortNum();

				pUsbHub->InitParent(p_pParent);
				pUsbHub->InitDevicePath(devicePath);
				pUsbHub->InitIndex(lPortNum);

				oResult = LoadChildDevices(pUsbHub, pDeviceInfo);

				p_pParent->AddChild(pUsbHub);
			}
			else
			{
				CUsbDevice* pUsbDevice = new CUsbDevice();

				pUsbDevice->InitParent(p_pParent);

				int lPortNum = pDeviceInfo->GetPortNum();
				CStdString devicePath = pDeviceInfo->GetDevicePath();
				int lVendorId = pDeviceInfo->GetVendorId();
				int lProductId = pDeviceInfo->GetProductId();
				int lSerialNumberId = pDeviceInfo->GetSerialNumberId();
				CStdString serialNumber = pDeviceInfo->GetSerialNumber();

				pUsbDevice->Init(lPortNum, devicePath, lVendorId, lProductId, lSerialNumberId, serialNumber);

				oResult = LoadChildDevices(pUsbDevice, pDeviceInfo);

				p_pParent->AddChild(pUsbDevice);
			}
		}
	}

	return oResult;
}

BOOL CUsbTreeFactoryBase::LoadRootHub(CUsbHostController* p_pHostController, CLinuxRootHub* p_pRootHub, CLinuxUsbHub*& p_pUsbHub)
{
	BOOL oResult(FALSE);

	if(p_pRootHub)
	{
		p_pUsbHub = new CLinuxUsbHub(TRUE);

		CStdString devicePath = p_pRootHub->GetDevicePath();
		DWORD ulBusNum = p_pRootHub->GetBusNum();
		DWORD ulBusCount = 0;

		p_pUsbHub->InitParent(p_pHostController);
		p_pUsbHub->Init(ulBusNum, devicePath, ulBusCount);

		oResult = LoadChildDevices(p_pUsbHub, p_pRootHub);

		if(!oResult)
		{
			delete p_pUsbHub;
			p_pUsbHub = 0;
		}
	}

	return oResult;
}

BOOL CUsbTreeFactoryBase::EnumerateDevices(std::list<CUsbDevice*>& p_rUsbDevices)
{
    BOOL oResult(TRUE);

	return oResult;
}

BOOL CUsbTreeFactoryBase::LoadUsbDevices(std::list<CUsbDevice*>& p_rUsbDevices)
{
	BOOL oResult(FALSE);

	oResult = EnumerateDevices(p_rUsbDevices);

	return oResult;
}
