#include <stdio.h>
#include <memory.h>
#include <unistd.h>
#include <algorithm>
#include <dirent.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <list>

#include <net/if.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <linux/can.h>
#include <linux/can/raw.h>
#include <linux/can/error.h>
#include <linux/can/netlink.h>

#include <Process/MmcProcess.h>
#include <Time/MmcHiResTimer.h>

#include <Drv/SocketCAN/MmcSocketCANHndlLinux.h>

using namespace std;

CMmcSocketCANHndlBase::CMmcSocketCANHndlBase()
{
	m_lChannelHandle = 0;
	m_ulBaudrate = 1000000;
	m_ulTimeout = 500;
	m_ulCobIdFilter = 0;
	m_lRtrFilter = 0;
}

CMmcSocketCANHndlBase::~CMmcSocketCANHndlBase()
{
}

bool CMmcSocketCANHndlBase::GetCanPortState(CStdString p_CanPort, CStdString& p_rState, int& p_rBitrate)
{
	bool oResult = false;

	const int BUFFER_SIZE = 1024;
	const char STATE_TOKEN[] = "state";
	const char BITRATE_TOKEN[] = "bitrate";
	CStdString command;
	FILE *pOutput = 0;
	char* pBuffer = new char[BUFFER_SIZE];
	char* pToken = 0;
	int stateFound = 0;
	int bitrateFound = 0;

	command.Format("ip -details link show %s", p_CanPort);

	pOutput = popen(command, "r");

	if (pOutput)
	{
		memset(pBuffer, 0, BUFFER_SIZE);

		while(fgets(pBuffer, BUFFER_SIZE-1, pOutput))
		{
			pToken = strtok (pBuffer," \r\n\t");

			while (pToken != NULL)
			{
				if(stateFound == 1)
				{
					p_rState = pToken;
					stateFound++;
				}

				if(bitrateFound == 1)
				{
					p_rBitrate = atoi(pToken);
					bitrateFound++;
				}

				if(strcmp(pToken, STATE_TOKEN) == 0)
				{
					stateFound++;
				}
				if(strcmp(pToken, BITRATE_TOKEN) == 0)
				{
					bitrateFound++;
				}

				pToken = strtok (NULL, " \r\n");
			}
		}

		pclose(pOutput);
		oResult = true;
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ReadCanPortOperState(CStdString p_PortName, CStdString& p_rOperstate)
{
	bool oResult = false;
	CStdString filename;
	long lFileSize = 0;
	FILE* pFile = 0;
	char* pBuffer = 0;
	::size_t result = 0;

	filename.Format("/sys/class/net/%s/operstate", p_PortName);

	pFile = fopen(filename, "r");

	if(pFile)
	{
		if(fseek(pFile, 0, SEEK_END) == 0)
		{
			lFileSize = ftell(pFile);

			if(lFileSize > 0)
			{
				pBuffer = new char[lFileSize + 1];

				memset(pBuffer, 0, lFileSize + 1);

				rewind(pFile);

				result = fread(pBuffer, sizeof(char), lFileSize, pFile);
				
				if(result == (::size_t)lFileSize)
				{
					oResult = true;
				}
				else if(feof(pFile))
				{
					oResult = true;
				}
				
				p_rOperstate = pBuffer;

				delete[] pBuffer;

				if(p_rOperstate.GetAt(p_rOperstate.GetLength()-1) == '\n')
				{
					p_rOperstate.GetBufferSetLength(p_rOperstate.GetLength()-1);
				}
			}
		}

		fclose(pFile);
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::IsCanPortUp(CStdString p_PortName)
{
	bool oResult = false;
	const CStdString STATE_DOWN = "down";
	CStdString state;

	if(ReadCanPortOperState(p_PortName, state))
	{
		if(state != STATE_DOWN)
		{
			oResult = true;
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::EnumAvailableCanPorts(list<CStdString>& p_rCanPortList)
{
	bool oResult = false;

	DIR* pDirectory = 0;
	dirent* pDirEntry = 0;
	CStdString dirName;
	CStdString canPortNamePrefix = "CAN";

	if((pDirectory = opendir("/sys/class/net")) != NULL)
	{
		oResult = true;

		while((pDirEntry = readdir(pDirectory)) != NULL && oResult)
		{
			dirName = pDirEntry->d_name;
			dirName.MakeUpper();

			if(dirName.Mid(0, canPortNamePrefix.GetLength()) == canPortNamePrefix)
			{
				p_rCanPortList.push_back(pDirEntry->d_name);
			}
		}

		closedir(pDirectory);
	}

	p_rCanPortList.sort();

	return oResult;
}

bool CMmcSocketCANHndlBase::CreateSocketCanPorts(list<CSocketCANPort*>& p_rCanPortList)
{
	bool oResult = false;
	list<CStdString> canInterfaceList;
	CSocketCANPort* pCanPort = 0;

	if(EnumAvailableCanPorts(canInterfaceList))
	{
		oResult = true;

		for(list<CStdString>::iterator it = canInterfaceList.begin(); it != canInterfaceList.end() && oResult; it++)
		{
			pCanPort = new CSocketCANPort(*it);

			oResult = pCanPort->Init();

			p_rCanPortList.push_back(pCanPort);
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ClearCanPorts(list<CSocketCANPort*>& p_rCanPortList)
{
	bool oResult = true;
	list<CSocketCANPort*>::iterator it = p_rCanPortList.begin();
	CSocketCANPort* pCANPort = 0;

	while(it != p_rCanPortList.end())
	{
		pCANPort = (*it);
		delete pCANPort;
		p_rCanPortList.erase(it);
		it = p_rCanPortList.begin();
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::SearchDeviceInfoByAddress(tSocketCANDeviceInfoList& p_rCANDeviceInfoList, CStdString p_Address, CSocketCANDeviceInfo*& p_rpCanDeviceInfo)
{
	bool oResult = false;
	CSocketCANDeviceInfo* pDeviceInfo = 0;

	if(p_rpCanDeviceInfo)
	{
		for(tSocketCANDeviceInfoList::iterator it = p_rCANDeviceInfoList.begin(); it != p_rCANDeviceInfoList.end(); it++)
		{
			pDeviceInfo = (*it);

			if(pDeviceInfo->GetDeviceAddress() == p_Address)
			{
				p_rpCanDeviceInfo = pDeviceInfo;
				oResult = true;
				break;
			}
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::GetDeviceCountByDriverName(tSocketCANDeviceInfoList& p_rCANDeviceInfoList, CStdString p_DriverName, DWORD& p_rDeviceCount)
{
	bool oResult = true;
	CSocketCANDeviceInfo* pDeviceInfo = 0;

	p_rDeviceCount = 0;

	for(tSocketCANDeviceInfoList::iterator it = p_rCANDeviceInfoList.begin(); it != p_rCANDeviceInfoList.end(); it++)
	{
		pDeviceInfo = (*it);

		if(pDeviceInfo->GetDriverName() == p_DriverName)
		{
			p_rDeviceCount++;
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ConvertToCanDeviceInfoList(tCanPortList& p_rCanPortList, tSocketCANDeviceInfoList& p_rCANDeviceInfoList)
{
	bool oResult = false;

	const CStdString PREFIX_INTERFACE_NAME(_T("CAN_"));

	CSocketCANPort* pPort = 0;
	CSocketCANDevice* pDevice = 0;
	CSocketCANDeviceInfo* pDeviceInfo = 0;
	DWORD lIndex = 0;

	for(list<CSocketCANPort*>::iterator it = p_rCanPortList.begin(); it != p_rCanPortList.end(); it++)
	{
		pPort = (*it);
		pDevice = pPort->GetDevice();

		if(pDevice)
		{
			if(SearchDeviceInfoByAddress(p_rCANDeviceInfoList, pDevice->GetAddress(), pDeviceInfo) && pDeviceInfo)
			{
				pDeviceInfo->AddPort(pPort->GetName());
			}
			else
			{
				pDeviceInfo = new CSocketCANDeviceInfo();

				pDeviceInfo->SetDriverName(pDevice->GetDriver());
				pDeviceInfo->SetInterfaceName(pDevice->GetInterface());

				GetDeviceCountByDriverName(p_rCANDeviceInfoList, pDevice->GetDriver(), lIndex);

				pDeviceInfo->SetInterfaceIndex(lIndex);
				pDeviceInfo->SetDeviceAddress(pDevice->GetAddress());
				pDeviceInfo->AddPort(pPort->GetName());

				p_rCANDeviceInfoList.push_back(pDeviceInfo);

				oResult = true;
			}
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::GetDeviceInfos(tSocketCANDeviceInfoList& p_rDeviceInfoList)
{
	bool oResult = false;
	tCanPortList canPortList;

	ResetLastError();

	if(CreateSocketCanPorts(canPortList))
	{
		oResult = ConvertToCanDeviceInfoList(canPortList, p_rDeviceInfoList);

		ClearCanPorts(canPortList);
	}

    return oResult;
}

bool CMmcSocketCANHndlBase::SetCanPortBitrate(CStdString p_PortName)
{
	bool oResult = false;
	const CStdString STATE_DOWN = "down";
	const CStdString STATE_UP = "up";
	CStdString command;	
	int lBaudrate = 0;
	CStdString state;
	int lResult = 0;
	bool oCanPortStateChanged = false;
	
	oResult = GetCanPortState(p_PortName, state, lBaudrate);
	
	if(oResult && m_ulBaudrate != (DWORD)lBaudrate)
	{			
		if(UserHasSufficientAccessRights())
		{	
			if(state != STATE_DOWN)
			{
				oResult = SetCanPortState(p_PortName, STATE_DOWN);

				oCanPortStateChanged = oResult;
			}

			if(oResult)
			{
				command.Format("timeout 2 sudo ip link set %s type can bitrate %d", p_PortName, m_ulBaudrate);

				lResult = system(command);

				oResult = (lResult != -1);
			}
			
			if(oCanPortStateChanged)
			{
				oResult = SetCanPortState(p_PortName, STATE_UP);
			}
		}
		else
		{
			m_eLastError = DEC_ACCESS_DENIED;
			oResult = false;
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::UserHasSufficientAccessRights()
{
	bool oResult = false;
	int lResult = -1;
	CStdString command;

	command = "sudo -n true >/dev/null 2>&1";

	lResult = system(command);

	if(lResult == 0)
	{
		oResult = true;
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::SetCanPortState(CStdString p_PortName, CStdString p_State)
{
	bool oResult = false;
	CStdString command;
	int lResult = 0;

	if(UserHasSufficientAccessRights())
	{		
		command.Format("timeout 2 sudo ip link set %s %s", p_PortName, p_State);

		lResult = system(command);
		
		oResult = (lResult != -1);
	}
	else
	{
		m_eLastError = DEC_ACCESS_DENIED;
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::OpenInterface(CSocketCANDeviceInfo& p_rInfo)
{
	bool oResult(false);
	CStdStringArray portList;
	CStdString state;

	ResetLastError();

	m_interfaceName = p_rInfo.GetInterfaceName();

	if(!m_interfaceName.IsEmpty())
	{
		oResult = true;

		portList = p_rInfo.GetPhysicalPortNames();

		for(CStdStringArray::iterator it = portList.begin(); it != portList.end(); it++)
		{
			if(!IsCanPortUp(*it))
			{
				oResult = SetCanPortBitrate(*it);

				if(oResult)
				{
					oResult = SetCanPortState(*it, "up");
				}
			}
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::CloseInterface()
{
	bool oResult(true);

	ResetLastError();

	m_interfaceName = _T("");
	m_lChannelHandle = -1;

	return oResult;
}

bool CMmcSocketCANHndlBase::OpenPort(CStdString p_portName)
{
	bool oResult = true;
	struct ifreq ifr;
	struct sockaddr_can addr;
	timeval timeout;

	ResetLastError();

	if(!IsCanPortUp(p_portName))
	{
		oResult = SetCanPortBitrate(p_portName);
	}

	if(oResult)
	{
		memset(&ifr, 0, sizeof(ifr));
		memset(&addr, 0, sizeof(addr));

		m_lChannelHandle = socket(PF_CAN, SOCK_RAW, CAN_RAW);

		if(m_lChannelHandle != -1)
		{
			strcpy(ifr.ifr_name, p_portName);

			ioctl(m_lChannelHandle, SIOCGIFINDEX, &ifr);

			addr.can_ifindex = ifr.ifr_ifindex;
			addr.can_family = PF_CAN;

			timeout.tv_sec = 0;
			timeout.tv_usec = m_ulTimeout * 1000;

			if(setsockopt(m_lChannelHandle, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout)) < 0)
			{
				oResult = false;
			}

			if(oResult && bind(m_lChannelHandle, (struct sockaddr*)&addr, sizeof(addr)) != -1)
			{
				m_ActivePortName = p_portName;
			}
			else
			{
				m_ActivePortName = "";
				close(m_lChannelHandle);
				m_lChannelHandle = 0;
			}
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ClosePort()
{
	bool oResult = false;

	ResetLastError();

	if(m_lChannelHandle != 0)
	{
		m_ActivePortName = "";
		close(m_lChannelHandle);
		oResult = true;
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::SetPortSettings(DWORD p_ulBaudrate, DWORD p_ulTimeout)
{
	bool oResult = true;

	ResetLastError();

	if(m_ulBaudrate != p_ulBaudrate)
	{
		m_ulBaudrate = p_ulBaudrate;
	}

	if(p_ulTimeout > 0)
	{
		m_ulTimeout = p_ulTimeout;
	}

	if(!m_ActivePortName.IsEmpty())
	{
		oResult = SetCanPortBitrate(m_ActivePortName);
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::GetPortSettings(DWORD* p_pulBaudrate, DWORD* p_pulTimeout)
{
	bool oResult = true;
	CStdString state;
	int lBitrate = 0;

	ResetLastError();

	if(p_pulTimeout)
	{
		*p_pulTimeout = m_ulTimeout;
	}

	if(!m_ActivePortName.IsEmpty())
	{
		oResult = GetCanPortState(m_ActivePortName, state, lBitrate);

		if(oResult)
		{
			m_ulBaudrate = lBitrate;
			if(p_pulBaudrate)
			{
				*p_pulBaudrate = lBitrate;
			}
		}
	}
	else
	{
		if(p_pulBaudrate)
		{
			*p_pulBaudrate = m_ulBaudrate;
		}
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::CheckStatusAndReset()
{
	bool oResult = true;

	ResetLastError();

	return oResult;
}

bool CMmcSocketCANHndlBase::ResetPort()
{
	bool oResult = true;

	ResetLastError();

	return oResult;
}

bool CMmcSocketCANHndlBase::SetCanFilter(DWORD p_ulCobIdFilter, int p_lRtrFilter)
{
	bool oResult = false;
	can_filter rfilter[1];

	rfilter[0].can_id = (canid_t)p_ulCobIdFilter;

	if(p_lRtrFilter)
	{
		rfilter[0].can_mask = (CAN_EFF_FLAG | CAN_RTR_FLAG | CAN_SFF_MASK);
	}
	else
	{
		rfilter[0].can_mask = (CAN_EFF_FLAG | CAN_SFF_MASK);
	}

	if(m_lChannelHandle)
	{
		oResult = (setsockopt(m_lChannelHandle, SOL_CAN_RAW, CAN_RAW_FILTER, &rfilter, sizeof(rfilter)) == 0);
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ReceiveFilteredCanFrame(DWORD p_ulCobIdFilter, int p_lRtrFilter, DWORD p_ulTimeout, DWORD* p_pulCobId, int* p_poRtr, unsigned char* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength)
{
	bool oResult = true;

	ResetLastError();

	if(p_ulCobIdFilter != m_ulCobIdFilter || p_lRtrFilter != m_lRtrFilter)
	{
		oResult = SetCanFilter(p_ulCobIdFilter, p_lRtrFilter);
	}

	if(oResult)
	{
		oResult = ReceiveCanFrame(p_ulTimeout, p_pulCobId, p_poRtr, p_pubDlc, p_ppDataBuffer, p_pulDataBufferLength);
	}

	return oResult;
}

bool CMmcSocketCANHndlBase::ReceiveCanFrame(DWORD p_ulTimeout, DWORD* p_pulCobId, int* p_plRtr, unsigned char* p_pubDlc, void** p_ppDataBuffer, DWORD* p_pulDataBufferLength)
{
	bool oResult = false;
	const int MAXDATASIZE = 128;
	can_frame frame;
	int lDataLength = 0;
	char frameBuffer[MAXDATASIZE];
	CMmcHiResTimer timer;

	ResetLastError();

	memset(&frame, 0, sizeof(frame));
	memset(frameBuffer, 0, MAXDATASIZE);

	if(m_lChannelHandle)
	{
		oResult = true;

		*p_pulDataBufferLength = 0;

		frame.can_dlc = *p_pulDataBufferLength;

		timer.start();

		lDataLength = read(m_lChannelHandle, frameBuffer, MAXDATASIZE);

		if(lDataLength == sizeof(frame))
		{
			memcpy(&frame, frameBuffer, sizeof(frame));

			*p_pulCobId = frame.can_id;
			*p_ppDataBuffer = (unsigned char*)malloc(frame.can_dlc);

			memset(*p_ppDataBuffer,0, frame.can_dlc);

			memcpy(*p_ppDataBuffer, &frame.data, frame.can_dlc);

			*p_pulDataBufferLength = frame.can_dlc;

			oResult = true;
		}
	}

	return oResult;
}

CStdString CMmcSocketCANHndlBase::GetInterfaceName()
{
	return m_interfaceName;
}

bool CMmcSocketCANHndlBase::TransmitCanFrame(DWORD p_ulCobId, int p_lRtr, unsigned char p_ubDlc, void* p_pDataBuffer, DWORD p_ulDataBufferLength)
{
	bool oResult = false;
	can_frame frame;

	ResetLastError();

	memset(&frame, 0, sizeof(frame));

	frame.can_id = p_ulCobId;

	if(p_ulDataBufferLength == sizeof(frame.data))
	{
		memcpy(&frame.data, p_pDataBuffer, p_ulDataBufferLength);

		frame.can_dlc = p_ubDlc;

		if(write(m_lChannelHandle, &frame, sizeof(frame)) >= 0)
		{
			oResult = true;
		}
		else
		{
			oResult = false;
		}
	}

	return oResult;
}
