#include <dirent.h>
#include <stddef.h>
#include <regex.h>
#include <Drv/SocketCAN/SocketCANDevice.h>

CSocketCANDevice::CSocketCANDevice()
{
}

CSocketCANDevice::~CSocketCANDevice()
{
}

CStdString CSocketCANDevice::GetCanPort()
{
	return m_CanPort;
}

CStdString CSocketCANDevice::GetAddress()
{
	return m_Address;
}

CStdString CSocketCANDevice::GetDriver()
{
	return m_Driver;
}

CStdString CSocketCANDevice::GetDriverType()
{
	return m_DriverType;
}

bool CSocketCANDevice::Init(CStdString p_CanPort)
{
	bool oResult = false;

	m_CanPort = p_CanPort;

	oResult = ReadDriverInfo(p_CanPort);

	if(oResult)
	{
		oResult = ReadAddressInfo(p_CanPort);
	}

	return oResult;
}

bool CSocketCANDevice::ReadDriverInfo(CStdString p_CanPort)
{
	bool oResult = false;
	DIR* pDirectory = 0;
	dirent* pDirEntry = 0;
	CStdString dirName;
	CStdString deviceDirName;
	int lSeparator = 0;

	deviceDirName.Format("/sys/class/net/%s/device/driver/module/drivers", p_CanPort);

	if((pDirectory = opendir(deviceDirName)) != NULL)
	{
		while((pDirEntry = readdir(pDirectory)) != NULL)
		{
			dirName = pDirEntry->d_name;

			lSeparator = dirName.Find(':');

			if(lSeparator != -1)
			{
				m_DriverPath = dirName;
				m_DriverType = dirName.Mid(0, lSeparator);
				m_Driver = dirName.Mid(lSeparator + 1, dirName.GetLength() - (lSeparator + 1));

				oResult = true;
				break;
			}
		}
		closedir(pDirectory);
	}
	else
	{
		m_DriverType = "direct";
		m_Driver = "generic";
		oResult = true;
	}

	return oResult;
}

bool CSocketCANDevice::IsDriverMatchingDevice(CStdString p_CanPort, CStdString p_AddressPath)
{
	bool oResult = false;
	DIR* pDirectory = 0;
	dirent* pDirEntry = 0;
	CStdString dirName;
	CStdString deviceDirName;

	deviceDirName.Format("/sys/class/net/%s/device/driver/%s/net", p_CanPort, p_AddressPath);

	if((pDirectory = opendir(deviceDirName)) != NULL)
	{
		while((pDirEntry = readdir(pDirectory)) != NULL)
		{
			dirName = pDirEntry->d_name;

			if(dirName == p_CanPort)
			{
				oResult = true;
				break;
			}
		}

		closedir(pDirectory);
	}

	return oResult;
}

CStdString CSocketCANDevice::GetInterface()
{
	CStdString interfaceName;
	CStdString driver = GetDriver();

	interfaceName.Format("CAN_%s", driver);

	return interfaceName;
}

bool CSocketCANDevice::SearchAddressDir(CStdString p_RootDir, CStdString p_DirToFind, CStdString p_SkipDir, CStdString& p_rResultDir)
{
	bool oResult = false;
	DIR* pDirectory = 0;
	dirent* pDirEntry = 0;
	CStdString dirName;
	CStdString rootDir;

	if((pDirectory = opendir(p_RootDir)) != NULL)
	{
		while((pDirEntry = readdir(pDirectory)) != NULL)
		{
			dirName = pDirEntry->d_name;

			if(dirName == p_DirToFind)
			{
				oResult = true;
				break;
			}
			else if((pDirEntry->d_type == DT_DIR || pDirEntry->d_type == DT_LNK)
					&& dirName != "."  && dirName != "..")
			{
				rootDir.Format("%s/%s/driver/module/drivers", p_RootDir, dirName);

				if(rootDir != p_SkipDir)
				{
					oResult = SearchAddressDir(rootDir, p_DirToFind, p_SkipDir, p_rResultDir);

					if(oResult)
					{
						p_rResultDir = dirName;
						break;
					}
				}
			}
		}

		closedir(pDirectory);
	}

	return oResult;
}

bool CSocketCANDevice::ReadAddressInfo(CStdString p_CanPort)
{
	bool oResult = false;
	CStdString deviceDriverDirName;
	CStdString skipModuleDirName;
	CStdString fullPath;
	CStdString address;

	deviceDriverDirName.Format("/sys/class/net/%s/device/driver", p_CanPort);
	skipModuleDirName.Format("/sys/class/net/%s/device/driver/module", p_CanPort);

	if(SearchAddressDir(deviceDriverDirName, m_DriverPath, skipModuleDirName, address))
	{
		m_Address = address;

		oResult = true;
	}

	return oResult;
}
