#include <iostream>
#include <sstream>
#include <fstream>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdexcept>

#include <MmcTypeDefinition.h>
#include <Xml/rapidxml_print.hpp>
#include <Text/MmcTextEncoding.h>

#include <Storage/MmcUserSettingsLinux.h>

using namespace rapidxml;
using namespace std;

CMmcUserSettingsBase::CMmcUserSettingsBase()
    : m_pCurrentNode(0)
    , m_oCreateKey(false)
    , m_oParsed(false)
{
    Init();
}

CMmcUserSettingsBase::CMmcUserSettingsBase(CStdString p_Key, bool p_oCreateKey)
    : CMmcUserSettingsDef(p_Key, p_oCreateKey)
{
    m_pCurrentNode = 0;
    m_pParent = 0;
    m_oCreateKey = p_oCreateKey;
    m_oParsed = false;

    Init();
}

CMmcUserSettingsBase::CMmcUserSettingsBase(CMmcUserSettingsBase *p_pParent, CStdString p_Key, bool p_oCreateKey)
: CMmcUserSettingsDef(p_pParent, p_Key, p_oCreateKey)
{
    m_oParsed = false;
    m_pParent = p_pParent;
    m_pCurrentNode = 0;
    m_oCreateKey = p_oCreateKey;

    Init();
}

CMmcUserSettingsBase::~CMmcUserSettingsBase()
{
}

bool CMmcUserSettingsBase::Open(CStdString p_Key, bool p_oCreateKey)
{
    bool oResult = false;
    CStdString nodeName;

    oResult = Open();

    if(oResult)
    {
        nodeName = ToValidXmlNodeName(p_Key);

        if(SelectSingleNode(nodeName))
        {
            oResult = OpenKey(p_Key);
        }
        else if(p_oCreateKey)
        {   
            oResult = CreateKeyRecursive(p_Key);

            if(oResult)
            {
                oResult = Save();
            }
        }
    }

    return oResult;
}

CStdString CMmcUserSettingsBase::GetRegistryFileFullPath()
{
    CStdString result;

    result.Format(_T("%s/%s"), m_MmcDir, m_MmcRegistryFileName);

    return result;
}

bool CMmcUserSettingsBase::Open()
{
    bool oResult = false;

    CStdString filename = GetRegistryFileFullPath();

    EnsureMmcDirectoryExists();

    if(FileExists(filename))
    {
        oResult = LoadDocument(filename);
    }

    if(!oResult)
    {
        if(CreateDocument(filename))
        {
            oResult = LoadDocument(filename);
        }
    }

    return oResult;
}

bool CMmcUserSettingsBase::IsParsed()
{
    return m_oParsed;
}

bool CMmcUserSettingsBase::Save()
{
    bool oResult = false;
    CStdString filename = GetRegistryFileFullPath();
    string content;

    rapidxml::print(back_inserter(content), m_Doc, 0);

    ofstream file;

    file.open(filename);

    if(file.is_open())
    {
        file << content;

        file.close();

        oResult = true;
    }

    return oResult;
}

bool CMmcUserSettingsBase::CreateKey(CStdString p_Key)
{
    bool oResult = true;
    CStdStringA newNode = Unicode2Mbcs(p_Key);

    xml_node<char>* pLastChild = m_Doc.allocate_node(node_element, m_Doc.allocate_string(newNode));

    if(m_pCurrentNode != 0)
    {
        m_pCurrentNode->append_node(pLastChild);
    }
    else
    {
        m_Doc.append_node(pLastChild);
    }

    m_pCurrentNode = pLastChild;

    return oResult;
}



CStdString CMmcUserSettingsBase::ToValidXmlNodeName(CStdString p_NodeName)
{
    CStdString nodeName;
    const int FIRST_CHAR_INDEX = 0;

    if(p_NodeName.FindOneOf(_T("/\\")) != -1)
    {
        list<CStdString> subkeyList;

        if(GetSubkeyList(p_NodeName, subkeyList, false))
        {
            for(list<CStdString>::iterator it=subkeyList.begin(); it!=subkeyList.end(); it++)
            {
                nodeName += ToValidXmlNodeName(*it);
                nodeName += _T("/");
            }

            if(nodeName.GetLength() > 0 && nodeName[nodeName.GetLength()-1] == '/')
            {
                nodeName.GetBufferSetLength(nodeName.GetLength()-1);
            }
        }
    }
    else
    {
        nodeName = p_NodeName;

        if(nodeName.Find(_T(" ")) != -1)
        {
            nodeName.Replace(_T(" "), _T("_"));
        }

        if(nodeName.GetLength() > 0 && isdigit(nodeName[FIRST_CHAR_INDEX]))
        {
            nodeName.Format(_T("_%s"), nodeName);
        }
    }

    return nodeName;
}

bool CMmcUserSettingsBase::GetSubkeyList(CStdString p_Key, list<CStdString>& p_rSubkeyList, bool p_oValidate)
{
    bool oResult = true;
    TCHAR seps[]   = _T("/\\");
    TCHAR* pch = strtok(p_Key.GetBuffer(), seps);

    if(pch)
    {
        while(pch != 0)
        {
            CStdString node = pch;

            if(p_oValidate)
            {
                node = ToValidXmlNodeName(node);
            }

            p_rSubkeyList.push_back(node);

            pch = _tcstok(0, seps);
        }
    }

    p_Key.ReleaseBuffer();

    return oResult;
}

bool CMmcUserSettingsBase::CreateSubkeysIterative(std::list<CStdString> p_SubkeyList)
{
    bool oResult = true;
    CStdString path;

    for(list<CStdString>::iterator it = p_SubkeyList.begin(); it != p_SubkeyList.end() && oResult; it++)
    {
        path += (*it);

        if(!OpenKey(path))
        {
            oResult = CreateKey(*it);
        }

        path += _T("/");
    }

    return oResult;
}

bool CMmcUserSettingsBase::CreateKeyRecursive(CStdString p_Key)
{
    bool oResult = true;
    list<CStdString> subkeyList;

    if(!OpenKey(p_Key))
    {
        subkeyList.push_back(m_RootNodeName);

        oResult = GetSubkeyList(p_Key, subkeyList);

        if(oResult)
        {
            oResult = CreateSubkeysIterative(subkeyList);
        }
    }

    return oResult;
}

bool CMmcUserSettingsBase::OpenKey(CStdString p_Key)
{
    bool oResult = false;
    xml_node<char>* pChildNode = 0;
    CStdString key;

    key = ToValidXmlNodeName(p_Key);

    pChildNode = SelectSingleNode(key, 0);

    if(pChildNode)
    {
        m_pCurrentNode = pChildNode;
        oResult = true;
    }

    return oResult;
}

xml_node<char>* CMmcUserSettingsBase::SelectSingleNode(CStdString p_XPathNode, int p_lNodeIndex)
{
    TCHAR seps[]   = _T("\\/");
    TCHAR* pch = _tcstok(p_XPathNode.GetBuffer(), seps);
    xml_node<char>* pChildNode = 0;
    xml_node<char>* pRootNode = GetRootNode();
    string lastNodeName;

    if(pRootNode)
    {
        while(pch != 0)
        {
            if(pChildNode == 0)
            {
                lastNodeName = Unicode2Mbcs(pch);

                if(pRootNode->name() == lastNodeName || pRootNode->name() == p_XPathNode)
                {
                    pChildNode = pRootNode;
                }
                else
                {
                    pChildNode = pRootNode->first_node(lastNodeName.c_str());

                    if(pChildNode==0)
                    {
                        break;
                    }
                }
            }
            else
            {
                lastNodeName = Unicode2Mbcs(pch);

                pChildNode = pChildNode->first_node(lastNodeName.c_str());
            }
            pch = _tcstok(0, seps);
        }
    }

    for(int i = 0; i < p_lNodeIndex; i++)
    {
        if(pChildNode)
        {
            pChildNode = pChildNode->next_sibling(lastNodeName.c_str());
        }
    }

    p_XPathNode.ReleaseBuffer();

    return pChildNode;
}

bool CMmcUserSettingsBase::Reset()
{
    bool oResult = true;

    m_Doc.remove_all_nodes();
    m_Doc.remove_all_attributes();

    return oResult;
}

bool CMmcUserSettingsBase::FileExists(CStdString p_FileName)
{
    bool oResult = false;

    ifstream file;

    file.open(p_FileName);

    if(file.is_open())
    {
        oResult = true;

        file.close();
    }

    return oResult;
}

bool CMmcUserSettingsBase::CreateDocument(CStdString p_FileName)
{
    bool oResult = false;
    ofstream file;
    CStdString content;

    Reset();

    file.open(p_FileName);

    if(file.is_open())
    {
        content.Format(_T("<?xml version=\"1.0\" encoding=\"utf-8\"?><%s></%s>"), m_RootNodeName, m_RootNodeName);

        file << content.c_str();

        file.close();

        oResult = true;
    }

    return oResult;
}

bool CMmcUserSettingsBase::LoadDocument(CStdString p_FileName)
{
    bool oResult = false;
    
    Reset();

    ifstream file;
    std::stringstream buffer;

    file.open(p_FileName);

    if(file.is_open())
    {
        string content;

        buffer << file.rdbuf();

        CStdStringA source = buffer.str();

        file.close();

        oResult = LoadFromString(&source);
    }

    return oResult;
}

xml_node<char>* CMmcUserSettingsBase::GetRootNode()
{
    xml_node<char>* childNode = m_Doc.first_node();
    xml_node<char>* pCurrentNode = 0;
    
    while(childNode)
    {
        if(childNode->type() == node_element)
        {
            pCurrentNode = childNode;
            break;
        }

        childNode = childNode->next_sibling();
    }

    if(pCurrentNode == 0)
    {
        pCurrentNode = m_Doc.document();
    }

    return pCurrentNode;
}

void CMmcUserSettingsBase::GoToRoot()
{
    xml_node<char>* childNode = m_Doc.first_node();

    m_pCurrentNode = 0;

    while(childNode)
    {
        if(childNode->type() == node_element)
        {
            m_pCurrentNode = childNode;
            break;
        }

        childNode = childNode->next_sibling();
    }

    if(m_pCurrentNode == 0)
    {
        m_pCurrentNode = m_Doc.document();
    }
}

bool CMmcUserSettingsBase::ValidateRootNode(rapidxml::xml_node<char>* p_pRootNode)
{
    bool oResult = false;

    if(p_pRootNode)
    {
        CStdString rootNodeName = p_pRootNode->name();

        if(m_RootNodeName == rootNodeName)
        {
            oResult = true;
        }
    }

    return oResult;
}

bool CMmcUserSettingsBase::LoadFromString(CStdStringA* p_pSourceString)
{
    bool oResult = false;

    if(p_pSourceString != 0)
    {
        try
        {
            m_Doc.parse<parse_full | parse_no_data_nodes>(m_Doc.allocate_string(p_pSourceString->GetBuffer()));

            GoToRoot();

            if(ValidateRootNode(m_pCurrentNode))
            {
                m_pRootKey = (void*) m_pCurrentNode;
                oResult = true;
            }
        }
        catch(runtime_error& e)
        {			
            oResult = false;
        }
        catch(parse_error& e)
        {            
            oResult = false;
        }
        catch(exception& e)
        {            
            oResult = false;
        }
        catch(...)
        {
            oResult = false;
        }

        p_pSourceString->ReleaseBuffer();
    }

    return oResult;
}

CStdString CMmcUserSettingsBase::GetKeyFullPath()
{
    CStdString result;
    CMmcUserSettingsDef* pParent = m_pParent;
    list<CStdString> pathList;

    while(pParent)
    {
        CStdString node = ToValidXmlNodeName(pParent->GetKeyName());

        pathList.push_back(node);

        pParent = pParent->GetParent();
    }

    for(list<CStdString>::reverse_iterator it=pathList.rbegin(); it!=pathList.rend(); ++it)
    {
        result = (*it);
        result += _T("\\");
    }

    return result;
}

bool CMmcUserSettingsBase::Init()
{
    bool oResult = true;
    char *homeDir = getenv(_T("HOME"));
    CStdString rootKey = GetKeyName();

    m_MmcDir.Format(_T("%s/.maxon_motor_ag"), homeDir);
    m_MmcRegistryFileName = _T("registry.xml");
    m_RootNodeName = _T("mmcRegistry");

    if(!IsParsed())
    {
        if(rootKey.IsEmpty() && m_pParent == 0)
        {
            oResult = Open();
        }
        else
        {
            CStdString nodeName = ToValidXmlNodeName(rootKey);
            CStdString key = nodeName;

            if(m_pParent)
            {
                key.Format(_T("%s%s"), GetKeyFullPath(), nodeName);
            }

            oResult = Open(key, m_oCreateKey);
        }

        oResult = oResult && m_pCurrentNode != 0;
    }

    return oResult;
}

bool CMmcUserSettingsBase::EnsureMmcDirectoryExists()
{
    bool oResult = true;
    struct stat st = {0};

    if (stat(m_MmcDir, &st) == -1)
    {
        if(mkdir(m_MmcDir, 0700) == -1)
        {
            oResult = false;
        }
    }

    return oResult;
}

std::list<CStdString> CMmcUserSettingsBase::GetChildrenNames()
{
    std::list<CStdString> childrenNames;
    
    if(Init())
    {
        xml_node<char>* pChild = m_pCurrentNode->first_node();
        
        while(pChild != 0)
        {
            CStdString nodeName;

            node_type type = pChild->type();

            if(type == node_element)
            {
                nodeName = Mbcs2Unicode(pChild->name());

                childrenNames.push_back(nodeName);
            }

            pChild = pChild->next_sibling();
        }
    }

    return childrenNames;
}

unsigned long CMmcUserSettingsBase::ReadULONG(CStdString p_Key)
{	
    unsigned long ulResult = 0;
    CStdString nodeValue = ReadString(p_Key);

    if(!nodeValue.IsEmpty())
    {
        ulResult = atol(nodeValue);
    }

    return ulResult;
}

CStdString CMmcUserSettingsBase::ReadString(CStdString p_Key)
{
    CStdString result;

    if(Init())
    {
        bool oKeyExists = false;

        if(p_Key.IsEmpty())
        {
            if(m_pCurrentNode)
            {
                node_type type = m_pCurrentNode->type();

                if(type == node_element)
                {
                    xml_node<char> *pStandardNode = m_pCurrentNode->first_node(_T("Standard"));

                    if(pStandardNode)
                    {
                        result = Mbcs2Unicode(pStandardNode->value());
                    }
                }
            }
        }
        else
        {
            CStdString nodeName = ToValidXmlNodeName(p_Key);

            if(ChildNodeExists(nodeName, oKeyExists) && oKeyExists)
            {
                xml_node<char>* pChildNode = 0;

                if(GetChildNode(nodeName, pChildNode) && pChildNode)
                {
                    CStdString nodeValue;
                    node_type type = pChildNode->type();

                    if(type == node_element)
                    {
                        xml_node<char> *pStandardNode = pChildNode->first_node(_T("Standard"));

                        if(pStandardNode)
                        {
                            result = Mbcs2Unicode(pStandardNode->value());
                        }
                        else
                        {
                            result = Mbcs2Unicode(pChildNode->value());
                        }
                    }
                }
            }
        }
    }

    return result;
}

bool CMmcUserSettingsBase::GetChildNode(CStdString p_NodeName, xml_node<char>*& p_rpChildNode)
{
    bool oResult = false;
    
    if(m_pCurrentNode)
    {
        xml_node<char>* pChild = m_pCurrentNode->first_node();

        while(pChild != 0)
        {
            CStdString nodeName;

            node_type type = pChild->type();

            if(type == node_element)
            {
                nodeName = Mbcs2Unicode(pChild->name());

                if(p_NodeName == nodeName)
                {
                    p_rpChildNode = pChild;
                    oResult = true;
                    break;
                }
            }

            pChild = pChild->next_sibling();
        }
    }

    return oResult;
}

bool  CMmcUserSettingsBase::ChildNodeExists(CStdString p_NodeName, bool& p_roExists)
{
    bool oResult = false;

    p_roExists = false;

    if(m_pCurrentNode)
    {
        oResult = true;

        xml_node<char>* pChild = m_pCurrentNode->first_node();

        while(pChild != 0)
        {
            CStdString nodeName;

            node_type type = pChild->type();

            if(type == node_element)
            {
                nodeName = Mbcs2Unicode(pChild->name());

                if(p_NodeName == nodeName)
                {
                    p_roExists = true;
                    break;
                }
            }

            pChild = pChild->next_sibling();
        }
    }

    return oResult;
}

bool  CMmcUserSettingsBase::Write(CStdString p_Key, CStdString p_Value)
{
    bool oResult = Init();
    bool oKeyExists = false;
    CStdString nodeName;
    xml_node<char>* pChildNode = 0;

    if(oResult)
    {
        nodeName = ToValidXmlNodeName(p_Key);

        if(ChildNodeExists(nodeName, oKeyExists) && oKeyExists)
        {
            if(GetChildNode(nodeName, pChildNode) && pChildNode)
            {
                pChildNode->value(m_Doc.allocate_string(Unicode2Mbcs(p_Value)));
            }
        }
        else
        {
            if(nodeName.IsEmpty())
            {
                xml_node<char> *pStandardNode = m_pCurrentNode->first_node(_T("Standard"));

                if(pStandardNode==0)
                {
                    pStandardNode = m_Doc.allocate_node(node_element, _T("Standard"));

                    m_pCurrentNode->append_node(pStandardNode);
                }

                if(pStandardNode)
                {
                    pStandardNode->value(m_Doc.allocate_string(Unicode2Mbcs(p_Value)));
                }
            }
            else
            {
                xml_node<char>* pChildNode = m_Doc.allocate_node(node_element, m_Doc.allocate_string(Unicode2Mbcs(nodeName)));

                if(pChildNode)
                {
                    pChildNode->value(m_Doc.allocate_string(Unicode2Mbcs(p_Value)));

                    m_pCurrentNode->insert_node(m_pCurrentNode->first_node(), pChildNode);
                }
            }
        }

        oResult = Save();
    }

    return oResult;
}

bool  CMmcUserSettingsBase::Write(CStdString p_Key, unsigned long p_Value)
{
    bool oResult = Init();

    if(oResult)
    {
        CStdString stringValue;

        stringValue.Format(_T("%ld"), p_Value);

        oResult = Write(p_Key, stringValue);
    }

    return oResult;
}

bool  CMmcUserSettingsBase::Delete(CStdString p_Key, bool p_oDeep)
{
    bool oResult = Init();
    CStdString nodeName = ToValidXmlNodeName(p_Key);
    xml_node<char>* pChildNode = 0;
    xml_node<char>* pParentNode = 0;

    pChildNode = SelectSingleNode(nodeName);

    if(pChildNode)
    {
        pParentNode = pChildNode->parent();

        if(pParentNode)
        {
            pParentNode->remove_node(pChildNode);

            oResult = Save();
        }
    }

    return oResult;
}
