#include <unistd.h>
#include <errno.h>
#include <Thread/MmcCriticalSection.h>
#include <Thread/MmcSingleLock.h>
#include "../../include/Thread/MmcCriticalSectionExLinux.h"

CMmcCriticalSectionExLinux::CMmcCriticalSectionExLinux()
{
	m_Name = "";
	m_oAbortLock = false;
}

CMmcCriticalSectionExLinux::~CMmcCriticalSectionExLinux()
{
	DeleteThreadIdToSingleLockMap();
}

void CMmcCriticalSectionExLinux::Init(CStdString name)
{
	m_Name = name;
}

bool CMmcCriticalSectionExLinux::Lock(unsigned int dTimeout, CStdString strTrace)
{
	const unsigned int k_SmallTimeout = 100;
	const unsigned int k_ShortTimeout = 100;

	unsigned int dTime;
	bool oLockResult = false;
	bool oAbortLock = false;

	CMmcSingleLock* pSingleLock;
	pSingleLock = FindSingleLock();
	if(pSingleLock)
	{
		if(!pSingleLock->IsLocked())
		{
			//Try to Lock
			dTime = dTimeout;
			while(!oLockResult && !oAbortLock && (dTime > 0))
			{
				oLockResult = pSingleLock->Lock(k_SmallTimeout);

				dTime -= k_ShortTimeout;

				oAbortLock = m_oAbortLock;
                usleep(k_ShortTimeout);
			}

			//Update Lock State
			if(oLockResult)
			{
				return true;
			}
		}
	}

	return false;
}

bool CMmcCriticalSectionExLinux::Unlock(CStdString strTrace)
{
	CMmcSingleLock* pSingleLock;

	pSingleLock = FindSingleLock();
	if(pSingleLock)
	{
		if(pSingleLock->IsLocked())
		{
			if(pSingleLock->Unlock())
			{
				return true;
			}
		}
	}

	return false;
}

bool CMmcCriticalSectionExLinux::IsLocked(CStdString strTrace)
{
	CMmcSingleLock* pSingleLock;

	pSingleLock = FindSingleLock();
	if(pSingleLock)
	{
		return pSingleLock->IsLocked();
	}

	return false;
}

bool CMmcCriticalSectionExLinux::AbortLock(bool oActive)
{
	m_oAbortLock = oActive;
	return true;
}

void CMmcCriticalSectionExLinux::DeleteThreadIdToSingleLockMap()
{
	for( ThreadToSingleLockMap::iterator it=m_MapThreadIdToSingleLock.begin();
			it != m_MapThreadIdToSingleLock.end(); it++ )
	{
		CMmcSingleLock* pSingleLock = (*it).second;
		if(pSingleLock)
			delete pSingleLock;
	}
}

CMmcSingleLock* CMmcCriticalSectionExLinux::FindSingleLock()
{
	pthread_t threadId = pthread_self();
	CMmcSingleLock* pSingleLock = NULL;

	ThreadToSingleLockMap::iterator it=m_MapThreadIdToSingleLock.find(threadId);

	if( it == m_MapThreadIdToSingleLock.end() )
	{
		pSingleLock = new CMmcSingleLock(&m_CriticalSection,false);

		m_MapThreadIdToSingleLock.insert(ThreadToSinglePair(threadId,pSingleLock));
	}
	else
		pSingleLock = (*it).second;

	return pSingleLock;
}
