#include <unistd.h>
#include <errno.h>
#include <Process/MmcProcess.h>
#include "../../include/Thread/MmcCriticalSectionLinux.h"

CMmcCriticalSectionLinux::CMmcCriticalSectionLinux()
{
	pthread_mutex_init(&m_sync, NULL);
}

CMmcCriticalSectionLinux::~CMmcCriticalSectionLinux()
{
	pthread_mutex_destroy(&m_sync);
}


CMmcCriticalSectionLinux::operator pthread_mutex_t* ()
{
	return &this->m_sync;
}

bool CMmcCriticalSectionLinux::InitialLock()
{
	bool oResult = false;
	struct timespec abs_time;
	const __time_t MAX_WAIT_TIME_IN_SEC = 10;
    int lResult = 0;

	clock_gettime(CLOCK_REALTIME , &abs_time);

	abs_time.tv_sec += MAX_WAIT_TIME_IN_SEC;

	lResult = pthread_mutex_timedlock (&m_sync, &abs_time);

	if (lResult == 0)
	{
		oResult = true;
	}

	return oResult;
}

bool CMmcCriticalSectionLinux::FastLock()
{
    bool oResult = false;

    oResult = (pthread_mutex_trylock(&m_sync) == 0);

    return oResult;
}

bool CMmcCriticalSectionLinux::TimedLock(unsigned int p_ulTimeout)
{
    bool oResult = false;
	const unsigned int MIN_WAIT_TIME_IN_MS = 10;
    unsigned int ulTimeout = p_ulTimeout;
	int lWaitResult = -1;
    
    lWaitResult = pthread_mutex_trylock(&m_sync);

	while(lWaitResult != 0 && ulTimeout > 0)
	{
		lWaitResult = pthread_mutex_trylock(&m_sync);

        if(lWaitResult == EBUSY)
        {
            MmcSleep(MIN_WAIT_TIME_IN_MS);
		    ulTimeout -= MIN_WAIT_TIME_IN_MS;
        }
        else
        {
            break;
        }
	}

	oResult = (lWaitResult == 0);

    return oResult;
}

bool CMmcCriticalSectionLinux::Lock(unsigned int ulTimeout)
{
    bool oResult = false;

	if(ulTimeout != 0)
	{
		oResult = TimedLock(ulTimeout);
	}
	else
	{
		oResult = FastLock();
	}

	return oResult;
}

bool CMmcCriticalSectionLinux::Unlock()
{
	bool oResult = (pthread_mutex_unlock(&m_sync) == 0);

	return oResult;
}
