#include <unistd.h>
#include <errno.h>
#include <Thread/MmcCriticalSection.h>
#include <Thread/MmcSemaphore.h>
#include "../../include/Thread/MmcSingleLockLinux.h"

CMmcSingleLockLinux::CMmcSingleLockLinux(CMmcCriticalSection* p_pSyncObject, bool p_oInitialLock)
{
	m_pSyncObject = p_pSyncObject;
	m_oAcquired = false;

	if (p_oInitialLock)
	{
		InitialLock();
	}
}

CMmcSingleLockLinux::CMmcSingleLockLinux(CMmcSemaphore* p_pSyncObject, bool p_oInitialLock)
{
	m_pSyncObject = p_pSyncObject;
	m_oAcquired = false;

	if (p_oInitialLock)
	{
		InitialLock();
	}
}

CMmcSingleLockLinux::~CMmcSingleLockLinux()
{
	if(IsLocked())
	{
		Unlock();
	}
}

void CMmcSingleLockLinux::InitialLock()
{
	if(m_pSyncObject)
	{
		m_oAcquired = m_pSyncObject->InitialLock();
	}
}

bool CMmcSingleLockLinux::Lock(unsigned int p_lTimeout /*=0*/)
{
	if(m_pSyncObject == 0)
	{
		m_oAcquired = true;
	}
	else
	{
		m_oAcquired = m_pSyncObject->Lock(p_lTimeout);
	}

	return m_oAcquired;
}

bool CMmcSingleLockLinux::Unlock()
{
	if(m_pSyncObject == 0)
	{
		m_oAcquired = false;
	}
	else
	{
		m_oAcquired = !m_pSyncObject->Unlock();
	}

	return m_oAcquired;
}

bool CMmcSingleLockLinux::IsLocked()
{
	return m_oAcquired;
}



