#include <Process/MmcProcess.h>
#include <MmcTypeDefinition.h>
#include <sys/resource.h>
#include <Thread/MmcSingleLock.h>
#include <unistd.h>

#include <Thread/MmcThreadPosix.h>

CMmcThreadPosix::CMmcThreadPosix()
{	
	m_thread = 0;
	m_ThreadID = 0;
}

CMmcThreadPosix::~CMmcThreadPosix()
{
	Stop();
}

unsigned int CMmcThreadPosix::ThreadFunc()
{
	SetThreadState(TBS_STARTED);
    
    
    while(!IsStopping())
	{		
	}

	SetThreadState(TBS_STOPPED);
    
	return -1;
}

void* CMmcThreadPosix::BaseThreadFunc(void *pParam)
{
	CMmcThreadPosix* pThread = reinterpret_cast<CMmcThreadPosix*>(pParam);

	if(pThread)
	{
		pThread->m_ThreadID = pthread_self();
		
		CMmcSingleLock lock(&pThread->m_Sync,false);
        
		lock.Lock();
		lock.Unlock();

   	   //Thread Func
   	   pThread?pThread->ThreadFunc():-1;

   	   //Reset
   	   pThread->m_ThreadID = 0;
   	}

   pthread_exit(0);
}

int CMmcThreadPosix::Start()
{
	int lResult = 1;

	if(CanStart())
	{
		pthread_attr_t attr;
		
		pthread_attr_init(&attr);
		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
        
        SetThreadState(TBS_STARTING);

		lResult = pthread_create(&m_thread, NULL, BaseThreadFunc, (void *)this);

		pthread_attr_destroy(&attr);		
	}

	return lResult==0;
}

int CMmcThreadPosix::Stop()
{
	BOOL oResult = TRUE;
	const int MAX_WAIT_TIMEOUT = 3000;
	const int WAIT_TIMEOUT = 100;
	int timeout = 0;

	SetThreadState(TBS_STOPPING);

	while(IsStarted())
	{
		if(m_thread != 0 && timeout > MAX_WAIT_TIMEOUT)
		{
			pthread_cancel(m_thread);
			break;
		}

		MmcSleep(WAIT_TIMEOUT);
        
		timeout += WAIT_TIMEOUT;
	}

	m_thread = 0;

	return oResult;
}

int CMmcThreadPosix::IsStillActive()
{
	int lResult = ENOENT;

	if(m_thread != 0)
	{
		lResult = pthread_tryjoin_np(m_thread, NULL);
	}

	return (lResult == EBUSY);
}

int CMmcThreadPosix::SetHighPriority()
{
	int which = PRIO_PROCESS;
	__pid_t pid;
	int lPriority = -20;
	int lResult;

	pid = getpid();

	lResult = setpriority(which, pid, lPriority);

	return lResult==0;
}


