//////////////////////////////////////////////////////////////////////////////
// MmcHiResTimer.cpp
// =========
// High Resolution Timer.
// This timer is able to measure the elapsed time with 1 micro-second accuracy
// in both Windows, Linux and Unix system 
//
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2003-01-13
// UPDATED: 2006-01-13
//
// Copyright (c) 2003 Song Ho Ahn
//////////////////////////////////////////////////////////////////////////////

#include <Time/MmcHiResTimer.h>
#include <stdlib.h>

///////////////////////////////////////////////////////////////////////////////
// constructor
///////////////////////////////////////////////////////////////////////////////
CMmcHiResTimerLinux::CMmcHiResTimerLinux()
{
	reset();
}



///////////////////////////////////////////////////////////////////////////////
// distructor
///////////////////////////////////////////////////////////////////////////////
CMmcHiResTimerLinux::~CMmcHiResTimerLinux()
{
}



///////////////////////////////////////////////////////////////////////////////
// start timer.
// startCount will be set at this point.
///////////////////////////////////////////////////////////////////////////////
double CMmcHiResTimerLinux::start()
{
    m_stopped = 0; // reset stop flag
    gettimeofday(&m_startCount, NULL);
    m_startTimeInMicroSec = (m_startCount.tv_sec * 1000000.0) + m_startCount.tv_usec;

    return m_startTimeInMicroSec;
}



///////////////////////////////////////////////////////////////////////////////
// stop the timer.
// endCount will be set at this point.
///////////////////////////////////////////////////////////////////////////////
void CMmcHiResTimerLinux::stop()
{
	m_stopped = 1; // set timer stopped flag

    gettimeofday(&m_endCount, NULL);
}

void CMmcHiResTimerLinux::reset()
{
	m_startCount.tv_sec = m_startCount.tv_usec = 0;
	m_endCount.tv_sec = m_endCount.tv_usec = 0;

	m_stopped = 0;
	m_startTimeInMicroSec = 0;
	m_endTimeInMicroSec = 0;
}

///////////////////////////////////////////////////////////////////////////////
// compute elapsed time in micro-second resolution.
// other getElapsedTime will call this first, then convert to correspond resolution.
///////////////////////////////////////////////////////////////////////////////
double CMmcHiResTimerLinux::getElapsedTimeInMicroSec()
{
    if(!m_stopped)
        gettimeofday(&m_endCount, NULL);

    m_startTimeInMicroSec = (m_startCount.tv_sec * 1000000.0) + m_startCount.tv_usec;
    m_endTimeInMicroSec = (m_endCount.tv_sec * 1000000.0) + m_endCount.tv_usec;

    return m_endTimeInMicroSec - m_startTimeInMicroSec;
}



///////////////////////////////////////////////////////////////////////////////
// divide elapsedTimeInMicroSec by 1000
///////////////////////////////////////////////////////////////////////////////
double CMmcHiResTimerLinux::getElapsedTimeInMilliSec()
{
    return this->getElapsedTimeInMicroSec() * 0.001;
}



///////////////////////////////////////////////////////////////////////////////
// divide elapsedTimeInMicroSec by 1000000
///////////////////////////////////////////////////////////////////////////////
double CMmcHiResTimerLinux::getElapsedTimeInSec()
{
    return this->getElapsedTimeInMicroSec() * 0.000001;
}



///////////////////////////////////////////////////////////////////////////////
// same as getElapsedTimeInSec()
///////////////////////////////////////////////////////////////////////////////
double CMmcHiResTimerLinux::getElapsedTime()
{
    return this->getElapsedTimeInSec();
}
