#include "stdafx.h"
#include "stdint.h"
#include <bitset>
#include <limits>

#include "MmcTypeDefinition.h"
#include <math.h>
#include <Log/MmcLog/MmcLog.h>
#include <Process/MmcProcess.h>
#include <Classes/MmcDataConversion.h>

#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/Command_DCS_Epos2.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenCommunicationDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsCanOpenDriveDef.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/DeviceCommandSet/DcsEpos2Def.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/BaseClasses/Command_VCS.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_CanOpen.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Configuration.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Device.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_ErrorHandling.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Firmware.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Gateway.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_InputsOutputs.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Motion.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_OperationMode.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Parameter.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_Scanning.h>
#include <CommunicationModel/CommonLayer/Classes/Commands/VirtualCommandSet/Command_VCS_StateMachine.h>
#include <CommunicationModel/DeviceCommandSet/BaseClasses/DeviceCommandSetManagerBase.h>
#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/Helper/ControllerGainsEpos2.h>
#include <Time/MmcHiResTimer.h>

#include <CommunicationModel/VirtualCommandSet/Classes/Gateway/Drive/GatewayDriveToEpos2.h>

using namespace std;

const CStdString GATEWAY_NODE_ID = _T("GatewayNodeId");
const CStdString NODE_ID = _T("NodeId");

CGatewayDriveToEpos2::CGatewayDriveToEpos2()
    : m_ulDeviceTimeoutBackup(0)
{
    InitErrorHandling();
}

CGatewayDriveToEpos2::~CGatewayDriveToEpos2()
{
}

CGateway* CGatewayDriveToEpos2::Clone()
{
    CGatewayDriveToEpos2* pClonedGateway;

    pClonedGateway = new CGatewayDriveToEpos2();
    *pClonedGateway = *this;

    return (CGatewayVCStoDCS*)pClonedGateway;
}

CGatewayDriveToEpos2& CGatewayDriveToEpos2::operator=(CGatewayDriveToEpos2& p_rOther)
{
    if(this != &p_rOther)
    {
        *((CGatewayVCStoEpos2*)this) = *((CGatewayVCStoEpos2*)&p_rOther);
    }

    return *this;
}

BOOL CGatewayDriveToEpos2::ProcessCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);

    if(CheckLayers(p_pCommand, p_pManager))
    {
        BOOL oLogMessageActive = p_pCommand->IsLogMessageActive();

        LogCmdMessageIn(p_pCommand, oLogMessageActive, _T("CGatewayDriveToEpos2"));

        oResult = ProcessDriveCommand(p_pCommand, p_pManager, p_hHandle, p_hTransactionHandle);

        LogCmdMessageOut(p_pCommand, oLogMessageActive, _T("CGatewayDriveToEpos2"));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::ProcessDriveCommand(CCommandRoot* p_pCommand, CLayerManagerBase* p_pManager, HANDLE p_hHandle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CCommand_VCS* pCommand_VCS = (CCommand_VCS*)p_pCommand;
    CDeviceCommandSetManagerBase* pDeviceCommandSetManager = (CDeviceCommandSetManagerBase*)p_pManager;

    // Lock critical section
    if(Lock(p_pCommand))
    {
        //Process command
        switch(p_pCommand->GetCommandId())
        {
            // Configuration
            case CONF_GET_CURRENT_REGULATOR_GAIN: oResult = Process_GetCurrentRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_POSITION_REGULATOR_GAIN: oResult = Process_GetPositionRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_POSITION_REGULATOR_FEED_FORWARD: oResult = Process_GetPositionRegulatorFeedForward(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_VELOCITY_REGULATOR_GAIN: oResult = Process_GetVelocityRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_VELOCITY_REGULATOR_FEED_FORWARD: oResult = Process_GetVelocityRegulatorFeedForward(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_CURRENT_REGULATOR_GAIN: oResult = Process_SetCurrentRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_POSITION_REGULATOR_GAIN: oResult = Process_SetPositionRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_POSITION_REGULATOR_FEED_FORWARD: oResult = Process_SetPositionRegulatorFeedForward(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_VELOCITY_REGULATOR_GAIN: oResult = Process_SetVelocityRegulatorGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_VELOCITY_REGULATOR_FEED_FORWARD: oResult = Process_SetVelocityRegulatorFeedForward(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Controller gains
            case CONF_SET_CONTROLLER_GAIN: oResult = Process_SetControllerGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_CONTROLLER_GAIN: oResult = Process_GetControllerGain(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //Safety
            case CONF_GET_MAX_FOLLOWING_ERROR: oResult = Process_GetMaxFollowingError(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_MAX_PROFILE_VELOCITY: oResult = Process_GetMaxProfileVelocity(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_MAX_ACCELERATION: oResult = Process_GetMaxAcceleration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_MAX_FOLLOWING_ERROR: oResult = Process_SetMaxFollowingError(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_MAX_PROFILE_VELOCITY: oResult = Process_SetMaxProfileVelocity(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_MAX_ACCELERATION: oResult = Process_SetMaxAcceleration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //Configuration motor
            case CONF_GET_MOTOR_PARAMETER: oResult = Process_GetMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_MOTOR_TYPE: oResult = Process_GetMotorType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_DC_MOTOR_PARAMETER: oResult = Process_GetDcMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_EC_MOTOR_PARAMETER: oResult = Process_GetEcMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_MOTOR_PARAMETER: oResult = Process_SetMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_MOTOR_TYPE: oResult = Process_SetMotorType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_DC_MOTOR_PARAMETER: oResult = Process_SetDcMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_EC_MOTOR_PARAMETER: oResult = Process_SetEcMotorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Configuration sensor
            case CONF_GET_ENCODER_PARAMETER: oResult = Process_GetEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_SENSOR_TYPE: oResult = Process_GetSensorType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_INC_ENCODER_PARAMETER: oResult = Process_GetIncEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_HALL_SENSOR_PARAMETER: oResult = Process_GetHallSensorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_GET_SSI_ABS_ENCODER_PARAMETER: oResult = Process_GetSsiAbsEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_ENCODER_PARAMETER: oResult = Process_SetEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_SENSOR_TYPE: oResult = Process_SetSensorType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_INC_ENCODER_PARAMETER: oResult = Process_SetIncEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_HALL_SENSOR_PARAMETER: oResult = Process_SetHallSensorParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case CONF_SET_SSI_ABS_ENCODER_PARAMETER: oResult = Process_SetSsiAbsEncoderParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Homing mode
            case MODE_HM_FIND_HOME: oResult = Process_FindHome(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_HM_GET_HOMING_PARAMETER: oResult = Process_GetHomingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_HM_SET_HOMING_PARAMETER: oResult = Process_SetHomingParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_HM_STOP_HOMING: oResult = Process_StopHoming(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_HM_DEFINE_POSITION: oResult = Process_DefinePosition(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_HM_GET_HOMING_STATE: oResult = Process_GetHomingState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Motion info
            case MOTION_GET_CURRENT_IS: oResult = Process_GetCurrentIs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MOTION_GET_CURRENT_IS_AVERAGED: oResult = Process_GetCurrentIsAveraged(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MOTION_GET_MOVEMENT_STATE: oResult = Process_GetMovementState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MOTION_GET_POSITION_IS: oResult = Process_GetPositionIs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MOTION_GET_VELOCITY_IS: oResult = Process_GetVelocityIs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MOTION_GET_VELOCITY_IS_AVERAGED: oResult = Process_GetVelocityIsAveraged(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Profile position mode
            case MODE_PPM_GET_POSITION_PROFILE: oResult = Process_GetPositionProfile(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_GET_TARGET_POSITION: oResult = Process_GetTargetPosition(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_HALT_POSITION_MOVEMENT: oResult = Process_HaltPositionMovement(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_MOVE_TO_POSITION: oResult = Process_MoveToPosition(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_SET_POSITION_PROFILE: oResult = Process_SetPositionProfile(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_ENABLE_POSITION_WINDOW: oResult = Process_EnablePositionWindow(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PPM_DISABLE_POSITION_WINDOW: oResult = Process_DisablePositionWindow(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Profile velocity mode
            case MODE_PVM_GET_TARGET_VELOCITY: oResult = Process_GetTargetVelocity(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_GET_VELOCITY_PROFILE: oResult = Process_GetVelocityProfile(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_HALT_VELOCITY_MOVEMENT: oResult = Process_HaltVelocityMovement(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_MOVE_WITH_VELOCITY: oResult = Process_MoveWithVelocity(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_SET_VELOCITY_PROFILE: oResult = Process_SetVelocityProfile(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_ENABLE_VELOCITY_WINDOW: oResult = Process_EnableVelocityWindow(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_PVM_DISABLE_VELOCITY_WINDOW: oResult = Process_DisableVelocityWindow(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Current mode
            case MODE_CUM_GET_CURRENT_MUST: oResult = Process_GetCurrentMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_CUM_SET_CURRENT_MUST: oResult = Process_SetCurrentMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_CUM_ACTIVATE_ANALOG_CURRENT_SETPOINT: oResult = Process_ActivateAnalogCurrentSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_CUM_DEACTIVATE_ANALOG_CURRENT_SETPOINT: oResult = Process_DeactivateAnalogCurrentSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_CUM_ENABLE_ANALOG_CURRENT_SETPOINT: oResult = Process_EnableAnalogCurrentSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_CUM_DISABLE_ANALOG_CURRENT_SETPOINT: oResult = Process_DisableAnalogCurrentSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Velocity mode
            case MODE_VEM_GET_VELOCITY_MUST: oResult = Process_GetVelocityMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_VEM_SET_VELOCITY_MUST: oResult = Process_SetVelocityMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_VEM_ACTIVATE_ANALOG_VELOCITY_SETPOINT: oResult = Process_ActivateAnalogVelocitySetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_VEM_DEACTIVATE_ANALOG_VELOCITY_SETPOINT: oResult = Process_DeactivateAnalogVelocitySetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_VEM_ENABLE_ANALOG_VELOCITY_SETPOINT: oResult = Process_EnableAnalogVelocitySetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_VEM_DISABLE_ANALOG_VELOCITY_SETPOINT: oResult = Process_DisableAnalogVelocitySetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Position mode
            case MODE_POM_GET_POSITION_MUST: oResult = Process_GetPositionMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_POM_SET_POSITION_MUST: oResult = Process_SetPositionMust(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_POM_ACTIVATE_ANALOG_POSITION_SETPOINT: oResult = Process_ActivateAnalogPositionSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_POM_DEACTIVATE_ANALOG_POSITION_SETPOINT: oResult = Process_DeactivateAnalogPositionSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_POM_ENABLE_ANALOG_POSITION_SETPOINT: oResult = Process_EnableAnalogPositionSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_POM_DISABLE_ANALOG_POSITION_SETPOINT: oResult = Process_DisableAnalogPositionSetpoint(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            //I/O
            case IO_DIGITAL_INPUT_CONFIGURATION: oResult = Process_DigitalInputConfiguration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_DIGITAL_OUTPUT_CONFIGURATION: oResult = Process_DigitalOutputConfiguration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_ANALOG_INPUT_CONFIGURATION: oResult = Process_AnalogInputConfiguration(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_ALL_DIGITAL_INPUTS: oResult = Process_GetAllDigitalInputs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_ALL_DIGITAL_OUTPUTS: oResult = Process_GetAllDigitalOutputs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_ANALOG_INPUT: oResult = Process_GetAnalogInput(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_ANALOG_INPUT_VOLTAGE: oResult = Process_GetAnalogInputVoltage(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_ANALOG_INPUT_STATE: oResult = Process_GetAnalogInputState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_SET_ALL_DIGITAL_OUTPUTS: oResult = Process_SetAllDigitalOutputs(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_SET_ANALOG_OUTPUT: oResult = Process_SetAnalogOutput(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_SET_ANALOG_OUTPUT_VOLTAGE: oResult = Process_SetAnalogOutputVoltage(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // I/O position marker
            case IO_SET_POSITION_MARKER_PARAMETER: oResult = Process_SetPositionMarkerParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_GET_POSITION_MARKER_PARAMETER: oResult = Process_GetPositionMarkerParameter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_ACTIVATE_POSITION_MARKER: oResult = Process_ActivatePositionMarker(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_DEACTIVATE_POSITION_MARKER: oResult = Process_DeactivatePositionMarker(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_READ_POSITION_MARKER_COUNTER: oResult = Process_ReadPositionMarkerCounter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_READ_POSITION_MARKER_CAPTURED_POSITION: oResult = Process_ReadPositionMarkerCapturedPosition(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case IO_RESET_POSITION_MARKER_COUNTER: oResult = Process_ResetPositionMarkerCounter(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // State machine
            case STATE_CLEAR_FAULT: oResult = Process_ClearFault(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_GET_DISABLE_STATE: oResult = Process_GetDisableState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_GET_ENABLE_STATE: oResult = Process_GetEnableState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_GET_FAULT_STATE: oResult = Process_GetFaultState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_GET_QUICK_STOP_STATE: oResult = Process_GetQuickStopState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_SET_DISABLE_STATE: oResult = Process_SetDisableState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_SET_ENABLE_STATE: oResult = Process_SetEnableState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case STATE_SET_QUICK_STOP_STATE: oResult = Process_SetQuickStopState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Operation mode
            case MODE_GET_OPERATION_MODE: oResult = Process_GetOperationMode(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case MODE_SET_OPERATION_MODE: oResult = Process_SetOperationMode(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Error handling
            case EH_RESET_DEVICE: oResult = Process_ResetDevice(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case EH_WAIT_FOR_REBOOT: oResult = Process_WaitForReboot(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case EH_CLEAR_DEVICE_ERRORS: oResult = Process_ClearDeviceErrors(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case EH_GET_NB_OF_DEVICE_ERRORS: oResult = Process_GetNbOfDeviceErrors(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case EH_GET_DEVICE_ERROR_CODE: oResult = Process_GetDeviceErrorCode(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case EH_GOTO_SAFE_STATE: oResult = Process_GotoSafeState(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Version
            case DEVICE_GET_VERSION: oResult = Process_GetVersion(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case DEVICE_GET_TYPE: oResult = Process_GetDeviceType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Persistence
            case PARAMETER_STORE: oResult = Process_Store(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case PARAMETER_RESTORE: oResult = Process_Restore(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case PARAMETER_RESTORE_FACTORY: oResult = Process_RestoreFactory(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Object dictionary
            case VCS_CANOPEN_GET_OBJECT: oResult = Process_GetObject(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_SET_OBJECT: oResult = Process_SetObject(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_RESTORE: oResult = Process_Restore(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_STORE: oResult = Process_Store(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_RESTORE_FACTORY: oResult = Process_RestoreFactory(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // General gateway
            case VCS_CANOPEN_REQUEST_CAN_FRAME: oResult = Process_RequestCANFrame(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_SEND_CAN_FRAME: oResult = Process_SendCANFrame(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_READ_CAN_FRAME: oResult = Process_ReadCANFrame(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_SEND_NMT_SERVICE: oResult = Process_SendNMTService(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // LayerSettingServices
            case VCS_CANOPEN_SEND_LSS_FRAME: oResult = Process_SendLSSFrame(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case VCS_CANOPEN_READ_LSS_FRAME: oResult = Process_ReadLSSFrame(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Scanning
            case SCAN_READ_VENDOR_ID: oResult = Process_ReadVendorId(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_DEVICE_TYPE: oResult = Process_ReadDeviceType(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_DEVICE_NAME: oResult = Process_ReadDeviceName(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_PRODUCT_CODE: oResult = Process_ReadProductCode(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_REVISION_NUMBER: oResult = Process_ReadRevisionNumber(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_SERIAL_NUMBER: oResult = Process_ReadSerialNumber(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_IS_IDENTIFY_SUPPORTED: oResult = Process_IsIdentifySupported(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_START_IDENTIFY: oResult = Process_StartIdentify(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_STOP_IDENTIFY: oResult = Process_StopIdentify(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_NODE_ID: oResult = Process_ReadNodeId(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_WRITE_NODE_ID: oResult = Process_WriteNodeId(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_READ_TRANSFER_RATE: oResult = Process_ReadTransferRate(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_WRITE_TRANSFER_RATE: oResult = Process_WriteTransferRate(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_STORE: oResult = Process_ScanStore(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_RESET_DEVICE: oResult = Process_ScanResetDevice(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SCAN_WAIT_FOR_REBOOT: oResult = Process_ScanWaitForReboot(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Firmwaredownload
            case FIRMWARE_GET_BASE_SECTOR_VERSION: oResult = Process_GetBaseSectorVersion(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case FIRMWARE_GET_SERIAL_NUMBER: oResult = Process_GetSerialNumber(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;

            // Gateway
            case GET_GATEWAY_SETTINGS: oResult = Process_GetGatewaySettings(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
            case SET_GATEWAY_SETTINGS: oResult = Process_SetGatewaySettings(pCommand_VCS, pDeviceCommandSetManager, p_hHandle, p_hTransactionHandle); break;
        }

        // Unlock critical section
        Unlock();
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::InitErrorHandling()
{
    BOOL oResult(FALSE);
    CErrorProducer errorProducer;
    CStdString className(_T("GatewayDriveToEpos2"));

    if(m_pErrorHandling)
    {
        //Init error producer
        errorProducer.Init(VIRTUAL_COMMAND_SET_LAYER, className);
        oResult = m_pErrorHandling->InitErrorProducer(&errorProducer);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetCurrentRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usI = 0;
    WORD usIndex;
    WORD usP = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Current Regulator P-Gain
        if(oResult)
        {
            usIndex = INDEX_CURRENT_GAIN;
            usSubIndex = SUBINDEX_CURRENT_P_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usP, &errorInfo);
        }

        //Read Current Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_CURRENT_GAIN;
            usSubIndex = SUBINDEX_CURRENT_I_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usI, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usP, sizeof(usP));
        p_pCommand->SetReturnParameterData(1, &usI, sizeof(usI));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usD = 0;
    WORD usI = 0;
    WORD usIndex;
    WORD usP = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Position Regulator P-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_P_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usP, &errorInfo);
        }

        //Read Position Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_I_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usI, &errorInfo);
        }

        //Read Position Regulator D-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_D_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usD, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usP, sizeof(usP));
        p_pCommand->SetReturnParameterData(1, &usI, sizeof(usI));
        p_pCommand->SetReturnParameterData(2, &usD, sizeof(usD));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionRegulatorFeedForward(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAccelerationFeedForward = 0;
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityFeedForward = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read velocity feed forward
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_VELOCITY_FEED_FORWARD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVelocityFeedForward, &errorInfo);
        }

        //Read acceleration feed forward
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_ACCELERATION_FEED_FORWARD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usAccelerationFeedForward, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usVelocityFeedForward, sizeof(usVelocityFeedForward));
        p_pCommand->SetReturnParameterData(1, &usAccelerationFeedForward, sizeof(usAccelerationFeedForward));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usI = 0;
    WORD usIndex;
    WORD usP = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Velocity Regulator P-Gain
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_P_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usP, &errorInfo);
        }

        //Read Velocity Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_I_GAIN;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usI, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usP, sizeof(usP));
        p_pCommand->SetReturnParameterData(1, &usI, sizeof(usI));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityRegulatorFeedForward(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAccelerationFeedForward = 0;
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityFeedForward = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read velocity feed forward
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_VELOCITY_FEED_FORWARD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usVelocityFeedForward, &errorInfo);
        }

        //Read acceleration feed forward
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_ACCELERATION_FEED_FORWARD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usAccelerationFeedForward, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usVelocityFeedForward, sizeof(usVelocityFeedForward));
        p_pCommand->SetReturnParameterData(1, &usAccelerationFeedForward, sizeof(usAccelerationFeedForward));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetCurrentRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usI;
    WORD usIndex;
    WORD usP;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usP, sizeof(usP));
        p_pCommand->GetParameterData(1, &usI, sizeof(usI));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write Current Regualator P-Gain
        if(oResult)
        {
            usIndex = INDEX_CURRENT_GAIN;
            usSubIndex = SUBINDEX_CURRENT_P_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usP, &errorInfo);
        }

        //Write Current Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_CURRENT_GAIN;
            usSubIndex = SUBINDEX_CURRENT_I_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usI, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetPositionRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usD;
    WORD usI;
    WORD usIndex;
    WORD usP;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usP, sizeof(usP));
        p_pCommand->GetParameterData(1, &usI, sizeof(usI));
        p_pCommand->GetParameterData(2, &usD, sizeof(usD));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Position Regulator P-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_P_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usP, &errorInfo);
        }

        //Read Position Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_I_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usI, &errorInfo);
        }

        //Write Position Regulator D-Gain
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_D_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usD, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetPositionRegulatorFeedForward(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAccelerationFeedForward = 0;
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityFeedForward = 0;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usVelocityFeedForward, sizeof(usVelocityFeedForward));
        p_pCommand->GetParameterData(1, &usAccelerationFeedForward, sizeof(usAccelerationFeedForward));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write velocity feed forward
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_VELOCITY_FEED_FORWARD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usVelocityFeedForward, &errorInfo);
        }

        //Write acceleration feed forward
        if(oResult)
        {
            usIndex = INDEX_POSITION_GAIN;
            usSubIndex = SUBINDEX_POSITION_ACCELERATION_FEED_FORWARD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usAccelerationFeedForward, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetVelocityRegulatorGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usI;
    WORD usIndex;
    WORD usP;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usP, sizeof(usP));
        p_pCommand->GetParameterData(1, &usI, sizeof(usI));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Velocity Regulator P-Gain
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_P_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usP, &errorInfo);
        }

        //Read Velocity Regulator I-Gain
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_I_GAIN;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usI, &errorInfo);
        }

        //Set ReturnParameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetVelocityRegulatorFeedForward(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAccelerationFeedForward = 0;
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityFeedForward = 0;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usVelocityFeedForward, sizeof(usVelocityFeedForward));
        p_pCommand->GetParameterData(1, &usAccelerationFeedForward, sizeof(usAccelerationFeedForward));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write velocity feed forward
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_VELOCITY_FEED_FORWARD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usVelocityFeedForward, &errorInfo);
        }

        //Write acceleration feed forward
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_GAIN;
            usSubIndex = SUBINDEX_VELOCITY_ACCELERATION_FEED_FORWARD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usAccelerationFeedForward, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetMaxFollowingError(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxFollowingError = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulMaxFollowingError, sizeof(ulMaxFollowingError));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //MaxFollowingError
        if(oResult)
        {
            usIndex = INDEX_MAX_FOLLOWING_ERROR;
            usSubIndex = SUBINDEX_MAX_FOLLOWING_ERROR;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxFollowingError, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetControllerGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CControllerGainsEpos2 controllerGain;
    CErrorInfo errorInfo;
    WORD usController;
    WORD usGain;
    WORD usIndex(0);
    WORD usSubIndex(0);
    WORD usValue;
    __uint64 ullValue(0);

    if (p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &usController, sizeof(usController));
        p_pCommand->GetParameterData(1, &usGain, sizeof(usGain));

        if (oResult)
        {
            oResult = controllerGain.GetControllerGainIndex(usController, usGain, usIndex, usSubIndex, errorInfo);
        }

        if (oResult)
        {
            if (ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo))
            {
                ullValue = usValue;
            }
            else
            {
                oResult = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ullValue, sizeof(ullValue));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetControllerGain(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CControllerGainsEpos2 controllerGain;
    CErrorInfo errorInfo;
    WORD usController;
    WORD usGain;
    WORD usIndex;
    WORD usSubIndex;
    WORD usValue;
    __uint64 ullValue(0);

    if (p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &usController, sizeof(usController));
        p_pCommand->GetParameterData(1, &usGain, sizeof(usGain));
        p_pCommand->GetParameterData(2, &ullValue, sizeof(ullValue));

        if (oResult)
        {
            oResult = controllerGain.GetControllerGainIndex(usController, usGain, usIndex, usSubIndex, errorInfo);
        }

        if (oResult)
        {
            usValue = (WORD)ullValue;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMaxFollowingError(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxFollowingError = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Max following error
        if(oResult)
        {
            usIndex = INDEX_MAX_FOLLOWING_ERROR;
            usSubIndex = SUBINDEX_MAX_FOLLOWING_ERROR;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxFollowingError, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulMaxFollowingError, sizeof(ulMaxFollowingError));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetMaxProfileVelocity(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxProfileVelocity = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulMaxProfileVelocity, sizeof(ulMaxProfileVelocity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //MaxProfileVelocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxProfileVelocity, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMaxProfileVelocity(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxProfileVelocity = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //MaxProfileVelocity
        if(oResult)
        {
            usIndex = INDEX_MAX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_MAX_PROFILE_VELOCITY;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxProfileVelocity, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulMaxProfileVelocity, sizeof(ulMaxProfileVelocity));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetMaxAcceleration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxAcceleration = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulMaxAcceleration, sizeof(ulMaxAcceleration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //MaxAcceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMaxAcceleration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulMaxAcceleration = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //MaxAcceleration
        if(oResult)
        {
            usIndex = INDEX_MAX_ACCELERATION;
            usSubIndex = SUBINDEX_MAX_ACCELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulMaxAcceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulMaxAcceleration, sizeof(ulMaxAcceleration));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPolePair;
    CErrorInfo errorInfo;
    WORD usContinuousCurrent;
    WORD usIndex;
    WORD usMotorType;
    WORD usPeakCurrent;
    WORD usSubIndex;
    WORD usThermalTimeConstant;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usMotorType, sizeof(usMotorType));
        p_pCommand->GetParameterData(1, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->GetParameterData(2, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->GetParameterData(3, &ubPolePair, sizeof(ubPolePair));
        p_pCommand->GetParameterData(4, &usThermalTimeConstant, sizeof(usThermalTimeConstant));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write motor type
        if(oResult)
        {
            usIndex = INDEX_MOTOR_TYPE;
            usSubIndex = SUBINDEX_MOTOR_TYPE;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMotorType, &errorInfo);
        }

        //Write continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usContinuousCurrent, &errorInfo);
        }

        //Write Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPeakCurrent, &errorInfo);
        }

        //Write pole paar
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_POLE_PAIR_NUMBER;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPolePair, &errorInfo);
        }

        //Read Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usThermalTimeConstant, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetMotorType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usMotorType;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usMotorType, sizeof(usMotorType));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write motor type
        if(oResult)
        {
            usIndex = INDEX_MOTOR_TYPE;
            usSubIndex = SUBINDEX_MOTOR_TYPE;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMotorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetDcMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usContinuousCurrent;
    WORD usIndex;
    WORD usPeakCurrent;
    WORD usSubIndex;
    WORD usThermalTimeConstant;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->GetParameterData(1, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->GetParameterData(2, &usThermalTimeConstant, sizeof(usThermalTimeConstant));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usContinuousCurrent, &errorInfo);
        }

        //Write Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPeakCurrent, &errorInfo);
        }

        //Write Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usThermalTimeConstant, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetEcMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPolePair;
    CErrorInfo errorInfo;
    WORD usContinuousCurrent;
    WORD usIndex;
    WORD usPeakCurrent;
    WORD usSubIndex;
    WORD usThermalTimeConstant;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->GetParameterData(1, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->GetParameterData(2, &usThermalTimeConstant, sizeof(usThermalTimeConstant));
        p_pCommand->GetParameterData(3, &ubPolePair, sizeof(ubPolePair));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usContinuousCurrent, &errorInfo);
        }

        //Write Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPeakCurrent, &errorInfo);
        }

        //Write Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usThermalTimeConstant, &errorInfo);
        }

        //Write pole paar
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_POLE_PAIR_NUMBER;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPolePair, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPolePair = 0;
    CErrorInfo errorInfo;
    WORD usContinuousCurrent = 0;
    WORD usIndex;
    WORD usMotorType = 0;
    WORD usPeakCurrent = 0;
    WORD usSubIndex;
    WORD usThermalTimeConstant = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Motortype
        if(oResult)
        {
            usIndex = INDEX_MOTOR_TYPE;
            usSubIndex = SUBINDEX_MOTOR_TYPE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMotorType, &errorInfo);
        }

        //Read continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usContinuousCurrent, &errorInfo);
        }

        //Read Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPeakCurrent, &errorInfo);
        }

        //Read pole paar
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_POLE_PAIR_NUMBER;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPolePair, &errorInfo);
        }

        //Read Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usThermalTimeConstant, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usMotorType, sizeof(usMotorType));
        p_pCommand->SetReturnParameterData(1, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->SetReturnParameterData(2, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->SetReturnParameterData(3, &ubPolePair, sizeof(ubPolePair));
        p_pCommand->SetReturnParameterData(4, &usThermalTimeConstant, sizeof(usThermalTimeConstant));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMotorType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usMotorType = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Motortype
        if(oResult)
        {
            usIndex = INDEX_MOTOR_TYPE;
            usSubIndex = SUBINDEX_MOTOR_TYPE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMotorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usMotorType, sizeof(usMotorType));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetDcMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usContinuousCurrent = 0;
    WORD usIndex;
    WORD usPeakCurrent = 0;
    WORD usSubIndex;
    WORD usThermalTimeConstant = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usContinuousCurrent, &errorInfo);
        }

        //Read Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPeakCurrent, &errorInfo);
        }

        //Read Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usThermalTimeConstant, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->SetReturnParameterData(1, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->SetReturnParameterData(2, &usThermalTimeConstant, sizeof(usThermalTimeConstant));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetEcMotorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPolePair = 0;
    CErrorInfo errorInfo;
    WORD usContinuousCurrent = 0;
    WORD usIndex;
    WORD usPeakCurrent = 0;
    WORD usSubIndex;
    WORD usThermalTimeConstant = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read continuous current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_CON_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usContinuousCurrent, &errorInfo);
        }

        //Read Peak current
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_MAX_PEAK_CURRENT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPeakCurrent, &errorInfo);
        }

        //Read Thermal Time constant
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_THERMAL_TIME_CONSTANT;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usThermalTimeConstant, &errorInfo);
        }

        //Read pole pairs
        if(oResult)
        {
            usIndex = INDEX_MOTOR_DATA;
            usSubIndex = SUBINDEX_POLE_PAIR_NUMBER;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPolePair, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usContinuousCurrent, sizeof(usContinuousCurrent));
        p_pCommand->SetReturnParameterData(1, &usPeakCurrent, sizeof(usPeakCurrent));
        p_pCommand->SetReturnParameterData(2, &usThermalTimeConstant, sizeof(usThermalTimeConstant));
        p_pCommand->SetReturnParameterData(3, &ubPolePair, sizeof(ubPolePair));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usCounts = 0;
    WORD usIndex;
    WORD usPositionSensorType = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read encoder resolution
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_ENCODER_PULSE_NUMBER;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usCounts, &errorInfo);
        }

        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_TYPE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPositionSensorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usCounts, sizeof(usCounts));
        p_pCommand->SetReturnParameterData(1, &usPositionSensorType, sizeof(usPositionSensorType));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetSensorType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSensorType = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read sensor type
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_TYPE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usSensorType, sizeof(usSensorType));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetIncEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_INC_ENCODER_POLARITY = 0x0000;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulEncoderResolution = 0;
    WORD usIndex;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read data rate
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_ENCODER_PULSE_NUMBER;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulEncoderResolution, &errorInfo);
        }

        //Read sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(BIT_MASK_INC_ENCODER_POLARITY & usSensorPolarity)
                {
                    oInvertedPolarity = TRUE;
                }
                else
                {
                    oInvertedPolarity = FALSE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulEncoderResolution, sizeof(ulEncoderResolution));
        p_pCommand->SetReturnParameterData(1, &oInvertedPolarity, sizeof(oInvertedPolarity));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetHallSensorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_HALL_SENSOR_POLARITY = 0x0002;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(BIT_MASK_HALL_SENSOR_POLARITY & usSensorPolarity)
                {
                    oInvertedPolarity = TRUE;
                }
                else
                {
                    oInvertedPolarity = FALSE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oInvertedPolarity, sizeof(oInvertedPolarity));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetSsiAbsEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_SSI_ENCODER_POLARITY = 0x0004;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDataRate = 0;
    WORD usIndex;
    WORD usNbOfDataBits = 0;
    WORD usNbOfMultiTurnDataBits = 0;
    WORD usNbOfSingleTurnDataBits = 0;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read data rate
        if(oResult)
        {
            usIndex = INDEX_SSI_ENCODER_CONFIGURATION;
            usSubIndex = SUBINDEX_SSI_ENCODER_DATA_RATE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usDataRate, &errorInfo);
        }

        //Read number data bits
        if(oResult)
        {
            usIndex = INDEX_SSI_ENCODER_CONFIGURATION;
            usSubIndex = SUBINDEX_SSI_ENCODER_NUMBER_OF_DATA_BITS;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usNbOfDataBits, &errorInfo);

            if(oResult)
            {
                //Multi turn data bits
                usNbOfMultiTurnDataBits = usNbOfDataBits;
                usNbOfMultiTurnDataBits >>= 8;

                //Single turn data bits
                usNbOfSingleTurnDataBits = (BYTE)usNbOfDataBits;
            }
        }

        //Read sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(BIT_MASK_SSI_ENCODER_POLARITY & usSensorPolarity)
                {
                    oInvertedPolarity = TRUE;
                }
                else
                {
                    oInvertedPolarity = FALSE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usDataRate, sizeof(usDataRate));
        p_pCommand->SetReturnParameterData(1, &usNbOfMultiTurnDataBits, sizeof(usNbOfMultiTurnDataBits));
        p_pCommand->SetReturnParameterData(2, &usNbOfSingleTurnDataBits, sizeof(usNbOfSingleTurnDataBits));
        p_pCommand->SetReturnParameterData(3, &oInvertedPolarity, sizeof(oInvertedPolarity));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usCounts;
    WORD usIndex;
    WORD usPositionSensorType;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usCounts, sizeof(usCounts));
        p_pCommand->GetParameterData(1, &usPositionSensorType, sizeof(usPositionSensorType));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write encoder resolution
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_ENCODER_PULSE_NUMBER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usCounts, &errorInfo);
        }

        //Write sensor type
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_TYPE;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPositionSensorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetSensorType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD SSI_ABS_ENCODER(4);
    const WORD SSI_ABS_ENCODER_BINARY(4);
    const WORD SSI_ABS_ENCODER_GREY(5);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usPositionSensorType(0);
    WORD usSensorType(0);
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usSensorType, sizeof(usSensorType));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_TYPE;

            if (usSensorType == SSI_ABS_ENCODER_BINARY || usSensorType == SSI_ABS_ENCODER_GREY)
            {
                usPositionSensorType = SSI_ABS_ENCODER;
            }
            else
            {
                usPositionSensorType = usSensorType;
            }

            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPositionSensorType, &errorInfo);
        }

        if (oResult && (usSensorType == SSI_ABS_ENCODER_BINARY || usSensorType == SSI_ABS_ENCODER_GREY))
        {
            oResult = SetSsiEncodingType(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usSensorType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetIncEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_INC_ENCODER_POLARITY_NORMAL = 0xFFFE;
    const WORD BIT_MASK_INC_ENCODER_POLARITY_INVERTED = 0x0001;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulEncoderResolution = 0;
    WORD usIndex;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulEncoderResolution, sizeof(ulEncoderResolution));
        p_pCommand->GetParameterData(1, &oInvertedPolarity, sizeof(oInvertedPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read data rate
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_ENCODER_PULSE_NUMBER;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulEncoderResolution, &errorInfo);
        }

        //Write sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(oInvertedPolarity)
                {
                    usSensorPolarity |= BIT_MASK_INC_ENCODER_POLARITY_INVERTED;
                }
                else
                {
                    usSensorPolarity &= BIT_MASK_INC_ENCODER_POLARITY_NORMAL;
                }

                usIndex = INDEX_SENSOR_CONFIGURATION;
                usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usSensorPolarity, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetHallSensorParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_HALL_SENSOR_POLARITY_NORMAL = 0xFFFD;
    const WORD BIT_MASK_HALL_SENSOR_POLARITY_INVERTED = 0x0002;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &oInvertedPolarity, sizeof(oInvertedPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(oInvertedPolarity)
                {
                    usSensorPolarity |= BIT_MASK_HALL_SENSOR_POLARITY_INVERTED;
                }
                else
                {
                    usSensorPolarity &= BIT_MASK_HALL_SENSOR_POLARITY_NORMAL;
                }

                usIndex = INDEX_SENSOR_CONFIGURATION;
                usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usSensorPolarity, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetSsiAbsEncoderParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_SSI_ENCODER_POLARITY_NORMAL = 0xFFFB;
    const WORD BIT_MASK_SSI_ENCODER_POLARITY_INVERTED = 0x0004;

    BOOL oInvertedPolarity = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDataRate = 0;
    WORD usIndex;
    WORD usNbOfDataBits = 0;
    WORD usNbOfMultiTurnDataBits = 0;
    WORD usNbOfSingleTurnDataBits = 0;
    WORD usSensorPolarity = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDataRate, sizeof(usDataRate));
        p_pCommand->GetParameterData(1, &usNbOfMultiTurnDataBits, sizeof(usNbOfMultiTurnDataBits));
        p_pCommand->GetParameterData(2, &usNbOfSingleTurnDataBits, sizeof(usNbOfSingleTurnDataBits));
        p_pCommand->GetParameterData(3, &oInvertedPolarity, sizeof(oInvertedPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write data rate
        if(oResult)
        {
            usIndex = INDEX_SSI_ENCODER_CONFIGURATION;
            usSubIndex = SUBINDEX_SSI_ENCODER_DATA_RATE;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usDataRate, &errorInfo);
        }

        //Write number data bits
        if(oResult)
        {
            //Multi turn data bits
            usNbOfDataBits = (BYTE)usNbOfMultiTurnDataBits;
            usNbOfDataBits <<= 8;

            //Single turn data bits
            usNbOfDataBits += (BYTE)usNbOfSingleTurnDataBits;

            usIndex = INDEX_SSI_ENCODER_CONFIGURATION;
            usSubIndex = SUBINDEX_SSI_ENCODER_NUMBER_OF_DATA_BITS;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usNbOfDataBits, &errorInfo);
        }

        //Write sensor polarity
        if(oResult)
        {
            usIndex = INDEX_SENSOR_CONFIGURATION;
            usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSensorPolarity, &errorInfo);

            if(oResult)
            {
                if(oInvertedPolarity)
                {
                    usSensorPolarity |= BIT_MASK_SSI_ENCODER_POLARITY_INVERTED;
                }
                else
                {
                    usSensorPolarity &= BIT_MASK_SSI_ENCODER_POLARITY_NORMAL;
                }

                usIndex = INDEX_SENSOR_CONFIGURATION;
                usSubIndex = SUBINDEX_POSITION_SENSOR_POLARITY;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usSensorPolarity, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetOperationMode(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    __int8 bMode;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &bMode, sizeof(bMode));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set Operation Mode
        if(oResult)
        {
            usIndex = INDEX_OPERATION_MODE;
            usSubIndex = SUBINDEX_OPERATION_MODE;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, bMode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetOperationMode(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    __int8 bMode = 0;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Operation Mode
        if(oResult)
        {
            usIndex = INDEX_OPERATION_MODE_DISPLAY;
            usSubIndex = SUBINDEX_OPERATION_MODE_DISPLAY;
            oResult = ReadInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &bMode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &bMode, sizeof(bMode));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ClearFault(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_FAULT(0x004F);

    BOOL oFault(TRUE);
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);
    WORD usIndex(0);
    WORD usSubIndex(0);

    if(p_pCommand)
    {
        //Init
        usIndex = INDEX_CONTROL_WORD;
        usSubIndex = SUBINDEX_CONTROL_WORD;

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Check Fault state
        if(oResult)
        {
            oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);
            if(oResult)
            {
                usDeviceState &= BIT_MASK_FAULT;
                oFault = (usDeviceState == DS_FAULT);
            }
        }
        //Clear fault 0
        if(oResult && oFault)
        {
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, DISABLE_VOLTAGE, &errorInfo);
        }
        //Clear fault 1
        if(oResult && oFault)
        {
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, FAULT_RESET, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetDisableState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_DISABLE = 0x004F;

    BOOL oIsDisabled;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Disable state
        if(oResult)
        {
            oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);

            if(oResult)
            {
                usDeviceState &= BIT_MASK_DISABLE;
                if(usDeviceState == DS_SWITCH_ON_DISABLED)
                    oIsDisabled = TRUE;
                else
                    oIsDisabled = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oIsDisabled, sizeof(oIsDisabled));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetEnableState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_ENABLE = 0x006F;

    BOOL oIsEnabled;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read enable state
        if(oResult)
        {
            oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);

            if(oResult)
            {
                usDeviceState &= BIT_MASK_ENABLE;
                if(usDeviceState == DS_OPERATION_ENABLED)
                    oIsEnabled = TRUE;
                else
                    oIsEnabled = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oIsEnabled, sizeof(oIsEnabled));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetFaultState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_FAULT = 0x004F;

    BOOL oIsInFault;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Fault state
        if(oResult)
        {
            oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);

            if(oResult)
            {
                usDeviceState &= BIT_MASK_FAULT;
                if(usDeviceState == DS_FAULT)
                    oIsInFault = TRUE;
                else
                    oIsInFault = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oIsInFault, sizeof(oIsInFault));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetQuickStopState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD BIT_MASK_QUICK_STOP = 0x006F;

    BOOL oIsQuickStopActive = FALSE;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Quick state
        oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);

        if(oResult)
        {
            usDeviceState &= BIT_MASK_QUICK_STOP;
            if(usDeviceState == DS_QUICK_STOP_ACTIV)
                oIsQuickStopActive = TRUE;
            else
                oIsQuickStopActive = FALSE;
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oIsQuickStopActive, sizeof(oIsQuickStopActive));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetDisableState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set Disable
        if(oResult) oResult = SetDisableState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetEnableState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Device State
        if(oResult) oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            switch(usDeviceState)
            {
                case DS_OPERATION_ENABLED:
                    oResult = TRUE;
                    break;

                case DS_SWITCH_ON_DISABLED:
                    if(oResult)
                        oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, SHUTDOWN, &errorInfo);
                    if(oResult)
                        oResult = CheckDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, DS_READY_TO_SWITCH_ON, &errorInfo);

                case DS_READY_TO_SWITCH_ON:
                case DS_SWITCHED_ON:
                case DS_QUICK_STOP_ACTIV:
                    if(oResult)
                        oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, SWITCH_ON_AND_ENABLE_OPERATION, &errorInfo);
                    if(oResult)
                        oResult = CheckDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, DS_OPERATION_ENABLED, &errorInfo);
                    break;
                default:
                    oResult = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetQuickStopState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Device State
        if(oResult) oResult = ReadDeviceState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &usDeviceState, &errorInfo);
        if(oResult)
        {
            if(((usDeviceState & DS_QUICK_STOP_ACTIV) || (usDeviceState & DS_OPERATION_ENABLED))&&(oResult))
            {
                if((usDeviceState & DS_OPERATION_ENABLED) == DS_OPERATION_ENABLED)
                {
                    //Set Quick Stop
                    usIndex = INDEX_CONTROL_WORD;
                    usSubIndex = SUBINDEX_CONTROL_WORD;
                    oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, QUICK_STOP, &errorInfo);
                }
                else
                {
                    oResult = TRUE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetCurrentIs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    short sCurrentIs;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //CurrentIs
        if(oResult)
        {
            usIndex = INDEX_CURRENT_ACTUAL_VALUE;
            usSubIndex = SUBINDEX_CURRENT_ACTUAL_VALUE;
            oResult = ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &sCurrentIs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &sCurrentIs, sizeof(sCurrentIs));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetCurrentIsAveraged(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    short sCurrentIsAveraged;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //CurrentIsAveraged
        if(oResult)
        {
            usIndex = INDEX_CURRENT_ACTUAL_VALUE_AVERAGED;
            usSubIndex = SUBINDEX_CURRENT_ACTUAL_VALUE_AVERAGED;
            oResult = ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &sCurrentIsAveraged, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &sCurrentIsAveraged, sizeof(sCurrentIsAveraged));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetMovementState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BOOL oTargetReached;
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDeviceState(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //TargetReached
        if(oResult)
        {
            //Read Device State
            usIndex = INDEX_STATUS_WORD;
            usSubIndex = SUBINDEX_STATUS_WORD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usDeviceState, &errorInfo);
            if(oResult)
            {
                usDeviceState = usDeviceState & 0x7FF;

                //Check target reached
                usDeviceState &= DS_TARGET_REACHED;
                if(usDeviceState == DS_TARGET_REACHED)
                    oTargetReached = TRUE;
                else
                    oTargetReached = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oTargetReached, sizeof(oTargetReached));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionIs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lPositionIs = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //PositionIs
        if(oResult)
        {
            usIndex = INDEX_POSITION_ACTUAL_VALUE;
            usSubIndex = SUBINDEX_POSITION_ACTUAL_VALUE;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lPositionIs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lPositionIs, sizeof(lPositionIs));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityIs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lVelocityIs = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //VeloctiyIs
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_ACTUAL_VALUE;
            usSubIndex = SUBINDEX_VELOCITY_ACTUAL_VALUE;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lVelocityIs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lVelocityIs, sizeof(lVelocityIs));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityIsAveraged(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lVelocityIsAveraged = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //VelocityIsAveraged
        if(oResult)
        {
            usIndex = INDEX_VELOCITY_ACTUAL_VALUE_AVERAGED;
            usSubIndex = SUBINDEX_VELOCITY_ACTUAL_VALUE_AVERAGED;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lVelocityIsAveraged, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lVelocityIsAveraged, sizeof(lVelocityIsAveraged));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetCurrentMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    short sCurrentMust = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Current parameter must
        usIndex = INDEX_CURRENT_SETTING_VALUE;
        usSubIndex = SUBINDEX_CURRENT_SETTING_VALUE;
        if(oResult) oResult = ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &sCurrentMust, &errorInfo);

        //Set ReturnParameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &sCurrentMust, sizeof(sCurrentMust));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetCurrentMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    short sCurrentMust;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &sCurrentMust, sizeof(sCurrentMust));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Current parameter must
        usIndex = INDEX_CURRENT_SETTING_VALUE;
        usSubIndex = SUBINDEX_CURRENT_SETTING_VALUE;
        if(oResult) oResult = WriteInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, sCurrentMust, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ActivateAnalogCurrentSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD CURRENT_CONFIGURATION(0);
    const DWORD DECIMAL_PLACES(2);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubNotationIndex(0);
    CErrorInfo errorInfo;
    CMmcDataConversion dataConversion;
    CStdString strScaling(_T(""));
    float fScaling(0);
    short sOffset(0);
    WORD usAnalogInputNumber(0);
    WORD usFractionDen(0);
    short sFractionNum(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));
        p_pCommand->GetParameterData(1, &fScaling, sizeof(fScaling));
        p_pCommand->GetParameterData(2, &sOffset, sizeof(sOffset));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, CURRENT_CONFIGURATION, &errorInfo);
        }

        if(oResult) oResult = dataConversion.Float2DecFloatStr(fScaling, DECIMAL_PLACES, &strScaling);
        if (oResult)
        {
            oResult = dataConversion.DecFloatStr2ShortFractional(strScaling, &sFractionNum, &usFractionDen, FALSE);
            if (!oResult)
            {
                errorInfo.Init(ERROR_DEVICE_CAN_VALUE_RANGE);
            }
        }

        //Setpoint notation index
        if(oResult)
        {
            unsigned short usPositiveNotationIndex = 0;

            dataConversion.DecFloat2Short(log10((float)usFractionDen), usPositiveNotationIndex, false);
            ubNotationIndex = (BYTE)((-1) * usPositiveNotationIndex);

            usIndex = INDEX_ANALOG_CURRENT_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_CURRENT_NOTATION_INDEX;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubNotationIndex, &errorInfo);
        }

        //Setpoint scaling
        if(oResult)
        {
            usIndex = INDEX_ANALOG_CURRENT_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_CURRENT_SCALING;
            oResult = WriteInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, sFractionNum, &errorInfo);
        }

        //Setpoint offset
        if(oResult)
        {
            usIndex = INDEX_ANALOG_CURRENT_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_CURRENT_OFFSET;
            oResult = WriteInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, sOffset, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DeactivateAnalogCurrentSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAnalogInputNumber(0);
    WORD usConfiguration(15);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);

            while(!oResult && (usConfiguration != 0))
            {
                usConfiguration--;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_EnableAnalogCurrentSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_CURRENT_ON(0x0001);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask |= EXECUTION_MASK_CURRENT_ON;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DisableAnalogCurrentSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_CURRENT_OFF(0xFFFE);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask &= EXECUTION_MASK_CURRENT_OFF;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lVelocityMust(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Velocity must
        usIndex = INDEX_VELOCITY_SETTING_VALUE;
        usSubIndex = SUBINDEX_VELOCITY_SETTING_VALUE;
        if(oResult) oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lVelocityMust, &errorInfo);

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lVelocityMust, sizeof(lVelocityMust));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetVelocityMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lVelocityMust;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get parameter data
        p_pCommand->GetParameterData(0, &lVelocityMust, sizeof(lVelocityMust));

        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Velocity must
        usIndex = INDEX_VELOCITY_SETTING_VALUE;
        usSubIndex = SUBINDEX_VELOCITY_SETTING_VALUE;
        if(oResult) oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lVelocityMust, &errorInfo);

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ActivateAnalogVelocitySetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD VELOCITY_CONFIGURATION(1);
    const DWORD DECIMAL_PLACES(2);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubNotationIndex(0);
    CErrorInfo errorInfo;
    CMmcDataConversion dataConversion;
    CStdString strScaling(_T(""));
    float fScaling(0);
    int lOffset(0);
    WORD usAnalogInputNumber(0);
    WORD usFractionDen(0);
    short sFractionNum(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));
        p_pCommand->GetParameterData(1, &fScaling, sizeof(fScaling));
        p_pCommand->GetParameterData(2, &lOffset, sizeof(lOffset));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, VELOCITY_CONFIGURATION, &errorInfo);
        }

        if(oResult) oResult = dataConversion.Float2DecFloatStr(fScaling, DECIMAL_PLACES, &strScaling);
        if (oResult)
        {
            oResult = dataConversion.DecFloatStr2ShortFractional(strScaling, &sFractionNum, &usFractionDen, FALSE);
            if (!oResult)
            {
                errorInfo.Init(ERROR_DEVICE_CAN_VALUE_RANGE);
            }
        }

        //Setpoint notation index
        if(oResult)
        {
            unsigned short usPositiveNotationIndex = 0;

            dataConversion.DecFloat2Short(log10((float)usFractionDen), usPositiveNotationIndex, false);
            ubNotationIndex = (BYTE)((-1) * usPositiveNotationIndex);

            usIndex = INDEX_ANALOG_VELOCITY_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_VELOCITY_NOTATION_INDEX;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubNotationIndex, &errorInfo);
        }

        //Setpoint scaling
        if(oResult)
        {
            usIndex = INDEX_ANALOG_VELOCITY_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_VELOCITY_SCALING;
            oResult = WriteInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, sFractionNum, &errorInfo);
        }

        //Setpoint offset
        if(oResult)
        {
            usIndex = INDEX_ANALOG_VELOCITY_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_VELOCITY_OFFSET;
            oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lOffset, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DeactivateAnalogVelocitySetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAnalogInputNumber(0);
    WORD usConfiguration(15);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);

            while(!oResult && (usConfiguration != 0))
            {
                usConfiguration--;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_EnableAnalogVelocitySetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_VELOCITY_ON(0x0002);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask |= EXECUTION_MASK_VELOCITY_ON;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DisableAnalogVelocitySetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_VELOCITY_OFF(0xFFFD);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask &= EXECUTION_MASK_VELOCITY_OFF;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lPositionMust(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Position must
        if(oResult)
        {
            usIndex = INDEX_POSITION_SETTING_VALUE;
            usSubIndex = SUBINDEX_POSITION_SETTING_VALUE;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lPositionMust, &errorInfo);
        }

        // Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lPositionMust, sizeof(lPositionMust));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetPositionMust(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lPositionMust;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        // Get parameter data
        p_pCommand->GetParameterData(0, &lPositionMust, sizeof(lPositionMust));

        // Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        // Position must
        usIndex = INDEX_POSITION_SETTING_VALUE;
        usSubIndex = SUBINDEX_POSITION_SETTING_VALUE;
        if(oResult) oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lPositionMust, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ActivateAnalogPositionSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_CONFIGURATION(2);
    const DWORD DECIMAL_PLACES(2);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubNotationIndex(0);
    CErrorInfo errorInfo;
    CMmcDataConversion dataConversion;
    CStdString strScaling(_T(""));
    float fScaling(0);
    int lOffset(0);
    WORD usAnalogInputNumber(0);
    WORD usFractionDen(0);
    short sFractionNum(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));
        p_pCommand->GetParameterData(1, &fScaling, sizeof(fScaling));
        p_pCommand->GetParameterData(2, &lOffset, sizeof(lOffset));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, POSITION_CONFIGURATION, &errorInfo);
        }

        if(oResult) oResult = dataConversion.Float2DecFloatStr(fScaling, DECIMAL_PLACES, &strScaling);
        if (oResult)
        {
            oResult = dataConversion.DecFloatStr2ShortFractional(strScaling, &sFractionNum, &usFractionDen, FALSE);
            if (!oResult)
            {
                errorInfo.Init(ERROR_DEVICE_CAN_VALUE_RANGE);
            }
        }

        //Setpoint notation index
        if(oResult)
        {
            unsigned short usPositiveNotationIndex = 0;

            dataConversion.DecFloat2Short(log10((float)usFractionDen), usPositiveNotationIndex, false);
            ubNotationIndex = (BYTE)((-1) * usPositiveNotationIndex);

            usIndex = INDEX_ANALOG_POSITION_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_POSITION_NOTATION_INDEX;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubNotationIndex, &errorInfo);
        }

        //Setpoint scaling
        if(oResult)
        {
            usIndex = INDEX_ANALOG_POSITION_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_POSITION_SCALING;
            oResult = WriteInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, sFractionNum, &errorInfo);
        }

        //Setpoint offset
        if(oResult)
        {
            usIndex = INDEX_ANALOG_POSITION_SETPOINT_CONFIGURATION;
            usSubIndex = SUBINDEX_ANALOG_POSITION_OFFSET;
            oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lOffset, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DeactivateAnalogPositionSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAnalogInputNumber = 0;
    WORD usConfiguration = 15;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNumber, sizeof(usAnalogInputNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input configuration
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
            usSubIndex = usAnalogInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);

            while(!oResult && (usConfiguration != 0))
            {
                usConfiguration--;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_EnableAnalogPositionSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_POSITION_ON = 0x0004;

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask |= EXECUTION_MASK_POSITION_ON;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DisableAnalogPositionSetpoint(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD EXECUTION_MASK_POSITION_OFF = 0xFFFB;

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usExecutionMask = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Analog input execution mask
        usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
        usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
        if(oResult) oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);
        if(oResult)
        {
            usExecutionMask &= EXECUTION_MASK_POSITION_OFF;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_FindHome(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    __int8 bHomingMethod;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulTimeout;
    WORD usIndex;
    WORD usStartHoming;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &bHomingMethod, sizeof(bHomingMethod));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set Homing Method
        if(oResult)
        {
            usIndex = INDEX_HOMING_METHOD;
            usSubIndex = SUBINDEX_HOMING_METHOD;
            oResult = WriteInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, bHomingMethod, &errorInfo);
        }

        //Send 0->1 Bit Change
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            usStartHoming = 0x000F; //000F => Homing operation inturrupt start = 0 / Enable operation = F
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usStartHoming, &errorInfo);
        }

        if (oResult && p_pManager)
        {
            oResult = p_pManager->DCS_GetProtocolStackSettings(p_hDCS_Handle, NULL, &ulTimeout);
        }

        BOOL oHomingAttainedBitReset(FALSE);
        WORD usHomingState(0);
        DWORD ulTimeStart;
        DWORD ulTimeBitTestRunning;

        //Start Time
        ulTimeStart = MmcGetTickCount();
        ulTimeBitTestRunning = 0;

        usIndex = INDEX_CONTROL_WORD;
        usSubIndex = SUBINDEX_CONTROL_WORD;
        while(oResult && !oHomingAttainedBitReset && (ulTimeBitTestRunning < ulTimeout))
        {
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usHomingState, &errorInfo);
            //Check homing attained (Bit 12)
            usHomingState &= HS_HOMING_ATTAINED;
            if(usHomingState == HS_HOMING_ATTAINED)
            {
                oHomingAttainedBitReset = FALSE;
            }
            else
            {
                oHomingAttainedBitReset = TRUE;
            }
            ulTimeBitTestRunning = MmcGetTickCount() - ulTimeStart;
        }

        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            usStartHoming = 0x001F; //001F => Homing operation start = 1 / Enable operation = F
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usStartHoming, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetHomingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulHomingAcceleration = 0;
    DWORD ulSpeedIndex = 0;
    DWORD ulSpeedSwitch = 0;
    int lHomeOffset = 0;
    int lHomePosition = 0;
    WORD usCurrentThreshold = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //HomingAcceleration
        if(oResult)
        {
            usIndex = INDEX_HOMING_ACCELERATION;
            usSubIndex = SUBINDEX_HOMING_ACCELERATION;
            oResult= ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulHomingAcceleration, &errorInfo);
        }

        //SpeedSwitch
        if(oResult)
        {
            usIndex = INDEX_HOMING_SPEED;
            usSubIndex = SUBINDEX_SPEED_SEARCH_SWITCH;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulSpeedSwitch, &errorInfo);
        }

        //SpeedIndex
        if(oResult)
        {
            usIndex = INDEX_HOMING_SPEED;
            usSubIndex = SUBINDEX_SPEED_SEARCH_ZERO;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulSpeedIndex, &errorInfo);
        }

        //HomeOffset
        if(oResult)
        {
            usIndex = INDEX_HOME_OFFSET;
            usSubIndex = SUBINDEX_HOME_OFFSET;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, (DWORD*) &lHomeOffset, &errorInfo);
        }

        //CurrentThreshold
        if(oResult)
        {
            usIndex = INDEX_CURRENT_THRESHOLD;
            usSubIndex = SUBINDEX_CURRENT_THRESHOLD;
            ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usCurrentThreshold, &errorInfo);
        }

        //HomePosition
        if(oResult)
        {
            usIndex = INDEX_HOME_POSITION;
            usSubIndex = SUBINDEX_HOME_POSITION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, (DWORD*) &lHomePosition, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulHomingAcceleration, sizeof(ulHomingAcceleration));
        p_pCommand->SetReturnParameterData(1, &ulSpeedSwitch, sizeof(ulSpeedSwitch));
        p_pCommand->SetReturnParameterData(2, &ulSpeedIndex, sizeof(ulSpeedIndex));
        p_pCommand->SetReturnParameterData(3, &lHomeOffset, sizeof(lHomeOffset));
        p_pCommand->SetReturnParameterData(4, &usCurrentThreshold, sizeof(usCurrentThreshold));
        p_pCommand->SetReturnParameterData(5, &lHomePosition, sizeof(lHomePosition));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetHomingParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulHomingAcceleration = 0;
    DWORD ulSpeedIndex = 0;
    DWORD ulSpeedSwitch = 0;
    int lHomeOffset = 0;
    int lHomePosition = 0;
    WORD usCurrentThreshold = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &ulHomingAcceleration, sizeof(ulHomingAcceleration));
        p_pCommand->GetParameterData(1, &ulSpeedSwitch, sizeof(ulSpeedSwitch));
        p_pCommand->GetParameterData(2, &ulSpeedIndex, sizeof(ulSpeedIndex));
        p_pCommand->GetParameterData(3, &lHomeOffset, sizeof(lHomeOffset));
        p_pCommand->GetParameterData(4, &usCurrentThreshold, sizeof(usCurrentThreshold));
        p_pCommand->GetParameterData(5, &lHomePosition, sizeof(lHomePosition));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //HomingAcceleration
        if(oResult)
        {
            usIndex = INDEX_HOMING_ACCELERATION;
            usSubIndex = SUBINDEX_HOMING_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulHomingAcceleration, &errorInfo);
        }

        //SpeedSwitch
        if(oResult)
        {
            usIndex = INDEX_HOMING_SPEED;
            usSubIndex = SUBINDEX_SPEED_SEARCH_SWITCH;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulSpeedSwitch, &errorInfo);
        }

        //SpeedIndex
        if(oResult)
        {
            usIndex = INDEX_HOMING_SPEED;
            usSubIndex = SUBINDEX_SPEED_SEARCH_ZERO;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulSpeedIndex, &errorInfo);
        }

        //HomeOffset
        if(oResult)
        {
            usIndex = INDEX_HOME_OFFSET;
            usSubIndex = SUBINDEX_HOME_OFFSET;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, (DWORD)lHomeOffset, &errorInfo);
        }

        //CurrentThreshold
        if(oResult)
        {
            usIndex = INDEX_CURRENT_THRESHOLD;
            usSubIndex = SUBINDEX_CURRENT_THRESHOLD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usCurrentThreshold, &errorInfo);
        }

        //HomePosition
        if(oResult)
        {
            usIndex = INDEX_HOME_POSITION;
            usSubIndex = SUBINDEX_HOME_POSITION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, (DWORD)lHomePosition, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DefinePosition(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lHomePosition = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &lHomePosition, sizeof(lHomePosition));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //HomePosition
        if(oResult)
        {
            usIndex = INDEX_HOME_POSITION;
            usSubIndex = SUBINDEX_HOME_POSITION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, (DWORD)lHomePosition, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_StopHoming(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex(INDEX_CONTROL_WORD);
    WORD usSubIndex(SUBINDEX_CONTROL_WORD);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, STOP_HOMING, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetHomingState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oHomingAttained;
    BOOL oHomingError;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usHomingState(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Homing Attained
        if(oResult)
        {
            //Read Homing State
            usIndex = INDEX_STATUS_WORD;
            usSubIndex = SUBINDEX_STATUS_WORD;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usHomingState, &errorInfo);
            if(oResult)
            {
                //Check homing attained
                usHomingState &= HS_HOMING_ATTAINED;
                if(usHomingState == HS_HOMING_ATTAINED)
                {
                    oHomingAttained = TRUE;
                }
                else
                {
                    oHomingAttained = FALSE;
                }

                //Check homing error
                usHomingState &= HS_HOMING_ERROR;
                if(usHomingState == HS_HOMING_ERROR)
                {
                    oHomingError = TRUE;
                }
                else
                {
                    oHomingError = FALSE;
                }
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &oHomingAttained, sizeof(oHomingAttained));
        p_pCommand->SetReturnParameterData(1, &oHomingError, sizeof(oHomingError));

    }
    else
    {
        oResult = FALSE;
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionProfile(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulProfileAcceleration;
    DWORD ulProfileDeceleration;
    DWORD ulProfileVelocity;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_PROFILE_VELOCITY;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulProfileVelocity, &errorInfo);
        }

        if(oResult)
        {
            usIndex = INDEX_PROFILE_ACCELERATION;
            usSubIndex = SUBINDEX_PROFILE_ACCELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulProfileAcceleration, &errorInfo);
        }

        if(oResult)
        {
            usIndex = INDEX_PROFILE_DECELERATION;
            usSubIndex = SUBINDEX_PROFILE_DECELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulProfileDeceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulProfileVelocity, sizeof(ulProfileVelocity));
        p_pCommand->SetReturnParameterData(1, &ulProfileAcceleration, sizeof(ulProfileAcceleration));
        p_pCommand->SetReturnParameterData(2, &ulProfileDeceleration, sizeof(ulProfileDeceleration));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetTargetPosition(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lTargetPosition;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_TARGET_POSITION;
            usSubIndex = SUBINDEX_TARGET_POSITION;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lTargetPosition, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lTargetPosition, sizeof(lTargetPosition));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_HaltPositionMovement(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        usIndex = INDEX_CONTROL_WORD;
        usSubIndex = SUBINDEX_CONTROL_WORD;
        if(oResult) oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, STOP_PROFILE, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::SetTargetPosition(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubNodeId, int p_lTargetPosition, CErrorInfo* p_ErrorInfo)
{
    BOOL oResult(FALSE);
    WORD usIndex = 0;
    WORD usSubIndex = 0;

    usIndex = INDEX_TARGET_POSITION;
    usSubIndex = SUBINDEX_TARGET_POSITION;

    oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, p_lTargetPosition, p_ErrorInfo);

    return oResult;
}

BOOL CGatewayDriveToEpos2::SetNewSetPoint(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubNodeId, BOOL p_oAbsolute, BOOL p_oImmediately, CErrorInfo* p_ErrorInfo)
{
    BOOL oResult(FALSE);
    WORD usIndex = 0;
    WORD usSubIndex = 0;

    usIndex = INDEX_CONTROL_WORD;
    usSubIndex = SUBINDEX_CONTROL_WORD;

    if(p_oAbsolute)
    {
        if(p_oImmediately)
        {
            //Send absolute and immediately
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, IMMEDIATE_ABS_POSITION, p_ErrorInfo);
        }
        else
        {
            //Send absolute
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, ABSOLUTE_POSITION, p_ErrorInfo);
        }
    }
    else
    {
        if(p_oImmediately)
        {
            //Send relative and immediately
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, IMMEDIATE_REL_POSITION, p_ErrorInfo);
        }
        else
        {
            //Send relative
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, RELATIVE_POSITION, p_ErrorInfo);
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::ResetNewSetPoint(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubNodeId, CErrorInfo* p_ErrorInfo)
{
    BOOL oResult(FALSE);
    WORD usIndex = 0;
    WORD usSubIndex = 0;

    usIndex = INDEX_CONTROL_WORD;
    usSubIndex = SUBINDEX_CONTROL_WORD;

    oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usIndex, usSubIndex, START_PROFILE, p_ErrorInfo);

    return oResult;
}

BOOL CGatewayDriveToEpos2::IsSetPointAckActive(WORD p_usStatusWord)
{
    const int SET_POINT_ACK_ACTIVE_BIT = 12;

    BOOL oResult(FALSE);
    bitset<sizeof(WORD)*8> statusWordBits(p_usStatusWord);

    oResult = statusWordBits.test(SET_POINT_ACK_ACTIVE_BIT);

    return oResult;
}

BOOL CGatewayDriveToEpos2::WaitForSetPointAcknowledge(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubNodeId, CErrorInfo* p_ErrorInfo)
{
    const DWORD MINIMAL_WAIT_TIME = 1;

    BOOL oNoTimeout = TRUE;
    BOOL oResult(FALSE);
    CMmcHiResTimer timer;
    DWORD ulTimeout = 0;
    DWORD ulTransferRate = 0;
    WORD usStatusWord = 0;

    if(p_pManager && p_hDCS_Handle)
    {
        p_pManager->DCS_GetProtocolStackSettings(p_hDCS_Handle, &ulTransferRate, &ulTimeout);

        timer.start();

        while((oResult = ReadStatusWord(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)p_ubNodeId, usStatusWord, p_ErrorInfo)) && (oNoTimeout = timer.getElapsedTimeInMilliSec() < ulTimeout))
        {
            if(IsSetPointAckActive(usStatusWord))
            {
                break;
            }

            MmcSleep(MINIMAL_WAIT_TIME);
        }

        if(!oNoTimeout)
        {
            oResult = FALSE;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_MoveToPosition(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oAbsolute;
    BOOL oImmediately;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lTargetPosition;

    if(p_pCommand)
    {
        //Get Parameterdata
        p_pCommand->GetParameterData(0, &lTargetPosition, sizeof(lTargetPosition));
        p_pCommand->GetParameterData(1, &oAbsolute, sizeof(oAbsolute));
        p_pCommand->GetParameterData(2, &oImmediately, sizeof(oImmediately));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            oResult = SetTargetPosition(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, lTargetPosition, &errorInfo);
        }

        if(oResult)
        {
            oResult = SetNewSetPoint(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, oAbsolute, oImmediately, &errorInfo);
        }

        if(oResult)
        {
            WaitForSetPointAcknowledge(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);
        }

        if(oResult)
        {
            oResult = ResetNewSetPoint(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetPositionProfile(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulProfileAcceleration;
    DWORD ulProfileDeceleration;
    DWORD ulProfileVelocity;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &ulProfileVelocity, sizeof(ulProfileVelocity));
        p_pCommand->GetParameterData(1, &ulProfileAcceleration, sizeof(ulProfileAcceleration));
        p_pCommand->GetParameterData(2, &ulProfileDeceleration, sizeof(ulProfileDeceleration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            //Send velocity
            usIndex = INDEX_PROFILE_VELOCITY;
            usSubIndex = SUBINDEX_PROFILE_VELOCITY;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulProfileVelocity, &errorInfo);
        }

        if(oResult)
        {
            //Send acceleration
            usIndex = INDEX_PROFILE_ACCELERATION;
            usSubIndex = SUBINDEX_PROFILE_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulProfileAcceleration, &errorInfo);
        }

        if(oResult)
        {
            //Send deceleration
            usIndex = INDEX_PROFILE_DECELERATION;
            usSubIndex = SUBINDEX_PROFILE_DECELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulProfileDeceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_EnablePositionWindow(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulPositionWindow;
    WORD usIndex;
    WORD usPositionWindowTime;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &ulPositionWindow, sizeof(ulPositionWindow));
        p_pCommand->GetParameterData(1, &usPositionWindowTime, sizeof(usPositionWindowTime));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            //Send position window
            usIndex = INDEX_POSITION_WINDOW;
            usSubIndex = SUBINDEX_POSITION_WINDOW;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulPositionWindow, &errorInfo);
        }

        if(oResult)
        {
            //Send position window time
            usIndex = INDEX_POSITION_WINDOW_TIME;
            usSubIndex = SUBINDEX_POSITION_WINDOW_TIME;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPositionWindowTime, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DisablePositionWindow(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulPositionWindow = (DWORD)(numeric_limits<unsigned int>::max)();
    WORD usIndex;
    WORD usPositionWindowTime = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            //Send position window
            usIndex = INDEX_POSITION_WINDOW;
            usSubIndex = SUBINDEX_POSITION_WINDOW;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulPositionWindow, &errorInfo);
        }

        if(oResult)
        {
            //Send position window time
            usIndex = INDEX_POSITION_WINDOW_TIME;
            usSubIndex = SUBINDEX_POSITION_WINDOW_TIME;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPositionWindowTime, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetTargetVelocity(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lTargetVelocity = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read target velocity
        if(oResult)
        {
            usIndex = INDEX_TARGET_VELOCITY;
            usSubIndex = SUBINDEX_TARGET_VELOCITY;
            oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lTargetVelocity, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lTargetVelocity, sizeof(lTargetVelocity));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVelocityProfile(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulProfileAcceleration = 0;
    DWORD ulProfileDeceleration = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read ProfileAcceleration
        if(oResult)
        {
            usIndex = INDEX_PROFILE_ACCELERATION;
            usSubIndex = SUBINDEX_PROFILE_ACCELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulProfileAcceleration, &errorInfo);
        }

        //Read ProfileDeceleration
        if(oResult)
        {
            usIndex = INDEX_PROFILE_DECELERATION;
            usSubIndex = SUBINDEX_PROFILE_DECELERATION;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulProfileDeceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulProfileAcceleration, sizeof(ulProfileAcceleration));
        p_pCommand->SetReturnParameterData(1, &ulProfileDeceleration, sizeof(ulProfileDeceleration));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_HaltVelocityMovement(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Send Stop
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, STOP_PROFILE, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_MoveWithVelocity(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lTargetVelocity;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Set parameter data
        p_pCommand->GetParameterData(0, &lTargetVelocity, sizeof(lTargetVelocity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Send target velocity
        if(oResult)
        {
            usIndex = INDEX_TARGET_VELOCITY;
            usSubIndex = SUBINDEX_TARGET_VELOCITY;
            oResult = WriteInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, lTargetVelocity, &errorInfo);
        }

        //Move
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, MOVE_WITH_VELOCITY, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetVelocityProfile(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulProfileAcceleration;
    DWORD ulProfileDeceleration;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulProfileAcceleration, sizeof(ulProfileAcceleration));
        p_pCommand->GetParameterData(1, &ulProfileDeceleration, sizeof(ulProfileDeceleration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set Acceleration
        if(oResult)
        {
            usIndex = INDEX_PROFILE_ACCELERATION;
            usSubIndex = SUBINDEX_PROFILE_ACCELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulProfileAcceleration, &errorInfo);
        }

        //Set Decceleration
        if(oResult)
        {
            usIndex = INDEX_PROFILE_DECELERATION;
            usSubIndex = SUBINDEX_PROFILE_DECELERATION;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulProfileDeceleration, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_EnableVelocityWindow(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulVelocityWindow;
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityWindowTime;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();
        p_pCommand->GetParameterData(0, &ulVelocityWindow, sizeof(ulVelocityWindow));
        p_pCommand->GetParameterData(1, &usVelocityWindowTime, sizeof(usVelocityWindowTime));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            //Send Velocity window
            usIndex = INDEX_VELOCITY_WINDOW;
            usSubIndex = SUBINDEX_VELOCITY_WINDOW;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulVelocityWindow, &errorInfo);
        }

        if(oResult)
        {
            //Send Velocity window time
            usIndex = INDEX_VELOCITY_WINDOW_TIME;
            usSubIndex = SUBINDEX_VELOCITY_WINDOW_TIME;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usVelocityWindowTime, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DisableVelocityWindow(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulVelocityWindow = (DWORD)(numeric_limits<unsigned int>::max)();
    WORD usIndex;
    WORD usSubIndex;
    WORD usVelocityWindowTime = 0;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->ResetStatus();

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult)
        {
            //Send Velocity window
            usIndex = INDEX_VELOCITY_WINDOW;
            usSubIndex = SUBINDEX_VELOCITY_WINDOW;
            oResult = WriteUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ulVelocityWindow, &errorInfo);
        }

        if(oResult)
        {
            //Send Velocity window time
            usIndex = INDEX_VELOCITY_WINDOW_TIME;
            usSubIndex = SUBINDEX_VELOCITY_WINDOW_TIME;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usVelocityWindowTime, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DigitalInputConfiguration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oExecutionMask;
    BOOL oMask;
    BOOL oPolarity;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usConfiguration;
    WORD usDigitalInputNb;
    WORD usIndex, usSubIndex, usValue;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalInputNb, sizeof(usDigitalInputNb));
        p_pCommand->GetParameterData(1, &usConfiguration, sizeof(usConfiguration));
        p_pCommand->GetParameterData(2, &oMask, sizeof(oMask));
        p_pCommand->GetParameterData(3, &oPolarity, sizeof(oPolarity));
        p_pCommand->GetParameterData(4, &oExecutionMask, sizeof(oExecutionMask));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write Digital Input Configuration
        usIndex = INDEX_DIGITAL_INPUT_CONFIGURATION;
        usSubIndex = usDigitalInputNb;
        usValue = usConfiguration;
        if(oResult) oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oMask);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oPolarity);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_EXECUTION_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oExecutionMask);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DigitalOutputConfiguration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oMask;
    BOOL oPolarity;
    BOOL oResult(FALSE);
    BOOL oState;
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usConfiguration;
    WORD usDigitalOutputNb;
    WORD usIndex, usSubIndex, usValue;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalOutputNb, sizeof(usDigitalOutputNb));
        p_pCommand->GetParameterData(1, &usConfiguration, sizeof(usConfiguration));
        p_pCommand->GetParameterData(2, &oState, sizeof(oState));
        p_pCommand->GetParameterData(3, &oMask, sizeof(oMask));
        p_pCommand->GetParameterData(4, &oPolarity, sizeof(oPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write Digital Output Configuration
        usIndex = INDEX_DIGITAL_OUTPUT_CONFIGURATION;
        usSubIndex = usDigitalOutputNb;
        usValue = usConfiguration;
        if(oResult) oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_STATE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oState);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oMask);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oPolarity);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_AnalogInputConfiguration(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oExecutionMask;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAnalogInputNb;
    WORD usConfiguration;
    WORD usIndex, usSubIndex, usValue;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usAnalogInputNb, sizeof(usAnalogInputNb));
        p_pCommand->GetParameterData(1, &usConfiguration, sizeof(usConfiguration));
        p_pCommand->GetParameterData(2, &oExecutionMask, sizeof(oExecutionMask));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write Analog Input Configuration
        usIndex = INDEX_ANALOG_INPUT_CONFIGURATION;
        usSubIndex = usAnalogInputNb;
        usValue = usConfiguration;
        if(oResult) oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);

        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_ANALOG_INPUT_EXECUTION_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usValue, &errorInfo);
            if(oResult)
            {
                usValue = GetNewBitMask(usConfiguration, usValue, oExecutionMask);
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usValue, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetAllDigitalInputs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex, usSubIndex;
    WORD usInputs = 0;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read state of all digital inputs
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_STATE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usInputs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usInputs, sizeof(usInputs));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetAllDigitalOutputs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usOutputs = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read state of all digital outputs
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_STATE;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usOutputs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usOutputs, sizeof(usOutputs));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetAnalogInput(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usAnalog = 0;
    WORD usIndex;
    WORD usNumber;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usNumber, sizeof(usNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read value of one analog input
        if(oResult)
        {
            usIndex = INDEX_ANALOG_INPUTS;
            usSubIndex = usNumber;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usAnalog, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usAnalog, sizeof(usAnalog));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetAnalogInputVoltage(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    long lVoltageValue(0);
    short sVoltageValue(0);
    WORD usIndex(0);
    WORD usNumber(0);
    WORD usSubIndex(0);

    if (p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usNumber, sizeof(usNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read value of one analog input
        if (oResult)
        {
            usIndex = INDEX_ANALOG_INPUTS;
            usSubIndex = usNumber;
            oResult = ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &sVoltageValue, &errorInfo);
        }

        //Convert
        lVoltageValue = (long)sVoltageValue;

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lVoltageValue, sizeof(lVoltageValue));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetAnalogInputState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lStateValue(0);
    short sStateValue(0);
    WORD usConfiguration(0);
    WORD usNumber(0);

    if (p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usConfiguration, sizeof(usConfiguration));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        switch (usConfiguration)
        {
            case AIC_CURRENT_SET_POINT:
            {
                if (ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, INDEX_ANALOG_CURRENT_SETPOINT_CONFIGURATION, SUBINDEX_ANALOG_CURRENT_SETPOINT, &sStateValue, &errorInfo))
                {
                    lStateValue = (int)sStateValue;
                    oResult = TRUE;
                }
            }; break;

            case AIC_VELOCITY_SET_POINT:
            {
                if (ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, INDEX_ANALOG_VELOCITY_SETPOINT_CONFIGURATION, SUBINDEX_ANALOG_VELOCITY_SETPOINT, &lStateValue, &errorInfo))
                {
                    oResult = TRUE;
                }
            }; break;

            case AIC_POSITION_SET_POINT:
            {
                if(ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, INDEX_ANALOG_POSITION_SETPOINT_CONFIGURATION, SUBINDEX_ANALOG_POSITION_SETPOINT, &lStateValue, &errorInfo))
                {
                    oResult = TRUE;
                }
            }; break;

            case AIC_GENERAL_PURPOSE_A:
            case AIC_GENERAL_PURPOSE_B:
            case AIC_GENERAL_PURPOSE_C:
            case AIC_GENERAL_PURPOSE_D:
            case AIC_GENERAL_PURPOSE_E:
            case AIC_GENERAL_PURPOSE_F:
            case AIC_GENERAL_PURPOSE_G:
            case AIC_GENERAL_PURPOSE_H:
            {
                if (GetAnalogInputNumber(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usConfiguration, usNumber, &errorInfo))
                {
                    if (ReadInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, INDEX_ANALOG_INPUTS, usNumber, &sStateValue, &errorInfo))
                    {
                        lStateValue = (int)sStateValue;
                        oResult = TRUE;
                    }
                }
            }; break;

            default:
            {
                errorInfo.Init(ERROR_VIRTUAL_DEVICE_CONFIG_NOT_SUPPORTED);
                oResult = FALSE;
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lStateValue, sizeof(lStateValue));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetAllDigitalOutputs(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usOutputs;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usOutputs, sizeof(usOutputs));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write state of all digital outputs
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_OUTPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_OUTPUT_STATE;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usOutputs, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetAnalogOutput(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usNumber;
    WORD usOutputValue;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usNumber, sizeof(usNumber));
        p_pCommand->GetParameterData(1, &usOutputValue, sizeof(usOutputValue));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write state of all digital outputs
        if(oResult)
        {
            usIndex = INDEX_ANALOG_OUTPUTS;
            usSubIndex = usNumber -1;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usOutputValue, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetAnalogOutputVoltage(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    long lOutputValue;
    WORD usIndex;
    WORD usNumber;
    WORD usOutputValue;
    WORD usSubIndex;

    if (p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usNumber, sizeof(usNumber));
        p_pCommand->GetParameterData(1, &lOutputValue, sizeof(lOutputValue));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Write state of all digital outputs
        if (oResult)
        {
            usIndex = INDEX_ANALOG_OUTPUTS;
            usSubIndex = usNumber - 1;
            usOutputValue = (WORD)lOutputValue;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usOutputValue, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetPositionMarkerParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPositionMarkerEdgeType = 0;
    BYTE ubPositionMarkerMode = 0;
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubPositionMarkerEdgeType, sizeof(ubPositionMarkerEdgeType));
        p_pCommand->GetParameterData(1, &ubPositionMarkerMode, sizeof(ubPositionMarkerMode));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Postition marker edge type
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_EDGE_TYPE;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPositionMarkerEdgeType, &errorInfo);
        }

        //Postition marker mode
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_MODE;
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, ubPositionMarkerMode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetPositionMarkerParameter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubPositionMarkerEdgeType = 0;
    BYTE ubPositionMarkerMode = 0;
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Postition marker edge type
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_EDGE_TYPE;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPositionMarkerEdgeType, &errorInfo);
        }

        //Postition marker mode
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_MODE;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubPositionMarkerMode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ubPositionMarkerEdgeType, sizeof(ubPositionMarkerEdgeType));
        p_pCommand->SetReturnParameterData(1, &ubPositionMarkerMode, sizeof(ubPositionMarkerMode));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ActivatePositionMarker(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_MARKER_CONFIGURATION(0x0003);
    const WORD POSITION_MARKER_MASK(0x0008);

    BOOL oPolarity;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usDigitalInputNumber;
    WORD usExecutionMask;
    WORD usIndex;
    WORD usMask;
    WORD usPolarity;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalInputNumber, sizeof(usDigitalInputNumber));
        p_pCommand->GetParameterData(1, &oPolarity, sizeof(oPolarity));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Digital input functionalities
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_CONFIGURATION;
            usSubIndex = usDigitalInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, POSITION_MARKER_CONFIGURATION, &errorInfo);
        }

        //Set digital input mask
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMask, &errorInfo);

            if(oResult)
            {
                usMask |= POSITION_MARKER_MASK;

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMask, &errorInfo);
            }
        }

        //Set digital input polarity
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_POLARITY;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPolarity, &errorInfo);

            if(oResult)
            {
                if(oPolarity)
                {
                    usPolarity |= POSITION_MARKER_MASK;
                }
                else
                {
                    usPolarity &= 0xFFF7;
                }

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPolarity, &errorInfo);
            }
        }

        //Set digital input execution mask
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_EXECUTION_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usExecutionMask, &errorInfo);

            if(oResult)
            {
                usExecutionMask |= POSITION_MARKER_MASK;

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usExecutionMask, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_DeactivatePositionMarker(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_MARKER_MASK(0xFFF7);

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usConfiguration(15);
    WORD usDigitalInputNumber(0);
    WORD usIndex;
    WORD usMask(0);
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usDigitalInputNumber, sizeof(usDigitalInputNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set digital output configuration
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_CONFIGURATION;
            usSubIndex = usDigitalInputNumber;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);

            while(!oResult && (usConfiguration != 0))
            {
                usConfiguration--;
                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usConfiguration, &errorInfo);
            }
        }

        //Set digital input mask
        if(oResult)
        {
            usIndex = INDEX_DIGITAL_INPUT_FUNCTIONALITIES;
            usSubIndex = SUBINDEX_DIGITAL_INPUT_MASK;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usMask, &errorInfo);

            if(oResult)
            {
                usMask |= POSITION_MARKER_MASK;

                oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usMask, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadPositionMarkerCounter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usPositionMarkerCounter;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Position marker counter
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_COUNTER;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usPositionMarkerCounter, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usPositionMarkerCounter, sizeof(usPositionMarkerCounter));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadPositionMarkerCapturedPosition(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD POSITION_MARKER_OFFSET = 0x0004;

    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    int lPositionMarkerCapturedPosition(0);
    WORD usCounterIndex;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usCounterIndex, sizeof(usCounterIndex));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Position marker captured position
        if(oResult)
        {
            if(usCounterIndex > 0)
            {
                usIndex = INDEX_POSITION_MARKER;
                usSubIndex = POSITION_MARKER_OFFSET + usCounterIndex;
                oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lPositionMarkerCapturedPosition, &errorInfo);
            }
            else
            {
                usIndex = INDEX_POSITION_MARKER;
                usSubIndex = SUBINDEX_POSITION_MARKER_CAPTURED_POSITION;
                oResult = ReadInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &lPositionMarkerCapturedPosition, &errorInfo);
            }
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &lPositionMarkerCapturedPosition, sizeof(lPositionMarkerCapturedPosition));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ResetPositionMarkerCounter(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usPositionMarkerCounter = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Position marker counter
        if(oResult)
        {
            usIndex = INDEX_POSITION_MARKER;
            usSubIndex = SUBINDEX_POSITION_MARKER_COUNTER;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, usPositionMarkerCounter, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetVersion(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usApplicationNumber = 0;
    WORD usApplicationVersion = 0;
    WORD usHardwareVersion = 0;
    WORD usIndex;
    WORD usSoftwareVersion = 0;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read HardwareVersion
        if(oResult)
        {
            usIndex = INDEX_VERSION;
            usSubIndex = SUBINDEX_HARDWARE_VERSION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usHardwareVersion, &errorInfo);
        }

        //Read SoftwareVersion
        if(oResult)
        {
            usIndex = INDEX_VERSION;
            usSubIndex = SUBINDEX_SOFTWARE_VERSION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usSoftwareVersion, &errorInfo);
        }

        //Read ApplicationNumber
        if(oResult)
        {
            usIndex = INDEX_VERSION;
            usSubIndex = SUBINDEX_APPLICATION_NUMBER;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usApplicationNumber, &errorInfo);
        }

        //Read ApplicationVersion
        if(oResult)
        {
            usIndex = INDEX_VERSION;
            usSubIndex = SUBINDEX_APPLICATION_VERSION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usApplicationVersion, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usHardwareVersion, sizeof(usHardwareVersion));
        p_pCommand->SetReturnParameterData(1, &usSoftwareVersion, sizeof(usSoftwareVersion));
        p_pCommand->SetReturnParameterData(2, &usApplicationNumber, sizeof(usApplicationNumber));
        p_pCommand->SetReturnParameterData(3, &usApplicationVersion, sizeof(usApplicationVersion));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetDeviceType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulDeviceType = 0;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Device Type
        if(oResult)
        {
            usIndex = INDEX_DEVICE_TYPE;
            usSubIndex = SUBINDEX_DEVICE_TYPE;
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ulDeviceType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulDeviceType, sizeof(ulDeviceType));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ResetDevice(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BOOL oWaitForReboot(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &oWaitForReboot, sizeof(oWaitForReboot));

        //Reset
        if(oResult) oResult = ResetDevice(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, oWaitForReboot, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_WaitForReboot(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Wait for reboot
        if(oResult) oResult = WaitForReboot(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ClearDeviceErrors(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Clear Fault
        if(oResult)
        {
            usIndex = INDEX_CONTROL_WORD;
            usSubIndex = SUBINDEX_CONTROL_WORD;
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, FAULT_RESET, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetNbOfDeviceErrors(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNbOfDeviceError = 0;
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Number of Device Error
        if(oResult)
        {
            usIndex = INDEX_ERROR_HISTORY;
            usSubIndex = SUBINDEX_NB_OF_ERRORS;
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &ubNbOfDeviceError, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ubNbOfDeviceError, sizeof(ubNbOfDeviceError));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetDeviceErrorCode(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubErrorNumber(0);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulDeviceErrorCode(0);

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubErrorNumber, sizeof(ubErrorNumber));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Code from Device Error
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, INDEX_ERROR_HISTORY, ubErrorNumber, &ulDeviceErrorCode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulDeviceErrorCode, sizeof(ulDeviceErrorCode));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GotoSafeState(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId = 0;
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Set Disable
        if(oResult) oResult = SetDisableState(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetObject(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubObjectSubIndex;
    CErrorInfo errorInfo;
    DWORD ulNbOfBytesRead = 0;
    DWORD ulNbOfBytesToRead;
    void* pData = NULL;
    WORD usObjectIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usObjectIndex, sizeof(usObjectIndex));
        p_pCommand->GetParameterData(1, &ubObjectSubIndex, sizeof(ubObjectSubIndex));
        p_pCommand->GetParameterData(2, &ulNbOfBytesToRead, sizeof(ulNbOfBytesToRead));

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //GetObject
        if(oResult)
        {
            pData = malloc(ulNbOfBytesToRead);
            memset(pData, 0, ulNbOfBytesToRead);

            oResult = ReadDataBufferObject(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usObjectIndex, ubObjectSubIndex, pData, ulNbOfBytesToRead, &ulNbOfBytesRead, &errorInfo);
        }

        //Set Returnparameter
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, pData, ulNbOfBytesToRead);
        p_pCommand->SetReturnParameterData(1, &ulNbOfBytesRead, sizeof(ulNbOfBytesRead));

        if(pData) free(pData);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetObject(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubObjectSubIndex;
    CErrorInfo errorInfo;
    DWORD ulNbOfBytesToWrite;
    DWORD ulNbOfBytesWritten = 0;
    void* pData = NULL;
    WORD usObjectIndex;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usObjectIndex, sizeof(usObjectIndex));
        p_pCommand->GetParameterData(1, &ubObjectSubIndex, sizeof(ubObjectSubIndex));
        p_pCommand->GetParameterData(3, &ulNbOfBytesToWrite, sizeof(ulNbOfBytesToWrite));
        pData = malloc(ulNbOfBytesToWrite);
        p_pCommand->GetParameterData(2, pData, ulNbOfBytesToWrite);

        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //SetObject
        if(oResult) oResult = WriteDataBufferObject(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usObjectIndex, ubObjectSubIndex, pData, ulNbOfBytesToWrite, &ulNbOfBytesWritten, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulNbOfBytesWritten, sizeof(ulNbOfBytesWritten));
        if(pData) free(pData);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_Store(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand && p_pManager)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Store
        if(oResult) oResult = StoreParameter(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_Restore(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand && p_pManager)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult) oResult = RestoreParameter(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_RestoreFactory(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand && p_pManager)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        if(oResult) oResult = RestoreFactoryParameter(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadLSSFrame(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const DWORD MAX_FRAME_SIZE = 8;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    DWORD ulDataBufferLength;
    DWORD ulErrorCode = 0;
    void* pDataBuffer = NULL;
    WORD usTimeout;

    if(p_pCommand)
    {
        //Prepare data buffer
        ulDataBufferLength = MAX_FRAME_SIZE;
        pDataBuffer = malloc(ulDataBufferLength);
        memset(pDataBuffer, 0, ulDataBufferLength);

        //Get parameter data
        p_pCommand->GetParameterData(0, &usTimeout, sizeof(usTimeout));

        //ReadLSSFrame
        oResult = DCS_ReadLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usTimeout, pDataBuffer, ulDataBufferLength, &errorInfo);
        if(!oResult) ulErrorCode = errorInfo.GetErrorCode();

        //Set ReturnParameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
        p_pCommand->SetReturnParameterData(1, pDataBuffer, ulDataBufferLength);

        //Free data buffer
        if(pDataBuffer) free(pDataBuffer);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SendLSSFrame(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const DWORD MAX_FRAME_SIZE = 8;

    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    DWORD ulDataBufferLength;
    DWORD ulErrorCode = 0;
    void* pDataBuffer = NULL;

    if(p_pCommand)
    {
        //Prepare data buffer
        ulDataBufferLength = p_pCommand->GetParameterLength(1);
        if(ulDataBufferLength > MAX_FRAME_SIZE) ulDataBufferLength = MAX_FRAME_SIZE;
        pDataBuffer = malloc(ulDataBufferLength);
        memset(pDataBuffer, 0, ulDataBufferLength);

        //Get parameter data
        p_pCommand->GetParameterData(0, pDataBuffer, ulDataBufferLength);

        //SendLSSFrame
        oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, pDataBuffer, ulDataBufferLength, &errorInfo);
        if(!oResult) ulErrorCode = errorInfo.GetErrorCode();

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulErrorCode, sizeof(ulErrorCode));
        if(pDataBuffer) free(pDataBuffer);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_RequestCANFrame(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    WORD usCobID;
    WORD usLength;
    void* pData = NULL;
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usCobID, sizeof(usCobID));
        p_pCommand->GetParameterData(1, &usLength, sizeof(usLength));
        pData = malloc(usLength);

        //RequestCANFrame
        oResult = DCS_RequestCANFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usCobID, usLength, pData, &errorInfo);

        //Set ReturnParameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, pData, usLength);
        if(pData) free(pData);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SendCANFrame(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    void* pData = NULL;
    WORD usCobID;
    WORD usLength;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usCobID, sizeof(usCobID));
        p_pCommand->GetParameterData(1, &usLength, sizeof(usLength));
        pData = malloc(usLength);
        memset(pData, 0, usLength);
        p_pCommand->GetParameterData(2, pData, usLength);

        //SendCANFrame
        oResult = DCS_SendCANFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usCobID, usLength, pData, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        if(pData) free(pData);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadCANFrame(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    DWORD ulTimeout;
    void* pData = NULL;
    WORD usCobID;
    WORD usLength;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usCobID, sizeof(usCobID));
        p_pCommand->GetParameterData(1, &usLength, sizeof(usLength));
        p_pCommand->GetParameterData(2, &ulTimeout, sizeof(ulTimeout));

        //ReadCANFrame
        pData = malloc(usLength);
        oResult = DCS_ReadCANFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usCobID, usLength, pData, ulTimeout, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, pData, usLength);
        if(pData) free(pData);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SendNMTService(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    WORD usCommandSpecifier;
    WORD usNodeIdentifer;

    if(p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &usNodeIdentifer, sizeof(usNodeIdentifer));
        p_pCommand->GetParameterData(1, &usCommandSpecifier, sizeof(usCommandSpecifier));

        //SendNMTService
        oResult = DCS_SendNMTService(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usNodeIdentifer, usCommandSpecifier, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadVendorId(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulVendorId(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_IDENTIFICATION, SUBINDEX_VENDOR_ID, &ulVendorId, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulVendorId, sizeof(ulVendorId));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadDeviceType(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulDeviceType(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_TYPE, SUBINDEX_DEVICE_TYPE, &ulDeviceType, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulDeviceType, sizeof(ulDeviceType));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadDeviceName(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    CStdString deviceName(_T(""));

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadStringObject(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_NAME, SUBINDEX_DEVICE_NAME, &deviceName, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, deviceName.GetBuffer(), deviceName.GetLength());
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadProductCode(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulProductCode(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_IDENTIFICATION, SUBINDEX_PRODUCT_CODE, &ulProductCode, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulProductCode, sizeof(ulProductCode));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadRevisionNumber(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulRevisionNumber(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_IDENTIFICATION, SUBINDEX_REVISION_NO, &ulRevisionNumber, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulRevisionNumber, sizeof(ulRevisionNumber));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadSerialNumber(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulSerialNumber(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt32Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_DEVICE_IDENTIFICATION, SUBINDEX_SERIAL_NO, &ulSerialNumber, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulSerialNumber, sizeof(ulSerialNumber));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_IsIdentifySupported(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const BYTE IDENTIFY_NOT_SUPPORTED = 0;
    const BYTE IDENTIFY_SUPPORTED = 1;
    const DWORD FIRST_FIRMWARE_SOFTWARE_VERSION_SUPPORTING_LSS = 0x21100000;
    const DWORD FIRMWARE_SOFTWARE_VERSION_MASK = 0xFFFF0000;

    BOOL oResult(TRUE);
    CErrorInfo errorInfo;
    BYTE ubIdentifySupported(IDENTIFY_NOT_SUPPORTED);
    DWORD ulRevisionNumber(0);

    errorInfo.Init(0);

    if (p_pCommand)
    {
        p_pCommand->GetParameterData(0, &ulRevisionNumber, sizeof(ulRevisionNumber));
    }

    ulRevisionNumber &= FIRMWARE_SOFTWARE_VERSION_MASK;

    if (ulRevisionNumber >= FIRST_FIRMWARE_SOFTWARE_VERSION_SUPPORTING_LSS)
    {
        ubIdentifySupported = IDENTIFY_SUPPORTED;
    }

    p_pCommand->SetStatus(oResult, &errorInfo);
    p_pCommand->SetReturnParameterData(0, &ubIdentifySupported, sizeof(ubIdentifySupported));

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_StartIdentify(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const WORD CLEAR_TIMEOUT = 10;
    const int DATA_SIZE = 8;
    const BYTE ALL_NODES = 0;

    BOOL oResult(FALSE);
    DWORD ulVendorId(0);
    DWORD ulProductCode(0);
    DWORD ulRevisionNumber(0);
    DWORD ulSerialNumber(0);
    WORD usTimeout(0);
    BYTE ubNodeId(0);
    BYTE data[DATA_SIZE];

    CErrorInfo errorInfo;

    if (p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ulVendorId, sizeof(ulVendorId));
        p_pCommand->GetParameterData(1, &ulProductCode, sizeof(ulProductCode));
        p_pCommand->GetParameterData(2, &ulRevisionNumber, sizeof(ulRevisionNumber));
        p_pCommand->GetParameterData(3, &ulSerialNumber, sizeof(ulSerialNumber));
        p_pCommand->GetParameterData(4, &usTimeout, sizeof(usTimeout));
        p_pCommand->GetParameterData(5, &ubNodeId, sizeof(ubNodeId));

        //Stop remote node
        DCS_SendNMTService(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ALL_NODES, NSI_STOP_REMOTE_NODE, NULL);

        //Clear LSS Queue
        oResult = ClearLSSFrameQueue(p_pManager, p_hDCS_Handle, p_hTransactionHandle, CLEAR_TIMEOUT, &errorInfo);

        //Send LSS Frame VendorID
        if (oResult)
        {
            memset(data, 0, DATA_SIZE);
            data[0] = CIA305_SWITCH_STATE_SELECTIVE_PROTOCOL_COMMAND_SPECIFIER_VENDOR_ID;
            *((DWORD*)&data[1]) = ulVendorId;
            oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, data, DATA_SIZE, &errorInfo);
        }

        //Send LSS Frame ProductCode
        if (oResult)
        {
            memset(data, 0, DATA_SIZE);
            data[0] = CIA305_SWITCH_STATE_SELECTIVE_PROTOCOL_COMMAND_SPECIFIER_PRODUCT_CODE;
            *((DWORD*)&data[1]) = ulProductCode;
            oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, data, DATA_SIZE, &errorInfo);
        }

        //Send LSS Frame RevisionNumber
        if (oResult)
        {
            memset(data, 0, DATA_SIZE);
            data[0] = CIA305_SWITCH_STATE_SELECTIVE_PROTOCOL_COMMAND_SPECIFIER_REVISION_NUMBER;
            *((DWORD*)&data[1]) = ulRevisionNumber;
            oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, data, DATA_SIZE, &errorInfo);
        }

        //Send LSS Frame SerialNumber
        if (oResult)
        {
            memset(data, 0, DATA_SIZE);
            data[0] = CIA305_SWITCH_STATE_SELECTIVE_PROTOCOL_COMMAND_SPECIFIER_SERIAL_NUMBER;
            *((DWORD*)&data[1]) = ulSerialNumber;
            oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, data, DATA_SIZE, &errorInfo);
        }

        if (oResult)
        {
            MmcSleep(100);

            //Read LSS Frame
            oResult = DCS_ReadLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usTimeout, data, DATA_SIZE, &errorInfo);
        }

        if (oResult)
        {
            oResult = (data[0] == CIA305_SWITCH_STATE_SELECTIVE_PROTOCOL_COMMAND_SPECIFIER_RESPONSE);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_StopIdentify(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const int DATA_SIZE = 8;
    const BYTE ALL_NODES = 0;

    BOOL oResult(FALSE);
    BYTE ubMode(0);
    BYTE data[DATA_SIZE];
    CErrorInfo errorInfo;

    if (p_pCommand)
    {
        //Get parameter data
        p_pCommand->GetParameterData(0, &ubMode, sizeof(ubMode));

        memset(data, 0, DATA_SIZE);
        data[0] = CIA305_SWITCH_STATE_GLOBAL_PROTOCOL_COMMAND_SPECIFIER_MODE;
        data[1] = ubMode;

        oResult = DCS_SendLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, data, DATA_SIZE, &errorInfo);

        //Enter NMT State pre-operational
        if (oResult)
        {
            oResult = DCS_SendNMTService(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ALL_NODES, NSI_ENTER_PRE_OPERATIONAL, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::ClearLSSFrameQueue(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usTimeout, CErrorInfo* p_pErrorInfo)
{
    const int MAX_COUNT = 127;
    const int DATA_SIZE = 8;

    BOOL oResult(TRUE);

    BYTE data[DATA_SIZE];
    int lCount = 0;

    while (DCS_ReadLSSFrame(p_pManager, p_hDCS_Handle, p_hTransactionHandle, p_usTimeout, data, DATA_SIZE, p_pErrorInfo))
    {
        MmcSleep(0);
        lCount++;
        if (lCount > MAX_COUNT)
        {
            oResult = false;
            break;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadNodeId(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubNodeIdValue(0);
    CErrorInfo errorInfo;
    DWORD ulNodeIdValue(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_NODE_ID, SUBINDEX_NODE_ID, &ubNodeIdValue, &errorInfo);
        }

        //Convert Type
        ulNodeIdValue = (DWORD)ubNodeIdValue;

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulNodeIdValue, sizeof(ulNodeIdValue));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_WriteNodeId(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    BYTE ubNodeIdValue(0);
    CErrorInfo errorInfo;
    DWORD ulNodeIdValue(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &ulNodeIdValue, sizeof(ulNodeIdValue));

        //Convert Type
        ubNodeIdValue = (BYTE)ulNodeIdValue;

        //Read Object
        if(oResult)
        {
            oResult = WriteUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, INDEX_NODE_ID, SUBINDEX_NODE_ID, ubNodeIdValue, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ReadTransferRate(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulTransferRate(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read Object
        if(oResult)
        {
            oResult = GetDeviceTransferRate(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, ulTransferRate, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulTransferRate, sizeof(ulTransferRate));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_WriteTransferRate(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    DWORD ulTransferRate(0);

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &ulTransferRate, sizeof(ulTransferRate));

        //Read Object
        if(oResult)
        {
            oResult = SetDeviceTransferRate(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, ulTransferRate, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ScanStore(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand && p_pManager)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Store
        if(oResult) oResult = StoreParameter(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ScanResetDevice(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oBroadcast(FALSE);
    BOOL oResult(FALSE);
    BOOL oWaitForReboot(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Get parameter data
        p_pCommand->GetParameterData(0, &oBroadcast, sizeof(oBroadcast));
        p_pCommand->GetParameterData(1, &oWaitForReboot, sizeof(oWaitForReboot));

        //Reset
        if(oResult) oResult = ResetDevice(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, oWaitForReboot, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_ScanWaitForReboot(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Reset
        if(oResult) oResult = WaitForReboot(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, &errorInfo);

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetBaseSectorVersion(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usBaseSectorVersion(0);
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read target velocity
        if(oResult)
        {
            usIndex = INDEX_VERSION;
            usSubIndex = SUBINDEX_BASE_VERSION;
            oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &usBaseSectorVersion, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &usBaseSectorVersion, sizeof(usBaseSectorVersion));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetSerialNumber(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    __uint64 llSerialNumber = 0;
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;
    WORD usIndex;
    WORD usSubIndex;

    if(p_pCommand)
    {
        //Get address parameter
        oResult = GetNodeId(p_pCommand, p_pManager, p_hDCS_Handle, ubNodeId, &errorInfo);

        //Read target velocity
        if(oResult)
        {
            usIndex = INDEX_SERIAL_NUMBER;
            usSubIndex = SUBINDEX_SERIAL_NUMBER;
            oResult = ReadUInt64Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, (WORD)ubNodeId, usIndex, usSubIndex, &llSerialNumber, &errorInfo);
        }

        //Set return parameter data
        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &llSerialNumber, sizeof(llSerialNumber));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_GetGatewaySettings(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    DWORD ulCANBitRate(0);
    WORD usCANBitRateIndex(0);
    WORD usNodeId(0);

    if(p_pCommand)
    {
        oResult = ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usNodeId, INDEX_CAN_BITRATE, SUBINDEX_CAN_BITRATE, &usCANBitRateIndex, &errorInfo);
        ulCANBitRate = ConvertBitrateIndexToValue_CiaStandard(usCANBitRateIndex);

        p_pCommand->SetStatus(oResult, &errorInfo);
        p_pCommand->SetReturnParameterData(0, &ulCANBitRate, sizeof(ulCANBitRate));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::Process_SetGatewaySettings(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle)
{
    const BOOL WAIT_FOR_REBOOT(TRUE);
    const WORD DEFAULT_NODE_ID(0);

    BOOL oResult(FALSE);
    BOOL oUpdateCanBitrate(FALSE);
    CErrorInfo errorInfo;
    DWORD ulCANBitRate(0);
    WORD usCANBitRateIndex(0);
    WORD usNodeId(0);

    oResult = IsGatewaySupported(p_pCommand, p_pManager, p_hDCS_Handle, &errorInfo);

    if(oResult && p_pCommand)
    {
        p_pCommand->GetParameterData(0, &ulCANBitRate, sizeof(ulCANBitRate));
        usCANBitRateIndex = ConvertBitrateValueToIndex_CiaStandard(ulCANBitRate);

        oResult = ReadNodeId(p_pManager, p_hDCS_Handle, p_hTransactionHandle, DEFAULT_NODE_ID, usNodeId, &errorInfo);

        if (oResult)
        {
            oResult = CheckUpdateCanBitrateRequired(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usNodeId, usCANBitRateIndex, oUpdateCanBitrate, &errorInfo);
        }

        if (oResult && oUpdateCanBitrate)
        {
            oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, usNodeId, INDEX_CAN_BITRATE, SUBINDEX_CAN_BITRATE, usCANBitRateIndex, &errorInfo);
        }

        if(oResult && !InitGatewayTimeout(p_pManager, p_hDCS_Handle, p_hTransactionHandle, &errorInfo))
        {
            oResult = FALSE;
        }

        if(oResult && !StoreParameter(p_pManager, p_hDCS_Handle, p_hTransactionHandle, BYTE(usNodeId), &errorInfo))
        {
            oResult = FALSE;
        }

        if(oResult && oUpdateCanBitrate)
        {
            oResult = ResetDevice(p_pManager, p_hDCS_Handle, p_hTransactionHandle, BYTE(usNodeId), WAIT_FOR_REBOOT, &errorInfo);
        }

        p_pCommand->SetStatus(oResult, &errorInfo);
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::IsGatewaySupported(CCommand_VCS* p_pCommand, CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    CErrorInfo errorInfo;
    CStdString protocolStackName(_T(""));

    if(p_pManager)
    {
        oResult = p_pManager->DCS_GetProtocolStackName(p_hDCS_Handle, &protocolStackName, p_pErrorInfo);

        if(oResult)
        {
            if((PROTOCOL_MAXON_SERIAL_V1 == protocolStackName) || (PROTOCOL_MAXON_SERIAL_V2 == protocolStackName))
            {
                oResult = TRUE;
            }
            else
            {
                oResult = FALSE;
                errorInfo.Init(ERROR_FUNCTION_NOT_SUPPORTED);

                if(p_pCommand)
                {
                    p_pCommand->SetStatus(oResult, &errorInfo);
                }
            }
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::ReadNodeId(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE p_ubNodeId, WORD& p_rusNodeId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    BYTE ubNodeId(0);
    CErrorInfo errorInfo;

    if (ReadUInt8Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, p_ubNodeId, INDEX_NODE_ID, SUBINDEX_NODE_ID, &ubNodeId, p_pErrorInfo))
    {
        p_rusNodeId = (WORD)ubNodeId;

        oResult = TRUE;
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::CheckUpdateCanBitrateRequired(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usCANBitRateIndex, BOOL& p_roUpdateCanBitrate, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);
    WORD usCANBitRateIndex(0);

    if(ReadUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, p_usNodeId, INDEX_CAN_BITRATE, SUBINDEX_CAN_BITRATE, &usCANBitRateIndex, p_pErrorInfo))
    {
        if (usCANBitRateIndex != p_usCANBitRateIndex)
        {
            p_roUpdateCanBitrate = TRUE;
        }

        oResult = TRUE;
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::SetSsiEncodingType(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, WORD p_usNodeId, WORD p_usSensorType, CErrorInfo* p_pErrorInfo)
{
    const WORD SSI_ABS_ENCODER_BINARY(4);
    const WORD SSI_ABS_ENCODER_GREY(5);
    const WORD SSI_ENCODER_BINARY_TYPE(0);
    const WORD SSI_ENCODER_GRAY_CODED(1);

    BOOL oResult(FALSE);
    WORD usEncodingType(-1);
    WORD usIndex(INDEX_SSI_ENCODER_CONFIGURATION);
    WORD usSubIndex(SUBINDEX_SSI_ENCODER_ENCODING_TYPE);

    if (p_usSensorType == SSI_ABS_ENCODER_BINARY)
    {
        usEncodingType = SSI_ENCODER_BINARY_TYPE;
    }
    else if (p_usSensorType == SSI_ABS_ENCODER_GREY)
    {
        usEncodingType = SSI_ENCODER_GRAY_CODED;
    }

    oResult = WriteUInt16Object(p_pManager, p_hDCS_Handle, p_hTransactionHandle, p_usNodeId, usIndex, usSubIndex, usEncodingType, p_pErrorInfo);

    return oResult;
}

BOOL CGatewayDriveToEpos2::GetGatewayNodeId(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rubGatewayNodeId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if (p_pManager)
    {
        if (p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, GATEWAY_NODE_ID, (BYTE*)&p_rubGatewayNodeId, sizeof(p_rubGatewayNodeId), p_pErrorInfo))
        {
            oResult = TRUE;
        }
        else
        {
            p_rubGatewayNodeId = 0;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::GetDeviceNodeId(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE& p_rubNodeId, CErrorInfo* p_pErrorInfo)
{
    BOOL oResult(FALSE);

    if (p_pManager)
    {
        if (p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_rubNodeId, sizeof(p_rubNodeId), p_pErrorInfo))
        {
            oResult = TRUE;
        }
        else
        {
            p_rubNodeId = 0;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::IsGatewayTopology(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle)
{
    BOOL oResult(FALSE);
    BYTE ubGatewayNodeId(0);
    BYTE ubNodeId(0);

    if (GetGatewayNodeId(p_pManager, p_hDCS_Handle, ubGatewayNodeId) && GetDeviceNodeId(p_pManager, p_hDCS_Handle, ubNodeId))
    {
        if (ubGatewayNodeId && ubNodeId && (ubGatewayNodeId != ubNodeId))
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::UpdateNodeId(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE p_ubNodeId, BYTE& p_rubNodeIdBackup)
{
    BOOL oResult(FALSE);

    if (p_pManager)
    {
        if (p_pManager->GetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_rubNodeIdBackup, sizeof(p_rubNodeIdBackup)))
        {
            oResult = p_pManager->SetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_ubNodeId, sizeof(p_ubNodeId));
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::ResetNodeId(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, BYTE p_ubNodeId)
{
    BOOL oResult(FALSE);

    if (p_pManager)
    {
        oResult = p_pManager->SetParameter(PT_DEVICE, p_hDCS_Handle, NODE_ID, (BYTE*)&p_ubNodeId, sizeof(p_ubNodeId));
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::SetGatewayTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, BYTE ubNodeId, DWORD p_ulTimeout)
{
    BOOL oResult(FALSE);
    DWORD ulDeviceTimeout(0);

    if (GetDeviceTimeout(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, ulDeviceTimeout))
    {
        m_ulDeviceTimeoutBackup = ulDeviceTimeout;

        if (ulDeviceTimeout != p_ulTimeout)
        {
            oResult = SetDeviceTimeout(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubNodeId, p_ulTimeout);
        }
        else
        {
            oResult = TRUE;
        }
    }

    return oResult;
}

BOOL CGatewayDriveToEpos2::SetGatewayDeviceTimeout(CDeviceCommandSetManagerBase* p_pManager, HANDLE p_hDCS_Handle, HANDLE p_hTransactionHandle, DWORD p_ulGatewayTimeout)
{
    BOOL oResult(FALSE);
    BYTE ubGatewayNodeId(0);
    BYTE ubNodeIdBackup(0);

    if (IsGatewayTopology(p_pManager, p_hDCS_Handle))
    {
        GetGatewayNodeId(p_pManager, p_hDCS_Handle, ubGatewayNodeId);

        if (UpdateNodeId(p_pManager, p_hDCS_Handle, ubGatewayNodeId, ubNodeIdBackup))
        {
            // Execute
            oResult = SetGatewayTimeout(p_pManager, p_hDCS_Handle, p_hTransactionHandle, ubGatewayNodeId, p_ulGatewayTimeout);

            ResetNodeId(p_pManager, p_hDCS_Handle, ubNodeIdBackup);
        }
    }
    else
    {
        oResult = TRUE;
    }

    return oResult;
}